////////////////////////////////////////////////////////////////////////////////
/*! @file APRI_ML327_Counter.cpp   Driver for APRI ML-327 counter.
- Part of Rigaku_APRI - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
- Copyright(C) 2010, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "APRI_ML327_Counter.h"
#include "attr.h"
#include "StringsAndQStrings.h"
#include <QDebug>
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sAPRI_ML327_Counter)

//------------------------------------------------------ sAPRI_ML327_Counter ---

void  sAPRI_ML327_Counter::Preconfigure (sNodePtr  DeviceCfg)
{
  sPtr<sFolderNode>              DeviceSettings;
  sPtr<sFolderNode>              DeviceLinks;
  DeviceCfg |= "Links";
  DeviceLinks = DeviceCfg >> "Links";
  if( DeviceLinks.IsCorrect() ){
    DeviceLinks |= "Communicator";
    DeviceLinks >> "Communicator" |= attr("Class","sTelecom");
  };
  DeviceCfg |= "Settings";
  DeviceSettings = DeviceCfg >> "Settings";
  if( DeviceSettings.IsCorrect() ){
  };
  sGauge::Preconfigure(DeviceCfg);
};


sAPRI_ML327_Counter::~sAPRI_ML327_Counter()
{
  Communicator.SetToNULL();
  if( TimeoutTimer ){
    delete TimeoutTimer;
    TimeoutTimer = NULL;
  };
};


sAPRI_ML327_Counter::sAPRI_ML327_Counter (sString   DevicetId
                                         ,sNodePtr  DeviceCfg)
    :sGauge(DevicetId,DeviceCfg)
    ,Communicator(this)
    ,CounterUnits("photon")

    //! @todo{Rigaku} review!!! проверить значение, можно в конфиг засунуть
    ,PeriodQuantum(0.010,"s")

    //! @todo{Rigaku} review!!! проверить значение, можно в конфиг засунуть
    ,MaxTimePar(10)

    ,TimePar(MaxTimePar)
    ,CountsPerStep(1)

    ,TheStatus(Newborn)  //в итоге это устройство рождается
                         //в состоянии Offline (от sDriver)
                         //и в состоянии цикла обработки команд Newborn
                         //Это состояние нужно только, чтобы присвоить его
                         //в конструкторе и в принципе могло бы быть исключено.
                         //
    ,ContactOrder(OnOff)
    ,Command("")
    ,CountNum(0)
    ,PrevCount(0)
    ,SumCount(0)
    ,TimeoutTimer(NULL)
{
  Preconfigure(DeviceConfig);
  sPtr<sTextNode>             CommunicatorId(DeviceLinks
                                                   >> "Communicator"
                                                   >> "DeviceId");
  if( CommunicatorId.IsCorrect() ){
    Communicator.SetTo(sString(*CommunicatorId));
  };
  if( Communicator.IsCorrect() ){
    connect(Communicator.operator ->() ,SIGNAL(GotMessage (QByteArray))
           ,this                       ,SLOT(on_MessageReceived(QByteArray)));
    TimeoutTimer = new QTimer(this);
    TimeoutTimer->setInterval(300);
    TimeoutTimer->setSingleShot(true);
    connect(TimeoutTimer, SIGNAL(timeout()), this, SLOT(SendErrorMessage()));
    connect(this ,SIGNAL(ErrorMessage (QByteArray))
           ,this ,SLOT(on_MessageReceived(QByteArray)));
  }else{
    //! @todo{Rigaku} review!!!
    //!!!!!!!!!!!!!!!!!!!!! вот на этом она и сваливалась
    //!!!!!!!!!!!!!!!!!!!!! точнее на отсутствии проверок в дальнейшем
    //
    //вообще, надо будет обратить внимание на нелогично разную логику:
    // для sRigakuScan сигналы подчинённых устройств коннектятся не в конструкторе,
    // как здесь, а в  sRigakuScan.Connect()
  };
  if( DeviceSettings.IsCorrect() ){
    sPtr<sIntegerNode>          I;

    int COrd; COrd = ContactOrder;                        // 0 - NotUsed
    DeviceSettings |= attr("ContactStartOrder",COrd);     // 1 - OnOff
    I = DeviceSettings >> "ContactStartOrder";            // 2 - OffOn
    if( I.IsCorrect() ){
        COrd = (int)(*I);
        switch (COrd){
        case 0 : { ContactOrder = NotUsed; break;}
        case 1 : { ContactOrder = OnOff; break;}
        case 2 : { ContactOrder = OffOn; break;}
        }
    }
  }
};

void  sAPRI_ML327_Counter::Connect ()
{
  if( DriverState() == Offline ){
    SetDriverState(Warming);
    Communicator->Connect();
    if (Communicator->DriverState() == Standby){
        TheStatus = Initiating;
        InitStep = 1;
        TimeoutTimer->start();
    } else {
        SetDriverState(Offline);
        emit NowOffline();
    }
  };
};


void  sAPRI_ML327_Counter::Disconnect ()
{
  if( DriverState() != Offline ){
    Communicator->Disconnect();
    SetDriverState(Offline);
    TimeoutTimer->stop();
    emit NowOffline();
  };
};


void  sAPRI_ML327_Counter::SetPeriod (sPhysValue  period)
{
  //! @todo{Rigaku} по логике надо проверять DriverState():
//  if( DriverState() == Standby ){

  //! @todo{Rigaku} review!!!
  try{
    sPhysValue                  CountsPerStepEvaluation(_Unitsless_);
    for( TimePar = MaxTimePar; TimePar > 0 ; TimePar-- ){
      CountsPerStepEvaluation = period / ( PeriodQuantum * TimePar );
      CountsPerStep = Round(CountsPerStepEvaluation.ArbValue());
      if( CountsPerStep < 1 ){
        CountsPerStep = 1;
      };
      ThePeriod = PeriodQuantum * (TimePar * CountsPerStep);
      if( (period/ThePeriod)-sPhysValue(1) < sPhysValue(0.20) ){
        //т.е. если отклонение шага от заданного не превышает 20%
        break;
      };
    };
  }catch(rxRAlgebra){
    SetDriverState(Invalid);
    emit Error(
        tr("Wrong units in the configuration settings for '%1'.")
        .arg(ToQString(DeviceIdentifier()))
        );
    return;
  };
};


void  sAPRI_ML327_Counter::Start ()
{
//  //! @todo{Rigaku} review!!!
//  if( DriverState() == Standby ){
//    Command.clear();
//    Command.append(QByteArray::number(TimePar, 16).toUpper());
//    if( Command.size() == 1 )  Command.prepend("0");
//    Command.prepend("S");
//    Communicator->SendMessage(Command);
//    TimeoutTimer->start();
//    TheStatus = CountStarting;
//    SetDriverState(Busy);
//  };
};


/*! Останавливает молотилку.

Может вызываться как по достижении конца диапазона сканирования, так и по
инициативе юзера или ядра - короче, в любой момент!
*/
void  sAPRI_ML327_Counter::Stop ()
{
  //! @todo{Rigaku} вообще-то логика должна бы быть такая
//  if( DriverState() == Busy ){
//    ... тут возможен разрыв: приконнектить слот к сигналу, послать запрос,
//        по получении ответа отконнектить и завершить переводом в Standby
//    SetDriverState(Standby);
//  };
//  А у тебя вроде бы Stop может попасть, например, на CountStarting и пропасть
//  То-то ты дублируешь вызов StopCount перед очередным сканированием!

    if (TheStatus == CountStarted){
        Communicator->SendMessage("WT");
//        TimeoutTimer->start(); Вызывает ошибку из-за ожидания ответа на
//                               сообщение конца сканирования.
        TheStatus = CountStopping;
        SetDriverState(Busy);
    }
};


void  sAPRI_ML327_Counter::StartWaitingForMarkSignal ()
{
  if( DriverState() == Standby ){
    //! @todo{Rigaku} review!!!
    if( ContactOrder == NotUsed ){
      emit GotMarkSignal();
    }else{
      Command.clear();
      Command.append(QByteArray::number(TimePar, 16).toUpper());
      if (Command.size() == 1) Command.prepend("0");
      Command.prepend("S");
      ContactMustBeOn = (ContactOrder == OnOff);
      ContactStateReached = false;
      Communicator->SendMessage("C");
      TimeoutTimer->start();
      TheStatus = ScanStarting;
      SetDriverState(Busy);
      TestStep = 1;
      //! @todo{Rigaku} надо, чтобы где-то там выдался сигнал GotMarkSignal()
    };
  };
};


bool sAPRI_ML327_Counter::CountIsStarted()
{
    return TheStatus == CountStarted;
}

void sAPRI_ML327_Counter::SendErrorMessage()
{
    QByteArray msg = "#";
    emit ErrorMessage (msg);
}

void sAPRI_ML327_Counter::on_MessageReceived(QByteArray Msg)
{
    TimeoutTimer->stop();
    switch (TheStatus)
    {
    case Newborn : {
        break;
        }
    case Initiating : {
        if (InitStep == 1 ) Communicator->SendMessage("G");
        if (InitStep == 3 ) Communicator->SendMessage("G");
        if (InitStep == 5)
        {
          Communicator->SendMessage("");
        }
        if (InitStep == 6)
        {
          if (Msg == "<"){
            SetDriverState(Standby);
            emit NowStandby();
          }
          else{
            SetDriverState(Offline);
            emit NowOffline();
          }
        }
        if (InitStep < 6) TimeoutTimer->start();
        InitStep++;
        break;
        }
    case ScanStarting : {
        qDebug() << "Scan starting, step " << TestStep << " Reply : " << Msg;
        TimeoutTimer->stop();
        switch (TestStep) {
        case 1 : {
            if ( (!ContactMustBeOn && (Msg == "!0"))
            || (ContactMustBeOn && (Msg == "!1"))) ContactStateReached = true;
            TimeoutTimer->start();
            break;
        }
        case 2 : {
            if (ContactStateReached){
                if (Msg == "<"){
                    ContactMustBeOn = !ContactMustBeOn;
                    ContactStateReached = false;
                    Communicator->SendMessage("C");
                    TimeoutTimer->start();
                } else {
                    SetDriverState(Offline);
                    emit NowOffline();
                }
            } else {
                Communicator->SendMessage("C");
                TimeoutTimer->start();
                TestStep = 0;
            }
            break;
        }
        case 3 : {
            if ( (!ContactMustBeOn && (Msg == "!0"))
            || (ContactMustBeOn && (Msg == "!1"))) ContactStateReached = true;
            TimeoutTimer->start();
            break;
        }
        case 4 : {
            if (ContactStateReached){
                if (Msg == "<") {
                    emit GotMarkSignal();
                    Communicator->SendMessage(Command);
                    TimeoutTimer->start();
                    TheStatus = CountStarting;
                }
                else{
                    SetDriverState(Offline);
                    emit NowOffline();
                }
            } else {
                Communicator->SendMessage("C");
                TimeoutTimer->start();
                TestStep = 2;
            }
            break;
        }
        }
        TestStep++;
        break;
        }
    case CountStarting : {
        TimeoutTimer->start();
        if (Msg == "$<"){
            PrevCount = 0;
            SumCount = 0;
            CountNum = 0;
            Communicator->SendMessage("TR");
            TheStatus = CountStarted;
            SetDriverState(Busy);
            qDebug() << "Counting is started " << TheStatus << " Reply : " << Msg;
        } else{
            SetDriverState(Offline);
            emit NowOffline();
        }
        break;
        }
    case CountStarted : {
        long C;
        bool OK;
        long CurrentCount = Msg.toLong(&OK,16);
        if (!OK) {
            Communicator->SendMessage("WT");
            SetDriverState(Offline);
            emit NowOffline();
        } else {
            if (CurrentCount < PrevCount)
                C = CurrentCount + 65536;
            else
                C = CurrentCount;
            C = C - PrevCount;
            PrevCount = CurrentCount;
            SumCount = SumCount + C;
            CountNum++;
            if (CountNum == CountsPerStep) {
////  Just for debuging
//                emit NewData(sPhysValue(999.0,"photon/s"));
                emit NewData(  sPhysValue(SumCount,CounterUnits)
                             / (CountsPerStep*TimePar*PeriodQuantum)
                            );
                SumCount = 0;
                CountNum = 0;
            }
        }
        break;
        }
    case CountStopping : {
        if (Msg == "<"){
            SetDriverState(Standby);
            emit NowStandby();
        } else{
            SetDriverState(Offline);
            emit NowOffline();
        }
        qDebug() << "Status (COff): " << TheStatus << " Reply : " << Msg;
        break;
        }
    }
}

//------------------------------------------------------------------------------
}; //namespace RA

