////////////////////////////////////////////////////////////////////////////////
/*! @file APRI_ML327_Counter.h   Driver for APRI ML-327 counter.
- Part of RigakuAPRI - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
- Copyright(C) 2010, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef APRI_ML327_Counter_H
#define APRI_ML327_Counter_H
#include "RigakuAPRI.h"
#include "Driver.h"
#include "Telecom.h"
#include "Gauge.h"
#include <QTimer>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sAPRI_ML327_Counter)

//------------------------------------------------------ sAPRI_ML327_Counter ---
/*!

*/
class  RigakuAPRI_EXPORT  sAPRI_ML327_Counter : public sGauge
{
  Q_OBJECT
  DRIVER(sAPRI_ML327_Counter)
  public:
    virtual                   ~sAPRI_ML327_Counter ();
    virtual void              Connect ();
    virtual void              Disconnect ();
    virtual void              SetPeriod (sPhysValue  period);
    virtual void              Start ();
    virtual void              Stop ();
    virtual void              StartWaitingForMarkSignal ();

    bool                      CountIsStarted();

    enum contactOrder {NotUsed, OnOff, OffOn};

  signals:
    void                      GotMarkSignal ();
                              /*!< Получен сигнал от контактов при развёртке по
                                   углам в Rigaku (случается при прохождении
                                   целого значения угла) */
//    void                      ContactOK();  //не используется - рудимент?
    void                      ErrorMessage (QByteArray);

  public slots:

  private: //types
    enum                      eStatus
                                {Newborn
                                ,Initiating
                                ,ScanStarting
                                ,CountStarting
                                ,CountStarted
                                ,CountStopping
                                }; //в приватной части можно завести
                                   //какие хочешь статусы для под-состояний,
                                   //         НО (!!!)
                                   //от sDevice унаследованы 4 eDeviceState
                                   //и их необходимо поддержать
                                   //DN: Newborn эквивалентно Offline
                                   //    Initiating - Warming
                                   //    Остальные состояния представляют собой различные фазы
                                   //    обработки команд и соответствуют Busy.

  private slots:
    void on_MessageReceived(QByteArray Msg);
    void SendErrorMessage();

  private:

  private: //fields
    sTelecom::sPtr            Communicator;
    sUnits                    CounterUnits;
    sPhysValue                PeriodQuantum;
                              /*!< Квант времени для коробочки APRI ML327 */
    int                       MaxTimePar;
                              /*!< Максимальное значение TimePar,
                                   чтобы счётчик не захлёбывался */
    int                       TimePar;
                              /*!< Время измерения для коробочки APRI ML327,
                                   выраженное в её квантах
                                   (sAPRI_ML327_Counter::PeriodQuantum) */
    int                       CountsPerStep;// it was called TheMultiple
                              /*!< Число опросов коробочки APRI ML327,
                                   аккумулируемых для выдачи одного отсчёта
                                   sAPRI_ML327_Counter
                                   (@sa sPeriodicGage::NewData()) */
    eStatus                   TheStatus;
    contactOrder              ContactOrder;
    QByteArray                Command;
    int                       InitStep;
    int                       TestStep;
    int                       CountNum;
    long                      PrevCount;
    long                      SumCount;
    bool                      ContactMustBeOn;
    bool                      ContactStateReached;

    //! @todo{Rigaku} надо будет передать это в sTelecom
    QTimer  *                 TimeoutTimer;

};

//------------------------------------------------------------------------------
}; //namespace RA
#endif //APRI_ML327_Counter_H
