////////////////////////////////////////////////////////////////////////////////
/*! @file RigakuAPRI.cpp Drivers for XRD Rigaku with APRI ML-327.
- Part of RigakuAPRI - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2011, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "RigakuAPRI.h"
#include "attr.h"
#include "TeleCOMPort.h"
#include "APRI_ML327_Counter.h"
#include "RigakuScan.h"
#include "Symbols.h"
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sRigakuAPRI)

//------------------------------------------------------------------------------

sVersion  sRigakuAPRI::Version ()
{
  return sVersion(RAP_VERS_MAJOR
                 ,RAP_VERS_MINOR
                 ,ZAKAVYCH(RAP_VERS_PATCH)
                 ,__DATE__
                 );
}


sVersion  sRigakuAPRI::CoreVersionUsed ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}


sVersion  sRigakuAPRI::CoreVersionMin ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}


QString  sRigakuAPRI::UserFriendlyName ()
{
  return trUtf8("Drivers for XRD Rigaku with APRI ML-327");
}


QString  sRigakuAPRI::About ()
{
  return trUtf8(
      "Provides 1 driver and 1 console:\n"
      "sAPRI_ML327_Counter -- driver for the APRI ML327 box;\n"
      "sRigakuScan -- user interface (console) for the scanning technique.\n\n"
      "Copyright (C) 2011, Dmitry K. Nelson, St.Petersburg, Russia.\n"
      "                    D.Nelson@mail.ioffe.ru\n"
      "Copyright (C) 2011, Viktor E. Bursian, St.Petersburg, Russia.\n"
      "                    Viktor.Bursian@mail.ioffe.ru"
                );
}


void  sRigakuAPRI::InstallDefaults (sPtr<sFolderNode>  DeviceList)
{
  try{
    sPtr<sFolderNode>              DeviceCfg;

    DeviceList |= "Communicator1";
    DeviceCfg=DeviceList >> "Communicator1";
    DeviceCfg << attr("Class","sTeleCOMPort");
    sTeleCOMPort::Preconfigure(DeviceCfg);
    DeviceCfg >> "Settings" << attr("Name","COM1");
    DeviceCfg >> "Settings" << attr("BaudRate",115200);
    DeviceCfg >> "Settings" << attr("Timeout",sPhysValue(60.0,"s"));

    DeviceList |= "APRI ML-327 Box";
    DeviceCfg=DeviceList >> "APRI ML-327 Box";
    DeviceCfg << attr("Class","sAPRI_ML327_Counter");
    sAPRI_ML327_Counter::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Communicator" << attr("DeviceId","Communicator1");

    DeviceList |= "XRD Rigaku Scanner";
    DeviceCfg=DeviceList >> "XRD Rigaku Scanner";
    DeviceCfg << attr("Class","sRigakuScan");
    sRigakuScan::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Gage" << attr("DeviceId","APRI ML-327 Box");

    /*________________________________________template for device
    DeviceList |= "_device_id_";
    DeviceCfg = DeviceList >> "_device_id_";
    DeviceCfg << attr("Class","_driver_class_name_");
    _driver_class_name_::Preconfigure(DeviceCfg);
    //if there are links:
    DeviceCfg >> "Links" >> "_link_name_"
        << attr("DeviceId","_device_id_for_the_link");
    //if you wanna change some of the default settings:
    DeviceCfg >> "Settings" << attr("",);
    DeviceCfg >> "Settings" << attr("",);
    */
  }catch(...){
    ;
  };
}


sRigakuAPRI::sRigakuAPRI ()
{
  sUnit::Register("photon");
  sUnit::Register(Symbol::Degree,sMetricPrefix::NoPrefixAllowed);
}

//------------------------------------------------------------------------------
} //namespace RA
