////////////////////////////////////////////////////////////////////////////////
/*! @file RigakuScan.h   XRD Rigaku Scanner.
- Part of RigakuAPRI - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
- Copyright(C) 2010, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef RigakuScan_H
#define RigakuScan_H
#include "RigakuAPRI.h"
#include "PeriodicGaugeRecorder.h"
#include "APRI_ML327_Counter.h"
#include <QComboBox>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sRigakuScan)

//-------------------------------------------------------------- sRigakuScan ---
/*! XRD Rigaku Scanner.
*/
class  RigakuAPRI_EXPORT   sRigakuScan : public sPeriodicGaugeRecorder
{
  Q_OBJECT
  CONSOLE(sRigakuScan)
  public://static const
    static csString           SpeedUnits;

  public:
    virtual                   ~sRigakuScan ();
    virtual void              RestoreState ();
    virtual void              StoreState ();

  public slots:

  protected slots:
    virtual void              StartScan ();
    virtual void              StartRecording ();
    virtual void              FinishScan ();

  protected: //fields
    sAPRI_ML327_Counter::sPtr PhotonCounter;
                              /*!< Совпадает с sPeriodicGaugeRecorder::Gauge,
                                   но позволяет пользоваться дополнительными
                                   возможностями класса sAPRI_ML327_Counter */

    sPhysValue                StartQuantum;

    static const int          MaxNumOfSpeeds = 12;
    real                      Speeds[MaxNumOfSpeeds];
    int                       NumOfSpeeds;

  protected: //fields

    QLabel *                  LblStepUnit;
    QLabel *                  LblSpeed;
    QComboBox *               FldSpeed;
    QLabel *                  LblSpeedUnit;
};

//------------------------------------------------------------------------------
}; //namespace RA
#endif
