////////////////////////////////////////////////////////////////////////////////
/*! @file Microsin_USB_IO.h   USB interface for the Microsin device.
- Part of the MSDN firmware.
- Part of the Research Assistant project.
- Copyright(C) 2016, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef Microsin_USB_IO_H
#define Microsin_USB_IO_H
#include "DescriptorsMSDN.h"
#include <inttypes.h>
//! @todo{MSDN} Неясная роль math
#include <math.h>

void  Send_EOL_ToUSB (void);

void  SendCharToUSB (char  character);

void  SendStringToUSB (const char * const  string_of_chars);

void  SendNumberToUSB (int32_t  number);

extern uint8_t                LeadingZeroes;
extern int8_t                 SendByByte;
extern int8_t                 USBByClock;
extern int8_t                 USBClockSignal;

void  MicrosinLEDTurnOn (void);

void  MicrosinLEDTurnOff (void);


//------------------------------------------------------------------------------
//      The following function is defined in CommandParser.c,
//      where the incomming stream syntax and semantics are restricted
//      to the common needs of MSDN device series.
//
//      However, one can provide another implementation.
//------------------------------------------------------------------------------

void  ProcessReceivedByte (const int16_t  received_byte);

//------------------------------------------------------------------------------
//      The following functions have to be defined somewhere
//              in the device-specific module(s)
//------------------------------------------------------------------------------

/*! Configures the board hardware and chip peripherals.

It is called at the very beginning of the main() function.

It is not defined in Microsin_USB_IO.c and has to be defined
somewhere in the device-specific module(s).
*/
void  SetupHardware (void);


/*! Called from the main() function right before the main program loop.

It is not defined in Microsin_USB_IO.c and has to be defined
somewhere in the device-specific module(s).
*/
void  BeforeLoop (void);


/*! Called from the main() function inside the main program loop.

It is not defined in Microsin_USB_IO.c and has to be defined
somewhere in the device-specific module(s).
*/
void  InLoop (void);

#endif
