////////////////////////////////////////////////////////////////////////////////
/*! @file DistWizard.cpp   Мастер формирования составного дистрибутива.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2011-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          VBursian@gmail.com
*///////////////////////////////////////////////////////////////////////////////
#include "DistWizard.h"
#include "StringsAndQStrings.h"
#include <QHBoxLayout>
#include <QGridLayout>
#include <QLabel>
#include <QButtonGroup>
#include <QRadioButton>
#include <QPushButton>
#include <QApplication>
#include <QCheckBox>
#include <QLibraryInfo>
#include "Trace.h"
#include <QDebug>
namespace RA {
//------------------------------------------------------------------------------
//-------------------------------------------------------------- sDistWizard ---

sDistWizard::sDistWizard (bool  testing)
    :QWizard()
    ,Job(new sDistribJob(testing))
{
  setWindowTitle(tr("Distributive Making Wizard"));
  if( Job->AnalizeProject() ){
    setPage(SelectPage,new sSelectPage(Job));
    setPage(MakePage,new sMakePage(Job));
  }
}

//-------------------------------------------------------------- sSelectPage ---

sSelectPage::sSelectPage (psDistribJob  job)
    :Job(job)
{
//QString ProjectName;
//#ifdef RA_PRO_NAME
//  ProjectName = ZAKAVYCH(RA_PRO_NAME) ;
//#endif
//qDebug() << "ProjectName="  << ProjectName ;

  setTitle("Selection");
//  setSubTitle("Choose what you want");
  TranslateRequestMapper = new QSignalMapper(this);

  QHBoxLayout *               HLayout = new QHBoxLayout;
  setLayout(HLayout);

  QGridLayout *               Layout = new QGridLayout;
  HLayout->addLayout(Layout);

  Report = new QTextEdit;
  HLayout->addWidget(Report);

  int                         Row;
  QButtonGroup *              CoreAndCommonPluginsOrExtraPluginSet;
  QRadioButton *              CoreAndCommonPluginsButton;
  QRadioButton *              ExtraPluginSetButton;
  QButtonGroup *              ReleaseOrPatch;
  QRadioButton *              ReleaseButton;
  QRadioButton *              PatchButton;
  QLabel *                    Label;
  QCheckBox *                 Chkbox;
  QPushButton *               Button;

  CoreAndCommonPluginsOrExtraPluginSet = new QButtonGroup;

  CoreAndCommonPluginsButton = new QRadioButton("core and common plugins");
  Layout->addWidget(CoreAndCommonPluginsButton,0,0,1,2);
  CoreAndCommonPluginsOrExtraPluginSet->addButton(CoreAndCommonPluginsButton);
  registerField("core_and_common_plugins",CoreAndCommonPluginsButton);
  connect(CoreAndCommonPluginsButton,SIGNAL(clicked())
         ,this,SIGNAL(completeChanged()));

  ExtraPluginSetButton = new QRadioButton("plugin set");
  Layout->addWidget(ExtraPluginSetButton,1,0,1,1);
  CoreAndCommonPluginsOrExtraPluginSet->addButton(ExtraPluginSetButton);
  registerField("extra_plugin_set",ExtraPluginSetButton);
  ExtraPluginSetButton->setChecked(true);
  connect(ExtraPluginSetButton,SIGNAL(clicked())
         ,this,SIGNAL(completeChanged()));

  PluginSetName = new QLineEdit("VB");
  PluginSetName->setMaximumWidth(60);
  Layout->addWidget(PluginSetName,1,1,1,1);
  registerField("PluginSetName",PluginSetName);
  connect(PluginSetName,SIGNAL(editingFinished())
         ,this,SIGNAL(completeChanged()));

  ReleaseOrPatch = new QButtonGroup;

  ReleaseButton = new QRadioButton("release");
  Layout->addWidget(ReleaseButton,0,4,1,1);
  ReleaseOrPatch->addButton(ReleaseButton);
  registerField("release",ReleaseButton);
  connect(ReleaseButton,SIGNAL(clicked()),this,SIGNAL(completeChanged()));

  PatchButton = new QRadioButton("patch");
  Layout->addWidget(PatchButton,1,4,1,1);
  ReleaseOrPatch->addButton(PatchButton);
  registerField("patch",PatchButton);
  PatchButton->setChecked(true);
  connect(PatchButton,SIGNAL(clicked()),this,SIGNAL(completeChanged()));

  Row = 1;

  Label = new QLabel("_____________________________________________");
  Layout->addWidget(Label,++Row,0,1,6);

  Label = new QLabel("Include");
  Layout->addWidget(Label,++Row,0,1,1);

  Label = new QLabel("Version");
  Layout->addWidget(Label,Row,1,1,1);

  Label = new QLabel("Translations");
  Layout->addWidget(Label,Row,2,1,3,Qt::AlignCenter);

  Chkbox = new QCheckBox("RA core");
  Layout->addWidget(Chkbox,++Row,0,1,1);
  registerField(QString("ra_core"),Chkbox);
  connect(Chkbox,SIGNAL(clicked()),this,SIGNAL(completeChanged()));
  connect(ReleaseButton,SIGNAL(toggled(bool)),Chkbox,SLOT(setChecked(bool)));

  Label = new QLabel;
  if( ! Job->Core ){
    Label->setText("---");
  }else{
    Label->setText(ToQString(Job->Core->Version.Valuable().Text()));
  }
  if( ! Job->Core->Version.IsValidAndNumeric() ){
    Label->setStyleSheet("color: red");
  }else if(  ! Job->Core->Todays ){
    Label->setStyleSheet("color: brown");
  }else{
    Label->setStyleSheet("color: green");
  }
  Layout->addWidget(Label,Row,1,1,1,Qt::AlignCenter);

  Label = new QLabel;
  Label->setObjectName(QString("TrInfo_RA"));
  Layout->addWidget(Label,Row,3,1,1,Qt::AlignRight);
  UpdateTrInfo("RA");

  Button = new QPushButton("translate");
  Layout->addWidget(Button,Row,4,1,1);
  connect(Button, SIGNAL(clicked()), TranslateRequestMapper, SLOT(map()));
  TranslateRequestMapper->setMapping(Button, "RA");

  for( QStringList::const_iterator  P = Job->Plugins.constBegin()
                                  ; P != Job->Plugins.constEnd(); ++P ){
    Row++;
    Chkbox = new QCheckBox( *P );
    Layout->addWidget(Chkbox,Row,0,1,1);
    registerField(QString("rap_")+(*P),Chkbox);
    if( ! Job->Plugin.count(*P) ){
      Chkbox->setEnabled(false);
      Chkbox->setStyleSheet("color: red");
    }else{
      connect(Chkbox,SIGNAL(clicked()),this,SIGNAL(completeChanged()));
      connect(ReleaseButton,SIGNAL(toggled(bool))
             ,Chkbox,SLOT(setChecked(bool)));
      Label = new QLabel;
      Label->setText(ToQString(Job->Plugin[*P]->Version.Valuable().Text()));
      if( ! Job->Plugin[*P]->Version.IsValidAndNumeric() ){
        Label->setStyleSheet("color: red");
      }else if( ! Job->Plugin[*P]->Todays ){
        Label->setStyleSheet("color: brown");
      }else{
        Label->setStyleSheet("color: green");
      }
      Layout->addWidget(Label,Row,1,1,1,Qt::AlignCenter);

      Label = new QLabel;
      Label->setObjectName(QString("TrInfo_")+(*P));
      Layout->addWidget(Label,Row,3,1,1,Qt::AlignRight);
      UpdateTrInfo(*P);

      Button = new QPushButton("translate");
      Layout->addWidget(Button,Row,4,1,1);
      connect(Button, SIGNAL(clicked()), TranslateRequestMapper, SLOT(map()));
      TranslateRequestMapper->setMapping(Button, *P);
    }
  }
  connect(TranslateRequestMapper ,SIGNAL(mapped(const QString &))
         ,this ,SLOT(Translate(const QString &)));
}


bool  sSelectPage::isComplete () const
{
  PluginSetName->setVisible(field("extra_plugin_set").toBool());
  PluginSetName->setText(PluginSetName->text().simplified());
  if( field("core_and_common_plugins").toBool() ){
    Job->DistBasename = "RA";
  }else{
    Job->DistBasename = "RA-PS-" + PluginSetName->text();
  }
  if( field("core_and_common_plugins").toBool()
      && field("release").toBool()
      && ! field("ra_core").toBool() )
    return false;
  if( field("extra_plugin_set").toBool() && field("ra_core").toBool() )
    return false;
  if( field("extra_plugin_set").toBool() && PluginSetName->text().isEmpty() )
    return false;
  if( field("core_and_common_plugins").toBool() &&  field("ra_core").toBool() )
    return true;
  for( QStringList::const_iterator  P = Job->Plugins.constBegin()
                                  ; P != Job->Plugins.constEnd(); ++P ){
    if( field(QString("rap_")+(*P)).toBool() )
      return true;
  }
  return false;
}


void  sSelectPage::UpdateTrInfo (const QString &  name)
{
  QLabel *                    Label;
  Label = findChild<QLabel*>(QString("TrInfo_")+name);
  if( Label ){
    bool                        TranslationExists = false;
    bool                        TranslationOK = true;
    QDir                        LocalesDir;
    (LocalesDir = Job->RADir).cd("locales");
    LocalesDir.setNameFilters( QStringList() << name + ".ru_RU.qm" );
    foreach( QString FName, LocalesDir.entryList(QDir::Files) ){
      TranslationExists = true;
      QFileInfo                   TrFileInfo(LocalesDir,FName);
      QString                     DllFName;
      if( name == "RA" ){
        DllFName = "RA";
        #ifdef RADist_DEBUG
          DllFName += "d";
        #endif
        if( Job->System == "win32" ){
          DllFName += ".exe";
        }
      }else{
        if( Job->System == "unix" ){
          DllFName = QString("librap")+FName.mid(0,FName.length()-6)+".so";
        }else{
          DllFName = QString("rap")+FName.mid(0,FName.length()-6)+".dll";
        }
      }
      QFileInfo                   DllFileInfo(Job->BinDir,DllFName);
      TranslationOK = TranslationOK
          && (TrFileInfo.lastModified() > DllFileInfo.lastModified());
    }
    if( ! TranslationExists ){
      Label->setText("absent");
      Label->setStyleSheet("color: red");
    }else if( ! TranslationOK ){
      Label->setText("check");
      Label->setStyleSheet("color: brown");
    }else{
      Label->setText("OK");
      Label->setStyleSheet("color: green");
    }
  }
}


void  sSelectPage::Translate (const QString &  name)
{
  QString                     Pro;
  if( name == "RA" ){
    Pro = QString("..") + QDir::separator() + "RA.pro";
  }else{
    Pro = QString("..") + QDir::separator()
        + "RAPlugins" + QDir::separator()
        + name + QDir::separator()
        + name + ".tr.pro";
  }
  Job->Report = Report;
  if( Job->System == "unix" ){
    if( ! Job->Execute( QLibraryInfo::location(QLibraryInfo::BinariesPath)
                          + QDir::separator() + "lupdate"
                      , QStringList()
//                          << QString("-no-obsolete")
                          << QString("-noobsolete")
                          << Pro
                      , Job->RADir.absoluteFilePath("locales")  ) )
      return;
  }else if( Job->System == "win32" ){
    if( ! Job->Execute( "cmd"
                      , QStringList()
                          << "/c"
                          << QLibraryInfo::location(QLibraryInfo::BinariesPath)
                              + QDir::separator() + "lupdate"
                              + ( Job->System=="win32" ? ".exe" : "" )
                          << QString("-noobsolete")
                          << Pro
                      , Job->RADir.absoluteFilePath("locales")  ) )
      return;
  }
  if( ! Job->Execute( QLibraryInfo::location(QLibraryInfo::BinariesPath)
                + QDir::separator() + "linguist"
                , QStringList()
                    << name + ".ru_RU.ts"
                , Job->RADir.absolutePath() + QDir::separator() + "locales" ) )
    return;
  if( ! Job->Execute( QLibraryInfo::location(QLibraryInfo::BinariesPath)
                + QDir::separator() + "lrelease"
                , QStringList()
                    << name + ".ru_RU.ts"
                    << "-qm"
                    << name + ".ru_RU.qm"
                , Job->RADir.absolutePath() + QDir::separator() + "locales" ) )
    return;
  UpdateTrInfo(name);
}

//---------------------------------------------------------------- sMakePage ---

sMakePage::sMakePage (psDistribJob  job)
    :sDoItPage(sDistWizard::MakePage
              ,"Ready to make the distributive"
              ,"Make it"
              ,"Making...")
    ,Job(job)
{
}


void  sMakePage::initializePage ()
{
  bool                        OK = true;
  sDoItPage::initializePage();
  Report->clear();
  OK = Job->MakeDirs(field("release").toBool());
  OutHTML(QString("<font color=navy>")
         +"<p>собираюсь сварганить дистрибутив:</p>"
         +"<li>"
         +( Job->Testing ? "<ul>TESTING MODE</ul>" : "" )
         +"<ul>"
         +( field("release").toBool() ? "release" : "patch")
         +"</ul>"
         +"<ul>"
         +( Job->Testing
          ? Job->DistDir.absolutePath()
          : Job->DistributivesDir.absoluteFilePath(Job->DistBasenameWithVersion)
                            + ( Job->System=="unix" ? ".run" : ".exe" )
          )
         +"</ul>"
         +"</li>"
         +"<p>___________________________________________________________</p>"
         +"</font>\n\n"
     );
  if( ! OK ){
    OutHTML(QString("<font color=red size=+6>")
           +"<p>Oops!!!</p>"
           +"<p>___________________________________________________________</p>"
           +"</font>\n\n"
       );
  }
}


bool  sMakePage::DoIt ()
{
  Job->Report = Report;
  if( Job->Core ){
    Job->Core->Selected = field("ra_core").toBool();
  }
  for( QStringList::const_iterator  P = Job->Plugins.constBegin()
                                  ; P != Job->Plugins.constEnd(); ++P ){
    if( Job->Plugin.count(*P) ){
      Job->Plugin[*P]->Selected = field(QString("rap_")+(*P)).toBool();
    }
  }
  if( Job->MakeIt(field("release").toBool()) ){
    OutHTML(QString("<font color=green><p><br></p>")
           +"<p>___________________________________________________________</p>"
           +"<p><font size=+2><b>Congratulations!</b></font></p>"
           +"<p>Distributive: "
           +( Job->Testing
            ? Job->DistDir.absolutePath()
            : Job->DistributivesDir.absoluteFilePath(Job->DistBasenameWithVersion)
                             + ( Job->System=="unix" ? ".run" : ".exe" )
            )
           +".</p>"
           +"<p>___________________________________________________________</p>"
           +"<p><br></p>"
         );
    return true;
  }
  OutHTML(QString("<font color=red><p><br></p>")
         +"<p>___________________________________________________________</p>"
         +"<p><font size=+2><b>Oops!</b></font></p>"
         +"<p>Poor you...  <b>:-(</b></p>"
         +"<p>___________________________________________________________</p>"
         +"<p><br></p>"
       );
  return false;
}

//------------------------------------------------------------------------------
} //namespace RA


