////////////////////////////////////////////////////////////////////////////////
/*! @file DistribJob.cpp
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2011-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          VBursian@gmail.com
*///////////////////////////////////////////////////////////////////////////////
#include "DistribJob.h"
#include "StringsAndQStrings.h"
#include "RAPlugins.h"
#include <QLibraryInfo>
#include <QDateTime>
#include <QPluginLoader>
#include <QDebug>
#include "Trace.h"
namespace RA {
//------------------------------------------------------------------------------

const QStringList   sDistribJob::CoreFiles4unix
                        ( QStringList()
<< "r"
                          << "RA.sh"
                          << "RAdmin.sh"
                          << "bin/RA"
                          << "bin/RAdmin"
                          << "bin/libRA*.so*"
                          << "desktop/install-desktop.sh"
                          << "desktop/RA.desktop"
                        );

const QStringList   sDistribJob::CoreFiles4win32
                        ( QStringList()
                          << "RA.bat"
                          << "RAdmin.bat"
                          << "bin/RA.exe"
                          << "bin/RAdmin.exe"
                          << "bin/RANet*.dll"
                          << "bin/RAGUI*.dll"
                        );

const QStringList   sDistribJob::CoreFiles4all
                        ( QStringList()
                          << "-xr!*.bak"
                          << "-xr!*~"
                          << "extrafiles/core"
                          << "help"
                          << "-x!help/*.qhcp"
                          << "-x!help/*.qhp"
//                          << "help/RA.qch"
//                          << "help/RA.qhc"
//                          << "help/images"
//                          << "help/*.html"
                          << "locales/RA.*.qm"
//                          << "HistoryLog.txt"
                          << "License.html"
                          << "Manual.html"
                        );

const QStringList   sDistribJob::CoreSources
                        ( QStringList()
                          << "-xr!*.bak"
                          << "-xr!*~"
//                          << "HistoryLog.txt"
                          << "License.html"
                          << "Manual.html"
                          << "RA.sh"
                          << "RA.bat"
                          << "RAdmin.sh"
                          << "RAdmin.bat"
                          << "RA.pro"
                          << "RA_common*.pri"
                          << "desktop"
                          << "docs"
                          << "extrafiles/core"
                          << "help"
                          << "-x!locales/*.qm"
                          << "locales/RA.*"
                          << "locales/*.sh"
                          << "locales/*.bat"
                          << "locales/*.txt"
                          << "RADist"
                          << "RAdmin"
                          << "RAGUI"
                          << "RAInst"
                          << "RAMain"
                          << "RANet"
                          << "RAPlugins/RAPlugins.pro"
                        );

/* %1 - plugin name
*/
const QStringList   sDistribJob::PluginFiles4unix
                        ( QStringList()
                          << "bin/librap%1.so"
                        );

/* %1 - plugin name, %2 - version.Major
*/
const QStringList   sDistribJob::PluginFiles4win32
                        ( QStringList()
                          << "bin/rap%1%2.dll"
                        );

/* %1 - plugin name
*/
const QStringList   sDistribJob::PluginFiles4all
                        ( QStringList()
                          << "locales/%1.*.qm"
                          << "extrafiles/%1"
                        );

/* %1 - plugin name
*/
const QStringList   sDistribJob::PluginSources
                        ( QStringList()
                          << "locales/%1.*.ts"
                          << "extrafiles/%1"
                          << "-xr!RAPlugins/%1/*.bak"
                          << "-xr!RAPlugins/%1/*~"
                          << "RAPlugins/%1"
                        );

const QStringList   sDistribJob::QtBinaries4RA_unix
                        ( QStringList()
                          << "assistant"
                        );

const QStringList   sDistribJob::QtBinaries4RA_win32
                        ( QStringList()
                          << "Qt5Network.dll"
                          << "Qt5Sql.dll"
                          << "Qt5SerialPort.dll"
                          << "Qt5Multimedia.dll"
                          << "Qt5PrintSupport.dll"
                          << "assistant.exe"
                            // for assistant:
                            << "Qt5Help.dll"
                            << "Qt5MultimediaWidgets.dll"
                            << "Qt5OpenGL.dll"
/*???*/                            << "Qt5Qml.dll"
                            << "Qt5Quick.dll"
                            << "Qt5V8.dll"
                            << "Qt5WebChannel.dll"
                            << "Qt5WebKit.dll"
                            << "Qt5WebKitWidgets.dll"
                            << "Qt5CLucene.dll"
                            << "Qt5Sensors.dll"
                            << "Qt5Positioning.dll"
                        );

const QStringList   sDistribJob::QtLibraries4RA_unix
                        ( QStringList()
                          << "libQt5Network.so*"
                          << "libQt5Sql.so*"
                          << "libQt5SerialPort.so*"
                          << "libQt5Multimedia.so*"
                          << "libQt5PrintSupport.so*"
                            // for assistant:
                            << "libQt5Help.so*"
                            << "libQt5MultimediaWidgets.so*"
                            << "libQt5OpenGL.so*"
/*???*/                            << "libQt5Qml.so*"
                            << "libQt5Quick.so*"
                            << "libQt5V8.so*"
                            << "libQt5WebChannel.so*"
                            << "libQt5WebKit.so*"
                            << "libQt5WebKitWidgets.so*"
                            << "libQt5CLucene.so*"
                            << "libQt5Sensors.so*"
                            << "libQt5Positioning.so*"
                        );

//names without "lib" prefix and ".dll" or ".so" extension (added automatically)
const QStringList   sDistribJob::QtPlugins4install
                        ( QStringList()
                          << "sqldrivers/qsqlite"
//...........
                        );

const QStringList   sDistribJob::QtPlugins4RA
                        ( QStringList()
                          << sDistribJob::QtPlugins4install
                          << "sqldrivers/qsqlmysql"
                          << "audio/qtaudio_windows" //for windows
                          << "audio/qtmedia_pulse"   //for linux
                          << "printsupport/windowsprintersupport" //for windows
                          << "printsupport/cupsprintersupport"    //for linux
                            // for assistant:
//...........
                        );

//------------------------------------------------------------- sDistribJob ---

sDistribJob::sDistribJob (bool  testing)
    :sApplicationJob(testing)
{
  (RADir = AppDir).cdUp();
//  (ThirdPartyDir = RADir).cdUp();
//  ThirdPartyDir.cdUp();
//  ThirdPartyDir.cd("Libraries");
  (ThirdPartyDir = RADir).cd("../../Libraries");

//  #ifdef RANET_DEBUG
//    (BinDir = RADir).cd("bin-debug");
//  #else
    (BinDir = RADir).cd("bin");
//  #endif
  ZipProgDir = ThirdPartyDir.absoluteFilePath("7z") + QDir::separator();
}


bool  sDistribJob::AnalizeProject ()
{
  Core = new sProjectInfo::sPartInfo(RA::RANet::Version ,QString());
  QDir                        PluginsDir;
  (PluginsDir = RADir).cd("RAPlugins");
  foreach( QString FName, PluginsDir.entryList(QDir::Dirs) ){
    if( (FName != ".") && (FName != "..") ){
      Plugins << FName;
    }
  }
  foreach( QString FName, BinDir.entryList(QDir::Files) ){
    TRACE(FromQString(FName));
    QPluginLoader loader(BinDir.absoluteFilePath(FName));
    QObject *plugin = loader.instance();
    if (plugin) {
      TRACE("plugin");
      RA::psInstrumentalPackage  IP = qobject_cast<RA::psInstrumentalPackage>
                                                  (plugin);
      if( IP ){
        sProjectInfo::psPartInfo    Part = new sProjectInfo::sPartInfo
                                                   (IP->Version()
                                                   ,IP->UserFriendlyName());
        QFileInfo                   FInfo(BinDir,FName);
        Part->Todays = (QDateTime::currentDateTime().date()
                        == FInfo.lastModified().date());
        Plugin[ToQString(IP->TrueType()->Name()).mid(1)] = Part;
        TRACE(IP->TrueType()->Name());
        TRACE(IP->Version().Text());
        TRACE(FromQString(IP->UserFriendlyName()));
        TRACE(Part->Todays);
      }
    }
  }
  return true;
}


bool  sDistribJob::MakeDirs (bool  this_is_a_release)
{
  bool                        OK = true;
  DistributivesDir = RADir;
  if( Testing ){
    DistDir = AppDir;
  }else{
    OK = OK && DistributivesDir.cdUp();
    OK = OK && DistributivesDir.mkpath("distributives");
    OK = OK && DistributivesDir.cd("distributives");
    QString                     DistDirName;
    DistDirName = DistBasename + "."
                + ToQString(Core->Version.Principal().Text()) + ".";
    if( this_is_a_release ){
      if( Core->Version.Stage == -1 ){
        DistDirName += "beta-";
      }
      DistDirName += "release";
    }else{
      DistDirName += "patch_" + QDate::currentDate().toString("yyMMdd");
    }
    DistDir = QDir(DistributivesDir.absoluteFilePath(DistDirName));
    OK = OK && DistDir.removeRecursively();
    OK = OK && DistributivesDir.mkdir(DistDirName);
  }
  DistFilesDir = QDir(DistDir.absoluteFilePath("distfiles"));
  OK = OK && DistFilesDir.removeRecursively();
  OK = OK && DistDir.mkdir("distfiles");
  DistBasenameWithVersion = DistDir.dirName() + "." + System + "_Qt" + QT_VERSION_STR;
  return OK;
}


bool  sDistribJob::MakeIt (bool  this_is_a_release)
{
  if( this_is_a_release ){
    if( ! GenerateDox() )  return false;
  }
  if( ! CompileHelp() )  return false;
  if( Core && Core->Selected ){
    if( ! MakeCoreDist() )  return false;
    if( ! PackCoreSrc() )  return false;
  }
  for( QStringList::const_iterator  P = Plugins.constBegin()
                                  ; P != Plugins.constEnd(); ++P ){
    if( Plugin.count(*P) && Plugin[*P]->Selected ){
      if( ! MakePluginDist(*P) )  return false;
      if( ! PackPluginSrc(*P) )  return false;
    }
  }
  if( ! MakeQtFiles() )  return false;
  if( ! CopyFiles4Install(this_is_a_release) )  return false;
  if( (System == "win32") && ! CopyFonts() )  return false;
//  if( ! Copy3dPartySrcZips() )  return false;
  if( ! Testing && ! WrapIntoSFX() )  return false;
  return true;
}


bool  sDistribJob::GenerateDox ()
{
  bool                        OK = true;
  QDir                        DocsDir( RADir.absoluteFilePath("docs") );
  QDir                        DocsHtmlDir( DocsDir.absoluteFilePath("html") );
//  if( ! Testing ){
//    OK = OK && DocsHtmlDir.removeRecursively();
//    if( ! OK )
//      Out("\n____________________________________________________________\n"
//          "Failed to clear the existing docs/html");
//  }
  OK = OK && Execute( "doxygen" , QStringList()
                    , DocsDir.absolutePath());
  return OK;
}


bool  sDistribJob::CompileHelp ()
{
  return Execute( QLibraryInfo::location(QLibraryInfo::BinariesPath)
                + QDir::separator() + "qcollectiongenerator"
                , QStringList() << "RA.qhcp" << "-o" << "RA.qhc"
                , RADir.absoluteFilePath("help")  );
}


bool  sDistribJob::MakeCoreDist ()
{
  QStringList                 Files(CoreFiles4all);
  QString                     ZipName;
  ZipName = DistFilesDir.absoluteFilePath("RA.core.")
          + ToQString(Core->Version.Valuable().Text())
          + "." + System + ".zip";
  if( System == "unix" ){
    Files << CoreFiles4unix;
  }else if( System == "win32" ){
    Files << CoreFiles4win32;
  }
  return Zip( ZipName , Files , RADir.absolutePath() );
}


bool  sDistribJob::PackCoreSrc ()
{
  QString                     ZipName;
  ZipName = DistFilesDir.absoluteFilePath("RA.core.")
          + ToQString(Core->Version.Valuable().Text())
          + ".src.zip";
  return Zip( ZipName , CoreSources , RADir.absolutePath() );
}


bool  sDistribJob::MakePluginDist (QString  plugin)
{
  QString                     ZipName;
  QStringList                 Files;
  ZipName = DistFilesDir.absoluteFilePath("RA.")
          + plugin + "."
          + ToQString(Plugin[plugin]->Version.Valuable().Text())
          + "." + System + ".zip";
  if( System == "unix" ){
    for( QStringList::const_iterator
          F = sDistribJob::PluginFiles4unix.constBegin()
          ; F != sDistribJob::PluginFiles4unix.constEnd() ; ++F ){
      Files << (*F).arg(plugin);
    }
  }else if( System == "win32" ){
    for( QStringList::const_iterator
          F = sDistribJob::PluginFiles4win32.constBegin()
          ; F != sDistribJob::PluginFiles4win32.constEnd() ; ++F ){
      Files << (*F).arg(plugin)
                   .arg(QString().setNum(Plugin[plugin]->Version.Major));
    }
  }
  for( QStringList::const_iterator
        F = sDistribJob::PluginFiles4all.constBegin()
        ; F != sDistribJob::PluginFiles4all.constEnd() ; ++F ){
    Files << (*F).arg(plugin);
  }
  return Zip( ZipName , Files , RADir.absolutePath() );
}


bool  sDistribJob::PackPluginSrc (QString  plugin)
{
  QString                     ZipName;
  ZipName = DistFilesDir.absoluteFilePath("RA.")
          + plugin + "."
          + ToQString(Plugin[plugin]->Version.Valuable().Text())
          + ".src.zip";
  QStringList                 Files;
  for( QStringList::const_iterator
        F = sDistribJob::PluginSources.constBegin()
        ; F != sDistribJob::PluginSources.constEnd() ; ++F ){
    Files << (*F).arg(plugin);
  }
  return Zip( ZipName , Files , RADir.absolutePath() );
}


bool  sDistribJob::MakeQtFiles ()
{
  bool                        OK = true;
  QString                     ZipName;
  QStringList                 Files;
  ZipName = DistFilesDir.absolutePath()
          + QDir::separator() + "%1"
          + "." + System + ".zip";
  if( System == "unix" ){
    OK = OK && Zip( ZipName.arg("QtBinaries")
                  , QtBinaries4RA_unix
                  , QLibraryInfo::location(QLibraryInfo::BinariesPath) );
    OK = OK && Zip( ZipName.arg("QtLibraries")
                  , QtLibraries4RA_unix
                  , QLibraryInfo::location(QLibraryInfo::LibrariesPath) );
  }else if( System == "win32" ){
    OK = OK && Zip( ZipName.arg("QtBinaries")
                  , QtBinaries4RA_win32
                  , QLibraryInfo::location(QLibraryInfo::BinariesPath) );
  }
  foreach( QString Plugin , QtPlugins4RA ){
    QString                     SubDirName( QFileInfo(Plugin).dir().path() );
    QString                     FileName( QFileInfo(Plugin).baseName() );
    if( System == "unix" ){
      FileName = QString("lib") + FileName + ".so";
    }else if( System == "win32" ){
      FileName += ".dll";
    }
    Files << QString("plugins") + QDir::separator() + SubDirName
                                + QDir::separator() + FileName;
  }
  OK = OK && Zip( ZipName.arg("QtPlugins") , Files
                , QLibraryInfo::location(QLibraryInfo::PrefixPath) );
  return OK;
}


bool  sDistribJob::CopyFiles4Install (bool  this_is_a_release)
{
  bool                        OK = true;
  Out("\n____________________________________________________________\n");
  Out("Copying files for install:\n");
  OK = OK && Copy( QDir(ThirdPartyDir.absoluteFilePath("7z")) ,DistFilesDir
                 , QStringList() << ( System == "unix" ? "7za" : "7za.exe" ) );
  OK = OK && Copy( QDir(RADir.absoluteFilePath("help")) ,DistFilesDir
                 , QStringList() << "GNU_GPL_3_0.html" << "GNU_FDL_1_3.html" );
  OK = OK && Copy( QDir(RADir.absoluteFilePath("RADist")) ,DistFilesDir
                 , QStringList() << "Licenses(briefly).html"  );
  if( ! Testing ){
    OK = OK && Copy( QDir(RADir.absoluteFilePath("RADist")) ,DistDir
                   , QStringList() << ( System == "unix" ? "install.sh"
                                                         : "install.bat" ) );
    OK = OK && Copy( QDir(RADir.absoluteFilePath("bin")) ,DistFilesDir
                   , QStringList() << ( System == "unix" ? "RAInst"
                                                         : "RAInst.exe" ) );
  }
  OK = OK && Copy( QDir(QLibraryInfo::location( ( System == "unix"
                                                ? QLibraryInfo::LibrariesPath
                                                : QLibraryInfo::BinariesPath
                                                ) ))
                 , DistFilesDir  , QtLibraries );
  if( System == "unix" ){
    bool _OK_1 = true;
    bool _OK_2 = true;
    _OK_1 = _OK_1 && Copy( QDir("/usr/lib/i386-linux-gnu") , DistFilesDir
                                                           , GNULibraries );
    _OK_1 = _OK_1 && Copy( QDir("/usr/lib/i386-linux-gnu/mesa") , DistFilesDir
                                                               , GNULibraries );
    _OK_2 = _OK_2 && Copy( QDir("/lib/x86_64-linux-gnu") , DistFilesDir
                                                          , GNULibraries );
    _OK_2 = _OK_2 && Copy( QDir("/usr/lib/x86_64-linux-gnu") , DistFilesDir
                                                          , GNULibraries );
    OK = OK && ( _OK_1 || _OK_2 );
  }else if( System == "win32" ){
    OK = OK && Copy( QDir(QLibraryInfo::location(QLibraryInfo::BinariesPath))
                   , DistFilesDir  , GNULibraries );
  }
  OK = OK && Copy( QDir(QLibraryInfo::location(QLibraryInfo::PluginsPath))
                 , DistFilesDir
                 , QStringList() << "-x!platforms/*d.dll" << "platforms" );
  QString                     ZipName;
  ZipName = DistFilesDir.absolutePath()
          + QDir::separator() + "%1"
          + "." + System + ".zip";
  foreach( QString Plugin , QtPlugins4install ){
    QString                     SubDirName( QFileInfo(Plugin).dir().dirName() );
    QString                     FileName( QFileInfo(Plugin).baseName() );
    if( System == "unix" ){
      FileName = QString("lib") + FileName + ".so";
    }else if( System == "win32" ){
      FileName += ".dll";
    }
    OK = OK && Unzip( ZipName.arg("QtPlugins")
                    , QStringList() << QString("plugins")
                                     + QDir::separator() + SubDirName
                                     + QDir::separator() + FileName
                    , DistFilesDir.absolutePath() );
  }
  if( ! this_is_a_release ){
    OK = OK && DistFilesDir.remove( ZipName.arg("QtBinaries") );
    if( System == "unix" ){
      OK = OK && DistFilesDir.remove( ZipName.arg("QtLibraries") );
    }
    OK = OK && DistFilesDir.remove( ZipName.arg("QtPlugins") );
  }
  return OK;
}


bool  sDistribJob::CopyFonts ()
{
  return Zip( DistFilesDir.absoluteFilePath(QString("fonts.")+System+".zip")
            , QStringList() << "*"
            , ThirdPartyDir.absoluteFilePath("fonts") );
}


bool  sDistribJob::Copy3dPartySrcZips ()
{
  bool                        OK = true;
//  if( field("ra_core").toBool() ){
//    OK = OK && QFile::copy(
//        DistributivesDir.absoluteFilePath("RA.xtras/singleapplication.1.0.src.zip")
//        ,DistFilesDir.absoluteFilePath("singleapplication.1.0.src.zip")
//                        );
//    OK = OK && QFile::copy(
//        DistributivesDir.absoluteFilePath("RA.xtras/XtraQt.src.zip")
//        ,DistFilesDir.absoluteFilePath("XtraQt.src.zip")
//                        );
//  }
//  if( field("rap_COMPort").toBool() || field("rap_SerialPort").toBool() ){
//    OK = OK && QFile::copy(
//        DistributivesDir.absoluteFilePath("RA.xtras/QSerialDevice.0.2.0.src.zip")
//        ,DistFilesDir.absoluteFilePath("QSerialDevice.0.2.0.src.zip")
//                        );
//  }
//  if( field("rap_NIDAQmxDrivers").toBool() ){
//    OK = OK && QFile::copy(
//        DistributivesDir.absoluteFilePath("RA.xtras/NIDAQmx.src.zip")
//        ,DistFilesDir.absoluteFilePath("NIDAQmx.src.zip")
//                        );
//  }
  return OK;
}


bool  sDistribJob::WrapIntoSFX ()
{
  bool                        OK = true;
  if( System == "unix" ){
    DistributivesDir.remove(DistBasenameWithVersion + ".run");
    OK = OK && Execute( ThirdPartyDir
                               .absoluteFilePath("makeself-2.1.5/makeself.sh")
                      , QStringList()
                        << DistDir.dirName()
                        << DistBasenameWithVersion + ".run"
                        << "Research Assistant Installer"
                        << "./install.sh"
                  , DistributivesDir.absolutePath() );
  }else if( System == "win32" ){
    DistributivesDir.remove(DistBasenameWithVersion + ".7z");
    DistributivesDir.remove(DistBasenameWithVersion + ".exe");
    OK = OK && Zip( QString("..") + QDir::separator() + DistBasenameWithVersion + ".7z"
                  , QStringList()
                        << QString("*")
                  , DistDir.absolutePath() );
    OK = OK && Execute( "cmd"
                      , QStringList()
                        << "/c"
                        << "copy"
                        << "/b"
                        << QDir::toNativeSeparators(
                             ThirdPartyDir.absoluteFilePath("7z/7zsd.sfx")
                                                   )
                        << "+"
                        << QDir::toNativeSeparators(
                             RADir.absoluteFilePath("RADist/scenario.win32.txt")
                                                    )
                        << "+"
                        << DistBasenameWithVersion + ".7z"
                        << DistBasenameWithVersion + ".exe"
                      , DistributivesDir.absolutePath() );
    OK = OK && DistributivesDir.remove(DistBasenameWithVersion + ".7z");
  }
//  OK = OK && DistDir.removeRecursively();
  return OK;
}

//------------------------------------------------------------------------------
} //namespace RA
