////////////////////////////////////////////////////////////////////////////////
/*! @file AttrDlg.cpp   Class sAttrDlg.
- Part of RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "AttrDlg.h"
#include "StringsAndQStrings.h"
#include "Helper.h"
#include <QEvent>
namespace RA {
//------------------------------------------------------------------------------

const QString &  sAttrDlg::ErrorMsgOnAddAttr ()
{
  static QString      Text (tr("Cannot add this attribute."));
  return Text;
}

//------------------------------------------------------------------------------

sAttrDlg::~sAttrDlg()
{
}


sAttrDlg::sAttrDlg (QWidget *parent)
  :QDialog(parent)
{
  if( objectName().isEmpty() )
    setObjectName("AttrDlg");
  resize(400, 200);
  Layout = new QVBoxLayout(this);
  Layout->setObjectName(QString::fromUtf8("verticalLayout"));
  areaAttrName = new QFrame(this);
  areaAttrName->setObjectName(QString::fromUtf8("areaAttrName"));
  areaAttrName->setFrameShape(QFrame::StyledPanel);
  areaAttrName->setFrameShadow(QFrame::Raised);
  horizontalLayout = new QHBoxLayout(areaAttrName);
  horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));

  groupBox = new QGroupBox(areaAttrName);
  groupBox->setObjectName(QString::fromUtf8("groupBox"));
  gridLayout = new QGridLayout(groupBox);
  gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
  gridLayout->setContentsMargins(2, 2, 2, 2);
  gridLayout->setVerticalSpacing(0);

  btnNameless = new QRadioButton(groupBox);
  btnNameless->setObjectName(QString::fromUtf8("btnNameless"));
  gridLayout->addWidget(btnNameless, 0, 0, 1, 1);

  btnAtomic = new QRadioButton(groupBox);
  btnAtomic->setObjectName(QString::fromUtf8("btnAtomic"));
  gridLayout->addWidget(btnAtomic, 1, 0, 1, 1);

  fldAtomId = new QLabel(groupBox);
  fldAtomId->setObjectName(QString::fromUtf8("fldAtomId"));
  gridLayout->addWidget(fldAtomId, 1, 1, 1, 1);

  btnNumeric = new QRadioButton(groupBox);
  btnNumeric->setObjectName(QString::fromUtf8("btnNumeric"));
  gridLayout->addWidget(btnNumeric, 2, 0, 1, 1);

  fldNumber = new QSpinBox(groupBox);
  fldNumber->setObjectName(QString::fromUtf8("fldNumber"));
//  QSizePolicy sizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
  QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
  sizePolicy.setHorizontalStretch(0);
  sizePolicy.setVerticalStretch(0);
  sizePolicy.setHeightForWidth(fldNumber->sizePolicy().hasHeightForWidth());
  fldNumber->setSizePolicy(sizePolicy);
//  fldNumber->setMaximumSize(QSize(150, 16777215));
  fldNumber->setMinimumSize(QSize(200, 0));
  fldNumber->setRange(0,std::numeric_limits<int>::max());
  fldNumber->setValue(0);
  gridLayout->addWidget(fldNumber, 2, 1, 1, 1);

  btnSymbolic = new QRadioButton(groupBox);
  btnSymbolic->setObjectName(QString::fromUtf8("btnSymbolic"));
  gridLayout->addWidget(btnSymbolic, 3, 0, 1, 1);

  fldSymbolicName = new QLineEdit(groupBox);
  fldSymbolicName->setObjectName(QString::fromUtf8("fldSymbolicName"));
  gridLayout->addWidget(fldSymbolicName, 3, 1, 1, 1);

  horizontalLayout->addWidget(groupBox);

  Layout->addWidget(areaAttrName);

  ValueSpace =new QGridLayout();
  Layout->addLayout(ValueSpace);


//  areaValueType = new QFormLayout();
//  areaValueType->setObjectName(QString::fromUtf8("areaValueType"));
//  lblValue = new QLabel(this);
//  lblValue->setObjectName(QString::fromUtf8("lblValue"));
//
//  areaValueType->setWidget(0, QFormLayout::LabelRole, lblValue);
//
//  fldValue = new QSpinBox(this);
//  fldValue->setObjectName(QString::fromUtf8("fldValue"));
//
//  areaValueType->setWidget(0, QFormLayout::FieldRole, fldValue);
//
//  Layout->addLayout(areaValueType);

  verticalSpacer = new QSpacerItem(20,40
                                  ,QSizePolicy::Minimum,QSizePolicy::Expanding);

  Layout->addItem(verticalSpacer);

  buttonBox = new QDialogButtonBox(this);
  buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
  buttonBox->setOrientation(Qt::Horizontal);
  buttonBox->setStandardButtons(QDialogButtonBox::Cancel
                               |QDialogButtonBox::Ok
                               |QDialogButtonBox::Help);

  Layout->addWidget(buttonBox);

  Retranslate();
  connect(buttonBox,SIGNAL(accepted()),this,SLOT(OnOK()));
  connect(buttonBox,SIGNAL(rejected()),this,SLOT(reject()));
  connect(buttonBox,SIGNAL(helpRequested()),this,SLOT(ShowHelp()));
  connect(btnAtomic,SIGNAL(toggled(bool)),fldAtomId,SLOT(setVisible(bool)));
  connect(btnAtomic,SIGNAL(clicked()),fldAtomId,SLOT(setFocus()));
  connect(btnNumeric,SIGNAL(toggled(bool)),fldNumber,SLOT(setVisible(bool)));
  connect(btnNumeric,SIGNAL(clicked()),fldNumber,SLOT(setFocus()));
  connect(btnSymbolic,SIGNAL(toggled(bool)),fldSymbolicName,SLOT(setVisible(bool)));
  connect(btnSymbolic,SIGNAL(clicked()),fldSymbolicName,SLOT(setFocus()));

  btnAtomic->setEnabled(false);
  SetAttrName(sAttrName());
}


void  sAttrDlg::SetAttrName (sAttrName  name)
{
  TheAttrName=name;
  switch( TheAttrName.Type() ){
    case sAttrName::cNameless:
        btnNameless->setChecked(true);
        fldAtomId->setVisible(false);
        fldNumber->setVisible(false);
        fldSymbolicName->setVisible(false);
        break;
    case sAttrName::cNumeric:
        btnNumeric->setChecked(true);
        fldAtomId->setVisible(false);
        fldNumber->setVisible(true);
        fldNumber->setValue(TheAttrName.ToNumber());
        fldSymbolicName->setVisible(false);
        fldNumber->setFocus();
        break;
    case sAttrName::cSymbolic:
        btnSymbolic->setChecked(true);
        fldAtomId->setVisible(false);
        fldNumber->setVisible(false);
        fldSymbolicName->setVisible(true);
        fldSymbolicName->setText(ToQString(TheAttrName.ToString()));
        fldSymbolicName->setFocus();
        break;
//    case sAttrName::cAtomic:
//        btnAtomic->setChecked(true);
//        fldAtomId->setVisible(true);
//        fldAtomId->setText(literal(TheAttrName.TextWithoutTypeIndication()));
//        fldNumber->setVisible(false);
//        fldSymbolicName->setVisible(false);
//        break;
  }
}


void  sAttrDlg::SetAttrNameEnabled (bool  enable)
{
  areaAttrName->setEnabled(enable);
}


void  sAttrDlg::Retranslate ()
{
  groupBox->setTitle(tr("Attribute Name"));
  btnNameless->setText(tr("Nameless","type of an attribute name"));
  btnAtomic->setText(tr("Atomic","type of an attribute name"));
  btnNumeric->setText(tr("Numeric","type of an attribute name"));
  btnSymbolic->setText(tr("Symbolic","type of an attribute name"));
}


void  sAttrDlg::ShowHelp ()
{
  sHelper::ShowKeyId("sAttrDlg");
}


void  sAttrDlg::OnOK ()
{
  if( Validate() )
    accept();
}


bool  sAttrDlg::Validate ()
{
  if( btnAtomic->isChecked() ){
    return false;
  }else if( btnNumeric->isChecked() ){
    TheAttrName=sAttrName(fldNumber->value());
    return true;
  }else if( btnSymbolic->isChecked() ){
    if( ! fldSymbolicName->text().isEmpty() ){
      TheAttrName=sAttrName(FromQString(fldSymbolicName->text()));
      return true;
    }else{

      fldSymbolicName->setFocus();
      return false;
    };
  }else{
    TheAttrName=sAttrName();
    return true;
  }
}


void sAttrDlg::changeEvent(QEvent *  e)
{
  QDialog::changeEvent(e);
  switch (e->type()) {
    case QEvent::LanguageChange:
        Retranslate();
        break;
    default:
        break;
  }
}

//------------------------------------------------------------------------------
} //namespace RA

