////////////////////////////////////////////////////////////////////////////////
/*! @file BasicNodesUI.cpp Функционал навигатора, специфичный для
                           базовых типов терминальных объектов сети.
- Part of RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "BasicNodesUI.h"
#include "BasicNodes.h"
#include "Navigator.h"
#include "EditNoteDialog.h"
#include "EditIntDialog.h"
#include "EditPhysValueDialog.h"
#include "EditPhysRangeDialog.h"
#include "NetExceptions.h"
#include <QMessageBox>
namespace RA {
//------------------------------------------------------------------------------

//-------------------------------------------------------------- sTextNodeUI ---

sTextNodeUI::sTextNodeUI ()
{
  sNavigatorViewport::MenuGenerators.push_front(&MenuGenerator);
}


void  sTextNodeUI::MenuGenerator (psNetViewMenu                context_menu
                                 ,sNet2TreeEvolution::pcsItem  tree_item)
{
  if( tree_item->AttrValue()->TrueType() == sTextNode::Type() )
    context_menu->Insert( new sNetViewAction
        (context_menu->parentWidget()
        ,&EditValue  ,/*checkable*/false ,/*checked*/false
        ,sNetViewMenu::tr("Edit note")
        ,sNetViewMenu::tr("View and edit the content of the note")
        ));
  if( sFolderNode::Type()->CanCast(tree_item->AttrValue().operator ->()) ){
    context_menu->AddNew->addAction( new sNetViewAction
        (context_menu->parentWidget()
        ,&AddNew  ,/*checkable*/false ,/*checked*/false
        ,sNetViewMenu::tr("note")
        ,sNetViewMenu::tr("Add new note")
        ));
  };
  if( tree_item->Parent() ){
    context_menu->InsertNew->addAction( new sNetViewAction
        (context_menu->parentWidget()
        ,&InsertNew  ,/*checkable*/false ,/*checked*/false
        ,sNetViewMenu::tr("note")
        ,sNetViewMenu::tr("Insert new note")
        ));
  };
}


void  sTextNodeUI::EditValue (sNet2TreeEvolution::pcsItem  tree_item
                                ,sPoint                       /*pos*/)
{
  /*! @todo{UI++} in-place editing */
  sEditNoteDialog             D;
  D.setWindowTitle( sEditNoteDialog::tr("Edit the note") );
  D.SetAttrName(tree_item->AttrName());
  D.SetAttrNameEnabled(false);
  sPtr<sTextNode>             SV(tree_item->AttrValue());
  ASSERT(SV.IsCorrect());
  D.SetValue(*SV);
  if( D.exec() ){
    SV->Assign(D.Value());
  };
}


void  sTextNodeUI::AddNew (sNet2TreeEvolution::pcsItem  tree_item
                          ,sPoint                       /*pos*/)
{
  sEditNoteDialog             D;
  D.setWindowTitle( sEditNoteDialog::tr("Add a note") );
  D.SetValue("");
  while( D.exec() ){
    try{
      tree_item->AttrValue()<<sAttribute(D.AttrName()
                                        ,sNodePtr(new sTextNode(D.Value())) );
      break;
    }catch( rxNetInvalidOp ){
      QMessageBox::critical(NULL ,D.windowTitle() ,D.ErrorMsgOnAddAttr() );
    }
  }
}


void  sTextNodeUI::InsertNew (sNet2TreeEvolution::pcsItem  tree_item
                             ,sPoint                       /*pos*/)
{
  sEditNoteDialog             D;
  D.setWindowTitle( sEditNoteDialog::tr("Insert a note") );
  D.SetValue("");
  sNodePtr                    NewNode;
  while( D.exec() ){
    try{
      NewNode = sNodePtr(new sTextNode(D.Value()));
      tree_item->AttrValue()<<sAttribute(D.AttrName(),NewNode);
      break;
    }catch( rxNetInvalidOp ){
      QMessageBox::critical(NULL ,D.windowTitle() ,D.ErrorMsgOnAddAttr() );
    }
  }
  //! @todo{Insert_attribute} move NewNode
}

//----------------------------------------------------------- sIntegerNodeUI ---

sIntegerNodeUI::sIntegerNodeUI ()
{
  sNavigatorViewport::MenuGenerators.push_front(&MenuGenerator);
}


void  sIntegerNodeUI::MenuGenerator (psNetViewMenu                context_menu
                                    ,sNet2TreeEvolution::pcsItem  tree_item)
{
  if( tree_item->AttrValue()->TrueType() == sIntegerNode::Type() )
    context_menu->Insert( new sNetViewAction
        (context_menu->parentWidget()
        ,&EditValue  ,/*checkable*/false ,/*checked*/false
        ,sNetViewMenu::tr("Edit integer")
        ,sNetViewMenu::tr("View and edit the integer attribute")
        ));
  if( sFolderNode::Type()->CanCast(tree_item->AttrValue().operator ->()) ){
    context_menu->AddNew->addAction( new sNetViewAction
        (context_menu->parentWidget()
        ,&AddNew  ,/*checkable*/false ,/*checked*/false
        ,sNetViewMenu::tr("integer")
        ,sNetViewMenu::tr("Add new integer attribute")
        ));
  };
  if( tree_item->Parent() ){
    context_menu->InsertNew->addAction( new sNetViewAction
        (context_menu->parentWidget()
        ,&InsertNew  ,/*checkable*/false ,/*checked*/false
        ,sNetViewMenu::tr("integer")
        ,sNetViewMenu::tr("Insert new integer attribute")
        ));
  };
}


void  sIntegerNodeUI::EditValue (sNet2TreeEvolution::pcsItem  tree_item
                                ,sPoint                       /*pos*/)
{
  /*! @todo{UI++} in-place editing */
  sEditIntDialog              D;
  D.setWindowTitle( sEditIntDialog::tr("Edit integer") );
  D.SetAttrName(tree_item->AttrName());
  D.SetAttrNameEnabled(false);
  sPtr<sIntegerNode>          IV(tree_item->AttrValue());
  ASSERT(IV.IsCorrect());
  D.SetValue((long int)(*IV));
  if( D.exec() ){
    IV->Assign(D.Value());
  };
}


void  sIntegerNodeUI::AddNew (sNet2TreeEvolution::pcsItem  tree_item
                             ,sPoint                       /*pos*/)
{
  sEditIntDialog              D;
  D.setWindowTitle( sEditIntDialog::tr("Add integer attribute") );
  while( D.exec() ){
    try{
      tree_item->AttrValue()
            <<sAttribute( D.AttrName()
                        , sNodePtr(new sIntegerNode(D.Value())) );
      break;
    }catch( rxNetInvalidOp ){
      QMessageBox::critical(NULL ,D.windowTitle() ,D.ErrorMsgOnAddAttr() );
    }
  }
}


void  sIntegerNodeUI::InsertNew (sNet2TreeEvolution::pcsItem  tree_item
                                ,sPoint                       /*pos*/)
{
  sEditIntDialog              D;
  D.setWindowTitle( sEditIntDialog::tr("Insert integer attribute") );
  sNodePtr                    NewNode;
  while( D.exec() ){
    try{
      NewNode = sNodePtr(new sIntegerNode(D.Value()));
      tree_item->AttrValue()<<sAttribute(D.AttrName(),NewNode);
      break;
    }catch( rxNetInvalidOp ){
      QMessageBox::critical(NULL ,D.windowTitle() ,D.ErrorMsgOnAddAttr() );
    }
  }
  //! @todo{Insert_attribute} move NewNode
}

//----------------------------------------------------------- sBooleanNodeUI ---

sBooleanNodeUI::sBooleanNodeUI ()
{
  sNavigatorViewport::MenuGenerators.push_front(&MenuGenerator);
}


void  sBooleanNodeUI::MenuGenerator (psNetViewMenu                context_menu
                                    ,sNet2TreeEvolution::pcsItem  tree_item)
{
  if( tree_item->AttrValue()->TrueType() == sBooleanNode::Type() )
    context_menu->Insert( new sNetViewAction
        (context_menu->parentWidget()
        ,&EditValue  ,/*checkable*/false ,/*checked*/false
        ,tr("Edit boolean")
        ,tr("View and edit the boolean attribute")
        ));
  if( sFolderNode::Type()->CanCast(tree_item->AttrValue().operator ->()) ){
    context_menu->AddNew->addAction( new sNetViewAction
        (context_menu->parentWidget()
        ,&AddNew  ,/*checkable*/false ,/*checked*/false
        ,tr("boolean")
        ,tr("Add new boolean attribute")
        ));
  };
  if( tree_item->Parent() ){
    context_menu->InsertNew->addAction( new sNetViewAction
        (context_menu->parentWidget()
        ,&InsertNew  ,/*checkable*/false ,/*checked*/false
        ,tr("boolean")
        ,tr("Insert new boolean attribute")
        ));
  };
}


void  sBooleanNodeUI::EditValue (sNet2TreeEvolution::pcsItem  tree_item
                                ,sPoint                       /*pos*/)
{
  /*! @todo{UI++} in-place editing */
  sPtr<sBooleanNode>          V(tree_item->AttrValue());
  ASSERT(V.IsCorrect());
  V->Assign( ! (bool)(*V) );
}


void  sBooleanNodeUI::AddNew (sNet2TreeEvolution::pcsItem  tree_item
                             ,sPoint                       /*pos*/)
{
  sAttrDlg                    D;
  D.setWindowTitle( tr("Add boolean attribute") );
  while( D.exec() ){
    try{
      tree_item->AttrValue()
            <<sAttribute( D.AttrName()
                        , sNodePtr(new sBooleanNode()) );
      break;
    }catch( rxNetInvalidOp ){
      QMessageBox::critical(NULL ,D.windowTitle() ,D.ErrorMsgOnAddAttr() );
    }
  }
}


void  sBooleanNodeUI::InsertNew (sNet2TreeEvolution::pcsItem  tree_item
                                ,sPoint                       /*pos*/)
{
  sAttrDlg                    D;
  D.setWindowTitle( tr("Insert boolean attribute") );
  sNodePtr                    NewNode;
  while( D.exec() ){
    try{
      NewNode = sNodePtr(new sBooleanNode());
      tree_item->AttrValue()<<sAttribute(D.AttrName(),NewNode);
      break;
    }catch( rxNetInvalidOp ){
      QMessageBox::critical(NULL ,D.windowTitle() ,D.ErrorMsgOnAddAttr() );
    }
  }
  //! @todo{Insert_attribute} move NewNode
}

//--------------------------------------------------------- sPhysValueNodeUI ---

sPhysValueNodeUI::sPhysValueNodeUI ()
{
  sNavigatorViewport::MenuGenerators.push_front(&MenuGenerator);

  //!@todo{UI!} qRegisterMetaType not in a right place (requered for signals)
  qRegisterMetaType<RA::sPhysValue>("sPhysValue");
  qRegisterMetaType<RA::sPhysPair> ("sPhysPair");
  qRegisterMetaType<RA::sPhysRange>("sPhysRange");
  qRegisterMetaType<RA::sString>   ("sString");
}


void  sPhysValueNodeUI::MenuGenerator (psNetViewMenu                context_menu
                                      ,sNet2TreeEvolution::pcsItem  tree_item)
{
//  if( tree_item->AttrValue()->TrueType() == sPhysValueNode::Type() ){
  if( sPhysValueNode::Type()->CanCast(tree_item->AttrValue().operator ->()) ){
    context_menu->Insert( new sNetViewAction
        (context_menu->parentWidget()
        ,&EditValue  ,/*checkable*/false ,/*checked*/false
        ,sNetViewMenu::tr("Edit phys.value")
        ,sNetViewMenu::tr("View and edit the real attribute")
        ));
  };
  if( sFolderNode::Type()->CanCast(tree_item->AttrValue().operator ->()) ){
    context_menu->AddNew->addAction( new sNetViewAction
        (context_menu->parentWidget()
        ,&AddNew  ,/*checkable*/false ,/*checked*/false
        ,sNetViewMenu::tr("phys.value")
        ,sNetViewMenu::tr("Add new phys.value attribute")
        ));
  };
  if( tree_item->Parent() ){
    context_menu->InsertNew->addAction( new sNetViewAction
        (context_menu->parentWidget()
        ,&InsertNew  ,/*checkable*/false ,/*checked*/false
        ,sNetViewMenu::tr("phys.value")
        ,sNetViewMenu::tr("Insert new phys.value attribute")
        ));
  };
}


void  sPhysValueNodeUI::EditValue (sNet2TreeEvolution::pcsItem  tree_item
                                  ,sPoint                       /*pos*/)
{
  /*! @todo{UI++} in-place editing */
  sPtr<sPhysValueNode>        PV(tree_item->AttrValue());
  ASSERT(PV.IsCorrect());
  sEditPhysValueDialog        D((sPhysValue)(*PV));
  D.setWindowTitle( sEditPhysValueDialog::tr("Edit phys.value") );
  D.SetAttrName(tree_item->AttrName());
  D.SetAttrNameEnabled(false);
  if( D.exec() ){
    PV->Assign(D.Value());
  };
}


void  sPhysValueNodeUI::AddNew (sNet2TreeEvolution::pcsItem  tree_item
                               ,sPoint                       /*pos*/)
{
  sEditPhysValueDialog        D( sPhysValue() , NULL );
  D.setWindowTitle( sEditPhysValueDialog::tr("Add phys.value attribute") );
  while( D.exec() ){
    try{
      tree_item->AttrValue()
            <<sAttribute( D.AttrName()
                        , sNodePtr(new sPhysValueNode(D.Value())) );
      break;
    }catch( rxNetInvalidOp ){
      QMessageBox::critical(NULL ,D.windowTitle() ,D.ErrorMsgOnAddAttr() );
    }
  }
}


void  sPhysValueNodeUI::InsertNew (sNet2TreeEvolution::pcsItem  tree_item
                                  ,sPoint                       /*pos*/)
{
  sEditPhysValueDialog        D((sPhysValue()));
  D.setWindowTitle( sEditPhysValueDialog::tr("Insert phys.value attribute") );
  sNodePtr                    NewNode;
  while( D.exec() ){
    try{
      NewNode = sNodePtr(new sPhysValueNode(D.Value()));
      tree_item->AttrValue()<<sAttribute(D.AttrName(),NewNode);
      break;
    }catch( rxNetInvalidOp ){
      QMessageBox::critical(NULL ,D.windowTitle() ,D.ErrorMsgOnAddAttr() );
    }
  }
  //! @todo{Insert_attribute} move NewNode
}


//--------------------------------------------------------- sPhysRangeNodeUI ---

sPhysRangeNodeUI::sPhysRangeNodeUI ()
{
  sNavigatorViewport::MenuGenerators.push_front(&MenuGenerator);
}


void  sPhysRangeNodeUI::MenuGenerator (psNetViewMenu                context_menu
                                      ,sNet2TreeEvolution::pcsItem  tree_item)
{
  if( tree_item->AttrValue()->TrueType() == sPhysRangeNode::Type() )
    context_menu->Insert( new sNetViewAction
        (context_menu->parentWidget()
        ,&EditValue  ,/*checkable*/false ,/*checked*/false
        ,sNetViewMenu::tr("Edit range")
        ,sNetViewMenu::tr("View and edit the range attribute")
        ));
  if( sFolderNode::Type()->CanCast(tree_item->AttrValue().operator ->()) ){
    context_menu->AddNew->addAction( new sNetViewAction
        (context_menu->parentWidget()
        ,&AddNew  ,/*checkable*/false ,/*checked*/false
        ,sNetViewMenu::tr("range")
        ,sNetViewMenu::tr("Add new range attribute")
        ));
  };
  if( tree_item->Parent() ){
    context_menu->InsertNew->addAction( new sNetViewAction
        (context_menu->parentWidget()
        ,&InsertNew  ,/*checkable*/false ,/*checked*/false
        ,sNetViewMenu::tr("range")
        ,sNetViewMenu::tr("Insert new range attribute")
        ));
  };
}


void  sPhysRangeNodeUI::EditValue (sNet2TreeEvolution::pcsItem  tree_item
                                  ,sPoint                       /*pos*/)
{
  /*! @todo{UI++} in-place editing */
  sEditPhysRangeDialog        D;
  D.setWindowTitle( sEditPhysRangeDialog::tr("Edit range") );
  D.SetAttrName(tree_item->AttrName());
  D.SetAttrNameEnabled(false);
  sPtr<sPhysRangeNode>        Ptr(tree_item->AttrValue());
  ASSERT(Ptr.IsCorrect());
  D.SetValue((sPhysRange)(*Ptr));
  if( D.exec() ){
    Ptr->Assign(D.Value());
  };
}


void  sPhysRangeNodeUI::AddNew (sNet2TreeEvolution::pcsItem  tree_item
                               ,sPoint                       /*pos*/)
{
  sEditPhysRangeDialog        D;
  D.setWindowTitle( sEditPhysRangeDialog::tr("Add range attribute") );
  while( D.exec() ){
    try{
      tree_item->AttrValue()
            <<sAttribute( D.AttrName()
                        , sNodePtr(new sPhysRangeNode(D.Value())) );
      break;
    }catch( rxNetInvalidOp ){
      QMessageBox::critical(NULL ,D.windowTitle() ,D.ErrorMsgOnAddAttr() );
    }
  }
}


void  sPhysRangeNodeUI::InsertNew (sNet2TreeEvolution::pcsItem  tree_item
                                  ,sPoint                       /*pos*/)
{
  sEditPhysRangeDialog        D;
  D.setWindowTitle( sEditPhysRangeDialog::tr("Insert range attribute") );
  sNodePtr                    NewNode;
  while( D.exec() ){
    try{
      NewNode = sNodePtr(new sPhysRangeNode(D.Value()));
      tree_item->AttrValue()<<sAttribute(D.AttrName(),NewNode);
      break;
    }catch( rxNetInvalidOp ){
      QMessageBox::critical(NULL ,D.windowTitle() ,D.ErrorMsgOnAddAttr() );
    }
  }
  //! @todo{Insert_attribute} move NewNode
}

//------------------------------------------------------------------------------
} //namespace RA
