////////////////////////////////////////////////////////////////////////////////
/*! @file CSVImportDialog.cpp   CSV-file import as a sDataListFolder.
- Part of RAGUI - Research Assistant Graphical User Interface.
- Uses  Qt v.5  - http://qt.io/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2017, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "CSVImportDialog.h"
#include "Globals.h"
#include "StringsAndQStrings.h"
#include "attr.h"
#include <QFileInfo>
namespace RA {
//------------------------------------------------------------------------------

sCSVImportDialog::~sCSVImportDialog ()
{
}


sCSVImportDialog::sCSVImportDialog (QString    file_path
                                   ,QWidget *  parent)
  :sEditDocketDialog(parent)
  ,File(new QFile(file_path,this))
{
  //create widgets
  {
    DelimiterLabel = new QLabel(tr("Delimiter"),this);
    DelimiterField = new QComboBox(this);
    XColumnLabel = new QLabel(tr("Abscissa column"),this);
    XColumnField = new QSpinBox(this);
    YColumnLabel = new QLabel(tr(",  ordinate column"),this);
    YColumnField = new QSpinBox(this);
    ReadUnitsChkBox = new QCheckBox(tr("Take units from line"),this);
    UnitsLineField = new QSpinBox(this);
    AbscissaUnitsLabel = new QLabel(tr("Units for abscissa:"),this);
    AbscissaUnitsField = new QLineEdit(this);
    OrdinateUnitsLabel = new QLabel(tr(", for ordinate:"),this);
    OrdinateUnitsField = new QLineEdit(this);
    ValuesFirstLineLabel = new QLabel(tr("Values start at line"),this);
    ValuesFirstLineField = new QSpinBox(this);
    HeaderLabel = new QLabel(tr("Header:"),this);
    HeaderText = new QTextEdit(this);
    InfoLabel = new QLabel(this);
    FooterLabel = new QLabel(tr("Ignored footer:"),this);
    FooterText = new QTextEdit(this);
  }
  //layout
  {
    int                       I = layout()->indexOf(buttonBox) - 1;
    QHBoxLayout *             L;

    (L = new QHBoxLayout())->setAlignment(Qt::AlignLeft);
    ((QVBoxLayout*)layout())->insertItem(I++,L);
    L->addWidget(DelimiterLabel);
    L->addWidget(DelimiterField);

    (L = new QHBoxLayout())->setAlignment(Qt::AlignLeft);
    ((QVBoxLayout*)layout())->insertItem(I++,L);
    L->addWidget(XColumnLabel);
    L->addWidget(XColumnField);
    L->addWidget(YColumnLabel);
    L->addWidget(YColumnField);

    (L = new QHBoxLayout())->setAlignment(Qt::AlignLeft);
    ((QVBoxLayout*)layout())->insertItem(I++,L);
    L->addWidget(ReadUnitsChkBox);
    L->addWidget(UnitsLineField);

    L = new QHBoxLayout();
    ((QVBoxLayout*)layout())->insertItem(I++,L);
    L->setAlignment(Qt::AlignLeft);
    L->addWidget(AbscissaUnitsLabel);
    L->addWidget(AbscissaUnitsField);
    L->addWidget(OrdinateUnitsLabel);
    L->addWidget(OrdinateUnitsField);

    L = new QHBoxLayout();
    ((QVBoxLayout*)layout())->insertItem(I++,L);
    L->setAlignment(Qt::AlignLeft);
    L->addWidget(ValuesFirstLineLabel);
    L->addWidget(ValuesFirstLineField);

    ((QVBoxLayout*)layout())->insertWidget(I++,HeaderLabel);
    ((QVBoxLayout*)layout())->insertWidget(I++,HeaderText);
    ((QVBoxLayout*)layout())->insertWidget(I++,InfoLabel,0,Qt::AlignHCenter);
    ((QVBoxLayout*)layout())->insertWidget(I++,FooterLabel);
    ((QVBoxLayout*)layout())->insertWidget(I++,FooterText);
  }
  //initial settings
  {
    fldDocket->setText(QFileInfo(*File).completeBaseName());
    DelimiterField->addItem("comma");
    DelimiterField->addItem("tabulation");
    XColumnField->setValue(1);
    YColumnField->setValue(2);
    ReadUnitsChkBox->setChecked(true);
    UnitsLineField->setValue(2);
    RestoreSettings();
    AbscissaUnitsField->setEnabled( ! ReadUnitsChkBox->isChecked() );
    OrdinateUnitsField->setEnabled( ! ReadUnitsChkBox->isChecked() );
    ValuesFirstLineField->setMinimum( ( ReadUnitsChkBox->isChecked()
                                      ? UnitsLineField->value()+1
                                      : 1                            ) );
  }
  //behaviour
  {
    connect(DelimiterField ,SIGNAL(currentIndexChanged(int))
           ,this,SLOT(ParseOnly()));
    connect(XColumnField ,SIGNAL(valueChanged(int))
           ,this,SLOT(ParseOnly()));
    connect(YColumnField ,SIGNAL(valueChanged(int))
           ,this,SLOT(ParseOnly()));
    connect(ReadUnitsChkBox ,SIGNAL(toggled(bool))
           ,this,SLOT(OnEditReadUnitsChkBox()));
    connect(UnitsLineField ,SIGNAL(valueChanged(int))
           ,this,SLOT(OnEditUnitsLineField()));
    connect(ValuesFirstLineField ,SIGNAL(valueChanged(int))
           ,this,SLOT(OnEditValuesFirstLineField()));
  }
  ParseOnly();
}


void  sCSVImportDialog::RestoreSettings ()
{
  if( RAGUI::CSVImportSettings.IsCorrect() ){
    sString                   S,U;
    bool                      B;
    integer                   I;
    if( get_from<sIntegerNode>(RAGUI::CSVImportSettings>>"Delimiter",I) )
      DelimiterField->setCurrentIndex(I);
    if( get_from<sIntegerNode>(RAGUI::CSVImportSettings>>"XColumn",I) )
      XColumnField->setValue(I);
    if( get_from<sIntegerNode>(RAGUI::CSVImportSettings>>"YColumn",I) )
      YColumnField->setValue(I);
    if( get_from<sBooleanNode>(RAGUI::CSVImportSettings>>"ReadUnits",B) )
      ReadUnitsChkBox->setChecked(B);
    if( get_from<sIntegerNode>(RAGUI::CSVImportSettings>>"UnitsLine",I) )
      UnitsLineField->setValue(I);
    if( get_from<sIntegerNode>(RAGUI::CSVImportSettings>>"ValuesFirstLine",I))
      ValuesFirstLineField->setValue(I);
    if( get_from<sTextNode>(RAGUI::CSVImportSettings>>"AbscissaUnits",S) )
      AbscissaUnitsField->setText(ToQString(S));
    if( get_from<sTextNode>(RAGUI::CSVImportSettings>>"OrdinateUnits",S) )
      OrdinateUnitsField->setText(ToQString(S));
  }
}


void  sCSVImportDialog::StoreSettings ()
{
  if( RAGUI::CSVImportSettings.IsCorrect() ){
    RAGUI::CSVImportSettings
        <<attr("Delimiter",DelimiterField->currentIndex())
        <<attr("XColumn",XColumnField->value())
        <<attr("YColumn",YColumnField->value())
        <<attr("ReadUnits",ReadUnitsChkBox->isChecked())
        <<attr("UnitsLine",UnitsLineField->value())
        <<attr("ValuesFirstLine",ValuesFirstLineField->value())
        <<attr("AbscissaUnits",FromQString(AbscissaUnitsField->text()))
        <<attr("OrdinateUnits",FromQString(OrdinateUnitsField->text()))
        ;
  }
}


bool  sCSVImportDialog::Validate ()
{
  if( sEditDocketDialog::Validate() && ParseOnly() && ParseAndCreate() ){
    StoreSettings();
    return true;
  }
  return false;
}


void  sCSVImportDialog::OnEditReadUnitsChkBox ()
{
  UnitsLineField->setEnabled( ReadUnitsChkBox->isChecked() );
  AbscissaUnitsField->setEnabled( ! ReadUnitsChkBox->isChecked() );
  OrdinateUnitsField->setEnabled( ! ReadUnitsChkBox->isChecked() );
  ValuesFirstLineField->setMinimum( ( ReadUnitsChkBox->isChecked()
                                    ? UnitsLineField->value()+1
                                    : 1                            ) );
  ParseOnly();
}


void  sCSVImportDialog::OnEditUnitsLineField ()
{
  ValuesFirstLineField->setMinimum( UnitsLineField->value()+1 );
  ParseOnly();
}


void  sCSVImportDialog::OnEditValuesFirstLineField ()
{
  ReadUnitsChkBox->setChecked( ReadUnitsChkBox->isChecked()
      && (UnitsLineField->value() < ValuesFirstLineField->value() ) );
  ParseOnly();
}


bool  sCSVImportDialog::Parse (bool  create_record)
{
  char                        Delimiter = ( DelimiterField->currentIndex()
                                          ? '\t'
                                          : ',');
  int                         XColumn = XColumnField->value();
  int                         YColumn = YColumnField->value();
  int                         LineNo = 0;
  int                         PointNo = 0;
  bool                        FooterStarted = false;
  sUnits                      UX,UY;
  real                        X = 0 ,Y = 0;
  bool                        UX_OK = false;
  bool                        UY_OK = false;
  bool                        X_OK,Y_OK;
  HeaderText->clear();
  FooterText->clear();
  if( ! ReadUnitsChkBox->isChecked() ){
    UX = sUnits( FromQString(AbscissaUnitsField->text().simplified()) );
    UX_OK = true;
    UY = sUnits( FromQString(OrdinateUnitsField->text().simplified()) );
    UY_OK = true;
  }else{
    AbscissaUnitsField->setText("");
    OrdinateUnitsField->setText("");
  }
  InfoLabel->setText("");
  if( ! File->open(QIODevice::ReadOnly | QIODevice::Text) ){
    InfoLabel->setText(tr("Cannot open file for reading!"));
  }else{
    while( ! File->atEnd() ){
      QByteArray Line = File->readLine();
      LineNo++;
      if( LineNo < ValuesFirstLineField->value() ){
        if( ReadUnitsChkBox->isChecked()
            && (LineNo == UnitsLineField->value()) ){
          QList<QByteArray> Blocks( Line.split(Delimiter) );
          if( Blocks.size() >= XColumn ){
            UX = sUnits( Blocks.at(XColumn-1).simplified().constData() );
            AbscissaUnitsField->setText(ToQString(UX.Text()));
            UX_OK = true;
          }
          if( Blocks.size() >= YColumn ){
            UY = sUnits( Blocks.at(YColumn-1).simplified().constData() );
            OrdinateUnitsField->setText(ToQString(UY.Text()));
            UY_OK = true;
          }
          Line = QByteArray("<font color=")
               + ( UX_OK && UY_OK ? "green" : "red" )
               + ">" + Line + "</font>";
        }
        if( LineNo == 1 ){
          HeaderText->setHtml(Line);
        }else{
          HeaderText->setHtml(HeaderText->toHtml()+Line);
        }
      }else if( ! FooterStarted ){
        QList<QByteArray> Blocks( Line.split(Delimiter) );
        if( (X_OK = (Blocks.size() >= XColumn)) ){
          X = Blocks.at(XColumn-1).simplified().toDouble(&X_OK);
        }
        if( (Y_OK = (Blocks.size() >= YColumn)) ){
          Y = Blocks.at(YColumn-1).simplified().toDouble(&Y_OK);
        }
        if( X_OK && Y_OK ){
          if( create_record ){
            if( PointNo == 0 ){
              Value = sNodePtr(new sDataListFolder(FromQString(fldDocket
                                                                   ->text())));
              Value->SetMultiplier(sPhysPair(sPhysValue(UX),sPhysValue(UY)));
              Value << attr("imported from",FromQString(File->fileName()));
            }
            Value->AddNewPoint(X,Y);
          }
          PointNo++;
        }else{
          FooterStarted = true;
//          FooterText->setHtml(Line);
          FooterText->append(Line);
        }
      }else{
//        FooterText->setHtml(FooterText->toHtml()+Line);
        FooterText->append(Line);
      }
    }
    File->close();
  }
  InfoLabel->setText(tr("%1 data point(s) imported",0,PointNo).arg(PointNo));
  if( UX_OK && UY_OK && (PointNo > 0) ){
    InfoLabel->setStyleSheet("color: green");
    return true;
  }
  InfoLabel->setStyleSheet("color: red");
  Value.SetToNULL();
  return false;
}

//------------------------------------------------------------------------------
} //namespace RA


