////////////////////////////////////////////////////////////////////////////////
/*! @file Chart.cpp   Виджет для показа графиков.
- Part of RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2010-2020, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#include "Chart.h"
#include "SymbolsGUI.h"
#include "StringsAndQStrings.h"
#include "attr.h"
#include "EditScalesDialog.h"
#include "Log.h"
#include <algorithm>
#include <QPainter>
#include <QApplication>
#include <QMessageBox>
#include <QMimeData>
#include <cmath>
namespace RA {
//------------------------------------------------------------------------------

bool                    sChartViewport::NegativeBackground = false;
QCursor *               sChartViewport::CrossHairDark = NULL;
QCursor *               sChartViewport::CrossHairBright = NULL;
QCursor *               sChartViewport::ZoomCursor = NULL;
QCursor *               sChartViewport::ShiftCursor = NULL;
QCursor *               sChartViewport::ZoomingCursor = NULL;
QCursor *               sChartViewport::ShiftingCursor = NULL;
const real              sChartViewport::ZoomOutFactor = 3.0;
//--------------------------------------------------------------- sChartItem ---

//sChartItem::~sChartItem ()
//{
//}


//sChartItem::sChartItem (psNetViewport               viewport
//                       ,sNet2TreeEvolution::psItem  tree_item
//                       ,pfChartItemDrawProc         draw_proc)
//    :sNetViewItem(viewport,tree_item)
//    ,Draw(draw_proc)
//{
//}

//--------------------------------------------------------------- sChartMenu ---

//sChartMenu::~sChartMenu ()
//{
//}


//sChartMenu::sChartMenu (psNetViewport               viewport
//                       ,sNet2TreeEvolution::psItem  tree_item)
//  :sNetViewMenu(viewport,tree_item)
//{
////  psChart  Chart(dynamic_cast<psChart>(viewport->parent()));
//}

//-------------------------------------------------------------- sChartScales ---

void  sChartScales::RestoreSettings (sPtr<sFolderNode>  C)
{
  if( C.IsCorrect() ){
    sPtr<sTextNode>           V;
    sPtr<sPhysRangeNode>      R;
    V = C >> "ScaleX.Variable";
    if( V.IsCorrect() )  X.Name=V->Value();
    V = C >> "ScaleY.Variable";
    if( V.IsCorrect() )  Y.Name=V->Value();

    R = C >> "ScaleX.Range";
    if( R.IsCorrect() )  X.Range=(sPhysRange)(*R);
    R = C >> "ScaleY.Range";
    if( R.IsCorrect() )  Y.Range=(sPhysRange)(*R);
  };
}


void  sChartScales::StoreSettings (sPtr<sFolderNode>  C) const
{
  if( C.IsCorrect() ){
    C   << attr("ScaleX.Variable",X.Name)
        << attr("ScaleX.Range",X.Range)
        << attr("ScaleY.Variable",Y.Name)
        << attr("ScaleY.Range",Y.Range);
  };
}

//----------------------------------------------------------------- sMargins ---

sMargins::sMargins (int top ,int  bottom ,int  left ,int right)
    :TheTop(top),TheBottom(bottom),TheLeft(left),TheRight(right)
{
}

//------------------------------------------------------------------- sTicks ---

sTicks::sTicks ()
    :Start(0.0),Interval(1.0),Length(6)
{
//  FPen=new TPen;
}


//------------------------------------------------------------------ sLabels ---

sLabels::~sLabels ()
{
  Clear();
}

sLabels::sLabels ()
    :MaxAmount(7)
{
}

void  sLabels::Clear ()
{
  List.clear();
}

//--------------------------------------------------------------- sAxis ---

sAxis::sAxis (bool  upside_down)
    :sScale(upside_down)
    ,Variable()
    ,Title()
    ,MultiplierExponent(0)
    ,Multiplier(1)
    ,Shift(0.0)
    ,LabelsArranged(false)
    ,LabelsOverlap(false)
{
  SetPenWidth(1);
  Title.setDocumentMargin(0);
  MinorTicks.Length=(MajorTicks.Length+1) / 2;
  SetRange(sPhysRange(0,10,sUnits()));
}


void  sAxis::SetPenWidth (int  width)
{
  PenWidth = width;
  SetOffset((PenWidth+1)/2);
}


int  sAxis::Value2Offset (real value)
{
  try{
    return Round((value-Range().From)/(Range().To-Range().From)*Size());
  }catch(...){
    return 0;
  };
}


void  sAxis::Arrange (const QFontMetrics &  font_metrics)
{
  MultiplierExponent=0;
  MultiplierExponent=3*Round(log10( rmax(std::abs(Range().From),std::abs(Range().To))
                                  )/3.0-0.5);
  Multiplier=power10(MultiplierExponent);
  MajorTicks.Interval
      = Multiplier * power10(Round( -1.0 + log10(
                   (Range().To-Range().From)/Multiplier/Labels.MaxAmount
                                                ) ));
  MinorTicks.Interval=MajorTicks.Interval / 10;
  ;
  if( ArrangeLabels(font_metrics) ){
    RANet::Log.Put(sLog::Debug,"Graph"
        ,sString("sAxis::Arrange: success in step #")<<1 );
  }else{
    MajorTicks.Interval*=2.0;
    MinorTicks.Interval=MajorTicks.Interval / 2;
    if( ArrangeLabels(font_metrics) ){
      RANet::Log.Put(sLog::Debug,"Graph"
          ,sString("sAxis::Arrange: success in step #")<<2 );
    }else{
      MajorTicks.Interval*=2.5;
      MinorTicks.Interval=MajorTicks.Interval / 5;
      if( ArrangeLabels(font_metrics) ){
        RANet::Log.Put(sLog::Debug,"Graph"
            ,sString("sAxis::Arrange: success in step #")<<3 );
      }else{
        MajorTicks.Interval*=2.0;
        MinorTicks.Interval=MajorTicks.Interval / 10;
        if( ArrangeLabels(font_metrics) ){
          RANet::Log.Put(sLog::Debug,"Graph"
              ,sString("sAxis::Arrange: success in step #")<<4 );
        }else{
          MajorTicks.Interval*=2.0;
          MinorTicks.Interval=MajorTicks.Interval / 2;
          if( ArrangeLabels(font_metrics) ){
            RANet::Log.Put(sLog::Debug,"Graph"
                ,sString("sAxis::Arrange: success in step #")<<5 );
          }else{
            MajorTicks.Interval*=2.5;
            MinorTicks.Interval=MajorTicks.Interval / 5;
            if( ArrangeLabels(font_metrics) ){
              RANet::Log.Put(sLog::Debug,"Graph"
                  ,sString("sAxis::Arrange: success in step #")<<6 );
            }else{
              MajorTicks.Interval*=2.0;
              MinorTicks.Interval=MajorTicks.Interval / 10;
              if( ArrangeLabels(font_metrics) ){
                RANet::Log.Put(sLog::Debug,"Graph"
                    ,sString("sAxis::Arrange: success in step #")<<7 );
              }else{
                MajorTicks.Interval*=2.0;
                MinorTicks.Interval=MajorTicks.Interval / 2;
                if( ArrangeLabels(font_metrics) ){
                  RANet::Log.Put(sLog::Debug,"Graph"
                      ,sString("sAxis::Arrange: success in step #")<<8 );
                }else{
                  MajorTicks.Interval*=2.5;
                  MinorTicks.Interval=MajorTicks.Interval / 5;
                  if( ArrangeLabels(font_metrics) ){
                    RANet::Log.Put(sLog::Debug,"Graph"
                        ,sString("sAxis::Arrange: success in step #")<<9 );
                  }else{
                    RANet::Log.Put(sLog::Debug,"Graph"
                        ,sString("sAxis::Arrange: no success !!!!!!!!!!!!!!!!!"));
                  };
                };
              };
            };
          };
        };
      };
    };
  };
  if(MultiplierExponent)
    RANet::Log.Put(sLog::Debug,"Graph"
        ,sString(" MultiplierExponent=")<<MultiplierExponent
        );
//  RANet::Log.Put(sLog::Debug,"Graph"
//      ,sString("sAxis::Arrange: LabelsArranged=")+(LabelsArranged ? "Y" : "N")
//      +sString(" LabelsOverlap=")+(LabelsOverlap ? "Y" : "N")
//      );
}


void  sAxis::PrearrangeLabels ()
{
  const int                   ShiftQuantumLog = 5;
                              /*!< =lg(ShiftQuantum/MinorTicks.Interval) */
  real                        tick;
  int                         NumOfMajorTicks = 0;
  int                         digits = 0;
  sString                     label_text;
  QString                     LabelText;
  real                        ShiftQuantum;
  real                        MidScale = (Range().From+Range().To)/2.0;

  LabelsArranged=false;
  ShiftQuantum=MinorTicks.Interval*power10(ShiftQuantumLog);
  MultiplierExponent=3*Round(log10( rmax(std::abs(Range().From),std::abs(Range().To))
                                  )/3.0-0.5);
  Multiplier=power10(MultiplierExponent);
  MajorTicks.Start=Round(Range().From/MajorTicks.Interval+0.499)*MajorTicks.Interval;
  MinorTicks.Start=MajorTicks.Start - MajorTicks.Interval;
  if( MidScale > 0.0 ){
    Shift=Round(floor(MidScale/ShiftQuantum))*ShiftQuantum;
  }else{
    Shift=Round(ceil(MidScale/ShiftQuantum))*ShiftQuantum;
  };
  if( std::abs(Shift) > PixelValue() ){
    MultiplierExponent -= 3;//ShiftQuantumLog;
    Multiplier=power10(MultiplierExponent);
  };
  Labels.Clear();
  if( MajorTicks.Interval/Multiplier < 0.999 ){
    digits=-Round(log10(MajorTicks.Interval/Multiplier)-0.499);
  };
  tick=MajorTicks.Start;
  while( tick < Range().To + PixelValue() ){
    LabelText.setNum(double((tick-Shift)/Multiplier),'f',digits);
    Labels.List.push_back(sLabel(LabelText));
    tick+=MajorTicks.Interval;
    NumOfMajorTicks++;
    if( NumOfMajorTicks > 1000*(int)Labels.MaxAmount ){
      RANet::Log.Put(sLog::Debug,"Graph"
          ,sString("cycling in sAxis::PrearrangeLabels: ")+Variable
          +"  Range().From="+sString::FromReal(Range().From,12)
          +"  Range().To="+sString::FromReal(Range().To,12)
          +"  MajorTicks.Interval="+sString::FromReal(MajorTicks.Interval,12)
          +"  MajorTicks.Start="+sString::FromReal(MajorTicks.Start,12)
//          +"  ="+sString::FromReal()
          );
      return;
    };
  };
  LabelsArranged = ! Labels.List.empty()
                && (Labels.List.size() <= Labels.MaxAmount);
//  RANet::Log.Put(sLog::Debug,"Graph",sString("Prearranged ")+Variable
//      +"  Labels.List.size()="+sString::FromInteger(Labels.List.size())
//      +"  MajorTicks.Start="+sString::FromReal(MajorTicks.Start,15)
//      +"  MinorTicks.Start="+sString::FromReal(MinorTicks.Start,15)
//      +"  MajorTicks.Interval="+sString::FromReal(MajorTicks.Interval,15)
//      +"  MinorTicks.Interval="+sString::FromReal(MinorTicks.Interval,15)
//      +"  LabelsArranged="+sString::FromInteger(LabelsArranged)
//      );
}


QString  sAxis::TitleText ()
{
  sPhysValue            U(Unit());
  int                   Exp=MultiplierExponent;

  U << Exp;
  Exp = 0;

  sString               ExpText;
  sString               UnitsText(U.Text(HTML,Laconic));
  sString               ShiftText;
  sString               VarText(Variable);
  if( ! Unit().IsPowerOfTen() ){
    RANet::Log.Put(sLog::Debug,"Graph",sString("TitleText: not a power of ten: ")+Unit().Text());
  };
  //---------
  if( VarText.Empty() ){
    VarText = sString("<i>") + VarText + "</i>";
  };
  if( Shift > std::abs(PixelValue()) ){
    if( Variable.Empty() ){
      ShiftText=" ＋ ";
      ShiftText << std::abs(Shift);
    }else{
      ShiftText=" － ";
      ShiftText << std::abs(Shift/Multiplier);
    };
  }else if( Shift < -std::abs(PixelValue()) ){
    if( Variable.Empty() ){
      ShiftText=" － ";
      ShiftText << std::abs(Shift);
    }else{
      ShiftText=" ＋ ";
      ShiftText <<  std::abs(Shift/Multiplier);
    };
  };
  if( Variable.Empty() && UnitsText.Empty() ){
    if( Exp != 0 )
      (((ExpText = Symbol::Multiplication) += "10<sup>") << Exp) += "</sup> ";
    return ToQString( ExpText  + ShiftText );
  }else if( Variable.Empty() ){
    /*! @todo{PhysValues} test and review this option */
    if( Exp != 0 )
      (((ExpText = Symbol::Multiplication) += "10<sup>") << Exp ) += "</sup> ";
    return ToQString( ExpText  + ShiftText + " [ " + UnitsText + " ]" );
  }else{
    if( Exp != 0 )  ((ExpText = "10<sup>") << (Exp) ) += "</sup> ";
    return ToQString( VarText + ShiftText
                          + " [ " + ExpText + UnitsText + " ]" );
  };
}

//--------------------------------------------------------------- sX_Axis ---


bool  sX_Axis::ArrangeLabels (const QFontMetrics &  font_metrics)
{
  real                        tick_value;
  int                         tick_pos;
  std::list<sLabel>::iterator label_index;
  QRect                       last_label_rect;
  bool                        first = true;
  int                         digit_width = font_metrics.width('0');

  sAxis::PrearrangeLabels();
  LabelsOverlap=false;
  label_index=Labels.List.begin();
  tick_value=MajorTicks.Start;
  while( label_index != Labels.List.end() ){
    tick_pos=Origin.x()+Value2Offset(tick_value);
    label_index->Rect =
        font_metrics.boundingRect(tick_pos
                                 ,Origin.y() + MajorTicks.Length
                                 ,0,0
                                 ,Qt::AlignHCenter|Qt::AlignTop
                                 ,label_index->Text);
    if( first ){
      first=false;
      LabelsArea=label_index->Rect;
      last_label_rect=label_index->Rect;
    }else{
      LabelsArea |= label_index->Rect;
      last_label_rect.setWidth(last_label_rect.width()+digit_width);
      LabelsOverlap = LabelsOverlap
          || label_index->Rect.intersects(last_label_rect);
      last_label_rect=label_index->Rect;
    };
    label_index++;
    tick_value+=MajorTicks.Interval;
  };
  Title.setHtml(TitleText());
  TitleArea = QRect(Origin.x() + Size()/2 - Title.size().width()/2
                   ,LabelsArea.bottom()
                   ,Title.size().width()
                   ,Title.size().height() );
//  RANet::Log.Put(sLog::Debug,"Graph",sString("ArrangeLabels result ")+Variable
//      +"  Labels.List.size()="+sString::FromInteger(Labels.List.size())
//      +"  LabelsArranged="+sString::FromInteger(LabelsArranged)
//      +"  LabelsOverlap="+sString::FromInteger(LabelsOverlap)
//      );
  return LabelsArranged && ! LabelsOverlap;
}


void  sX_Axis::Draw (QPainter *  painter)
{
//  RANet::Log.Put(sLog::Debug,"Graph","-------------------X-drawing...");
  real                        tick_value;
  int                         tick_pos;
  QPen                        Pen=painter->pen();
  painter->save();
  Pen.setWidth(PenWidth);
  painter->setPen(Pen);
  painter->drawLine( Origin.x()        , Origin.y()
                   , Origin.x()+Size() , Origin.y());
  painter->drawLine( OppositeOrigin.x()        , OppositeOrigin.y()
                   , OppositeOrigin.x()+Size() , OppositeOrigin.y() );
  if( LabelsArranged ){
    tick_value=MinorTicks.Start;
    while( tick_value < Range().To + PixelValue() ){
//      RANet::Log.Put(sLog::Debug,"Graph",sString("min tick ")+sString::FromReal(tick_value,15));
      if( tick_value >= Range().From ){
        tick_pos=Origin.x()+Value2Offset(tick_value);
        painter->drawLine(tick_pos ,Origin.y()
                         ,tick_pos ,Origin.y() + MinorTicks.Length);
        painter->drawLine(tick_pos ,OppositeOrigin.y()
                         ,tick_pos ,OppositeOrigin.y() - MinorTicks.Length);
      };
      tick_value+=MinorTicks.Interval;
    };
//    RANet::Log.Put(sLog::Debug,"Graph",sString("min ticks done "));
    tick_value=MajorTicks.Start;
    while( tick_value < Range().To + PixelValue() ){
      tick_pos=Origin.x()+Value2Offset(tick_value);
      painter->drawLine(tick_pos ,Origin.y()
                       ,tick_pos ,Origin.y() + MajorTicks.Length);
      painter->drawLine(tick_pos ,OppositeOrigin.y()
                       ,tick_pos ,OppositeOrigin.y() - MajorTicks.Length);
      tick_value+=MajorTicks.Interval;
    };
  };
//  RANet::Log.Put(sLog::Debug,"Graph",sString("ticks done "));
  painter->restore();
  if( LabelsArranged && ! LabelsOverlap ){
    for( std::list<sLabel>::iterator label_index = Labels.List.begin() ;
                                     label_index != Labels.List.end() ;
                                     label_index++ ){
      painter->drawText(label_index->Rect
                       ,Qt::AlignHCenter|Qt::AlignTop
                       ,label_index->Text);
    };
  };
  painter->save();
  painter->translate( TitleArea.topLeft() );
  Title.setHtml(QString("<font color=")
               +painter->pen().color().name(QColor::HexArgb)
               +">"
               +TitleText()
               +"</font>");
  Title.drawContents(painter);
  painter->restore();
//  RANet::Log.Put(sLog::Debug,"Graph","-------------------X-drawn----------------");
}

//--------------------------------------------------------------- sY_Axis ---


bool  sY_Axis::ArrangeLabels (const QFontMetrics &  font_metrics)
{
  real                        tick_value;
  int                         tick_pos;
  std::list<sLabel>::iterator label_index;
  QRect                       last_label_rect;
  bool                        first = true;

  sAxis::PrearrangeLabels();
  LabelsOverlap=false;
  label_index=Labels.List.begin();
  tick_value=MajorTicks.Start;
  while( label_index != Labels.List.end() ){
    tick_pos=Origin.y()-Value2Offset(tick_value);
    label_index->Rect =
        font_metrics.boundingRect(Origin.x() - MajorTicks.Length
                                 ,tick_pos
                                 ,0,0
                                 ,Qt::AlignVCenter|Qt::AlignRight
                                 ,label_index->Text);
    if( first ){
      first=false;
      LabelsArea=label_index->Rect;
      last_label_rect=label_index->Rect;
    }else{
      LabelsArea |= label_index->Rect;
      LabelsOverlap = LabelsOverlap
          || label_index->Rect.intersects(last_label_rect);
      last_label_rect=label_index->Rect;
    };
    label_index++;
    tick_value+=MajorTicks.Interval;
  };
  Title.setHtml(TitleText());
  TitleArea = QRect(2
                   ,Origin.y() - Size()/2 + Title.size().width()/2
                   ,Title.size().height()
                   ,Title.size().width() );
//  RANet::Log.Put(sLog::Debug,"Graph",sString("ArrangeLabels result ")+Variable
//      +"  Labels.List.size()="+sString::FromInteger(Labels.List.size())
//      +"  LabelsArranged="+sString::FromInteger(LabelsArranged)
//      +"  LabelsOverlap="+sString::FromInteger(LabelsOverlap)
//      );
  return LabelsArranged && ! LabelsOverlap;
}


void  sY_Axis::Draw (QPainter *  painter)
{
//  RANet::Log.Put(sLog::Debug,"Graph","-------------------Y-drawing...");
  real                        tick_value;
  int                         tick_pos;
  QPen                        Pen=painter->pen();
  painter->save();
  Pen.setWidth(PenWidth);
  painter->setPen(Pen);
  painter->drawLine( Origin.x() , Origin.y()
                   , Origin.x() , Origin.y()-Size() );
  painter->drawLine( OppositeOrigin.x() , OppositeOrigin.y()
                   , OppositeOrigin.x() , OppositeOrigin.y()-Size() );
  if( LabelsArranged ){
    tick_value=MinorTicks.Start;
    while( tick_value < Range().To + PixelValue() ){
      if( tick_value >= Range().From ){
        tick_pos=Origin.y()-Value2Offset(tick_value);
        painter->drawLine(Origin.x()                             ,tick_pos
                         ,Origin.x() - MinorTicks.Length         ,tick_pos);
        painter->drawLine(OppositeOrigin.x()                     ,tick_pos
                         ,OppositeOrigin.x() + MinorTicks.Length ,tick_pos);
      };
      tick_value+=MinorTicks.Interval;
    };
    tick_value=MajorTicks.Start;
    while( tick_value < Range().To + PixelValue() ){
      tick_pos=Origin.y()-Value2Offset(tick_value);
      painter->drawLine(Origin.x()                             ,tick_pos
                       ,Origin.x() - MajorTicks.Length         ,tick_pos);
      painter->drawLine(OppositeOrigin.x()                     ,tick_pos
                       ,OppositeOrigin.x() + MajorTicks.Length ,tick_pos);
      tick_value+=MajorTicks.Interval;
    };
  };
  painter->restore();
  if( LabelsArranged && ! LabelsOverlap ){
    for( std::list<sLabel>::iterator label_index = Labels.List.begin() ;
                                     label_index != Labels.List.end() ;
                                     label_index++ ){
      painter->drawText(label_index->Rect
                       ,Qt::AlignVCenter|Qt::AlignRight
                       ,label_index->Text);
    };
  };
  painter->save();
  painter->translate( TitleArea.topLeft() );
  painter->rotate(-90);
  Title.setHtml(QString("<font color=")
               +painter->pen().color().name(QColor::HexArgb)
               +">"
               +TitleText()
               +"</font>");
  Title.drawContents(painter);
  painter->restore();
//  RANet::Log.Put(sLog::Debug,"Graph","-------------------Y-drawn----------------");
}

//----------------------------------------------------------- sChartViewport ---

sChartViewport::~sChartViewport ()
{
  Clear();
}

sChartViewport::sChartViewport(psX_Axis             x_axis
                              ,psY_Axis             y_axis
                              ,psNet2TreeEvolution  evolution
                              ,QWidget *            parent)
  :sNetViewport(evolution,parent)
  ,XAxis(x_axis)
  ,YAxis(y_axis)
  ,MouseOperationIsAboutToStart(false)
  ,MouseOperationStarted(false)
  ,DragIsAboutToStart(false)
  ,CurveDropsAreAllowed(false)
{
  ZoomRect = new QRubberBand(QRubberBand::Rectangle, this);
//  setAttribute(Qt::WA_NoBackground);
//  setAttribute(Qt::WA_NoSystemBackground);
//  setAttribute(Qt::WA_OpaquePaintEvent);
//setAutoFillBackground ( bool enabled )
  if( ! CrossHairDark ){
    CrossHairDark = new QCursor
                        (QPixmap(":/RAGUI/cursors/CrossHairDark.png"),11,11);
    CrossHairBright = new QCursor
                        (QPixmap(":/RAGUI/cursors/CrossHairBright.png"),11,11);
    ZoomCursor = new QCursor(Qt::SizeAllCursor);
    ShiftCursor = new QCursor(Qt::OpenHandCursor);
    ZoomingCursor = new QCursor(Qt::UpArrowCursor);
    ShiftingCursor = new QCursor(Qt::ClosedHandCursor);
  };
  setMouseTracking(true);
  setCursor(( NegativeBackground ? *CrossHairBright : *CrossHairDark ));
  setAcceptDrops(CurveDropsAreAllowed);
  connect(this,SIGNAL(CursorPositionChanged(QPoint))
         ,this,SLOT(OnCursorPositionChanged(QPoint)));
}


void  sChartViewport::FormItems ()
{
  Clear();
//  sNet2TreeEvolution::psItem  I = Evolution->FirstVisible();
//  while( I ){
//    sGraphObject::sPtr    NodePtr(I->AttrValue());
//    if( NodePtr.IsCorrect() ){
//      sCurve::sPtr              CurvePtr(NodePtr);
//      if( CurvePtr.IsCorrect() ){
//        Items.push_back(new sChartItem(this,I,DrawProc));
//      };
//      if( true /*sMark*/ ){
////        поиск принадлежности среди предков по эволюции
//      };
//    };
//    I=I->NextVisible();
//  }
  update();
}


void  sChartViewport::Clear ()
{
//  psChartItem            I;
//  while( ! Items.empty() ){
//    I=Items.back();
//    Items.pop_back();
//    delete I;
//  };
}


sNet2TreeEvolution::psItem  sChartViewport::FindTreeItemAt (const QPoint &  pos)
{
  int                         MinDist = 5;
  sIntPoint                   Pos(pos.x(),pos.y());
  sNet2TreeEvolution::psItem  ClosestItem = NULL;
  sNet2TreeEvolution::psItem  I = Evolution->FirstVisible();
  while( I ){
    if( psGraphObject  O = dynamic_cast<psGraphObject>
                                           (I->AttrValue().operator ->()) ){
      int  D = O->Distance(Pos,Scales());
      if( D <= MinDist ){
        MinDist=D;
        ClosestItem=I;
      };
    };
    I=I->NextVisible();
  };
  return ClosestItem;
}


void  sChartViewport::AllowCurveDrops (bool  Allow)
{
  CurveDropsAreAllowed=Allow;
  setAcceptDrops(CurveDropsAreAllowed);
}


sChartScales  sChartViewport::ChartScales () const
{
  return sChartScales(sChartScale(XAxis->Variable,XAxis->PhysRange())
                    ,sChartScale(YAxis->Variable,YAxis->PhysRange()) );
}


void  sChartViewport::SetChartScales (sChartScales  scales)
{
  XAxis->Variable = scales.X.Name;
  XAxis->SetRange(scales.X.Range);
  YAxis->Variable = scales.Y.Name;
  YAxis->SetRange(scales.Y.Range);
  emit ScalesChanged();
}


void  sChartViewport::ChangeChartScales (sChartScales  scales)
{
  ScalesHistory.push(ChartScales());
  SetChartScales(scales);
}


void  sChartViewport::SetReference (sPhysPair  P)
{
  TheReference=P;
}


void  sChartViewport::SetReference (QPoint  P)
{
  TheReference=T(P);
}


void  sChartViewport::SetNominalSize (QSize S)
{
  NominalSize=S;
}


void  sChartViewport::SetNominalOrigin (QPoint P)
{
  NominalOrigin=P;
}


QPoint  sChartViewport::T (sPhysPair  point) const
{
  return QPoint( XAxis->T(point.X) , YAxis->T(point.Y) );
}


QPoint  sChartViewport::T (rcsPoint  point) const
{
  return QPoint( XAxis->T(point.X) , YAxis->T(point.Y) );
}


sPhysPair  sChartViewport::T (QPoint  point) const
{
  return sPhysPair( XAxis->T(point.x()) , YAxis->T(point.y()) );
}


void  sChartViewport::ZoomOut ()
{
  ScalesHistory.push(ChartScales());
  XAxis->SetRange( - 0.5*(ZoomOutFactor-1.0)*NominalSize.width()
                 ,   0.5*(ZoomOutFactor+1.0)*NominalSize.width() );
  YAxis->SetRange(   0.5*(ZoomOutFactor+1.0)*NominalSize.height()
                 , - 0.5*(ZoomOutFactor-1.0)*NominalSize.height() );
  emit ScalesChanged();
}


void  sChartViewport::RescaleBack ()
{
  if( ! ScalesHistory.empty() ){
    SetChartScales(ScalesHistory.top());
    ScalesHistory.pop();
  };
}


void  sChartViewport::RescaleToShowAll ()
{
  sBoundaries                 B( BoundariesForAllObjects() );
  ScalesHistory.push(ChartScales());
  XAxis->SetRange(sPhysRange( B.X.From() - 0.05*B.X.Size()
                            , B.X.To()   + 0.05*B.X.Size() ));
  YAxis->SetRange(sPhysRange( B.Y.From() - 0.05*B.Y.Size()
                            , B.Y.To()   + 0.05*B.Y.Size() ));
  emit ScalesChanged();
}


sBoundaries  sChartViewport::BoundariesForAllObjects ()
{
//  RANet::Log.OpenBlock(sLog::Debug,"sChartViewport::BoundariesForAllObjects");
  int                         NumberOfVisibleGraphObjects = 0;
  sBoundaries                 B;
  sBoundaries                 TotalB;
  bool                        AllVisibleAreInSameUnits = true;
  sBoundaries                 TotalBWithinScales;

  TotalBWithinScales = sBoundaries( sPhysRange(real_nan,real_nan,_Unitsless_)
                      , sPhysRange(real_nan,real_nan,_Unitsless_) );
  sNet2TreeEvolution::psItem  I = Evolution->FirstVisible();
  while( I ){
    if( psGraphObject  O = dynamic_cast<psGraphObject>
                                           (I->AttrValue().operator ->()) ){
      sBoundaries                 BB(sPhysRange(XAxis->Scale().Unit().Units())
                                    ,sPhysRange(YAxis->Scale().Unit().Units())
                                      );
      NumberOfVisibleGraphObjects++;
      B = O->Boundaries();
//      RANet::Log.Put(sLog::Debug,"sChartViewport::BoundariesForAllObjects"
//          ,sString("O->Boundaries: X: ")+B.X.Text()+"  Y:"+B.Y.Text());
      if( NumberOfVisibleGraphObjects == 1 ){
        TotalB = B;
      }else if( AllVisibleAreInSameUnits ){
        try{
          TotalB |= B;
        }catch(rxRAlgebra){
          AllVisibleAreInSameUnits = false;
        }
      }
      try{
        if( ! B.IsNaN() ){
          BB.X = B.X;
          BB.Y = B.Y;
          if( TotalBWithinScales.X.IsNaN() ){
            TotalBWithinScales.X = BB.X;
          }else{
            TotalBWithinScales.X |= BB.X;
          };
          if( TotalBWithinScales.Y.IsNaN() ){
            TotalBWithinScales.Y = BB.Y;
          }else{
            TotalBWithinScales.Y |= BB.Y;
          };
        }else if( ! B.X.IsNaN() ){
          BB.X = B.X;
          if( TotalBWithinScales.X.IsNaN() ){
            TotalBWithinScales.X = BB.X;
          }else{
            TotalBWithinScales.X |= BB.X;
          };
        }else if( ! B.Y.IsNaN() ){
          BB.Y = B.Y;
          if( TotalBWithinScales.Y.IsNaN() ){
            TotalBWithinScales.Y = BB.Y;
          }else{
            TotalBWithinScales.Y |= BB.Y;
          };
        };
      }catch(rxRAlgebra){};
//      RANet::Log.Put(sLog::Debug,"sChartViewport::BoundariesForAllObjects"
//          ,sString("BB.X: ")+BB.X.Text()+"  BB.Y:"+BB.Y.Text());
//      RANet::Log.Put(sLog::Debug,"sChartViewport::BoundariesForAllObjects"
//          ,sString("TotalBWithinScales.X: ")+TotalBWithinScales.X.Text()
//                  +"  TotalBWithinScales.Y:"+TotalBWithinScales.Y.Text());
    };
    I=I->NextVisible();
  };
  if( NumberOfVisibleGraphObjects == 0 ){
    B = sBoundaries(XAxis->PhysRange(),YAxis->PhysRange());
  }else if( AllVisibleAreInSameUnits ){
    B = TotalB;
  }else{
    B = TotalBWithinScales;
  }
  if(       B.X.From().IsInf() ){
    B.X = sPhysRange( XAxis->PhysRange().From() ,B.X.To() );
  }else if( B.X.To().IsInf() ){
    B.X = sPhysRange( B.X.From()                ,XAxis->PhysRange().To() );
  }else if( B.Y.From().IsInf() ){
    B.Y = sPhysRange( YAxis->PhysRange().From() ,B.Y.To() );
  }else if( B.Y.To().IsInf() ){
    B.Y = sPhysRange( B.Y.From()                ,YAxis->PhysRange().To() );
  }
//  RANet::Log.Put(sLog::Debug,"sChartViewport::BoundariesForAllObjects"
//      ,sString("result: X: ")+B.X.Text()+"  Y: "+B.Y.Text());
//  RANet::Log.CloseBlock(sLog::Debug,"sChartViewport::BoundariesForAllObjects");
  return B;
}


void  sChartViewport::paintEvent (QPaintEvent *)
{
  QPainter                    Painter(this);
  QPen                        Pen=Painter.pen();
  NegativeBackground = (Pen.color().value() > 127);
  DrawGraphObjects(&Painter);
  try{
    QPoint                      R(T(TheReference));
    Pen.setStyle(Qt::DotLine);
    Painter.setPen(Pen);
    Painter.drawLine(0     ,R.y() ,size().width() ,R.y());
    Painter.drawLine(R.x() ,0     ,R.x()          ,size().height());
  }catch(rxRAlgebra){}
}


void  sChartViewport::DrawGraphObjects (QPainter *  painter)
{
  sNet2TreeEvolution::psItem  I = Evolution->FirstVisible();
  while( I ){
//    if( /*psCurve  O =*/ dynamic_cast<psCurve>
//                                           (I->AttrValue().operator ->()) ){
//      (*DrawProc)(painter,this,I);
//    }else if( true /*sMark*/ ){
//      //поиск принадлежности среди предков по эволюции
//    };
    psGraphObject           Object(dynamic_cast<psGraphObject>
                                                (I->AttrValue().operator ->()));
    if( Object ){
      pcsAppearance                ObjectAppearance(Object->Appearance());
      for( std::list<pcfDrawProc>::const_iterator i = DrawProcCollection.begin()
                                                ; i != DrawProcCollection.end()
                                                ; i++ ){
        if( (*i)(painter,this,Object,ObjectAppearance) )  break;
      }
    }
    I=I->NextVisible();
  }
}


void  sChartViewport::keyPressEvent (QKeyEvent *  event)
{
  if( event->modifiers() & Qt::ControlModifier ){
      setCursor(*ZoomCursor);
  }else{

  }
}


void sChartViewport::mousePressEvent (QMouseEvent *  event)
{
  if( event->button() == Qt::LeftButton ){
    MouseOperationStartPosition = event->pos();
    if( event->modifiers() & Qt::ControlModifier ){
      MouseOperationStarted=true;
      MouseOperationType=Zooming;
      ZoomRect->setGeometry(QRect(MouseOperationStartPosition
                                 ,MouseOperationStartPosition).normalized());
      ZoomRect->show();
      setCursor(*ZoomingCursor);
    }else if(  event->modifiers() & Qt::ShiftModifier ){
      MouseOperationStarted=true;
      MouseOperationType=MovingFrame;
      MouseOperationLastPosition = event->pos();
      ScalesHistory.push(ChartScales());
      setCursor(*ShiftingCursor);
//    }else if( Host->StartDraggingAnObjectIfAny() ){
//      Host->MouseOperationStarted=true;
//      Host->MouseState=VisibleObjectDragging;
    }else{
      MouseOperationStarted=true;
      MouseOperationType=MovingInitialPoint;
      SetReference(event->pos());
      update();
    };
//    MouseOperationIsAboutToStart = true;
//    DragIsAboutToStart = true;
  }else{
    MouseOperationStarted=false;
  };
}


void sChartViewport::mouseReleaseEvent (QMouseEvent *  event)
{
  if( event->button() == Qt::LeftButton ){
    if( MouseOperationStarted ){
      if( MouseOperationType == MovingInitialPoint ){
        MouseOperationStarted=false;
        SetReference(event->pos());
        emit CursorPositionChanged(event->pos());
        update();
      }else if( MouseOperationType == MovingFrame ){
        if( (event->pos() == MouseOperationStartPosition)
            && ! ScalesHistory.empty() ){
          ScalesHistory.pop();
        };
      }else if( MouseOperationType == Zooming ){
        ZoomRect->hide();
        if( (ZoomRect->width() > 2) && (ZoomRect->height() > 2) ){
          ScalesHistory.push(ChartScales());
          XAxis->SetRange( ZoomRect->pos().x()
                         , ZoomRect->pos().x() + ZoomRect->width()  );
          YAxis->SetRange( ZoomRect->pos().y() + ZoomRect->height()
                         , ZoomRect->pos().y()                      );
          emit ScalesChanged();
        };
      };
//      if( Shift.Contains(ssCtrl) ){
//        Host->MouseState=Zooming;
//      }else if( Shift.Contains(ssShift) ){
//        Host->MouseState=MovingFrame;
//      }else{
//        Host->MouseState=MovingInitialPoint;
//      };
    }else{ //(Button == mbLeft) && ! Host->MouseOperationStarted
    };
//    if( DragIsAboutToStart
//        && ((event->pos() - MouseOperationStartPosition).manhattanLength()
//                                       < QApplication::startDragDistance()) ) {
//      //left click (possibly a part of a double click)
//      DragIsAboutToStart = false;
//    };
  }else if( event->button() == Qt::MidButton ){
    RescaleBack();
  };
  MouseOperationStarted=false;
  setCursor(( NegativeBackground ? *CrossHairBright : *CrossHairDark ));
}


void sChartViewport::mouseMoveEvent (QMouseEvent *  event)
{
  emit CursorPositionChanged(event->pos());
  if( event->buttons() & Qt::LeftButton ) {
    if( MouseOperationStarted ){
      if( MouseOperationType == MovingInitialPoint ){
        SetReference(event->pos());
        update();
      }else if( MouseOperationType == MovingFrame ){
        if( (event->pos() - MouseOperationLastPosition).manhattanLength() > 3 ){
          XAxis->ShiftRange( MouseOperationLastPosition.x() - event->pos().x() );
          YAxis->ShiftRange( MouseOperationLastPosition.y() - event->pos().y() );
          emit ScalesChanged();
          MouseOperationLastPosition=event->pos();
        };
      }else if( MouseOperationType == Zooming ){
        ZoomRect->setGeometry(QRect(MouseOperationStartPosition
                                   ,event->pos()               ).normalized());
      };
    };
  }else{
    MouseOperationStarted=false;
    setCursor(( NegativeBackground ? *CrossHairBright : *CrossHairDark ));
//    if( Shift.Contains(ssCtrl) ){
//      Host->MouseState=Zooming;
//    }else if( Shift.Contains(ssShift) ){
//      Host->MouseState=MovingFrame;
//    }else{
//      Host->MouseState=MovingInitialPoint;
//    };
  };
//  if( MouseOperationIsAboutToStart ){
//    if( (event->pos() - MouseOperationStartPosition).manhattanLength()
//        > QApplication::startDragDistance() ){
//      MouseOperationStarted=true;
//    };
//  }else{
//    MouseOperationStarted=false;
//    DragIsAboutToStart = false;
//  };
}


void sChartViewport::mouseDoubleClickEvent (QMouseEvent *  event)
{
  if( event->button() == Qt::LeftButton ) {
    if( event->modifiers() & Qt::ControlModifier ){
      ZoomOut();
    }else if(  event->modifiers() & Qt::ShiftModifier ){
      RescaleToShowAll();
    };
  };
  MouseOperationStarted=false;
}


void  sChartViewport::dragEnterEvent (QDragEnterEvent *  event)
{
  if( event->mimeData()
           ->hasFormat(ToQString(sNet2TreeEvolution::sItem::MimeType)) ){
//    sPtr<sDependence>           Curve(sNet2TreeEvolution::sItem::DraggedItem
//                                                                ->AttrValue());
//    if( Curve.IsCorrect() ){
//      event->setDropAction(Qt::LinkAction);
//      event->accept();
//    };
    sNodePtr                  DraggedNode(sNet2TreeEvolution::sItem::DraggedItem
                                          ->AttrValue());
    psGraphObject             GrOb = dynamic_cast<psGraphObject>
                                                 (DraggedNode.operator ->());
    if( GrOb ){
      event->setDropAction(Qt::LinkAction);
      event->accept();
    };
  };
}


void  sChartViewport::dropEvent (QDropEvent *  event)
{
  if( event->mimeData()
           ->hasFormat(ToQString(sNet2TreeEvolution::sItem::MimeType)) ){
//    sPtr<sDependence>           Curve(sNet2TreeEvolution::sItem::DraggedItem
//                                                                ->AttrValue());
//    if( Curve.IsCorrect() ){
//      event->setDropAction(Qt::LinkAction);
//      event->accept();
//      TreeRoot()->AttrValue() << sAttribute(sAttrName(),Curve);
//      Evolution->Update();
//      sBoundaries                 B( Curve->Boundaries() );
    sNodePtr                  DraggedNode(sNet2TreeEvolution::sItem::DraggedItem
                                          ->AttrValue());
    psGraphObject             GrOb = dynamic_cast<psGraphObject>
                                                 (DraggedNode.operator ->());
    if( GrOb ){
      event->setDropAction(Qt::LinkAction);
      event->accept();
      TreeRoot()->AttrValue() << sAttribute(sAttrName(),DraggedNode);
      Evolution->Update();
      sBoundaries                 B( GrOb->Boundaries() );
      ScalesHistory.push(ChartScales());
      XAxis->SetRange(sPhysRange( B.X.From() - 0.05*B.X.Size()
                                , B.X.To()   + 0.05*B.X.Size() ));
      YAxis->SetRange(sPhysRange( B.Y.From() - 0.05*B.Y.Size()
                                , B.Y.To()   + 0.05*B.Y.Size() ));
      try{
        ScalesHistory.top().X.Range.Units().ToBasic() + B.X.Units().ToBasic();
      }catch(rxRAlgebra){
        XAxis->Variable = "";
      };
      try{
        ScalesHistory.top().Y.Range.Units().ToBasic() + B.Y.Units().ToBasic();
      }catch(rxRAlgebra){
        YAxis->Variable = "";
      };
      emit ScalesChanged();
//      RescaleToShowAll();
    };
  };
}


void  sChartViewport::contextMenuEvent (QContextMenuEvent *  event)
{
  sNet2TreeEvolution::psItem  TreeItem = FindTreeItemAt(event->pos());
  if( ! TreeItem )
    TreeItem = TreeRoot();
  sNetViewMenu/*sChartMenu*/                  PopupMenu(this,TreeItem);
  PopupMenu.SetHeader(TreeItem->HyperText());
  if( ! MenuGenerators.empty() ){
    for( sNetViewMenuGenerators::iterator  i=MenuGenerators.begin();
                                           i!=MenuGenerators.end(); ++i)
      (*i)(&PopupMenu,TreeItem);
  }
  if( PopupMenu.AddNew->isEmpty() ){
    delete PopupMenu.AddNew;  PopupMenu.AddNew=NULL;
  }
  if( PopupMenu.InsertNew->isEmpty() ){
    delete PopupMenu.InsertNew;  PopupMenu.InsertNew=NULL;
  }
  if( TreeItem == TreeRoot() ){
    psChart  Chart(dynamic_cast<psChart>(parentWidget()));
    if( Chart ){
      if( ! PopupMenu.actions().empty() )  PopupMenu.addSeparator();
      PopupMenu.addActions(Chart->actions());
      if( Chart->parentWidget()->parentWidget() ){
        if( ! PopupMenu.actions().empty() )  PopupMenu.addSeparator();
        PopupMenu.addActions(Chart->parentWidget()->parentWidget()->actions());
      }
    }
  }
  QAction *                   SelectedAction
                                  = PopupMenu.exec(event->globalPos());
  if(psNetViewAction MyAction=dynamic_cast<psNetViewAction>(SelectedAction) ){
    (*(MyAction->DoIt))(TreeItem,sPoint(event->pos().x(),event->pos().y()));
    Evolution->Update();
  }
}


void  sChartViewport::OnCursorPositionChanged (QPoint  cursor)
{
  sPhysPair                   Cursor(XAxis->T(cursor.x())
                                    ,YAxis->T(cursor.y()));
  emit CursorPositionChanged(Cursor ,Reference()
                            ,XAxis->Variable ,YAxis->Variable);
}

//-------------------------------------------------------------- sChart ---

sChart::~sChart ()
{
  if( Viewport ){ delete Viewport;  Viewport=NULL; };
  if( XAxis ){ delete XAxis;  XAxis=NULL; };
  if( YAxis ){ delete YAxis;  YAxis=NULL; };
}


sChart::sChart (psNet2TreeEvolution  evolution
               ,QWidget *            parent)
  :QFrame(parent)
  ,XAxis( new sX_Axis )
  ,YAxis( new sY_Axis )
  ,Viewport( new sChartViewport(XAxis,YAxis,evolution,this) )
//  ,Margins(10,30,60,10)
  ,Margins(10,40,70,10)
{
//  setContextMenuPolicy(Qt::DefaultContextMenu);

  UpdateAxes();
  connect(Viewport,SIGNAL(ScalesChanged()),this,SLOT(UpdateAxes()));
  connect(Viewport ,SIGNAL(CursorPositionChanged(sPhysPair,sPhysPair
                                                ,sString,sString))
         ,this     ,SIGNAL(CursorPositionChanged(sPhysPair,sPhysPair
                                                ,sString,sString)));

  QAction *                   A;
  A = new QAction(QIcon(":/RAGUI/icons/zoomout.png") ,tr("Zoom out") ,this);
  connect(A,SIGNAL(triggered()),this,SLOT(ZoomOut()));
  addAction(A);
  A = new QAction(QIcon(":/RAGUI/icons/rescaletoshowall.png")
                 ,tr("Rescale to show all") ,this);
  connect(A,SIGNAL(triggered()),this,SLOT(RescaleToShowAll()));
  addAction(A);
  A = new QAction(QIcon(":/RAGUI/icons/rescaleback.png")
                 ,tr("Return to previous scale") ,this);
  /*! @todo{UI++} it sets up a global shortcut (ignores key focus);
  if there are 2 Charts, it says: ambigous shortcut*/
//  A->setShortcut(Qt::Key_Backspace);
  connect(A,SIGNAL(triggered()),this,SLOT(RescaleBack()));
  addAction(A);
  A = new QAction("separator",this);
  A->setSeparator(true);
  addAction(A);
  A = new QAction(tr("Edit scales..."),this);
  connect(A,SIGNAL(triggered()),this,SLOT(EditScales()));
  addAction(A);
}


void  sChart::RestoreScales (sPtr<sFolderNode>  C)
{
  if( C.IsCorrect() ){
    sPtr<sTextNode>           V;
    sPtr<sPhysRangeNode>      R;
    V = C >> "ScaleX.Variable";
    if( V.IsCorrect() )  XAxis->Variable=V->Value();
    V = C >> "ScaleY.Variable";
    if( V.IsCorrect() )  YAxis->Variable=V->Value();

    R = C >> "ScaleX.Range";
    if( R.IsCorrect() )  XAxis->SetRange((sPhysRange)(*R));
    R = C >> "ScaleY.Range";
    if( R.IsCorrect() )  YAxis->SetRange((sPhysRange)(*R));
    UpdateAxes();
  };
}


void  sChart::StoreScales (sPtr<sFolderNode>  C) const
{
  if( C.IsCorrect() ){
    C   << attr("ScaleX.Variable",XAxis->Variable)
        << attr("ScaleX.Range",XAxis->PhysRange())
        << attr("ScaleY.Variable",YAxis->Variable)
        << attr("ScaleY.Range",YAxis->PhysRange());
  };
}


void  sChart::SetChartScales (sChartScales  scales)
{
  Viewport->SetChartScales(scales);
}


void  sChart::AllowCurveDrops (bool  Allow)
{
  Viewport->AllowCurveDrops(Allow);
}


void  sChart::paintEvent (QPaintEvent *)
{
  QPainter                    painter(this);
  XAxis->Draw(&painter);
  YAxis->Draw(&painter);
}


void  sChart::Print (QPainter *  painter)
{
  XAxis->Draw(painter);
  YAxis->Draw(painter);
  painter->save();
  painter->translate(Viewport->pos());
  painter->setClipRect(Viewport->rect());
  Viewport->DrawGraphObjects(painter);
  painter->restore();
}


void  sChart::resizeEvent (QResizeEvent *)
{
  if( Viewport ){
    UpdateAxes();
  }
}


void sChart::mouseDoubleClickEvent (QMouseEvent *  /*event*/)
{
  EditScales();
}


void  sChart::contextMenuEvent (QContextMenuEvent *  event)
{
  QMenu                       PopupMenu(this);
  PopupMenu.addActions(actions());
  if( parentWidget()->parentWidget() ){
    if( ! PopupMenu.actions().empty() )  PopupMenu.addSeparator();
    PopupMenu.addActions(parentWidget()->parentWidget()->actions());
  }
  PopupMenu.exec(event->globalPos());
}


void  sChart::EditScales ()
{
  sEditScalesDialog                  D(parentWidget());
  D.fldAbscissVariableName->setText(ToQString(XAxis->Variable));
  D.fldOrdinateVariableName->setText(ToQString(YAxis->Variable));
  D.fldAbscissRange->SetValue(XAxis->PhysRange());
  D.fldOrdinateRange->SetValue(YAxis->PhysRange());
  if( D.exec() ){
    Viewport->ChangeChartScales(sChartScales
        (sChartScale(FromQString(D.fldAbscissVariableName->text().simplified())
                   ,D.fldAbscissRange->Value())
        ,sChartScale(FromQString(D.fldOrdinateVariableName->text().simplified())
                   ,D.fldOrdinateRange->Value())
        ));
    UpdateAxes();
  };
}


void  sChart::UpdateAxes ()
{
  QPoint                      TopLeft (Margins.Left(),Margins.Top());
  QSize                       ViewportNominalSize ( size().width()
                                                   -Margins.Left()
                                                   -Margins.Right()
                                                  , size().height()
                                                   -Margins.Top()
                                                   -Margins.Bottom()
                                                  );
  int                         AxisLinewidth = 1;

  Viewport->SetNominalSize(ViewportNominalSize);
  XAxis->SetSize(ViewportNominalSize.width());
  YAxis->SetSize(ViewportNominalSize.height());

  XAxis->SetPenWidth(AxisLinewidth);
  YAxis->SetPenWidth(AxisLinewidth);
  XAxis->Origin=QPoint( TopLeft.x()
                      , TopLeft.y() + ViewportNominalSize.height() );
  XAxis->OppositeOrigin=QPoint( TopLeft.x() , TopLeft.y() );
  YAxis->Origin=QPoint( XAxis->Origin.x() , XAxis->Origin.y() );
  YAxis->OppositeOrigin=QPoint(TopLeft.x() + ViewportNominalSize.width()
                              ,TopLeft.y() + ViewportNominalSize.height() );

  Viewport->resize(ViewportNominalSize.width()  - AxisLinewidth
                  ,ViewportNominalSize.height() - AxisLinewidth);
  Viewport->SetNominalOrigin(QPoint( -(AxisLinewidth+1)/2
                                   , -(AxisLinewidth+1)/2 ));
  Viewport->move(Margins.Left() + (AxisLinewidth+1)/2
                ,Margins.Top()  + (AxisLinewidth+1)/2 );

  XAxis->Arrange(fontMetrics());
  YAxis->Arrange(fontMetrics());
  setMinimumSize(40 + Margins.Left() + Margins.Right()
                ,40 + Margins.Top()  + Margins.Bottom() );
  update();
}

//------------------------------------------------------------------------------
} //namespace RA

