////////////////////////////////////////////////////////////////////////////////
/*! @file Chart.h   Виджет для показа графиков.
- Part of RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2010-2020, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef Chart_H
#define Chart_H
#include "NetView.h"
#include "Units.h"
#include <QMouseEvent>
#include <QContextMenuEvent>
#include <QFontMetrics>
#include <QRubberBand>
#include <QTextDocument>
#include <stack>
namespace RA {
//------------------------------------------------------------------------------
//ANNOUNCE_CLASS(sChartItem)
//ANNOUNCE_CLASS(sChartMenu)
ANNOUNCE_CLASS(sChartScale)
ANNOUNCE_CLASS(sChartScales)
ANNOUNCE_CLASS(sMargins)
ANNOUNCE_CLASS(sTicks)
ANNOUNCE_CLASS(sLabel)
ANNOUNCE_CLASS(sLabels)
ANNOUNCE_CLASS(sAxis)
ANNOUNCE_CLASS(sX_Axis)
ANNOUNCE_CLASS(sY_Axis)

ANNOUNCE_CLASS(sChartViewport)
ANNOUNCE_CLASS(sChart)

typedef bool                  fDrawProc (QPainter *       painter
                                        ,psChartViewport  viewport
                                        ,psGraphObject    object
                                        ,pcsAppearance    appearance);
typedef const fDrawProc *     pcfDrawProc;

//--------------------------------------------------------------- sChartItem ---

//class sChartItem : public sNetViewItem
//{
//  public:
////    virtual                   ~sChartItem ();
//    explicit                  sChartItem (psNetViewport
//                                         ,sNet2TreeEvolution::psItem
//                                         ,pfChartItemDrawProc);
//  public://fields
//    cpfChartItemDrawProc      Draw;
//    QRect                     ItemArea;
//  protected:
//};

//--------------------------------------------------------------- sChartMenu ---

//class sChartMenu : public sNetViewMenu
//{
//  Q_OBJECT
//  public:
////    virtual                   ~sChartMenu ();
//                              sChartMenu (psNetViewport
//                                         ,sNet2TreeEvolution::psItem);
//};

//-------------------------------------------------------------- sChartScale ---

class RAGUI_EXPORT  sChartScale
{
  public:
                              sChartScale ()
                                  :Name(),Range()
                                {}
                              sChartScale (sString  name ,sPhysRange  range)
                                  :Name(name),Range(range)
                                {}
                              sChartScale (rcsChartScale  S)
                                  :Name(S.Name),Range(S.Range)
                                {}
    rsChartScale               operator = (rcsChartScale  S)
                                { Name=S.Name; Range=S.Range; return *this; }
  public: //fields
    sString                   Name;
    sPhysRange                Range;
};

//------------------------------------------------------------- sChartScales ---
/*!

@todo{elegancy} Переписать с использованием sScales?
*/

class RAGUI_EXPORT  sChartScales
{
  public:
                              sChartScales ()
                                  :X(),Y()
                                {}
                              sChartScales (sChartScale  x ,sChartScale  y)
                                  :X(x),Y(y)
                                {}
                              sChartScales (rcsChartScales  S)
                                  :X(S.X),Y(S.Y)
                                {}
    rsChartScales             operator = (rcsChartScales  S)
                                { X=S.X; Y=S.Y; return *this; }
    void                      RestoreSettings (sPtr<sFolderNode>);
    void                      StoreSettings (sPtr<sFolderNode>) const;

  public: //fields
    sChartScale               X;
    sChartScale               Y;
};

//----------------------------------------------------------------- sMargins ---

class sMargins : public QObject
{
  Q_OBJECT
  public:
                              sMargins (int  top  ,int  bottom
                                       ,int  left ,int  right);
    int                       Top () const
                                { return TheTop; }
    void                      SetTop (int i)
                                { TheTop=i; emit Changed(); }
    int                       Bottom () const
                                { return TheBottom; }
    void                      SetBottom (int i)
                                { TheBottom=i; emit Changed(); }
    int                       Left () const
                                { return TheLeft; }
    void                      SetLeft (int i)
                                { TheLeft=i; emit Changed(); }
    int                       Right () const
                                { return TheRight; }
    void                      SetRight (int i)
                                { TheRight=i; emit Changed(); }
  signals:
    void                      Changed ();

  private: //fields
    int                       TheTop;
    int                       TheBottom;
    int                       TheLeft;
    int                       TheRight;
};

//------------------------------------------------------------------- sTicks ---

class sTicks
{
  public:
//    virtual                   ~sTicks ();
                              sTicks ();
  public: //fields
    real                      Start;
    real                      Interval;
    int                       Length;
//    TPen *                    Pen;
  private: //fields
};

//------------------------------------------------------------------- sLabel ---

class sLabel
{
  public:
//    virtual                   ~sLabel ();
                              sLabel (const QString &  text)
                                  :Text(text)
                                {}
  public: //fields
    QString                   Text;
    QRect                     Rect;
};

//------------------------------------------------------------------ sLabels ---

class sLabels
{
  public:
    virtual                   ~sLabels ();
                              sLabels ();
    void                      Clear ();
  public: //fields
    unsigned int              MaxAmount;
    std::list<sLabel>         List;
};

//-------------------------------------------------------------------- sAxis ---

class sAxis : public sScale
{
  public:
                              sAxis (bool  upside_down);
    virtual void              SetPenWidth (int  width);
    sScale                    Scale ()
                                { return *this; }
    int                       Value2Offset (real);
    virtual void              Arrange (const QFontMetrics &);
    virtual void              Draw (QPainter *) =0;

  protected:
    void                      PrearrangeLabels ();
    virtual bool              ArrangeLabels (const QFontMetrics &) =0;
    QString                   TitleText ();

  public: //fields
    sString                   Variable;
    QPoint                    Origin; //!< starting position of the axis line
    QPoint                    OppositeOrigin; //!< origin of complimentary axis
    QRect                     LabelsArea;
    QRect                     TitleArea;

  protected: //fields
    int                       PenWidth;
    sTicks                    MajorTicks;
    sTicks                    MinorTicks;
    sLabels                   Labels;
    QTextDocument             Title;
    int                       MultiplierExponent;
    real                      Multiplier;
    real                      Shift;
    bool                      LabelsArranged;
    bool                      LabelsOverlap;

  private: //fields
};

//------------------------------------------------------------------ sX_Axis ---

class sX_Axis : public sAxis
{
  public:
                              sX_Axis ()
                                  :sAxis(false)
                                { Variable="x"; }
    virtual void              Draw (QPainter *);
  protected:
    virtual bool              ArrangeLabels (const QFontMetrics &);
};

//------------------------------------------------------------------ sY_Axis ---

class sY_Axis : public sAxis
{
  public:
                              sY_Axis ()
                                  :sAxis(true)
                                { Variable="y"; }
    virtual void              Draw (QPainter *);
  protected:
    virtual bool              ArrangeLabels (const QFontMetrics &);
};

//----------------------------------------------------------- sChartViewport ---

class RAGUI_EXPORT  sChartViewport : public sNetViewport
{
  Q_OBJECT
  public://static
    static bool               NegativeBackground;
    static QCursor *          CrossHairDark;
    static QCursor *          CrossHairBright;
    static QCursor *          ZoomCursor;
    static QCursor *          ShiftCursor;
    static QCursor *          ZoomingCursor;
    static QCursor *          ShiftingCursor;
    static sNetViewMenuGenerators
                              MenuGenerators;
    static std::list
               <pcfDrawProc>  DrawProcCollection;

  public:
    virtual                   ~sChartViewport ();
    explicit                  sChartViewport (psX_Axis             x_axis
                                             ,psY_Axis             y_axis
                                             ,psNet2TreeEvolution  evolution
                                             ,QWidget *            parent = 0);
    virtual void              FormItems ();
    virtual sNet2TreeEvolution::psItem
                              FindTreeItemAt (const QPoint &);
    void                      AllowCurveDrops (bool  Allow = true);
    sScales                   Scales () const
                                {
                                  return sScales(XAxis->Scale(),YAxis->Scale());
                                }
    sChartScales              ChartScales () const;
    void                      SetChartScales (sChartScales);
    void                      ChangeChartScales (sChartScales);
    sPhysPair                 Reference ()
                                { return TheReference; }
    void                      SetReference (sPhysPair  P);
    void                      SetReference (QPoint  P);
    void                      ZoomOut ();
    void                      RescaleToShowAll ();
    void                      SetNominalSize (QSize);
    void                      SetNominalOrigin (QPoint);
    QPoint                    T (sPhysPair) const;
    QPoint                    T (rcsPoint) const;
    sPhysPair                 T (QPoint) const;
    virtual void              DrawGraphObjects (QPainter *);

  public slots:
    void                      RescaleBack ();

  protected://types
    enum  eMouseOperationType  {MovingInitialPoint
                               ,Zooming
                               ,MovingFrame
                               ,VisibleObjectDragging
                               };

  signals:
    void                      ScalesChanged ();
    void                      CursorPositionChanged (QPoint  cursor);
    void                      CursorPositionChanged (sPhysPair  position
                                                    ,sPhysPair  reference
                                                    ,sString    x_title
                                                    ,sString    y_title);

  protected:
    virtual sBoundaries       BoundariesForAllObjects ();

  protected://inherited
    virtual void              paintEvent (QPaintEvent *);
    virtual void	            keyPressEvent (QKeyEvent *);
    virtual void              mousePressEvent (QMouseEvent *);
    virtual void              mouseReleaseEvent (QMouseEvent *);
    virtual void              mouseMoveEvent (QMouseEvent *);
    virtual void              mouseDoubleClickEvent (QMouseEvent *);
    virtual void              dragEnterEvent (QDragEnterEvent *);
    virtual void              dropEvent (QDropEvent *);
    virtual void              contextMenuEvent (QContextMenuEvent *);

  private slots:
    void                      OnCursorPositionChanged (QPoint  cursor);

  private:
    void                      Clear ();

  protected://fields
    std::stack<sChartScales>  ScalesHistory;

  private://static
    static const real         ZoomOutFactor;

  private://fields
//    sChartItems               Items;
//    psChartItem               ContextItem;
    psX_Axis                  XAxis;
    psY_Axis                  YAxis;
    sPhysPair                 TheReference;
    QSize                     NominalSize;
    QPoint                    NominalOrigin;
    bool                      MouseOperationIsAboutToStart;
    QPoint                    MouseOperationStartPosition;
    QPoint                    MouseOperationLastPosition;
    bool                      MouseOperationStarted;
    eMouseOperationType       MouseOperationType;
    bool                      DragIsAboutToStart;
    bool                      CurveDropsAreAllowed;
    QRubberBand *             ZoomRect;
};

//------------------------------------------------------------------- sChart ---

class RAGUI_EXPORT  sChart : public QFrame
{
  Q_OBJECT
  public:
    virtual                   ~sChart ();
    explicit                  sChart (psNet2TreeEvolution  evolution
                                     ,QWidget *            parent = 0);
    sChartScales              ChartScales () const
                                { return Viewport->ChartScales(); }
    void                      SetChartScales (sChartScales);
    sPhysPair                 Reference ()
                                { return Viewport->Reference(); }
    void                      SetReference (sPhysPair  reference)
                                { Viewport->SetReference(reference); }
    void                      AllowCurveDrops (bool  Allow = true);
    virtual void              RestoreScales (sPtr<sFolderNode>);
    virtual void              StoreScales (sPtr<sFolderNode>) const;
    virtual void              Print (QPainter *);

  protected://inherited
    virtual void              paintEvent (QPaintEvent *);
    virtual void              resizeEvent (QResizeEvent *);
    virtual void              mouseDoubleClickEvent (QMouseEvent *);
    virtual void              contextMenuEvent (QContextMenuEvent *);

  signals:
    void                      CursorPositionChanged (sPhysPair  position
                                                    ,sPhysPair  reference
                                                    ,sString    x_title
                                                    ,sString    y_title);

  public slots:
    void                      OnNetChange ()
                                { Viewport->FormItems(); }
    void                      UpdateAxes ();
    void                      ZoomOut ()
                                { Viewport->ZoomOut(); }
    void                      RescaleBack ()
                                { Viewport->RescaleBack(); }
    void                      RescaleToShowAll ()
                                { Viewport->RescaleToShowAll(); }
    void                      EditScales ();

  private://fields
    psX_Axis                  XAxis;
    psY_Axis                  YAxis;
    psChartViewport           Viewport;
    sMargins                  Margins;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif // Chart_H

