////////////////////////////////////////////////////////////////////////////////
/*! @file ColorDialog.cpp   Color Select Dialog with custom colors that are
                            stored in the net.
- Part of RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2016, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#include "ColorDialog.h"
#include "Globals.h"
namespace RA {
//------------------------------------------------------------------------------
//------------------------------------------------------------- sColorDialog ---

sColorDialog::sColorDialog (QWidget *  parent)
    :QColorDialog(parent)
{
  setOptions( DontUseNativeDialog | ShowAlphaChannel );
}


sColorDialog::sColorDialog (rcsColor   current_color
                           ,QWidget *  parent)
    :QColorDialog(parent)
{
  setOptions( DontUseNativeDialog | ShowAlphaChannel );
  SetCurrentColor(current_color);
}


int  sColorDialog::exec ()
{
  int                         Result;
  sColorSet                   OldCustomColors;
  bool                        Got = false;
  if( get_from<sColorSetNode>(RAGUI::CustomColors,OldCustomColors) ){
    Got = true;
    for( int i = 0 ; i < OldCustomColors.Size() ; i++ ){
      setCustomColor(i ,QColor(OldCustomColors[i].Color.Red()
                              ,OldCustomColors[i].Color.Green()
                              ,OldCustomColors[i].Color.Blue()
                              ,OldCustomColors[i].Color.Alpha() ));
    }
  }
  Result = QColorDialog::exec();
  if( Result && Got ){
    bool                        ToPut = false;
    sColorSet                   NewCustomColors;
    sColor                      C;
    for( int i = 0 ; i < 16 ; i++ ){
      C = sColor(customColor(i).red()
                ,customColor(i).green()
                ,customColor(i).blue()
                ,customColor(i).alpha() );
      NewCustomColors.Add(C);
      ToPut = ToPut || (i >= OldCustomColors.Size())
                    || (C != OldCustomColors[i].Color);
    }
    if( ToPut )
      RAGUI::CustomColors->Assign(NewCustomColors);
  }
  return Result;
}

//------------------------------------------------------------------------------
} //namespace RA
