////////////////////////////////////////////////////////////////////////////////
/*! @file ColorsUI.h   Dialogs and menu generator for colors and color sets.
- Part of RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2015, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef ColorsUI_H
#define ColorsUI_H
#include "NetView.h"
#include <QAbstractButton>
#include <QSignalMapper>
#include <QLineEdit>
#include <QDialog>
#include <QGridLayout>
#include <QPushButton>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sColorSetView)
ANNOUNCE_CLASS(sColorSetSelect)
ANNOUNCE_CLASS(sColorSetEdit)

//---------------------------------------------------------------- sColorsUI ---

class sColorsUI : public QObject
{
  Q_OBJECT
  public:
                              sColorsUI ();
    static void               MenuGenerator (psNetViewMenu
                                            ,sNet2TreeEvolution::pcsItem);
    static void               EditColor (sNet2TreeEvolution::pcsItem ,sPoint);
    static void               EditColorSet (sNet2TreeEvolution::pcsItem
                                           ,sPoint);
    static void               AddNewColor (sNet2TreeEvolution::pcsItem ,sPoint);
    static void               AddNewColorSet (sNet2TreeEvolution::pcsItem
                                             ,sPoint);
};

//------------------------------------------------------------ sColorSetView ---
/*!

*/
class sColorSetView : public QFrame
{
  Q_OBJECT
  public: //static const
    static const int          cMaxSize = 32;

  public:
                              sColorSetView (sColorSet  color_set
                                            ,sColor     background
                                            ,bool       editable = false
                                            ,QWidget *  parent = NULL);
    bool                      Selected () const
                                { return HasSelection; }
    int                       SelectedIndex () const;

  signals:
    void                      Choosed (int  color_index);

  public slots:
    void                      Select (sColor  color);
    void                      Select (int  index);

  public: //fields
    sColorSet                 ColorSet;

  private slots:
    void                      ChangeName (int  index);
    void                      Restyle ();

  private:
    typedef QAbstractButton * pQAbstractButton;
    typedef QLineEdit *       pQLineEdit;

  private: //fields
    sColor                    Background;
    bool                      Editable;
    bool                      HasSelection;
    QSignalMapper *           ColorSelectMapper;
    QSignalMapper *           NameEditedMapper;
    pQAbstractButton          Button [cMaxSize];
    pQLineEdit                Name [cMaxSize];
};

//---------------------------------------------------------- sColorSetSelect ---
/*!

*/
class sColorSetSelect : public QFrame
{
  Q_OBJECT
  public:
    explicit                  sColorSetSelect (sColorSet  color_set
                                              ,sColor     background
                                              ,QWidget *  parent = NULL);
    rcsColor                  Color () const
                                { return TheColor; }
    void                      SetColor (rcsColor  color);

  signals:
    void                      ColorChanged ();

  protected slots:
    void                      OnStandardColorChoosed (int  color_index);
    void                      EditAndSetCustomColor ();
    void                      RestyleCustomColorButton ();

  private: //fields
    sColor                    TheColor;
    psColorSetView            StandardColors;
    sColor                    Background;
    QAbstractButton *         CustomColorButton;
};


//------------------------------------------------------------ sColorSetEdit ---
/*!

*/
class sColorSetEdit : public QDialog
{
  Q_OBJECT
  public:
    explicit                  sColorSetEdit (sColorSet  color_set
                                            ,sColor     background
                                            ,QWidget *  parent = NULL);
    rcsColorSet               Colors () const
                                { return ColorSetView->ColorSet; }

  protected:
    void                      SetColors (rcsColorSet  colors);

  protected slots:
    void                      Refresh ();
    void                      MoveUp ();
    void                      MoveDown ();
    void                      Edit ();
    void                      Remove ();
    void                      Add ();
    void                      AddBasicColors ();

  protected: //fields
    psColorSetView            ColorSetView;
    sColor                    Background;
    QGridLayout *             Layout;
    QPushButton *             MoveUpButton;
    QPushButton *             EditButton;
    QPushButton *             RemoveButton;
    QPushButton *             MoveDownButton;
    QPushButton *             AddButton;
    QPushButton *             AddBasicColorsButton;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
