////////////////////////////////////////////////////////////////////////////////
/*! @file ConfigureDeviceDialog.h   Class sConfigureDeviceDialog.
- Part of RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2010-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef ConfigureDeviceDialog_H
#define ConfigureDeviceDialog_H
#include "Navigator.h"
#include <QDialog>
#include <QCheckBox>
#include <QLineEdit>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QPushButton>
#include <QComboBox>
#include <QTimer>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sConfigureDeviceDialog)

//--------------------------------------------------- sConfigureDeviceDialog ---

class RAGUI_EXPORT  sConfigureDeviceDialog : public QDialog
{
  Q_OBJECT
  public:
    virtual                   ~sConfigureDeviceDialog ();
    explicit                  sConfigureDeviceDialog (sString    device_id
                                                     ,QWidget *  parent = 0);

  private:
    ANNOUNCE_CLASS(sLinkTableRow)
    class  sLinkTableRow
    {
      public:
        explicit                  sLinkTableRow (sString        link_name
                                                ,sString        class_name
                                                ,sString        dev_id
                                                ,QGridLayout *  layout
                                                ,int            row);
        void                      FillComboBox (sString  current_dev_id);
      public: //fields
        sString                   Link;
        sString                   Class;
        sString                   DevId;
        QComboBox *               ComboBox;
    };

  private:
    void                      FormLinks ();

  private slots:
    void                      Uninstall ();
    void                      CheckNewDeviceId (QString);
    void                      Done ();

  private: //fields
    sString                   DeviceId;
    sPtr<sFolderNode>         DeviceCfg;
    bool                      Disabled;
    sPtr<sFolderNode>         DeviceLinks;
    psNet2TreeEvolution       Evolution;
    psNavigator               Navigator;
    std::list<psLinkTableRow> LinkTable;

    QVBoxLayout *             Layout;
    QPushButton *             BtnUninstall;
    QCheckBox *               BoxDisabled;
    QLineEdit *               FldDeviceId;
    QFrame *                  LinkFrame;
    QGridLayout *             LinkLayout;
    QTimer *                  UpdatingTimer;
    QPushButton *             BtnDone;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
