////////////////////////////////////////////////////////////////////////////////
/*! @file CurvesUI.cpp Функционалы навигатора и окна графика,
                       специфичные для кривых.
- Part of RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2010-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#include "CurvesUI.h"
#include "Navigator.h"
#include "Curves.h"
#include "Globals.h"
#include <QGridLayout>
#include <QDialogButtonBox>
namespace RA {
//------------------------------------------------------------------------------
//----------------------------------------------------------------- sCurveUI ---

sCurveUI::sCurveUI ()
{
  sChartViewport::DrawProcCollection.push_front( & Draw );
  sNavigatorViewport::MenuGenerators.push_front(&MenuGenerator);
}


bool  sCurveUI::Draw (QPainter *       painter
                     ,psChartViewport  viewport
                     ,psGraphObject    object
                     ,pcsAppearance    appearance)
{
  psCurve                     Curve(dynamic_cast<psCurve>(object));
  if( Curve ){
    bool                        Draw = true;
    if( pcsAppearance A = dynamic_cast<pcsAppearance> (appearance) ){
      Draw = ! A->Hidden;
    }
    if( Draw ){
      painter->save();
      bool                        DrawLine = true;
      bool                        DrawPoints = false;
      sCurve::psTracer            T = Curve->GetCurveTracer(viewport->Scales());
      QPen                        Pen=painter->pen();
      if( pcsLineAppearance A = dynamic_cast<pcsLineAppearance> (appearance) ){
        QColor                      C( A->LineColor.Red()
                                      ,A->LineColor.Green()
                                      ,A->LineColor.Blue()
                                      ,A->LineColor.Alpha() );
        Pen.setColor(C);
        Pen.setWidth(A->LineThickness);
        painter->setPen(Pen);
      }
      pcsCurveAppearance          CurveApp(dynamic_cast<pcsCurveAppearance>
                                                               (appearance) );
      if( CurveApp ){
        DrawLine = CurveApp->ShowLine;
        DrawPoints = CurveApp->ShowPoints;
      }
      if( DrawLine ){
        while( T->NextLineSegment() ){
          painter->drawLine(viewport->T(T->LineSegment.Start)
                           ,viewport->T(T->LineSegment.End  ));
        }
      }
      if( DrawPoints ){
        if( CurveApp->Filled ){
          painter->setBrush(QColor(CurveApp->FillColor.Red()
                                  ,CurveApp->FillColor.Green()
                                  ,CurveApp->FillColor.Blue()
                                  ,CurveApp->FillColor.Alpha()  ));
          Pen.setColor(painter->brush().color());
          Pen.setWidth(0);
        }
        if( CurveApp->Contoured ){
          Pen.setColor(QColor(CurveApp->ContourColor.Red()
                             ,CurveApp->ContourColor.Green()
                             ,CurveApp->ContourColor.Blue()
                             ,CurveApp->ContourColor.Alpha()));
          Pen.setWidth(CurveApp->ContourThickness);
        }
        painter->setPen(Pen);
        QPointF                     Center;
        qreal                       S = CurveApp->PointSize / 2.0;
        while( T->NextPoint() ){
          Center = viewport->T(T->Point);
          switch( CurveApp->PointShape ){
            case sPointAppearance::cCircle:
              painter->drawEllipse(Center,1.25*S,1.25*S);
              break;
            case sPointAppearance::cSquare:
              painter->drawRect(QRectF(Center.x()-S ,Center.y()-S ,2*S ,2*S ));
              break;
            default:
  //            painter->fillRect(QRect(Center.x()-HalfwidthX
  //                                   ,Center.y()-HalfwidthY
  //                                   ,2*HalfwidthX+1 ,2*HalfwidthY+1)
  //                             ,Qt::SolidPattern);
              break;
          }
        }
      }
      painter->restore();
    }
    return true;
  }
  return false;
}


void  sCurveUI::MenuGenerator (psNetViewMenu                context_menu
                              ,sNet2TreeEvolution::pcsItem  tree_item)
{
  psCurve                     Curve(dynamic_cast<psCurve>
                                         (tree_item->AttrValue().operator->()));
  if( Curve ){
    context_menu->Insert( new sNetViewAction
        (context_menu->parentWidget()
        ,&EditCurveAppearance  ,/*checkable*/false ,/*checked*/false
        ,sNetViewMenu::tr("Change appearance")
        ,sNetViewMenu::tr("Change color, thickness, etc.")
        ));
  }
}


void  sCurveUI::EditCurveAppearance (sNet2TreeEvolution::pcsItem  tree_item
                                    ,sPoint                       /*pos*/)
{
  psCurve                     Curve( dynamic_cast<psCurve>
                                        (tree_item->AttrValue().operator->()) );
  ASSERT( Curve );
  pcsCurveAppearance          OldCA( dynamic_cast<pcsCurveAppearance>
                                                 (Curve->Appearance()) );
  psCurveAppearance           NewCA( OldCA
                                   ? new sCurveAppearance(*OldCA)
                                   : new sCurveAppearance );
  //!@todo{UI} надо передавать родительский виджет для наследования стилей
  //!          и цвет фона или дневной/ночной режим
  sEditCurveAppearanceDialog  D(NewCA
                               ,RAGUI::LineColorSet->Value()
                               ,( RAGUI::NightRegime ? sColor::cBlack
                                                     : sColor::cWhite )
                               );
  if( D.exec() ){
    Curve->SetAppearance(NewCA);
  }else{
    delete NewCA;
  }
}

//------------------------------------------------ sEditLineAppearanceDialog ---

sEditLineAppearanceDialog::sEditLineAppearanceDialog
                                           (psLineAppearance  appearance
                                           ,sColorSet         color_set
                                           ,sColor            background
                                           ,QWidget *         parent)
    :QDialog(parent)
    ,Appearance(appearance)
{
  setWindowTitle(tr("Line appearance"));

  NotHiddenCheckBox = new QCheckBox(tr("show"),this);
  NotHiddenCheckBox->setChecked( ! Appearance->Hidden );

  LineThicknessLabel = new QLabel(tr("thickness"),this);
  LineThicknessField = new QSpinBox(this);
  LineThicknessField->setRange(sLineAppearance::MinLineThickness
                              ,sLineAppearance::MaxLineThickness);
  LineThicknessField->setValue(Appearance->LineThickness);
  LineThicknessField->setMaximumWidth(LineThicknessField->height()*3);

  LineColors = new sColorSetSelect(color_set,background,this);
  LineColors->SetColor(Appearance->LineColor);

  QDialogButtonBox *  ButtonBox = new QDialogButtonBox(this);
  ButtonBox->setOrientation(Qt::Horizontal);
  ButtonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

  QGridLayout *       Layout = new QGridLayout(this);
  setLayout(Layout);
  Layout->setVerticalSpacing(1);
  Layout->setHorizontalSpacing(3);
  Layout->setMargin(5);
  Layout->setAlignment(Qt::AlignTop);

  Layout->addWidget(NotHiddenCheckBox     , 0,0,1,2);
  Layout->addWidget(LineThicknessLabel    ,10,0     ,Qt::AlignRight);
  Layout->addWidget(LineThicknessField    ,10,1);
  Layout->addWidget(LineColors            ,20,0,1,2);
  Layout->addWidget(new QLabel(" ",this)  ,30,0);
  Layout->addWidget(ButtonBox             ,31,0,1,8);

  Refresh();

  connect(NotHiddenCheckBox ,SIGNAL(toggled(bool))
         ,this              ,SLOT(Refresh())  );
  connect(LineThicknessField ,SIGNAL(valueChanged(int))
         ,this               ,SLOT(Refresh()));
  connect(LineColors ,SIGNAL(ColorChanged()) ,this ,SLOT(Refresh()));
  connect(ButtonBox, SIGNAL(accepted()), this, SLOT(accept()));
  connect(ButtonBox, SIGNAL(rejected()), this, SLOT(reject()));
  resize(100,100);
}


void  sEditLineAppearanceDialog::Refresh ()
{
  Appearance->Hidden = ! NotHiddenCheckBox->isChecked();
  Appearance->LineThickness = LineThicknessField->value();
  Appearance->LineColor = LineColors->Color();

  LineThicknessLabel->setEnabled( ! Appearance->Hidden );
  LineThicknessField->setEnabled( ! Appearance->Hidden );
  LineColors->setEnabled( ! Appearance->Hidden );
}

//----------------------------------------------- sEditCurveAppearanceDialog ---

sEditCurveAppearanceDialog::sEditCurveAppearanceDialog
                                           (psCurveAppearance  appearance
                                           ,sColorSet          color_set
                                           ,sColor             background
                                           ,QWidget *          parent)
    :sEditLineAppearanceDialog(appearance,color_set,background,parent)
    ,Appearance(appearance)
{
  setWindowTitle(tr("Curve appearance"));

  LineCheckBox = new QCheckBox(tr("__line____"),this);
  LineCheckBox->setChecked( Appearance->ShowLine );

  PointsCheckBox = new QCheckBox(tr("__points___________________"),this);
  PointsCheckBox->setChecked( Appearance->ShowPoints );

  FilledCheckBox = new QCheckBox(tr("filled"),this);
  FilledCheckBox->setChecked( Appearance->Filled );

  ContouredCheckBox = new QCheckBox(tr("contoured"),this);
  ContouredCheckBox->setChecked( Appearance->Contoured );

  PointSizeLabel = new QLabel(tr("size"),this);
  PointSizeField = new QSpinBox(this);
  PointSizeField->setRange(sPointAppearance::MinSize
                          ,sPointAppearance::MaxSize);
  PointSizeField->setValue(Appearance->PointSize);
  PointSizeField->setMaximumWidth(PointSizeField->height()*4);

  ContourThicknessLabel1 = new QLabel(tr("contour line"),this);
  ContourThicknessLabel2 = new QLabel(tr("thickness"),this);
  ContourThicknessField = new QSpinBox(this);
  ContourThicknessField->setRange(sPointAppearance::MinContourThickness
                                 ,sPointAppearance::MaxContourThickness);
  ContourThicknessField->setValue(Appearance->ContourThickness);
  ContourThicknessField->setMaximumWidth(ContourThicknessField->height()*3);

  PointShapeLabel = new QLabel(tr("shape"),this);
  PointShapeField = new QComboBox(this);
  PointShapeField->addItem("●",sPointAppearance::cCircle);
  PointShapeField->addItem("◼",sPointAppearance::cSquare); //○◻◇▽△●◼◆▼▲
  PointShapeField->setMaximumWidth(PointShapeField->height()*3);
  PointShapeField->setCurrentIndex( Appearance->PointShape );

  FillSameColorCheckBox = new QCheckBox(tr("same color"),this);
  FillSameColorCheckBox->setToolTip(tr("same color as line"));
  FillSameColorCheckBox->setChecked( Appearance->FillColor
                                     == Appearance->LineColor );

  CntrSameColorCheckBox = new QCheckBox(tr("same color"),this);
  CntrSameColorCheckBox->setToolTip(tr("same color as line"));
  CntrSameColorCheckBox->setChecked( Appearance->ContourColor
                                     == Appearance->LineColor );

  FillingColors = new sColorSetSelect(color_set,background,this);
  FillingColors->SetColor(Appearance->FillColor);

  ContourColors = new sColorSetSelect(color_set,background,this);
  ContourColors->SetColor(Appearance->ContourColor);

  QGridLayout *       Layout = (QGridLayout *)layout();

  Layout->addWidget(LineCheckBox              , 1,0,1,2);
  Layout->addWidget(new QLabel(" ",this)      , 1,2);
  Layout->addWidget(PointsCheckBox            , 1,3,1,5);
  Layout->addWidget(FilledCheckBox            ,10,3,1,2);
  Layout->addWidget(new QLabel(" ",this)      ,10,5);
  Layout->addWidget(ContouredCheckBox         ,10,6,1,2);
  Layout->addWidget(PointSizeLabel            ,11,3     ,Qt::AlignRight);
  Layout->addWidget(PointSizeField            ,11,4);
  Layout->addWidget(ContourThicknessLabel1    ,11,6,1,2 ,Qt::AlignHCenter);
  Layout->addWidget(ContourThicknessLabel2    ,12,6     ,Qt::AlignRight);
  Layout->addWidget(ContourThicknessField     ,12,7);
  Layout->addWidget(PointShapeLabel           ,12,3     ,Qt::AlignRight);
  Layout->addWidget(PointShapeField           ,12,4);
  Layout->addWidget(FillSameColorCheckBox     ,13,3,1,2 ,Qt::AlignHCenter);
  Layout->addWidget(CntrSameColorCheckBox     ,13,6,1,2 ,Qt::AlignHCenter);

  Layout->addWidget(FillingColors             ,20,3,1,2);
  Layout->addWidget(ContourColors             ,20,6,1,2);

  Refresh();

  connect(LineCheckBox ,SIGNAL(toggled(bool))
         ,this         ,SLOT(Refresh())  );
  connect(PointsCheckBox ,SIGNAL(toggled(bool))
         ,this           ,SLOT(Refresh())  );
  connect(FilledCheckBox ,SIGNAL(toggled(bool))
         ,this           ,SLOT(OnTogleFilled(bool)));
  connect(ContouredCheckBox ,SIGNAL(toggled(bool))
         ,this              ,SLOT(OnTogleContoured(bool)));
  connect(PointSizeField ,SIGNAL(editingFinished()) ,this ,SLOT(Refresh()));
  connect(ContourThicknessField ,SIGNAL(editingFinished())
         ,this ,SLOT(Refresh()));
  connect(PointShapeField ,SIGNAL(currentIndexChanged(int))
         ,this            ,SLOT(Refresh()));
  connect(FillSameColorCheckBox ,SIGNAL(toggled(bool))
         ,this                  ,SLOT(Refresh())  );
  connect(CntrSameColorCheckBox ,SIGNAL(toggled(bool))
         ,this                  ,SLOT(Refresh())  );
  connect(FillingColors ,SIGNAL(ColorChanged()) ,this ,SLOT(Refresh()));
  connect(ContourColors ,SIGNAL(ColorChanged()) ,this ,SLOT(Refresh()));
}


void  sEditCurveAppearanceDialog::OnTogleFilled (bool  checked)
{
  if( ! checked )  ContouredCheckBox->setChecked(true);
  Refresh();
}


void  sEditCurveAppearanceDialog::OnTogleContoured (bool  checked)
{
  if( ! checked )  FilledCheckBox->setChecked(true);
  Refresh();
}


void  sEditCurveAppearanceDialog::Refresh ()
{
  sEditLineAppearanceDialog::Refresh();

  Appearance->ShowLine = LineCheckBox->isChecked();
  Appearance->ShowPoints = PointsCheckBox->isChecked();
  Appearance->Filled = FilledCheckBox->isChecked();
  Appearance->Contoured = ContouredCheckBox->isChecked();
  Appearance->PointSize = PointSizeField->value();
  Appearance->PointShape = sPointAppearance::eShape(
                                             PointShapeField->currentIndex()
                                                     );
  Appearance->ContourThickness = ContourThicknessField->value();
  Appearance->FillColor = ( FillSameColorCheckBox->isChecked()
                          ? Appearance->LineColor
                          : FillingColors->Color()
                          );
  Appearance->ContourColor = ( CntrSameColorCheckBox->isChecked()
                             ? Appearance->LineColor
                             : ContourColors->Color()
                             );
  if( Appearance->ShowLine && Appearance->ShowPoints
      && Appearance->Filled && FillSameColorCheckBox->isChecked()
      && (Appearance->FillColor != Appearance->LineColor)        )
    FillingColors->SetColor( Appearance->LineColor );
  if( Appearance->ShowLine && Appearance->ShowPoints
      && Appearance->Contoured && CntrSameColorCheckBox->isChecked()
      && (Appearance->ContourColor != Appearance->LineColor)            )
    ContourColors->SetColor( Appearance->LineColor );

  LineCheckBox->setEnabled( ! Appearance->Hidden );
  PointsCheckBox->setEnabled( ! Appearance->Hidden );
  LineThicknessLabel->setEnabled( ! Appearance->Hidden
                                 && Appearance->ShowLine );
  LineThicknessField->setEnabled( ! Appearance->Hidden
                                 && Appearance->ShowLine );
  FilledCheckBox->setEnabled( ! Appearance->Hidden
                             && Appearance->ShowPoints );
  ContouredCheckBox->setEnabled( ! Appearance->Hidden
                                && Appearance->ShowPoints );
  PointSizeLabel->setEnabled( ! Appearance->Hidden
                             && Appearance->ShowPoints);
  PointSizeField->setEnabled( ! Appearance->Hidden
                             && Appearance->ShowPoints);
  PointShapeLabel->setEnabled( ! Appearance->Hidden
                              && Appearance->ShowPoints);
  PointShapeField->setEnabled( ! Appearance->Hidden
                              && Appearance->ShowPoints);
  ContourThicknessLabel1->setEnabled( ! Appearance->Hidden
                              && Appearance->ShowPoints
                              && Appearance->Contoured );
  ContourThicknessLabel2->setEnabled( ! Appearance->Hidden
                              && Appearance->ShowPoints
                              && Appearance->Contoured );
  ContourThicknessField->setEnabled( ! Appearance->Hidden
                              && Appearance->ShowPoints
                              && Appearance->Contoured );
  FillSameColorCheckBox->setEnabled( ! Appearance->Hidden
                                    && Appearance->ShowPoints
                                    && Appearance->Filled );
  CntrSameColorCheckBox->setEnabled( ! Appearance->Hidden
                                    && Appearance->ShowPoints
                                    && Appearance->Contoured );
  LineColors->setEnabled( ! Appearance->Hidden
      &&( Appearance->ShowLine
        ||( Appearance->ShowPoints
          &&( (Appearance->Filled && FillSameColorCheckBox->isChecked())
            ||(Appearance->Contoured && CntrSameColorCheckBox->isChecked())
            )
          )
        )
      );
  FillingColors->setVisible( ! Appearance->Hidden
                            && Appearance->ShowPoints
                            && Appearance->Filled
                            && ! FillSameColorCheckBox->isChecked() );
  ContourColors->setVisible( ! Appearance->Hidden
                            && Appearance->ShowPoints
                            && Appearance->Contoured
                             && ! CntrSameColorCheckBox->isChecked() );
}

//------------------------------------------------------------------------------
} //namespace RA
