////////////////////////////////////////////////////////////////////////////////
/*! @file CurvesUI.h   Функционалы навигатора и окна графика,
                       специфичные для кривых.
- Part of RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2010-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef CurvesUI_H
#define CurvesUI_H
#include "NetView.h"
#include "Chart.h"
#include "ColorsUI.h"
#include <QPainter>
#include <QDialog>
#include <QCheckBox>
#include <QSpinBox>
#include <QButtonGroup>
#include <QRadioButton>
#include <QComboBox>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sEditLineAppearanceDialog)
ANNOUNCE_CLASS(sEditCurveAppearanceDialog)

//----------------------------------------------------------------- sCurveUI ---

class sCurveUI
{
  public:
                              sCurveUI ();
    static bool               Draw (QPainter *       painter
                                   ,psChartViewport  viewport
                                   ,psGraphObject    object
                                   ,pcsAppearance    appearance);
    static void               MenuGenerator (psNetViewMenu
                                            ,sNet2TreeEvolution::pcsItem);
    static void               EditCurveAppearance (sNet2TreeEvolution::pcsItem
                                                  ,sPoint);
};

//------------------------------------------------ sEditLineAppearanceDialog ---
/*!

*/
class sEditLineAppearanceDialog : public QDialog
{
  Q_OBJECT
  public:
    explicit                  sEditLineAppearanceDialog
                                    (psLineAppearance  appearance
                                    ,sColorSet         color_set
                                    ,sColor            background
                                    ,QWidget *         parent = NULL);

  protected slots:
    virtual void              Refresh ();

  protected: //fields
    psLineAppearance          Appearance;
    QCheckBox *               NotHiddenCheckBox;
    QLabel *                  LineThicknessLabel;
    QSpinBox *                LineThicknessField;
    psColorSetSelect LineColors;
};

//----------------------------------------------- sEditCurveAppearanceDialog ---
/*!

*/
class sEditCurveAppearanceDialog : public sEditLineAppearanceDialog
{
  Q_OBJECT
  public:
    explicit                  sEditCurveAppearanceDialog
                                    (psCurveAppearance  appearance
                                    ,sColorSet          color_set
                                    ,sColor             background
                                    ,QWidget *          parent = NULL);

  protected slots:
    virtual void              Refresh ();

  private slots:
    void                      OnTogleFilled (bool  checked);
    void                      OnTogleContoured (bool  checked);

  protected: //fields
    psCurveAppearance         Appearance;
    QCheckBox *               LineCheckBox;
    QCheckBox *               PointsCheckBox;
    QCheckBox *               FilledCheckBox;
    QCheckBox *               ContouredCheckBox;
//    QButtonGroup *            PointModeSelect;
//    QRadioButton *            FilledMode;
//    QRadioButton *            ContouredMode;
//    QRadioButton *            FilledAndContouredMode;
    QLabel *                  PointSizeLabel;
    QSpinBox *                PointSizeField;
    QLabel *                  ContourThicknessLabel1;
    QLabel *                  ContourThicknessLabel2;
    QSpinBox *                ContourThicknessField;
    QLabel *                  PointShapeLabel;
    QComboBox *               PointShapeField;
    QCheckBox *               FillSameColorCheckBox;
    QCheckBox *               CntrSameColorCheckBox;
    psColorSetSelect          FillingColors;
    psColorSetSelect          ContourColors;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
