////////////////////////////////////////////////////////////////////////////////
/*! @file DBLinkingDialogs.h   Classes sEditDBInfoDialog and sEditDBLinkDialog.
- Part of RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Uses  Qt v.5  - http://qt.io/
- Copyright(C) 2013-2017, Viktor E. Bursian, St.Petersburg, Russia.
                          VBursian@gmail.com
*///////////////////////////////////////////////////////////////////////////////
#ifndef DBLinkingDialogs_H
#define DBLinkingDialogs_H
#include "RAGUI_global.h"
#include "DB_Link.h"
#include <QDialog>
#include <QComboBox>
#include <QLineEdit>
#include <QSpinBox>
#include <QLabel>
#include <QCheckBox>
#include <QToolButton>
#include <QRegExp>
#include <QRegExpValidator>
#include <QPushButton>
namespace RA {
//------------------------------------------------------------------------------

//-------------------------------------------------------- sEditDBInfoDialog ---

class RAGUI_EXPORT  sEditDBInfoDialog : public QDialog
{
  Q_OBJECT
  public://types
    struct sLinker { sLinker (); };

  public://static
    static const QRegExp      PrefixRegExpr;
    static const QRegExp      SchemaRegExpr;
    static const QRegExp      DBNameRegExpr;
    static const sLinker      Linker;
    static bool               Modify (psDB_Info  info);

  public:
    explicit                  sEditDBInfoDialog (psDB_Info  info
                                                ,QString    title
                                                ,QWidget *  parent =0);

  protected slots:
    virtual void              ShowHelp ();
    virtual void              OnOK ();
    virtual void              OnTypeChanged (int);
    virtual void              OnRelativeCheckBoxChange ();
    virtual void              ShowURL ();
    virtual void              Browse ();
    virtual void              ShowPassword ();
    virtual void              MaskPassword ();

  protected:
    virtual bool              Validate ();
    virtual void              AcceptChanges ();

  protected://fields
    psDB_Info                 Info;
    sDB_Info                  TempInfo;

    QLabel *                  URLLabel;
    QLabel *                  URLField;
    QComboBox *               TypeField;
    QLabel *                  HostLabel;
    QLineEdit *               HostField;
    QLabel *                  PortLabel;
    QSpinBox *                PortField;
    QLabel *                  LoginLabel;
    QLineEdit *               LoginField;
    QLabel *                  PasswordLabel;
    QLineEdit *               PasswordField;
    QToolButton *             ShowPasswordButton;
    QLabel *                  DBNameLabel;
    QLineEdit *               DBNameField;
    QLabel *                  SchemaLabel;
    QLineEdit *               SchemaField;
    QLabel *                  PrefixLabel;
    QLineEdit *               PrefixField;
    QPushButton *             BrowseButton;
    QCheckBox *               RelativeCheckBox;
    QLabel *                  CurrentDirField;
    QRegExpValidator *        DBNameValidator;
    QLabel *                  OptionsLabel;
    QLineEdit *               OptionsField;
};

//-------------------------------------------------------- sEditDBLinkDialog ---

class RAGUI_EXPORT  sEditDBLinkDialog : public sEditDBInfoDialog
{
  Q_OBJECT
  public:
    explicit                  sEditDBLinkDialog (psDB_Link  link
                                                ,QString    title
                                                ,QWidget *  parent =0);
  private://fields
    psDB_Link                 Link;
    QLabel *                  IdDisplay;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
