////////////////////////////////////////////////////////////////////////////////
/*! @file DeviceStates.h   Basic classes for device state machine definition.
- Part of RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2010-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef DeviceStates_H
#define DeviceStates_H
#include "Device.h"
#include "RAGUI_global.h"
#include "PhysValue.h"
#include <QStateMachine>
#include <QFinalState>
#include <QSignalTransition>
#include <QTimer>
#include <QDebug>
#include <typeinfo>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sCommand)
//template<class data_type> class  sDirective
ANNOUNCE_CLASS(sState)
ANNOUNCE_CLASS(sParallelStateGroup)
ANNOUNCE_CLASS(sDeviceStateMachine)

//----------------------------------------------------------------- sCommand ---
/*! 
*/
class  RAGUI_EXPORT  sCommand
{
  public:
                              sCommand (sString  name_for_debugging = sString())
                                  :Id(QEvent::Type(QEvent::registerEventType()))
                                  ,NameForDebugging(name_for_debugging)
                                {}
    const QEvent::Type        Id;
    const sString             NameForDebugging;
  private:
                              sCommand (rcsCommand)
                                  :Id(cDummyCommandEventType)
                                {}
    rsCommand                 operator = (rcsCommand)
                                { return *this; }
                              sCommand (QEvent::Type  I)
                                  :Id(I)
                                {}
  private://static const
    static const QEvent::Type cDummyCommandEventType;
};


//--------------------------------------------------------------- sDirective ---
/*! 
*/
template<class data_type>
class  sDirective : public sCommand
{
  public:
                              sDirective (sString name_for_debugging =sString())
                                  :sCommand(name_for_debugging+"<...>")
//                                  ,DataTypeName(typeid(data_type).name())
                                {
//                                  qDebug()<<"sDirective <"<<DataTypeName<<">";
                                }
//    const QString             DataTypeName;
};

//------------------------------------------------------------------- sState ---
/*! A state of a sDeviceStateMachine instance.
*/
class RAGUI_EXPORT  sState : public QState
{
  Q_OBJECT
  public:
                              sState (QState *  parent_state
                                     ,int       timeout_in_ms
                                     ,QString   object_name = QString())
                                  :QState(ExclusiveStates,parent_state)
                                  ,TheFinalState(new QFinalState(this))
                                  ,TheOnFinishTransition(NULL)
                                { Initiate(timeout_in_ms,object_name); }
                              sState (QState *  parent_state
                                     ,QString   object_name = QString())
                                  :QState(ExclusiveStates,parent_state)
                                  ,TheFinalState(new QFinalState(this))
                                  ,TheOnFinishTransition(NULL)
                                { Initiate(-1,object_name); }
                              sState (QState *    parent_state
                                     ,sPhysValue  timeout
                                     ,QString     object_name = QString())
                                  :QState(ExclusiveStates,parent_state)
                                  ,TheFinalState(new QFinalState(this))
                                  ,TheOnFinishTransition(NULL)
                                { 
                                  Initiate(Round((timeout/sPhysValue(1,"ms"))
                                                 .ToReal())
                                          ,object_name); 
                                }
//    QFinalState *             Fin () const 
    QAbstractState *          Fin () const 
                                //!< returns the pointer to the final substate
                                { return TheFinalState; }
    void                      SetTimeout (int  timeout_in_ms);
    void                      SetTimeout (sPhysValue  timeout);
  
  public://state machine structure definition  
    psState                   SetAsInitial ();
    psState                   OnEnterCall (QObject *  object ,literal  slot);                                
    psState                   OnEnterCall (literal  slot)
                                { return OnEnterCall(DeviceObject(),slot); }                                
    psState                   OnFinishCall (QObject *  object ,literal  slot);                                
    psState                   OnFinishCall (literal  slot)
                                { return OnFinishCall(DeviceObject(),slot); }                                
    psState                   OnExitCall (QObject *  object ,literal  slot);                                
    psState                   OnExitCall (literal  slot)
                                { return OnExitCall(DeviceObject(),slot); }                                
    psState                   AddTransition (const QObject *   sender
                                            ,const char *      signal
                                            ,QAbstractState *  to_state);
    psState                   AddTransition (const char *      signal
                                            ,QAbstractState *  to_state)
                                {
                                  return AddTransition(DeviceObject(),signal
                                                      ,to_state);
                                }
    psState                   AddTransition (QAbstractState *  to_state);
    psState                   RemoveOnFinishTransition ();
    template<class data_type>
    psState                   AddTransition (literal           signal_with_data
                                            ,QAbstractState *  to_state
                                            ,data_type         password)
                                {
                                  sGuardedSignalTransition<data_type> *   T
                                        (new sGuardedSignalTransition<data_type>
                                              (DeviceObject(),signal_with_data
                                              ,password));
                                  T->setTargetState(to_state);
                                  addTransition(T);
                                  return this;
                                }
    psState                   AddTransition (const char *     signal
                                            ,const char *     slot_or_signal
                                            ,QAbstractState * to_state);
    template<class data_type>
    psState                   AddTransition (literal    signal_with_data
                                            ,literal    slot_or_signal_with_data
                                            ,QAbstractState * to_state)
                                {
                                  sSignalTransition<data_type> *     T
                                      (new sSignalTransition<data_type>
                                              (DeviceObject(),signal_with_data
                                              ,slot_or_signal_with_data));
                                  T->setTargetState(to_state);
                                  addTransition(T);
                                  return this;
                                }
    psState                   AddTransition (rcsCommand        command
                                            ,QAbstractState *  to_state);
    psState                   AddTransition (rcsCommand       command
                                            ,literal          slot_or_signal
                                            ,QAbstractState * to_state);
    template<class data_type>
    psState                   AddTransition (const sDirective
                                                   <data_type> &  directive
                                            ,QAbstractState *     to_state)
                                {
                                  sDirectiveTransition<data_type> *   T
                                        (new sDirectiveTransition<data_type>
                                                      (HostDriver(),directive));
                                  T->setTargetState(to_state);
                                  addTransition(T);
                                  return this;
                                }
    template<class data_type>
    psState                   AddTransition (const sDirective
                                                   <data_type> &  directive
                                            ,literal    slot_or_signal_with_data
                                            ,QAbstractState *     to_state)
                                {
                                  sDirectiveTransition<data_type> *   T
                                        (new sDirectiveTransition<data_type>
                                              (HostDriver(),directive
                                              ,slot_or_signal_with_data));
                                  T->setTargetState(to_state);
                                  addTransition(T);
                                  return this;
                                }
    psState                   AddReaction (const char *  signal
                                          ,const char *  slot_or_signal);
    template<class data_type>
    psState                   AddReaction (literal  signal_with_data
                                          ,literal  slot_or_signal_with_data)
                                {
                                  addTransition
                                      (new sSignalTransition<data_type>
                                              (DeviceObject(),signal_with_data
                                              ,slot_or_signal_with_data));
                                  return this;
                                }
    template<class data_type>
    psState                   AddReaction (literal       signal_with_data
                                          ,data_type     password
                                          ,literal       slot_or_signal)
                                {
                                  addTransition
                                      (new sGuardedSignalTransition<data_type>
                                              (DeviceObject(),signal_with_data
                                              ,password
                                              ,slot_or_signal)   );
                                  return this;
                                }
    psState                   AddReaction (rcsCommand    command
                                          ,literal       slot_or_signal);
    template<class data_type>
    psState                   AddReaction (const sDirective<data_type> &
                                                     directive
                                          ,literal   slot_or_signal_with_data)
                                {
                                  sDirectiveTransition<data_type> *   T
                                        (new sDirectiveTransition<data_type>
                                                (HostDriver(),directive
                                                ,slot_or_signal_with_data));
                                  addTransition(T);
                                  return this;
                                }
    
  public:  
    void                      addTransition (QAbstractTransition *  transition)
                                { QState::addTransition(transition); }
//    QSignalTransition *       addTransition(const QObject *   sender
//                                           ,const char *      signal
//                                           ,QAbstractState *  target)
//                                { 
//                                  return QState::addTransition
//                                                      (sender,signal,target); 
//                                }
//    QSignalTransition *       addTransition (QAbstractState *  target)
//                                { 
//                                  return QState::addTransition
//                                            (this,SIGNAL(finished()),target); 
//                                }

  signals:
    void                      Timeout ();
  
  protected: //reimplementing inherited
    virtual void              onEntry (QEvent *);
    virtual void              onExit (QEvent *);
  
  protected slots:
    virtual void              OnFinish ();

  private: //types
  public: //types
    //............................................. sState::sCommandEvent ...
    ANNOUNCE_CLASS(sCommandEvent)
    class  RAGUI_EXPORT  sCommandEvent : public QEvent
    {
      public:
                                  sCommandEvent
                                        (psDevice                sender
                                        ,rcsCommand              command
                                        ,sDevice::eControlChange control_change
                                                        =sDevice::cLeaveControl)
                                      :QEvent(command.Id)
                                      ,Sender(sender)
                                      ,ControlChange(control_change)
                                      ,NameForDebugging
                                          (command.NameForDebugging)
                                    {}
        const psDevice            Sender;
        const sDevice::eControlChange
                                  ControlChange;
        const sString             NameForDebugging;
    };
    //............................... sState::sDirectiveEvent <data_type> ...
    template <class data_type>
    class  sDirectiveEvent : public sCommandEvent
    {
      public:
                                  sDirectiveEvent
                                        (psDevice                 sender
                                        ,const sDirective<data_type> &
                                                                  directive
                                        ,const data_type &        data
                                        ,sDevice::eControlChange  control_change
                                                        =sDevice::cLeaveControl)
                                      :sCommandEvent(sender,directive
                                                    ,control_change)
                                      ,Data(data)
                                    {}
        const data_type           Data;
    };
    //........................................ sState::sCommandTransition ...
    ANNOUNCE_CLASS(sCommandTransition)
    class  RAGUI_EXPORT  sCommandTransition : public QAbstractTransition
    {
      public:
                                  sCommandTransition
                                        (psDriver             host_driver
                                        ,rcsCommand           command     )
                                      :HostDriver(host_driver)
                                      ,Identifier(command.Id)
                                      ,IvokesSlotOrSignal(false)
                                      ,WithData(false)
                                    {}
                                  sCommandTransition
                                        (psDriver             host_driver
                                        ,rcsCommand           command
                                        ,const char *         slot_or_signal)
                                      :HostDriver(host_driver)
                                      ,Identifier(command.Id)
                                      ,IvokesSlotOrSignal(true)
                                      ,WithData(false)
                                      ,SlotOrSignal(sState::SoS_Name
                                                               (slot_or_signal))
                                      ,DataType(sState::SoS_Type
                                                               (slot_or_signal))
                                    {}
      protected:
        virtual bool              eventTest (QEvent *);
        virtual void              onTransition (QEvent *);
      protected: //fields
        psDriver                  HostDriver;
        const QEvent::Type        Identifier;
        const bool                IvokesSlotOrSignal;
        bool                      WithData;
        const QString             SlotOrSignal;
        const QString             DataType;
    };
  private: //types
    //.......................... sState::sDirectiveTransition <data_type> ...
    template <class data_type>
    class  sDirectiveTransition : public sCommandTransition
    {
      public:
                                  sDirectiveTransition
                                        (psDriver             host_driver
                                        ,const sDirective<data_type> &
                                                                    directive)
                                      :sCommandTransition(host_driver,directive)
                                    {}
                                  sDirectiveTransition
                                        (psDriver             host_driver
                                        ,const sDirective<data_type> &
                                                                    directive
                                        ,const char *         signal_or_slot)
                                      :sCommandTransition(host_driver,directive
                                                         ,signal_or_slot)
                                    {
                                      WithData = true;
// ненадёжная проверка, но непонятно как проверить, что,
// например, N2RA10sPhysValueE означает sPhysValue, а не sPhysValue*
//                                      if( ! QString(typeid(data_type).name())
//                                                       .contains(DataType) )
//                                        qDebug()<<"Wrong affiliation of signal"
//                                              <<"or slot"<<SlotOrSignal<<"("
//                                              <<DataType<<") with"
//                                              <<"sDirective<"
//                                              <<typeid(data_type).name()<<">";
                                    }
        virtual void              onTransition (QEvent *  event)
                                    {
                                      sDirectiveEvent<data_type> *  E =
                                            static_cast<sDirectiveEvent
                                                                  <data_type>*>
                                                       (event);
                                      QMetaObject::invokeMethod
                                            ((QObject*)HostDriver
                                            ,SlotOrSignal.toLatin1().constData()
                                            ,Qt::DirectConnection
                                            ,QArgument<data_type>
                                                (DataType.toLatin1().constData()
                                                ,E->Data)
                                            );
                                    }
    };
    //.............................. sState::sSignalTransition<data_type> ...
    template <class data_type>
    class  sSignalTransition : public QSignalTransition
    {
      public:
                                  sSignalTransition
                                          (QObject *     parent_object
                                          ,const char *  signal
                                          ,const char *  slot_or_signal)
                                      :QSignalTransition(parent_object,signal)
                                      ,SlotOrSignal(sState::SoS_Name
                                                               (slot_or_signal))
                                      ,DataType(sState::SoS_Type
                                                               (slot_or_signal))
                                    {}
      protected:
        virtual void              onTransition (QEvent *  event)
                                    {
                                      QStateMachine::SignalEvent *   E =
                                                    static_cast<QStateMachine
                                                                ::SignalEvent *>
                                                               (event);
                                      if( E->arguments().first()
                                                     .canConvert<data_type>() ){
                                        QMetaObject::invokeMethod
                                            (E->sender()
                                            ,SlotOrSignal.toLatin1().constData()
                                            ,Qt::DirectConnection
                                            ,QArgument<data_type>
                                                (DataType.toLatin1()
                                                                    .constData()
                                                ,E->arguments().first()
                                                           .value<data_type>()
                                                )
                                            );
                                      }else{
                                        qDebug()<<"Wrong template argument <"
                                              +QString(typeid(data_type).name())
                                              +"> used for slot or signal"
                                              <<SlotOrSignal+"("+DataType+")";
                                      }
                                    }
      protected: //fields
        const QString             SlotOrSignal;
        const QString             DataType;
    };
    //....................... sState::sGuardedSignalTransition<data_type> ...
    template <class data_type>
    class  sGuardedSignalTransition : public sSignalTransition<data_type>
    {
      public:
                                  sGuardedSignalTransition
                                          (QObject *     parent_object
                                          ,const char *  signal
                                          ,data_type     password
                                          ,const char *  slot_or_signal = NULL)
                                      :sSignalTransition<data_type>
                                                        (parent_object,signal
                                                        ,slot_or_signal )
                                      ,Password(password)
                                    {}
      protected:
        virtual bool              eventTest (QEvent *  event)
                                    {
                                      if( QSignalTransition::eventTest(event) ){
                                        QStateMachine::SignalEvent *   E =
                                                    static_cast<QStateMachine
                                                                ::SignalEvent *>
                                                               (event);
                                        if( E->arguments().first()
                                                     .canConvert<data_type>() ){
                                          return ( E->arguments().first()
                                                           .value<data_type>()
                                                 == Password );
                                        }else{
                                          qDebug()<<"Wrong template argument <"
                                              +QString(typeid(data_type).name())
                                              +"> used for guarded signal "
                                               "transition";
                                        }
                                      }
                                      return false;
                                    }
      protected: //fields
        const data_type           Password;
    };
    //.......................................................................

  private: //static
    static QString            SoS_Name (const char *  signal_or_slot);
    static QString            SoS_Type (const char *  signal_or_slot);
    static QString            SoS_Text (const char *  signal_or_slot);

  private: //private constructor
                              sState (ChildMode  /*=ParallelStates*/
                                     ,QState *   parent_state
                                     ,int        timeout_in_ms = -1 
                                     ,QString    object_name = QString())
                                  :QState(ParallelStates,parent_state)
                                  ,TheFinalState(NULL)
                                  ,TheOnFinishTransition(NULL)
                                { Initiate(timeout_in_ms,object_name); }
    void                      Initiate (int      timeout_in_ms
                                       ,QString  object_name);
    psDriver                  HostDriver ();
    QObject *                 DeviceObject ();                                
    sString                   DeviceIdentifier ();
    void                      DeviceSetInvalid (QString  reason);

  private: //fields
    QFinalState *             TheFinalState;
    int                       TheTimeoutValue;
    QTimer *                  TimeoutClock;
    QSignalTransition *       TheOnFinishTransition;
    
  friend class sParallelStateGroup;
};


//------------------------------------------------------ sParallelStateGroup ---
/*! A composite state of a sDeviceStateMachine instance with its child states
    to be executed in parallel.



*/
class RAGUI_EXPORT  sParallelStateGroup : public sState
{
  Q_OBJECT
  public:
                              sParallelStateGroup (QState *  parent_state
                                                  ,int       timeout_in_ms
                                                  ,QString   object_name
                                                                  = QString())
                                  :sState(ParallelStates,parent_state
                                         ,timeout_in_ms,object_name)
                                {}
                              sParallelStateGroup (QState *  parent_state
                                                  ,QString   object_name 
                                                                   = QString()
                                                  ,int       timeout_in_ms = -1)
                                  :sState(ParallelStates,parent_state
                                         ,timeout_in_ms,object_name)
                                {}
};


//------------------------------------------------------ sDeviceStateMachine ---
/*! Прототип машины состояний для драйверов и консолей.


@par The state machine
@dotfile "DeviceStates.state_graph"

Legend: 
- thick solid black arrow - unconditional transition (when the state finishes);
- thin solid black arrow - transition on signal or command or directive;
- dotted black circular arrow - reaction on signal or command or directive
  (place the mouse over the line to bring up a hint).
  
@note Complete functionality of the upper-level state machine is defined
      in the sDriver and sConsole classes (a little bit differently).

*/
class RAGUI_EXPORT  sDeviceStateMachine : public QStateMachine
{
  Q_OBJECT
  public: //types
    enum                      eState
                                {cWarming
                                ,cOperating
                                ,cCooling
                                ,cFrozen
                                };

//  public://commands
//    static csCommand          cResetCommand;

  public:
    explicit                  sDeviceStateMachine (psDevice  host_device);
    psDevice                  Device () const
                                { return TheHostDevice; }

  signals: //command-signals
    void                      WarmUp ();
                                  /*!< Establishes the connection, performs all
                                  nessesary initiations and checks readiness.*/
    void                      CoolDown ();
                                  /*!< Carefully switches the device off
                                  (may take time).*/

  signals: //outgoing
    void                      Warmed ();
    void                      Cooled ();
    void                      NewUpperLevelState (sDeviceStateMachine::eState);
    void                      NewUpperLevelState (QString  new_state_name);

  signals: //for internal use
//    void                      Quit ();

  public: //upper level of the state machine
//    psState                   Universe;
    psState                   Warming;
    psState                   Operating;
    psState                   Cooling;
    psState                   Frozen;
    QFinalState *             FinalState;
    QSignalTransition *       EmitWarmedAndReturn;

  private slots:
    void                      Into_Warming ();
    void                      Into_Operating ();
    void                      Into_Cooling ();
    void                      Into_Frozen ();

  private:
    void                      OrderWarmUp ()
                                {
                                  emit WarmUp();
                                }
  
  private: //fields
    psDevice                  TheHostDevice;

  friend class sDriverLink;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
