////////////////////////////////////////////////////////////////////////////////
/*! @file EditNoteDialog.cpp   Редактирование текстового атрибута.
- Part of RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "EditNoteDialog.h"
#include "StringsAndQStrings.h"
namespace RA {
//------------------------------------------------------------------------------

sEditNoteDialog::~sEditNoteDialog ()
{
}


sEditNoteDialog::sEditNoteDialog (QWidget *  parent)
  :sAttrDlg(parent)
  ,PreventCycling(false)
{
  lblValue = new QLabel(this);
  lblValue->setObjectName(QString::fromUtf8("lblValue"));
  lblValue->setText(tr("Source HTML:"));
  ValueSpace->addWidget(lblValue,0,0,1,1/*,Qt::Alignment*/);

  fldValuePlain = new QTextEdit(this);
  fldValuePlain->setObjectName(QString::fromUtf8("fldValuePlain"));
  ValueSpace->addWidget(fldValuePlain,1,0,1,2/*,Qt::Alignment*/);

  lblValueHTML = new QLabel(this);
  lblValueHTML->setObjectName(QString::fromUtf8("lblValueHTML"));
  lblValueHTML->setText(tr("Formatted\ntext:"));
  ValueSpace->addWidget(lblValueHTML,2,0,1,1/*,Qt::Alignment*/);

  fldValueHTML = new QTextEdit(this);
  fldValueHTML->setObjectName(QString::fromUtf8("fldValueHTML"));
  ValueSpace->addWidget(fldValueHTML,2,1,1,1/*,Qt::Alignment*/);

  fldValueHTML->setEnabled(false);

  connect(fldValuePlain,SIGNAL(textChanged()),this,SLOT(UpdateHTML()));
  connect(fldValueHTML,SIGNAL(textChanged()),this,SLOT(UpdatePlain()));
};


void  sEditNoteDialog::SetValue (rcsString  value)
{
  fldValuePlain->setPlainText(ToQString(value));
  PreventCycling=false;
  fldValuePlain->setFocus();
};


bool  sEditNoteDialog::Validate ()
{
  TheValue=FromQString(fldValuePlain->toPlainText());
  return sAttrDlg::Validate();
};


void  sEditNoteDialog::UpdatePlain ()
{
  if( ! PreventCycling ){
    TheValue=FromQString(fldValueHTML->toHtml());
    PreventCycling=true;
    fldValuePlain->setPlainText(ToQString(TheValue));
    PreventCycling=false;
  };
};


void  sEditNoteDialog::UpdateHTML ()
{
  if( ! PreventCycling ){
    TheValue=FromQString(fldValuePlain->toPlainText());
    PreventCycling=true;
    fldValueHTML->setHtml(ToQString(TheValue));
    PreventCycling=false;
  };
};

//------------------------------------------------------------------------------
}; //namespace RA


