/*! @file ExportDataDialog.h   Class sExportDataDialog.
- Part of RAGUI - Research Assistant Graphical User Interface.
- Uses  Qt v.5  -  http://qt.io/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2017, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "ExportDataDialog.h"
#include "Globals.h"
#include "Record.h"
#include "Files.h"
#include "StringsAndQStrings.h"
#include "attr.h"
#include "SymbolsGUI.h"
#include <QLabel>
#include <QInputDialog>
#include <QHBoxLayout>
#include <QGridLayout>
#include <QTextStream>
#include <QMessageBox>
//#include <QDebug>
namespace RA {
//------------------------------------------------------------------------------
//-------------------------------------------------------- sExportDataDialog ---

sExportDataDialog::sExportDataDialog (sString         attr_name
                                     ,sNodePtr        node
                                     ,QWidget *       parent)
    :QFileDialog(parent)
    ,Node(node)
    ,XHeader("x")
    ,YHeader("y")
{
  if( ! attr_name.Empty() )
    SuggestedFileName = attr_name + "=";
  AttrNameAndNodeText = SuggestedFileName + Node->Text(Plain,Verbose);
  ExpressionNode = Node;
  if( ExpressionNode.IsCorrect() ){
    SuggestedFileName += "(";
    SuggestedFileName += ExpressionNode->Docket();
    SuggestedFileName += ")";
    DataFunction = dynamic_cast<psDataFunction>(ExpressionNode->Result());
  }else{
    sPtr<sRecord>               R(Node);
    if( R.IsCorrect() ){
      SuggestedFileName = R->FullIdentifier().Tail(1);
      YHeader = SuggestedFileName;
      SuggestedFileName = SuggestedFileName.Translate('.','_');
    }else{
      SuggestedFileName += Node->Text(Plain,Casual);
    }
    DataFunction = dynamic_cast<psDataFunction>(Node.operator ->());
  }
  if( DataFunction ){
    XUnits = XNativeUnits = DataFunction->Multiplier().X.Units();
    YUnits = YNativeUnits = DataFunction->Multiplier().Y.Units();
  }
  setOption(QFileDialog::DontUseNativeDialog);
  setWindowTitle(tr("Export curve"));
  setFileMode(QFileDialog::AnyFile);
  setAcceptMode(QFileDialog::AcceptSave);
  setLabelText(QFileDialog::FileType,(tr("Show files of type")));
  Filters <<tr("CSV(comma separated values) files (*.csv)")
          <<tr("Data files (*.dat)")
          <<tr("Text files (*.txt)")
//          <<tr("Related files (*.csv *.dat *.txt)")
          <<tr("All files (*)");
  setNameFilters(Filters);
  setNameFilterDetailsVisible(false);

  XUnitsField = new sTextField(tr("Abscissa units"),XUnits.Text(),this);
  YUnitsField = new sTextField(tr("Ordinate units"),YUnits.Text(),this);
  DelimiterField = new QComboBox;
  DelimiterField->addItem(tr("comma"),QString(","));
  DelimiterField->addItem(tr("tab"),QString("\t"));
  DelimiterField->addItem(tr("space"),QString(" "));
  InsertHeaderField = new QCheckBox(tr("insert header"));
  InsertHeaderField->setChecked(true);
  XHeaderField = new sTextField(XHeader,this);
  YHeaderField = new sTextField(YHeader,this);
  InsertUnitsField = new QCheckBox(tr("with units"));
  InsertUnitsField->setChecked(true);
  AppendAttrsField = new QCheckBox(tr("append attributes"));
  AppendAttrsField->setChecked(true);
  AttrsAsideField = new QComboBox;
  AttrsAsideField->addItem(tr("at bottom"),QVariant(false));
  AttrsAsideField->addItem(tr("aside")    ,QVariant(true));
  AppendSubAttrsField = new QCheckBox(tr("and their attrs"));
  AppendSubAttrsField->setChecked(true);
  AppendSubSubAttrsField = new QCheckBox(tr("and theirs"));
  AppendSubSubAttrsField->setChecked(false);
  EOLField = new QComboBox;
  EOLField->addItem(tr("LF (Unix,MacOS)"),QString("\n"));
  EOLField->addItem(tr("CR+LF (mustdie)"),QString("\r\n"));

  QGridLayout *               TopLayout(dynamic_cast<QGridLayout*>(layout()));
  TopLayout->addWidget(new QLabel("   "),0,TopLayout->columnCount(),2,1);
  SideLayout = new QVBoxLayout;
  TopLayout->addLayout(SideLayout,0,TopLayout->columnCount(),2,1);
  SideLayout->addWidget(XUnitsField);
  SideLayout->addWidget(YUnitsField);
  QHBoxLayout*                L = new QHBoxLayout;
  SideLayout->addLayout(L);
  L->addWidget(new QLabel(tr("Delimiter:")));
  L->addWidget(DelimiterField);
  SideLayout->addWidget(InsertHeaderField);
  L = new QHBoxLayout;
  SideLayout->addLayout(L);
  L->addWidget(new QLabel("    "));
  L->addWidget(XHeaderField);
  L->addWidget(YHeaderField);
  L = new QHBoxLayout;
  SideLayout->addLayout(L);
  L->addWidget(new QLabel("    "));
  L->addWidget(InsertUnitsField,1);
  SideLayout->addWidget(AppendAttrsField);
  L = new QHBoxLayout;
  SideLayout->addLayout(L);
  L->addWidget(new QLabel("    "));
  L->addWidget(AttrsAsideField,1);
  L = new QHBoxLayout;
  SideLayout->addLayout(L);
  L->addWidget(new QLabel("    "));
  L->addWidget(AppendSubAttrsField,1);
  L = new QHBoxLayout;
  SideLayout->addLayout(L);
  L->addWidget(new QLabel("          "));
  L->addWidget(AppendSubSubAttrsField,1);
  L = new QHBoxLayout;
  SideLayout->addLayout(L);
  L->addWidget(new QLabel(tr("EOL:")));
  L->addWidget(EOLField,1);

  SideLayout->addStretch(2);

  resize(width()+300,height());

  //две акции, не распознать, которая что
  {
//    qDebug() << "Actions";
//    QAction * A;
//    foreach (A, actions()) {
//      qDebug() << A->text() << A->isEnabled() << A->toolTip() << A->menuRole() << A->objectName();
//    }
//    qDebug() << "------------";
  }

  //Huck the Qt!: две кнопки, отличаются текстом и тем,
  //что одна disabled, потому что поле имени файла пустое
  {
//    qDebug() << "Buttons";
    QPushButton * B;
    foreach (B, findChildren<QPushButton *>()) {
//      qDebug() << B->objectName() << B->isEnabled() << B->text() << B->actions().size() << B->autoDefault();
      if( ! B->isEnabled() ){
        SaveButton = B;
        break;
      }
    }
//    qDebug() << "------------";
  }
  //Huck the Qt!: у этого поля хотя бы задано objectName()
  FileNameField = findChild<QLineEdit *>("fileNameEdit");

  disconnect(this ,SIGNAL(accepted()) ,this ,0 );
  connect(this ,SIGNAL(accepted()) ,this ,SLOT(Export()) );
  connect(XUnitsField ,SIGNAL(Changed())
         ,this        ,SLOT(OnUnitsChanged()) );
  connect(YUnitsField ,SIGNAL(Changed())
         ,this        ,SLOT(OnUnitsChanged()) );
  connect(DelimiterField ,SIGNAL(currentIndexChanged(int))
         ,this           ,SLOT(OnDelimiterChanged()) );
//  connect(this ,SIGNAL(filterSelected(QString))
//         ,this ,SLOT(OnFilterChange()) );
  connect(this ,SIGNAL(fileSelected(QString)) ,this ,SLOT(CheckFileName()) );
  connect(InsertHeaderField ,SIGNAL(toggled(bool))
         ,XHeaderField      ,SLOT(setEnabled(bool)) );
  connect(InsertHeaderField ,SIGNAL(toggled(bool))
         ,YHeaderField      ,SLOT(setEnabled(bool)) );
  connect(InsertHeaderField ,SIGNAL(toggled(bool))
         ,InsertUnitsField ,SLOT(setEnabled(bool)) );
  connect(AppendAttrsField ,SIGNAL(toggled(bool))
         ,AttrsAsideField  ,SLOT(setEnabled(bool)) );
  connect(AppendAttrsField    ,SIGNAL(toggled(bool))
         ,AppendSubAttrsField ,SLOT(setEnabled(bool)) );
  connect(AppendAttrsField       ,SIGNAL(toggled(bool))
         ,AppendSubSubAttrsField ,SLOT(setEnabled(bool)) );
  connect(AppendSubAttrsField    ,SIGNAL(toggled(bool))
         ,AppendSubSubAttrsField ,SLOT(setEnabled(bool)) );

  ValidateFileName(SuggestedFileName);
  selectFile(ToQString(SuggestedFileName+".csv"));
  DelimiterField->setCurrentIndex(0);
  OnDelimiterChanged();

  if( RAGUI::CSVExportSettings.IsCorrect() ){
    sString                   S,U;
    bool                      B;
    integer                   I;
    if( get_from<sTextNode>(RAGUI::CSVExportSettings>>"Dir",S) )
      setDirectory(ToQString(S));
    if( get_from<sIntegerNode>(RAGUI::CSVExportSettings>>"Delimiter",I) )
      DelimiterField->setCurrentIndex(I);
    if( get_from<sIntegerNode>(RAGUI::CSVExportSettings>>"EOL",I) )
      EOLField->setCurrentIndex(I);
    if( get_from<sBooleanNode>(RAGUI::CSVExportSettings>>"InsertUnits",B) )
      InsertUnitsField->setChecked(B);
    if( get_from<sBooleanNode>(RAGUI::CSVExportSettings>>"InsertHeader",B) )
      InsertHeaderField->setChecked(B);
    if( get_from<sBooleanNode>(RAGUI::CSVExportSettings>>"AppendSubSubAttrs",B))
      AppendSubSubAttrsField->setChecked(B);
    if( get_from<sBooleanNode>(RAGUI::CSVExportSettings>>"AppendSubAttrs",B) )
      AppendSubAttrsField->setChecked(B);
    if( get_from<sBooleanNode>(RAGUI::CSVExportSettings>>"AppendAttrs",B) )
      AppendAttrsField->setChecked(B);
    if( get_from<sIntegerNode>(RAGUI::CSVExportSettings>>"AttrsAside",I) )
      AttrsAsideField->setCurrentIndex(I);
    U = XNativeUnits.Text();
    if( U.Empty() )  U="___N_O_N_E___";
    if( get_from<sTextNode>(RAGUI::CSVExportSettings>>"XUnitsSubstitutions"
                                                       >>U , S) ){
//      qDebug()<<"subst "<< ToQString(XNativeUnits.Text()) << ToQString(S);
      XUnitsField->SetText(S);
    }
    U = YNativeUnits.Text();
    if( U.Empty() )  U="___N_O_N_E___";
    if( get_from<sTextNode>(RAGUI::CSVExportSettings>>"YUnitsSubstitutions"
                                                       >>U , S) ){
//      qDebug()<<"subst "<< ToQString(YNativeUnits.Text()) << ToQString(S);
      YUnitsField->SetText(S);
    }
  }

  if( FileNameField ){
    connect(FileNameField ,SIGNAL(textEdited(QString))
           ,this ,SLOT(CheckFileName()) ,Qt::QueuedConnection);
  }
}


void  sExportDataDialog::Export ()
{
//  qDebug()<<"Export";
  QString                   EOL(EOLField->currentData().toString());
  bool                      AttrsAside(AttrsAsideField->currentData().toBool());
  sCurve::psTracer          T = NULL;
  try{
    if( ! selectedFiles().empty() ){
      XHeader = XHeaderField->Text();
      YHeader = YHeaderField->Text();
      XUnits = sUnits(FromQString(XUnitsField->QtText().simplified()));
      YUnits = sUnits(FromQString(YUnitsField->QtText().simplified()));
      T = DataFunction->GetCurveTracer(sScales(sScale(sPhysValue(XUnits))
                                              ,sScale(sPhysValue(YUnits))) );
      QString                     FileName(selectedFiles().at(0));
      QFile                       File(FileName);
      if( ! File.open(QFile::WriteOnly | QFile::Text) ){
        QMessageBox::warning(NULL,windowTitle()
                            ,tr("Cannot write file %1:\n%2.")
                            .arg(FileName).arg(File.errorString()));
        return;
      }
      QTextStream                 Out(&File);
      bool                        First = true;
      if( InsertHeaderField->isChecked() ){
        Out << (First ? (First=false,"") : EOL)
            << XHeader << Delimiter << YHeader;
        if( AttrsAside && AppendAttrsField->isChecked() ){
          Out << Delimiter << "attributes";
        }
        if( InsertUnitsField->isChecked() ){
          Out << (First ? (First=false,"") : EOL)
              << ToQString(XUnits.Text()) << Delimiter
              << ToQString(YUnits.Text());
          if( AttrsAside && AppendAttrsField->isChecked() ){
            Out << Delimiter << "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@";
          }
        }
      }
      if( ! AttrsAside ){
        while( T->NextPoint() ){
          Out << (First ? (First=false,"") : EOL)
              << (double)(T->Point.X) << Delimiter << (double)(T->Point.Y);
        }
      }
      if( AppendAttrsField->isChecked() ){
        QString            Indent0(QString(Symbol::Ellipsis).repeated(2));
        QString            Indent1(QString(Symbol::Ellipsis).repeated(4));
        QString            Indent2(QString(Symbol::Ellipsis).repeated(6));
        sString            AttrText;
        Out << (First ? (First=false,"") : EOL);
        if( AttrsAside ){
          if( T->NextPoint() ){
            Out << (double)(T->Point.X) << Delimiter
                << (double)(T->Point.Y) << Delimiter;
          }else{
            Out << "-" << Delimiter << "-" << Delimiter;
          }
        }
        Out << AttrNameAndNodeText;
        for_each_attribute( A , Node ){
          AttrText = A->Name().Text();
          if( ! AttrText.Empty() )  AttrText += "=";
          AttrText += A->Value()->Text(Plain);
          Out << (First ? (First=false,"") : EOL);
          if( AttrsAside ){
            if( T->NextPoint() ){
              Out << (double)(T->Point.X) << Delimiter
                  << (double)(T->Point.Y) << Delimiter;
            }else{
              Out << "-" << Delimiter << "-" << Delimiter;
            }
          }
          Out << Indent0 << ToQString(AttrText);
          if( AppendSubAttrsField->isChecked() ){
            for_each_attribute( SA , A->Value() ){
              AttrText = SA->Name().Text();
              if( ! AttrText.Empty() )  AttrText += "=";
              AttrText += SA->Value()->Text(Plain);
              Out << (First ? (First=false,"") : EOL);
              if( AttrsAside ){
                if( T->NextPoint() ){
                  Out << (double)(T->Point.X) << Delimiter
                      << (double)(T->Point.Y) << Delimiter;
                }else{
                  Out << "-" << Delimiter << "-" << Delimiter;
                }
              }
              Out << Indent1 << ToQString(AttrText);
              if( AppendSubSubAttrsField->isChecked() ){
                for_each_attribute( SSA , SA->Value() ){
                  AttrText = SSA->Name().Text();
                  if( ! AttrText.Empty() )  AttrText += "=";
                  AttrText += SSA->Value()->Text(Plain);
                  Out << (First ? (First=false,"") : EOL);
                  if( AttrsAside ){
                    if( T->NextPoint() ){
                      Out << (double)(T->Point.X) << Delimiter
                          << (double)(T->Point.Y) << Delimiter;
                    }else{
                      Out << "-" << Delimiter << "-" << Delimiter;
                    }
                  }
                  Out << Indent2 << ToQString(AttrText);
                }end_attr_loop()
              }
            }end_attr_loop()
          }
        }end_attr_loop()
      }
      if( AttrsAside ){
        while( T->NextPoint() ){
          Out << (First ? (First=false,"") : EOL)
              << (double)(T->Point.X) << Delimiter << (double)(T->Point.Y);
        }
      }
      if( T ){ delete T; T=NULL; }
      File.close();
      if( RAGUI::CSVExportSettings.IsCorrect() ){
        RAGUI::CSVExportSettings
            <<attr("Dir",FromQString(directory().absolutePath()))
            <<attr("Delimiter",DelimiterField->currentIndex())
            <<attr("EOL",EOLField->currentIndex())
            <<attr("InsertHeader",InsertHeaderField->isChecked())
            <<attr("InsertUnits",InsertUnitsField->isChecked())
            <<attr("AppendAttrs",AppendAttrsField->isChecked())
            <<attr("AttrsAside",AttrsAsideField->currentIndex())
            <<attr("AppendSubAttrs",AppendSubAttrsField->isChecked())
            <<attr("AppendSubSubAttrs",AppendSubSubAttrsField->isChecked())
            ;
        if( XUnits.Text() != XNativeUnits.Text() ){
          sString                   U(XNativeUnits.Text());
          if( U.Empty() )  U="___N_O_N_E___";
          (RAGUI::CSVExportSettings >= "XUnitsSubstitutions")
              << attr(U,XUnits.Text());
        }
        if( YUnits.Text() != YNativeUnits.Text() ){
          sString                   U(YNativeUnits.Text());
          if( U.Empty() )  U="___N_O_N_E___";
          (RAGUI::CSVExportSettings >= "YUnitsSubstitutions")
              << attr(U,YUnits.Text());
        }
      }
//      QFileDialog::done(QDialog::Accepted);
    }
  }catch(...){
    QMessageBox::critical(NULL,windowTitle(),"Error");
    if( T ){ delete T; T=NULL; }
  }
}


void  sExportDataDialog::UpdateSaveButton ()
{
//  qDebug()<<"UpdateSaveButton";
  if( SaveButton ){
    SaveButton->setEnabled(  ! XUnitsField->HasAttentionFlag()
                          && ! YUnitsField->HasAttentionFlag()
                          && ValidFileName);
  }
}


void  sExportDataDialog::OnUnitsChanged ()
{
//  qDebug()<<"OnUnitsChanged";
  try{
    XUnits = sUnits(FromQString(XUnitsField->QtText().simplified()));
    (sPhysValue(XUnits) / DataFunction->Multiplier().X).ToReal();
    XUnitsField->UnsetAttention();
  }catch( rxRAlgebra ){
    XUnitsField->SetAttention();
  }
  try{
    YUnits = sUnits(FromQString(YUnitsField->QtText().simplified()));
    (sPhysValue(YUnits)/DataFunction->Multiplier().Y).ToReal();
    YUnitsField->UnsetAttention();
  }catch( rxRAlgebra ){
    YUnitsField->SetAttention();
  }
  UpdateSaveButton();
}


void  sExportDataDialog::OnDelimiterChanged ()
{
//  qDebug()<<"OnDelimiterChanged";
  switch( DelimiterField->currentIndex() ){
    case 0:
      Delimiter = ",";
//      setDefaultSuffix("csv");
      break;
    case 1:
      Delimiter = "\t";
//      setDefaultSuffix("dat");
      break;
    case 2:
      Delimiter = " ";
//      setDefaultSuffix("txt");
      break;
    default:
      break;
  }
  selectNameFilter(Filters.at(DelimiterField->currentIndex()));
}


//void sExportDataDialog::OnFilterChange()
//{
//  qDebug() << "filter change";
//}


void  sExportDataDialog::CheckFileName ()
{
//  qDebug()<<"CheckFileName";
  QString                     IllegalChars("|><?*:\\\"");
  QString                     FileName(FileNameField->text());
  int                         Pos;
  ValidFileName = true;
  for( int i = 0 ; i < IllegalChars.length() ; ++i ){
    Pos = FileName.indexOf(IllegalChars[i]);
    if( Pos > -1 ){
      FileNameField->setSelection(Pos,1);
      ValidFileName = false;
      break;
    }
  }
  UpdateSaveButton();
}


void  sExportDataDialog::ValidateFileName (rsString  filename)
{
//  qDebug()<<"ValidateFileName";
  filename.Replace("<sub>" ,"_{");
  filename.Replace("</sub>","}");
  filename.Replace("<sup>" ,"^{");
  filename.Replace("</sup>","}");
  filename.Replace("<b>","");
  filename.Replace("</b>","");
  filename.Replace("*","×");
  filename.Replace("/","÷");
  filename = sPath::ValidateName(filename);
}

//------------------------------------------------------------------------------
} //namespace RA
