////////////////////////////////////////////////////////////////////////////////
/*! @file FlexibleFields.h   Input fields with useful features.
- Part of RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5  - http://qt.io/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2010-2017, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef FlexibleFields_H
#define FlexibleFields_H
#include "RAGUI_global.h"
#include "PhysValue.h"
#include "Ranges.h"
#include "StringsAndQStrings.h"
#include <QWidget>
#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QKeyEvent>
#include <QContextMenuEvent>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sFlexibleField)
ANNOUNCE_CLASS(sSpinDigitField)
ANNOUNCE_CLASS(sTextField)

//----------------------------------------------------------- sFlexibleField ---

class RAGUI_EXPORT  sFlexibleField : public QLineEdit
{
  Q_OBJECT
  public:
    explicit                  sFlexibleField (QWidget *  parent = 0);
    bool                      ReadOnly ()
                                { return ReadOnlyFlag; }
  public slots:
    psFlexibleField           SetReadOnly (bool  read_only = true);
    void                      ResizeField ();

  signals:
    void                      EscapeKeyPressed ();
    void                      EnterKeyPressed ();
    void                      GotFocus ();
    void                      LostFocus ();

  protected://inherited
    virtual void              keyPressEvent (QKeyEvent *);
    virtual void              focusInEvent (QFocusEvent *);
    virtual void              focusOutEvent (QFocusEvent *);

  private slots:

  private: //fields
    bool                      ReadOnlyFlag;
    bool                      TextChangedFlag;
};


//---------------------------------------------------------- sSpinDigitField ---

class RAGUI_EXPORT  sSpinDigitField : public sFlexibleField
{
  Q_OBJECT
  public:
    explicit                  sSpinDigitField (QWidget *  parent = 0);

  signals:
    void                      ChangedBySpinning ();

  protected://inherited
    virtual void              keyPressEvent (QKeyEvent *);
    virtual void              contextMenuEvent (QContextMenuEvent *);

  private:
    void                      Spin (QString &  Text
                                   ,int        CurPos
                                   ,bool       upwards);
};


//--------------------------------------------------------------- sTextField ---

class RAGUI_EXPORT  sTextField : public QWidget
{
  Q_OBJECT
  public:
    virtual                   ~sTextField ();
    explicit                  sTextField (QWidget *  parent = 0)
                                  :QWidget(parent)
                                { Construct(); }
    explicit                  sTextField (QString    label
                                         ,QWidget *  parent = 0)
                                  :QWidget(parent),TheLabel(label)
                                { Construct(); }
    explicit                  sTextField (sString    default_text
                                         ,QWidget *  parent = 0)
                                  :QWidget(parent)
                                  ,TheDefaultText(default_text)
                                { Construct(); }
                              sTextField (QString    label
                                         ,sString    default_text
                                         ,QWidget *  parent = 0)
                                  :QWidget(parent),TheLabel(label)
                                  ,TheDefaultText(default_text)
                                { Construct(); }
    QString                   Label () const
                                { return TheLabel; }
    sString                   Text () const
                                { return FromQString(FldValue->text()); }
    QString                   QtText () const
                                { return FldValue->text(); }
    bool                      ReadOnly ()
                                { return FldValue->ReadOnly(); }
    bool                      HasAttentionFlag () const
                                { return TheAttentionFlag; }

  signals:
    void                      LostFocus ();
    void                      Changed ();
    void                      Edited ();

  public slots:
    virtual psTextField       SetLabel (QString);
    virtual psTextField       SetReadOnly (bool);
    virtual psTextField       SetReadOnly ()
                                { return SetReadOnly(true); }
    virtual psTextField       UnsetReadOnly ()
                                { return SetReadOnly(false); }
    virtual psTextField       SetAttention (bool);
    virtual psTextField       SetAttention ()
                                { return SetAttention(true); }
    virtual psTextField       UnsetAttention ()
                                { return SetAttention(false); }
    virtual psTextField       UnsetAttentionOnEdit (bool  unset = true)
                                { UnsetAttentionOnEditFlag=unset; return this; }
    virtual psTextField       SetText (sString);
    virtual psTextField       SetDefaultText (sString  default_text)
                                { TheDefaultText=default_text; return this; }

  private:
    void                      Construct ();

  private: //fields
    bool                      TheAttentionFlag;
    QString                   TheLabel;
    sString                   TheDefaultText;
    bool                      UnsetAttentionOnEditFlag;
    QLabel *                  LblLabel;
    psFlexibleField           FldValue;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
