////////////////////////////////////////////////////////////////////////////////
/*! @file FolderNodeUI.cpp   Функционал навигатора, специфичный для кластеров.
- Part of RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "FolderNodeUI.h"
#include "Navigator.h"
#include "EditDocketDialog.h"
#include "NetExceptions.h"
#include <QMessageBox>
namespace RA {
//------------------------------------------------------------------------------
//------------------------------------------------------------ sFolderNodeUI ---

sFolderNodeUI::sFolderNodeUI ()
{
  sNavigatorViewport::MenuGenerators.push_front(&MenuGenerator);
}


void  sFolderNodeUI::MenuGenerator (psNetViewMenu                context_menu
                                   ,sNet2TreeEvolution::pcsItem  tree_item)
{
  if( tree_item->AttrValue()->TrueType() == sFolderNode::Type() ){
    context_menu->Insert( new sNetViewAction
        (context_menu->parentWidget()
        ,&EditValue  ,/*checkable*/false ,/*checked*/false
        ,sNetViewMenu::tr("Edit the object's docket")
        ,sNetViewMenu::tr("View and edit the object's docket")
        ));
  };
  if( sFolderNode::Type()->CanCast(tree_item->AttrValue().operator ->()) ){
    context_menu->AddNew->addAction( new sNetViewAction
        (context_menu->parentWidget()
        ,&AddNew  ,/*checkable*/false ,/*checked*/false
        ,sNetViewMenu::tr("object")
        ,sNetViewMenu::tr("Add new object")
        ));
  };
  if( tree_item->Parent() ){
    context_menu->InsertNew->addAction( new sNetViewAction
        (context_menu->parentWidget()
        ,&InsertNew  ,/*checkable*/false ,/*checked*/false
        ,sNetViewMenu::tr("object")
        ,sNetViewMenu::tr("Insert new object")
        ));
  };
}


void  sFolderNodeUI::EditValue (sNet2TreeEvolution::pcsItem  tree_item
                               ,sPoint                       /*pos*/)
{
  /*! @todo{UI++} in-place editing */
  sEditDocketDialog           D;
  D.setWindowTitle( sEditDocketDialog::tr("Edit the object's docket") );
  D.SetAttrName(tree_item->AttrName());
  D.SetAttrNameEnabled(false);
  sPtr<sFolderNode>           C(tree_item->AttrValue());
  ASSERT(C.IsCorrect());
  D.SetDocket(C->Docket());
  if( D.exec() ){
    C->SetDocket(D.Docket());
  };
}


void  sFolderNodeUI::AddNew (sNet2TreeEvolution::pcsItem  tree_item
                            ,sPoint                       /*pos*/)
{
  sEditDocketDialog           D;
  D.setWindowTitle( sEditDocketDialog::tr("Add an object") );
  D.SetDocket("");
  while( D.exec() ){
    try{
      tree_item->AttrValue()
            << sAttribute( D.AttrName()
                         , sNodePtr( new sFolderNode(D.Docket()) ) );
      break;
    }catch( rxNetInvalidOp ){
      QMessageBox::critical(NULL ,D.windowTitle() ,D.ErrorMsgOnAddAttr() );
    }
  }
}


void  sFolderNodeUI::InsertNew (sNet2TreeEvolution::pcsItem  tree_item
                               ,sPoint                       /*pos*/)
{
  sEditDocketDialog           D;
  D.setWindowTitle( sEditDocketDialog::tr("Insert an object") );
  D.SetDocket("");
  sPtr<sFolderNode>           NewNode;
  while( D.exec() ){
    try{
      NewNode = sNodePtr( new sFolderNode(D.Docket()) );
      tree_item->Parent()->AttrValue()<<sAttribute(D.AttrName(),NewNode);
      break;
    }catch( rxNetInvalidOp ){
      QMessageBox::critical(NULL ,D.windowTitle() ,D.ErrorMsgOnAddAttr() );
    }
  }
  //! @todo{Insert_attribute} move NewNode
}

//------------------------------------------------------------------------------
} //namespace RA
