////////////////////////////////////////////////////////////////////////////////
/*! @file Globals.h   Global attribute names.
- Part of RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5.0  -  http://qt-project.org/
- Copyright(C) 2013, Viktor E. Bursian, St.Petersburg, Russia.
                     VBursian@gmail.com
*///////////////////////////////////////////////////////////////////////////////
#ifndef Globals_H
#define Globals_H
#include "RAGUI_global.h"
#include "AttrName.h"
namespace RA {
//------------------------------------------------------------------------------

/*!  The names of the attributes of the root of the RANet knowledgebase.

These attributes are necessary for the functionality of RA program.
*/
namespace GlobalAttr {
//------------------------------------------------------------------------------

/*! Points to a folder node, which contains the user preferences
    and the desktop contents.
*/
RAGUI_EXPORT extern const sAttrName   Desktop;

/*! Points to a folder node, which has to be shown,
    when sRAMainWindow::ActnOpenData is activated.
*/
RAGUI_EXPORT extern const sAttrName   Data;

/*! Points to a folder node, which contains information about the plugins.
*/
RAGUI_EXPORT extern const sAttrName   Plugins;

/*! Points to a folder node, which contains information about the devices.
*/
RAGUI_EXPORT extern const sAttrName   Devices;

/*! Points to a text node with the font family name.
*/
RAGUI_EXPORT extern const sAttrName   FontFamily;

/*! Points to an integer node with the font size.
*/
RAGUI_EXPORT extern const sAttrName   FontSize;

/*! Points to a folder node with user defined styles. It is optional. If
absent, then the default styles will be used for Normal and Night regimes. If
the root has this attribute with an empty folder as its value, then RA creates
2 text attributes with default style sheets. The later can be changed manually
with RAdmin.
*/
RAGUI_EXPORT extern const sAttrName   Styles;

/*! Points to a color set node with the basic line color set.
*/
RAGUI_EXPORT extern const sAttrName   LineColorSet;

/*! Points to a color set node which stores the custom colors from
    standard color select dialog.
*/
RAGUI_EXPORT extern const sAttrName   CustomColors;

/*! Points to a text node with the language.
*/
RAGUI_EXPORT extern const sAttrName   Language;

/*! Points to a folder node, which describes the unit system.
*/
RAGUI_EXPORT extern const sAttrName   UnitSystem;

/*! Points to a folder node with record numbering data.
*/
RAGUI_EXPORT extern const sAttrName   RecordNumbering;

/*! Points to a folder node, into which the new records are stored.
*/
RAGUI_EXPORT extern const sAttrName   RecordHistory;

/*! Points to a folder node, which has to be shown,
    when sRAMainWindow::ActnOpenRecords is activated.
*/
RAGUI_EXPORT extern const sAttrName   RecordsToBrowse;

/*! Points to a folder node, which has to be shown,
    when sRAMainWindow::ActnOpenSamples is activated.
*/
RAGUI_EXPORT extern const sAttrName   Samples;

/*! Points to a folder node, which has to be shown,
    when sRAMainWindow::ActnOpenTopics is activated.
*/
RAGUI_EXPORT extern const sAttrName   Topics;

/*! Points to a folder node, with CSV export settings.
*/
RAGUI_EXPORT extern const sAttrName   CSVExportSettings;

/*! Points to a folder node, with CSV export settings.
*/
RAGUI_EXPORT extern const sAttrName   CSVImportSettings;

/*! Points to a folder node, with SOS import settings.
*/
RAGUI_EXPORT extern const sAttrName   SOSImportSettings;

    //also globals are:
    //               RecordNumbering >> "LastRecordDate"
    //               RecordNumbering >> "LastRecordNo"

//------------------------------------------------------------------------------
} //namespace GlobalAttr


namespace RAGUI {
//------------------------------------------------------------------------------

RAGUI_EXPORT extern bool                          NightRegime;

RAGUI_EXPORT extern RA::sPtr<RA::sColorSetNode>   LineColorSet;
RAGUI_EXPORT extern RA::sPtr<RA::sColorSetNode>   CustomColors;
RAGUI_EXPORT extern RA::sPtr<RA::sFolderNode>     CSVExportSettings;
RAGUI_EXPORT extern RA::sPtr<RA::sFolderNode>     CSVImportSettings;
RAGUI_EXPORT extern RA::sPtr<RA::sFolderNode>     SOSImportSettings;

//------------------------------------------------------------------------------
} //namespace RAGUI

//------------------------------------------------------------------------------
} //namespace RA
#endif
