////////////////////////////////////////////////////////////////////////////////
/*! @file Helper.cpp   Организация справки (Help'а).
- Part of RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2010-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          VBursian@gmail.com
*///////////////////////////////////////////////////////////////////////////////
#include "Helper.h"
#include "Excepts.h"
#include "Environment.h"
#include "StringsAndQStrings.h"
#include <QByteArray>
#include <QString>
#include <QDir>
#include <QLibraryInfo>
#include <QMessageBox>
#include <QTimer>
namespace RA {
//------------------------------------------------------------------------------

psHelper                      sHelper::TheHelper = NULL;
const int                     sHelper::LongDelay = 1000;
const int                     sHelper::ShortDelay = 100;

//------------------------------------------------------------------ sHelper ---

void  sHelper::Initiate (sString  collection_file
                        ,sString  namespace_prefix
                        ,sString  virtual_folder)
{
  TheHelper = new sHelper(collection_file,namespace_prefix,virtual_folder);
}


void  sHelper::Terminate ()
{
  if( TheHelper ){
    delete TheHelper;  TheHelper = NULL;
  };
}


sHelper::~sHelper ()
{
  if( TheProcess ) {
    if( TheProcess->state() == QProcess::Running ){
      TheProcess->terminate();
      TheProcess->waitForFinished(3000);
    };
    delete TheProcess;  TheProcess=NULL;
  };
}


sHelper::sHelper (sString  collection_file
                 ,sString  namespace_prefix
                 ,sString  virtual_folder)
    :TheFileName(collection_file)
    ,TheNamespace(namespace_prefix+"."+RANet::Version.Principal().Text())
    ,TheVirtualFolder(virtual_folder)
    ,HideBookmarks(false)
    ,HideSearch(false)
    ,BookmarksAreHidden( ! HideBookmarks )
    ,SearchIsHidden( ! HideSearch )
    ,TheProcess(new QProcess())
    ,JustStarted(true)
{
}


sString  sHelper::PrependSettings ()
{
  sString                     Commands;
  if( TheHelper->JustStarted ){
    Commands = "ExpandToc 1;";
  }
  if( TheHelper->HideBookmarks && ! TheHelper->BookmarksAreHidden ){
    Commands += "hide bookmarks;";
    TheHelper->BookmarksAreHidden = true;
  }
  if( ! TheHelper->HideBookmarks && TheHelper->BookmarksAreHidden ){
    Commands += "show bookmarks;";
    TheHelper->BookmarksAreHidden = false;
  }
  if( TheHelper->HideSearch && ! TheHelper->SearchIsHidden ){
    Commands += "hide search;";
    TheHelper->SearchIsHidden = true;
  }
  if( ! TheHelper->HideSearch && TheHelper->SearchIsHidden ){
    Commands += "show search;";
    TheHelper->SearchIsHidden = false;
  }
  return Commands;
}


void  sHelper::ShowIndex ()
{
  if( ! TheHelper )
    return;
  TheHelper->PostponedSend( TheHelper->PrependSettings()
                          + "activateIdentifier index" );
}


void  sHelper::ShowKeyId (sString  key_id)
{
  if( ! TheHelper )
    return;
  TheHelper->PostponedSend( TheHelper->PrependSettings()
                          + "activateIdentifier " + key_id );
}


void  sHelper::ShowKeyword (sString  keyword)
{
  if( ! TheHelper )
    return;
  TheHelper->PostponedSend( TheHelper->PrependSettings()
                          + "activateIdentifier " + keyword );
}


void  sHelper::ShowRelativeURL (sString  reference)
{
  if( ! TheHelper )
    return;
  TheHelper->PostponedSend( TheHelper->PrependSettings()
                          + "SetSource "
                          + reference
                          );
}


void  sHelper::ShowQtHelpSchemeURL (sString  reference
                                   ,sString  part     )
{
  if( ! TheHelper )
    return;
  TheHelper->PostponedSend( TheHelper->PrependSettings()
                          + "SetSource "
                          + "qthelp://"
                          + TheHelper->TheNamespace
                          + "." + part
                          + "/" + TheHelper->TheVirtualFolder
                          + "/" + part
                          + "/" + reference
                          );
}


bool  sHelper::Start ()
{
  if( ! TheProcess )
    return false;
  if( TheProcess->state() != QProcess::Running ){
    JustStarted=true;
    Delay=LongDelay;
    QStringList                 Arguments;
    Arguments << QLatin1String("-collectionFile")
              << ToQString(TheFileName)
              << QLatin1String("-enableRemoteControl");
    QString                     TheQtAssistantProgram;
    TheQtAssistantProgram = QLatin1String("assistant");

    TheProcess->start(TheQtAssistantProgram, Arguments);

    if( ! TheProcess->waitForStarted() ){
      TheQtAssistantProgram = QLibraryInfo::location(QLibraryInfo::BinariesPath)
                            + QDir::separator()
                            +TheQtAssistantProgram;

      if( TheProcess->state() == QProcess::Running ){
        TheProcess->terminate();
        TheProcess->waitForFinished(3000);
      };
      TheProcess->start(TheQtAssistantProgram, Arguments);

      if( ! TheProcess->waitForStarted() ){
        QMessageBox::critical(NULL
                             ,tr("Researh Assistant")
                             ,tr("Unable to launch Qt Assistant (%1)")
                                                  .arg(TheQtAssistantProgram) );
        return false;
      };
    };
  };
  return true;
}


void sHelper::PostponedSend (sString  Command)
{
  if( ! Start() )
    return;
  Cmd=Command;
  QTimer::singleShot(Delay,this,SLOT(Send()));
  JustStarted=false;
  Delay=ShortDelay;
}


void sHelper::Send ()
{
  TheProcess->write( QByteArray((literal)Cmd) + '\0' );
  QTimer::singleShot(Delay,this,SLOT(Sync()));
}


void  sHelper::Sync ()
{
  TheProcess->write( QByteArray("syncContents") + '\0' );
}

//------------------------------------------------------------------------------
} //namespace RA
