////////////////////////////////////////////////////////////////////////////////
/*! @file MarksUI.cpp Функционалы навигатора и окна графика,
                       специфичные для разметки.
- Part of RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "MarksUI.h"
#include "Navigator.h"
#include "Marks.h"
#include "AttrDlg.h"
#include "NetExceptions.h"
#include "Globals.h"
#include <QMessageBox>
namespace RA {
//------------------------------------------------------------------------------

//---------------------------------------------------------- sPositionMarkUI ---

sPositionMarkUI::sPositionMarkUI ()
{
  sNavigatorViewport::MenuGenerators.push_front(&MenuGenerator);
}


void  sPositionMarkUI::MenuGenerator (psNetViewMenu                context_menu
                                     ,sNet2TreeEvolution::pcsItem  tree_item)
{
  if( sPositionMark::Type()->CanCast(tree_item->AttrValue().operator ->()) ){
    context_menu->Insert( new sNetViewAction
        (context_menu->parentWidget()
        ,&EditAppearance  ,/*checkable*/false ,/*checked*/false
        ,sNetViewMenu::tr("Change appearance")
        ,sNetViewMenu::tr("Change color, thickness, etc.")
        ));
  }
}


void  sPositionMarkUI::EditAppearance (sNet2TreeEvolution::pcsItem  tree_item
                                      ,sPoint                       /*pos*/)
{
  psPositionMark              Mark( dynamic_cast<psPositionMark>
                                        (tree_item->AttrValue().operator->()) );
  ASSERT( Mark );
  pcsPositionMarkAppearance   Old( dynamic_cast<pcsPositionMarkAppearance>
                                                 (Mark->Appearance()) );
  psPositionMarkAppearance    New( Old
                                 ? new sPositionMarkAppearance(*Old)
                                 : new sPositionMarkAppearance()    );
  sEditPositionMarkAppearanceDialog
                              D(New
                               ,sXMark::Type()->CanCast(Mark)
                               ,RAGUI::LineColorSet->Value()
                               ,( RAGUI::NightRegime ? sColor::cBlack
                                                     : sColor::cWhite )
                               );
  if( D.exec() ){
    Mark->SetAppearance(New);
  }else{
    delete New;
  }
}

//----------------------------------------------------------------- sXMarkUI ---

sXMarkUI::sXMarkUI ()
{
  sNavigatorViewport::MenuGenerators.push_front(&MenuGenerator);
  sChartViewport::DrawProcCollection.push_front( & Draw );
}


void  sXMarkUI::MenuGenerator (psNetViewMenu                context_menu
                              ,sNet2TreeEvolution::pcsItem  tree_item)
{
  if( sFolderNode::Type()->CanCast(tree_item->AttrValue().operator ->()) ){
    context_menu->AddNew->addAction( new sNetViewAction
        (context_menu->parentWidget()
        ,&AddNew  ,/*checkable*/false ,/*checked*/false
        ,sNetViewMenu::tr("x-mark")
        ,sNetViewMenu::tr("Add new x-mark attribute")
        ));
  }
}


bool  sXMarkUI::Draw (QPainter *       painter
                     ,psChartViewport  viewport
                     ,psGraphObject    object
                     ,pcsAppearance    appearance)
{
  psXMark                     Mark(dynamic_cast<psXMark>(object));
  if( Mark ){
    bool                        Draw = true;
    if( pcsAppearance A = dynamic_cast<pcsAppearance> (appearance) ){
      Draw = ! A->Hidden;
    }
    if( Draw ){
      painter->save();
      QPen                        Pen=painter->pen();
      if( pcsLineAppearance A = dynamic_cast<pcsLineAppearance> (appearance) ){
        QColor                      C( A->LineColor.Red()
                                      ,A->LineColor.Green()
                                      ,A->LineColor.Blue()
                                      ,A->LineColor.Alpha() );
        Pen.setColor(C);
        Pen.setWidth(A->LineThickness);
        painter->setPen(Pen);
      }
//      pcsPositionMarkAppearance   A (dynamic_cast<pcsPositionMarkAppearance>
//                                                                (appearance));
      try{
        int                   M(viewport->Scales().X.T((sPhysValue)(*Mark)));
        painter->drawLine(M  ,0  ,M ,viewport->size().height());
      }catch(rxRAlgebra){};
      painter->restore();
    }
    return true;
  }
  return false;
}


void  sXMarkUI::AddNew (sNet2TreeEvolution::pcsItem  tree_item
                       ,sPoint                       /*pos*/)
{
  sAttrDlg                    D;
  D.setWindowTitle( tr("Add x-mark attribute") );
  sPtr<sXMark>                NewNode;
  while( D.exec() ){
    NewNode = sNodePtr(new sXMark);
    try{
      tree_item->AttrValue() << sAttribute(D.AttrName(),NewNode);
      break;
    }catch( rxNetInvalidOp ){
      QMessageBox::critical(NULL ,D.windowTitle()
          ,tr("Uups!  Iz-z-zya-a-a!","Add attribute") );
    }
  }
}

//----------------------------------------------------------------- sYMarkUI ---

sYMarkUI::sYMarkUI ()
{
  sNavigatorViewport::MenuGenerators.push_front(&MenuGenerator);
  sChartViewport::DrawProcCollection.push_front( & Draw );
}


void  sYMarkUI::MenuGenerator (psNetViewMenu                context_menu
                              ,sNet2TreeEvolution::pcsItem  tree_item)
{
  if( sFolderNode::Type()->CanCast(tree_item->AttrValue().operator ->()) ){
    context_menu->AddNew->addAction( new sNetViewAction
        (context_menu->parentWidget()
        ,&AddNew  ,/*checkable*/false ,/*checked*/false
        ,sNetViewMenu::tr("y-mark")
        ,sNetViewMenu::tr("Add new y-mark attribute")
        ));
  }
}


bool  sYMarkUI::Draw (QPainter *       painter
                     ,psChartViewport  viewport
                     ,psGraphObject    object
                     ,pcsAppearance    appearance)
{
  psYMark                     Mark(dynamic_cast<psYMark>(object));
  if( Mark ){
    bool                        Draw = true;
    if( pcsAppearance A = dynamic_cast<pcsAppearance> (appearance) ){
      Draw = ! A->Hidden;
    }
    if( Draw ){
      painter->save();
      QPen                        Pen=painter->pen();
      if( pcsLineAppearance A = dynamic_cast<pcsLineAppearance> (appearance) ){
        QColor                      C( A->LineColor.Red()
                                      ,A->LineColor.Green()
                                      ,A->LineColor.Blue()
                                      ,A->LineColor.Alpha() );
        Pen.setColor(C);
        Pen.setWidth(A->LineThickness);
        painter->setPen(Pen);
      }
//      pcsPositionMarkAppearance   A (dynamic_cast<pcsPositionMarkAppearance>
//                                                                (appearance));
      try{
        int                   M(viewport->Scales().Y.T((sPhysValue)(*Mark)));
        painter->drawLine(0  ,M  ,viewport->size().width() ,M);
      }catch(rxRAlgebra){};
      painter->restore();
    }
    return true;
  }
  return false;
}


void  sYMarkUI::AddNew (sNet2TreeEvolution::pcsItem  tree_item
                       ,sPoint                       /*pos*/)
{
  sAttrDlg                    D;
  D.setWindowTitle( tr("Add y-mark attribute") );
  sPtr<sYMark>                NewNode;
  while( D.exec() ){
    NewNode = sNodePtr(new sYMark);
    try{
      tree_item->AttrValue() << sAttribute(D.AttrName(),NewNode);
      break;
    }catch( rxNetInvalidOp ){
      QMessageBox::critical(NULL ,D.windowTitle()
          ,tr("Uups!  Iz-z-zya-a-a!","Add attribute") );
    }
  }
}

//---------------------------------------- sEditPositionMarkAppearanceDialog ---

sEditPositionMarkAppearanceDialog::sEditPositionMarkAppearanceDialog
                                     (psPositionMarkAppearance  appearance
                                     ,bool                      it_is_an_X_mark
                                     ,sColorSet                 color_set
                                     ,sColor                    background
                                     ,QWidget *                 parent)
    :sEditLineAppearanceDialog(appearance,color_set,background,parent)
    ,Appearance(appearance)
{
  setWindowTitle(tr("Mark appearance"));

  ModeBox = new QComboBox(this);
  ModeBox->addItem("infinite line",sPositionMarkAppearance::cInfiniteLine);
  ModeBox->addItem("guide towards axis",sPositionMarkAppearance::cGuideTowardsAxis);
  ModeBox->addItem("long dash across",sPositionMarkAppearance::cLongDashAcross);
  if( it_is_an_X_mark ){
    ModeBox->addItem("arrow up",sPositionMarkAppearance::cUpOrRightArrow);
    ModeBox->addItem("arrow down",sPositionMarkAppearance::cDownOrLeftArrow);
  }else{
    ModeBox->addItem("arrow right",sPositionMarkAppearance::cUpOrRightArrow);
    ModeBox->addItem("arrow left",sPositionMarkAppearance::cDownOrLeftArrow);
  }
  ModeBox->setCurrentIndex(Appearance->Mode);

  LengthBox = new QSpinBox(this);
  LengthBox->setRange(0,29);
  LengthBox->setMaximumWidth(LengthBox->height()*4);
  LengthBox->setValue(Appearance->MarkLength);

  LengthLabel = new QLabel(tr("% of scale"),this);

  QGridLayout *       Layout = (QGridLayout *)layout();
  Layout->addWidget(ModeBox              , 1,0,1,2);
  Layout->addWidget(LengthBox            , 2,0);
  Layout->addWidget(LengthLabel          , 2,1);

  Refresh();

  connect(ModeBox ,SIGNAL(currentIndexChanged(int))
         ,this    ,SLOT(Refresh()));
  connect(LengthBox ,SIGNAL(editingFinished()) ,this ,SLOT(Refresh()));
}


void  sEditPositionMarkAppearanceDialog::Refresh ()
{
  sEditLineAppearanceDialog::Refresh();

  Appearance->Mode = sPositionMarkAppearance::eMode(ModeBox->currentIndex());
  Appearance->MarkLength = LengthBox->value();

  ModeBox->setEnabled( ! Appearance->Hidden );
  LengthBox->setEnabled( ! Appearance->Hidden
                        &&( Appearance->Mode
                             > sPositionMarkAppearance::cGuideTowardsAxis));
  LengthLabel->setEnabled( LengthBox->isEnabled() );
}

//------------------------------------------------------------------------------
} //namespace RA
