////////////////////////////////////////////////////////////////////////////////
/*! @file Navigator.cpp   Навигация по сети, просмотр и редактирование.
- Part of RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2010-2016, Viktor E. Bursian, St.Petersburg, Russia.
                          VBursian@gmail.com
*///////////////////////////////////////////////////////////////////////////////
#include "Navigator.h"
#include "Net.h"
#include "Environment.h"
#include "Trace.h"
#include "StringsAndQStrings.h"
#include "SymbolsGUI.h"
#include "AttrDlg.h"
#include "FileNode.h"
#include "CSVImportDialog.h"
#include "attr.h"
#include <QPainter>
#include <QApplication>
#include <QMessageBox>
#include <QMimeData>
#include <QDrag>
namespace RA {
//------------------------------------------------------------------------------

int                           sNavigator::UpdatingPeriod = 500;

//---------------------------------------------------------- sNavigatorItem ---

sNavigatorItem::~sNavigatorItem ()
{
  if( Expander ){
    delete Expander;  Expander=NULL;
  }
}


sNavigatorItem::sNavigatorItem (psNetViewport               viewport
                               ,sNet2TreeEvolution::psItem  tree_item)
  :sNetViewItem(viewport,tree_item)
  ,Expander(NULL)
{
  if( TreeItem->IsFolderNode() ){
    sString                     S("<font");
//    S += " font-family=DejaVuSans";
//    S += " face=\"FreeSans\"";
    S += " size=+1";
    S += " color=chocolate";
    S += ">";
    switch( TreeItem->Expansion() ){
        case sNet2TreeEvolution::NotExpandable:
            S+=Symbol::NotExpandable;
            break;
        case sNet2TreeEvolution::FullyCollapsed:
            S+=Symbol::FullyCollapsed;
            break;
        case sNet2TreeEvolution::Inline_OmitRepetitions:
            S+=Symbol::Inline_OmitRepetitions;
            break;
        case sNet2TreeEvolution::Inline:
            S+=Symbol::Inline;
            break;
        case sNet2TreeEvolution::CanonicallyExpanded:
            S+=Symbol::CanonicallyExpanded;
            break;
    }
    S += "</font>";
    Expander=new QTextDocument();
    Expander->setDocumentMargin(0);
    Expander->setHtml(ToQString(S));
  }
  Representation.setDocumentMargin(0);
  Representation.setHtml(ToQString(TreeItem->HyperText()));
}

//---------------------------------------------------------- sNavigatorMenu ---

//sNavigatorMenu::~sNavigatorMenu ()
//{
//}


sNavigatorMenu::sNavigatorMenu (psNetViewport               viewport
                               ,sNet2TreeEvolution::psItem  tree_item)
  :sNetViewMenu(viewport,tree_item)
{
//  if( tree_item->Parent() ){
//    QAction *       RemoveAction=new QAction(tr("Delete this attribute"),this);
//    Insert(RemoveAction);
//    connect(RemoveAction,SIGNAL(triggered()),viewport,SLOT(RemoveAttribute()));
//  }
}

//-------------------------------------------------------- sNavigatorViewport ---

sNavigatorViewport::~sNavigatorViewport ()
{
  Clear();
}

sNavigatorViewport::sNavigatorViewport(psNet2TreeEvolution  evolution
                                      ,QWidget *            parent)
  :sNetViewport(evolution,parent)
  ,Indentation(15)
  ,ExpanderWidth(15)
  ,IconWidth(0)
  /*,Items*/
  ,DragIsAboutToStart(false)
  /*,DragStartPosition*/
{
//  resize(400,200); не работает

//     QFontMetrics metric(font());
//     QSize size = metric.size(Qt::TextSingleLine, text);
  DestinationRect = new QRubberBand(QRubberBand::Rectangle, this);
  setAcceptDrops(true);
//! @todo{UI} лучше бы вместо chocolate везде использовать цвет,
//  взятый из role Highlight из QApplication::palette().color(QPalette::Highlight)
//  и затемненный раза в два
//  setStyleSheet("QFrame {color: chocolate; background: white}");
//  setStyleSheet("color: red");//changes color of all sub-widgets, including menu
}


void  sNavigatorViewport::FormItems ()
{
  Clear();
  sNet2TreeEvolution::psItem  I = Evolution->FirstVisible();
  while( I ){
    Items.push_back(new sNavigatorItem(this,I));
    I=I->NextVisible();
  }
  UpdateLayout();
}


void  sNavigatorViewport::UpdateLayout ()
{
  if( ! Items.empty() ){
    qreal                 lineWidth = parentWidget()->width();
    int                   leading = 0; //fontMetrics.leading();
    qreal                 total_h = leading;
    qreal                 total_w = 0;
    qreal                 h;
    qreal                 w;

    lineWidth -= (ExpanderWidth+IconWidth);
    for(sNavigatorItems::iterator  i=Items.begin(); i!=Items.end(); ++i){
      psNavigatorItem        I=(*i);
      qreal                   IndentWidth=Indentation*(I->TreeItem->Level()-1);
      if( I->Expander ){
        I->ExpanderArea=QRect(IndentWidth ,total_h
                             ,I->Expander->size().width()
                             ,I->Expander->size().height()  );
      }
      I->Representation.setTextWidth(lineWidth-IndentWidth);
      h=I->Representation.size().height();
      w=I->Representation.size().width();
      I->RepresentationArea=QRect(IndentWidth+ExpanderWidth+IconWidth ,total_h
                                 ,w ,h);
      I->ItemArea=QRect(0 ,total_h ,1 ,1);
      I->ItemArea |= I->RepresentationArea;
      total_h+=h;
      if( total_w < w + IndentWidth )
        total_w = w + IndentWidth;
    }
    total_w +=(ExpanderWidth+IconWidth);
    total_h+=17;
    if( total_h < parentWidget()->height() )
      total_h = parentWidget()->height();
    resize(total_w,total_h);
    setToolTip(QString());
  }else{
    resize(parentWidget()->width(),parentWidget()->height());
    setToolTip(tr("drop data items here"));
  }
  update();
}


void  sNavigatorViewport::Clear ()
{
  psNavigatorItem            I;
  while( ! Items.empty() ){
    I=Items.back();
    Items.pop_back();
    delete I;
  }
}


psNavigatorItem  sNavigatorViewport::FindExpanderAt (const QPoint &  pos)
{
  if( ! Items.empty() ){
    for(sNavigatorItems::iterator  i=Items.begin(); i!=Items.end(); ++i)
      if( (*i)->Expander )
        if( (*i)->ExpanderArea.contains(pos) )
          return (*i);
  }
  return NULL;
}


psNavigatorItem  sNavigatorViewport::FindRepresentationAt (const QPoint &  pos)
{
  if( ! Items.empty() ){
    for(sNavigatorItems::iterator  i=Items.begin(); i!=Items.end(); ++i)
      if( (*i)->RepresentationArea.contains(pos) )
        return (*i);
  }
  return NULL;
}


psNavigatorItem  sNavigatorViewport::FindItemAt (const QPoint &  pos)
{
  if( ! Items.empty() ){
    for(sNavigatorItems::iterator  i=Items.begin(); i!=Items.end(); ++i)
      if( (*i)->ItemArea.contains(pos) )
        return (*i);
  }
  return NULL;
}


void  sNavigatorViewport::RemoveAttribute ()
{
  ASSERT(ContextItem);
  sNet2TreeEvolution::psItem  TreeItem = ContextItem->TreeItem;
  sNodePtr                    HostNode = TreeItem->Parent()->AttrValue();
  sNodePtr                    Node = TreeItem->AttrValue();
  if( QMessageBox::Yes ==
      QMessageBox::question(parentWidget()
                           ,tr("Delete this attribute")
                           ,tr("Are you sure?")
                           ,QMessageBox::Yes | QMessageBox::No   )  ){
    if( TreeItem->AttrName().Type() == sAttrName::cNameless )
      HostNode.DelAttr(Node);
    else
      HostNode.DelAttr(TreeItem->AttrName());
    //Update(TreeItem->Parent());
    Evolution->Update();
  }
}


void  sNavigatorViewport::Draw (QPainter *  painter)
{
  if( ! Items.empty() ){
    psNavigatorItem      I;
    QString              FB(QString("<font color=")
                           +painter->pen().color().name(QColor::HexArgb)
                           +">");
    QString              FE("</font>");
    for(sNavigatorItems::iterator  i=Items.begin(); i!=Items.end(); ++i){
      I=*i;
      if( I->Expander ){
        painter->save();
        painter->translate( I->ExpanderArea.topLeft() );
        I->Expander->drawContents(painter);
        painter->restore();
      }
      painter->save();
      painter->translate( I->RepresentationArea.topLeft() );
      I->Representation.setHtml(FB+ToQString(I->TreeItem->HyperText())+FE);
      I->Representation.drawContents(painter);
      painter->restore();
    }
  }
}


void  sNavigatorViewport::paintEvent (QPaintEvent *)
{
  QPainter                    Painter(this);
  Draw(&Painter);
}


void sNavigatorViewport::mousePressEvent (QMouseEvent *  event)
{
  if( event->button() == Qt::LeftButton ){
    DragStartPosition = event->pos();
    DragIsAboutToStart = true;
  }
}


void sNavigatorViewport::mouseMoveEvent (QMouseEvent *  event)
{
  if( event->buttons() & Qt::LeftButton ){
    if( (event->pos() - DragStartPosition).manhattanLength()
                                        > QApplication::startDragDistance() ){
      DragIsAboutToStart = false;
//      if( event->modifiers() & (Qt::ControlModifier | Qt::ShiftModifier) ){
        StartDraggingAnObjectIfAny();
//      }
    }
  }
}


void sNavigatorViewport::mouseReleaseEvent (QMouseEvent *  event)
{
  if( event->button() == Qt::LeftButton ) {
    if( DragIsAboutToStart
        && ((event->pos() - DragStartPosition).manhattanLength()
                                       < QApplication::startDragDistance()) ) {
      //left click (possibly a part of a double click)
      DragIsAboutToStart = false;
      psNavigatorItem      I = FindExpanderAt(event->pos());
      if( I  )
        I->TreeItem->ChangeExpansion();
    }
  }
}


void sNavigatorViewport::mouseDoubleClickEvent (QMouseEvent *  /*event*/)
{
  /*! @todo{UI++} in-place editing */
  /*! @todo{UI Focus} */
}


void sNavigatorViewport::StartDraggingAnObjectIfAny ()
{
  psNavigatorItem             ViewItem= FindRepresentationAt(DragStartPosition);
  if( ViewItem ){
    Evolution->Lock();
    ASSERT( ! sNet2TreeEvolution::sItem::DraggedItem );
    sNet2TreeEvolution::sItem::DraggedItem = ViewItem->TreeItem;
    QDrag *                     drag = new QDrag(this);
    QMimeData *                 mimeData = new QMimeData;
    QByteArray                  data;
    mimeData
        ->setData(ToQString(sNet2TreeEvolution::sItem::MimeType), data);
    drag->setMimeData(mimeData);

//    Qt::DropAction              DropAct;

    /*DropAct =*/ drag->exec( Qt::MoveAction | Qt::CopyAction | Qt::LinkAction);
    /*
    switch( DropAct ){
      case Qt::MoveAction:
          TRACE("Moved");
          break;
      case Qt::CopyAction:
          TRACE("Copied");
          break;
      case Qt::LinkAction:
          TRACE("Linked");
          break;
      default:
          TRACE(DropAct);
    }
    */
    sNet2TreeEvolution::sItem::DraggedItem = NULL;
    Evolution->Unlock();
  }
}


void  sNavigatorViewport::dragEnterEvent (QDragEnterEvent *  event)
{
//  TRACE("drag enter")
//  TRACE(FromQString(event->mimeData()->formats().join("|")))
  if(    event->mimeData()->hasUrls()
      || event->mimeData()->hasImage()
      || event->mimeData()->hasText()
      || event->mimeData()
           ->hasFormat(ToQString(sNet2TreeEvolution::sItem::MimeType)) ){
    event->acceptProposedAction();
    DestinationRect->setGeometry(QRect(event->pos()
                                      ,event->pos()).normalized());
    DestinationRect->show();
  }
}


void  sNavigatorViewport::dragMoveEvent (QDragMoveEvent *  event)
{
//  if( event->answerRect().intersects(dropFrame->geometry()) )
//    ;
  const bool                  ThereAreURLs(event->mimeData()->hasUrls());
  const bool                  ThereIsAttribute(event->mimeData()->hasFormat(
                                  ToQString(sNet2TreeEvolution::sItem::MimeType)
                                                                           ));
  if( ThereAreURLs || ThereIsAttribute ){
    psNavigatorItem             ViewItem = FindItemAt(event->pos());
    sNet2TreeEvolution::psItem  DestinationItem = ( ViewItem
                                                  ? ViewItem->TreeItem
                                                  : TreeRoot() );
    if( ViewItem ){
      DestinationRect->setGeometry(ViewItem->ItemArea);
    }else{
      DestinationRect->setGeometry(QRect(event->pos()
                                        ,event->pos()).normalized());
    }
    if( ThereAreURLs ){
      if( (sEnvironment::User.AccessLevel != sUser::cNewbie)
          && DestinationItem->IsFolderNode()
          && (  (event->proposedAction() == Qt::LinkAction)
             || (event->proposedAction() == Qt::CopyAction)
             || (event->proposedAction() == Qt::MoveAction)  ) ){
        event->acceptProposedAction();
      }else{
        event->ignore();
      }
    }else if( ThereIsAttribute ){
      if( event->proposedAction() == Qt::LinkAction ){
        if( (sEnvironment::User.AccessLevel != sUser::cNewbie)
            && DestinationItem->IsFolderNode() ){
          event->acceptProposedAction();
        }else{
          event->ignore();
        }
      }else if( event->proposedAction() == Qt::CopyAction ){
        if( (sEnvironment::User.AccessLevel == sUser::cAdmin)
            && DestinationItem->IsFolderNode() ){
          event->acceptProposedAction();
//        }else{
//          event->ignore();
        }
      }else if( event->proposedAction() == Qt::MoveAction ){
        if( sEnvironment::User.AccessLevel == sUser::cNewbie ){
          event->ignore();
        }else{
          event->acceptProposedAction();
        }
      }
    }
  }else if( event->mimeData()->hasImage() || event->mimeData()->hasText() ){
    psNavigatorItem             ViewItem = FindItemAt(event->pos());
//    sNet2TreeEvolution::psItem  DestinationItem = ( ViewItem
//                                                  ? ViewItem->TreeItem
//                                                  : TreeRoot() );
    if( ViewItem ){
      DestinationRect->setGeometry(ViewItem->ItemArea);
    }else{
      DestinationRect->setGeometry(QRect(event->pos()
                                        ,event->pos()).normalized());
    }
    event->setDropAction(Qt::CopyAction);
    event->accept();
  }
}


void  sNavigatorViewport::dragLeaveEvent (QDragLeaveEvent *  /*event*/)
{
  DestinationRect->hide();
}


void  sNavigatorViewport::dropEvent (QDropEvent *  event)
{
//  if( event->source() == this && event->possibleActions() & Qt::MoveAction )
//    return;
  const bool                  ThereAreURLs(event->mimeData()->hasUrls());
  const bool                  ThereIsAttribute(event->mimeData()->hasFormat(
                                  ToQString(sNet2TreeEvolution::sItem::MimeType)
                                                                           ));
  DestinationRect->hide();
//  if( ThereAreURLs || ThereIsAttribute ){
    psNavigatorItem             ViewItem = FindItemAt(event->pos());
    sNet2TreeEvolution::psItem  DestinationItem = ( ViewItem
                                                  ? ViewItem->TreeItem
                                                  : TreeRoot() );

    if( ThereAreURLs ){
      if( (sEnvironment::User.AccessLevel != sUser::cNewbie)
          && DestinationItem->IsFolderNode()
          && (  (event->proposedAction() == Qt::LinkAction)
             || (event->proposedAction() == Qt::CopyAction)
             || (event->proposedAction() == Qt::MoveAction)  ) ){
        event->acceptProposedAction();
        AcceptDroppedURLs(DestinationItem->AttrValue()
                         ,event->mimeData()->urls()
                         ,event->proposedAction());
      }else{
        event->ignore();
      }
    }else if( ThereIsAttribute ){
      if( event->proposedAction() == Qt::LinkAction ){
        if( (sEnvironment::User.AccessLevel != sUser::cNewbie)
            && DestinationItem->IsFolderNode() ){
          event->acceptProposedAction();
          sAttrDlg                      D(parentWidget());
          D.setWindowTitle(tr("Link to a datanet node"));
              //"The dragged value will be linked to the destination object."
              //"  The newborn attribute may have the same or a different name."
          D.SetAttrName(sNet2TreeEvolution::sItem::DraggedItem->AttrName());
          if( D.exec() ){
            DestinationItem->AttrValue()
                << sAttribute(D.AttrName()
                             ,sNet2TreeEvolution::sItem
                                                    ::DraggedItem->AttrValue()
                             );
          }
        }
      }else if( event->proposedAction() == Qt::CopyAction ){
        if( (sEnvironment::User.AccessLevel == sUser::cAdmin)
            && DestinationItem->IsFolderNode() ){
          event->acceptProposedAction();
//          TRACE("copied")
            //"The dragged object will be ascribed as a model "
            //"to the destination object. "
          // Process the data from the event.
//        }else{
//          event->ignore();
        }
      }else if( event->proposedAction() == Qt::MoveAction ){
        if( sEnvironment::User.AccessLevel == sUser::cNewbie ){
          event->ignore();
        }else{
          event->acceptProposedAction();
//          TRACE("moved")
          // Process the data from the event.
        }
      }
    }else if( event->mimeData()->hasImage() ){
//      TRACE("image")
      event->setDropAction(Qt::CopyAction);
      event->accept();
      return;
    }else if( event->mimeData()->hasText() ){
//      TRACE("text")
//      TRACE(FromQString(event->mimeData()->text()))
      event->setDropAction(Qt::CopyAction);
      event->accept();
      return;
    }else{
//      TRACE("drop ignore")
//      TRACE(FromQString(event->mimeData()->formats().join("|")))
      event->ignore();
    }
//  }
}


bool  sNavigatorViewport::AcceptDroppedURLs (sPtr<sFolderNode>  destination
                                            ,QList<QUrl>        list_of_URLs
                                            ,Qt::DropAction     action)
{
  bool                        OK = false;
  if( destination.IsCorrect() ){
    OK = true;
    foreach( QUrl URL , list_of_URLs ){
      bool OK_here = AcceptDroppedURL(destination,URL,action);
      OK = OK_here && OK;
    }
  }
  return OK;
}


bool  sNavigatorViewport::AcceptDroppedURL (sPtr<sFolderNode>  destination
                                           ,QUrl               an_URL
                                           ,Qt::DropAction     action)
{
  //!@todo{UI} Logic needs to be reviewed

//  TRACE(FromQString(an_URL.toDisplayString()))
//  TRACE(FromQString(an_URL.toLocalFile()))
  if( an_URL.isLocalFile() ){
    sPath                       Path(FromQString(an_URL.toLocalFile()));
    if( action == Qt::CopyAction ){
      if(    (Path.Ext().UpperCase() == "CSV")
          || (Path.Ext().UpperCase() == "DAT") ){
        sCSVImportDialog                      D(an_URL.toLocalFile()
                                               ,parentWidget());
        D.setWindowTitle(tr("Import CSV file as a data function"));
        D.SetAttrName(sAttrName(Path.Name()));
        if( D.exec() && ! D.Value.IsNULL() ){
          destination << sAttribute(D.AttrName(),D.Value);
          return true;
        }else{
          return false;
        }
      }
      destination << attr( Path.Name() + ( Path.Ext().Empty()
                                         ? sString()
                                         : sString(".")+Path.Ext() )
                         ,sNodePtr(new sFileNode(Path))
                         );
      return true;
    }
  }
  return false;
}


void  sNavigatorViewport::contextMenuEvent (QContextMenuEvent *  event)
{
  ContextItem = FindItemAt(event->pos());
  Evolution->Lock();
  sNet2TreeEvolution::psItem  TreeItem = ( ContextItem
                                         ? ContextItem->TreeItem
                                         : TreeRoot() );
//  pcsTag                      NodeType = TreeItem->AttrValue()->TrueType();
  sNavigatorMenu             PopupMenu(this,TreeItem);
//  sNavigatorMenu             PopupMenu(parentWidget(),TreeItem);
  PopupMenu.SetHeader(TreeItem->HyperText());
  if( ! MenuGenerators.empty() ){
    for( sNetViewMenuGenerators::iterator  i=MenuGenerators.begin();
                                           i!=MenuGenerators.end(); ++i)
      (*i)(&PopupMenu,TreeItem);
  }
  if( TreeItem->Parent() ){
    QAction *       RemoveAction=new QAction(tr("Delete this attribute"),this);
    PopupMenu.Insert(RemoveAction);
    connect(RemoveAction,SIGNAL(triggered()),this,SLOT(RemoveAttribute()));
  }
  if( PopupMenu.AddNew->isEmpty() ){
    delete PopupMenu.AddNew;  PopupMenu.AddNew=NULL;
  }
  /*! @patch{Insert_attribute} */
//  if( PopupMenu.InsertNew->isEmpty() ){
    delete PopupMenu.InsertNew;  PopupMenu.InsertNew=NULL;
//  }
  QAction *                   SelectedAction
                                  = PopupMenu.exec(event->globalPos());
  if(psNetViewAction MyAction=dynamic_cast<psNetViewAction>(SelectedAction) ){
    (*(MyAction->DoIt))(TreeItem,sPoint(event->pos().x(),event->pos().y()));
    Evolution->Update();
  }
  Evolution->Unlock();
}

//-------------------------------------------------------------- sNavigator ---

sNavigator::~sNavigator ()
{
  if( Viewport ){ delete Viewport;  Viewport=NULL; }
}


sNavigator::sNavigator (psNet2TreeEvolution  evolution
                       ,QWidget *            parent)
  :QScrollArea(parent)
{
  Viewport=new sNavigatorViewport(evolution,this);
  setWidget(Viewport);
  setWidgetResizable(false);
}


void  sNavigator::resizeEvent (QResizeEvent *)
{
  if( Viewport ){
    Viewport->UpdateLayout();
  }
}


void  sNavigator::Print (QPainter *  painter)
{
  Viewport->resize(size());
  viewport()->resize(size());
  Viewport->FormItems();
  Viewport->UpdateLayout();
  Viewport->Draw(painter);
}

//------------------------------------------------------------------------------
} //namespace RA

