////////////////////////////////////////////////////////////////////////////////
/*! @file Navigator.h   Навигация по сети, просмотр и редактирование.
- Part of RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2010-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          VBursian@gmail.com
*///////////////////////////////////////////////////////////////////////////////
#ifndef Navigator_H
#define Navigator_H
#include "NetView.h"
#include <QMouseEvent>
#include <QContextMenuEvent>
#include <QScrollArea>
#include <QTextDocument>
#include <QRubberBand>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sNavigatorItem)
ANNOUNCE_CLASS(sNavigatorMenu)
ANNOUNCE_CLASS(sNavigatorViewport)
ANNOUNCE_CLASS(sNavigator)

typedef std::list<psNavigatorItem>  sNavigatorItems;

//----------------------------------------------------------- sNavigatorItem ---

class sNavigatorItem : public sNetViewItem
{
  public:
    virtual                   ~sNavigatorItem ();
    explicit                  sNavigatorItem (psNetViewport
                                             ,sNet2TreeEvolution::psItem);
  public://fields
    QRect                     ItemArea;
    QTextDocument *           Expander;
    QRect                     ExpanderArea;
    QTextDocument             Representation;
    QRect                     RepresentationArea;
  protected:
};

//---------------------------------------------------------- sNavigatorMenu ---

class sNavigatorMenu : public sNetViewMenu
{
  Q_OBJECT
  public:
//    virtual                   ~sNavigatorMenu ();
                              sNavigatorMenu (psNetViewport
                                             ,sNet2TreeEvolution::psItem);
};

//------------------------------------------------------- sNavigatorViewport ---

class sNavigatorViewport : public sNetViewport
{
  Q_OBJECT
  public://static
    static sNetViewMenuGenerators
                              MenuGenerators;
  public:
    virtual                   ~sNavigatorViewport ();
    explicit                  sNavigatorViewport (psNet2TreeEvolution
                                                 ,QWidget *  parent = 0);
    virtual void              FormItems ();
    virtual void              UpdateLayout ();
    virtual psNavigatorItem   FindExpanderAt (const QPoint &);
    virtual psNavigatorItem   FindRepresentationAt (const QPoint &);
    virtual psNavigatorItem   FindItemAt (const QPoint &);
    virtual void              Draw (QPainter *);

  signals:

  public slots:
    void                      RemoveAttribute ();
//    void                      EditAllocation ();

  protected://inherited
    virtual void              paintEvent (QPaintEvent *);
//    virtual void              resizeEvent (QResizeEvent *);
    virtual void              mousePressEvent (QMouseEvent *);
    virtual void              mouseMoveEvent (QMouseEvent *);
    virtual void              mouseReleaseEvent (QMouseEvent *);
    virtual void              mouseDoubleClickEvent (QMouseEvent *);
    virtual void              dragEnterEvent (QDragEnterEvent *);
    virtual void              dragMoveEvent (QDragMoveEvent *);
    virtual void              dragLeaveEvent (QDragLeaveEvent *);
    virtual void              dropEvent (QDropEvent *);
    virtual void              contextMenuEvent (QContextMenuEvent *);
  private:
    void                      Clear ();
    void                      StartDraggingAnObjectIfAny ();
    bool                      AcceptDroppedURLs (sPtr<sFolderNode>  destination
                                                ,QList<QUrl>        list_of_URLs
                                                ,Qt::DropAction     action);
    bool                      AcceptDroppedURL (sPtr<sFolderNode>  destination
                                               ,QUrl               an_URL
                                               ,Qt::DropAction     action);
  public://fields
    qreal                     Indentation;
    qreal                     ExpanderWidth;
    qreal                     IconWidth;
  private://fields
    sNavigatorItems           Items;
    bool                      DragIsAboutToStart;
    QPoint                    DragStartPosition;
    psNavigatorItem           ContextItem;
    QRubberBand *             DestinationRect;
};

//--------------------------------------------------------------- sNavigator ---

class RAGUI_EXPORT  sNavigator : public QScrollArea
{
  Q_OBJECT
  public://static
    static int                UpdatingPeriod;

  public:
    virtual                   ~sNavigator ();
    explicit                  sNavigator (psNet2TreeEvolution  evolution
                                         ,QWidget *            parent = 0);
    virtual void              Print (QPainter *);

  protected:
    virtual void              resizeEvent (QResizeEvent *);

  public slots:
    void                      OnNetChange ()
                                { Viewport->FormItems(); }

  private://fields
    psNavigatorViewport       Viewport;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif

