////////////////////////////////////////////////////////////////////////////////
/*! @file Net2TreeEvolution.cpp   Разворачивание сети в виде дерева.
- Part of RAGUI - Research Assistant Graphical User Interface.
- Uses  QtCore v.4.6  -  http://qt.nokia.com/
  (actually, it uses 1 signal and 1 slot; could be implemented with callbacks,
  then moved into RANet, where it belongs)
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2013, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "Net2TreeEvolution.h"
#include "Environment.h"
#include "Log.h"
namespace RA {
//------------------------------------------------------------------------------

sNet2TreeEvolution::psItem       sNet2TreeEvolution::sItem::DraggedItem = NULL;
const sString                    sNet2TreeEvolution::sItem::MimeType
                                              ("application/ra-evolutionitem");

//------------------------------------------------ sNet2TreeEvolution::sItem ---

sNet2TreeEvolution::sItem::~sItem ()
{
  while( FirstChild ){
    psItem                      I=FirstChild;
    FirstChild=I->NextSibling;
    delete I;
  };
  if( Appearance ){ delete Appearance;  Appearance=NULL; };
}


sNet2TreeEvolution::sItem::sItem (psNet2TreeEvolution  E
                                 ,sAttrName       AName
                                 ,sNodePtr             AValue)
  :Appearance(NULL)
  ,Host(E)
  ,TheParent(NULL)
  ,FirstChild(NULL)
  ,NextSibling(NULL)
  ,TheLevel(0)
  ,IsFolderNodeFlag(false)
  ,TheExpansion(FullyCollapsed)
  ,TheAttrName(AName)
  ,TheAttrValue(AValue)
{
  if( ! TheAttrValue.IsNULL() ){
    IsFolderNodeFlag=sFolderNode::Type()->CanCast(TheAttrValue.operator ->());
  };
}


sString  sNet2TreeEvolution::sItem::HyperText (eTextDetalization  D)// const
{
  sString                     InlineDelimiterPrefix
                                ="<font color=chocolate><b>";
  sString                     InlineDelimiterSuffix
                                ="</b></font>";
  sString                     EqualSign
                                =InlineDelimiterPrefix+"="+InlineDelimiterSuffix;
  sString                     hyper_text;
  hyper_text=TheAttrName.Text();
  if( ! TheAttrValue.IsNULL() ){
    sString                     value_text(TheAttrValue->Text(HTML,D));
    if( ! hyper_text.Empty() && ! value_text.Empty() )
      hyper_text+=EqualSign;
    hyper_text+=value_text;
  };
  if( TheExpansion == Inline ){
    psItem                      I=FirstChild;
    bool                        first=true;
    bool                        had_children=false;
    while( I ){
      had_children=true;
      if( first ){
        hyper_text+=InlineDelimiterPrefix+" {"+InlineDelimiterSuffix;
        first=false;
      }else{
        hyper_text+=InlineDelimiterPrefix+"; "+InlineDelimiterSuffix;
      };
      hyper_text+=I->HyperText(Laconic);
      I=I->NextSibling;
    };
    if( had_children )
      hyper_text+=InlineDelimiterPrefix+"}"+InlineDelimiterSuffix;
  };
  return hyper_text;
}


void  sNet2TreeEvolution::sItem::AppendChild (psItem  child)
{
  if( ! FirstChild ){
    FirstChild=child;
  }else{
    psItem                      I=FirstChild;
    while( I->NextSibling ){
      I=I->NextSibling;
    };
    I->NextSibling=child;
  };
  child->TheParent=this;
  child->NextSibling=NULL;
  child->TheLevel=TheLevel+1;
}


sNet2TreeEvolution::psItem  sNet2TreeEvolution::sItem::NextVisible () const
{
  if( FirstChild && (TheExpansion == CanonicallyExpanded) ){
    return FirstChild;
  }else if( NextSibling ){
    return NextSibling;
  }else{
    pcsItem                      I=this;
    while( I->TheParent ){
      I=I->TheParent;
      if( I->NextSibling )
        return I->NextSibling;
    };
    return NULL;
  }
}


void  sNet2TreeEvolution::sItem::ChangeExpansion ()
{
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wpragmas"
  #pragma GCC diagnostic ignored "-Wimplicit-fallthrough"
  switch( TheExpansion ){
      case sNet2TreeEvolution::NotExpandable:
          break;
      case sNet2TreeEvolution::FullyCollapsed:
//          TheExpansion=sNet2TreeEvolution::Inline_OmitRepetitions;
//          Expand();
//          Host->EmitChangedSignal();
//          break;
      case sNet2TreeEvolution::Inline_OmitRepetitions:
          if( sEnvironment::User.AccessLevel != sUser::cNewbie ){
            TheExpansion=sNet2TreeEvolution::Inline;
            Expand();
            Host->EmitChangedSignal();
            break;
          };
      case sNet2TreeEvolution::Inline:
          TheExpansion=sNet2TreeEvolution::CanonicallyExpanded;
          if( sEnvironment::User.AccessLevel == sUser::cNewbie ){
            Expand();
          };
          Host->EmitChangedSignal();
          break;
      case sNet2TreeEvolution::CanonicallyExpanded:
          TheExpansion=sNet2TreeEvolution::FullyCollapsed;
          Collapse();
          Host->EmitChangedSignal();
          break;
  };
  #pragma GCC diagnostic pop
}


bool  sNet2TreeEvolution::sItem::Update (bool  Silent)
{
  bool                        Changed = false;
  if( TheAttrValue.IsNULL() ){
    //do nothing
  }else{
    if( TheAttrValue.NodeIsRecentlyEdited() ){
      TheAttrValue.AcknowledgeThatNodeIsRecentlyEdited();
      Changed=true;
      switch( TheExpansion ){
      case sNet2TreeEvolution::NotExpandable:
        {
          sPtr<sFolderNode>           Folder (TheAttrValue);
          if( Folder.IsCorrect() ){
            if( Folder->HasAttributes() ){
              TheExpansion=sNet2TreeEvolution::FullyCollapsed;
//              Changed=true;
            }
          }
        };
        break;
      case sNet2TreeEvolution::FullyCollapsed:
        Collapse();
        break;
      case sNet2TreeEvolution::Inline_OmitRepetitions:
      case sNet2TreeEvolution::Inline:
      case sNet2TreeEvolution::CanonicallyExpanded:
        Expand(); //may change TheExpansion to NotExpandable
//        if( Expand() ) //may change TheExpansion to NotExpandable
//          Changed=true;
        break;
      };
    };
    psItem                      Child=FirstChild;
    while( Child ){
      if( Child->Update(/*Silent=*/ true) )
        Changed=true;
      Child=Child->NextSibling;
    };
  };
  if( Changed && ! Silent )
    Host->EmitChangedSignal();
  return Changed;
}


bool  sNet2TreeEvolution::sItem::Expand ()
{
  bool                        Changed = false;
  bool                        HasAttrs = false;
  sPtr<sFolderNode>           Folder (TheAttrValue);
  psItem                      FirstOldChild=FirstChild;
  FirstChild=NULL;
  if( Folder.IsCorrect() ){
    for_each_attribute(A,Folder){
      HasAttrs = true;
      sAttrName              ChildName(A->Name());
      sNodePtr                    ChildValue(A->Value());
      psItem                      OldChild=FirstOldChild;
      psItem                      PrevOldChild=NULL;
      bool                        Found = false;
      //ищем с таким же именем (к безымянному подходит любой безымянный)
      while( OldChild && ! Found ){
        if(    (OldChild->TheAttrName == ChildName)
               || (   (OldChild->TheAttrName.Type() == sAttrName::cNameless)
                      && (ChildName.Type() == sAttrName::cNameless)
                      )  ){
          Found=true;
        }else{
          Changed=true;//не полное соответствие
          PrevOldChild=OldChild;
          OldChild=OldChild->NextSibling;
        };
      };
      if( Found && (   OldChild->TheAttrValue.IsNULL()
                       || (OldChild->TheAttrValue == ChildValue) ) ){
        psItem                      MatchingOldChild=OldChild;
        //выдрать MatchingOldChild из старого списка
        OldChild=OldChild->NextSibling;
        if( PrevOldChild ){
          PrevOldChild->NextSibling=OldChild;
        }else{
          FirstOldChild=OldChild;
        };
        AppendChild(MatchingOldChild);
      }else{
        AppendChild(new sItem(Host,ChildName,ChildValue));
        Changed=true;
      };
    }end_attr_loop()
  }

  //удаляем старые, если остались
  while( FirstOldChild ){
    Changed=true;//не полное соответствие
    psItem                      I=FirstOldChild;
    FirstOldChild=I->NextSibling;
    delete I;
  };
  if( ! HasAttrs && TheParent ){
    TheExpansion=sNet2TreeEvolution::NotExpandable;
    Changed=true;
  };
  return Changed;
}


void  sNet2TreeEvolution::sItem::Collapse ()
{
  psItem                      Child=FirstChild;
  psItem                      PreviousChild=NULL;
  while( Child ){
    Child->Disconnect();
    if( Child->Useless() ){
      psItem                      UselessChild=Child;
      Child=Child->NextSibling;
      if( PreviousChild ){
        PreviousChild->NextSibling=Child;
      }else{
        FirstChild=Child;
      };
      delete UselessChild;
    }else{
      PreviousChild=Child;
      Child=Child->NextSibling;
    };
  };
}


void  sNet2TreeEvolution::sItem::Disconnect ()
{
  Collapse();
  TheAttrValue.SetToNULL();
}


bool  sNet2TreeEvolution::sItem::Useless ()
{
  if( Appearance ){
    return false;
  };
  return true;
}

//------------------------------------------------------- sNet2TreeEvolution ---

sNet2TreeEvolution::~sNet2TreeEvolution ()
{
  if( TheTreeRoot ){
    delete TheTreeRoot;  TheTreeRoot=NULL;
  };
}


sNet2TreeEvolution::sNet2TreeEvolution (sAttribute  root_node)
    :sStorable()
    ,Locked(0)
{
  TheTreeRoot=new sItem(this ,root_node.Name(),root_node.Value());
}


sNet2TreeEvolution::sNet2TreeEvolution (std::istream &  a_stream
                                       ,rcsVersion      version)
    :sStorable(a_stream,version)
    ,Locked(0)
{
}


void  sNet2TreeEvolution::Store (std::ostream &  a_stream) const
{
  sStorable::Store(a_stream);
}


void  sNet2TreeEvolution::Initiate ()
{
  TheTreeRoot->TheExpansion = CanonicallyExpanded;
  TheTreeRoot->Expand();
  emit Changed();
}


void  sNet2TreeEvolution::Update ()
{
  if( ! Locked ){
    TheTreeRoot->Update();
  }else{
    RANet::Log.Put(sLog::Debug,"Net2Tree","sNet2TreeEvolution::Update: Locked");
  };
}

//------------------------------------------------------------------------------
} //namespace RA
