////////////////////////////////////////////////////////////////////////////////
/*! @file Net2TreeEvolution.h   Разворачивание сети в виде дерева.
- Part of RAGUI - Research Assistant Graphical User Interface.
- Uses  QtCore v.4.6  -  http://qt.nokia.com/
  (actually, it uses 1 signal and 1 slot; could be implemented with callbacks,
  then moved into RANet, where it belongs)
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2013, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef Net2TreeEvolution_H
#define Net2TreeEvolution_H
#include "NodePtr.h"
#include "FolderNode.h"
#include "Graphicals.h"
#include "RAGUI_global.h"
#include <QObject>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sNet2TreeEvolution)

//------------------------------------------------------- sNet2TreeEvolution ---
/*! Служебный класс, предназначенный для просмотра части сети в виде
    древовидной рекурсивной развёртки некоего кластера.

Используется в абстрактном классе sNetViewport и, соответственно, в его потомках
sNavigatorViewport и sChartViewport, обеспечивающих основную функциональность
классов sNavigator, sChart, sWorkbook и sPlot.

В sPlot один общий элемент класса sNet2TreeEvolution используется обоими
виджетами sNavigator и sChart, в результате чего на графике отображаются только
те узлы, которые видны в легенде (которые сделаны видимыми путём раскрытия
содержащих их кластеров), а не все узлы, до которых в принципе можно дойти.

Обеспечивает сохранение состояния свёрнутости/развёрнутости ветвей при
сохранении графиков (sPlot) и журналов (sWorkbook) на диске.

*/
class RAGUI_EXPORT  sNet2TreeEvolution : public QObject ,public sStorable
{
  Q_OBJECT
  STORABLE(sNet2TreeEvolution)
  public: //types
    enum eExpansion    {NotExpandable,FullyCollapsed
                       ,Inline_OmitRepetitions,Inline
                       ,CanonicallyExpanded};
  public: //classes
    //------------------------------------ sNet2TreeEvolution::sItem ---
    ANNOUNCE_CLASS(sItem)
    class sItem
    {
      public: //static
        static psItem             DraggedItem;
        static const sString      MimeType;

      public:
                                  ~sItem ();
        psItem                    Parent () const
                                    { return TheParent; };
        psItem                    NextVisible () const;
        void                      ChangeExpansion ();
        int                       Level () const
                                    { return TheLevel; };
        bool                      IsFolderNode () const
                                    { return IsFolderNodeFlag; };
        eExpansion                Expansion () const
                                    { return TheExpansion; };
        sString                   HyperText (eTextDetalization  D=Casual);// const;
        sAttrName            AttrName () const
                                    { return TheAttrName; };
        sNodePtr                  AttrValue () const
                                    { return TheAttrValue; };
      public://fields
        psAppearance              Appearance;

      private:
                                  sItem (psNet2TreeEvolution
                                        ,sAttrName
                                        ,sNodePtr);
        void                      AppendChild (psItem  child);
//        void                      SetExpansion (eExpansion);
        bool                      Update (bool  Silent = false);
        bool                      Expand ();
        void                      Collapse ();
        void                      Disconnect ();
        bool                      Useless ();

      private://prohibited operations
                                  sItem (rcsItem)
                                    {};
        rsItem                    operator = (rcsItem)
                                    { return *this; };

      private://fields
        psNet2TreeEvolution       Host;
        psItem                    TheParent;
        psItem                    FirstChild;
        psItem                    NextSibling;
        int                       TheLevel;
        bool                      IsFolderNodeFlag;
        eExpansion                TheExpansion;
        sAttrName            TheAttrName;
        sNodePtr                  TheAttrValue;

      friend class sNet2TreeEvolution;
    };
    //------------------------------------
  public:
    virtual                   ~sNet2TreeEvolution ();
    explicit                  sNet2TreeEvolution (sAttribute  root_node);
    psItem                    FirstVisible () const
                                { return TheTreeRoot->NextVisible(); };
    psItem                    TreeRoot ()
                                { return TheTreeRoot; };
//    sNodePtr                  Root ()
//                                {
//                                  return ( TheTreeRoot ? TheTreeRoot->AttrValue()
//                                                    : NULL);
//                                };
    virtual void              Initiate ();
    void                      Lock ()
                                { Locked++; };
    void                      Unlock ()
                                { ASSERT(Locked>0); Locked--; };

  public slots:
    virtual void              Update ();

  signals:
    void                      Changed ();

  private:
    void                      EmitChangedSignal ()
                                { emit Changed(); };
  private:
                              sNet2TreeEvolution (rcsNet2TreeEvolution)
                                  :sAdam(),QObject(),sStorable()
                                {};
    void                      operator= (rcsNet2TreeEvolution) {};
  private: //fields
    psItem                    TheTreeRoot;
    int                       Locked;

  friend class sItem;
};

//------------------------------------------------------------------------------
}; //namespace RA
#endif // Net2TreeEvolution_H
