////////////////////////////////////////////////////////////////////////////////
/*! @file NetView.h   Прото-виджет просмотра сети.
- Part of RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2010-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef NetView_H
#define NetView_H
#include "Curves.h"
#include "Net2TreeEvolution.h"
#include <QMenu>
#include <QLabel>
#include <QWidgetAction>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sNetViewItem)
ANNOUNCE_CLASS(sNetViewMenu)
ANNOUNCE_CLASS(sNetViewAction)
ANNOUNCE_ABSTRACT_CLASS(sNetViewport)

typedef std::list<psNetViewItem>  sNetViewItems;

typedef void fNetViewMenuGenerator (psNetViewMenu ,sNet2TreeEvolution::pcsItem);
typedef const fNetViewMenuGenerator *     pcfNetViewMenuGenerator;
typedef std::list
            <pcfNetViewMenuGenerator>     sNetViewMenuGenerators;

typedef void fNetViewActionHandler (sNet2TreeEvolution::pcsItem ,sPoint);
typedef fNetViewActionHandler *           pfNetViewActionHandler;
typedef const pfNetViewActionHandler      cpfNetViewActionHandler;

//------------------------------------------------------------- sNetViewItem ---

class sNetViewItem
{
  public:
    virtual                   ~sNetViewItem ()
                                {}
    explicit                  sNetViewItem (psNetViewport
                                           ,sNet2TreeEvolution::psItem);
  public://fields
//    cpsNetViewport              Viewport;
    sNet2TreeEvolution::cpsItem TreeItem;
  protected:
};

//---------------------------------------------------------- sNetViewMenu ---

class sNetViewMenu : public QMenu
{
  Q_OBJECT
  public:
    virtual                   ~sNetViewMenu ();
                              sNetViewMenu (psNetViewport
                                           ,sNet2TreeEvolution::pcsItem);
    void                      SetHeader (rcsString);
    void                      Insert (QAction *);
  public:
    QMenu *                   AddNew;
    QMenu *                   InsertNew;
  private:
    QAction *                 InsertionPoint;
    QLabel *                  HeaderLabel;
    QWidgetAction *           HeaderAction;
};

//-------------------------------------------------------- sNetViewAction ---

class sNetViewAction : public QAction
{
  Q_OBJECT
  public:
                              sNetViewAction
                                  (QObject*                   Parent
                                  ,pfNetViewActionHandler  Handler
                                  ,bool                       Checkable
                                  ,bool                       Checked
                                  ,QString                    Name
                                  ,QString                    Hint
                                  );
    cpfNetViewActionHandler   DoIt;
};

//-------------------------------------------------------- sNetViewport ---

class sNetViewport : public QFrame//QWidget
{
  Q_OBJECT
  public:
    virtual                   ~sNetViewport ();
    explicit                  sNetViewport (psNet2TreeEvolution
                                           ,QWidget *  parent = 0);
//    sNodePtr                  Root ()
//                                { return Evolution->Root(); };
    sNet2TreeEvolution::psItem
                              TreeRoot ()
                                { return Evolution->TreeRoot(); }
    virtual void              Update ()
                                { Evolution->Update(); }

  public slots:
    virtual void              FormItems () =0;

  protected://fields
    cpsNet2TreeEvolution      Evolution;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
