////////////////////////////////////////////////////////////////////////////////
/*! @file NetWidgets.cpp   Простенькие виджеты для работы с объектами RANet.
- Part of RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "NetWidgets.h"
#include "Nodes.h"
#include "Net2TreeEvolution.h"
#include "StringsAndQStrings.h"
#include <QMessageBox>
#include <QMenu>
#include <QMimeData>
namespace RA {
//------------------------------------------------------------------------------

//---------------------------------------------------------- sDropNodeWidget ---

sDropNodeWidget::sDropNodeWidget (QWidget *  parent)
    :QLabel(parent)
    ,Prompt(FromQString(tr("drop datanet node here")))
{
  ChangeValue(sNodePtr());
  setAcceptDrops(true);
  setMinimumWidth(height());
}


void  sDropNodeWidget::SetValue (sNodePtr  V)
{
  if( ChangeValue(V) )  emit Changed();
}


void  sDropNodeWidget::Clear ()
{
  SetValue(sNodePtr());
}


void  sDropNodeWidget::OnMenuClear ()
{
  if( ChangeValue(sNodePtr()) ){
    emit Changed();
    emit Edited();
  }
}


void  sDropNodeWidget::OnDropValue (sNodePtr  V)
{
  if( ChangeValue(V) ){
    emit Changed();
    emit Edited();
  }
}


bool  sDropNodeWidget::ChangeValue (sNodePtr  V)
{
  bool                        ValueChanged = ! (V == Value);
  Value=V;
  if( Value.IsNULL() ){
    setText("");
    setToolTip(ToQString(Prompt));
  }else{
    sString                     T(Value->Text(HTML));
    setText(ToQString(T));
    /*! @todo{UI trifles} show ToolTip if text goes out of the borders*/
    if( T.Length() > 20 ){
      setToolTip(ToQString(T));
    }else{
      setToolTip(QString());
    }
  }
  return ValueChanged;
}


void sDropNodeWidget::mouseReleaseEvent (QMouseEvent *  event)
{
  if( event->button() == Qt::LeftButton )
    QMessageBox::information(parentWidget()
        ,tr("Link to a datanet node")
        ,tr("This field may (or should) contain a link to "
            "another element of data. \n\n")
        +ToQString(Prompt));
}


void  sDropNodeWidget::contextMenuEvent (QContextMenuEvent *  event)
{
  QMenu                       PopupMenu(parentWidget());
  PopupMenu.addAction("Clear",this,SLOT(OnMenuClear()));
  PopupMenu.exec(event->globalPos());
}


void  sDropNodeWidget::dragEnterEvent (QDragEnterEvent *  event)
{
  if( event->mimeData()
           ->hasFormat(ToQString(sNet2TreeEvolution::sItem::MimeType)) ){
    event->setDropAction(Qt::LinkAction);
    event->accept();
  };
}


void  sDropNodeWidget::dropEvent (QDropEvent *  event)
{
  if( event->mimeData()
           ->hasFormat(ToQString(sNet2TreeEvolution::sItem::MimeType)) ){
    event->setDropAction(Qt::LinkAction);
    event->accept();
    OnDropValue(sNet2TreeEvolution::sItem::DraggedItem->AttrValue());
  };
}

//-------------------------------------------------------- sSelectNodeWidget ---

sSelectNodeWidget::sSelectNodeWidget (QWidget *  parent)
    :QComboBox(parent)
{
  SetFolderNode(sNodePtr());
  SelectValue(sNodePtr());
}


void  sSelectNodeWidget::SetFolderNode (sNodePtr  /*folder_node*/)
{
  //! @patch

//  insertItem(I,"");
//  setCurrentIndex(0);
}


void  sSelectNodeWidget::SelectValue (sNodePtr  /*node*/)
{
  //! @patch

}

//------------------------------------------------------------------------------
} //namespace RA



