////////////////////////////////////////////////////////////////////////////////
/*! @file NetWidgets.h   Простенькие виджеты для работы с объектами RANet.
- Part of RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef NetWidgets_H
#define NetWidgets_H
#include "NodePtr.h"
#include "RAGUI_global.h"
#include <QMouseEvent>
#include <QContextMenuEvent>
#include <QDragEnterEvent>
#include <QDropEvent>
#include <QLabel>
#include <QComboBox>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sDropNodeWidget)
ANNOUNCE_CLASS(sSelectNodeWidget)

//---------------------------------------------------------- sDropNodeWidget ---

class RAGUI_EXPORT  sDropNodeWidget : public QLabel
{
  Q_OBJECT
  public:
                              sDropNodeWidget (QWidget *  parent = NULL);
    void                      SetValue (sNodePtr);

  public slots:
    void                      Clear ();

  signals:
    void                      Changed ();
    void                      Edited ();

  protected://inherited
    virtual void              mouseReleaseEvent (QMouseEvent *);
    virtual void              contextMenuEvent (QContextMenuEvent *);
    virtual void              dragEnterEvent (QDragEnterEvent *);
    virtual void              dropEvent (QDropEvent *);

  public: //fields
    sNodePtr                  Value;
    sString                   Prompt;

  private slots:
    void                      OnMenuClear ();

  private:
    void                      OnDropValue (sNodePtr);
    bool                      ChangeValue (sNodePtr);
};

//-------------------------------------------------------- sSelectNodeWidget ---
/*! Производный от QComboBox виджет, выбирающий атрибут из кластера.

*/
class RAGUI_EXPORT  sSelectNodeWidget : public QComboBox
{
  Q_OBJECT
  public:
                              sSelectNodeWidget (QWidget *  parent = NULL);
    void                      SetFolderNode (sNodePtr);
    void                      SelectValue (sNodePtr);

  signals:
    void                      SelectionChanged ();

  public slots:
    void                      Clear ()
                                { SetFolderNode(sNodePtr()); }

  protected://inherited

  public: //fields
    sNodePtr                  FolderNode;
    sNodePtr                  Value;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
