////////////////////////////////////////////////////////////////////////////////
/*! @file PhysRangeField.h   Виджет-редактор диапазона физвеличины.
- Part of RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2010-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef PhysRangeField_H
#define PhysRangeField_H
#include "RAGUI_global.h"
#include "Ranges.h"
#include "PhysValueField.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sPhysRangeField)

//---------------------------------------------------------- sPhysRangeField ---
/*! Виджет-редактор физдиапазона.

 @todo{UI++} Добавить возможность запрета редактирования From или To.

*/
class RAGUI_EXPORT  sPhysRangeField : public QWidget
{
  Q_OBJECT
  public://type
    typedef             bool  tValidationProc (rsPhysRange);
                                //!< may change its argument
                                //!  returns true on success
  public:
    virtual                   ~sPhysRangeField ();
                              sPhysRangeField (QWidget *   parent
                                              ,QString     label
                                              ,sPhysRange  value
                                        ,bool  units_are_editable = false
                                        ,bool  precision_depends_on_value_digits
                                                                         = true)
                                  :QWidget(parent)
                                { 
                                  Construct(label,value,units_are_editable
                                           ,precision_depends_on_value_digits); 
                                }
                              sPhysRangeField (QWidget *   parent
                                              ,sPhysRange  value
                                        ,bool  units_are_editable = false
                                        ,bool  precision_depends_on_value_digits
                                                                         = true)
                                  :QWidget(parent)
                                { 
                                  Construct(QString(),value,units_are_editable
                                           ,precision_depends_on_value_digits); 
                                }
    QString                   Label () const
                                { return TheLabel; }
    sPhysRange                Value ();
    bool                      UnitsAreEditable ()
                                { return UnitsAreEditableFlag; }
    bool                      PrecisionDependsOnValueDigits ()
                                { return PrecisionDependsOnValueDigitsFlag; }
    void                      SetValidationProc (tValidationProc *);
    bool                      Active () const;

  signals:
    void                      LostFocus ();
    void                      Edited ();

  public slots:
    virtual psPhysRangeField  SetLabel (QString);
    virtual psPhysRangeField  SetUnitsEditable (bool);
    virtual psPhysRangeField  SetPrecisionDependentOnValueDigits (bool);
    virtual psPhysRangeField  SetReadOnly (bool);
    virtual psPhysRangeField  SetReadOnly ()
                                { return SetReadOnly(true); }
    virtual psPhysRangeField  UnsetReadOnly ()
                                { return SetReadOnly(false); }
    virtual psPhysRangeField  SetAttention (bool);
    virtual psPhysRangeField  SetAttention ()
                                { return SetAttention(true); }
    virtual psPhysRangeField  UnsetAttention ()
                                { return SetAttention(false); }
    virtual psPhysRangeField  UnsetAttentionOnEdit (bool  unset = true)
                                { UnsetAttentionOnEditFlag=unset; return this; }
    virtual psPhysRangeField  SetValidRange (sPhysRange);
    virtual psPhysRangeField  SetValue (sPhysRange);

  private slots:
    void                      PossiblyLeavingFromFrom ();
    void                      PossiblyLeaving ();
    void                      UpdateImage ();
    void                      UpdateVisibility ();
    void                      ShowOrder ();
    void                      ShowUnits ();
    void                      UpdateValueByFrom ();
    void                      UpdateValue ();

  private:
    void                      Construct (QString     label
                                        ,sPhysRange  value
                                        ,bool  units_are_editable = false
                                        ,bool  precision_depends_on_value_digits
                                                                        = true
                                        );
    bool                      Validate (rsPhysRange);

  private: //fields
    QString                   TheLabel;
    psPhysRange               TheValue;
    bool                      UnitsAreEditableFlag;
    bool                      PrecisionDependsOnValueDigitsFlag;
    bool                      UnsetAttentionOnEditFlag;
    psPhysRange               ValidRange;
    tValidationProc *         ValidationProc;
    QLabel *                  LblLabel;
    psSpinDigitField          FldFrom;
    QLabel *                  LblFromTo;
    psSpinDigitField          FldTo;
    QPushButton *             BtnOrder;
    QLabel *                  LblOrder;
    psFlexibleField           FldOrder;
    QLabel *                  LblUnits;
    QPushButton *             BtnUnits;
    psFlexibleField           FldUnits;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
