////////////////////////////////////////////////////////////////////////////////
/*! @file PhysValueField.cpp   Виджет-редактор физвеличины.
- Part of RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2010-2013, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "PhysValueField.h"
#include "SymbolsGUI.h"
#include "Trace.h"
#include <QIntValidator>
namespace RA {
//------------------------------------------------------------------------------
//---------------------------------------------------------- sPhysValueField ---
/*! @class sPhysValueField

@bug не избавиться от неопределённости в погрешности.

@bug странное поведение, если уходить кнопкой табуляции.

@todo{UI++} Отдельные Set___ для всех опций с возвратом this.

@todo{UI++} Хорошо бы запрещать вывод порядка (и кнопки), чтобы Gain=20.000е3
            показывалось как Gain=20000.

@todo{UI++} И/или индивидуальные (для поля) настройки показа метрических префиксов,
            чтобы .

*/

sPhysValueField::~sPhysValueField ()
{
  if( TheValue ){ delete TheValue;  TheValue = NULL; };
}


void  sPhysValueField::Construct (QString    label
                                 ,sPhysValue value
                                 ,bool       units_are_editable
                                 ,bool       error_is_visible
                                 ,bool       error_depends_on_value_digits
                                 )
{
  TheValue = new sPhysValue(value);
  UnitsAreEditableFlag = units_are_editable;
  PrecisionIsVisibleFlag = error_is_visible;
  PrecisionDependsOnValueDigitsFlag = error_depends_on_value_digits;
  TheAttentionFlag = false;
  UnsetAttentionOnEditFlag = true;
  ValidRange = NULL;
  ValidationProc = NULL;
  BtnError = NULL;
  FldError = NULL;

  QHBoxLayout *       Layout = new QHBoxLayout(this);
  Layout->setMargin(0);
  Layout->setSpacing(0);
  Layout->setAlignment(Qt::AlignTop);

  LblLabel = new QLabel(this);
  SetLabel(label);
  Layout->addWidget(LblLabel);

  FldValue = new sSpinDigitField(this);
  QDoubleValidator *          Validator = new QDoubleValidator(FldValue);
  Validator->setNotation(QDoubleValidator::StandardNotation);
  FldValue->setValidator(Validator);
  Layout->addWidget(FldValue);

  connect(FldValue,SIGNAL(GotFocus()),this,SLOT(UpdateVisibility()));
  if( PrecisionIsVisibleFlag && PrecisionDependsOnValueDigitsFlag )
    connect(FldValue,SIGNAL(textChanged(QString))
           ,this    ,SLOT(UpdateErrorField(QString)));
  connect(FldValue,SIGNAL(EnterKeyPressed()),this,SLOT(UpdateValue()));
  connect(FldValue,SIGNAL(ChangedBySpinning()),this,SLOT(UpdateValue()));
  connect(FldValue,SIGNAL(LostFocus()),this,SLOT(PossiblyLeaving()));

  if( PrecisionIsVisibleFlag ){
    BtnError = new QPushButton(QString::fromUtf8("±"),this);
    BtnError->setFocusPolicy(Qt::ClickFocus);
    BtnError->setMaximumWidth(12);
    BtnError->setCheckable(true);
    Layout->addWidget(BtnError);

    FldError = new sSpinDigitField(this);
    FldError->setFocusPolicy(Qt::ClickFocus);
    FldError->setEnabled( ! PrecisionDependsOnValueDigitsFlag );
    Layout->addWidget(FldError);

    connect(BtnError,SIGNAL(toggled(bool)),this,SLOT(ShowError(bool)));
    connect(FldError,SIGNAL(EnterKeyPressed()),this,SLOT(UpdateValue()));
    connect(FldError,SIGNAL(ChangedBySpinning()),this,SLOT(UpdateValue()));
    connect(FldError,SIGNAL(LostFocus()),this,SLOT(PossiblyLeaving()));
  };

  BtnOrder = new QPushButton(QString::fromUtf8(Symbol::Multiplication),this);
  BtnOrder->setFocusPolicy(Qt::ClickFocus);
  BtnOrder->setMaximumWidth(12);
  Layout->addWidget(BtnOrder);
  BtnOrder->setVisible(false);

  connect(BtnOrder,SIGNAL(clicked()),this,SLOT(ShowOrder()));

  LblOrder = new QLabel(QString::fromUtf8(Symbol::Multiplication)+"10",this);
  Layout->addWidget(LblOrder);

  FldOrder = new sFlexibleField(this);
  FldOrder->setFocusPolicy(Qt::ClickFocus);
  FldOrder->setText("0");
  FldOrder->setValidator(new QIntValidator(-30,30,FldOrder));
  FldOrder->setMinimumHeight(FldOrder->height() / 2 + 1);
  FldOrder->setMaximumHeight(FldOrder->minimumHeight());
  Layout->addWidget(FldOrder,0,Qt::AlignTop);

  connect(FldOrder,SIGNAL(EnterKeyPressed()),this,SLOT(UpdateValue()));
  connect(FldOrder,SIGNAL(LostFocus()),this,SLOT(PossiblyLeaving()));

  LblUnits = new QLabel(this);
  Layout->addWidget(LblUnits);
  LblUnits->setVisible( ! UnitsAreEditableFlag );

  BtnUnits = new QPushButton("[]",this);
  BtnUnits->setFocusPolicy(Qt::ClickFocus);
  BtnUnits->setMaximumWidth(16);
  Layout->addWidget(BtnUnits);
  BtnUnits->setVisible(false);

  connect(BtnUnits,SIGNAL(clicked()),this,SLOT(ShowUnits()));

  FldUnits = new sFlexibleField(this);
  FldUnits->setFocusPolicy(Qt::ClickFocus);
  FldUnits->setText(" ");
  Layout->addWidget(FldUnits);
  FldUnits->setVisible( UnitsAreEditableFlag );

  connect(FldUnits,SIGNAL(EnterKeyPressed()),this,SLOT(UpdateValue()));
  connect(FldUnits,SIGNAL(LostFocus()),this,SLOT(PossiblyLeaving()));

  TheValue->Normalize();
  UpdateImage();
}


psPhysValueField  sPhysValueField::SetLabel (QString  label)
{
  TheLabel = label;
  if( ! label.isEmpty() )
    label = label + "=";
  LblLabel->setVisible( ! label.isEmpty() );
  LblLabel->setText(label);
  return this;
}


psPhysValueField  sPhysValueField::SetUnitsEditable (bool  editable)
{
  UnitsAreEditableFlag = editable;
  LblUnits->setVisible( ! UnitsAreEditableFlag );
  FldUnits->setVisible( UnitsAreEditableFlag );
  return this;
}


psPhysValueField  sPhysValueField::SetPrecisionVisible (bool  /*visible*/)
{
  //!@patch
  //PrecisionIsVisibleFlag=visible;
  return this;
}


psPhysValueField  sPhysValueField::SetPrecisionDependentOnValueDigits
                                      (bool  /*dependent*/)
{
  //!@patch
  //PrecisionDependsOnValueDigitsFlag=dependent;
  return this;
}


psPhysValueField  sPhysValueField::SetReadOnly (bool  read_only)
{
  FldValue->setReadOnly(read_only);
//  FldValue->setStyleSheet(( read_only ? "color: maroon" : "" ));
  return this;
}


psPhysValueField  sPhysValueField::SetAttention (bool  attentive)
{
  TheAttentionFlag = attentive;
  LblLabel->setStyleSheet(( attentive ? "color: red" : "" ));
  return this;
}


bool  sPhysValueField::Active () const
{
  return FldValue->hasFocus()
      || (BtnError && BtnError->hasFocus())
      || (FldError && FldError->hasFocus())
      || BtnOrder->hasFocus()
      || FldOrder->hasFocus()
      || BtnUnits->hasFocus()
      || FldUnits->hasFocus();
}


void  sPhysValueField::PossiblyLeaving ()
{
  if( ! Active() ){
//    TRACE(TheLabel+" leaving");
    UpdateValue();
    emit LostFocus();
  };
}


void  sPhysValueField::UpdateImage ()
{
//  TRACE(TheLabel+" UpdateImage");
  /*! @bug не всегда своевременно обновляются ширины полей значения и ошибки */
  /*! @bug не всегда своевременно обновляются число цифр для поля значения */
  unsigned short int          D = TheValue->Digits();
  if( FldError && FldError->isVisible() && ! PrecisionDependsOnValueDigitsFlag ){
    ++D;
  };
  FldValue->setText( ToQString(
                          sString::FromReal(TheValue->ValueMantissa(),D)   ));
  if( FldError && ! PrecisionDependsOnValueDigitsFlag ){
    FldError->setText(ToQString(
                        sString::FromReal(TheValue->ErrorMantissa()
                                          ,TheValue->Digits()+1)       ));
  };
  FldOrder->setText( QString().setNum( TheValue->Order() ) );
  FldUnits->setText(ToQString(TheValue->Units().Text(Plain)));
  LblUnits->setText(ToQString(TheValue->Units().Text(HTML)));
  UpdateVisibility();
}


void  sPhysValueField::UpdateVisibility ()
{
//  TRACE(TheLabel+" UpdateVisibility");
  bool                        active = Active();
  if( FldError ){
    FldError->setVisible(BtnError->isChecked());
    BtnError->setFlat(BtnError->isChecked());
    BtnError->setVisible( active || BtnError->isChecked() );
  };
  if( FldOrder->text().simplified().isEmpty()
      || (FldOrder->text().simplified() == "0") ){
    BtnOrder->setVisible(active);
    LblOrder->setVisible(false);
    FldOrder->setVisible(false);
  }else{
    BtnOrder->setVisible(false);
    LblOrder->setVisible(true);
    if( active ){
      LblOrder->setText( QString::fromUtf8(Symbol::Multiplication)+"10" );
    }else{
      LblOrder->setText( QString::fromUtf8(Symbol::Multiplication)
                       + "10<sup>"
                       + FldOrder->text()
                       + "</sup>" );
    };
    FldOrder->setVisible(active);
  };
  if( UnitsAreEditableFlag ){
    if( FldUnits->text().simplified().isEmpty() ){
      BtnUnits->setVisible(active);
      FldUnits->setVisible(false);
      LblUnits->setVisible(false);
    }else{
      BtnUnits->setVisible(false);
      FldUnits->setVisible(active);
      LblUnits->setVisible(!active);
    };
  }else{
    BtnUnits->setVisible(false);
    FldUnits->setVisible(false);
    LblUnits->setVisible( ! FldUnits->text().simplified().isEmpty() );
  };
//  TRACE(UnitsAreEditable);
//  TRACE(FldUnits->isVisible());
//  TRACE(LblUnits->isVisible());
//  TRACE(FromQString(FldUnits->text()));
//  TRACE(FromQString(LblUnits->text()));
  adjustSize();
}


void  sPhysValueField::ShowError (bool  show)
{
  BtnError->setFlat(show);
  FldError->setVisible(show);
  if( show )
    FldError->setFocus();
  adjustSize();
}


void  sPhysValueField::ShowOrder ()
{
  LblOrder->setVisible(true);
  FldOrder->setVisible(true);
  FldOrder->setFocus();
  BtnOrder->setVisible(false);
  adjustSize();
}


void  sPhysValueField::ShowUnits ()
{
  FldUnits->setVisible(true);
  FldUnits->setFocus();
  BtnUnits->setVisible(false);
  LblUnits->setVisible(false);
  adjustSize();
}


void  sPhysValueField::UpdateErrorField (QString  value_text)
{
//  TRACE(TheLabel+" UpdateErrorField");
  try{
    real                        V = 0.0;
    real                        E = 0.0;
    sString                     VT;
    size_t                      DotPos;
    unsigned short int          D = 1;
    VT=FromQString(value_text.simplified());
    if( VT.Empty() ){
      FldError->setText("");
    }else{
      sString(VT) >> V;
      if( isinf(V) || isnan(V) ){
        VT=sString::FromReal(std::numeric_limits<real>::quiet_NaN(),0);
        FldError->setText(ToQString(VT));
      }else{
        VT=VT.Translate('1','0').Translate('2','0').Translate('3','0')
             .Translate('4','0').Translate('5','0').Translate('6','0')
             .Translate('7','0').Translate('8','0').Translate('9','0')
             .Translate('-',' ').Translate('+',' ');
        DotPos=VT.Pos(".");
        if( DotPos < VT.Length() ){
          D=VT.Length()-DotPos-1;
        }else{
          VT+=".";
        };
        VT+="5";
        VT >> E;
        FldError->setText(ToQString( sString::FromReal(E,D) ));
      };
    };
  }catch(...){
  };
}


void  sPhysValueField::UpdateValue ()
{
//  TRACE(TheLabel+" UpdateValue");
  try{
    real                        V = TheValue->ValueMantissa();
    real                        E = TheValue->ErrorMantissa();
    int                         O = 0;
    sUnits                      U;
    sString                     VT;
    VT=FromQString(FldValue->text().simplified());
    if( ! VT.Empty() ){
      sString(VT) >> V;
    };
    if( isinf(V) || isnan(V) ){
      E = std::numeric_limits<real>::quiet_NaN();
    }else{
      if( PrecisionDependsOnValueDigitsFlag ){
        VT=VT.Translate('1','0').Translate('2','0').Translate('3','0')
             .Translate('4','0').Translate('5','0').Translate('6','0')
             .Translate('7','0').Translate('8','0').Translate('9','0');
        if( VT.Pos(".") == VT.Length() )
          VT+=".";
        VT+="5";
        VT >> E;
      };
      if( ! PrecisionDependsOnValueDigitsFlag && ! isnan(E) && FldError )
        if( ! FldError->text().simplified().isEmpty() )
          FromQString(FldError->text()) >> E;
      if( ! FldOrder->text().simplified().isEmpty() )
        FromQString(FldOrder->text()) >> O;
    };
    if( UnitsAreEditableFlag ){
      U = sUnits::Parse(FromQString(FldUnits->text()));
    }else{
      U = TheValue->Units();
    };
    psPhysValue                 R = new sPhysValue(V*power10(O),E*power10(O),U);
    if( Validate(*R) ){
      delete TheValue;
      TheValue = R;
      TheValue->Normalize();
    }else{
      delete R;
    };
    /*! @patch emit Edited signal IF NECESSARY */
    emit Edited();
  }catch(.../*EConvertError& E*/){
    TRACE("CONVERSION ERROR");
  };
  UpdateImage();
}


sPhysValue  sPhysValueField::Value ()
{
  ASSERT(TheValue);
  return *TheValue;
}


psPhysValueField  sPhysValueField::SetValue (sPhysValue  R)
{
  if( TheValue ) delete TheValue;
  TheValue = new sPhysValue(R);
  TheValue->Normalize();
  UpdateImage();
  return this;
}


psPhysValueField  sPhysValueField::SetValidRange (sPhysRange  range)
{
  if( ValidRange )
    delete ValidRange;
  ValidRange = new sPhysRange(range);
  if( ValidRange ){
    UnitsAreEditableFlag=false;
    try{
      if( (*TheValue) < ValidRange->From() )
        (*TheValue) = ValidRange->From();
      if( ValidRange->To() < (*TheValue) )
        (*TheValue) = ValidRange->To();
    }catch(...){
      delete TheValue;
      TheValue = new sPhysValue(ValidRange->From());
      TheValue->Normalize();
    };
  };
  UpdateImage();
  return this;
}


void  sPhysValueField::SetValidationProc (tValidationProc *  proc)
{
  ValidationProc=proc;
}


bool  sPhysValueField::Validate (rsPhysValue  R)
{
  try{
    if( ValidRange ){
      if( R < ValidRange->From() ){
        R = ValidRange->From();
      }else if( ValidRange->To() < R ){
        R = ValidRange->To();
      };
    };
    if( ValidationProc ){
      return (*ValidationProc)(R);
    }else{
      return true;
    };
  }catch(...){
    return false;
  };
}

//------------------------------------------------------------------------------
} //namespace RA


