////////////////////////////////////////////////////////////////////////////////
/*! @file PhysValueField.h   Виджет-редактор физвеличины.
- Part of RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2010-2013, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef PhysValueField_H
#define PhysValueField_H
#include "FlexibleFields.h"
#include "PhysValue.h"
#include "Ranges.h"
#include "StringsAndQStrings.h"
#include <QWidget>
#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QKeyEvent>
#include <QContextMenuEvent>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sPhysValueField)

//---------------------------------------------------------- sPhysValueField ---
/*! Виджет-редактор физвеличины.

*/

class RAGUI_EXPORT  sPhysValueField : public QWidget
{
  Q_OBJECT
  public://type
    typedef             bool  tValidationProc (rsPhysValue);
                                //!< may change its argument
                                //!  returns true on success
  public:
    virtual                   ~sPhysValueField ();
                              sPhysValueField (QWidget *   parent
                                              ,QString     label
                                              ,sPhysValue  value
                                            ,bool  units_are_editable = false
                                            ,bool  error_is_visible  = false
                                            ,bool  error_depends_on_value_digits
                                                                         = true)
                                  :QWidget(parent)
                                {
                                  Construct(label,value,units_are_editable
                                           ,error_is_visible
                                           ,error_depends_on_value_digits);
                                }
                              sPhysValueField (QWidget *   parent
                                              ,sPhysValue  value
                                            ,bool  units_are_editable = false
                                            ,bool  error_is_visible  = false
                                            ,bool  error_depends_on_value_digits
                                                                         = true)
                                  :QWidget(parent)
                                {
                                  Construct(QString(),value,units_are_editable
                                           ,error_is_visible
                                           ,error_depends_on_value_digits);
                                }
    QString                   Label () const
                                { return TheLabel; }
    sPhysValue                Value ();
    bool                      ReadOnly ()
                                { return FldValue->ReadOnly(); }
    bool                      HasAttentionFlag () const
                                { return TheAttentionFlag; }
    bool                      UnitsAreEditable ()
                                { return UnitsAreEditableFlag; }
    bool                      PrecisionIsVisible ()
                                { return PrecisionIsVisibleFlag; }
    bool                      PrecisionDependsOnValueDigits ()
                                { return PrecisionDependsOnValueDigitsFlag; }
    void                      SetValidationProc (tValidationProc *);
    bool                      Active () const;

  signals:
    void                      LostFocus ();
    void                      Edited ();

  public slots:
    virtual psPhysValueField  SetLabel (QString);
    virtual psPhysValueField  SetUnitsEditable (bool);
    virtual psPhysValueField  SetPrecisionVisible (bool);
    virtual psPhysValueField  SetPrecisionDependentOnValueDigits (bool);
    virtual psPhysValueField  SetReadOnly (bool);
    virtual psPhysValueField  SetReadOnly ()
                                { return SetReadOnly(true); }
    virtual psPhysValueField  UnsetReadOnly ()
                                { return SetReadOnly(false); }
    virtual psPhysValueField  SetAttention (bool);
    virtual psPhysValueField  SetAttention ()
                                { return SetAttention(true); }
    virtual psPhysValueField  UnsetAttention ()
                                { return SetAttention(false); }
    virtual psPhysValueField  UnsetAttentionOnEdit (bool  unset = true)
                                { UnsetAttentionOnEditFlag=unset; return this; }
    virtual psPhysValueField  SetValidRange (sPhysRange);
    virtual psPhysValueField  SetValue (sPhysValue);

  private slots:
    void                      PossiblyLeaving ();
    void                      UpdateImage ();
    void                      UpdateVisibility ();
    void                      ShowError (bool);
    void                      ShowOrder ();
    void                      ShowUnits ();
    void                      UpdateErrorField (QString);
    void                      UpdateValue ();

  private:
    void                      Construct (QString     label
                                        ,sPhysValue  value
                                        ,bool     units_are_editable = false
                                        ,bool     error_is_visible  = false
                                        ,bool     error_depends_on_value_digits
                                                                        = true
                                        );
    bool                      Validate (rsPhysValue);

  private: //fields
    bool                      TheAttentionFlag;
    QString                   TheLabel;
    psPhysValue               TheValue;
    bool                      UnitsAreEditableFlag;
    bool                      PrecisionIsVisibleFlag;
    bool                      PrecisionDependsOnValueDigitsFlag;
    bool                      UnsetAttentionOnEditFlag;
    psPhysRange               ValidRange;
    tValidationProc *         ValidationProc;
    QLabel *                  LblLabel;
    psSpinDigitField          FldValue;
    QPushButton *             BtnError;
    psSpinDigitField          FldError;
    QPushButton *             BtnOrder;
    QLabel *                  LblOrder;
    psFlexibleField           FldOrder;
    QLabel *                  LblUnits;
    QPushButton *             BtnUnits;
    psFlexibleField           FldUnits;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
