////////////////////////////////////////////////////////////////////////////////
/*! @file Plot.h   A child window class sPlot.
- Part of RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2010-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          VBursian@gmail.com
*///////////////////////////////////////////////////////////////////////////////
#ifndef Plot_H
#define Plot_H
#include "Window.h"
#include "Navigator.h"
#include "Chart.h"
#include <QTimer>
#include <QLabel>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QSplitter>
#include <QPrinter>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sPlot)

//-------------------------------------------------------------------- sPlot ---

class sPlot : public sWindow
{
  Q_OBJECT
  public:
    virtual                   ~sPlot ();
    explicit                  sPlot (sNodePtr   config
                                    ,sString    default_title
                                    ,QWidget *  parent       );
    virtual const QIcon &     Icon ();
    virtual void              RestoreState ();
    virtual void              StoreState ();

  public slots:
    void                      PutLegendAtTheBottom (bool);
    void                      PrintPreview ();
    void                      Print (QPrinter *);
//    void                      RescaleToShowAll ();
//    void                      ZoomOut ();
//    void                      PreviousScale ();

  signals:
    void                      CursorPositionChanged (sPhysPair  position
                                                    ,sPhysPair  reference
                                                    ,sString    x_title
                                                    ,sString    y_title);

  protected: //inherited
    virtual void              focusInEvent (QFocusEvent *);
    virtual void              focusOutEvent (QFocusEvent *);
    virtual void              contextMenuEvent (QContextMenuEvent *);
    virtual void              Retranslate ()
                                {
                                  sWindow::Retranslate();
                                  Translate();
                                }

  private:
    void                      Translate ();

  private: //fields
    psNet2TreeEvolution       Evolution;
    psChart                   Chart;
    psNavigator               Legend;
    QTimer *                  UpdatingTimer;
    QLayout *                 Layout;
    QSplitter *               Splitter;
//    QMenu *                   MenuExtension;
//    QAction *                 PA;
    QAction *                 ActnLegendBottom;
    QAction *                 ActnLegendRight;
    QAction *                 ActnPrint;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
