////////////////////////////////////////////////////////////////////////////////
/*! @file RADock.cpp A dock-style envelope for the child windows in RA.
- Part of RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2010-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "RADock.h"
#include "StringsAndQStrings.h"
#include "attr.h"
#include "Trace.h"
#include "Log.h"
namespace RA {
//------------------------------------------------------------------------------
//------------------------------------------------------------------ sRADock ---

sRADock::~sRADock ()
{
  if( Subwidget ){ delete Subwidget;  Subwidget=NULL; }
}


sRADock::sRADock (QMainWindow *  Parent
                 ,sString        ObjectName4Qt
                 ,sNodePtr       Config
                 ,psWindow       Widget)
    :QDockWidget(Parent)
    ,QuittingStarted(false)
    ,QuittingAllowed(false)
    ,Subwidget(Widget)
{
  setObjectName(ToQString(ObjectName4Qt));
  setWindowTitle(ToQString(Subwidget->Title()));
  setWindowIcon(Widget->Icon());
  setAttribute(Qt::WA_DeleteOnClose);
  setWidget(Subwidget);
  Subwidget->setParent(this);
  Subwidget->setObjectName("Subwidget");
  Config |= attr("DockArea",int(Qt::RightDockWidgetArea));
  Qt::DockWidgetArea          DockArea;
  DockArea=Qt::DockWidgetArea(int(rsIntegerNode( * (Config>>"DockArea") )));
  Parent->addDockWidget(DockArea,this);
  connect(Subwidget,SIGNAL(TitleChanged()),this,SLOT(UpdateTitle()));
}


void sRADock::UpdateTitle ()
{
  setWindowTitle(ToQString(Subwidget->Title()));
}


void  sRADock::StartQuitting ()
{
  RANet::Log.Put(sLog::Debug,"Quitting"
      ,sString("sRADock::StartQuitting: ")+Subwidget->Title());
  QuittingStarted = true;
  connect(Subwidget,SIGNAL(QuittingIsAllowed())
         ,this,SLOT(FinishQuitting())
         ,Qt::QueuedConnection);
  Subwidget->DoClose();
}


void  sRADock::FinishQuitting ()
{
  if( QuittingStarted ){
    RANet::Log.Put(sLog::Debug,"Quitting"
        ,sString("sRADock::FinishQuitting: ")+Subwidget->Title());
    QuittingAllowed = true;
    close();
  }
}


void  sRADock::closeEvent (QCloseEvent *  event)
{
  if( QuittingAllowed ){
    RANet::Log.Put(sLog::Debug,"Quitting"
        ,sString("sRADock::closeEvent accepted: ")+Subwidget->Title());
    event->accept();
  }else if( ! QuittingStarted ){
    if( Subwidget->CanClose() ){
      setEnabled(false);
      Subwidget->Predestruct();
      StartQuitting();
    }else{
      Subwidget->DonotClose();
    }
    event->ignore();
  }else{
    RANet::Log.Put(sLog::Debug,"Quitting"
        ,sString("sRADock::closeEvent ignored: ")+Subwidget->Title());
    event->ignore();
  }
}


void  sRADock::focusInEvent (QFocusEvent *  event)
{
  sString sRADock_focusInEvent(Subwidget->TheTitle);
  TRACE(sRADock_focusInEvent);
  QDockWidget::focusInEvent(event);
  
  //!@todo{UI++} see QApplication::focusChanged signal and QApplication::focusWidget()
}


void  sRADock::focusOutEvent (QFocusEvent *  event)
{
  sString sRADock_focusOutEvent(Subwidget->TheTitle);
  TRACE(sRADock_focusOutEvent);
  QDockWidget::focusOutEvent(event);
}

//------------------------------------------------------------------------------
} //namespace RA
