////////////////////////////////////////////////////////////////////////////////
/*! @file RADock.h   A dock-style envelope for the child windows in RA.
- Part of RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2010-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef RADock_H
#define RADock_H
#include "Strings.h"
#include "Window.h"
#include <QDockWidget>
#include <QMainWindow>
#include <QFocusEvent>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sRADock)

//------------------------------------------------------------------ sRADock ---
/*! Provides common appearance and behaviour of the child windows of RA.

@todo{UI++} see examples/widgets/mainwindows/mainwindow
(be aware: setting a custom title bar works strange for the case when the dock
is outside the main window).
*/
class sRADock : public QDockWidget
{
  Q_OBJECT
  public:
    virtual                   ~sRADock ();
    explicit                  sRADock (QMainWindow *  Parent
                                      ,sString        ObjectName4Qt
                                      ,sNodePtr       Config
                                      ,psWindow       Widget);
    psWindow                  Window () const
                                { return Subwidget; }

  public slots:
    void                      StartQuitting ();
    void                      FinishQuitting ();

  private slots:
    void                      UpdateTitle ();

  protected: //inherited
    virtual void              closeEvent (QCloseEvent *);
    virtual void              focusInEvent (QFocusEvent *);
    virtual void              focusOutEvent (QFocusEvent *);

  private: //fields
    bool                      QuittingStarted;
    bool                      QuittingAllowed;
    psWindow                  Subwidget;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
