////////////////////////////////////////////////////////////////////////////////
/*! @file RAMainWindow.h   Declares common main window class
                           of the Research Assistant program.
- Part of RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2010-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef RAMainWindow_H
#define RAMainWindow_H
#include "RAGUI_global.h"
#include "Environment.h"
#include "NodePtr.h"
#include "FolderNode.h"
#include "BasicNodes.h"
#include "Graphicals.h"
#include "StringsAndQStrings.h"
#include "RAPlugins.h"
#include "Globals.h"
#include "ActionSignallingText.h"
#include "ByeScreen.h"
#include <list>
#include <QMainWindow>
#include <QTranslator>
#include <QFocusEvent>
#include <QAction>
#include <QActionGroup>
#include <QMenu>
#include <QLabel>
#include <QToolBar>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sRAMainWindow)

//------------------------------------------------------------ sRAMainWindow ---
/*! Главное окно.
*/
class RAGUI_EXPORT  sRAMainWindow : public QMainWindow
{
  Q_OBJECT
  public://static const
    static csAttrName         ChildrenAttrName;
    static csAttrName         UserAccessLevelAttrName;
    static csAttrName         NightRegimeAttrName;
    static csAttrName         DockNestingAttrName;
    static csAttrName         DesktopGeometryAttrName;
    static csAttrName         DesktopStateAttrName;

  public:
    virtual                   ~sRAMainWindow ();
    explicit                  sRAMainWindow (sNodePtr     root
                                            ,psByeScreen  bye_screen = NULL);

  signals:
    void                      QuittingIsAllowed ();
    void                      WindowOpened ();

  public slots:
    void                      UpdateMenuInstrument ();
    void                      UpdateMenuInstallNewDevice ();
    void                      UpdateMenuConfigureDevice ();
    void                      RestoreLayout ();
    void                      StoreLayout ();
    void                      NewWorkbook ();
//    void                      NewWorkbook (sNodePtr  Content);
    void                      NewPlot ();
    void                      OpenInstrument (QString Id);
    void                      OpenConsole (QString Id);
    void                      InstallNewDevice (QString ClassName);
    void                      ConfigureDevice (QString Id);
    void                      OpenData ();
    void                      OpenRecords ();
    void                      OpenSamples ();
    void                      OpenTopics ();
    void                      ShowHelpContents ();
    void                      CreateWindow (sNodePtr  Cfg);
    void                      ShowCursorInfo (sPhysPair  position
                                             ,sPhysPair  reference
                                             ,sString    x_title
                                             ,sString    y_title);

  public:
//    QToolBar *                ToolBarPlot;

  protected://inherited
    virtual void              showEvent (QShowEvent *) override;
    virtual void              changeEvent (QEvent *) override;
    virtual void              closeEvent (QCloseEvent *) override;
    virtual void              focusInEvent (QFocusEvent *) override;
    virtual void              focusOutEvent (QFocusEvent *) override;

  private:
    virtual void              Retranslate ();
    bool                      SetMainDataPtrs ();
    void                      SetLanguage (QString locale_name = QString());
    void                      RestoreDesktop ();
    void                      StoreDesktop ();
    void                      RestoreWindows (sNodePtr  SetOfCfg);
    void                      ClearClosedWindows (sNodePtr  SetOfCfg);
    sString                   GetObjectName4Qt (sNodePtr   SetOfCfg
                                               ,sNodePtr   Cfg
                                               ,rcsString  Type);

    void                      LoadPlugins ();
    void                      CreateActions ();
    void                      CreateMenus ();
    void                      CreateToolBars ();
    void                      CreateStatusBar ();
    void                      InstallPluginDefaults (psInstrumentalPackage);
    void                      FillMenuLanguage ();

  private slots:
    void                      CompleteConstruction ();
    void                      ChangeLanguage (QString);
    void                      ChangeUser (QString);
    void                      ChangeUserAccessLevel (sUser::eAccessLevel);
    void                      SetNightRegime (bool  night = false);
    void                      ChangeBasicLineColors ();
    void                      ChangeFont ();
    void                      About ();
    void                      AboutPlugin (QString AboutText);
    void                      CheckClosingChildren ();
    void                      FinishQuitting ();

  private: //fields
    sPtr<sFolderNode>         TheRoot;
    bool                      SuccessfullyConstructed;
    bool                      QuittingStarted;
    bool                      QuittingAllowed;
    psByeScreen               ByeScreen;
    QTimer *                  ClosingTimer;
    sPtr<sFolderNode>         Desktop;
    sPtr<sFolderNode>         DataToBrowse;
    sPtr<sFolderNode>         Plugins;
    sPtr<sTextNode>           FontFamily;
    sPtr<sIntegerNode>        FontSize;
    sPtr<sFolderNode>         Styles;
    sPtr<sTextNode>           Language;
    sPtr<sFolderNode>         UnitSystem;
    sPtr<sFolderNode>         RecordsToBrowse;
    sPtr<sFolderNode>         SamplesToBrowse;
    sPtr<sFolderNode>         TopicsToBrowse;
    sPtr<sFolderNode>         Children;
    std::list
      <psInstrumentalPackage> LoadedPackages;
//    bool                      IgnoreLanguageChangeEvent;

  private: //fields
    QTranslator *             QtTranslator;
    QTranslator *             Translator;
    QWidget *                 CentralWidget;
//    QSignalMapper *           windowMapper;
//    QAction *                 ActnNewWorkbook;
    QAction *                 ActnNewPlot;
    QAction *                 ActnOpenTopics;
    QAction *                 ActnOpenSamples;
    QAction *                 ActnOpenRecords;
    QAction *                 ActnOpenData;
//    QAction *                 ActnPrint;
    QAction *                 ActnExit;
//    QAction *                 ActnUndo;
//    QAction *                 ActnRedo;
//    QAction *                 ActnCut;
//    QAction *                 ActnCopy;
//    QAction *                 ActnPaste;
    psActionSignallingText    ActnNewbieLevel;
    psActionSignallingText    ActnExpertLevel;
    psActionSignallingText    ActnAdminLevel;
    QAction *                 ActnSetNightRegime;
    QAction *                 ActnSetDockNesting;
    QAction *                 ActnChangeBasicLineColors;
    QAction *                 ActnChangeFont;
    QAction *                 ActnHelpContents;
    QAction *                 ActnAbout;
    QAction *                 ActnAboutQt;
    QActionGroup *            WindowListActions;
//    QMenuBar *                menubar;
    QMenu *                   MenuFile;
//    QMenu *                   MenuEdit;
//    QMenu *                   MenuWorkbook;
//    QMenu *                   MenuPlot;
    QMenu *                   MenuInstrument;
    QMenu *                   MenuSettings;
    QMenu *                   MenuLanguage;
    QMenu *                   MenuUser;
    QMenu *                   MenuInstallNewDevice;
    QMenu *                   MenuInstallNewDriver;
    QMenu *                   MenuInstallNewConsole;
    QMenu *                   MenuInstallNewGadget;
    QMenu *                   MenuConfigureDevice;
//    QMenu *                   MenuWindow;
    QMenu *                   MenuHelp;
    QMenu *                   MenuAboutPlugins;
    QLabel *                  CursorStatusX;
    QLabel *                  CursorStatusDX;
    QLabel *                  CursorStatusY;
    QLabel *                  CursorStatusDY;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
