////////////////////////////////////////////////////////////////////////////////
/*! @file RecordUI.cpp   Функционал навигатора, специфичный для записей.
- Part of RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "RecordUI.h"
#include "Record.h"
#include "Navigator.h"
#include "StringsAndQStrings.h"
#include <QInputDialog>
#include <QMessageBox>
namespace RA {
//------------------------------------------------------------------------------
//---------------------------------------------------------------- sRecordUI ---

sRecordUI::sRecordUI()
{
  sNavigatorViewport::MenuGenerators.push_front(&MenuGenerator);
}


void  sRecordUI::MenuGenerator (psNetViewMenu                context_menu
                               ,sNet2TreeEvolution::pcsItem  tree_item)
{
  if( sRecord::Type()->CanCast(tree_item->AttrValue().operator ->()) )
    context_menu->Insert( new sNetViewAction
        (context_menu->parentWidget()
        ,&Disregard  ,/*checkable*/false ,/*checked*/false
        ,sNetViewMenu::tr("Disregard record")
        ,sNetViewMenu::tr("Mark a mistaken record as disregarded")
        ));
}


void  sRecordUI::Disregard (sNet2TreeEvolution::pcsItem  tree_item
                           ,sPoint                       /*pos*/)
{
  bool OK;
  QString reason = QInputDialog::getText(NULL
      ,sNavigatorViewport::tr("Disregard record")
      ,sNavigatorViewport::tr(
          "<b><u>Note:</u></b><br>"
          "If the record is correct, but you consider it as not so important<br>"
          "and you do not wish to see it in this topic or plot,<br>"
          "then you should choose <b>'Delete this attribute'</b> option.<br><br>"
          "You should disregard the record only in the case,<br>"
          "when it is <b>erronous and can confuse you</b> in the future.<br><br>"
          "Please provide a reason:")
      ,QLineEdit::Normal ,QString() ,&OK);
  if( OK ){
    if( ! reason.isEmpty() ){
      sPtr<sRecord>             R(tree_item->AttrValue());
      ASSERT(R.IsCorrect());
      R->DisregardRecord(FromQString(reason));
    }else{
      QMessageBox::critical(NULL
          ,sNavigatorViewport::trUtf8("Я обиделась!")
          ,sNavigatorViewport::trUtf8(
              "Дорогой Юзер!<br><br>"
              "Я так стараюсь, делаю записи, храню их...<br>"
              "А ты даже не соизволишь объяснить, что с этой записью не так!<br>"
              "Я обиделась и не буду слушаться.<br><br>"
              "Больше не твоя,<br>"
              "Research Assistant."
          ));
    };
  };
}

//------------------------------------------------------------------------------
} //namespace RA
