////////////////////////////////////////////////////////////////////////////////
/*! @file Window.cpp   Прото-виджет для дочерних окон главного окна RA.
- Part of RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2010-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          VBursian@gmail.com
*///////////////////////////////////////////////////////////////////////////////
#include "Window.h"
#include "attr.h"
#include "Log.h"
namespace RA {
//------------------------------------------------------------------------------
//------------------------------------------------------------------ sWindow ---

sWindow::~sWindow ()
{
  RANet::Log.Put(sLog::Debug,"Quitting"
                ,sString("sWindow destructor: ")+TheTitle);
  WidgetConfig.SetToNULL();
}


sWindow::sWindow (sNodePtr   config
                 ,sString    default_title
                 ,QWidget *  parent       )
    :QFrame(parent)
    ,WidgetConfig(config)
{
  setObjectName("sWindow");
  sPtr<sTextNode>             T (WidgetConfig >> "Title");
  if( T.IsCorrect() ){
    TheTitle=T->Value();
    AutoTitled=false; //supposingly; can be true in Confiruration
    WidgetConfig |= attr("AutoTitled",AutoTitled);
  }else{
    TheTitle=default_title;
    WidgetConfig << attr("Title",TheTitle);
    AutoTitled=true;
    WidgetConfig << attr("AutoTitled",AutoTitled);
  }
  AutoTitled = bool(rsBooleanNode( * (WidgetConfig>>"AutoTitled") ));
//  setFocusPolicy(Qt::ClickFocus);
  setFocusPolicy(Qt::StrongFocus);
}


void  sWindow::RestoreState ()
{
  if( ! WidgetConfig.IsNULL() ){
  }
}


void  sWindow::StoreState ()
{
  if( ! WidgetConfig.IsNULL() ){
  }
}


psRAMainWindow  sWindow::MainWindow ()
{
  psRAMainWindow              W = NULL;
  if( parent() ){
    W = dynamic_cast<psRAMainWindow>( parent() );
    if( ! W )
      W = dynamic_cast<psRAMainWindow>( parent()->parent() );
  }
  return W;
}


bool  sWindow::CanClose ()
{
  RANet::Log.Put(sLog::Debug,"Quitting"
                ,sString("ClosingRequested: '") + Title() + "'");
  emit ClosingRequested();
  return true;
}


void  sWindow::DonotClose ()
{
  RANet::Log.Put(sLog::Debug,"Quitting"
                ,sString("ClosingCanceled: '") + Title() + "'");
  emit ClosingCanceled();
}


void  sWindow::DoClose ()
{
  RANet::Log.Put(sLog::Debug,"Quitting"
                ,sString("Closing: '") + Title() + "'");
  emit QuittingIsAllowed();
}


void  sWindow::changeEvent (QEvent *  e)
{
  QFrame::changeEvent(e);
  switch (e->type()) {
    case QEvent::LanguageChange:
        Retranslate();
        break;
    default:
        break;
  }
}


void  sWindow::focusInEvent (QFocusEvent *  event)
{
  RANet::Log.Put(sLog::Debug,"Focus"
                ,sString("sWindow::focusInEvent: ")+TheTitle);
  emit FocusReceived(true);
  QFrame::focusInEvent(event);
}


void  sWindow::focusOutEvent (QFocusEvent *  event)
{
  RANet::Log.Put(sLog::Debug,"Focus"
                ,sString("sWindow::focusOutEvent: ")+TheTitle);
  QFrame::focusOutEvent(event);
}


sString  sWindow::Title ()
{
  return TheTitle;
}


void  sWindow::Predestruct ()
{
//  sNet::Manager->Undo.GroupOpen();
  StoreState();
//  sNet::Manager->Undo.GroupFlush();
//  sNet::Manager->Undo.GroupOpen(sString("destroy window ")
//                               +Title()
//                               ,new sDummyRedoItem);
  WidgetConfig << attr("Destroyed",true);
//  sNet::Manager->Undo.Push(new sUndoDestroyWindow(MainWindow,WidgetConfig));
//  sNet::Manager->Undo.GroupClose();
}

//------------------------------------------------------- sUndoDestroyWindow ---

//void  sUndoDestroyWindow::UndoIt ()
//{
//  MainWindow->CreateWindow(Config);
//}

//------------------------------------------------------------------------------
} //namespace RA

