////////////////////////////////////////////////////////////////////////////////
/*! @file Workbook.cpp Рабочий журнал экспериментатора.
- Part of RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2010-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "Workbook.h"
#include "BasicNodes.h"
#include <QGridLayout>
namespace RA {
//------------------------------------------------------------------------------
//---------------------------------------------------------------- sWorkbook ---

sWorkbook::~sWorkbook()
{
  if( Navigator ){ delete Navigator;  Navigator=NULL; };
  if( Evolution ){ delete Evolution;  Evolution=NULL; };
}


sWorkbook::sWorkbook (sNodePtr   config
                     ,sString    default_title
                     ,QWidget *  parent       )
  :sWindow(config,default_title,parent)
{
  QGridLayout *               Layout;
  Layout = new QGridLayout(this);
  Layout->setMargin(0);
  sPtr<sFolderNode>           root(WidgetConfig>>"Content");
  ASSERT(root.IsCorrect());
  Evolution = new sNet2TreeEvolution(sAttribute(sAttrName(Title()),root));
  Navigator = new sNavigator(Evolution ,this);
  Layout->addWidget(Navigator, 0, 0, 1, 1);

  setMinimumSize(80,80);

  connect(Evolution,SIGNAL(Changed()),Navigator,SLOT(OnNetChange()));
  Evolution->Initiate();

  UpdatingTimer = new QTimer(this);
  connect(UpdatingTimer, SIGNAL(timeout()), Evolution, SLOT(Update()));
  UpdatingTimer->start(sNavigator::UpdatingPeriod);
}


const QIcon &  sWorkbook::Icon ()
{
  static QIcon *  MyIcon = new QIcon(":/RAGUI/icons/book.png");
  return *MyIcon;
}


void  sWorkbook::RestoreState ()
{
  sWindow::RestoreState();
  if( ! WidgetConfig.IsNULL() ){
  }
}


void  sWorkbook::StoreState ()
{
  sWindow::StoreState();
  if( ! WidgetConfig.IsNULL() ){
  }
}

//------------------------------------------------------------------------------
} //namespace RA

