////////////////////////////////////////////////////////////////////////////////
/*! @file CommonDist.cpp   Общие классы для создания и инсталяции дистрибутивов.
- Uses selected files from RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2011-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          VBursian@gmail.com
*///////////////////////////////////////////////////////////////////////////////
#include "CommonDist.h"
#include <QApplication>
#include <QMessageBox>
#include <QWizard>
#include <QVBoxLayout>
#include <QLabel>
#include <QAbstractButton>
#include <QTimer>
#include <QProcess>
#include <QDebug>
namespace RA {
//------------------------------------------------------------------------------

const QStringList   sApplicationJob::QtLibraries
                        ( QStringList()
                          << "Qt5Core.dll"          << "libQt5Core.so*"
                          << "Qt5Gui.dll"           << "libQt5Gui.so*"
                          << "Qt5Widgets.dll"       << "libQt5Widgets.so*"
                          << "icu*.dll"             //<< "libicu*"
//apt-get install libicu52?
                                                    << "libicui18n.so*"
                                                    << "libicuuc.so*"
                                                    << "libicudata.so*"
                        );

const QStringList   sApplicationJob::GNULibraries
                        ( QStringList()
                          //win32:
                          << "D3DCOMPILER*.dll"
                          << "LIBGCC_S_SJLJ-1.dll"
                          << "libgcc_s_dw2-1.dll"
                          << "LIBGLESV2.dll"
                          << "libstdc++-6.dll"
                          << "libwinpthread-1.dll"
                          //unix:
                          << "libstdc++.so*"
                          << "libgobject-2.0.so*"
                          << "libX11.so*"
                          << "libgthread-2.0.so*"
                          << "libffi.so*"
                          << "libxcb.so*"
                          << "libglapi.so*"
                          << "libXext.so*"
                          << "libXdamage.so*"
                          << "libXfixes.so*"
                          << "libX11-xcb.so*"
                          << "libxcb-glx.so*"
                          << "libxcb-dri2.so*"
                          << "libxcb-dri3.so*"
                          << "libxcb-present.so*"
                          << "libxcb-sync.so*"
                          << "libxshmfence.so*"
                          << "libXxf86vm.so*"
                          << "libdrm.so*"
                          << "libXau.so*"
                          << "libXdmcp.so*"
                          << "libGL.so*"
                          << "libm.so*"
                          << "libgcc_s.so*"
                          << "libc.so*"
                          << "libpthread.so*"
                          << "libglib-2.0.so*"
                          << "libdl.so*"
                          << "librt.so*"
                          << "libpcre.so*"
                        );

//-------------------------------------------------- sProjectInfo::sPartInfo ---

sProjectInfo::sPartInfo::sPartInfo ()
    :Version(-1,-1)
    ,Todays(false)
//    ,TranslationExists(false)
//    ,TranslationOK(false)
    ,Selected(false)
{
}


sProjectInfo::sPartInfo::sPartInfo (rcsVersion  version
                                   ,QString     user_friendly_name)
    :Version(version)
    ,UserFriendlyName(user_friendly_name)
    ,Todays(false)
//    ,TranslationExists(false)
//    ,TranslationOK(false)
    ,Selected(false)
{
}

//------------------------------------------------------------- sProjectInfo ---

sProjectInfo::sProjectInfo ()
    :Core(NULL)
{
}

//---------------------------------------------------------- sApplicationJob ---

sApplicationJob::sApplicationJob (bool  testing)
    :sProjectInfo()
    ,Testing(testing)
    ,AppDir(qApp->applicationDirPath())
    ,Report(NULL)
{
  #ifdef Q_OS_WIN32
    System = "win32";
  #else
    #ifdef Q_OS_UNIX
      System = "unix";
    #else
      System = "unknownsystem";
    #endif
  #endif
}


void  sApplicationJob::Out (QString  text)
{
  if( Report ){
    Report->insertPlainText(text);
    Report->moveCursor(QTextCursor::End);
    Report->repaint();
  }
}


bool  sApplicationJob::Execute (QString      prog
                               ,QStringList  args
                               ,QString      working_dir
                               ,int          time_out/*s*/)
{
  QProcess *                  Process = new QProcess;
  int                         t = 0;
  int                         ExitCode;
  Out("\n____________________________________________________________\n");
  Out(prog+"   "+args.join(" ")+"   (in "+working_dir+")\n\n");
  Process->setWorkingDirectory(working_dir);
  Process->start( prog , args );
  if( ! Process->waitForStarted() ){
    Out("*** ERROR *** cannot start");
    return false;
  }
  while( ! Process->waitForFinished(100) ){
    t++;
    Out(QString(Process->read(100)));
    if( (time_out > 0) && (t > 10*time_out) ){
      Out("*** ERROR *** timeout");
      return false;
    }
  }
  Out(Process->readAll());
  ExitCode = Process->exitCode();
  Out( QString("\n_______________________________________exit_code_")
      +QString().setNum(ExitCode)
      +QString("___\n") );
  return (ExitCode == 0);
}


bool  sApplicationJob::Zip (QString      zip_name
                           ,QStringList  files
                           ,QString      working_dir
                           ,int          time_out/*s*/)
{
  return Execute(ZipProgDir+"7za"+( System == "win32" ? ".exe" : "" )
                ,QStringList() << "a" << "-r" << zip_name << files
                ,working_dir
                ,time_out);
}


bool  sApplicationJob::Unzip (QString      zip_name
                             ,QStringList  files
                             ,QString      destination_dir
                             ,int          time_out/*s*/)
{
  return Execute(ZipProgDir+"7za"+( System == "win32" ? ".exe" : "" )
                ,QStringList() << "x" << "-y"
                               << QString("-o")+destination_dir
                               << zip_name << files
                ,AppDir.canonicalPath()
                ,time_out);
}


bool  sApplicationJob::Unzip (QString      zip_name
                             ,QString      destination_dir
                             ,int          time_out/*s*/)
{
  return Execute(ZipProgDir+"7za"+( System == "win32" ? ".exe" : "" )
                ,QStringList() << "x" << "-y"
                               << QString("-o")+destination_dir
                               << zip_name
                ,AppDir.canonicalPath()
                ,time_out);
}


bool  sApplicationJob::Copy (const QDir & from_dir, const QDir & to_dir, const QStringList & files)
{
  if( files.isEmpty() )
    return true;
  QString                     Temp("temp.7z");
  AppDir.remove(Temp);
  return Zip(AppDir.absoluteFilePath(Temp),files,from_dir.absolutePath())
      && Unzip(AppDir.absoluteFilePath(Temp),to_dir.absolutePath())
      && AppDir.remove(Temp);
}

//----------------------------------------------------------- sExecutivePage ---

void  sExecutivePage::Out (QString  text)
{
  if( Report ){
    Report->insertPlainText(text);
    Report->moveCursor(QTextCursor::End);
    Report->repaint();
  }
}


void  sExecutivePage::OutHTML (QString  text)
{
  if( Report ){
    Report->append(text);
    Report->moveCursor(QTextCursor::End);
    Report->repaint();
  }
}

//---------------------------------------------------------------- sDoItPage ---

sDoItPage::sDoItPage (int              page_id
                     ,const QString &  prompt_title
                     ,const QString &  perform_button_text
                     ,const QString &  performance_title
                     ,const QString &  done_title
                     ,const QString &  error_title)
    :PromtTitle(prompt_title)
    ,PerformButtonText(perform_button_text)
    ,PerformanceTitle(performance_title)
    ,DoneTitle(done_title)
    ,ErrorTitle(error_title)
    ,Success(new QCheckBox)
    ,Done(false)
{
  setTitle(PromtTitle);
  setCommitPage(true);

  QVBoxLayout *               Layout = new QVBoxLayout;
  setLayout(Layout);

  Report = new QTextEdit(this);
  Layout->addWidget(Report);
  Report->setReadOnly(true);

  Success->setChecked(false);
  registerField(QString("Success")+QString().setNum(page_id)
               ,Success);
}


void  sDoItPage::setVisible (bool  visible)
{
  QWizardPage::setVisible(visible);
  if( visible ){
    CommitText = wizard()->buttonText(QWizard::CommitButton);
    wizard()->setButtonText(QWizard::CommitButton
                           ,wizard()->buttonText(QWizard::NextButton));
    wizard()->setButtonText(QWizard::CustomButton1,PerformButtonText);
    wizard()->setOption(QWizard::HaveCustomButton1, true);
    connect(wizard() ,SIGNAL(customButtonClicked(int))
           ,this     ,SLOT(CallDoIt()));
  } else {
    wizard()->setButtonText(QWizard::CommitButton,CommitText);
    wizard()->setOption(QWizard::HaveCustomButton1, false);
    disconnect(wizard() ,SIGNAL(customButtonClicked(int))
              ,this     ,SLOT(CallDoIt()));
  }
}


//void  sDoItPage::initializePage ()
//{
//  registerField(QString("Success")+QString().setNum(wizard()->currentId())
//               ,Success);
//}


bool  sDoItPage::isComplete () const
{
  return Done;
}


void  sDoItPage::CallDoIt ()
{
  bool                          OK;

  setTitle(PerformanceTitle);
  CommitIsVisible = wizard()->button(QWizard::CommitButton)->isVisible();
  FinishIsVisible = wizard()->button(QWizard::FinishButton)->isVisible();
  CancelIsVisible = wizard()->button(QWizard::CancelButton)->isVisible();
  wizard()->button(QWizard::CustomButton1)->setVisible(false);
  wizard()->button(QWizard::BackButton)->setVisible(false);
  wizard()->button(QWizard::CommitButton)->setVisible(false);
  wizard()->button(QWizard::FinishButton)->setVisible(false);
  wizard()->button(QWizard::CancelButton)->setVisible(false);
  OK = DoIt();
  Done = true;
  emit completeChanged();
//  OK = false;
  if( OK ){
    Success->setChecked(true);
//    setField(QString("Success")+QString().setNum(wizard()->currentId()),true);
    setTitle(DoneTitle);
  }else{
    setTitle(ErrorTitle);
  }
  wizard()->button(QWizard::CustomButton1)->setVisible(false);
  wizard()->button(QWizard::BackButton)->setVisible(false);
  wizard()->button(QWizard::CommitButton)->setVisible(CommitIsVisible);
  wizard()->button(QWizard::FinishButton)->setVisible(FinishIsVisible);
  wizard()->button(QWizard::CancelButton)->setVisible(CancelIsVisible
                                                      && ! FinishIsVisible);
}

//------------------------------------------------------------------------------
} //namespace RA
