////////////////////////////////////////////////////////////////////////////////
/*! @file CommonDist.h   Общие классы для создания и инсталяции дистрибутивов.
- Uses selected files from RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2011-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          VBursian@gmail.com
*///////////////////////////////////////////////////////////////////////////////
#ifndef CommonDist_H
#define CommonDist_H
//#include "RANet_global.h"
#include "Version.h"
#include <QString>
#include <QStringList>
#include <QDir>
#include <QWizardPage>
#include <QTextEdit>
#include <QCheckBox>
#include <map>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sProjectInfo)
ANNOUNCE_CLASS(sApplicationJob)

//------------------------------------------------------------- sProjectInfo ---

class sProjectInfo
{
  public://types
    ANNOUNCE_CLASS(sPartInfo)
    class  sPartInfo
    {
      public:
                                  sPartInfo ();
                                  sPartInfo (rcsVersion  version
                                            ,QString     user_friendly_name);
      public://fields
        sVersion                  Version;
        QString                   UserFriendlyName;
        bool                      Todays;
//        bool                      TranslationExists;
//        bool                      TranslationOK;
        bool                      Selected;
    };

  public:
                              sProjectInfo ();
  public://fields
    sVersion                  PrincipalVersion;
    psPartInfo                Core;
    QStringList               Plugins;
    std::map<QString
            ,psPartInfo>      Plugin;
    QStringList               Languages;
};

//---------------------------------------------------------- sApplicationJob ---

class sApplicationJob : public sProjectInfo
{
  public://static const
    static const QStringList  QtLibraries;
    static const QStringList  GNULibraries;

  public:
                              sApplicationJob (bool  testing);
    virtual void              Out (QString  text);
    virtual bool              Execute (QString      prog
                                      ,QStringList  args
                                      ,QString      working_dir
                                      ,int          time_out/*s*/ = -1);
    virtual bool              Zip (QString      zip_name
                                  ,QStringList  files
                                  ,QString      working_dir
                                  ,int          time_out/*s*/ = -1);
    virtual bool              Unzip (QString      zip_name
                                    ,QStringList  files
                                    ,QString      destination_dir
                                    ,int          time_out/*s*/ = -1);
    virtual bool              Unzip (QString  zip_name
                                    ,QString  destination_dir
                                    ,int      time_out/*s*/ = -1);
    virtual bool              Copy (const QDir &         from_dir
                                   ,const QDir &         to_dir
                                   ,const QStringList &  files   );

  public://fields
    bool                      Testing;
    QString                   System;
    QDir                      AppDir;
    QDir                      DistFilesDir; //!< contains distributive zip-files
    QString                   ZipProgDir;   //!< contains the 7za program
                                            //!< (if not empty, then ends with
                                            //!< separator)
    QTextEdit *               Report;
};

//----------------------------------------------------------- sExecutivePage ---

class  sExecutivePage : public QWizardPage
{
  Q_OBJECT
  public:
                              sExecutivePage ()
                                  :Report(NULL)
                                {}
    virtual void              Out (QString  text);
    virtual void              OutHTML (QString  text);

  protected://fields
    QTextEdit *               Report;
};

//---------------------------------------------------------------- sDoItPage ---

class  sDoItPage : public sExecutivePage
{
  Q_OBJECT
  public:
                              sDoItPage (int              page_id
                                        ,const QString &  prompt_title
                                        ,const QString &  perform_button_text
                                        ,const QString &  performance_title
                                        ,const QString &  done_title
                                                            = QString("Done")
                                        ,const QString &  error_title
                                                            = QString("Error")
                                        );

  public slots: //reimplementing inherited:
    virtual void	            setVisible (bool  visible);

  public: //reimplementing inherited:
//    virtual void              initializePage ();
    virtual bool              isComplete () const;

  public://fields
    QString                   PromtTitle;
    QString                   PerformButtonText;
    QString                   PerformanceTitle;
    QString                   DoneTitle;
    QString                   ErrorTitle;

  protected:
    virtual bool              DoIt () =0;

  private slots:
    void                      CallDoIt ();

  private://fields
    QCheckBox *               Success;
    bool                      Done;
    bool                      CommitIsVisible;
    bool                      FinishIsVisible;
    bool                      CancelIsVisible;
    QString                   CommitText;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
