////////////////////////////////////////////////////////////////////////////////
/*! @file InstallWizard.cpp   Мастер установки составного дистрибутива.
- Uses selected files from RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2011-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          VBursian@gmail.com
*///////////////////////////////////////////////////////////////////////////////
#include "InstallWizard.h"
#include "StringsAndQStrings.h"
#include <QVBoxLayout>
#include <QGridLayout>
#include <QLineEdit>
#include <QPushButton>
#include <QFileDialog>
#include <QCheckBox>
#include <QMessageBox>
#include <QApplication>
#include <QTextStream>
#include <QDebug>
namespace RA {
//------------------------------------------------------------------------------
//----------------------------------------------------------- sInstallWizard ---

sInstallWizard::sInstallWizard (bool  testing)
    :QWizard()
    ,Job( new sInstallJob(testing) )
{
  setWindowTitle(tr("Research Assistant Installation Wizard"));
  setPixmap(QWizard::LogoPixmap,QPixmap(":/install/ra_eye"));
//  setOption(QWizard::NoCancelButtonOnLastPage);

  if( Job->AnalizeDistributive() ){
    setPage( IntroPage , new sIntroPage(Job) );
    setPage( MainSelectionPage , new sMainSelectionPage(Job) );
    setPage( ProgDirPage , new sProgDirPage(Job) );
    setPage( PartsSelectionPage , new sPartsSelectionPage(Job) );
    setPage( LicensePage , new sLicensePage(Job) );
    setPage( InstallationPage , new sInstallationPage(Job) );
    setPage( ConclusionPage , new sConclusionPage );
    setPage( SorryPage , new sSorryPage );
  }
}


int  sInstallWizard::nextId () const
{
  switch( currentId() ){
    case MainSelectionPage:
        if( field("install_prog").toBool() ){
          return ProgDirPage;
        }else if( field("install_sources").toBool() ){
          return InstallationPage;
        }else if( field("install_fonts").toBool() ){
          return InstallationPage;
        }else{
          return SorryPage;
        }
    case InstallationPage:
        if( field(QString("Success")
                 +QString().setNum(InstallationPage)).toBool() ){
          return ConclusionPage;
        }else{
          return SorryPage;
        }
    case ConclusionPage:
    case SorryPage:
        return -1;
    default:
        return QWizard::nextId();
  }
}

//--------------------------------------------------------------- sIntroPage ---

sIntroPage::sIntroPage (psInstallJob  job)
    :Job(job)
{
  setTitle(tr("Introduction"));
  if( Job->Core )
    setSubTitle(tr("RA distributive, version %1")
               .arg(ToQString(Job->Core->Version.Text()))
               );
  setPixmap(QWizard::WatermarkPixmap,QPixmap(":/install/aboutra.png"));

  QVBoxLayout *               Layout = new QVBoxLayout;
  setLayout(Layout);

  QLabel *                    Label = new QLabel(tr(
      "This wizard will help you to install or upgrade\n"
      "the Research Assistant (RA) program and/or its data.\n\n"
      "On the next few pages you will\n"
//      "-- read more about RA,\n"
      "-- select folders for the installation,\n"
      "-- choose what exactly you wish to install or upgrade,\n"
      "-- confirm your agreement to the free software license,\n"
      "-- wait about a minute until everything is done for you,\n"
//      "-- run RA (optional)."
                                                    ));
  Label->setWordWrap(true);
  Layout->addWidget(Label);
}

//------------------------------------------------------- sMainSelectionPage ---

sMainSelectionPage::sMainSelectionPage (psInstallJob  job)
    :Job(job)
{
  setTitle("What can I do for you?");
  setSubTitle(" ");

  QVBoxLayout *               Layout = new QVBoxLayout;
  QCheckBox *                 Chkbox;

  setLayout(Layout);

  Chkbox = new QCheckBox(tr("install or update RA program"
                            " and/or selected plugins"));
  Layout->addWidget(Chkbox);
  Chkbox->setChecked(true);
  registerField(QString("install_prog"),Chkbox);
  connect(Chkbox,SIGNAL(clicked()),this,SIGNAL(completeChanged()));

  Chkbox = new QCheckBox(tr("install Ubuntu true-type font"));
  Chkbox->setToolTip(tr(
      "Ubuntu font has all specicific characters that are necessary for RA. "
      "It is recommended to use this font under Windows too."
                       ));
  Layout->addWidget(Chkbox);
  Chkbox->setChecked(false);
  registerField(QString("install_fonts"),Chkbox);
  connect(Chkbox,SIGNAL(clicked()),this,SIGNAL(completeChanged()));
  Chkbox->setEnabled( ! Job->FontsZip.isEmpty() );

  Chkbox = new QCheckBox(tr("install the \"sandbox\" database"));
  Chkbox->setToolTip(tr(
      "Creates a container for RA knowledgebase with the simpliest structure "
      "(just one sqlite-type file), which is recommended for a quick start."
                       ));
  Layout->addWidget(Chkbox);
  Chkbox->setChecked(false);
  registerField(QString("install_sandbox"),Chkbox);
  connect(Chkbox,SIGNAL(clicked()),this,SIGNAL(completeChanged()));

  Chkbox = new QCheckBox(tr("add a shortcut"));
  Layout->addWidget(Chkbox);
  Chkbox->setChecked(false);
  registerField(QString("install_launcher"),Chkbox);
  connect(Chkbox,SIGNAL(clicked()),this,SIGNAL(completeChanged()));
  Chkbox->setEnabled( Job->System == "unix" );

  Chkbox = new QCheckBox(tr("install source texts (for programmers)"));
  Layout->addWidget(Chkbox);
  Chkbox->setChecked(false);
  registerField(QString("install_sources"),Chkbox);
  connect(Chkbox,SIGNAL(clicked()),this,SIGNAL(completeChanged()));
}


bool  sMainSelectionPage::isComplete () const
{
  return field("install_prog").toBool()
      || field("install_fonts").toBool()
      || field("install_sandbox").toBool()
      || field("install_launcher").toBool()
      || field("install_sources").toBool()
      ;
}

//------------------------------------------------------------- sProgDirPage ---

sProgDirPage::sProgDirPage (psInstallJob  job)
    :Job(job)
{
  setTitle("Folders");
  setSubTitle(" ");

  int                         Row = 0;
  QGridLayout *               Layout = new QGridLayout;
  setLayout(Layout);

  QLabel *                    Label;
  Label = new QLabel(tr("Program folder:"));
  Layout->addWidget(Label,Row++,1,1,1);

  QLineEdit *                 ProgDirField = new QLineEdit;
  Layout->addWidget(ProgDirField,Row,1,1,1);
  ProgDirField->setText(QDir::toNativeSeparators(Job->ProgDir.path()));
  registerField("ProgDir*",ProgDirField);
//  ProgDirField->setReadOnly(true);
//  connect(ProgDirField,SIGNAL(editingFinished())
//         ,this,S

  QPushButton *               ProgDirButton = new QPushButton(tr("Browse"));
  Layout->addWidget(ProgDirButton,Row++,2,1,1);
  connect(ProgDirButton,SIGNAL(clicked()),this,SLOT(ProgDirBrowse()));
}


void  sProgDirPage::ProgDirBrowse ()
{
  QFileDialog                   D;
  D.setWindowTitle(tr("Select program folder"));
  D.setOption(QFileDialog::DontUseNativeDialog,true);
  D.setFileMode(QFileDialog::Directory);
//  D.setOption(QFileDialog::ShowDirsOnly,true);
  if( ! field("ProgDir").toString().trimmed().isEmpty() )
    D.setDirectory(field("ProgDir").toString().trimmed());
  if( D.exec() && ! D.selectedFiles().empty() ){
    QString                     FileName(D.selectedFiles().at(0));
    if( ! FileName.endsWith(ToQString(Job->PrincipalVersion.Text())) ){
      if( ! FileName.contains("RA")
          && ! ( FileName.contains("Research")
                 && FileName.contains("Assistant") ) ){
        FileName += "/ResearchAssistant";
      }
      FileName += QDir::separator();
      FileName += Job->PrincipalVersion.Text();
    }
    setField("ProgDir",QDir::toNativeSeparators(QDir::cleanPath(FileName)));
    emit completeChanged();
  }
}


bool  sProgDirPage::FindPreviousInstallations ()
{
  return false;
}


bool  sProgDirPage::isComplete () const
{
  if( ! field("ProgDir").toString().trimmed().isEmpty() ){
    Job->ProgDir = QDir( field("ProgDir").toString().trimmed() );
    Job->Updating = Job->ProgDir.exists();
    return true;
  }
  return false;
}

//------------------------------------------------------ sPartsSelectionPage ---

sPartsSelectionPage::sPartsSelectionPage (psInstallJob  job)
    :Job(job)
{
  setTitle("Select program parts");
  setSubTitle(" ");

  int                         Row = 0;
  QGridLayout *               Layout = new QGridLayout;
  QCheckBox *                 Chkbox;
  QLabel *                    Label;

  setLayout(Layout);
  if( Job->Core ){
    Chkbox = new QCheckBox(tr("Install RA program"));
    Layout->addWidget(Chkbox,Row,1,1,2);
    Chkbox->setChecked(true);
    registerField(QString("ra_core"),Chkbox);
    connect(Chkbox,SIGNAL(toggled(bool)),this,SLOT(Validate(bool)));

    Label = new QLabel(tr("version: %1")
                      .arg(ToQString(Job->Core->Version.Text())));
    Layout->addWidget(Label,Row++,3,1,1);

    Label = new QLabel(tr("with the following plugins:"));
    Layout->addWidget(Label,Row++,1,1,3,Qt::AlignHCenter);
  }else{
    Label = new QLabel(tr("Install the following plugins:"));
    Layout->addWidget(Label,Row++,1,1,3,Qt::AlignHCenter);
  }
  for( QStringList::const_iterator  P = Job->Plugins.constBegin()
                                  ; P != Job->Plugins.constEnd(); ++P ){
    Chkbox = new QCheckBox( *P );
    Layout->addWidget(Chkbox,Row,2,1,1);
    Chkbox->setChecked(true);
    registerField(QString("rap_")+(*P),Chkbox);
    connect(Chkbox,SIGNAL(toggled(bool)),this,SLOT(Validate(bool)));

    Label = new QLabel(tr("version: %1")
                      .arg(ToQString(Job->Plugin[*P]
                                                   ->Version.Text())));
    Layout->addWidget(Label,Row++,3,1,1);
  }
}


void  sPartsSelectionPage::Validate (bool  checked)
{
  QStringList::const_iterator P;
  if( checked ){
    for( P = Job->Plugins.constBegin() ;
           P != Job->Plugins.constEnd(); ++P ){
      if( field( QString("rap_") + (*P) ).toBool() ){
        if( Job->Core )
          setField(QString("ra_core"),true);
      }
    }
  }else{
    if( Job->Core && ! field( QString("ra_core") ).toBool() ){
      for( P = Job->Plugins.constBegin() ;
             P != Job->Plugins.constEnd(); ++P ){
        setField( QString("rap_") + (*P) , false );
      }
    }
  }
}

//------------------------------------------------------------- sLicensePage ---

sLicensePage::sLicensePage (psInstallJob  job)
    :Job(job)
{
  setTitle("License Agreement");
  setSubTitle("Please read the license.");

  QVBoxLayout *               Layout = new QVBoxLayout;
  setLayout(Layout);

//  QTextBrowser *              Viewer = new QTextBrowser;
//  Layout->addWidget(Viewer);
//  Viewer->setSource(QUrl("ShortLicense.html"));

  Viewer = new QTextEdit;
  Viewer->setReadOnly(true);
  Viewer->setMinimumWidth(600);
  Viewer->setMinimumHeight(350);
  Layout->addWidget(Viewer);

  btnIntro = new QPushButton("Back to the brief license overview");
  Layout->addWidget(btnIntro);
  connect(btnIntro,SIGNAL(clicked()) ,this,SLOT(ShowIntro()));

  btnGPL = new QPushButton("Show me GNU General Public License");
  Layout->addWidget(btnGPL);
  connect(btnGPL,SIGNAL(clicked()) ,this,SLOT(ShowGPL()));

  btnFDL = new QPushButton("Show me GNU Free Documentation License");
  Layout->addWidget(btnFDL);
  connect(btnFDL,SIGNAL(clicked()) ,this,SLOT(ShowFDL()));

  chkboxAgree = new QCheckBox(tr("  I accept the license agreement  "));
  Layout->addWidget(chkboxAgree);

  connect(chkboxAgree,SIGNAL(clicked())
         ,this,SIGNAL(completeChanged()));
}


void  sLicensePage::initializePage ()
{
  ShowIntro();
}


void  sLicensePage::ShowIntro ()
{
  QFile     file(Job->DistFilesDir.absoluteFilePath("Licenses(briefly).html"));
  if( ! file.open(QFile::ReadOnly | QFile::Text) ){
    QMessageBox::warning(this, tr("install")
                        ,tr("Cannot read file %1:\n%2.")
                           .arg(file.fileName()).arg(file.errorString()));
  }else{
    QTextStream               in(&file);
    in.setCodec("UTF-8"); //QTextCodec::codecForName("utf8") ??
    QApplication::setOverrideCursor(Qt::WaitCursor);
    Viewer->setHtml(in.readAll());
    QApplication::restoreOverrideCursor();
    btnIntro->setVisible(false);
    btnGPL->setVisible(true);
    btnFDL->setVisible(true);
  }
}


void  sLicensePage::ShowGPL ()
{
  QFile     file(Job->DistFilesDir.absoluteFilePath("GNU_GPL_3_0.html"));
  if( ! file.open(QFile::ReadOnly | QFile::Text) ){
    QMessageBox::warning(this, tr("install")
                        ,tr("Cannot read file %1:\n%2.")
                           .arg(file.fileName()).arg(file.errorString()));
  }else{
    QTextStream               in(&file);
    in.setCodec("UTF-8"); //QTextCodec::codecForName("utf8") ??
    QApplication::setOverrideCursor(Qt::WaitCursor);
    Viewer->setHtml(in.readAll());
    QApplication::restoreOverrideCursor();
    btnIntro->setVisible(true);
    btnGPL->setVisible(false);
    btnFDL->setVisible(true);
  }
}


void  sLicensePage::ShowFDL ()
{
  QFile     file(Job->DistFilesDir.absoluteFilePath("GNU_FDL_1_3.html"));
  if( ! file.open(QFile::ReadOnly | QFile::Text) ){
    QMessageBox::warning(this, tr("install")
                        ,tr("Cannot read file %1:\n%2.")
                           .arg(file.fileName()).arg(file.errorString()));
  }else{
    QTextStream               in(&file);
    in.setCodec("UTF-8"); //QTextCodec::codecForName("utf8") ??
    QApplication::setOverrideCursor(Qt::WaitCursor);
    Viewer->setHtml(in.readAll());
    QApplication::restoreOverrideCursor();
    btnIntro->setVisible(true);
    btnGPL->setVisible(true);
    btnFDL->setVisible(false);
  }
}


bool  sLicensePage::isComplete () const
{
  return  chkboxAgree->isChecked();
}

//-------------------------------------------------------- sInstallationPage ---

sInstallationPage::sInstallationPage (psInstallJob  job)
    :sDoItPage(sInstallWizard::InstallationPage
              ,"Ready to install"
              ,"Install"
              ,"Installing...")
    ,Job(job)
{
}


void  sInstallationPage::initializePage ()
{
  sDoItPage::initializePage();

  Job->Report = Report;
  if( Job->Core ){
    Job->Core->Selected = field("ra_core").toBool();
  }
  for( QStringList::const_iterator  P = Job->Plugins.constBegin()
                                  ; P != Job->Plugins.constEnd(); ++P ){
    if( Job->Plugin.count(*P) ){
      Job->Plugin[*P]->Selected = field(QString("rap_")+(*P)).toBool();
    }
  }
  Job->Install_Prog = field("install_prog").toBool();
  Job->Install_Fonts = field("install_fonts").toBool();
  Job->Install_Sandbox = field("install_sandbox").toBool();
  Job->Install_Launcher = field("install_launcher").toBool();
  Job->Install_Sources = field("install_sources").toBool();

  OutHTML(Job->Plan());
  if( ! Job->MakeDirs() ){
    OutHTML(QString("<font color=red size=+6>")
           +"<p>Oops!!!</p>"
           +"<p>___________________________________________________________</p>"
           +"</font>\n\n"
       );
  }
}


bool  sInstallationPage::DoIt ()
{
  if( Job->Install() ){
    OutHTML(QString("<font color=green><p><br></p>")
           +"<p>___________________________________________________________</p>"
           +"<p><font size=+2><b>Congratulations!</b></font></p>"
           +"<p>"
           +Job->Resume
           +"</p>"
           +"<p>___________________________________________________________</p>"
           +"<p><br></p>"
         );
    return true;
  }
  OutHTML(QString("<font color=red><p><br></p>")
         +"<p>___________________________________________________________</p>"
         +"<p><font size=+2><b>Oops!&nbsp;&nbsp;&nbsp;&nbsp;:-(</b></font></p>"
           +"<p>"
           +Job->Resume
           +"</p>"
         +"<p>___________________________________________________________</p>"
         +"<p><br></p>"
       );
  return false;
}


//---------------------------------------------------------- sConclusionPage ---

sConclusionPage::sConclusionPage ()
{
  setTitle("Conclusion");

  QVBoxLayout *               Layout = new QVBoxLayout;
  setLayout(Layout);

  Label = new QLabel;
  Label->setWordWrap(true);
  Layout->addWidget(Label);
}


void  sConclusionPage::initializePage ()
{
  if( field(QString("Success")
           +QString().setNum(sInstallWizard::InstallationPage)).toBool() ){
    Label->setText("Done.\nHave a nice day!");
  }else{
    Label->setText("Uhps!\nPoor you...");
  }
}


//--------------------------------------------------------------- sSorryPage ---

sSorryPage::sSorryPage ()
{
  setTitle("Sorry");

  QVBoxLayout *               Layout = new QVBoxLayout;
  setLayout(Layout);

  Label = new QLabel;
  Label->setWordWrap(true);
  Layout->addWidget(Label);
}


void  sSorryPage::initializePage ()
{
  if( field(QString("Success")
           +QString().setNum(sInstallWizard::InstallationPage)).toBool() ){
    Label->setText("Done.\nHave a nice day!");
  }else{
    Label->setText("Uhps!\nPoor you...");
  }
}

//------------------------------------------------------------------------------
} //namespace RA



