////////////////////////////////////////////////////////////////////////////////
/*! @file SingleApplication.cpp Provides classes to  prevent running of
                                another instance of the same program.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2010-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          VBursian@gmail.com
*///////////////////////////////////////////////////////////////////////////////
#include "SingleApplication.h"
#include <QFile>
#include <QStringList>
#include <QDebug>
//------------------------------------------------------------------------------
//------------------------------------------------------- sSingleApplication ---

/*! Checks and fires up LocalServer or closes the program
    if another instance already exists.

@param argc
@param argv
@param unique_name

Initially based on
https://github.com/Itehnological/SingleApplication
by Itay Grudev, 2012.


when the qlocalserver crashes for any reson it does not release the socket
file and any further trying to connect fail cause he sees that some app is
already listening on that socket, solving this issue by manualy deleting the
socket temp file solve the issue of crash case but kill the idea of single
application cause you can start two servers concurrently this way

*/
sSingleApplication::sSingleApplication (int      argc
                                       ,char *   argv[]
                                       ,QString  unique_name )
    :QApplication(argc, argv)
    ,FirstInstanceFlag(false)
{
  QLocalSocket                Socket;
  Socket.connectToServer(unique_name);
  if( Socket.waitForConnected(100) ){
    Socket.write("CMD:showUp");
    Socket.flush();
    QThread::msleep(100);
    Socket.close();
  }else if( Socket.error() == QLocalSocket::ServerNotFoundError ){
    FirstInstanceFlag = true;
    server = new sLocalServer(unique_name);
    server->start();
    QObject::connect(server ,SIGNAL(showUp())
                    ,this   ,SIGNAL(ShowUp()) ,Qt::QueuedConnection);
  }else if( Socket.error() == QLocalSocket::ConnectionRefusedError ){
    qDebug() << "Socket.error" << Socket.error() << Socket.errorString()
             << "previous instance probably crashed";
    //  file /tmp/RA__lock
    FirstInstanceFlag = true;
    server = new sLocalServer(unique_name);
    server->start();
    QObject::connect(server ,SIGNAL(showUp())
                    ,this   ,SIGNAL(ShowUp()) ,Qt::QueuedConnection);
  }else{
    qDebug() << "Socket.error" << Socket.error() << Socket.errorString();
  }
}


sSingleApplication::~sSingleApplication ()
{
  if( FirstInstanceFlag ){
    server->terminate();
  }
}

//------------------------------------------------------------- sLocalServer ---

sLocalServer::sLocalServer (QString  LocalServerName)
    :TheLocalServerName(LocalServerName)
{
}


sLocalServer::~sLocalServer()
{
  server->close();
  for(int i = 0; i < clients.size(); ++i)
  {
    clients[i]->close();
  }
}

/**
* -----------------------
* QThread requred methods
* -----------------------
*/

/**
* @brief run
* Initiate the thread.
*/
void sLocalServer::run()
{
  server = new QLocalServer();
  server->setSocketOptions(QLocalServer::WorldAccessOption);
  QObject::connect(server, SIGNAL(newConnection()), this, SLOT(slotNewConnection()));
  QObject::connect(this, SIGNAL(privateDataReceived(QString)), this, SLOT(slotOnData(QString)));

#ifdef Q_OS_UNIX
  // Make sure the temp address file is deleted
  QFile address(TheLocalServerName);
  if(address.exists()){
    address.remove();
  }
#endif

  server->listen(TheLocalServerName);
  while(server->isListening() == false){
    server->listen(TheLocalServerName);
    msleep(100);
  }
  exec();
}

/**
* @brief LocalServer::exec
* Keeps the thread alive. Waits for incomming connections
*/
void sLocalServer::exec()
{
  while(server->isListening())
  {
    msleep(100);
    server->waitForNewConnection(100);
    for(int i = 0; i < clients.size(); ++i)
    {
      if(clients[i]->waitForReadyRead(100)){
        QByteArray data = clients[i]->readAll();
        emit privateDataReceived(data);
      }
    }
  }
}

/**
* -------
* SLOTS
* -------
*/

/**
* @brief LocalServer::slotNewConnection
* Executed when a new connection is available
*/
void sLocalServer::slotNewConnection()
{
  clients.push_front(server->nextPendingConnection());
}

/**
* @brief LocalServer::slotOnData
* Executed when data is received
* @param data
*/
void sLocalServer::slotOnData(QString data)
{
  if(data.contains("CMD:", Qt::CaseInsensitive)){
    onCMD(data);
  } else {
    emit dataReceived(data);
  }
}

/**
* -------
* Helper methods
* -------
*/

void sLocalServer::onCMD(QString data)
{
  // Trim the leading part from the command
  data.replace(0, 4, "");

  QStringList commands;
  commands << "showUp";

  switch(commands.indexOf(data)){
    case 0:
      emit showUp();
  }
}

//------------------------------------------------------------------------------
