////////////////////////////////////////////////////////////////////////////////
/*! @file SingleApplication.h   Provides classes to  prevent running of 
                                another instance of the same program.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2010-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          VBursian@gmail.com
*///////////////////////////////////////////////////////////////////////////////
#ifndef SingleApplication_H
#define SingleApplication_H
#include <QApplication>
#include <QThread>
#include <QVector>
#include <QLocalServer>
#include <QLocalSocket>
//------------------------------------------------------------------------------

class sLocalServer;

//------------------------------------------------------- sSingleApplication ---
/*! A substitution for QApplication, which prevents running of another instance
    of the same program.
*/
class sSingleApplication : public QApplication
{
  Q_OBJECT
  public:
    virtual                   ~sSingleApplication ();
    explicit                  sSingleApplication (int      argc
                                                 ,char *   argv[]
                                                 ,QString  unique_name);
    bool                      IsFirstInstance () const
                                { return FirstInstanceFlag; }

  signals:
    void                      ShowUp ();
  
  private:
    bool                      FirstInstanceFlag;
    sLocalServer *             server;
};

//------------------------------------------------------------- sLocalServer ---

class sLocalServer : public QThread
{
  Q_OBJECT
public:
  sLocalServer (QString  LocalServerName);
  ~sLocalServer();
  void shut();

protected:
  void run();
  void exec();

signals:
  void dataReceived(QString data);
  void privateDataReceived(QString data);
  void showUp();

private slots:
  void slotNewConnection();
  void slotOnData(QString data);

  private:
    QString                   TheLocalServerName;
  QLocalServer* server;
  QVector<QLocalSocket*> clients;
  void onCMD(QString data);

};

//------------------------------------------------------------------------------
#endif 
