////////////////////////////////////////////////////////////////////////////////
/*! @file Adam.h   Реализация полиморфизма.
- Part of RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 1994-2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef Adam_H
#define Adam_H
#include <map>
#include "General.h"
#include "Strings.h"
namespace RA {
//------------------------------------------------------------------------------

using std::map;

ANNOUNCE_CLASS(sTag)
ANNOUNCE_ABSTRACT_CLASS(sAdam)

bool operator <= (psAdam PtrToDerivedType ,rcsTag BaseType);

//--------------------------------------------------------------------- sTag ---

class RANet_EXPORT  sTag
{
  public: // static
    static pcsTag             ByName (rcsString);
  public:
    virtual sString           Name () const
                                { return sString(TheName); };
    virtual bool              CanCast (psAdam) const
                                { return false; };
  protected:
                              sTag (literal  name);
  protected: // typedefs
    typedef map<sString,pcsTag>
                              sTagMapByName;
  protected: // static fields
    static sTagMapByName      TagMapByName;
  private: // fields
    literal                   TheName;
  private: // copying is prohibited
                              sTag (rcsTag);
    rsTag                     operator = (rcsTag);
};

//------------------------------------------------------------------- ADAM() ---

#define ADAM(CLASSNAME)                                                      \
  private:                                                                   \
  public:                                                                    \
    class sIntrinsicTag : public sTag                                        \
    {                                                                        \
      public:                                                                \
                                  sIntrinsicTag ()                           \
                                      :sTag(#CLASSNAME)                      \
                                    {};                                      \
        virtual bool              CanCast (psAdam P) const                   \
                                    { return bool(                           \
                                          dynamic_cast<p##CLASSNAME>(P)      \
                                                 );                          \
                                    };                                       \
    };                                                                       \
    static sIntrinsicTag      AdamInheritorTag;                              \
  public:                                                                    \
    static pcsTag             Type()                                         \
                                { return &AdamInheritorTag;};                \
    virtual pcsTag            TrueType () const                              \
                                {return &AdamInheritorTag;};                 \
  private:


//---------------------------------------------- DEFINE_CLASS_TAG(CLASSNAME) ---
/*! Этот макрос должен присутствовать для каждого класса, публично
    порождённого от RA::sAdam.  Он определяет статический тэг класса, нужный
    для работы с полиморфными объектами.

Помещается он в один из cpp-файлов -- чаще всего, в начало cpp-файла,
содержащего определения функций-методов этого класса.  В качестве аргумента
указывается имя класса.  Точка с запятой в конце не ставится (Си-плюс-плюсу она
не мешает, но doxygen от неё дуреет -- воспринимает всю конструкцию как
предобъявление функции).

Использование:
@code
DEFINE_CLASS_TAG(sMyClass)
@endcode

@macro
*/
#define DEFINE_CLASS_TAG(CLASSNAME)                                          \
CLASSNAME::sIntrinsicTag   CLASSNAME::AdamInheritorTag;


//-------------------------------------------------------------------- sAdam ---
/*! Базовый класс для полиморфных объектов
*/
class RANet_EXPORT  sAdam
{
  ADAM(sAdam)
  public:
    virtual                   ~sAdam ()
                                {}
  protected:
                              sAdam ()
                                {}
};

//---------------------------------------------- operator <= (psAdam,pcsTag) ---

inline bool operator <= (psAdam object ,rcsTag base_type)
{
  return base_type.CanCast(object);
}

//------------------------------------------------------------------------------
} //namespace RA
#endif // Adam_H
