////////////////////////////////////////////////////////////////////////////////
/*! @file ArrayDataFunction.cpp Function given by an array of ordinates.
- Part of RANet - Research Assistant Net Library.
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "ArrayDataFunction.h"
//#include "Log.h"
namespace RA {
//------------------------------------------------------------------------------
//---------------------------------------------- sArrayDataFunction::sTracer ---

sArrayDataFunction::sTracer::sTracer (pcsArrayDataFunction  host_curve
                                     ,rcsScales               scales    )
    :TheFunction(host_curve)
    ,Scaling(1.0,1.0)
    ,LastPointNo(host_curve->NumberOfPoints())
    ,PointNo(0)
    ,LineSegmentNo(1)
{
  try{
    Scaling.X = (host_curve->Multiplier().X / scales.X.Unit()).ToReal();
    Scaling.Y = (host_curve->Multiplier().Y / scales.Y.Unit()).ToReal();

//    sPhysValue                  SX(_Unitsless_);
//    sPhysValue                  SY(_Unitsless_);
//    SX = host_curve->Multiplier().X / scales.X.Unit();
//    SY = host_curve->Multiplier().Y / scales.Y.Unit();
////    SX.Normalize();
////    SY.Normalize();
//    Scaling.X = SX.ArbValue();
//    Scaling.Y = SY.ArbValue();

//    RANet::Log.Put(sLog::Debug,"Graph"
//        ,sString("sArrayDataFunction::sTracer::sTracer: Scaling X:")
//        +SX.Text()+" Y:"+SY.Text());
  }catch(rxRAlgebra){
//    RANet::Log.Put(sLog::Debug,"Graph"
//                  ,"sArrayDataFunction::sTracer::sTracer: WrongUnits");
    PointNo = LastPointNo;
  }
  LineSegmentNo = PointNo + 1;
}


bool  sArrayDataFunction::sTracer::NextPoint ()
{
  if( PointNo < LastPointNo ){
    Point.X= (TheFunction->X0() + TheFunction->Step() * PointNo) * Scaling.X;
    Point.Y= TheFunction->Values[PointNo] * Scaling.Y;
    PointNo++;
    return true;
  }else{
    return false;
  }
}


bool  sArrayDataFunction::sTracer::NextLineSegment ()
{
  if( LineSegmentNo < LastPointNo ){
    LineSegment.Start.X = (TheFunction->X0()
                             + TheFunction->Step() * (LineSegmentNo-1)
                          ) * Scaling.X;
    LineSegment.Start.Y = TheFunction->Values[LineSegmentNo-1] * Scaling.Y;
    LineSegment.End.X   = (TheFunction->X0() + TheFunction->Step() * LineSegmentNo
                          ) * Scaling.X;
    LineSegment.End.Y   = TheFunction->Values[LineSegmentNo] * Scaling.Y;
    LineSegmentNo++;
    return true;
  }else{
    return false;
  }
}

//------------------------------------------------------- sArrayDataFunction ---

sArrayDataFunction::~sArrayDataFunction ()
{
  if( Values ){
    delete [] Values;  Values=NULL;
  }
}


sArrayDataFunction::sArrayDataFunction (int  number_of_points
                                       ,real x0
                                       ,real step)
    :sDataFunction()
    ,TheNumberOfPoints(number_of_points) ,TheX0(x0) ,TheStep(step) ,Values(NULL)
{
  ASSERT( TheNumberOfPoints > 0 );
  Values= new real [TheNumberOfPoints];
  for( int PointNo=0 ; PointNo < TheNumberOfPoints ; PointNo++ ){
    Values[PointNo]=0.0;
  }
}


sArrayDataFunction::sArrayDataFunction (rcsPhysPair  multiplier
                                       ,int          number_of_points
                                       ,real         x0
                                       ,real         step)
    :sDataFunction(multiplier)
    ,TheNumberOfPoints(number_of_points) ,TheX0(x0) ,TheStep(step) ,Values(NULL)
{
  ASSERT( TheNumberOfPoints > 0 );
  Values= new real [TheNumberOfPoints];
  for( int PointNo=0 ; PointNo < TheNumberOfPoints ; PointNo++ ){
    Values[PointNo]=0.0;
  }
}


sArrayDataFunction::sArrayDataFunction (std::istream &  a_stream
                                       ,rcsVersion      version)
    :sDataFunction(a_stream,version)
    ,TheNumberOfPoints(0) ,TheX0(0) ,TheStep(1) ,Values(NULL)
{
  if( version <= sVersion(0,9) ){
  }else if( version < sVersion(1,5) ){
    a_stream.READ_INTO(TheNumberOfPoints);
    ASSERT( TheNumberOfPoints > 0 );
    realIOv14 R;
    a_stream.READ_INTO(R);  TheX0=R;
    a_stream.READ_INTO(R);  TheStep=R;
    Values= new real [TheNumberOfPoints];
    for( int PointNo=0 ; PointNo < TheNumberOfPoints ; PointNo++ ){
      a_stream.READ_INTO(R);  Values[PointNo]=R;
    }
  }else{
    a_stream.READ_INTO(TheNumberOfPoints);
    ASSERT( TheNumberOfPoints > 0 );
    realIO R;
    a_stream.READ_INTO(R);  TheX0=R;
    a_stream.READ_INTO(R);  TheStep=R;
    Values= new real [TheNumberOfPoints];
    for( int PointNo=0 ; PointNo < TheNumberOfPoints ; PointNo++ ){
      a_stream.READ_INTO(R);  Values[PointNo]=R;
    }
  }
}


void  sArrayDataFunction::Store (std::ostream & a_stream) const
{
  sDataFunction::Store(a_stream);
  a_stream.WRITE_FROM(TheNumberOfPoints);
  realIO R;
  R=TheX0;  a_stream.WRITE_FROM(R);
  R=TheStep;  a_stream.WRITE_FROM(R);
  for( int PointNo=0 ; PointNo < TheNumberOfPoints ; PointNo++ ){
    R=Values[PointNo];  a_stream.WRITE_FROM(R);
  }
}


sCurve::psTracer  sArrayDataFunction::GetCurveTracer (rcsScales  scales) const
{
  return new sArrayDataFunction::sTracer(this,scales);
}


sPoint  sArrayDataFunction::Point (const int N) const
{
  ASSERT( N >= 0 );
  ASSERT( N < TheNumberOfPoints );
  return sPoint(TheX0+N*TheStep,Values[N]);
}


real  sArrayDataFunction::PointY (const int N) const
{
  ASSERT( N >= 0 );
  ASSERT( N < TheNumberOfPoints );
  return Values[N];
}


void  sArrayDataFunction::SetX0 (real  x0)
{
  TheX0 = x0;
//  if( ! RecalculateBoundaries )
//    LastBoundaries.X = sPhysRange( ... );
  RecalculateBoundaries = true;
}


void  sArrayDataFunction::SetStep (real  step)
{
  TheStep = step;
//  if( ! RecalculateBoundaries )
//    LastBoundaries.X = sPhysRange( ... );
  RecalculateBoundaries = true;
}


void  sArrayDataFunction::SetValue (int N ,real  V)
{
  if( (N >= 0) && (N < TheNumberOfPoints) ){
    Values[N]=V;
//    if( ! RecalculateBoundaries )
//      LastBoundaries.Y |= V * Multiplier().Y;
    RecalculateBoundaries = true;
  }
}

//------------------------------------------------------------------------------
} //namespace RA




