////////////////////////////////////////////////////////////////////////////////
/*! @file AttrName.cpp   Class sAttrName.
- Part of RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 1999-2013, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "AttrName.h"
#include "NetExceptions.h"
namespace RA {
//------------------------------------------------------------------------------

#ifdef RANET_DEBUG
long int                      sAttrName::Counter(0);
#endif

//---------------------------------------------------------------- sAttrName ---

sAttrName::~sAttrName ()
{
  IF_DEBUG(--Counter;)
}


sAttrName::sAttrName ()
    :T(cNameless)
    ,Numeric(0)
    ,Symbolic()
{
  IF_DEBUG(++Counter;)
}


sAttrName::sAttrName (integer  number)
    :T(cNumeric)
    ,Numeric(number)
    ,Symbolic()
{
  IF_DEBUG(++Counter;)
}


sAttrName::sAttrName (rcsString  symbolic_name)
    :T(cSymbolic)
    ,Numeric(0)
    ,Symbolic(symbolic_name)
{
  if( Symbolic.Empty() )
    throw xAttrName("Empty symbolic name");
  IF_DEBUG(++Counter;)
}


sAttrName::sAttrName (literal  symbolic_name)
    :T(cSymbolic)
    ,Numeric(0)
    ,Symbolic(symbolic_name)
{
  if( Symbolic.Empty() )
    throw xAttrName("Empty symbolic name");
  IF_DEBUG(++Counter;)
}


integer  sAttrName::ToNumber () const
{
  ASSERT( T == cNumeric );
  return Numeric;
}


sString  sAttrName::ToString () const
{
  ASSERT( T == cSymbolic );
  return Symbolic;
}


sString  sAttrName::Text () const // visual representation for user
{
  sString                 Result;
  switch( T ){
    case cNameless:
//        text="{}";
        break;
    case cNumeric:
        Result = "[";
        Result << Numeric;
        Result += "]";
        break;
    case cSymbolic:
        Result = Symbolic;
        break;
//    case cAtomic:
//        text="(";
//        text+="atom"; //find by AtomId
//        text+=")";
//        break;
  }
  return Result;
}


//sString  sAttrName::TextWithoutTypeIndication () const
//{
//  sString                 Result;
//  switch( T ){
//    case cNameless:
//        break;
//    case cNumeric:
//        Result << Numeric;
//        break;
//    case cSymbolic:
//        Result = Symbolic;
//        break;
////    case cAtomic:
////        text="atom"; //find by AtomId
////        break;
//  }
//  return Result;
//}


bool  sAttrName::operator == (rcsAttrName  other) const
{
  if( T == other.T ){
    switch( T ){
      case cNameless:
          return true;
      case cNumeric:
          return Numeric == other.Numeric;
      case cSymbolic:
          return Symbolic == other.Symbolic;
    }
  }
  return false;
}


bool  sAttrName::operator == (integer  number) const
{
  if( T == cNumeric ){
    return Numeric == number;
  }
  return false;
}


bool  sAttrName::operator == (rcsString  symbolic_name) const
{
  if( T == cSymbolic ){
    return Symbolic == symbolic_name;
  }
  return false;
}


bool  sAttrName::operator == (literal  symbolic_name) const
{
  if( T == cSymbolic ){
    return Symbolic == symbolic_name;
  }
  return false;
}


sAttrName::sAttrName (std::istream &  a_stream)
{
  IF_DEBUG(++Counter;)
  a_stream.READ_INTO(T);
  switch( T ){
    case cNameless:
        break;
    case cNumeric:
        a_stream.READ_INTO(Numeric);
        break;
    case cSymbolic:
        Symbolic.Read(a_stream);
        break;
  }
}


void  sAttrName::Store (std::ostream &  a_stream) const
{
  a_stream.WRITE_FROM(T);
  switch( T ){
    case cNameless:
        break;
    case cNumeric:
        a_stream.WRITE_FROM(Numeric);
        break;
    case cSymbolic:
        Symbolic.Write(a_stream);
        break;
  }
}


sAttrName::sAttrName (rcsAttrName other)
    :T(other.T)
{
  IF_DEBUG(++Counter;)
  switch( T ){
    case cNameless:
        break;
    case cNumeric:
        Numeric = other.Numeric;
        break;
    case cSymbolic:
        Symbolic = other.Symbolic;
        break;
  }
}


rsAttrName  sAttrName::operator = (rcsAttrName  other)
{
  T=other.T;
  switch( T ){
    case cNameless:
        break;
    case cNumeric:
        Numeric = other.Numeric;
        break;
    case cSymbolic:
        Symbolic = other.Symbolic;
        break;
  }
  return *this;
}

//------------------------------------------------------------------------------
} //namespace RA
