////////////////////////////////////////////////////////////////////////////////
/*! @file Attribute.cpp Class sAttribute.
- Part of RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 1994-2013, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "Attribute.h"
//------------------------------------------------------------------------------
namespace RA {

#ifdef RANET_DEBUG
long int                      sAttribute::Counter(0);
#endif

//--------------------------------------------------------------- sAttribute ---
/*! @class sAttribute

@todo{Net} Не надо ли всю логику IOStatus и Ordering переместить в sFolder::sAttrList?

@par sAttribute instance lifecycle:
-  (bearing or copy constructor)--> cNewborn
-  cNewborn ---(Save)--> cSynchronized
-  cNewborn ---(DelAttr)--> actually deleted
-  (loading by sNet)--> cSynchronized
-  cSynchronized ---(DelAttr)--> cDeleted
-  cDeleted ---(Save)--> actually deleted


@par Attribute ordering:
- вставка нового
- перемещение имеющегося
- когда перенумеровывать:
  - всегда
  - когда конфликт
  - плевать на конфликты, только при загрузке, спасении, обновлении
*/

sAttribute::~sAttribute ()
{
  IF_DEBUG(--Counter;)
}


sAttribute::sAttribute (rcsAttribute  other)
    :TheName(other.TheName)
    ,TheValue(other.TheValue)
    ,IOStatus(other.IOStatus)
    ,StatusTimeStamp(other.StatusTimeStamp)
{
  IF_DEBUG(++Counter;)
}


sAttribute::sAttribute (rcsAttrName  attribute_name
                       ,sNodePtr     value_ptr
                       )
    :TheName(attribute_name)
    ,TheValue(value_ptr)
    ,IOStatus(cNewborn)
    ,StatusTimeStamp()
{
  IF_DEBUG(++Counter;)
}


/*!

@note The created sAttribute instance takes ownership of @c node_location.
*/
sAttribute::sAttribute ( rcsAttrName     attribute_name
                       , psNodeLocation  node_location
                       , tIOStatus       io_status)
    :TheName(attribute_name)
    ,TheValue(node_location)
    ,IOStatus(io_status)
    ,StatusTimeStamp()
{
  IF_DEBUG(++Counter;)
}


rsAttribute sAttribute::operator =(rcsAttribute  other)
{
  TheName = other.TheName;
  TheValue = other.TheValue;
  IOStatus = other.IOStatus;
  StatusTimeStamp = other.StatusTimeStamp;
  return *this;
}

//------------------------------------------------------------------------------
} //namespace RA

