////////////////////////////////////////////////////////////////////////////////
/*! @file Color.cpp   Classes sColor and sColorSet.
- Part of RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2013, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "Color.h"
#include "Trace.h"
#include <sstream>
using std::max;
using std::min;
namespace RA {
//------------------------------------------------------------------------------

const sColor                  sColor::cWhite     (255,255,255);
const sColor                  sColor::cGrey      (128,128,128);
const sColor                  sColor::cBlack     (  0,  0,  0);

const sColor                  sColor::cRed       (255,  0,  0);
const sColor                  sColor::cGreen     (  0,255,  0);
const sColor                  sColor::cBlue      (  0,  0,255);

const sColor                  sColor::cDarkRed   (128,  0,  0);
const sColor                  sColor::cDarkGreen (  0,128,  0);
const sColor                  sColor::cDarkBlue  (  0,  0,128);

const sColor                  sColor::cMagenta   (255,  0,255);
const sColor                  sColor::cYellow    (255,255,  0);
const sColor                  sColor::cCyan      (  0,255,255);

const sColor                  sColor::cPurple    (128,  0,128);
const sColor                  sColor::cOlive     (128,128,  0);
const sColor                  sColor::cTeal      (  0,128,128);

const sColor                  sColor::cChocolate (210,105, 30);
const sColor                  sColor::cCoral     (255,127, 80);
const sColor                  sColor::cOrange  	 (255,140,  0);
const sColor                  sColor::cOrange1 	 (255,128,  0);
const sColor                  sColor::cOrange2 	 (255,165,  0);
const sColor                  sColor::cGold  	   (255,215,  0);
/*
*/

//------------------------------------------------------------------- sColor ---

sColor::sColor (std::istream &  a_stream ,rcsVersion  version)
    :sStorable(a_stream,version)
{
  a_stream.READ_INTO(R);
  a_stream.READ_INTO(G);
  a_stream.READ_INTO(B);
  a_stream.READ_INTO(A);
}


void  sColor::Store (std::ostream &  a_stream) const
{
  sStorable::Store(a_stream);
  a_stream.WRITE_FROM(R);
  a_stream.WRITE_FROM(G);
  a_stream.WRITE_FROM(B);
  a_stream.WRITE_FROM(A);
}


sColor::sColor (rcsColor  C)
    :sStorable()
    ,R(C.R) ,G(C.G) ,B(C.B) ,A(C.A)
{
}


rsColor  sColor::operator = (rcsColor  C)
{
  A=C.A;  R=C.R;  G=C.G;  B=C.B;
  return *this;
}


bool  sColor::operator == (rcsColor  C) const
{
  return  ( R == C.R ) && ( G == C.G ) && ( B == C.B ) && ( A == C.A );
}


sString  sColor::HexARGB () const
{
  std::ostringstream          S;
  S.flags( std::ios::hex | std::ios::right | std::ios::uppercase);
  S << '#';
  if( A != 0xFF ){
    S.width(2);
    S.fill('0');
    S << (unsigned int)A;
  };
  S.width(2);
  S.fill('0');
  S << (unsigned int)R;
  S.width(2);
  S.fill('0');
  S << (unsigned int)G;
  S.width(2);
  S.fill('0');
  S << (unsigned int)B;
  return sString( S.str().c_str() );
}


sString  sColor::StyleTagRGBA () const
{
  std::ostringstream          S;
  S << "rgba(";
  S << (unsigned int)R;
  S << ",";
  S << (unsigned int)G;
  S << ",";
  S << (unsigned int)B;
  if( A != 0xFF ){
    S << ",";
    S.setf(std::ios::fixed,std::ios::floatfield);   // floatfield set to fixed
    S.precision(3);
    S << float((unsigned int)A) / 255.0;
  };
  S << ")";
  return sString( S.str().c_str() );
}


void  sColor::Bleach (/*short_*/real  BleachingStrength)
{
  R = min(integer(255)
         ,max(integer(0)
             ,Round(2.55e2 * (1 - (1 - R/2.55e2)/BleachingStrength ))));
  G = min(integer(255)
         ,max(integer(0)
             ,Round(2.55e2 * (1 - (1 - G/2.55e2)/BleachingStrength ))));
  B = min(integer(255)
         ,max(integer(0)
             ,Round(2.55e2 * (1 - (1 - B/2.55e2)/BleachingStrength ))));
}

//---------------------------------------------------------------- sColorSet ---

sColorSet  sColorSet::Basic ()
{
  return sColorSet()
            .Add(sColor(  0,  0,200),"blue")
            .Add(sColor(230,  0,  0),"red")
            .Add(sColor(  0,197, 30),"green")
            .Add(sColor(195, 97,  0),"chocolate")
            .Add(sColor(  0,200,200),"teal")
            .Add(sColor(255,171,  0),"orange")
            .Add(sColor(130,  0,200),"violet")
            .Add(sColor(223,  0,134),"crimson")
            .Add(sColor(145,203, 29),"olive")
            ;
}


sColorSet::~sColorSet ()
{
  Items.clear();
}


sColorSet::sColorSet ()
    :TheSize(0)
    ,TheCurrentIndex(0)
{
}


sColorSet::sColorSet (std::istream &  a_stream ,rcsVersion  version)
    :sStorable(a_stream,version)
{
  a_stream.READ_INTO(TheSize);
  a_stream.READ_INTO(TheCurrentIndex);
  for( int i = 0 ; i < TheSize ; i++ ){
    sColor                      C(a_stream,version);
    sString                     S;
    S.Read(a_stream);
    Items.insert(std::pair<int,sItem>(i,sItem(C,S)));
  }
}


void  sColorSet::Store (std::ostream &  a_stream) const
{
  sStorable::Store(a_stream);
  a_stream.WRITE_FROM(TheSize);
  a_stream.WRITE_FROM(TheCurrentIndex);
  for( int i = 0 ; i < TheSize ; i++ ){
    Items.at(i).Color.Store(a_stream);
    Items.at(i).Name.Write(a_stream);
  }
}


sColorSet::sColorSet (rcsColorSet  other)
    :sStorable()
    ,TheSize(other.TheSize)
    ,TheCurrentIndex(other.TheCurrentIndex)
{
  for( int i = 0 ; i < other.TheSize ; i++ ){
    Items.insert(std::pair<int,sItem>(i,sItem( other.Items.at(i).Color
                                             , other.Items.at(i).Name ) ));
  }
}


rsColorSet  sColorSet::operator = (rcsColorSet  other)
{
  TheSize = other.TheSize;
  TheCurrentIndex = other.TheCurrentIndex;
  Items.clear();
  for( int i = 0 ; i < other.TheSize ; i++ ){
    Items.insert(std::pair<int,sItem>(i,sItem( other.Items.at(i).Color
                                             , other.Items.at(i).Name ) ));
  }
  return *this;
}


void  sColorSet::Clear ()
{
  Items.clear();
  TheSize = 0;
  TheCurrentIndex = 0;
}


rsColorSet  sColorSet::Add (rcsColor   color
                           ,rcsString  name )
{
  Items.insert(std::pair<int,sItem>( TheSize++ , sItem(color,name) ));
  return *this;
}


sColor  sColorSet::CurrentColor () const
{
  if( (0 <= TheCurrentIndex) && (TheCurrentIndex < TheSize) ){
    return  sColor( Items.at(TheCurrentIndex).Color );
  }else{
    return  sColor();
  }
}


void  sColorSet::SetToFirst ()
{
  TheCurrentIndex = 0;
}


void   sColorSet::ShiftToNext ()
{
  if( TheSize > 0 ){
    TheCurrentIndex++;
    TheCurrentIndex = TheCurrentIndex % TheSize;
  }
}


//void   sColorSet::ShiftBack ()
//{
//  if( TheSize > 0 ){
//    TheCurrentIndex--;
//    TheCurrentIndex = TheCurrentIndex % TheSize;
//  }
//}


bool  sColorSet::SetCurrentIndex (int  index)
{
  if( (0 <= index) && (index < TheSize) ){
    TheCurrentIndex = index;
    return true;
  }
  return false;
}


sColorSet::sItem  sColorSet::operator [] (int  index) const
{
  if( (index >= 0) && (index < TheSize) ){
    return  Items.at(index);
  }else{
    return  sItem(sColor(),sString());
  }
}


//sColor  sColorSet::Color (int  index) const
//{
//  if( (index >= 0) && (index < TheSize) ){
//    return  sColor( Items.at(index).Color );
//  }else{
//    return  sColor();
//  }
//}


//sString  sColorSet::Name (int  index) const
//{
//  if( (index >= 0) && (index < TheSize) ){
//    return  Items.at(index).Name;
//  }else{
//    return  sString();
//  }
//}

void  sColorSet::Change (int        index
                        ,rcsColor   color
                        ,rcsString  name)
{
  if( (index >= 0) && (index < TheSize) ){
    Items.erase(index);
    Items.insert(std::pair<int,sItem>( index , sItem(color,name) ));
  }
}

//------------------------------------------------------------------------------
} //namespace RA

