////////////////////////////////////////////////////////////////////////////////
/*! @file Color.h   Classes sColor and sColorSet.
- Part of RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef Color_H
#define Color_H
#include "Storable.h"
#include <map>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sColor)
ANNOUNCE_CLASS(sColorSet)

//------------------------------------------------------------------- sColor ---

class RANet_EXPORT  sColor : public virtual sStorable
{
  STORABLE(sColor)
  public: //static
    static const sColor       cWhite;
    static const sColor       cGrey;
    static const sColor       cBlack;
    static const sColor       cRed;
    static const sColor       cGreen;
    static const sColor       cBlue;
    static const sColor       cDarkRed;
    static const sColor       cDarkGreen;
    static const sColor       cDarkBlue;
    static const sColor       cMagenta;
    static const sColor       cYellow;
    static const sColor       cCyan;
    static const sColor       cPurple;
    static const sColor       cOlive;
    static const sColor       cTeal;
    static const sColor       cChocolate;
    static const sColor       cCoral;
    static const sColor       cOrange;
    static const sColor       cOrange1;
    static const sColor       cOrange2;
    static const sColor       cGold;

  public:
                              sColor (byte red ,byte green ,byte blue
                                     ,byte alpha = 0xFF)
                                  :R(red),G(green),B(blue),A(alpha)
                                {}
                              sColor ()
                                  :R(0),G(0),B(0),A(0xFF)
                                {}
                              sColor (rcsColor);
    rsColor                   operator = (rcsColor);
    bool                      operator == (rcsColor) const;
    bool                      operator != (rcsColor C) const
                                { return !operator==(C); }
    int                       Red () const
                                { return R; }
    int                       Green () const
                                { return G; }
    int                       Blue () const
                                { return B; }
    int                       Alpha () const
                                { return A; }
    sString                   HexARGB () const;
    sString                   StyleTagRGBA () const;
    void                      Bleach (/*short_*/real  BleachingStrength = 2.0);

  private: //fields
    byte                      R;
    byte                      G;
    byte                      B;
    byte                      A;
};

//---------------------------------------------------------------- sColorSet ---
/*! Набор цветов со встроенным механизмом последовательного (циклически
замкнутого) перечисления элементов для назначения цветов кривым и т.п.

*/
class RANet_EXPORT  sColorSet : public virtual sStorable
{
  STORABLE(sColorSet)
  public: //type
    ANNOUNCE_CLASS(sItem)
    class RANet_EXPORT  sItem
    {
      public:
                                  sItem (rcsColor   color
                                        ,rcsString  name )
                                      :Color(color),Name(name)
                                    {}
                                  sItem (rcsItem  other)
                                      :Color(other.Color),Name(other.Name)
                                    {}
      public:
        csColor                   Color;
        csString                  Name;
    };

  public: //static
    static sColorSet          Basic ();/*!< возвращает базовый набор цветов*/

  public:
    virtual                   ~sColorSet ();
                              sColorSet ();/*!< порождает пустой набор цветов*/
                              sColorSet (rcsColorSet);
    rsColorSet                operator = (rcsColorSet);
    virtual bool              Empty () const
                                { return TheSize == 0; }
    virtual int               Size () const
                                { return TheSize; }
    virtual void              Clear ();
    virtual rsColorSet        Add (rcsColor   color
                                  ,rcsString  name = sString() );
    virtual sColor            CurrentColor () const;
    virtual void              SetToFirst ();
    virtual void              ShiftToNext ();
//    virtual void              ShiftBack ();
    virtual int               CurrentIndex () const
                                { return TheCurrentIndex; }
    virtual bool              SetCurrentIndex (int  index);
    virtual sItem             operator [] (int  index) const;
//    virtual sColor            Color (int  index) const;
//    virtual sString           Name (int  index) const;
    virtual void              Change (int        index
                                     ,rcsColor   color
                                     ,rcsString  name = sString() );

  private: //fields
    int                       TheSize;
    int                       TheCurrentIndex;
    std::map<int,sItem>       Items;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
