////////////////////////////////////////////////////////////////////////////////
/*! @file ColorNodes.cpp   Цветовые объекты сети.
- Part of RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2015, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "ColorNodes.h"
#include "Symbols.h"
#include <cstring>
#include <QMutexLocker>
namespace RA {
//------------------------------------------------------------------------------
//--------------------------------------------------------------- sColorNode ---

sColorNode::sColorNode (std::istream &  a_stream ,rcsVersion  version)
  :sTerminalNode(a_stream,version)
  ,TheValue(a_stream,version)
{
}


void sColorNode::Store (std::ostream &  a_stream) const
{
  sTerminalNode::Store(a_stream);
  TheValue.Store(a_stream);
}


sString  sColorNode::Text (eTextFormat        F
                          ,eTextDetalization  D)
{
  return Value().StyleTagRGBA()
        +( D==Laconic
         ? ""
         : ( F==HTML ? sString("<font color=")+Value().HexARGB()+">"
                     : sString() )
           +", i.e. "
           +( F==HTML ? "</font>" : "" )
           +Value().HexARGB()
         );
}


sColor  sColorNode::Value ()
{
  QMutexLocker                BodyLocker(&BodyMutex);
  return TheValue;
}


void  sColorNode::Assign (sColor  V)
{
  QMutexLocker                BodyLocker(&BodyMutex);
  if( TheValue != V ){
    TheValue = V;
    MarkAsEdited();
  }
}


void  sColorNode::CopyFrom (psNode  other)
{
  sTerminalNode::CopyFrom(other);
  psColorNode                 Other = dynamic_cast<psColorNode>(other);
  if( Other && (TheValue != Other->Value()) ){
    TheValue = Other->Value();
    MarkAsEdited();
  }
}


//------------------------------------------------------------ sColorSetNode ---

sColorSetNode::sColorSetNode (std::istream &  a_stream ,rcsVersion  version)
  :sTerminalNode(a_stream,version)
  ,TheValue(a_stream,version)
{
}


void sColorSetNode::Store (std::ostream &  a_stream) const
{
  sTerminalNode::Store(a_stream);
  TheValue.Store(a_stream);
}


sString  sColorSetNode::Text (eTextFormat        /*F*/
                             ,eTextDetalization  /*D*/)
{
  QMutexLocker                BodyLocker(&BodyMutex);
  sString                     T("set of ");
  T << TheValue.Size();
  T += " colors";
  return T;
}


sColorSet  sColorSetNode::Value ()
{
  QMutexLocker                BodyLocker(&BodyMutex);
  return TheValue;
}


void  sColorSetNode::Assign (sColorSet  V)
{
  QMutexLocker                BodyLocker(&BodyMutex);
  TheValue=V;
  MarkAsEdited();
}


bool sColorSetNode::Empty()
{
  QMutexLocker                BodyLocker(&BodyMutex);
  return TheValue.Empty();
}


sColor  sColorSetNode::CurrentColor ()
{
  QMutexLocker                BodyLocker(&BodyMutex);
  return TheValue.CurrentColor();
}


void  sColorSetNode::SetToFirst ()
{
  QMutexLocker                BodyLocker(&BodyMutex);
  TheValue.SetToFirst();
  MarkAsEdited();
}


void sColorSetNode::ShiftToNext ()
{
  QMutexLocker                BodyLocker(&BodyMutex);
  TheValue.ShiftToNext();
  MarkAsEdited();
}


void  sColorSetNode::CopyFrom (psNode  other)
{
  sTerminalNode::CopyFrom(other);
  psColorSetNode              Other = dynamic_cast<psColorSetNode>(other);
  if( Other ){
    TheValue = Other->Value();
    MarkAsEdited();
  }
}

//------------------------------------------------------------------------------
} //namespace RA

