////////////////////////////////////////////////////////////////////////////////
/*! @file ColorNodes.h   Цветовые объекты сети.
- Part of RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2015, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef ColorNodes_H
#define ColorNodes_H
#include "BasicNodes.h"
#include "Color.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_ABSTRACT_CLASS(sColorNode)
ANNOUNCE_CLASS(sColorSetNode)

//--------------------------------------------------------------- sColorNode ---
/*! Объект сети данных типа цвет.

*/
class RANet_EXPORT  sColorNode : public sTerminalNode
{
  STORABLE(sColorNode)

  public:
                              sColorNode ()
                                  :sTerminalNode(),TheValue()
                                {}
                              sColorNode (sColor  V)
                                  :sTerminalNode(),TheValue(V)
                                {}
    virtual sString           Text (eTextFormat        F
                                   ,eTextDetalization  D
                                   ); //!< visual representation for user
                              operator sColor ()
                                { return Value(); }
    virtual sColor            Value ();
    virtual void              Assign (sColor);

  protected:
    virtual void              CopyFrom (psNode);

  private:
    sColor                    TheValue;
};


//------------------------------------------------------------ sColorSetNode ---
/*! Объект сети данных типа набор цветов.

*/
class RANet_EXPORT  sColorSetNode : public sTerminalNode
{
  STORABLE(sColorSetNode)

  public:
                              sColorSetNode ()
                                  :sTerminalNode(),TheValue()
                                {}
                              sColorSetNode (rcsColorSet  V)
                                  :sTerminalNode(),TheValue(V)
                                {}
    virtual sString           Text (eTextFormat        F
                                   ,eTextDetalization  /*D*/
                                   ); //!< visual representation for user
                              operator sColorSet ()
                                { return Value(); }
    virtual sColorSet         Value ();
    virtual void              Assign (sColorSet);
    virtual bool              Empty ();
    virtual sColor            CurrentColor ();
    virtual void              SetToFirst ();
    virtual void              ShiftToNext ();

  protected:
    virtual void              CopyFrom (psNode);

  private:
    sColorSet                 TheValue;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif

