////////////////////////////////////////////////////////////////////////////////
/*! @file DataArrayFolder.h   Зависимость, заданная массивом ординат.
- Part of RANet - Research Assistant Net Library.
- Copyright(C) 2010-2017, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef DataArrayFolder_H
#define DataArrayFolder_H
#include "ArrayDataFunction.h"
#include "FolderNode.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sDataArrayFolder)

//--------------------------------------------------------- sDataArrayFolder ---
/*! Зависимость, заданная массивом ординат.

В ней число точек фиксировано и точки индексируются (то есть можно доступаться
по номеру).  Абциссы заведомо эквидистантны и задаются начальной точкой и шагом.
*/
class RANet_EXPORT  sDataArrayFolder : public sArrayDataFunction
                                     , public sFolderNode
{
  STORABLE(sDataArrayFolder)
  public:
    virtual                   ~sDataArrayFolder ();
    explicit                  sDataArrayFolder (int   number_of_points
                                               ,real  x0 = 0
                                               ,real  step = 1);
                              sDataArrayFolder (rcsPhysPair  multiplier
                                               ,int          number_of_points
                                               ,real         x0 = 0
                                               ,real         step = 1);
                              sDataArrayFolder (sString      docket
                                               ,rcsPhysPair  multiplier
                                               ,int          number_of_points
                                               ,real         x0 = 0
                                               ,real         step = 1);
    virtual void              SetAppearance (psAppearance);
    virtual sString           Hint ()
                                { return sFolderNode::Text(Plain,Laconic); }
//    virtual sBoundaries       Boundaries ();
    virtual sString           Text (eTextFormat ,eTextDetalization);
    virtual psMathValue       MathValue ();
    virtual void              SetValue (int N ,real  V);

  protected:
    virtual void              CopyFrom (psNode);
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
