////////////////////////////////////////////////////////////////////////////////
/*! @file DataArrayRecord.h   A record with a data array (as a dependence).
- Part of RANet - Research Assistant Net Library.
- Copyright(C) 2010-2017, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef DataArrayRecord_H
#define DataArrayRecord_H
#include "ArrayDataFunction.h"
#include "Record.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sDataArrayRecord)

//--------------------------------------------------------- sDataArrayRecord ---
/*! A record with a data array (interpreted as a dependence).

В ней число точек фиксировано и точки индексируются (то есть можно доступаться
по номеру).  Абциссы заведомо эквидистантны и задаются начальной точкой и шагом.
*/
class RANet_EXPORT  sDataArrayRecord : public sArrayDataFunction
                                     , public sRecord
{
  STORABLE(sDataArrayRecord)
  public:
    virtual                   ~sDataArrayRecord ();
    explicit                  sDataArrayRecord (int   number_of_points
                                               ,real  x0 = 0
                                               ,real  step = 1);
                              sDataArrayRecord (rcsPhysPair  multiplier
                                               ,int          number_of_points
                                               ,real         x0 = 0
                                               ,real         step = 1);
                              sDataArrayRecord (int          record_no
                                               ,rcsPhysPair  multiplier
                                               ,int          number_of_points
                                               ,real         x0 = 0
                                               ,real         step = 1);
    virtual void              SetAppearance (psAppearance);
    virtual sString           Hint ()
                                { return sRecord::Text(Plain,Laconic); }
//    virtual sBoundaries       Boundaries ();
    virtual sString           Text (eTextFormat ,eTextDetalization);
    virtual psMathValue       MathValue ();
    virtual void              SetValue (int N ,real  V);

  protected:
    virtual void              CopyFrom (psNode);
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
