////////////////////////////////////////////////////////////////////////////////
/*! @file DataListFolder.cpp   Зависимость, заданная списком точек.
- Part of RANet - Research Assistant Net Library.
- Copyright(C) 2010-2017, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#include "DataListFolder.h"
//#include "Log.h"
namespace RA {
//------------------------------------------------------------------------------
//---------------------------------------------------------- sDataListFolder ---

sDataListFolder::~sDataListFolder ()
{
}


sDataListFolder::sDataListFolder (sString  docket)
    :sListDataFunction()
    ,sFolderNode(docket)
{
}


sDataListFolder::sDataListFolder (sString      docket
                                 ,rcsPhysPair  multiplier)
    :sListDataFunction(multiplier)
    ,sFolderNode(docket)
{
}


sDataListFolder::sDataListFolder (rcsPhysPair  multiplier)
    :sListDataFunction(multiplier)
    ,sFolderNode()
{
}


sDataListFolder::sDataListFolder (std::istream &  a_stream
                                 ,rcsVersion      version)
    :sListDataFunction(a_stream,version)
    ,sFolderNode(a_stream,version)
{
  if( version <= sVersion(0,9) ){
    sPhysPair                     M(a_stream,version);
    TheMultiplier = M;
    int                           NumberOfPoints;
    sDataPoint                    DP;
    realIOv14                        R;
    a_stream.READ_INTO(Sorted);
    a_stream.READ_INTO(NumberOfPoints);
    for( int PointNo=0 ; PointNo < NumberOfPoints ; PointNo++ ){
      a_stream.READ_INTO(R);  DP.X=R;
      a_stream.READ_INTO(R);  DP.Y=R;
      ThePoints.push_back(DP);
    }
  }
}


void  sDataListFolder::Store (std::ostream & a_stream) const
{
  sListDataFunction::Store(a_stream);
  sFolderNode::Store(a_stream);
}


void  sDataListFolder::CopyFrom (psNode  other)
{
  sFolderNode::CopyFrom(other);
  psDataListFolder        Other = dynamic_cast<psDataListFolder>(other);
  if( Other ){
    Sorted = Other->Sorted;
    ThePoints.clear();
    for( std::list<sDataPoint>::const_iterator Finger = Other->ThePoints.begin()
                               ; Finger != Other->ThePoints.end() ; Finger++ ){
      ThePoints.push_back(*Finger);
    };
    MarkAsEdited();
  }
}


//sCurve::psTracer  sDataListFolder::GetCurveTracer (rcsScales  scales) const
//{

//  //!@patch Та же проблема, что и с границами: при просмотре редактируется
//  //if( ! Sorted ){
//  //  ThePoints.sort();
//  //  Sorted=true;
//  //  MarkAsEdited();
//  //}

//  return new sDataListFolder::sTracer(this,scales);
//}


sString  sDataListFolder::Text (eTextFormat        format
                               ,eTextDetalization  detalization)
{
  sString                     T;
  T = sFunction::Text(format,detalization);
  T += sFolderNode::Text(format,detalization);
  if( detalization > Laconic ){
    T += sString(" [") + TheMultiplier.Y.Units().Text(format)
        +sString("(") + TheMultiplier.X.Units().Text(format) + sString(")]");
  }
  return T;
}


void  sDataListFolder::SetAppearance (psAppearance  appearance)
{
  QMutexLocker                BodyLocker(&BodyMutex);
  sListDataFunction::SetAppearance(appearance);
  MarkAsEdited();
}


psMathValue  sDataListFolder::MathValue ()
{
  QMutexLocker                BodyLocker(&BodyMutex);
  return Replica();
}


void  sDataListFolder::AddNewPoint (sPoint  P)
{
  QMutexLocker                BodyLocker(&BodyMutex);
  sListDataFunction::AddNewPoint(P);
  MarkAsEdited();
}

//------------------------------------------------------------------------------
} //namespace RA


