////////////////////////////////////////////////////////////////////////////////
/*! @file DataListFolder.h   Зависимость, заданная списком точек.
- Part of RANet - Research Assistant Net Library.
- Copyright(C) 2010-2017, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef DataListFolder_H
#define DataListFolder_H
#include "ListDataFunction.h"
#include "FolderNode.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sDataListFolder)

//---------------------------------------------------------- sDataListFolder ---
/*! Зависимость, заданная списком точек.

В отличие от sDataArrayFolder и от sDataTableFolder в этом типе
представления зависимости число точек заранее не задаётся, их можно добавлять.
Абциссы произвольны. При добавлении точек список автоматически сортируется по
неубыванию абцисс. Как следствие, доступ по номеру и редактирование точки не
предусмотрены.
*/
class RANet_EXPORT  sDataListFolder : public sListDataFunction
                                    , public sFolderNode
{
  STORABLE(sDataListFolder)
  public:
    virtual                   ~sDataListFolder ();
      explicit                sDataListFolder (sString  docket =sString());
                              sDataListFolder (sString      docket
                                              ,rcsPhysPair  multiplier);
      explicit                sDataListFolder (rcsPhysPair  multiplier);
//    virtual sCurve::psTracer  GetCurveTracer (rcsScales  scales) const;
    virtual void              SetAppearance (psAppearance);
    virtual sString           Hint ()
                                { return sFolderNode::Text(Plain,Laconic); }
//    virtual sBoundaries       Boundaries ();
    virtual sString           Text (eTextFormat ,eTextDetalization);
    virtual psMathValue       MathValue ();
    virtual void              AddNewPoint (sPoint);

    //!@todo{?} Why should I repeat this definition from sListDataFunction?
    virtual void              AddNewPoint (real  x ,real  y)
                                { AddNewPoint(sPoint(x,y)); }

  protected:
    virtual void              CopyFrom (psNode);
};

//------------------------------------------------------------------------------
} //namespace RA
#endif

