////////////////////////////////////////////////////////////////////////////////
/*! @file DataListRecord.h   A record with a data list (as a dependence).
- Part of RANet - Research Assistant Net Library.
- Copyright(C) 2010-2017, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef DataListRecord_H
#define DataListRecord_H
#include "ListDataFunction.h"
#include "Record.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sDataListRecord)

//---------------------------------------------------------- sDataListRecord ---
/*! A record with a data list (interpreted as a dependence).

В отличие от sDataArrayRecord и от sDataTableRecord в этом типе
представления зависимости число точек заранее не задаётся, их можно добавлять.
Абциссы произвольны. При добавлении точек список автоматически сортируется по
неубыванию абцисс. Как следствие, доступ по номеру и редактирование точки не
предусмотрены.
*/
class RANet_EXPORT  sDataListRecord : public sListDataFunction
                                    , public sRecord
{
  STORABLE(sDataListRecord)
  public:
    virtual                   ~sDataListRecord ();
                              sDataListRecord ();
      explicit                sDataListRecord (rcsPhysPair  multiplier);
//    virtual sCurve::psTracer  GetCurveTracer (rcsScales  scales) const;
    virtual void              SetAppearance (psAppearance);
    virtual sString           Hint ()
                                { return sRecord::Text(Plain,Laconic); }
//    virtual sBoundaries       Boundaries ();
    virtual sString           Text (eTextFormat ,eTextDetalization);
    virtual psMathValue       MathValue ();
    virtual void              AddNewPoint (sPoint);

    //!@todo{?} Why should I repeat this definition from sListDataFunction?
    virtual void              AddNewPoint (real  x ,real  y)
                                { AddNewPoint(sPoint(x,y)); }

  protected:
    virtual void              CopyFrom (psNode);
};

//------------------------------------------------------------------------------
} //namespace RA
#endif

