////////////////////////////////////////////////////////////////////////////////
/*! @file DataTableFolder.cpp Таблично-заданная зависимость.
- Part of RANet - Research Assistant Net Library.
- Copyright(C) 2010-2017, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#include "DataTableFolder.h"
//#include "Log.h"
namespace RA {
//------------------------------------------------------------------------------
//--------------------------------------------------------- sDataTableFolder ---

sDataTableFolder::~sDataTableFolder ()
{
  if( XValues ){
    delete [] XValues;  XValues=NULL;
  };
  if( YValues ){
    delete [] YValues;  YValues=NULL;
  };
}


sDataTableFolder::sDataTableFolder (int  number_of_points)
    :sTableDataFunction(number_of_points)
    ,sFolderNode()
{
}


sDataTableFolder::sDataTableFolder (rcsPhysPair  multiplier
                                   ,int          number_of_points)
    :sTableDataFunction(multiplier,number_of_points)
    ,sFolderNode()
{
}


sDataTableFolder::sDataTableFolder (sString      docket
                                   ,rcsPhysPair  multiplier
                                   ,int          number_of_points)
    :sTableDataFunction(multiplier,number_of_points)
    ,sFolderNode(docket)
{
}


sDataTableFolder::sDataTableFolder (std::istream &  a_stream
                                   ,rcsVersion      version)
    :sTableDataFunction(a_stream,version)
    ,sFolderNode(a_stream,version)
{
  if( version <= sVersion(0,9) ){
    sPhysPair                 M(a_stream,version);
    TheMultiplier = M;
    a_stream.READ_INTO(TheNumberOfPoints);
    ASSERT( TheNumberOfPoints > 0 );
    XValues= new real [TheNumberOfPoints];
    YValues= new real [TheNumberOfPoints];
    realIOv14 R;
    for( int PointNo=0 ; PointNo < TheNumberOfPoints ; PointNo++ ){
      a_stream.READ_INTO(R);  XValues[PointNo]=R;
      a_stream.READ_INTO(R);  YValues[PointNo]=R;
    }
  }
}


void  sDataTableFolder::Store (std::ostream & a_stream) const
{
  sTableDataFunction::Store(a_stream);
  sFolderNode::Store(a_stream);
}


void  sDataTableFolder::CopyFrom (psNode  other)
{
  sFolderNode::CopyFrom(other);
  psDataTableFolder      Other = dynamic_cast<psDataTableFolder>(other);
  if( Other ){
    if( TheNumberOfPoints != Other->TheNumberOfPoints ){
      if( XValues ){
        delete [] XValues;  XValues = NULL;
      }
      if( YValues ){
        delete [] YValues;  YValues = NULL;
      }
      TheNumberOfPoints = Other->TheNumberOfPoints;
      if( TheNumberOfPoints > 0 ){
        XValues = new real [TheNumberOfPoints];
        YValues = new real [TheNumberOfPoints];
      }
    }
    for( int PointNo=0 ; PointNo < TheNumberOfPoints ; PointNo++ ){
      XValues[PointNo] = Other->XValues[PointNo];
      YValues[PointNo] = Other->YValues[PointNo];
    };
    MarkAsEdited();
  }
}


//sCurve::psTracer  sDataTableFolder::GetCurveTracer (rcsScales  scales) const
//{
//  return new sDataTableFolder::sTracer(this,scales);
//}


sString  sDataTableFolder::Text (eTextFormat        format
                                ,eTextDetalization  detalization)
{
  sString                     T;
  T = sFunction::Text(format,detalization);
  T += sFolderNode::Text(format,detalization);
  if( detalization > Laconic ){
    T += sString(" [") + TheMultiplier.Y.Units().Text(format)
        +sString("(") + TheMultiplier.X.Units().Text(format) + sString(")]");
  }
  return T;
}


void  sDataTableFolder::SetAppearance (psAppearance  appearance)
{
  QMutexLocker                BodyLocker(&BodyMutex);
  sTableDataFunction::SetAppearance(appearance);
  MarkAsEdited();
}


psMathValue  sDataTableFolder::MathValue ()
{
  QMutexLocker                BodyLocker(&BodyMutex);
  return Replica();
}


void  sDataTableFolder::SetValue (int N ,rcsPoint P)
{
  QMutexLocker                BodyLocker(&BodyMutex);
  if( (N >= 0) && (N < TheNumberOfPoints) ){
    XValues[N]=P.X;
    YValues[N]=P.Y;
//    if( ! RecalculateBoundaries )
//      LastBoundaries |= sPhysPair( P.X*Multiplier().X , P.Y*Multiplier().Y );
    RecalculateBoundaries = true;
    MarkAsEdited();
  }
}


void  sDataTableFolder::SetXValue (int N ,real  X)
{
  QMutexLocker                BodyLocker(&BodyMutex);
  if( (N >= 0) && (N < TheNumberOfPoints) ){
    XValues[N]=X;
//    if( ! RecalculateBoundaries )
//      LastBoundaries.X |= X * Multiplier().X;
    RecalculateBoundaries = true;
    MarkAsEdited();
  }
}


void  sDataTableFolder::SetYValue (int N ,real  Y)
{
  QMutexLocker                BodyLocker(&BodyMutex);
  if( (N >= 0) && (N < TheNumberOfPoints) ){
    YValues[N]=Y;
//    if( ! RecalculateBoundaries )
//      LastBoundaries.Y |= Y * Multiplier().Y;
    RecalculateBoundaries = true;
    MarkAsEdited();
  }
}

//------------------------------------------------------------------------------
} //namespace RA




