////////////////////////////////////////////////////////////////////////////////
/*! @file DataTableFolder.h   Таблично-заданная зависимость.
- Part of RANet - Research Assistant Net Library.
- Copyright(C) 2010-2017, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef DataTableFolder_H
#define DataTableFolder_H
#include "TableDataFunction.h"
#include "FolderNode.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sDataTableFolder)

//--------------------------------------------------------- sDataTableFolder ---
/*! Таблично-заданная зависимость.

В нём число точек фиксировано и точки индексируются (то есть можно доступаться
по номеру), как в sDataArrayFolder, но абциссы произвольны, как в
sDataListFolder.

Упорядоченность абцисс - на совести программиста.
*/
class RANet_EXPORT  sDataTableFolder : public sTableDataFunction
                                     , public sFolderNode
{
  STORABLE(sDataTableFolder)
  public:
    virtual                   ~sDataTableFolder ();
    explicit                  sDataTableFolder (int   number_of_points);
                              sDataTableFolder (rcsPhysPair  multiplier
                                               ,int          number_of_points);
                              sDataTableFolder (sString      docket
                                               ,rcsPhysPair  multiplier
                                               ,int          number_of_points);
//    virtual sCurve::psTracer  GetCurveTracer (rcsScales  scales) const;
    virtual void              SetAppearance (psAppearance);
    virtual sString           Hint ()
                                { return sFolderNode::Text(Plain,Laconic); }
//    virtual sBoundaries       Boundaries ();
    virtual sString           Text (eTextFormat ,eTextDetalization);
    virtual psMathValue       MathValue ();
    virtual void              SetValue (int N ,rcsPoint);
    virtual void              SetXValue (int N ,real  X);
    virtual void              SetYValue (int N ,real  Y);

  protected:
    virtual void              CopyFrom (psNode);
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
