////////////////////////////////////////////////////////////////////////////////
/*! @file DataTableRecord.h   A record with a data table (as a dependence).
- Part of RANet - Research Assistant Net Library.
- Copyright(C) 2010-2017, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef DataTableRecord_H
#define DataTableRecord_H
#include "TableDataFunction.h"
#include "Record.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sDataTableRecord)

//--------------------------------------------------------- sDataTableRecord ---
/*! A record with a data table (interpreted as a dependence).

В нём число точек фиксировано и точки индексируются (то есть можно доступаться
по номеру), как в sDataArrayRecord, но абциссы произвольны, как в
sDataListRecord.

Упорядоченность абцисс - на совести программиста.
*/
class RANet_EXPORT  sDataTableRecord : public sTableDataFunction
                                     , public sRecord
{
  STORABLE(sDataTableRecord)
  public:
    virtual                   ~sDataTableRecord ();
    explicit                  sDataTableRecord (int   number_of_points);
                              sDataTableRecord (rcsPhysPair  multiplier
                                               ,int          number_of_points);
//    virtual sCurve::psTracer  GetCurveTracer (rcsScales  scales) const;
    virtual void              SetAppearance (psAppearance);
    virtual sString           Hint ()
                                { return sRecord::Text(Plain,Laconic); }
//    virtual sBoundaries       Boundaries ();
    virtual sString           Text (eTextFormat ,eTextDetalization);
    virtual psMathValue       MathValue ();
    virtual void              SetValue (int N ,rcsPoint);
    virtual void              SetXValue (int N ,real  X);
    virtual void              SetYValue (int N ,real  Y);

  protected:
    virtual void              CopyFrom (psNode);
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
