////////////////////////////////////////////////////////////////////////////////
/*! @file Device.cpp   Classes sDevice and sDeviceTag.
- Part of RANet - Research Assistant Net Library.
- Copyright(C) 2010-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "Device.h"
#include "BasicNodes.h"
#include "attr.h"
namespace RA {
//------------------------------------------------------------------------------

//--------------------------------------------------------------- sDeviceTag ---

sDeviceTag::sDeviceTag (literal          classname
                       ,eInstrumentType  classtype
                       ,pfPreconfigure   preconf)
    :sInstrumentTag(classname,classtype)
    ,ThePreconfigureFunctionPtr(preconf)
{
  AvailableClassList[sString(classname)]=classtype;
}

//------------------------------------------------------------------ sDevice ---
/*! @class sDevice


@todo{Instruments} Add functionality: Иерархию классов по именам

*/


void  sDevice::Preconfigure (sNodePtr  DeviceCfg)
{
  if( ! DeviceCfg.IsNULL() ){
    DeviceCfg |= attr("Disabled",false);
    DeviceCfg |= attr("TraceStates",false);
  }
}


bool  sDevice::IsRunning (sString  DeviceId)
{
  map<sString,psDevice>::iterator    D = RunningDevicesByName
                                                            .find(DeviceId);
  return ( D != RunningDevicesByName.end() );
}


bool  sDevice::Matches (sString  ClassName
                       ,sString  /*BaseClassName*/)
{
  //! @patch{Instruments} Device genealogy should be based on Adam's genealogy
  return IsDriver(ClassName);
}


sString  sDevice::InstallNew (sString  ClassName)
{
  if( DeviceList.IsCorrect() ){
    sPtr<sFolderNode>              DeviceCfg;
    sString                     DeviceId;
    int                         i = 0;
    do{
      DeviceId = ClassName.Tail(1) + "-";
      DeviceId << (++i);
      DeviceCfg = DeviceList >> DeviceId;
    }while( DeviceCfg.IsCorrect() );
    DeviceList |= DeviceId;
    DeviceCfg = DeviceList >> DeviceId;
    if( DeviceCfg.IsCorrect() ){
      DeviceCfg << attr("Class",ClassName);
      sPtr<sTextNode>             ClassNameNP;
      ClassNameNP = DeviceCfg >> "Class";
      if( ClassNameNP.IsCorrect() ){
        pcsTag                      Tag = sTag::ByName(ClassName);
        if( Tag ){
          pcsDeviceTag         DTag = dynamic_cast<pcsDeviceTag>(Tag);
          if( DTag ){
            (*(DTag->ThePreconfigureFunctionPtr))(DeviceCfg);
          }else{
            throw xProgramError("sDevice::InstallNew(): wrong ClassName"
                               ,__FILE__,__LINE__);
          }
        }else{
          throw xProgramError("sDevice::InstallNew(): wrong ClassName"
                             ,__FILE__,__LINE__);
        }
        return DeviceId;
      }
    }
  }
  return sString();
}


sDevice::~sDevice ()
{
  int n = RunningDevicesByName.erase(DeviceIdentifier());
  ABORT_IF_NOT(n==1);
  DeviceLinks.SetToNULL();
  DeviceSettings.SetToNULL();
  DeviceConfig.SetToNULL();
}


sDevice::sDevice (sString   DeviceId
                 ,sNodePtr  DeviceCfg)
    :DeviceConfig(DeviceCfg)
    ,TheDeviceIdentifier(DeviceId)
    ,InvalidFlag(false)
    ,TraceStatesFlag(false)
{
  std::pair<map<sString,psDevice>::iterator,bool>  P;
  P=RunningDevicesByName.insert(
          std::pair<sString,psDevice>(DeviceIdentifier(),this)
                                          );
  ASSERT(P.second);
//  можно проще, поскольку здесь проверка не нужна:
//  CreatedDriverByName[DeviceIdentifier]=this;

  //!@todo{NetIO} Обращение к сети в конструкторе sDevice.  Вытащить?
  DeviceSettings = DeviceConfig >> "Settings";
  DeviceLinks = DeviceConfig >> "Links";
  get_from<sBooleanNode>(DeviceConfig>>"TraceStates",TraceStatesFlag);
}


void sDevice::SetInvalid (const QString &  reason)
{
  InvalidFlag=true;
  if( ! TheInvalidityReason.isEmpty() )
    TheInvalidityReason+='\n';
  TheInvalidityReason+=reason;
}

//------------------------------------------------------------------------------
} //namespace RA
