////////////////////////////////////////////////////////////////////////////////
/*! @file Device.h   Classes sDevice and sDeviceTag.
- Part of RANet - Research Assistant Net Library.
- Copyright(C) 2010-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef Device_H
#define Device_H
#include "Instrument.h"
#include "Excepts.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sDeviceTag)
ANNOUNCE_ABSTRACT_CLASS(sDevice)
ANNOUNCE_CLASS(xDeviceLaunch)

//-------- predeclaration
ANNOUNCE_ABSTRACT_CLASS(sDriverLink)
ANNOUNCE_ABSTRACT_CLASS(sDriver)
ANNOUNCE_ABSTRACT_CLASS(sConsole)

//--------------------------------------------------------------- sDeviceTag ---

class RANet_EXPORT  sDeviceTag : public sInstrumentTag
{
  protected: // types
    typedef void              fPreconfigure (sNodePtr);
    typedef fPreconfigure *   pfPreconfigure;

  protected:
                              sDeviceTag (literal          classname
                                         ,eInstrumentType  classtype
                                         ,pfPreconfigure   preconf);
  private: // fields
    pfPreconfigure            ThePreconfigureFunctionPtr;

  friend class sDevice;
  friend class sDriverLink;
  friend class sConsole;
};


//------------------------------------------------------------------ sDevice ---
/*! Абстрактный базовый класс для классов, обслуживающих устройства.

Обеспечивает следующую функциональность:
- интерактивную инсталляцию устройств
  (создание соответствующего кластера в конфиге);
- иерархическое задание параметров (Preconfigure);
- запуск не более одного устройства с заданным идентификатором
  (TheDeviceIdentifier).

*/
class RANet_EXPORT  sDevice : virtual public sInstrument
{
  ADAM(sDevice)

  public: //types
    enum                      eControlChange
                                {cLeaveControl
                                ,cTakeControl
                                ,cReleaseControl
                                };

  public: //static
    static bool               NoneIsRunning ()
                                { return RunningDevicesByName.empty(); }
    static void               Preconfigure (sNodePtr  DevCfg);
    static bool               IsRunning (sString  DeviceId);
    static bool               Matches (sString  ClassName
                                      ,sString  BaseClassName);
    static sString            InstallNew (sString  ClassName);

  public: //static fields
    static sPtr<sFolderNode>     DeviceList;

  public:
    virtual                   ~sDevice ();
    sString                   DeviceIdentifier () const
                                { return TheDeviceIdentifier; }
    bool                      IsInvalid () const
                                { return InvalidFlag; }
    virtual void              SetInvalid (const QString &  reason =QString());
    QString                   InvalidityReason () const
                                { return TheInvalidityReason; }
    bool                      TraceStates () const
                                { return TraceStatesFlag; }

  protected: // static
    static map<sString
              ,psDevice>      RunningDevicesByName;

  protected:
                              sDevice (sString   DeviceId
                                      ,sNodePtr  DeviceCfg);

  protected: //fields
    sNodePtr                  DeviceConfig;
    sPtr<sFolderNode>         DeviceSettings;
    sPtr<sFolderNode>         DeviceLinks;

  private: // это, запрещенные операции над sDevice
                              sDevice (rcsDevice);
    void                      operator = (rcsDevice);
    void                      operator & ();

  private: //fields
    sString                   TheDeviceIdentifier;
    bool                      InvalidFlag;
    QString                   TheInvalidityReason;
    bool                      TraceStatesFlag;
};


//------------------------------------------------------------ xDeviceLaunch ---

class RANet_EXPORT  xDeviceLaunch : public xProgramError
{
  public:
                              xDeviceLaunch (rcsString  text)
                                  :xProgramError
                                      (text,"",0,"xDeviceLaunch")
                                {}
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
