////////////////////////////////////////////////////////////////////////////////
/*! @file Environment.cpp   Среда работы программы и глобальные опции.
- Part of RANet - Research Assistant Net Library.
- Uses  Qt v.5  - http://qt.io/
- Copyright(C) 2010-2020, Viktor E. Bursian, St.Petersburg, Russia.
                          VBursian AT gmail DOT com
*///////////////////////////////////////////////////////////////////////////////
#include "Environment.h"
#include "StringsAndQStrings.h"
#include "Log.h"
#include <iostream>
#include <stdlib.h>
#include <unistd.h>
#include <QFileInfo>
#include <QFile>
#include <QDir>
#include "Trace.h"
namespace RA {
//------------------------------------------------------------------------------

sPath                         sEnvironment::ProgramFilePath;
sPath                         sEnvironment::HelpFilePath;
sPath                         sEnvironment::RADir;
sUser                         sEnvironment::User;
std::list<sString>            sEnvironment::Arguments;
std::map<sString,bool>        sEnvironment::BoolOptions;
std::map<sString,sString>     sEnvironment::StringOptions;

//------------------------------------------------------------- sEnvironment ---

sEnvironment::sEnvironment (int argc ,char *argv[])
{
  RANet::Log.OpenBlock(sLog::Info,"Environment");

  ProgramFilePath=argv[0];

  RANet::Log.Put(sLog::Debug,"Environment"
                ,sString("QDir::currentPath() = '")
                +FromQString(QDir::currentPath())
                +"'");
  RANet::Log.Put(sLog::Debug,"Environment"
                ,sString("QDir::current().path() = '")
                +FromQString(QDir::current().path())
                +"'");
  RANet::Log.Put(sLog::Debug,"Environment"
                ,sString("QDir::current().absolutePath() = '")
                +FromQString(QDir::current().absolutePath())
                +"'");
  RANet::Log.Put(sLog::Debug,"Environment"
                ,sString("QDir::current().canonicalPath() = '")
                +FromQString(QDir::current().canonicalPath())
                +"'");
  RANet::Log.Put(sLog::Debug,"Environment"
                ,sString("Program Path  = '")+ProgramFilePath.PathStr()+"'");
  RANet::Log.Put(sLog::Debug,"Environment"
                ,sString("Program Dir  = '")+ProgramFilePath.Dir().PathStr()+"'");
  RANet::Log.Put(sLog::Debug,"Environment"
                ,sString("Program Name = '")+ProgramFilePath.Name()+"'");
  RANet::Log.Put(sLog::Debug,"Environment"
                ,sString("Program Ext  = '")+ProgramFilePath.Ext()+"'");

  QDir        Dir(QFileInfo(argv[0]).absolutePath());
  Dir.cdUp();

  RADir=FromQString(Dir.absolutePath());
  RANet::Log.Put(sLog::Debug,"Environment"
                ,sString("RADir.PathStr() = '")+RADir.PathStr()+"'");

  Dir.cd("help");
  HelpFilePath = FromQString(Dir.absolutePath()
                            +QDir::separator()+"RA.qhc");
  RANet::Log.Put(sLog::Debug,"Environment"
                ,sString("HelpFilePath  = '")+HelpFilePath.PathStr()+"'");

//!@todo{refinement} http://www.boost.org/doc/libs/1_53_0/doc/html/program_options.html

//!@todo{refinement} options -b+ and -b-

//!@todo{refinement} option aliases: an alias value doesn't substitute main
//!                  option value if both are given

  for( int i = 1 ; i < argc ; ++i ){
    sString                     Arg(argv[i]);
    if( Arg[0] == '-' ){
      size_t                      P = Arg.Pos("=");
      if( P == Arg.Length() ){
        BoolOptions[Arg] = ! BoolOption(Arg,false);
        RANet::Log.Put(sLog::Info,"Environment"
                      ,sString("option ")+Arg
                      +( BoolOption(Arg,false) ? "=true" : "=false" )
                      );
      }else{
        sString                     ArgName(Arg.Substr(0,P));
        sString                     ArgValue(Arg.Tail(P+1));
        RANet::Log.Put(sLog::Info,"Environment"
                      ,sString("option ")+ArgName+"='"+ArgValue+"'");
        SetStringOption(ArgName,ArgValue);
      }
    }else{
      RANet::Log.Put(sLog::Info,"Environment"
                    ,sString("arguments ")+"+='"+Arg+"'");
      Arguments.push_back(Arg);
    }
  }
  sString                     WD( StringOption("--workdir"
                                              ,StringOption("-wd")) );
  if( ! WD.Empty() ){
    if( ! QDir::setCurrent(ToQString(WD)) )
      RANet::Log.Put(sLog::Critical,"Environment"
          ,sString("Cannot change working directory to '")+WD+"'");
  }
  RANet::Log.Set(BoolOption("--console",BoolOption("-c"
                                                  #ifdef RANET_DEBUG
                                                  ,true
                                                  #endif
                                                  ))
                ,StringOption("--logfile",StringOption("-lf"
                                                      #ifndef RANET_DEBUG
                                                      ,"log.txt"
                                                      #endif
                                                      ))
                );
  if( BoolOption("--admin",false) ){
    User.MaxAccessLevel=sUser::cAdmin;
  }
  RANet::Log.CloseBlock(sLog::Info,"Environment");
}


bool  sEnvironment::BoolOption (sString  ParamName ,bool DefaultValue)
{
  std::map<sString,bool>::const_iterator   p=BoolOptions.find(ParamName);
  return ( p != BoolOptions.end()  ?  p->second  :  DefaultValue );
}


sString  sEnvironment::StringOption (sString  ParamName ,sString  DefaultValue)
{
  std::map<sString,sString>::const_iterator   p=StringOptions.find(ParamName);
  return ( p != StringOptions.end()  ?  p->second  :  DefaultValue );
}


void sEnvironment::SetStringOption(sString param_name, sString value)
{
  StringOptions.insert(std::pair<sString,sString>(param_name,value));
}

//------------------------------------------------------------------------------
} //namespace RA
