////////////////////////////////////////////////////////////////////////////////
/*! @file ExpressionNode.cpp Class sExpressionNode.
- Part of RANet - Research Assistant Net Library.
- Copyright(C) 2014-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#include "ExpressionNode.h"
#include "NetMath.h"
#include "Functions.h"
namespace RA {
//------------------------------------------------------------------------------

//---------------------------------------------------------- sExpressionNode ---

sExpressionNode::~sExpressionNode ()
{
  if( TheResult ){ delete TheResult; TheResult=NULL; };
}

sExpressionNode::sExpressionNode (sString  source)
    :sFolderNode(source)
    ,sGraphObject()
    ,TheResult(NULL)
{
}


sExpressionNode::sExpressionNode (std::istream &  a_stream
                                 ,rcsVersion      version)
    :sFolderNode(a_stream,version)
    ,sGraphObject(a_stream,version)
    ,TheResult(NULL)
{
  bool                        ValueIsStored;
  a_stream.READ_INTO(ValueIsStored);
}


void  sExpressionNode::Store (std::ostream & a_stream) const
{
  sFolderNode::Store(a_stream);
  sGraphObject::Store(a_stream);
  bool                        ValueIsStored = false;
  a_stream.WRITE_FROM(ValueIsStored);
}


psMathValue  sExpressionNode::MathValue ()
{
  if( Result() )
    return TheResult->Replica();
  return NULL;
}


sString  sExpressionNode::Text (eTextFormat        F
                               ,eTextDetalization  D)
{
  sString                     T;
  T = sFolderNode::Text(F,D);
  if( ! Result() ){
    if( ! Docket().Empty() ){
      if( F == HTML )  T += "<font color=red>";
      T += " = ERR!!!";
      if( F == HTML )  T += "</font>";
    }
  }else{
    sString                     R(TheResult->Text(F,D));
    if( sMath::Valid(TheResult) ){
      if( ! R.Empty() )
        (T += " = ") += R;
    }else{
      psMathError               E = psMathError(TheResult);
      T = Docket().Substr(0,E->Cursor1)
          + "<font color=red>"
          + Docket().Substr(E->Cursor1,E->Cursor2-E->Cursor1)
          + "</font>"
          + Docket().Tail(E->Cursor2)
          + "<font color=red>"
          + " = ERROR: " + E->Message
          + "</font>";
    }
  }
  return T;
}


void  sExpressionNode::SetDocket (rcsString  new_docket)
{
  if( TheResult ){ delete TheResult; TheResult=NULL; }
  sFolderNode::SetDocket(new_docket);
}


sString  sExpressionNode::Hint ()
{
  return  Text(Plain,Casual);
}


//pcsAppearance  sExpressionNode::Appearance () const
//{
//  QMutexLocker                BodyLocker(&BodyMutex);
//  if( TheResult ){
//    psFunction                  F = dynamic_cast<psFunction>(TheResult);
//    if( F  )
//      return F->Appearance();
//  }
//  return NULL;
//}


void  sExpressionNode::SetAppearance (psAppearance  appearance)
{
  QMutexLocker                BodyLocker(&BodyMutex);
  sGraphObject::SetAppearance(appearance);
  if( TheResult ){
    psFunction                  F = dynamic_cast<psFunction>(TheResult);
    if( F  ){
      F->SetAppearance(new sCurveAppearance( * dynamic_cast<psCurveAppearance>
                                                           (appearance) ) );
    }
  }
  MarkAsEdited();
}


psMathValue  sExpressionNode::Result ()
{
  if( Docket().Empty() ){
    if( TheAppearance ){ delete TheAppearance; TheAppearance=NULL; }
    if( TheResult ){ delete TheResult; TheResult=NULL; }
  }else{
    if( ! TheResult ){
      sNetMath                E(sNodePtr(this),Docket());
      TheResult = E.Calculate();
      if( psFunction F =  dynamic_cast<psFunction>(TheResult) ){
        if( dynamic_cast<psCurveAppearance>(TheAppearance) ){
          F->SetAppearance(new sCurveAppearance( * dynamic_cast
                                                     <psCurveAppearance>
                                                     (TheAppearance)     ));
        }
      }
    }
  }
  return TheResult;
}


void  sExpressionNode::CopyFrom (psNode  other)
{
  sFolderNode::CopyFrom(other);
  psExpressionNode            Other = dynamic_cast<psExpressionNode>(other);
  if( Other ){
//    TheStartTime = Other->TheStartTime;
//    TheDuration = Other->TheDuration;
//    MarkAsEdited();
  }
}


sBoundaries  sExpressionNode::Boundaries ()
{
  psFunction                  F = dynamic_cast<psFunction>(Result());
  if( F )
    return F->Boundaries();
  return sGraphObject::Boundaries();
}


sBoundaries  sExpressionNode::Boundaries () const
{
  psFunction                  F = dynamic_cast<psFunction>(TheResult);
  if( F )
    return F->Boundaries();
  return sGraphObject::Boundaries();
}


int  sExpressionNode::Distance (rcsIntPoint  position
                               ,rcsScales    scales  )
{
  psFunction                  F = dynamic_cast<psFunction>(Result());
  if( F )
    return F->Distance(position,scales);
  return sGraphObject::Distance(position,scales);
}

//------------------------------------------------------------------------------
} //namespace RA
