////////////////////////////////////////////////////////////////////////////////
/*! @file FileNode.cpp   Базовые типы терминальных объектов сети.
- Part of RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2016, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "FileNode.h"
#include "Symbols.h"
#include <QMutexLocker>
#include <fstream>
namespace RA {
//------------------------------------------------------------------------------

//DEFINE_CLASS_TAG(sUndoAssignValue2FileNode)

//---------------------------------------------------------------- sFileNode ---

sFileNode::sFileNode (std::istream &  a_stream ,rcsVersion  version)
    :sByteArrayNode(a_stream,version)
{
  sString                     PathString;
  PathString.Read(a_stream);
  ThePath = PathString;
}


void sFileNode::Store (std::ostream &  a_stream) const
{
  sByteArrayNode::Store(a_stream);
  ThePath.PathStr().Write(a_stream);
}


sFileNode::sFileNode (rcsPath  path)
    :sByteArrayNode()
    ,ThePath(path)
{
  ImportFrom(ThePath);
}


sString  sFileNode::Text (eTextFormat        /*F*/
                         ,eTextDetalization  D)
{
  sString                     P(Symbol::BinaryData);
  sString                     T(ThePath.PathStr());
  P += P;
  if( D == Laconic ){
    if( T.Length() > CutOffLengthForLaconic-4 ){
      T = ThePath.Name() + "." + ThePath.Ext();
      if( T.Length() > CutOffLengthForLaconic-4 ){
        T = ThePath.Ext();
      }
    }
  }
  return P + T + P;
}


sPath  sFileNode::Path ()
{
  QMutexLocker                BodyLocker(&BodyMutex);
  return ThePath;
}


void  sFileNode::AssignPath (rcsPath  new_path)
{
  QMutexLocker                BodyLocker(&BodyMutex);
//  sNet::Manager->Undo.Push( new sUndoAssignValue2FileNode(sNodePtr(this)
//                                                     ,value) );
  if( new_path.PathStr() !=  ThePath.PathStr() ){
    ThePath = new_path;
    MarkAsEdited();
  }
}


void  sFileNode::AssignPath (rcsString  new_path_string)
{
  QMutexLocker                BodyLocker(&BodyMutex);
//  sNet::Manager->Undo.Push( new sUndoAssignValue2FileNode(sNodePtr(this)
//                                                     ,value) );
  if( new_path_string !=  ThePath.PathStr() ){
    ThePath = new_path_string;
    MarkAsEdited();
  }
}


void  sFileNode::CopyFrom (psNode  other)
{
  sByteArrayNode::CopyFrom(other);
  psFileNode                  Other = dynamic_cast<psFileNode>(other);
  if( Other ){
    sPath                       P = Other->Path();
    if( P.PathStr() !=  ThePath.PathStr() ){
      ThePath = P;
      MarkAsEdited();
    }
  }
}


bool  sFileNode::ImportFrom (sPath  path)
{
  bool                        OK = false;
  std::ifstream               F(path.PathStr());   // opens file
  if( F.is_open() ){
    char                        C;
    while( F.get(C) ){
      TheValue.push_back(C);
    }
    OK = F.eof();
    F.close();
  }
  return OK;
}


bool  sFileNode::ExportTo (sPath  path)
{
  bool                        OK = false;
  std::ofstream               F(path.PathStr());   // opens file
  if( F.is_open() ){
//    char                        C;
//    while( F.get(C) ){
//      TheValue.append(C);
//    }
//    OK = F.eof();
    F.close();
  }
  return OK;
}

//------------------------------------------------ sUndoAssignValue2FileNode ---

//void  sUndoAssignValue2FileNode::UndoIt ()
//{
//  rsFileNode(*NodePtr).SetValue( literal(OldValue) );
//};


//void  sUndoAssignValue2FileNode::RedoIt ()
//{
//  rsFileNode(*NodePtr).SetValue( literal(NewValue) );
//};


//sString  sUndoAssignValue2FileNode::Hint ()
//{
//  return  sString("change value '") + OldValue
//        + sString("' -> '") + NewValue + sString("'");
//};


//------------------------------------------------------------------------------
} //namespace RA

