////////////////////////////////////////////////////////////////////////////////
/*! @file Graphicals.h   Графические примитивы.
- Part of RANet - Research Assistant Net Library.
- Copyright(C) 2000-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef Graphicals_H
#define Graphicals_H
#include "Storable.h"
#include "Color.h"
#include "PhysValue.h"
#include "Ranges.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sPoint)
ANNOUNCE_CLASS(sLineSegment)
ANNOUNCE_CLASS(sRectangle)

ANNOUNCE_CLASS(sPhysPair)
ANNOUNCE_CLASS(sBoundaries)

ANNOUNCE_CLASS(sIntPoint)
ANNOUNCE_CLASS(sScale)
ANNOUNCE_CLASS(sScales)

ANNOUNCE_CLASS(sAppearance)
ANNOUNCE_CLASS(sLineAppearance)
ANNOUNCE_CLASS(sSpotAppearance)
ANNOUNCE_CLASS(sPointAppearance)

ANNOUNCE_ABSTRACT_CLASS(sGraphObject)

//------------------------------------------------------------------- sPoint ---

class RANet_EXPORT  sPoint
{
  public:
                              sPoint ()
                                  :X(0.0),Y(0.0)
                                {}
                              sPoint (real  x ,real  y)
                                  :X(x),Y(y)
                                {}
                              sPoint (rcsPoint P)
                                  :X(P.X),Y(P.Y)
                                {}
    rsPoint                   operator = (rcsPoint P)
                                { X=P.X;  Y=P.Y; return *this; }
    rsPoint                   operator = (rcsIntPoint P);
//    bool                      operator == (rcsPoint P) const
//                                { return (X==P.X)&&(Y==P.Y); }
//    bool                      operator < (rcsPoint P) const
//                                { return (X<P.X)&&(Y<P.Y); }
    rsPoint                   operator += (rcsPoint P)
                                { X+=P.X;  Y+=P.Y; return *this; }
    rsPoint                   operator -= (rcsPoint P)
                                { X-=P.X;  Y-=P.Y; return *this; }
    sPoint                    operator + (rcsPoint P) const
                                { return sPoint( X+P.X , Y+P.Y ); }
    sPoint                    operator - (rcsPoint P) const
                                { return sPoint( X-P.X , Y-P.Y ); }

  public: //fields
    real                      X;
    real                      Y;
};

//------------------------------------------------------------- sLineSegment ---

class RANet_EXPORT  sLineSegment
{
  public: //fields
    sPoint                    Start;
    sPoint                    End;
};

//--------------------------------------------------------------- sRectangle ---

class RANet_EXPORT  sRectangle
{
  public:
                              sRectangle ()
                                {}
                              sRectangle (sRange x ,sRange y)
                                  :X(x),Y(y)
                                {}
                              sRectangle (rcsRectangle B)
                                  :X(B.X),Y(B.Y)
                                {}
    rsRectangle              operator = (rcsRectangle B)
                                { X=B.X; Y=B.Y; return *this; }
    bool                      operator == (rcsRectangle B) const
                                { return (X==B.X) && (Y==B.Y); }
//    rsRectangle              operator |= (rcsRectangle);
//    rsRectangle              operator |= (rcsPoint);

  public: //fields
    sRange                    X;
    sRange                    Y;
};

//---------------------------------------------------------------- sPhysPair ---

class RANet_EXPORT  sPhysPair : public virtual sStorable
{
  STORABLE(sPhysPair)
  public:
                              sPhysPair ()
                                  :X(),Y()
                                {}
                              sPhysPair (sPhysValue  x ,sPhysValue  y)
                                  :X(x),Y(y)
                                {}
                              sPhysPair (rcsPhysPair  P)
                                  :sStorable(),X(P.X),Y(P.Y)
                                {}
    rsPhysPair                operator = (rcsPhysPair  P)
                                { X=P.X;  Y=P.Y; return *this; }
    rsPhysPair                operator += (rcsPhysPair  P)
                                { X+=P.X;  Y+=P.Y; return *this; }
    rsPhysPair                operator -= (rcsPhysPair  P)
                                { X-=P.X;  Y-=P.Y; return *this; }
    sPhysPair                 operator + (rcsPhysPair  P) const
                                { return sPhysPair( X+P.X , Y+P.Y ); }
    sPhysPair                 operator - (rcsPhysPair  P) const
                                { return sPhysPair( X-P.X , Y-P.Y ); }

  public: //fields
    sPhysValue                X;
    sPhysValue                Y;
};

//-------------------------------------------------------------- sBoundaries ---

class RANet_EXPORT  sBoundaries : public virtual sStorable
{
  STORABLE(sBoundaries)
  public:
                              sBoundaries ()
                                {}
                              sBoundaries (sPhysRange  x ,sPhysRange  y)
                                  :X(x),Y(y)
                                {}
                              sBoundaries (rcsBoundaries B)
                                  :sStorable(),X(B.X),Y(B.Y)
                                {}
  public:
    bool                      UnitsAreFixed () const
                                {
                                  return X.UnitsAreFixed() && Y.UnitsAreFixed();
                                }
    void                      FixTheUnits ()
                                { X.FixTheUnits(); Y.FixTheUnits(); }
    bool                      IsInf () const
                                { return X.IsInf() || Y.IsInf(); }
    bool                      IsNaN () const
                                { return X.IsNaN() || Y.IsNaN(); }
  public://operators
    rsBoundaries              operator = (rcsBoundaries B)
                                { X=B.X; Y=B.Y; return *this; }
//    bool                      operator == (rcsBoundaries B) const
//                                { return (X==B.X) && (Y==B.Y); }
    rsBoundaries              operator |= (rcsBoundaries);
                                  //!< operator 'expand to contain both'
    rsBoundaries              operator |= (rcsPhysPair);
                                  //!< operator 'expand to contain the point'

  public: //fields
    sPhysRange                X;
    sPhysRange                Y;
};

//---------------------------------------------------------------- sIntPoint ---

class RANet_EXPORT  sIntPoint
{
  public:
                              sIntPoint ()
                                  :X(0.0),Y(0.0)
                                {}
                              sIntPoint (int  x ,int  y)
                                  :X(x),Y(y)
                                {}
                              sIntPoint (rcsIntPoint P)
                                  :X(P.X),Y(P.Y)
                                {}
    rsIntPoint                operator = (rcsIntPoint P)
                                { X=P.X;  Y=P.Y; return *this; }
    rsIntPoint                operator = (rcsPoint P)
                                { X=Round(P.X);  Y=Round(P.Y); return *this; }
    bool                      operator == (rcsIntPoint P) const
                                { return (X==P.X)&&(Y==P.Y); }
//    bool                      operator < (rcsIntPoint P) const
//                                { return (X<P.X)&&(Y<P.Y); };
    rsIntPoint                operator += (rcsIntPoint P)
                                { X+=P.X;  Y+=P.Y; return *this; }
    rsIntPoint                operator -= (rcsIntPoint P)
                                { X-=P.X;  Y-=P.Y; return *this; }
    sIntPoint                 operator + (rcsIntPoint P) const
                                { return sIntPoint( X+P.X , Y+P.Y ); }
    sIntPoint                 operator - (rcsIntPoint P) const
                                { return sIntPoint( X-P.X , Y-P.Y ); }
    int                       ManhattanLength () const
                                { return std::abs(X)+std::abs(Y); }

  public: //fields
    int                       X;
    int                       Y;
};

//---------------

inline  rsPoint   sPoint::operator = (rcsIntPoint P)
{
  X=P.X;  Y=P.Y;  return *this;
}


//------------------------------------------------------------------- sScale ---

class RANet_EXPORT  sScale
{
  public:
    virtual                   ~sScale ()
                                {}
                              sScale (bool  upside_down);
                              sScale (rcsPhysValue  unit);
                              sScale (rcsScale  scale);
    rsScale                   operator = (rcsScale  scale);
    sPhysRange                PhysRange () const
                                {
                                            //! @test (real)(sPhysValue)
                                  return sPhysRange(TheRange.From * TheUnit
                                                   ,TheRange.To * TheUnit  );
                                }
    sPhysValue                Unit () const
                                { return TheUnit; }
    sRange                    Range () const
                                { return TheRange; }
    virtual void              SetRange (sPhysRange);
    void                      SetRange (rcsRange);
    void                      SetRange (int  from ,int  to);
    void                      ShiftRange (int  shift);
    real                      PixelValue () const
                                { return (TheRange.To-TheRange.From)/TheSize; }
    int                       Size () const
                                { return TheSize; }
    virtual void              SetSize (int);
    int                       T (real) const;
    int                       T (sPhysValue) const;
    sPhysValue                T (int) const;
    real                      Tr2real (int) const;
    real                      Tr2real (sPhysValue) const;

  protected:
    virtual void              SetOffset (int  offset)
                                { Offset = offset; }
  private: //fields
    sPhysValue                TheUnit;
    sRange                    TheRange;
    int                       TheSize;
    int                       Offset;  //!< shifts discrete coordinate system
                                       //!< (by halfwidth of the axis line)
    bool                      UpsideDown;
    bool                      NoTr;
};

//------------------------------------------------------------------ sScales ---

class RANet_EXPORT  sScales
{
  public:
//                              sScales ()
//                                  :X(),Y()
//                                {};
                              sScales (rcsScale  x ,rcsScale  y)
                                  :X(x),Y(y)
                                {}
                              sScales (rcsScales  S)
                                  :X(S.X),Y(S.Y)
                                {}
    rsScales                  operator = (rcsScales  S)
                                { X=S.X; Y=S.Y; return *this; }
    sIntPoint                 T (rcsPoint  P) const
                                { return sIntPoint(X.T(P.X),Y.T(P.Y)); }
    sIntPoint                 T (rcsPhysPair  P) const
                                { return sIntPoint(X.T(P.X),Y.T(P.Y)); }
    sPhysPair                 T (rcsIntPoint  P) const
                                { return sPhysPair(X.T(P.X),Y.T(P.Y)); }

  public: //fields
    sScale                    X;
    sScale                    Y;
};


//-------------------------------------------------------------- sAppearance ---
/*! Базовый класс для атрибутов внешнего вида узлов сети (цвета кривых и т.п.).


*/
class RANet_EXPORT  sAppearance : virtual public sStorable
{
  STORABLE(sAppearance)
  public:
    virtual                   ~sAppearance ();
    explicit                  sAppearance (bool  hidden = false);
//    virtual psAppearance      Replica () const;
//    virtual psAppearance      AssignFrom (pcsAppearance);

  public: //fields
    bool                      Hidden;
};

//---------------------------------------------------------- sLineAppearance ---

class RANet_EXPORT  sLineAppearance : virtual public sAppearance
{
  STORABLE(sLineAppearance)
  public:
    enum                      eLineStyle
                                {cLineSolid,cLineDotted,cLineDashed};
    static const eLineStyle   DefaultLineStyle;
    static const byte         DefaultLineThickness;
    static const byte         MinLineThickness;
    static const byte         MaxLineThickness;

  public:
    virtual                   ~sLineAppearance ();
                              sLineAppearance ();
      explicit                sLineAppearance (rcsColor  color);
                              sLineAppearance (rcsColor    color
                                              ,byte        line_thickness
                                              ,eLineStyle  line_style
                                                            = DefaultLineStyle);
//    virtual psAppearance      AssignFrom (pcsAppearance);

  public: //fields
    sColor                    LineColor;
    byte                      LineThickness;
    eLineStyle                LineStyle;
};


//---------------------------------------------------------- sSpotAppearance ---

class RANet_EXPORT  sSpotAppearance : virtual public sAppearance
{
  STORABLE(sSpotAppearance)
  public:
    static const byte         DefaultContourThickness;
    static const byte         MinContourThickness;
    static const byte         MaxContourThickness;

  public:
                              sSpotAppearance ();
    explicit                  sSpotAppearance (rcsColor);
//    virtual psAppearance      AssignFrom (pcsAppearance);

  public: //fields
    sColor                    FillColor;
    sColor                    ContourColor;
    bool                      Filled;
    bool                      Contoured;
    byte                      ContourThickness;
};

//--------------------------------------------------------- sPointAppearance ---

class RANet_EXPORT  sPointAppearance : public sSpotAppearance
{
  STORABLE(sPointAppearance)
  public:
    enum                      eShape
                                {cCircle,cSquare,cDiamond};
    static const eShape       DefaultShape;
    static const byte         DefaultSize;
    static const byte         MinSize;
    static const byte         MaxSize;

  public:
                              sPointAppearance ();
    explicit                  sPointAppearance (rcsColor);
                              sPointAppearance (rcsColor  color
                                               ,byte      point_size
                                               ,eShape    point_shape);
//    virtual psAppearance      AssignFrom (pcsAppearance);

  public: //fields
    byte                      PointSize;
    eShape                    PointShape;
};


//------------------------------------------------------------- sGraphObject ---

class RANet_EXPORT  sGraphObject : virtual public sStorable
{
  ABSTRACT_STORABLE(sGraphObject)
  public:
    virtual                   ~sGraphObject ();

  public:
    virtual pcsAppearance     Appearance () const
                                { return TheAppearance; }
    virtual void              SetAppearance (psAppearance);
    virtual sBoundaries       Boundaries ();
    virtual sBoundaries       Boundaries () const;
    virtual int               Distance (rcsIntPoint  position
                                       ,rcsScales    scales  );
    virtual sString           Hint () =0;

  protected:
    explicit                  sGraphObject ()
                                  :sAdam(),sStorable(),TheAppearance(NULL)
                                {}

  protected: //fields
    psAppearance              TheAppearance;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif

