////////////////////////////////////////////////////////////////////////////////
/*! @file ListDataFunction.cpp   Function given by a list of X-Y-pairs.
- Part of RANet - Research Assistant Net Library.
- Copyright(C) 2010-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "ListDataFunction.h"
#include "Log.h"
namespace RA {
//------------------------------------------------------------------------------
//----------------------------------------------- sListDataFunction::sTracer ---

sListDataFunction::sTracer::sTracer (pcsListDataFunction  host_curve
                                    ,rcsScales            scales    )
    :TheFunction(host_curve)
    ,Scaling(1.0,1.0)
    ,ArbRange()
    ,WrongUnits(false)
    ,SegmentFingerInitiated(false)
    ,PointFingerInitiated(false)
{
  try{
    Scaling.X = (host_curve->Multiplier().X / scales.X.Unit()).ToReal();
    Scaling.Y = (host_curve->Multiplier().Y / scales.Y.Unit()).ToReal();

//    sPhysValue                  SX(_Unitsless_);
//    sPhysValue                  SY(_Unitsless_);
//    SX = host_curve->Multiplier().X / scales.X.Unit();
//    SY = host_curve->Multiplier().Y / scales.Y.Unit();
////    SX.Normalize();
////    SY.Normalize();
//    Scaling.X = SX.ArbValue();
//    Scaling.Y = SY.ArbValue();

//    RANet::Log.Put(sLog::Debug,"Graph"
//                  ,"sListDataFunction::sTracer::sTracer: Scaling");
  }catch(rxRAlgebra){
    RANet::Log.Put(sLog::Debug,"Graph"
                  ,"sListDataFunction::sTracer::sTracer: WrongUnits");
    WrongUnits = true;
  };
}


bool  sListDataFunction::sTracer::NextPoint ()
{
  if( WrongUnits )
    return false;
  if( ! PointFingerInitiated ){
    PointFinger=TheFunction->ThePoints.begin();
    PointFingerInitiated=true;
  };
  if( PointFinger == TheFunction->ThePoints.end() ){
    return false;
  }else{
    Point.X= PointFinger->X * Scaling.X;
    Point.Y= PointFinger->Y * Scaling.Y;
    PointFinger++;
    return true;
  };
}


bool  sListDataFunction::sTracer::NextLineSegment ()
{
  if( WrongUnits )
    return false;
  if( ! SegmentFingerInitiated ){
    SegmentFinger=TheFunction->ThePoints.begin();
    SegmentFingerInitiated=true;
  };
  if( SegmentFinger == TheFunction->ThePoints.end() ){
    return false;
  }else{
    LineSegment.Start.X= SegmentFinger->X * Scaling.X;
    LineSegment.Start.Y= SegmentFinger->Y * Scaling.Y;
    SegmentFinger++;
    if( SegmentFinger == TheFunction->ThePoints.end() ){
      return false;
    }else{
      LineSegment.End.X= SegmentFinger->X * Scaling.X;
      LineSegment.End.Y= SegmentFinger->Y * Scaling.Y;
      return true;
    };
  };
}

//-------------------------------------------------------- sListDataFunction ---

sListDataFunction::~sListDataFunction ()
{
  ThePoints.clear();
}


sListDataFunction::sListDataFunction ()
    :sDataFunction()
    ,Sorted(true)
{
}


sListDataFunction::sListDataFunction (rcsPhysPair  multiplier)
    :sDataFunction(multiplier)
    ,Sorted(true)
{
}


sListDataFunction::sListDataFunction (std::istream &  a_stream
                                         ,rcsVersion      version)
    :sDataFunction(a_stream,version)
{
  if( version <= sVersion(0,9) ){
  }else if( version < sVersion(1,5) ){
    int                           NumberOfPoints;
    sDataPoint                    DP;
    realIOv14                        R;
    a_stream.READ_INTO(Sorted);
    a_stream.READ_INTO(NumberOfPoints);
    for( int PointNo=0 ; PointNo < NumberOfPoints ; PointNo++ ){
      a_stream.READ_INTO(R);  DP.X=R;
      a_stream.READ_INTO(R);  DP.Y=R;
      ThePoints.push_back(DP);
    }
  }else{
    int                           NumberOfPoints;
    sDataPoint                    DP;
    realIO                        R;
    a_stream.READ_INTO(Sorted);
    a_stream.READ_INTO(NumberOfPoints);
    for( int PointNo=0 ; PointNo < NumberOfPoints ; PointNo++ ){
      a_stream.READ_INTO(R);  DP.X=R;
      a_stream.READ_INTO(R);  DP.Y=R;
      ThePoints.push_back(DP);
    }
  }
}


void  sListDataFunction::Store (std::ostream & a_stream) const
{
  int                               NumberOfPoints(ThePoints.size());
  std::list<sDataPoint>
                  ::const_iterator  Finger;
  sDataFunction::Store(a_stream);
  a_stream.WRITE_FROM(Sorted);
  a_stream.WRITE_FROM(NumberOfPoints);
  realIO                            R;
  for( Finger = ThePoints.begin() ; Finger != ThePoints.end() ; Finger++ ){
    R=Finger->X;  a_stream.WRITE_FROM(R);
    R=Finger->Y;  a_stream.WRITE_FROM(R);
  }
}


sCurve::psTracer  sListDataFunction::GetCurveTracer (rcsScales  scales) const
{

  //!@patch Та же проблема, что и с границами: при просмотре редактируется
  //if( ! Sorted ){
  //  ThePoints.sort();
  //  Sorted=true;
  //  MarkAsEdited();
  //}

  return new sListDataFunction::sTracer(this,scales);
}


void  sListDataFunction::AddNewPoint (sPoint  P)
{
  sDataPoint                  DP(P);
  if( ! ThePoints.empty() && Sorted ){
    sDataPoint                      LastPoint(ThePoints.back());
    Sorted=( LastPoint < DP );
  };
  ThePoints.push_back(DP);
  if( ! RecalculateBoundaries )
    LastBoundaries |= sPhysPair( P.X*Multiplier().X , P.Y*Multiplier().Y );
}

//------------------------------------------------------------------------------
} //namespace RA


