////////////////////////////////////////////////////////////////////////////////
/*! @file Log.cpp   Собирает структурированную трассировочную информацию.
- Part of RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2011, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "Log.h"
#include "Trace.h"
#include <iostream>
namespace RA {
//------------------------------------------------------------------------------

//--------------------------------------------------------------- sLog::sItem---

sLog::sItem::sItem (psItem    parent
                   ,unsigned int  level
                   ,eSeverity severity
                   ,sString   topic
                   ,sString   message )
    :TheTime()
    ,TheSeverity(severity)
    ,TheTopic(topic)
    ,TheMessage(message)
    ,TheLevel(level)
    ,TheParent(parent)
{

}


sString  sLog::sItem::Text () const
{
  sString                     T;
  T += TheTime.Text();
  T += "  ";
//  T << TheSeverity;
//  T += " \t";
  T += TheTopic;
  T += " \t";
  T += TheMessage;
  return T;
}

//---------------------------------------------------------------------- sLog---

sLog::~sLog ()
{
  while( ! Items.empty() ){
    psItem                      I( *(Items.begin()) );
    Items.pop_front();
    delete I;
  }
}


sLog::sLog ()
    :Unset(true)
    ,CurrentBlock(NULL)
    ,CurrentLevel(0)
    ,TheShowSeverity(Admin)
    ,ConsoleOutput(false)
    ,FileOutput(true)
//    ,KeepInMemory(false)
{
  #ifdef RANET_DEBUG
    ConsoleOutput = true;
    FileOutput = false;
  #endif
}


void  sLog::Close (int  return_code)
{
  if( return_code ){
    Put(Critical,"return_code",sString::FromInteger(return_code));
  }
  if( FileOutput ){
    LogFile.Close();
  }
}


void  sLog::Put (eSeverity  severity
                ,sString    topic
                ,sString    message )
{
//  // The following slightly speeds up the work in Release mode.
//  // However, in this case, neither SetShowSeverity(sLog::Debug),
//  // nor AlwaysShow("some_topic") will have an effect.
//  #ifndef RANET_DEBUG
//    if( severity <= Debug )
//      return;
//  #endif
  psItem                      I( new sItem(CurrentBlock
                                          ,CurrentLevel
                                          ,severity
                                          ,topic
                                          ,message ) );
  if( Unset ){
    Items.push_back(I);
  }else{
    Out(I);
    delete I;
  }
}


void  sLog::Out (pcsItem  item)
{
  if(    (  (item->Severity() >= TheShowSeverity)
         && (TopicsToHide.count(item->Topic()) == 0) )
      || (TopicsToShow.count(item->Topic()) != 0) ){
    if( ConsoleOutput ){
      for( unsigned int i = 0 ; i < item->Level() ; i++)  std::cout << "\t";
      std::cout << (literal)(item->Text()) << std::endl;
    }
    if( FileOutput ){
      for( unsigned int i = 0 ; i < item->Level() ; i++)  LogFile << "\t";
      LogFile << (literal)(item->Text()) << std::endl;
    }
  }
}


void  sLog::OpenBlock (eSeverity  severity
                      ,sString    topic
                      ,sString    message )
{
  Put(severity,topic,message);
  CurrentLevel++;
}


void  sLog::OpenBlock (eSeverity  severity
                      ,sString    topic   )
{
  OpenBlock(severity,topic,"started");
}


void  sLog::CloseBlock (eSeverity  severity
                       ,sString    topic)
{
//  if( ! CurrentBlock || (severity != CurrentBlock->TheSeverity)
//                     || (topic != CurrentBlock->TheTopic)       ){
//    Put(,,"");
//  };
  Put(severity,topic,"finished"); //  duration=   total=");
  CurrentLevel--;
}


void  sLog::SetShowSeverity (eSeverity  severity)
{
  TheShowSeverity = severity;
}


void  sLog::AlwaysHide (sString  topic)
{
  TopicsToHide.insert(topic);
  TopicsToShow.erase(topic);
}


void  sLog::AlwaysShow (sString  topic)
{
  TopicsToShow.insert(topic);
  TopicsToHide.erase(topic);
}


void  sLog::Set (bool     console_output
                ,sString  log_file_path)
{
  Unset = false;
  ConsoleOutput = console_output;
  FileOutput = false;
  if( ! log_file_path.Empty() ){
    LogFile.Assign(log_file_path,sFile::Textual);
    LogFile.Open(sFile::Appending);
    FileOutput = LogFile.is_open();
  }
  while( ! Items.empty() ){
    psItem                      I( *(Items.begin()) );
    Out(I);
    Items.pop_front();
    delete I;
  }
}

//------------------------------------------------------------------------------
} //namespace RA


