////////////////////////////////////////////////////////////////////////////////
/*! @file Log.h   Собирает структурированную трассировочную информацию.
- Part of RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2011, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef Log_H
#define Log_H
#include "Strings.h"
#include "Time.h"
#include "Files.h"
#include <list>
#include <set>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sLog)

namespace RANet {

RANet_EXPORT extern  sLog                Log;

}

//---------------------------------------------------------------------- sLog---
/*!

@macro
*/
#ifdef  RANET_DEBUG
  #define DEBUG_LOG(TOPIC,E)     RANet::Log.Put(sLog::Debug,#TOPIC,E)
#else
  #define DEBUG_LOG(TOPIC,E)     {}
#endif

//---------------------------------------------------------------------- sLog---
/*! Собирает структурированную трассировочную информацию.

цикл с фильтром
вывод в поток
*/
class RANet_EXPORT  sLog
{
  public://types
    enum                          eSeverity {MinorSeverity //!< never happening
                                            ,Debug
                                            ,Admin
                                            ,Info
                                            ,Warning
                                            ,Critical
                                            ,ProgramError
                                            ,Conditionless
                                            ,MajorSeverity //!< never happening
                                            };
    ANNOUNCE_CLASS(sItem)
    class sItem //: public virtual sStorable
    {
      public:
        unsigned int              Level () const
                                    { return TheLevel; }
        psItem                    Parent () const
                                    { return TheParent; }
        sTime                     Time () const
                                    { return TheTime; }
        eSeverity                 Severity () const
                                    { return TheSeverity; }
        sString                   Topic () const
                                    { return TheTopic; }
        sString                   Message () const
                                    { return TheMessage; }
        sString                   Text () const;

      private:
                                  sItem (psItem     parent
                                        ,unsigned int  level
                                        ,eSeverity  severity
                                        ,sString    topic
                                        ,sString    message );
      private://fields
        sTime                     TheTime;
        eSeverity                 TheSeverity;
        sString                   TheTopic;
        sString                   TheMessage;
        unsigned int              TheLevel;
        psItem                    TheParent;
      friend class sLog;
    };

  public:
                              ~sLog ();
                              sLog ();
    void                      Close (int return_code = 0);
    void                      Put (eSeverity  severity
                                  ,sString    topic
                                  ,sString    message );
    void                      OpenBlock (eSeverity  severity
                                        ,sString    topic
                                        ,sString    message );
    void                      OpenBlock (eSeverity  severity
                                        ,sString    topic   );
    void                      CloseBlock (eSeverity  severity
                                         ,sString    topic);
    void                      SetShowSeverity (eSeverity  severity);
    void                      AlwaysHide (sString  topic);
    void                      AlwaysShow (sString  topic);
    void                      Set (bool     console_output
                                  ,sString  log_file_path);

  protected:
    void                      Out (pcsItem  item);

  private://fields
    bool                      Unset;
    std::list<psItem>         Items;
    psItem                    CurrentBlock;
    unsigned int              CurrentLevel;
    eSeverity                 TheShowSeverity;
    std::set<sString>         TopicsToHide;
    std::set<sString>         TopicsToShow;
    bool                      ConsoleOutput;
    bool                      FileOutput;
    sFile                     LogFile;
//    bool                      KeepInMemory;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
