////////////////////////////////////////////////////////////////////////////////
/*! @file Marks.h   Базовые классы разметки.
- Part of RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2000-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef Marks_H
#define Marks_H
#include "Graphicals.h"
#include "BasicNodes.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_ABSTRACT_CLASS(sMark)
ANNOUNCE_ABSTRACT_CLASS(sPositionMark)
ANNOUNCE_CLASS(sPositionMarkAppearance)
ANNOUNCE_CLASS(sXMark)
ANNOUNCE_CLASS(sYMark)
//ANNOUNCE_ABSTRACT_CLASS(sIntervalMark)
//ANNOUNCE_CLASS(sXIntervalMark)
//ANNOUNCE_CLASS(sYIntervalMark)
//ANNOUNCE_CLASS(sBaselineMark)

//-------------------------------------------------------------------- sMark ---

class RANet_EXPORT  sMark : public sGraphObject
{
  ABSTRACT_STORABLE(sMark)
  public:
    virtual                   ~sMark ()
                                {}

  protected:
    explicit                  sMark ()
                                  :sGraphObject()
                                {}
};

//------------------------------------------------------------ sPositionMark ---

class RANet_EXPORT  sPositionMark : public sPhysValueNode
                                  , public sMark
{
  ABSTRACT_STORABLE(sPositionMark)
  public:
    virtual                   ~sPositionMark ()
                                {}
  protected:
    explicit                  sPositionMark ()
                                  :sPhysValueNode()
                                  ,sMark()
                                {}

  public:
    virtual sString           Hint ()
                                { return sPhysValueNode::Text(Plain,Laconic); }
    virtual void              SetAppearance (psAppearance);
    virtual sString           Text (eTextFormat        /*F=Plain*/
                                   ,eTextDetalization  /*D=Casual*/
                                   ); //!< visual representation for user

  protected:
    virtual void              CopyFrom (psNode);
};

//--------------------------------------------------------- sCurveAppearance ---

class RANet_EXPORT  sPositionMarkAppearance : public sLineAppearance
{
  STORABLE(sPositionMarkAppearance)
  public: //types
    enum                      eMode {cInfiniteLine
                                    ,cGuideTowardsAxis
                                    ,cLongDashAcross
                                    ,cUpOrRightArrow
                                    ,cDownOrLeftArrow};

  public:
    virtual                   ~sPositionMarkAppearance ();
    explicit                  sPositionMarkAppearance ();
//    virtual psAppearance      AssignFrom (pcsAppearance);

  public: //fields
    eMode                     Mode;
    byte                      MarkLength; //!< percentage of scale
};

//------------------------------------------------------------------- sXMark ---

class RANet_EXPORT  sXMark : public sPositionMark
{
  STORABLE(sXMark)
  public:
                              sXMark ()
                                  :sPositionMark()
                                {}
    virtual sBoundaries       Boundaries ();
    virtual int               Distance (rcsIntPoint  position
                                       ,rcsScales    scales  );
};

//------------------------------------------------------------------- sYMark ---

class RANet_EXPORT  sYMark : public sPositionMark
{
  STORABLE(sYMark)
  public:
                              sYMark ()
                                  :sPositionMark()
                                {}
    virtual sBoundaries       Boundaries ();
    virtual int               Distance (rcsIntPoint  position
                                       ,rcsScales    scales  );
};

/*
//----------------------------------------------------------------------------

class RANet_EXPORT  sIntervalMark : public sMark
{
  public:
    virtual void              Store (rsObjectStream);
    virtual void              AssignIntrinsicContent (sNode::sPtr);
    virtual sString           Text () ;
    virtual real              From () const
                                { return FFrom; };
    virtual void              SetFrom (real   F)
                                { FFrom=F; };
    virtual real              To () const
                                { return FTo; };
    virtual void              SetTo (real   T)
                                { FTo=T; };

  protected:
                              sIntervalMark (real   AFrom ,real   ATo);
                              sIntervalMark (rsObjectStream);

  private:
    real                      FFrom;
    real                      FTo;

  DEFCLASSTAG
};

//----------------------------------------------------------------------------

class RANet_EXPORT  sXIntervalMark : public sIntervalMark
{
  public:
                              sXIntervalMark (real   AFrom ,real   ATo)
                                  :sIntervalMark(AFrom,ATo)
                                {};
                              sXIntervalMark (rsObjectStream S)
                                  :sIntervalMark(S)
                                {};
    virtual sPtr              Copy () const;
    virtual sBoundaries       Boundaries ();
    virtual int               Distance (sPoint P
                                       ,real   XPixelValue
                                       ,real   YPixelValue);

  private:
    static psStorable         Load (rsObjectStream S)
                                { return new sXIntervalMark(S); };
  DEFCLASSTAG
};

//----------------------------------------------------------------------------

class RANet_EXPORT  sYIntervalMark : public sIntervalMark
{
  public:
                              sYIntervalMark (real   AFrom ,real   ATo)
                                  :sIntervalMark(AFrom,ATo)
                                {};
                              sYIntervalMark (rsObjectStream S)
                                  :sIntervalMark(S)
                                {};
    virtual sPtr              Copy () const;
    virtual sBoundaries       Boundaries ();
    virtual int               Distance (sPoint P
                                       ,real   XPixelValue
                                       ,real   YPixelValue);

  private:
    static psStorable         Load (rsObjectStream S)
                                { return new sYIntervalMark(S); };
  DEFCLASSTAG
};

//----------------------------------------------------------------------------

class RANet_EXPORT  sBaselineMark : public sXIntervalMark
{
  public:
                              sBaselineMark (real   AFrom ,real   ATo)
                                  :sXIntervalMark(AFrom,ATo)
                                {};
                              sBaselineMark (rsObjectStream S)
                                  :sXIntervalMark(S)
                                {};
    virtual sPtr              Copy () const;

  private:
    static psStorable         Load (rsObjectStream S)
                                { return new sBaselineMark(S); };
  DEFCLASSTAG
};

*/
//------------------------------------------------------------------------------
} //namespace RA
#endif

