////////////////////////////////////////////////////////////////////////////////
/*! @file NetExceptions.h   Exception classes representing problems
 *                          in RANet Knowledgebase functionality.
- Part of RANet - Research Assistant Net Library.
- Copyright(C) 1994-2018, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef NetExceptions_H
#define NetExceptions_H
#include "Excepts.h"
#include "NodePtr.h"
#include "Attribute.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(xNet)
ANNOUNCE_CLASS(xAttrName)
ANNOUNCE_CLASS(xNetIO)
ANNOUNCE_CLASS(xNetOpen)
ANNOUNCE_CLASS(xNetInvalidOp)
ANNOUNCE_CLASS(xNetTimeout)
ANNOUNCE_CLASS(xGetNullNode)
ANNOUNCE_CLASS(xSetAttr)
ANNOUNCE_CLASS(xAscribeAttr)
ANNOUNCE_CLASS(xMakeSubcluster)
ANNOUNCE_CLASS(xAscribeModel)
ANNOUNCE_CLASS(xNetConsistency)

//--------------------------------------------------------------------- xNet ---

class RANet_EXPORT  xNet : public xException
{
  public:
                              xNet (rcsString  msg_text
                                   ,rcsString  src_file_name = sString()
                                   ,int        line_no       = 0
                                   ,rcsString  class_name
                                                       = sString("xNet")
                                   )
                                  :xException(msg_text,src_file_name,line_no
                                             ,class_name)
                                {}
//                              xNet (rcsString  title
//                                   ,rcsString  text);
//                              xNet (rcsString  text);
//                              xNet ();
};

//------------------------------------------------------------------- xNetIO ---

class RANet_EXPORT  xAttrName : public xNet
{
  public:
                              xAttrName (rcsString  msg_text
                                        ,rcsString  src_file_name = sString()
                                        ,int        line_no       = 0
                                        ,rcsString  class_name
                                                       = sString("xAttrName")
                                   )
                                  :xNet(msg_text,src_file_name,line_no
                                       ,class_name)
                                {}
//                              xAttrName (rcsString  text)
//                                  :xNet("xAttrName exception",text)
//                                {}
};

//------------------------------------------------------------------- xNetIO ---

class RANet_EXPORT  xNetIO : public xNet
{
  public:
                              xNetIO (rcsString  msg_text
                                     ,rcsString  src_file_name = sString()
                                     ,int        line_no       = 0
                                     ,rcsString  class_name
                                                       = sString("xNetIO")
                                   )
                                  :xNet(msg_text,src_file_name,line_no
                                       ,class_name)
                                {}
//                              xNetIO (rcsString  text)
//                                  :xNet("xNetIO exception",text)
//                                {}
};

//----------------------------------------------------------------- xNetOpen ---

class RANet_EXPORT  xNetOpen : public xNetIO
{
  public:
                              xNetOpen (rcsString  text)
                                  :xNetIO(text)
                                {}
};

//------------------------------------------------------------ xNetInvalidOp ---

class RANet_EXPORT  xNetInvalidOp : public xNet
{
  public:
                              xNetInvalidOp
                                  (rcsString  msg_text
                                  ,rcsString  src_file_name = sString()
                                  ,int        line_no       = 0
                                  ,rcsString  class_name
                                                      = sString("xNetInvalidOp")
                                  )
                                  :xNet(msg_text,src_file_name,line_no
                                       ,class_name)
                                {}
//                              xNetInvalidOp (rcsString  text)
//                                  :xNet("xNetInvalidOp exception",text)
//                                {}
};

//-------------------------------------------------------------- xNetTimeout ---

class RANet_EXPORT  xNetTimeout : public xNetIO
{
  public:
};

//------------------------------------------------------------- xGetNullNode ---

class RANet_EXPORT  xGetNullNode : public xNet
{
  public:
                              xGetNullNode (sNodePtr);
};

//------------------------------------------------------------------------------

class RANet_EXPORT  xSetAttr : public xNet
{
  public:
                              xSetAttr
                                  (rcsString  msg_text
                                  ,rcsString  src_file_name = sString()
                                  ,int        line_no       = 0
                                  ,rcsString  class_name
                                                      = sString("xSetAttr")
                                  )
                                  :xNet(msg_text,src_file_name,line_no
                                       ,class_name)
                                {}
                              xSetAttr (sNodePtr,sAttribute)
                                  :xNet("")
                                {}
                              xSetAttr (sNodePtr,sAttrName)
                                  :xNet("")
                                {}
};

//------------------------------------------------------------------------------

class RANet_EXPORT  xAscribeAttr : public xSetAttr
{
  public:
                              xAscribeAttr (sNodePtr  N ,sAttribute  A)
                                  :xSetAttr(N,A)
                                {}
};

//------------------------------------------------------------------------------

class RANet_EXPORT  xMakeSubcluster : public xSetAttr
{
  public:
                              xMakeSubcluster (sNodePtr       N
                                              ,sAttrName AName)
                                  :xSetAttr(N,AName)
                                {}
};

//------------------------------------------------------------ xAscribeModel ---

class RANet_EXPORT  xAscribeModel : public xNet
{
  public:
                              xAscribeModel (sNodePtr to_node ,sNodePtr model);
};

//---------------------------------------------------------- xNetConsistency ---

class RANet_EXPORT  xNetConsistency : public xNet
{
  public:
                              xNetConsistency (rcsString  title
                                              ,rcsString  text)
                                  :xNet(title,text)
                                {}
                              xNetConsistency (rcsString  text)
                                  :xNet("xNetConsistency exception",text)
                                {}
};

//------------------------------------------------------------------------------
} //namespace RA
#endif


