////////////////////////////////////////////////////////////////////////////////
/*! @file Record.cpp   Class sRecord.

        Внимание: часть методов класса RA::sRecord
        реализованы в файле RecordCustom.cpp или
        в другом месте, в зависимости от кастомизации.

- Part of RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "Record.h"
#include "attr.h"
namespace RA {
//------------------------------------------------------------------------------

/*!@todo{UI++} Implement using sRecord::cDateSeparator and
sRecord::cNumberSeparator.
Check whether they are different or not (another parsing algorithm),
and possible conflicts with expressions.

Suitable formats: #2015.11.02.1  #2015~11~02.1  #2015_11_02.57  #2015~11~02`57
                  #11.02.1       #11~02.1       #11_02.57       #11~02`57
                  #02.1          #02.1          #02.57          #02`57
*/
const char                    sRecord::cDateSeparator   = '.';
const char                    sRecord::cNumberSeparator = '.';

sRecord::fNewRecordNumber *   sRecord::NewRecordNumber = NULL;
sPtr<sFolderNode>             sRecord::RecordNumberingData;
sPtr<sFolderNode>             sRecord::ChronologicalStorage;

//------------------------------------------------------------------ sRecord ---

sRecord::sRecord ()
    :sFolderNode()
    ,TheStartTime()
    ,TheDuration(0.0)
{
  if( NewRecordNumber ){
    TheRecordNo = (*NewRecordNumber)(TheStartTime);
  }else{
    TheRecordNo = Default_NewRecordNumber(TheStartTime);
  }
}


sRecord::sRecord (int  record_no)
    :sFolderNode()
    ,TheStartTime(sTime::FromMillisecondsSinceEpoch(0))
    ,TheDuration(0.0)
    ,TheRecordNo(record_no)
{
}


sRecord::sRecord (std::istream &  a_stream ,rcsVersion  version)
    :sFolderNode(a_stream,version)
    ,TheStartTime(a_stream,version)
{
  if( version < sVersion(1,5) ){
    realIOv14 R;
    a_stream.READ_INTO(R);  TheDuration=R;
  }else{
    realIO R;
    a_stream.READ_INTO(R);  TheDuration=R;
  }
  a_stream.READ_INTO(TheRecordNo);
}


void  sRecord::Store (std::ostream &  a_stream) const
{
  sFolderNode::Store(a_stream);
  TheStartTime.Store(a_stream);
  realIO R;
  R=TheDuration;  a_stream.WRITE_FROM(R);
  a_stream.WRITE_FROM(TheRecordNo);
}


sString  sRecord::Text (eTextFormat        F
                       ,eTextDetalization  D)
{
  sString                     T;
  sString                     C;
  if( D == Verbose ){
    T = FullIdentifier();
  }else{
    T = ShortIdentifier();
  }
  /*! @todo{UI++} show record's start time and duration */
  C = sFolderNode::Text(F,D);
  if( ! C.Empty() )
    (T += " ") += C;
  return T;
}


void  sRecord::CopyFrom (psNode  other)
{
  sFolderNode::CopyFrom(other);
  psRecord                    Other = dynamic_cast<psRecord>(other);
  if( Other ){
    TheStartTime = Other->TheStartTime;
    TheDuration = Other->TheDuration;
    TheRecordNo = Other->TheRecordNo;
    MarkAsEdited();
  }
}


#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wsign-compare"
bool  sRecord::ParseIdentifier (rcsString  src
                               ,int &      cursor
                               ,int &      year
                               ,int &      month
                               ,int &      day
                               ,int &      number)
{
  sTime                       Now;
  int                         C0 = 0;
  year = -1;
  month = -1;
  day = -1;
  number = -1;
  if( (cursor < src.Length()) && (src[cursor] == '#') ){
    cursor++;  C0 = cursor;
    while( (cursor < src.Length()) && (src[cursor] >= '0')
                                   && (src[cursor] <= '9') )
      cursor++;
    if( cursor > C0 ){
      src.Substr(C0,cursor-C0) >> number;
      if( (number > 0) && (cursor < src.Length()) && (src[cursor] == '.') ){
        cursor++;  C0 = cursor;
        while( (cursor < src.Length()) && (src[cursor] >= '0')
                                       && (src[cursor] <= '9') )
          cursor++;
        if( cursor > C0 ){
          day = number;
          src.Substr(C0,cursor-C0) >> number;
          if( (number > 0) && (cursor < src.Length()) && (src[cursor] == '.') ){
            cursor++;  C0 = cursor;
            while( (cursor < src.Length()) && (src[cursor] >= '0')
                                           && (src[cursor] <= '9') )
              cursor++;
            if( cursor > C0 ){
              month = day;  day = number;
              src.Substr(C0,cursor-C0) >> number;
              if( (number > 0) && (cursor < src.Length())
                                                      && (src[cursor] == '.') ){
                cursor++;  C0 = cursor;
                while( (cursor < src.Length()) && (src[cursor] >= '0')
                                               && (src[cursor] <= '9') )
                  cursor++;
                if( cursor > C0 ){
                  year = month;  month = day;  day = number;
                  src.Substr(C0,cursor-C0) >> number;
                }
              }
            }
          }
        }
      }
    }
  }
  if( year < 0 ){
    year = Now.Year();
  }else if( year < 89 ){
    year += 2000;
  }else if( (year >= 89) && (year <= 99) ){
    year += 1900;
  }
  if( month < 0 )  month = Now.Month();
  if( day < 0 )  day = Now.Day();
  if( (number > 0) && (year > 0) && (month >0 ) && (month < 13)
                                 && (day >0 ) && (day < 32) ){
    return true;
  }
  return false;
}
#pragma GCC diagnostic pop


sPtr<sRecord>  sRecord::FindRecord (integer  year
                                   ,integer  month
                                   ,integer  day
                                   ,integer  number)
{
  sPtr<sRecord>               Record;
  sPtr<sRecord>               R;
  sPtr<sFolderNode>           Allocation(ChronologicalStorage
                                                  >> year >> month >> day );
  if( Allocation.IsCorrect() ){
    for_each_nameless_attribute( A , Allocation ){
      R = A;
      if( R.IsCorrect() && (R->StartTime().Year() == year)
                        && (R->StartTime().Month() == month)
                        && (R->StartTime().Day() == day)
                        && (R->RecordNo() == number)  ){
        Record = R;
        break;
      }
    }end_attr_loop();
  }
  return Record;
}


sString  sRecord::FullIdentifier () const
{
  sString                     T("#");
  T<<StartTime().Year();
  T += '.';
  if( StartTime().Month() <= 9 )
    T += '0';
  T << StartTime().Month();
  T += '.';
  if( StartTime().Day() <= 9 )
    T += '0';
  T << StartTime().Day();
  T += '.';
  T << TheRecordNo;
  return T;
}


sString  sRecord::ShortIdentifier () const
{
  sTime                       Now;
  sString                     T("#");
  bool                        B = (StartTime().Year() != Now.Year());
  if( B ){
    T << StartTime().Year();
    T += '.';
  }
  B = B || (StartTime().Month() != Now.Month());
  if( B ){
    if( StartTime().Month() <= 9 )
      T += '0';
    T << StartTime().Month();
    T += '.';
  }
  B = B || (StartTime().Day() != Now.Day());
  if( B ){
    if( StartTime().Day() <= 9 )
      T += '0';
    T << StartTime().Day();
    T += '.';
  }
  T << TheRecordNo;
  return T;
}


//bool  sRecord::IdentifierMatches (rcsString  Id) const
//{
//  sString                     FullId(Id);
//  return FullIdentifier()==FullId;
//}


int  sRecord::Default_NewRecordNumber (rcsTime  record_start_time)
{
  static int                  N = 0;
  int                         NewNumber = ++N;
  if( RecordNumberingData.IsCorrect() ){
    static bool                 FirstCall =true;
    int                         SD = record_start_time.Year()*10000
        + record_start_time.Month()*100
        + record_start_time.Day();
    if( FirstCall ){
      FirstCall=false;
      RecordNumberingData |= attr("LastRecordDate",0);
    };
    sPtr<sIntegerNode>          LD (RecordNumberingData >> "LastRecordDate");
    if( LD.IsCorrect() && ( (long int)(*LD) == SD ) ){
      sPtr<sIntegerNode>          RN(RecordNumberingData >> "LastRecordNo");
      NewNumber=(long int)(*RN)+1;
      RecordNumberingData << attr("LastRecordNo",NewNumber);
    }else{
      NewNumber=1;
      RecordNumberingData << attr("LastRecordDate",SD);
      RecordNumberingData << attr("LastRecordNo",NewNumber);
    }
  }
  return NewNumber;
}


sPtr<sFolderNode>  sRecord::AllocateRecord ()
{
  sPtr<sFolderNode>           Allocation;
  if( ChronologicalStorage.IsCorrect() ){
    Allocation = ChronologicalStorage;
    Allocation |= StartTime().Year();
    Allocation = Allocation >> StartTime().Year();
    Allocation |= StartTime().Month();
    Allocation = Allocation >> StartTime().Month();
    Allocation |= StartTime().Day();
    Allocation = Allocation >> StartTime().Day();
    if( Allocation.IsCorrect() ){
//      QMutexLocker                BodyLocker(&BodyMutex);
      Allocation << sAttribute(sAttrName(/*TheRecordNo*/),sNodePtr(this));
//      MarkAsEdited();
    }
  }
  return Allocation;
}


void  sRecord::AllocateHistorically (sPtr<sFolderNode>  in_here)
{
  if( in_here.IsCorrect() ){
    in_here |= StartTime().Year();
    in_here = in_here >> StartTime().Year();
    in_here |= StartTime().Month();
    in_here = in_here >> StartTime().Month();
    in_here |= StartTime().Day();
    in_here = in_here >> StartTime().Day();
    if( in_here.IsCorrect() ){
      in_here << sAttribute(sAttrName(),sNodePtr(this));
    }
  }
}


void  sRecord::DisregardRecord (sString  Reason)
{
  SetAttr(attr("<font color=red><b>disregarded</b></font>",Reason));
  if( ChronologicalStorage.IsCorrect() ){
    sPtr<sFolderNode>          Allocation;
    Allocation = ChronologicalStorage >> StartTime().Year()
                           >> StartTime().Month() >> StartTime().Day();
    if( Allocation.IsCorrect() ){
//      Allocation->DelAttr(sAttrName(RecordNo()));
      Allocation.DelAttr(sNodePtr(this));
    }
  }
}

//------------------------------------------------------------------------------
} //namespace RA
