////////////////////////////////////////////////////////////////////////////////
/*! @file StaticInitiators.cpp Initiates static members in right order.
- Part of RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2013-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#include "Log.h"
#include "Adam.h"
#include "Storable.h"
#include "Time.h"
#include "Color.h"
#include "Nodes.h"
#include "FolderNode.h"
#include "Units.h"
#include "PhysValue.h"
#include "Ranges.h"
#include "BasicNodes.h"
#include "FileNode.h"
#include "ColorNodes.h"
#include "Graphicals.h"
#include "Record.h"
#include "Curves.h"
#include "Marks.h"
#include "Functions.h"
#include "ArrayDataFunction.h"
#include "ListDataFunction.h"
#include "TableDataFunction.h"
#include "DataArrayFolder.h"
#include "DataListFolder.h"
#include "DataTableFolder.h"
#include "DataArrayRecord.h"
#include "DataListRecord.h"
#include "DataTableRecord.h"
#include "ExpressionNode.h"
#include "BasicMath.h"
#include "PhysMath.h"
#include "NetMath.h"
#include "Instrument.h"
#include "Device.h"
namespace RA {
//------------------------------------------------------------------------------

namespace RANet {

sLog                          Log;

} //namespace RANet

sTag::sTagMapByName   sTag::TagMapByName;
DEFINE_CLASS_TAG( sAdam )
DEFINE_CLASS_TAG( sStorable )
DEFINE_CLASS_TAG( sMathValue )
DEFINE_CLASS_TAG( sMathError )
DEFINE_CLASS_TAG( sRealValue )
DEFINE_CLASS_TAG( sIntegerValue )
DEFINE_CLASS_TAG(sTime)
DEFINE_CLASS_TAG(sColor)
DEFINE_CLASS_TAG(sColorSet)
DEFINE_CLASS_TAG( sNode )   //see more in Nodes.cpp
//DEFINE_CLASS_TAG( sNullNode )
//DEFINE_CLASS_TAG( sLostNode )
DEFINE_CLASS_TAG(sFolderNode)
DEFINE_CLASS_TAG(sUnit)
DEFINE_CLASS_TAG(sUnits)
DEFINE_CLASS_TAG(sPhysValue)
DEFINE_CLASS_TAG(sPhysRange)
DEFINE_CLASS_TAG(sTerminalNode)
DEFINE_CLASS_TAG(sBooleanNode)
DEFINE_CLASS_TAG(sIntegerNode)
DEFINE_CLASS_TAG(sByteArrayNode)
DEFINE_CLASS_TAG(sTextNode)
DEFINE_CLASS_TAG(sPhysValueNode)
DEFINE_CLASS_TAG(sPhysRangeNode)
DEFINE_CLASS_TAG(sFileNode)
DEFINE_CLASS_TAG(sColorNode)
DEFINE_CLASS_TAG(sColorSetNode)
DEFINE_CLASS_TAG(sPhysPair)
DEFINE_CLASS_TAG(sBoundaries)
DEFINE_CLASS_TAG(sAppearance)
DEFINE_CLASS_TAG(sGraphObject)
DEFINE_CLASS_TAG(sLineAppearance)
DEFINE_CLASS_TAG(sSpotAppearance)
DEFINE_CLASS_TAG(sPointAppearance)
DEFINE_CLASS_TAG(sRecord)
DEFINE_CLASS_TAG(sCurveAppearance)
DEFINE_CLASS_TAG(sCurve)
DEFINE_CLASS_TAG(sFunction)
DEFINE_CLASS_TAG(sDataFunction)
DEFINE_CLASS_TAG(sAnalyticFunction)
DEFINE_CLASS_TAG(sMark)
DEFINE_CLASS_TAG(sPositionMark)
DEFINE_CLASS_TAG(sXMark)
DEFINE_CLASS_TAG(sYMark)
DEFINE_CLASS_TAG(sPositionMarkAppearance)

//DEFINE_CLASS_TAG(sIntervalMark)
//DEFINE_CLASS_TAG(sXIntervalMark)
//DEFINE_CLASS_TAG(sYIntervalMark)
//DEFINE_CLASS_TAG(sBaselineMark)

DEFINE_CLASS_TAG(sArrayDataFunction)
DEFINE_CLASS_TAG(sListDataFunction)
DEFINE_CLASS_TAG(sTableDataFunction)
DEFINE_CLASS_TAG(sDataArrayFolder)
DEFINE_CLASS_TAG(sDataListFolder)
DEFINE_CLASS_TAG(sDataTableFolder)
DEFINE_CLASS_TAG(sDataArrayRecord)
DEFINE_CLASS_TAG(sDataListRecord)
DEFINE_CLASS_TAG(sDataTableRecord)
DEFINE_CLASS_TAG(sExpressionNode)

csUnits                       _Unitsless_;

csString          sMetricPrefix::StandardAllowance
                          ("Y..Z..E..P..T..G..M..k..0..m..u..n..p..f..a..z..y");
                         //"Y..Z..E..P..T..G..M..khd0dcm..u..n..p..f..a..z..y"

csString          sMetricPrefix::NoPrefixAllowed
                          ("-..-..-..-..-..-..-..-..0..-..-..-..-..-..-..-..-");
                         //"Y..Z..E..P..T..G..M..khd0dcm..u..n..p..f..a..z..y"

sUnit::sSystem                sUnit::System;
sUnit::pfRegisterDialog       sUnit::RegisterDialog = NULL;
psUnit                        sUnit::_1_ = new sUnit();

sMath::sDefinitions                sMath::Definitions;
sPhysMath::sMathOpRegistrator    sPhysMath::MathOpRegistrator;
sNetMath::sMathOpRegistrator     sNetMath::MathOpRegistrator;

map<sString,eInstrumentType>  sInstrumentTag::AvailableClassList;

DEFINE_CLASS_TAG(sInstrument)

sPtr<sFolderNode>                sDevice::DeviceList;
map<sString,psDevice>         sDevice::RunningDevicesByName;

DEFINE_CLASS_TAG(sDevice)


//------------------------------------------------------------------------------
} //namespace RA
