////////////////////////////////////////////////////////////////////////////////
/*! @file Storable.cpp   Потоковый ввод-вывод полиморфных объектов.
- Part of RANet - Research Assistant Net Library.
- Copyright(C) 1994-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "Storable.h"
namespace RA {
//------------------------------------------------------------------------------

//---------------------------------------------------------------- sStorable ---

psStorable  sStorable::PolymorphicLoad (rcsString       version_and_class
                                       ,std::istream &  a_stream)
{
  psStorable                  Result = NULL;
  sVersion                    Version(0,0);
  sString                     ClassName;
  if( SplitToVersionAndClass(version_and_class,Version,ClassName) ){

    //! @todo{NetIO} шрам от переименования классов
    if( ClassName == "sArrayTypeDependence" )  ClassName = "sDataArrayRecord";
    if( ClassName == "sListTypeDependence" )  ClassName = "sDataListRecord";
    if( ClassName == "sTableTypeDependence" )  ClassName = "sDataTableRecord";
    //----------

    pcsTag                      tag = sTag::ByName(ClassName);
    if( tag ){
      pcsStorableTag            s_tag = dynamic_cast<pcsStorableTag>(tag);
      if( s_tag ){
        Result = (s_tag->LoadFunction)(a_stream,Version);
      }else{
        throw xProgramError("sStorable::PolymorphicLoad: not a storable class"
                           ,__FILE__,__LINE__);
      };
    }else{
      throw xNotImplemented("sStorable::PolymorphicLoad for unknown class");
    };
  };
  return Result;
}


/*! Reads a class name (C++-name), creates new instance of this class with its
    constructor (std::istream &).
@return Pointer to the newborn sStorable descendant.

@note May return NULL, if NULL was stored with
      StorePolymorphic(std::ostream &,pcsStorable).
*/
psStorable  sStorable::PolymorphicLoad (std::istream &  a_stream)
{
  sString                     VerClass;
  VerClass.Read(a_stream);
  if( ! VerClass.Empty() )
    return PolymorphicLoad(VerClass,a_stream);
  return NULL;
}


/*! Stores a sStorable descendant (with its virtual Store(std::ostream &)),
    prepended by version of RANet and by the object's class name (C++-name)
    for later loading.
@return Version+Classname or empty sString, if NULL is stored (the latter
        is used as a terminal in a stored sequence of polymorphic objects).
@note Only the principal part of version is stored, namely
      RANet::Version.Principal().Text()
*/
sString  sStorable::StorePolymorphic (std::ostream &  a_stream
                                     ,pcsStorable     storable_object)
{
  sString                     VerClass;
  if( storable_object )
    VerClass = storable_object->VersionAndClass();
  VerClass.Write(a_stream);
  if( storable_object )
    storable_object->Store(a_stream);
  return VerClass;
}


sString  sStorable::VersionAndClass () const
{
  return  RANet::Version.Principal().Text() + TrueType()->Name();
}


bool sStorable::SplitToVersionAndClass (rcsString  version_and_class
                                       ,rsVersion  version
                                       ,rsString   class_name)
{
  class_name = version_and_class;
  if( ! class_name.Empty() ){

    //!@patch
    if( class_name[0] != 's' ){

      int                         Major = -1;
      int                         Minor = -1;
      class_name >> Major;
      if( class_name[0] == '.' ){
        class_name = class_name.Tail(1);
        class_name >> Minor;
      }
      if( (Major >= 0) && (Minor >= 0) ){
        version = sVersion(Major,Minor);
      }else{
        version = sVersion(0,0);
      }

    }

  }
  return ! class_name.Empty();
}


void sStorable::Store (std::ostream &) const
{
}


sStorable::sStorable (std::istream & ,rcsVersion)
{
}

//------------------------------------------------------------------------------
} //namespace RA
