////////////////////////////////////////////////////////////////////////////////
/*! @file Storable.h   Потоковый ввод-вывод полиморфных объектов.
- Part of RANet - Research Assistant Net Library.
- Copyright(C) 1994-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef Storable_H
#define Storable_H
#include "Adam.h"
#include "Version.h"
#include "Excepts.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sStorableTag)
ANNOUNCE_ABSTRACT_CLASS(sStorable)

typedef psStorable            tLoadFunction (std::istream & ,rcsVersion);

//------------------------------------------------------------- sStorableTag ---

class RANet_EXPORT  sStorableTag : public sTag
{
  public:
  protected:
                              sStorableTag (literal                name
                                           ,const tLoadFunction &  loadfunction)
                                  :sTag(name)
                                  ,LoadFunction(loadfunction)
                                {}

  private: // fields
    const tLoadFunction &     LoadFunction;

  friend class sStorable;
};

//! @todo{?} Load & Store(rsObjecStream) could make a copy of argument?!

//--------------------------------------------------------------- STORABLE() ---
/*!


@macro
*/
#define STORABLE(CLASSNAME)                                                  \
  private:                                                                   \
  public:                                                                    \
    class sIntrinsicTag : public sStorableTag                                \
    {                                                                        \
      public:                                                                \
                                  sIntrinsicTag ()                           \
                                      :sStorableTag(#CLASSNAME,Load)         \
                                    {};                                      \
        virtual bool              CanCast (psAdam P) const                   \
                                    { return bool(                           \
                                          dynamic_cast<p##CLASSNAME>(P)      \
                                                 );                          \
                                    };                                       \
    };                                                                       \
    static sIntrinsicTag      AdamInheritorTag;                              \
  public:                                                                    \
    static pcsTag             Type()                                         \
                                { return &AdamInheritorTag;};                \
    virtual pcsTag            TrueType () const                              \
                                {return &AdamInheritorTag;};                 \
  public:                                                                    \
    virtual void              Store (std::ostream &) const;                  \
                              CLASSNAME (std::istream & ,rcsVersion);        \
  private:                                                                   \
    static psStorable         Load (std::istream &  stream                   \
                                   ,rcsVersion      version)                 \
                                {return new CLASSNAME(stream,version);};     \
  private:


//------------------------------------------------------ ABSTRACT_STORABLE() ---
/*!


@macro
*/
#define ABSTRACT_STORABLE(CLASSNAME)                                         \
  ADAM(CLASSNAME)                                                            \
  public:                                                                    \
    virtual void              Store (std::ostream &) const;                  \
                              CLASSNAME (std::istream & ,rcsVersion);        \
  private:


//---------------------------------------------------------------- sStorable ---
/*! Базовый класс для полиморфных объектов, сохраняемых в потоках

*/
class RANet_EXPORT  sStorable : virtual public sAdam
{
  ABSTRACT_STORABLE(sStorable)
  public://static
    static psStorable         PolymorphicLoad (rcsString       version_and_class
                                              ,std::istream &  a_stream);
    static psStorable         PolymorphicLoad (std::istream &);
    static sString            StorePolymorphic(std::ostream &
                                              ,pcsStorable);
  public:
    virtual                   ~sStorable ()
                                {}
    sString                   VersionAndClass () const;

  protected:
                              sStorable ()
                                  :sAdam()
                                {}
  private:
    static bool               SplitToVersionAndClass
                                                   (rcsString  version_and_class
                                                   ,rsVersion  version
                                                   ,rsString   class_name);
};

//----------------------------------------------- WRITE_FROM() , READ_INTO() ---

/*!


@macro
*/
#define WRITE_FROM(X)  write((const char *)&X,sizeof(X))

/*!


@macro
*/
#define READ_INTO(X)  read((char *)&X,sizeof(X))

//------------------------------------------------------------------------------
} //namespace RA
#endif
