////////////////////////////////////////////////////////////////////////////////
/*! @file String.h   Class sString and related.
- Part of RANet - Research Assistant Net Library.
- Copyright(C) 1994-2017, Viktor E. Bursian, St.Petersburg, Russia.
                          VBursian@gmail.com
*///////////////////////////////////////////////////////////////////////////////
#ifndef Strings_H
#define Strings_H
#include "General.h"
#include <iostream>
#include <utility>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sString)

//std::ostream &  operator << (std::ostream &  ,rcsString);
//std::istream &  operator >> (std::istream &  ,rsString);

//using namespace std::rel_ops; NEVER EVER USE using IN HEADERS!!!

//------------------------------------------------------------------ sString ---
/*! A string class specific for RANet.

@note It is not intended to be a universal string class, rather, it is
      a string class with minimal interface as required by RANet.

@par Why not @c std::string or QString or QByteArray?
-# It appeared in my texts when the @c string template was not yet
   in C++ standard, and also way before Qt was born.
-# I wish to keep the very basics of RANet independent from a third party
   non-standard library, like Qt. (However, it should probably be rewritten
   with, say, C++11 std::u16string).
-# It has some specific features; so, I would anyway need at least a thin
   wrapper over a standard string.

@bug Some of its functionality works bad with UTF-8 strings (allthough, one
can initialize sString with a UTF-8 literal). Same applies to the null chars
inside the string (allthough, they can be put into sString's body
programmaticaly).
*/
class RANet_EXPORT  sString
{
  public://static
    static sString            FromReal (real                number
                                       ,unsigned short int  digits_after_dot);
    static sString            FromInteger (integer  number);

  public:
    virtual                   ~sString ();
                              sString ();
                              sString (rcsString);
                              sString (literal);
                              sString (char);
    bool                      Empty () const;
    size_t                    Length () const;
//    char                      operator [] (size_t) const; //first char index==0
    char                      operator [] (int) const; //first char index==0
    /*!< @todo{elegancy} op[] must return char& ? */
//                              operator literal () const;
                              operator ptr_to_const_char () const;
    sString                   Substr (size_t from ,size_t length) const;
    sString                   Tail (size_t from) const;
    size_t                    Pos (rcsString) const;
    size_t                    Pos (literal) const;
    rsString                  operator =  (rcsString);
    rsString                  operator =  (literal);
    sString                   operator + (rcsString) const;
    sString                   operator + (literal) const;
//    sString                   operator + (char) const;
    rsString                  operator += (rcsString);
    rsString                  operator += (literal);
    rsString                  operator += (const char);
    rsString                  operator << (integer);
//    rsString                  operator << (const int);
//    rsString                  operator << (const unsigned int);
//    rsString                  operator << (const long int);
//    rsString                  operator << (const unsigned long int);
    rsString                  operator << (real);
//    rsString                  operator << (const double);
    rsString                  operator >> (integer &);
    rsString                  operator >> (real &);
    bool                      operator == (rcsString) const;
    bool                      operator == (literal) const;
    bool                      operator != (rcsString) const;
    bool                      operator != (literal) const;
    bool                      operator < (rcsString) const;
    /*!< @todo{elegancy}  ==(char)  <,>,<=,>=  >>int/float */
    sString                   UpperCase () const;
    sString                   Trim () const;
    sString                   TrimSyntactic (literal Delimiters
                                                     = DefaultDelimiters) const;
    sString                   Translate (char O ,char N) const;
    void                      EatLeadingSpaces ();
    int                       WipeOut (literal  substring
                                      ,bool     only_first = false);
    int                       Replace (literal  substring
                                      ,literal  replacement
                                      ,bool     only_first = false);
    virtual void              Read (std::istream &);
    virtual void              Write (std::ostream &) const;
//    virtual void              InputTextLine (std::istream &);
  protected:
    void                      SetValue (literal);

  private: //fields
    char *                    Body;
    size_t                    CurrentLength;
//    size_t                    CurrentSpace;

  private: //constants
    static literal           EmptyLiteral;
    static literal           DefaultDelimiters;
    static const sString     PlusInfinity;
    static const sString     MinusInfinity;
};


//inline  size_t  sString::Pos (rcsString S) const
//{
//  return Pos( (literal)S );
//}


inline  rsString  sString::operator =  (literal V)
{
  SetValue(V); return *this;
}


//inline  sString  sString::operator + (rcsString S) const
//{
//  return operator+(literal(S));
//}


//inline  rsString  sString::operator += (rcsString S)
//{
//  return operator+=(literal(S));//! @todo{bomb} что за нах!
//}


inline  bool  sString::operator != (rcsString S) const
{
  return !( operator==(S) );
}


inline  bool  sString::operator != (literal L) const
{
  return !( operator==(L) );
}


inline  rsString  sString::operator << (integer  number)
{
  operator+=( FromInteger(number) );
  return *this;
}


//inline  std::ostream &  operator<< (std::ostream & a_stream ,rcsString a_string)
//{
//  a_string.Write(a_stream);
//  return  a_stream;
//}

//inline  std::istream &  operator>> (std::istream & a__stream ,rsString a_string)
//{
//  a_string.Read(a__stream);
//  return  a__stream;
//}

//------------------------------------------------------------------------------
} //namespace RA
#endif

