////////////////////////////////////////////////////////////////////////////////
/*! @file TableDataFunction.h   Function given by data set in X-Y-table form.
- Part of RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef TableDataFunction_H
#define TableDataFunction_H
#include "Functions.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sTableDataFunction)

//------------------------------------------------------- sTableDataFunction ---
/*! An implementation of sDataFunction with the data set in X-Y-table form.

В нём число точек фиксировано и точки индексируются (то есть можно доступаться
по номеру), как в sArrayDataFunction, но абциссы произвольны, как в
sListDataFunction.

Упорядоченность абцисс - на совести программиста.
*/
class RANet_EXPORT  sTableDataFunction : public sDataFunction
{
  STORABLE(sTableDataFunction)

  public: //classes
    ANNOUNCE_CLASS(sTracer)
    class RANet_EXPORT  sTracer : public sCurve::sTracer
    {
      public:
                                  sTracer (pcsTableDataFunction  host_curve
                                          ,rcsScales             scales    );
        virtual                   ~sTracer ()
                                    {}
      public:
        virtual bool              NextPoint ();
        virtual bool              NextLineSegment ();
      protected:
        pcsTableDataFunction      TheFunction;
      private:
        sPoint                    Scaling;
        int                       LastPointNo;
        int                       PointNo;
        int                       LineSegmentNo;
    };

  public:
    virtual                   ~sTableDataFunction ();
                              sTableDataFunction (sPhysPair  multiplier
                                                 ,int        number_of_points);
                              sTableDataFunction (int   number_of_points);
//                              sTableDataFunction (rcsTableDataFunction);
//    virtual psMathValue       Replica () const
//                                { return new sTableDataFunction(*this); }
    virtual sString           Hint ()
                                { return sString(); }
    virtual sCurve::psTracer  GetCurveTracer (rcsScales  scales) const;
    virtual sPhysValue        Value (rcsPhysValue  x) const;
    virtual int               NumberOfPoints () const
                                { return TheNumberOfPoints; }
    virtual sPoint            Point (const int N) const;
    virtual real              PointX (const int N) const;
    virtual real              PointY (const int N) const;
    virtual void              SetValue (int N ,rcsPoint);
    virtual void              SetXValue (int N ,real  X);
    virtual void              SetYValue (int N ,real  Y);

  protected: //fields
    int                       TheNumberOfPoints;
    real   *                  XValues;
    real   *                  YValues;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
