////////////////////////////////////////////////////////////////////////////////
/*! @file Time.cpp   Время.
- This implementation uses  Qt v.4.6  -  http://qt.nokia.com/
- Part of RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "Time.h"
#include "StringsAndQStrings.h"
#include <QDateTime>
//#include "Trace.h"
namespace RA {
//------------------------------------------------------------------------------

//-------------------------------------------------------------------- sTime ---
/*! @class sTime

@todo{Net} mutex!

@todo{--Qt} see chrono in C++11 or Boost.Chrono
*/


/*! Format: yyyy-MM-dd hh:mm:ss.zzz
*/
sTime  sTime::FromISO_8601(rcsString  text)
{
  return sTime::FromMillisecondsSinceEpoch(
      QDateTime::fromString(QString::fromUtf8((literal)text)
                           ,"yyyy-MM-dd hh:mm:ss.zzz")
                                          .toMSecsSinceEpoch()
                                          );
}


/*! Format: hh:mm:ss
*/
real  sTime::ToSeconds(rcsString  text)
{
  return  QTime::fromString(QString::fromUtf8((literal)text)
                           ,"hh:mm:ss"
                           ).msecsSinceStartOfDay() * 1e-3;
}


sTime::sTime()
    :_UTC( QDateTime::currentDateTimeUtc().toMSecsSinceEpoch() )
{
/*
  QDateTime  T = QDateTime::currentDateTime();
  TRACE(T.toString().toUtf8().data());
  _UTC = T.toMSecsSinceEpoch();
  TRACE(Text());
  T = QDateTime::currentDateTimeUtc();
  TRACE(T.toString().toUtf8().data());
  _UTC = T.toMSecsSinceEpoch();
  TRACE(Text());
----output:
***TRACE( T.toString().toUtf8().data() )=сб нояб. 28 20:54:33 2015.
***TRACE( Text() )=2015-11-28 20:54:33.770.
***TRACE( T.toString().toUtf8().data() )=сб нояб. 28 17:54:33 2015 GMT.
***TRACE( Text() )=2015-11-28 20:54:33.770.
*/
}


sTime::sTime (rcsTime  other)
    :sStorable()
    ,_UTC( other._UTC )
{
}


sTime::sTime (int year ,int month ,int date
             ,int hour ,int minute ,int second
             ,int millisecond)
    :_UTC(QDateTime(QDate(year,month,date)
                   ,QTime(hour,minute,second,millisecond)).toMSecsSinceEpoch())
{
}


sTime::sTime (std::istream &  a_stream ,rcsVersion  version)
    :sStorable(a_stream,version)
{
  a_stream.READ_INTO(_UTC);
}


void  sTime::Store (std::ostream &  a_stream) const
{
  a_stream.WRITE_FROM(_UTC);
}


rsTime  sTime::operator = (rcsTime  other)
{
  _UTC = other._UTC;
  return *this;
}


bool  sTime::operator == (rcsTime other) const
{
  return (_UTC == other._UTC);
}


bool  sTime::operator < (rcsTime other) const
{
  return (_UTC < other._UTC);
}


int  sTime::Year () const
{
  return QDateTime::fromMSecsSinceEpoch(_UTC).date().year();
}


int  sTime::Month () const
{
  return QDateTime::fromMSecsSinceEpoch(_UTC).date().month();
}


int  sTime::Day () const
{
  return QDateTime::fromMSecsSinceEpoch(_UTC).date().day();
}


sString  sTime::Text () const
{
  return FromQString( QDateTime::fromMSecsSinceEpoch(_UTC)
                                        .toString("yyyy-MM-dd hh:mm:ss.zzz") );
}


rsTime  sTime::operator += (real  seconds)
{
  _UTC += Round(seconds*1e3);
  return *this;
}


sTime  sTime::operator + (real  seconds) const
{
  return FromMillisecondsSinceEpoch( _UTC + Round(seconds*1e3) );
}


real  sTime::operator - (rcsTime  other) const
{
  return ( _UTC - other._UTC )*1e-3;
}


sTime  sTime::LastMidnight () const
{
  sTime                   T;
  T._UTC = QDateTime( QDateTime::fromMSecsSinceEpoch(_UTC).date() )
              .toMSecsSinceEpoch();
  return T;
}

//------------------------------------------------------------------------------
} //namespace RA
