////////////////////////////////////////////////////////////////////////////////
/*! @file UnitSystemIO.cpp   Хранение системы единиц в сети.
- Part of RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2011-2013, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "UnitSystemIO.h"
#include "attr.h"
#include "Units.h"
namespace RA {
//------------------------------------------------------------------------------

void  UnitSystemLoad (sPtr<sFolderNode>  UnitSystemPtr)
{
  if( ! UnitSystemPtr.IsCorrect() )
    return;

  for_each_attribute( AF , UnitSystemPtr>>"Units" ) {
    sString                     Symbol( AF->Name().Text() );
    sPtr<sTextNode>             PrefixAllowance(AF->Value()>>"PrefixAllowance");
    sPtr<sPhysValueNode>        Definition(AF->Value()>>"Definition");
    if(       PrefixAllowance.IsCorrect() && Definition.IsCorrect() ){
      sUnit::Register(Symbol,(sString)(*PrefixAllowance)
                            ,(sPhysValue)(*Definition)   );
    }else if( PrefixAllowance.IsCorrect() ){
      sUnit::Register(Symbol,(sString)(*PrefixAllowance));
    }else if( Definition.IsCorrect() ){
      sUnit::Register(Symbol,(sPhysValue)(*Definition));
    }else{
      sUnit::Register(Symbol);
    }
  }end_attr_loop()
}


void  UnitSystemStore (sPtr<sFolderNode>  UnitSystemPtr)
{
  if( ! UnitSystemPtr.IsCorrect() )
    return;

  UnitSystemPtr |= "Units";
  sPtr<sFolderNode>           UnitsPtr( UnitSystemPtr>>"Units" );

  if( ! UnitsPtr.IsCorrect() )
    return;

  for( map<sString,psUnit>::const_iterator i = sUnit::System.UnitList.begin()
                                         ; i != sUnit::System.UnitList.end()
                                         ; i++ ){
    sString                     Symbol( i->second->Symbol() );
    sString                     NewPA( i->second->PrefixAllowance() );
    psPhysValue                 NewD( i->second->Definition() );
    sPtr<sFolderNode>           U (UnitsPtr>>Symbol);
    if( U.IsCorrect() ){
      sPtr<sTextNode>             PA(U>>"PrefixAllowance");
      sPtr<sPhysValueNode>        D(U>>"Definition");
      if( PA.IsCorrect() ){
        if( NewPA == sMetricPrefix::StandardAllowance ){
          U.DelAttr("PrefixAllowance");
        }else{
          U << attr("PrefixAllowance",NewPA);
        }
      }else if( NewPA != sMetricPrefix::StandardAllowance ){
        U |= attr("PrefixAllowance",NewPA);
      }
      if( D.IsCorrect() ){
        if( NewD ){
          try{
            real Ratio;
            Ratio = ( D->Value() / (*NewD) ).ToReal();
            if( std::abs(Ratio-1.0) > 1e-7 )
              U << attr("Definition",*NewD);
          }catch( rxRAlgebra ){
            U << attr("Definition",*NewD);
          }
        }else{
          U.DelAttr("Definition");
        }
      }else if( NewD ){
        U << attr("Definition",*NewD);
      }
    }else{
      U = (UnitsPtr >= Symbol );
      if( NewPA != sMetricPrefix::StandardAllowance ){
        U << attr("PrefixAllowance",NewPA);
      }
      if( NewD ){
        U << attr("Definition",*NewD);
      }
    }
  }
}

//------------------------------------------------------------------------------
} //namespace RA
