////////////////////////////////////////////////////////////////////////////////
/*! @file Units.cpp   Единицы измерения физвеличин.
- Part of RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2007-2011, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "Units.h"
#include "Symbols.h"
#include "PhysValue.h"
#include "Log.h"
//#include "Trace.h"
namespace RA {
//------------------------------------------------------------------------------

//csUnits                       _Unitsless_;

//csString          sMetricPrefix::StandardAllowance
//                          ("Y..Z..E..P..T..G..M..k..0..m..u..n..p..f..a..z..y");
//                         //"Y..Z..E..P..T..G..M..khd0dcm..u..n..p..f..a..z..y"

//csString          sMetricPrefix::NoPrefixAllowed
//                          ("-..-..-..-..-..-..-..-..0..-..-..-..-..-..-..-..-");
//                         //"Y..Z..E..P..T..G..M..khd0dcm..u..n..p..f..a..z..y"

//sUnit::sSystem                sUnit::System;
//sUnit::pfRegisterDialog       sUnit::RegisterDialog = NULL;
//psUnit                        sUnit::_1_ = new sUnit();

//------------------------------------------------------------ sMetricPrefix ---

sString  sMetricPrefix::Symbol (int  Log)
{
  switch( Log ){
    case  24:  return sString("Y");
    case  21:  return sString("Z");
    case  18:  return sString("E");
    case  15:  return sString("P");
    case  12:  return sString("T");
    case   9:  return sString("G");
    case   6:  return sString("M");
    case   3:  return sString("k");
    case   2:  return sString("h");
    case   1:  return sString("da");
    case   0:  return sString("");
    case  -1:  return sString("d");
    case  -2:  return sString("c");
    case  -3:  return sString("m");
    case  -6:  return sString(Symbol::Micro);
    case  -9:  return sString("n");
    case -12:  return sString("p");
    case -15:  return sString("f");
    case -18:  return sString("a");
    case -21:  return sString("z");
    case -24:  return sString("y");
    default:   return sString("?");
  };
}


int  sMetricPrefix::Exp (sString  Prefix)
{
  if( Prefix == ""){           return   0;
  }else if( Prefix == "k"){    return   3;
  }else if( Prefix == "m"){    return  -3;
  }else if( Prefix == "M"){    return   6;
  }else if( Prefix ==
            Symbol::Micro){    return  -6;
  }else if( Prefix == "G"){    return   9;
  }else if( Prefix == "n"){    return  -9;
  }else if( Prefix == "T"){    return  12;
  }else if( Prefix == "p"){    return -12;
  }else if( Prefix == "P"){    return  15;
  }else if( Prefix == "f"){    return -15;
  }else if( Prefix == "E"){    return  18;
  }else if( Prefix == "a"){    return -18;
  }else if( Prefix == "Z"){    return  21;
  }else if( Prefix == "z"){    return -21;
  }else if( Prefix == "Y"){    return  24;
  }else if( Prefix == "y"){    return -24;
  }else if( Prefix == "h"){    return   2;
  }else if( Prefix == "c"){    return  -2;
  }else if( Prefix == "da"){   return   1;
  }else if( Prefix == "d"){    return  -1;
  }else{
    return 0;
  };
}


real  sMetricPrefix::Multiplier (int  Log)
{
  switch( Log ){
    case   0:  return (real)1.0e0;
    case   3:  return (real)1.0e3;
    case  -3:  return (real)1.0e-3;
    case   6:  return (real)1.0e6;
    case  -6:  return (real)1.0e-6;
    case   9:  return (real)1.0e9;
    case  -9:  return (real)1.0e-9;
    case  12:  return (real)1.0e12;
    case -12:  return (real)1.0e-12;
    case  15:  return (real)1.0e15;
    case -15:  return (real)1.0e-15;
    case  18:  return (real)1.0e18;
    case -18:  return (real)1.0e-18;
    case  21:  return (real)1.0e21;
    case -21:  return (real)1.0e-21;
    case  24:  return (real)1.0e24;
    case -24:  return (real)1.0e-24;
    case   2:  return (real)1.0e2;
    case  -2:  return (real)1.0e-2;
    case   1:  return (real)1.0e1;
    case  -1:  return (real)1.0e-1;
    default: return power10(Log);
  };
}


sString  sMetricPrefix::Name (int  Log)
{
  switch( Log ){
    case  24:  return sString("yotta");
    case  21:  return sString("zetta");
    case  18:  return sString("exa");
    case  15:  return sString("peta");
    case  12:  return sString("tera");
    case   9:  return sString("giga");
    case   6:  return sString("mega");
    case   3:  return sString("kilo");
    case   2:  return sString("hecto");
    case   1:  return sString("deca");
    case   0:  return sString("");
    case  -1:  return sString("deci");
    case  -2:  return sString("centi");
    case  -3:  return sString("milli");
    case  -6:  return sString("micro");
    case  -9:  return sString("nano");
    case -12:  return sString("pico");
    case -15:  return sString("femto");
    case -18:  return sString("atto");
    case -21:  return sString("zepto");
    case -24:  return sString("yocto");
    default:   return sString("?????");
  };
}


sString  sMetricPrefix::CombinedSymbol (int  Log)
{
  sString                     PP(Symbol(Log));
  if( PP != "?" ){
    return PP;
  }else{
    PP = "";
    while( (Log < -2) || (Log > 2) ){
      int                         L;
      L = (Log / 3) * 3;
      if( L > 24 ){
        L = 24;
      }else if( L < -24 ){
        L = -24;
      };
      PP = Symbol(L) + PP;
      Log -= L;
    };
    PP = Symbol(Log) + PP;
  };
  return PP;
}

//----------------------------------------------------------- sUnit::sSystem ---

sUnit::sSystem::~sSystem ()
{
//  StoragePlace.SetToNULL();
//  while( ! UnitList.empty() ){
//    pop
//    delete ;
//  };
}


sUnit::sSystem::sSystem ()
{

}

//-------------------------------------------------------------------- sUnit ---
/*! Регистрирует производную единицу измерений.

Если такая единица уже была зарегистрирована, то это не ошибка: для неё
подменяются prefix_allowance и definition.

*/
psUnit  sUnit::Register (sString     symbol
                        ,sString     prefix_allowance
                        ,sPhysValue  definition)
{
  if( symbol.Empty() ){
    return _1_;
  }else{
    psUnit                    ExistingU = Find(symbol);
    if( ExistingU ){
      if( ExistingU->ThePrefixAllowance != prefix_allowance ){
        ExistingU->ThePrefixAllowance = prefix_allowance;
//        RANet::Log.Put(sLog::Debug,"Units"
//                      ,sString("Prefix allowance changed for '")+symbol+"'");
      };
      sString                     Old("basic");
      if( ExistingU->TheDefinition ){
        Old = ExistingU->TheDefinition->Text();
        delete ExistingU->TheDefinition;
      };
      ExistingU->TheDefinition = new sPhysValue(definition);
//      RANet::Log.Put(sLog::Debug,"Units"
//                    ,sString("Definition changed for '")+symbol
//                    +"': was "+Old+"; now "+definition.Text());
      return ExistingU;
    }else{
      psUnit              NewU( new sUnit(symbol
                                         ,prefix_allowance
                                         ,new sPhysValue(definition) ) );
      System.UnitList[NewU->Symbol()]=NewU;
//      RANet::Log.Put(sLog::Debug,"Units"
//                    ,sString("New unit registered: ")
//                    +symbol+" = "+definition.Text());
      return NewU;
    };
  };
}


/*! Регистрирует базовую единицу измерений.

Если такая единица уже была зарегистрирована, то это не ошибка: для неё
подменяется prefix_allowance.  При этом, если она была производной (то есть
имела definition), то такой и останется.

*/
psUnit  sUnit::Register (sString  symbol
                        ,sString  prefix_allowance)
{
  if( symbol.Empty() ){
    return _1_;
  }else{
    psUnit                    ExistingU = Find(symbol);
    if( ExistingU ){
      if( ExistingU->ThePrefixAllowance != prefix_allowance ){
        ExistingU->ThePrefixAllowance = prefix_allowance;
//        RANet::Log.Put(sLog::Debug,"Units"
//                      ,sString("Prefix allowance changed for '")+symbol+"'");
      };
      return ExistingU;
    }else{
      psUnit              NewU( new sUnit(symbol,prefix_allowance,NULL) );
      System.UnitList[NewU->Symbol()]=NewU;
//      RANet::Log.Put(sLog::Debug,"Units"
//                    ,sString("New unit registered: ")+symbol);
      return NewU;
    };
  };
}


/*! Регистрирует производную единицу измерений.

Если такая единица уже была зарегистрирована, то это не ошибка: для неё
подменяется definition.  Если она была базовой, то больше не будет.  Заданное
ранее prefix_allowance не изменяется.

*/
psUnit  sUnit::Register (sString     symbol
                        ,sPhysValue  definition)
{
  if( symbol.Empty() ){
    return _1_;
  }else{
    psUnit                      ExistingU = Find(symbol);
    if( ExistingU ){
      sString                     Old("basic");
      if( ExistingU->TheDefinition ){
        Old = ExistingU->TheDefinition->Text();
        delete ExistingU->TheDefinition;
      };
      ExistingU->TheDefinition = new sPhysValue(definition);
//      RANet::Log.Put(sLog::Debug,"Units"
//                    ,sString("Definition changed for '")+symbol
//                    +"': was "+Old+"; now "+definition.Text());
      return ExistingU;
    }else{
      psUnit              NewU( new sUnit(symbol
                                         ,sMetricPrefix::StandardAllowance
                                         ,new sPhysValue(definition) ) );
      System.UnitList[NewU->Symbol()]=NewU;
//      RANet::Log.Put(sLog::Debug,"Units"
//                    ,sString("New unit registered: ")
//                    +symbol+" = "+definition.Text());
      return NewU;
    };
  };
}


/*! Регистрирует базовую единицу измерений.

Если такая единица уже была зарегистрирована, то это не ошибка. При этом, если
она была производной (то есть имела definition), то такой и останется. Заданное
ранее prefix_allowance не изменяется.

*/
psUnit  sUnit::Register (sString  symbol)
{
  if( symbol.Empty() ){
    return _1_;
  }else{
    psUnit                    ExistingU = Find(symbol);
    if( ExistingU ){
      return ExistingU;
    }else{
      psUnit              NewU( new sUnit(symbol
                                         ,sMetricPrefix::StandardAllowance
                                         ,NULL) );
      System.UnitList[NewU->Symbol()]=NewU;
//      RANet::Log.Put(sLog::Debug,"Units"
//                    ,sString("New unit registered: ")+symbol+" (basic)");
      return NewU;
    };
  };
}


psUnit  sUnit::Find (sString  symbol)
{
  map<sString,psUnit>::iterator    U = System.UnitList.find(symbol);
  if( U == System.UnitList.end() ){
    return NULL;
  }else{
    return U->second;
  };
}


bool  sUnit::ParseUnitWithPrefix (sString   lexem
                                 ,psUnit &  unit
                                 ,int &     prefixlg)
{
  prefixlg=0;
  unit=sUnit::Find(lexem);
  if( ! unit ){
    for( int L = 24 ; (L >= -24) && (unit == NULL) ; --L ){
      sString                     Prefix = sMetricPrefix::Symbol(L);
      int                         K=0;
      if( ! Prefix.Empty() && (Prefix != "?") ){
        if( (lexem.Substr(0,Prefix.Length()) == Prefix)
            && (lexem.Length() > Prefix.Length())        ){
//          TRACE(Prefix+"|"+lexem.Tail(Prefix.Length()));
          if( ParseUnitWithPrefix(lexem.Tail(Prefix.Length()),unit,K) ){
//            TRACE(unit->Symbol());
            prefixlg=L+K;
            break;
          };
        };
      };
    };
  };
  return ! (unit == NULL);
}


/*! Резервирует наиболее важные единицы.

Помогает избежать следующей проблемы:

Новые единицы автоматически регистрируются в плагинах и диалоге.  И, например,
"ms" будет зарегистрирована как базовая, если до этого не регистрировалась "s".
Если "s" зарегистрирована до первого появления "ms", то "ms" распознаётся как
milli-"s".

Последнее, впрочем, не мешает всяким извращенцам явно зарегистрировать "ms" со
смыслом, отличным от milli-"s". Тогда "ms" будет означать, например, единицу
маразма, "Mms" - мегамаразм, но "ks" и "ns" останутся производными от "s".

*/
void  sUnit::MakeVeryBasicDefinitions ()
{
  Register("m");
  Register("g");
  Register("s");
  Register("A");
  Register("K");
  //cannot use sPhysValue yet!
}

//------------------------------------

/*! copying is not allowed */
sUnit::sUnit (rcsUnit  U)
    :sAdam()
    ,sStorable()
    ,Is_1_Flag(U.Is_1_Flag)
    ,IsObsoleteFlag(U.IsObsoleteFlag)
    ,TheSymbol(U.TheSymbol)
    ,ThePrefixAllowance(U.ThePrefixAllowance)
    ,TheDefinition(U.TheDefinition)
{
}


/*! assignment op is not allowed */
rsUnit  sUnit::operator = (rcsUnit  U)
{
  Is_1_Flag = U.Is_1_Flag;
  IsObsoleteFlag = U.IsObsoleteFlag;
  TheSymbol = U.TheSymbol;
  ThePrefixAllowance = U.ThePrefixAllowance;
  TheDefinition = U.TheDefinition;
  return *this;
}


sUnit::sUnit ()
    :Is_1_Flag(true)
    ,IsObsoleteFlag(false)
    ,TheSymbol(Symbol::Unit_1)
    ,ThePrefixAllowance(sMetricPrefix::NoPrefixAllowed)
    ,TheDefinition(NULL)
{
}


sUnit::sUnit (sString      symbol
             ,sString      prefix_allowance
             ,psPhysValue  definition)
    :Is_1_Flag(false)
    ,IsObsoleteFlag(false)
    ,TheSymbol(symbol)
    ,ThePrefixAllowance(prefix_allowance)
    ,TheDefinition(definition)
{
}


sUnit::sUnit (std::istream &  a_stream ,rcsVersion  version)
    :sStorable(a_stream,version)
    ,TheDefinition(NULL)
{
  bool                        HasDefinition;
  a_stream.READ_INTO(Is_1_Flag);
  a_stream.READ_INTO(IsObsoleteFlag);
  TheSymbol.Read(a_stream);
  ThePrefixAllowance.Read(a_stream);
  a_stream.READ_INTO(HasDefinition);
  if( HasDefinition ){
    TheDefinition = new sPhysValue(a_stream,version);
  };
  Register(TheSymbol);
}


void  sUnit::Store (std::ostream &  a_stream) const
{
  bool                        HasDefinition = (TheDefinition != NULL);
  sStorable::Store(a_stream);
  a_stream.WRITE_FROM(Is_1_Flag);
  a_stream.WRITE_FROM(IsObsoleteFlag);
  TheSymbol.Write(a_stream);
  ThePrefixAllowance.Write(a_stream);
  a_stream.WRITE_FROM(HasDefinition);
  if( HasDefinition ){
    TheDefinition->Store(a_stream);
  };
}


bool  sUnit::PrefixAllowed (int  prefix_index) const
{
  char                        A = ThePrefixAllowance[24-prefix_index];
  return (A != '.') && (A != '-');
}

//------------------------------------------------------------------- sUnits ---

sUnits  sUnits::Parse (sString  S)
{
//  RANet::Log.OpenBlock(sLog::Debug,"sUnits::Parse",S);
  sUnits                      UU;
  bool                        Division = false;
  size_t                      P = std::min( S.Pos("*") , S.Pos("/") );
  sString                     Term;
  int                         PrefixLg;
  int                         Power;
  psUnit                      Unit;
  while( ! S.Empty() ){
    Term = S.Substr(0,P);
    Power=1;
    size_t                      PP = std::min( Term.Pos("+") , Term.Pos("-") );
    for( char D = '0' ; D <= '9' ; ++D )
      PP = std::min( PP , Term.Pos(sString(D)) );
    if( PP < Term.Length() ){
      sString                     PowerText=Term.Tail(PP);
      Term=Term.Substr(0,PP);
      PowerText >> Power;
    };
    if( Division )  Power = - Power;
    Term.EatLeadingSpaces();
    if( ! Term.Empty() ){
      if( sUnit::ParseUnitWithPrefix(Term,Unit,PrefixLg) ){
        UU.Product.push_back(sTerm(Unit,Power,PrefixLg));
      }else if( sUnit::RegisterDialog ){
        if( (*sUnit::RegisterDialog)(Term) ){
          if( sUnit::ParseUnitWithPrefix(Term,Unit,PrefixLg) ){
            UU.Product.push_back(sTerm(Unit,Power,PrefixLg));
          };
        };
      }else{
        UU.Product.push_back(sTerm(sUnit::Register(Term),Power,0));
      };
    };
    if( P < S.Length() )
      Division = (S[P] == '/');
    S = S.Tail(P+1);
    P = std::min( S.Pos("*") , S.Pos("/") );
  };
//  RANet::Log.Put(sLog::Debug,"sUnits::Parse",UU.Text());
//  RANet::Log.CloseBlock(sLog::Debug,"sUnits::Parse");
  return UU;
}

//----------------------------------------

sUnits::~sUnits ()
{
  Product.clear();
}


sUnits::sUnits (std::istream &  a_stream ,rcsVersion  version)
    :sStorable(a_stream,version)
{
  a_stream.READ_INTO(InBasicTerms);
  a_stream.READ_INTO(Refined);
  bool                        Terminator = false;
  a_stream.READ_INTO(Terminator);
  while( ! Terminator ){
    sString                     Symbol;
    int                         Power;
    int                         PrefixLg;
    Symbol.Read(a_stream);
    a_stream.READ_INTO(Power);
    a_stream.READ_INTO(PrefixLg);
    Product.push_back( sTerm(sUnit::Register(Symbol),Power,PrefixLg) );
    a_stream.READ_INTO(Terminator);
  };
}


void  sUnits::Store (std::ostream &  a_stream) const
{
  sStorable::Store(a_stream);
  a_stream.WRITE_FROM(InBasicTerms);
  a_stream.WRITE_FROM(Refined);
  bool                        Terminator = false;
  for( tProduct::const_iterator i = Product.begin()
                              ; i != Product.end() ; i++ ){
    a_stream.WRITE_FROM(Terminator);
    i->Unit->Symbol().Write(a_stream);
    a_stream.WRITE_FROM(i->Power);
    a_stream.WRITE_FROM(i->PrefixLg);
  };
  Terminator = true;
  a_stream.WRITE_FROM(Terminator);
}


sUnits::sUnits (rcsUnits  U)
    :sAdam()
    ,sStorable()
    ,Product(U.Product)
    ,InBasicTerms(U.InBasicTerms)
    ,Refined(U.Refined)
{
}


sUnits::sUnits ()
    :InBasicTerms(true)
    ,Refined(true)
{
}


sUnits::sUnits (psUnit unit
               ,int    power
               ,int    prefix_lg)
    :InBasicTerms(unit->IsBasic())
    ,Refined(true)
{
  Product.push_back(sTerm(unit,power,prefix_lg));
}


sUnits::sUnits (sString  text)
{
  try{
    sUnits                      U( sUnits::Parse(text) );
    Product = U.Product;
    InBasicTerms = U.InBasicTerms;
    Refined = U.Refined;
  }catch(...){
    Product.clear();
    InBasicTerms = true;
    Refined = true;
  };
}


bool  sUnits::Is_1_ () const
{
  return Product.empty();
}


rsUnits  sUnits::operator = (rcsUnits  U)
{
  Product = U.Product;
  InBasicTerms = U.InBasicTerms;
  Refined = U.Refined;
  return *this;
}


sPhysValue  sUnits::operator * (rcsUnits  U) const
{
  sPhysValue                  Result(*this);
  Result.TheOrder += Result.TheUnits.MultBy(U);
  return Result;
}


sPhysValue  sUnits::operator / (rcsUnits  U) const
{
  sPhysValue                  Result(*this);
  Result.TheOrder += Result.TheUnits.DivideBy(U);
  return Result;
}


int  sUnits::MultBy (rcsUnits  U)
{
  for( tProduct::const_iterator i = U.Product.begin()
                              ; i != U.Product.end() ; i++ ){
    Product.push_back(*i);
  };
  InBasicTerms = false;
  Refined = false;
  return SimplifyProduct();
}


int  sUnits::DivideBy (rcsUnits  U)
{
  for( tProduct::const_iterator i = U.Product.begin()
                              ; i != U.Product.end() ; i++ ){
    Product.push_back( sTerm( i->Unit , - i->Power , i->PrefixLg ) );
  };
  InBasicTerms = false;
  Refined = false;
  return SimplifyProduct();
}


sPhysValue  sUnits::ToBasic () const
{
  real                        Coeff = 1.0;
  sUnits                      U(*this);
  int                         Order;
  SubstituteDefinitions(Coeff,U);
  U.InBasicTerms = true;
  U.Refined = false;
  Order = U.SimplifyProduct();
  sPhysValue                  Result(Coeff,U);
  Result << Order;
  return Result;
}


//static
void  sUnits::SubstituteDefinitions (real &   coeff
                                    ,rsUnits  units)
{
  sUnits                      Result;
  bool                        Changed = false;
  for( tProduct::const_iterator i = units.Product.begin()
                              ; i != units.Product.end() ; i++ ){
    psPhysValue                 D = i->Unit->Definition();
    if( D ){
      Changed = true;
      real                        C = sMetricPrefix::Multiplier(i->PrefixLg)
                                      * D->TheValueMantissa
                                      * power10(D->TheOrder);
      sUnits                      DU(D->Units());
      if( i->Power > 0 ){
        for( int p = i->Power ; p != 0 ; p-- )  coeff *= C;
      }else if( i->Power < 0 ){
        for( int p = i->Power ; p != 0 ; p++ )  coeff /= C;
      };
      for( tProduct::const_iterator j = DU.Product.begin()
                                  ; j != DU.Product.end() ; j++ ){
        Result.Product.push_back( sTerm(j->Unit
                                       ,j->Power * i->Power
                                       ,j->PrefixLg          ) );
      };
    }else{
      Result.Product.push_back(*i);
    };
  };
  if( Changed ){
    SubstituteDefinitions(coeff,Result);
    units = Result;
  };
}


bool  sUnits::Comparable (rcsUnits  units
                         ,real &    ratio) const
{
//  RANet::Log.OpenBlock(sLog::Debug,"sUnits::Comparable"
//                      ,sString("Comparing '")+Text()+"' to '"+units.Text()+"'");
  sPhysValue                  R( ToBasic() / units.ToBasic() );
  if( R.TheUnits.Product.empty() ){
    ratio = R.TheValueMantissa*power10(R.TheOrder);
//    RANet::Log.Put(sLog::Debug,"sUnits::Comparable"
//                  ,sString("Comparable with ratio=")+R.Text());
//    RANet::Log.CloseBlock(sLog::Debug,"sUnits::Comparable");
    return true;
  };
//  RANet::Log.Put(sLog::Debug,"sUnits::Comparable",sString("NOT comparable"));
//  RANet::Log.CloseBlock(sLog::Debug,"sUnits::Comparable");
  return false;
}


/*! Упрощает результат произведения или деления единиц.

В результате сомножители отсортированы, не дублируются, не содержат _1_
и лишены метрических префиксов.
Результирующий показатель порядкового множителя возвращается.
*/
int  sUnits::SimplifyProduct (int  log_shift)
{
  tProduct                    P(Product);
  P.sort();
  Product.clear();
  while( ! P.empty() ){
    sTerm                       T(P.front());
    P.pop_front();
    log_shift += T.PrefixLg*T.Power;
    T.PrefixLg = 0;
    if( T.Unit == sUnit::_1_ ){
      //do nothing
    }else if( Product.empty() ){
      Product.push_back(T);
    }else if( Product.back().Unit->Symbol() != T.Unit->Symbol() ){
      Product.push_back(T);
    }else{
      Product.back().Power += T.Power;
      if( Product.back().Power == 0 ){
        Product.pop_back();
      };
    };
  };
  return log_shift;
}


void  sUnits::SmartSort ()
{
  tProduct                    P(Product);
  Product.clear();
  for( tProduct::const_iterator i = P.begin() ; i != P.end() ; i++ ){
    if( i->Power > 0 )  Product.push_back(*i);
  };
  for( tProduct::const_iterator i = P.begin() ; i != P.end() ; i++ ){
    if( i->Power < 0 )  Product.push_back(*i);
  };
}


int  sUnits::DissolveOrder (unsigned short int  level
                           ,int                 log_shift
                           ,int                 min_rest_expected
                           ,int                 max_rest_expected)
{
//  RANet::Log.OpenBlock(sLog::Debug,"sUnits::DissolveOrder"
//                                  ,sString("'")+Text()+"'");
//  TRACENOCR(log_shift);
  log_shift = SimplifyProduct(log_shift);
  if( ! Product.empty() ){
    SmartSort();
    //! @todo{PhysValues} начинать с разрешённых префиксов
    tPrefixes                   BestChoice(0,Product.size());
    int                         BestWeight = std::numeric_limits<int>::min();
    int                         BestRestOrder = log_shift;
    FindBestChoice(BestChoice,BestWeight,BestRestOrder
                  ,min_rest_expected,max_rest_expected
                  ,BestChoice,0         ,log_shift    ,0
                  ,level);

    int                         ii=0;
    for( tProduct::iterator i = Product.begin() ; i != Product.end() ; i++ ){
      i->PrefixLg = BestChoice[ii++];
    };
    log_shift = BestRestOrder;
  };
//  RANet::Log.Put(sLog::Debug,"sUnits::DissolveOrder",sString("result '")+Text()+"'");
//  RANet::Log.CloseBlock(sLog::Debug,"sUnits::DissolveOrder");
  return log_shift;
}


void  sUnits::FindBestChoice (tPrefixes &  best_choice
                             ,int       &  best_weight
                             ,int       &  best_rest_order
                             ,int          min_rest_expected
                             ,int          max_rest_expected
                             ,tPrefixes    current_prefixes
                             ,int          weight
                             ,int          rest_order
                             ,size_t       term_to_vary
                             ,unsigned short int  level
                             )
{
  if(    (term_to_vary >= level)
      || (term_to_vary >= current_prefixes.size()) ){
    if( (rest_order > max_rest_expected) || (rest_order < min_rest_expected) )
      weight -= 10000;
    if( weight > best_weight ){
      best_weight = weight;
      best_rest_order = rest_order;
      best_choice = current_prefixes;
//      TRACE(best_weight);
//      TRACE(best_rest_order);
//      sString                     best_text;
//      for( tProduct::const_iterator i = best_choice.begin()
//                                  ; i != best_choice.end() ; i++ ){
//        if( ! best_text.Empty() ){
//            best_text += "*";
//        };
//        if( i->PrefixLg ){
//          best_text += sMetricPrefix::CombinedSymbol(i->PrefixLg);
//        };
//        best_text += i->Unit->Symbol();
//        if( i->Power != 1 ){
//          best_text << i->Power;
//        };
//      };
//      TRACE(best_text);
    };
  }else{
    tProduct::const_iterator    i = Product.begin();
    for( size_t ii = 0 ; ii < term_to_vary ; ii++ ){
      i++;
    };
    for( int L = 24 ; L >= -24 ; --L ){
      if( i->Unit->PrefixAllowed(L) ){
        int                         WeightDownshift;
        WeightDownshift = ( L==0 ? 0                        //за каждый префикс
                                 : ( term_to_vary==0 ?  700   //на первом месте
                                                     : 1000 ))//на остальных
                        + ( (L % 3)==0 ? 0 : 2000 ) //за каждый "кривой" префикс
                        + ( L>0 ? L*30 : -L*35 )  //за "величину" префикса
                        ;
        current_prefixes[term_to_vary] = L;
        FindBestChoice(best_choice,best_weight,best_rest_order
                      ,min_rest_expected,max_rest_expected
                      ,current_prefixes
                      ,weight - WeightDownshift
                      ,rest_order - L * i->Power
                      ,term_to_vary+1
                      ,level);
      };
    };
  };
}


//void  sUnits::OrderPushIn (int  log_shift)
//{
//  if( log_shift == 0 ){
//    //do nothing
//  }else if( Product.empty() ){
//    Product.push_back(sTerm(sUnit::_1_,1,log_shift));
//  }else{
//    for( tProduct::iterator i = Product.begin()
//                          ; (i != Product.end()) && (log_shift != 0) ; i++ ){
//      i->PrefixLg += log_shift / i->Power;
//      log_shift = log_shift % i->Power;
//    };
//    if( log_shift != 0 ){
//      Product.push_front(sTerm(sUnit::_1_,1,log_shift));
//    };
//  };
//}


sString  sUnits::Text (eTextFormat  F) const
{
  sString                     T;
  int                         NegativePowerCount = 0;
  int                         TermCount = 0;
  int                         NumOfTerms = 0;
  for( tProduct::const_iterator i = Product.begin()
                              ; i != Product.end() ; i++ ){
    NumOfTerms++;
    if( i->Power < 0 )  NegativePowerCount++;
  };
  for( tProduct::const_iterator i = Product.begin()
                              ; i != Product.end() ; i++ ){
    int                         i_Power = i->Power;
    TermCount++;
    if( ! T.Empty() ){
      if( (i_Power < 0) && (NegativePowerCount == 1)
                        && (TermCount == NumOfTerms) ){
        T += "/";
        i_Power = - i_Power;
      }else{
        if( F == Plain ){
          T += "*";
        }else{
          T += "·";
        };
      };
    };
    if( i->PrefixLg ){
      T += sMetricPrefix::CombinedSymbol(i->PrefixLg);
      IF_DEBUG( if( F == HTML ) T += "|" );
    };
    T += i->Unit->Symbol();
    if( i_Power != 1 ){
      if( F == HTML )  T += "<sup>";
      T << i_Power;
      if( F == HTML )  T += "</sup>";
    };
  };
  return T;
}

//------------------------------------------------------------------------------
} //namespace RA

