////////////////////////////////////////////////////////////////////////////////
/*! @file Version.cpp   Source code version identification.
- Part of RANet - Research Assistant Net Library.
- Copyright(C) 2010-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "Version.h"
namespace RA {
//------------------------------------------------------------------------------

namespace RANet {

  sVersion const              Version (RA_VERS_MAJOR
                                      ,RA_VERS_MINOR
                                      ,ZAKAVYCH(RA_VERS_PATCH)
                                      ,__DATE__
                                      );
}

//----------------------------------------------------------------- sVersion ---

sVersion::sVersion (sString  text)
    :Major(-9),Minor(-9),Patch(-9),Stage(-9)
{
  size_t                      P;
  sString                     PatchAndStage;
  text >> Major;
  P = text.Pos(".");
  if( P < text.Length() ){
    text = text.Tail(P+1);
    text >> Minor;
    P = text.Pos(".");
    if( P < text.Length() ){
      text = text.Tail(P+1);
      P = text.Pos(" (");
      if( P == text.Length() ){
        PatchAndStage = text;
      }else{
        PatchAndStage = text.Substr(0,P);
      }
    }
  }
  ParsePatchAndStage(PatchAndStage);
}


sVersion::sVersion (int        major_part
                   ,int        minor_part
                   ,rcsString  patch_and_stage
                   ,rcsString  build_date)
    :Major(major_part),Minor(minor_part),Patch(-9),Stage(-9)
    ,BuildDate(build_date)
{
  ParsePatchAndStage(patch_and_stage);
}


void  sVersion::ParsePatchAndStage (rcsString  patch_and_stage)
{
  sString                     S1(patch_and_stage);
  size_t                      P = patch_and_stage.Pos("-");
  if( P == patch_and_stage.Length() ){
    Stage = 0;
  }else{
    S1 = patch_and_stage.Substr(0,P);
    sString                     S2(patch_and_stage.Tail(P+1));
    if( S2 == "alpha" ){
      Stage = -2;
    }else if( S2 == "beta" ){
      Stage = -1;
    }
  }
  sString                     T;
  sString(S1) >> Patch;
  T << Patch;
  if( T != S1 )  Patch = -9;
}


sVersion::sVersion ()
    :Major(-9),Minor(-9),Patch(-9),Stage(-9)
{
}


sVersion::sVersion (int  major_part
                   ,int  minor_part)
    :Major(major_part),Minor(minor_part),Patch(-9),Stage(-9)
{
}


sVersion::sVersion (int  major_part
                   ,int  minor_part
                   ,int  patch_part
                   ,int  stage_part)
    :Major(major_part),Minor(minor_part),Patch(patch_part),Stage(stage_part)
{
}


sVersion::sVersion (rcsVersion  other)
    :Major(other.Major),Minor(other.Minor),Patch(other.Patch)
    ,Stage(other.Stage),BuildDate(other.BuildDate)
{
}


rsVersion  sVersion::operator = (rcsVersion  other)
{
  Major = other.Major;
  Minor = other.Minor;
  Patch = other.Patch;
  Stage = other.Stage;
  BuildDate = other.BuildDate;
  return *this;
}


sString  sVersion::Text () const
{
  sString                     V;
  if( ! IsValid() ){
    V = "invalid version";
  }else{
    V << Major;
    V += ".";
    V << Minor;
    if( Patch >= 0 ){
      V += ".";
      V << Patch;
      switch( Stage ){
        case -2:
          V += "-alpha";
          break;
        case -1:
          V += "-beta";
          break;
        case 0:
          break;
        default:
          V += "-???";
          break;
      }
    }
    if( ! BuildDate.Empty() ){
      V += " (";
      V += BuildDate;
      V += ")";
    }
  }
  return V;
}


bool  sVersion::IsValid () const
{
  return  (Major >= 0) && (Minor >= 0);
}


bool  sVersion::IsValidAndNumeric () const
{
  return  (Major >= 0) && (Minor >= 0) && (Patch >= 0) && (Stage == 0);
}


bool  sVersion::operator == (rcsVersion  V) const
{
  return  (Major == V.Major) && (Minor == V.Minor) && (Patch == V.Patch)
                                                   && (Stage == V.Stage);
}


bool  sVersion::operator < (rcsVersion  V) const
{
  return   (Major < V.Major)
      || ( (Major == V.Major) && (Minor < V.Minor) )
      || ( (Major == V.Major) && (Minor == V.Minor) && (Patch < V.Patch) )
      || ( (Major == V.Major) && (Minor == V.Minor) && (Patch == V.Patch)
                                                    && (Stage < V.Stage) )
      ;
}


bool  sVersion::operator <= (rcsVersion  V) const
{
  return  operator < (V) || operator == (V);
}


bool  sVersion::operator > (rcsVersion  V) const
{
  return   (Major > V.Major)
      || ( (Major == V.Major) && (Minor > V.Minor) )
      || ( (Major == V.Major) && (Minor == V.Minor) && (Patch > V.Patch) )
      || ( (Major == V.Major) && (Minor == V.Minor) && (Patch == V.Patch)
                                                    && (Stage > V.Stage) )
      ;
}


bool  sVersion::operator >= (rcsVersion  V) const
{
  return  operator > (V) || operator == (V);
}

//------------------------------------------------------------------------------
} //namespace RA
