////////////////////////////////////////////////////////////////////////////////
/*! @file attr.h   Function attr(...) for constructing attributes.
- Part of RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2006-2013, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef attr_H
#define attr_H
#include "Attribute.h"
#include "FolderNode.h"
#include "BasicNodes.h"
namespace RA {
//------------------------------------------------------------------------------

//------------------------------------------------------------------- folder ---


inline sAttribute  folder ()
{
  return sAttribute(sAttrName() ,sNodePtr(new sFolderNode) );
}


inline sAttribute  folder (sString docket)
{
  return sAttribute(sAttrName() ,sNodePtr(new sFolderNode(docket)) );
}


inline sAttribute  folder (literal docket)
{
  return sAttribute(sAttrName() ,sNodePtr(new sFolderNode(docket)) );
}

inline sAttribute  folder (literal name ,sString docket)
{
  return sAttribute(sAttrName(name) ,sNodePtr(new sFolderNode(docket)) );
}


inline sAttribute  folder (literal name ,literal docket)
{
  return sAttribute(sAttrName(name) ,sNodePtr(new sFolderNode(docket)) );
}


inline sAttribute  folder (integer name ,sString docket)
{
  return sAttribute(sAttrName(name) ,sNodePtr(new sFolderNode(docket)) );
}


inline sAttribute  folder (integer name ,literal docket)
{
  return sAttribute(sAttrName(name) ,sNodePtr(new sFolderNode(docket)));
}


//--------------------------------------------------------------------- attr ---

inline sAttribute  attr (rcsAttrName name ,sNodePtr value)
{
  return sAttribute( name ,value );
}


inline sAttribute  attr (rcsAttrName name ,sString value)
{
  return sAttribute( name ,sNodePtr(new sTextNode(value)) );
}


inline sAttribute  attr (rcsAttrName name ,literal value)
{
  return sAttribute( name ,sNodePtr(new sTextNode(value)) );
}


inline sAttribute  attr (rcsAttrName name ,bool value)
{
  return sAttribute( name ,sNodePtr(new sBooleanNode(value)) );
}


inline sAttribute  attr (rcsAttrName name ,int value)
{
  return sAttribute( name ,sNodePtr(new sIntegerNode(value)) );
}


inline sAttribute  attr (rcsAttrName name ,long int value)
{
  return sAttribute( name ,sNodePtr(new sIntegerNode(value)) );
}


inline sAttribute  attr (rcsAttrName name ,sPhysValue value)
{
  return sAttribute( name ,sNodePtr(new sPhysValueNode(value)) );
}


inline sAttribute  attr (rcsAttrName name ,double value)
{
  return sAttribute( name ,sNodePtr(new sPhysValueNode(sPhysValue(value))) );
}


inline sAttribute  attr (rcsAttrName name ,long double value)
{
  return sAttribute( name ,sNodePtr(new sPhysValueNode(sPhysValue(value))) );
}


inline sAttribute  attr (rcsAttrName name ,sPhysRange value)
{
  return sAttribute( name ,sNodePtr(new sPhysRangeNode(value)) );
}

//-------------------------------------------------------------- nameless---


inline sAttribute  attr (sNodePtr value)
{
  return sAttribute( sAttrName() ,value );
}


inline sAttribute  attr (sString value)
{
  return sAttribute( sAttrName() ,sNodePtr(new sTextNode(value)) );
}


inline sAttribute  attr (literal value)
{
  return sAttribute(sAttrName() ,sNodePtr(new sTextNode(value)) );
}


inline sAttribute  attr (bool value)
{
  return sAttribute( sAttrName() ,sNodePtr(new sBooleanNode(value)) );
}


inline sAttribute  attr (int value)
{
  return sAttribute( sAttrName() ,sNodePtr(new sIntegerNode(value)) );
}


inline sAttribute  attr (long int value)
{
  return sAttribute( sAttrName() ,sNodePtr(new sIntegerNode(value)) );
}


inline sAttribute  attr (sPhysValue value)
{
  return sAttribute(sAttrName() ,sNodePtr(new sPhysValueNode(value)) );
}


inline sAttribute  attr (double value)
{
  return sAttribute(sAttrName()
                   ,sNodePtr(new sPhysValueNode(sPhysValue(value))) );
}


inline sAttribute  attr (long double value)
{
  return sAttribute(sAttrName()
                   ,sNodePtr(new sPhysValueNode(sPhysValue(value))) );
}


inline sAttribute  attr (sPhysRange value)
{
  return sAttribute(sAttrName() ,sNodePtr(new sPhysRangeNode(value)) );
}

//-------------------------------------------------------------- symbolic---

inline sAttribute  attr (literal name ,sNodePtr value)
{
  return sAttribute( sAttrName(name) ,value );
}


inline sAttribute  attr (literal name ,sString value)
{
  return sAttribute( sAttrName(name) ,sNodePtr(new sTextNode(value)) );
}


inline sAttribute  attr (literal name ,literal value)
{
  return sAttribute( sAttrName(name) ,sNodePtr(new sTextNode(value)) );
}


inline sAttribute  attr (literal name ,bool value)
{
  return sAttribute( sAttrName(name) ,sNodePtr(new sBooleanNode(value)) );
}


inline sAttribute  attr (literal name ,int value)
{
  return sAttribute( sAttrName(name) ,sNodePtr(new sIntegerNode(value)) );
}


inline sAttribute  attr (literal name ,long int value)
{
  return sAttribute( sAttrName(name) ,sNodePtr(new sIntegerNode(value)) );
}


inline sAttribute  attr (literal name ,sPhysValue value)
{
  return sAttribute(sAttrName(name) ,sNodePtr(new sPhysValueNode(value)) );
}


inline sAttribute  attr (literal name ,double value)
{
  return sAttribute(sAttrName(name)
                   ,sNodePtr(new sPhysValueNode(sPhysValue(value))) );
}


inline sAttribute  attr (literal name ,long double value)
{
  return sAttribute(sAttrName(name)
                   ,sNodePtr(new sPhysValueNode(sPhysValue(value))) );
}


inline sAttribute  attr (literal name ,sPhysRange value)
{
  return sAttribute(sAttrName(name)
                   ,sNodePtr(new sPhysRangeNode(value)) );
}

//-------------------------------------------------------------- numeric ---

inline sAttribute  attr (integer name ,sNodePtr value)
{
  return sAttribute( sAttrName(name) ,value );
}


inline sAttribute  attr (integer name ,sString value)
{
  return sAttribute( sAttrName(name) ,sNodePtr(new sTextNode(value)) );
}


inline sAttribute  attr (integer name ,literal value)
{
  return sAttribute(sAttrName(name) ,sNodePtr(new sTextNode(value)) );
}


inline sAttribute  attr (integer name ,bool value)
{
  return sAttribute(sAttrName(name) ,sNodePtr(new sBooleanNode(value)) );
}


inline sAttribute  attr (integer name ,int value)
{
  return sAttribute(sAttrName(name) ,sNodePtr(new sIntegerNode(value)) );
}


inline sAttribute  attr (integer name ,long int value)
{
  return sAttribute(sAttrName(name) ,sNodePtr(new sIntegerNode(value)) );
}


inline sAttribute  attr (integer name ,sPhysValue value)
{
  return sAttribute(sAttrName(name) ,sNodePtr(new sPhysValueNode(value)) );
}


inline sAttribute  attr (integer name ,double value)
{
  return sAttribute(sAttrName(name)
                   ,sNodePtr(new sPhysValueNode(sPhysValue(value))) );
}


inline sAttribute  attr (integer name ,long double value)
{
  return sAttribute(sAttrName(name)
                   ,sNodePtr(new sPhysValueNode(sPhysValue(value))) );
}


inline sAttribute  attr (integer name ,sPhysRange value)
{
  return sAttribute(sAttrName(name)
                   ,sNodePtr(new sPhysRangeNode(value)) );
}

//------------------------------------------------------------------------------
} //namespace RA
#endif
