////////////////////////////////////////////////////////////////////////////////
/*! @file BasicDevices.cpp  Пакет базовых драйверов.
- Part of BasicDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "BasicDevices.h"
#include "PhysValue.h"
#include "TeleCom.h"
#include "CCDDriver.h"
#include <QMetaType>
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sBasicDevices)

//------------------------------------------------------------------------------

sVersion  sBasicDevices::Version ()
{
  return sVersion(RAP_VERS_MAJOR
                 ,RAP_VERS_MINOR
                 ,ZAKAVYCH(RAP_VERS_PATCH)
                 ,__DATE__
                 );
}


sVersion  sBasicDevices::CoreVersionUsed ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}


sVersion  sBasicDevices::CoreVersionMin ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}


QString  sBasicDevices::UserFriendlyName ()
{
  return trUtf8("Basic devices");
}


QString  sBasicDevices::About ()
{
  return trUtf8(
      "The package includes abstract base classes for some common device types, "
      "abstract base classes of consoles for common techniques, as well as "
      "some classes that can be used instantly.\n\n"
      "Copyright (C) 2010, Viktor E. Bursian, St.Petersburg, Russia.\n"
      "                    Viktor.Bursian@mail.ioffe.ru"
                );
}


sBasicDevices::sBasicDevices ()
{
  qRegisterMetaType<RA::sTelex>("sTelex");

  qRegisterMetaType<RA::sCCDDriver::sUShortArray>("sCCDDriver::sUShortArray");
  qRegisterMetaType<RA::sCCDDriver::sDoubleArray>("sCCDDriver::sDoubleArray");
  qRegisterMetaType<RA::sCCDDriver::sParameters>("sCCDDriver::sParameters");

  sUnit::Register("s");
  sUnit::Register("m");
  sUnit::Register("min"
                 ,sMetricPrefix::NoPrefixAllowed
                 //Attention! When you change here the prefix allowance, the
                 //program ignores (and resets!) the settings made by user in
                 //the RANet Knowledgebase.
                 ,sPhysValue(60,"s")
                 //Same about the definition of a derivative unit
                 );
  sUnit::Register("hour"
                 ,sMetricPrefix::NoPrefixAllowed
                 ,sPhysValue(3600,"s")
                 );
}

//------------------------------------------------------------------------------
} //namespace RA
