////////////////////////////////////////////////////////////////////////////////
/*! @file CCDDriver.h   Абстрактный базовый класс драйвера ПЗС-линейки.
- Part of BasicDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2017, Dmitrii Nelson, St.Petersburg, Russia.
                          d.nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef CCDDriver_H
#define CCDDriver_H
#include "BasicDevices.h"
#include "Driver.h"
namespace RA {


//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sCCDDriver)

//----------------------------------------------------------- sCCDDriver ---
/*! Абстрактный базовый класс драйвера ПЗС-линейки.

*/

class BasicDevices_EXPORT  sCCDDriver : public sDriver
{
  Q_OBJECT
  ABSTRACT_DRIVER(sCCDDriver)

  public: // types
    class sUShortArray
    {
      public:
        int         ArrSize;
        ushort *    ArrBody;
        virtual    ~sUShortArray(){
                      if (ArrBody){
                        delete [] ArrBody;
                        ArrBody = NULL;
                        ArrSize = 0;
                      }
                    }
                    sUShortArray():ArrSize(0),ArrBody(NULL){}
                    int Size(){
                      return ArrSize;
                    }
                    void SetSize(int size){
                      if (ArrSize > 0){
                        delete [] ArrBody;
                        ArrBody = NULL;
                        ArrSize = 0;
                      }
                      if (size > 0){
                        ArrSize = size;
                        ArrBody = new ushort [ArrSize];
                        memset (ArrBody, 0, ArrSize*sizeof(ushort));
                      }
                    }
                    sUShortArray(const sUShortArray & ad){
                      this->ArrSize = ad.ArrSize;
                      this->ArrBody = new ushort [ad.ArrSize];
                      memcpy(this->ArrBody,ad.ArrBody,ad.ArrSize*sizeof(ushort));
                    }
                    sUShortArray & operator = (const sUShortArray & ad){
                      if (this != & ad) {
                        delete [] ArrBody;
                        this->ArrSize = ad.ArrSize;
                        this->ArrBody = new ushort [ad.ArrSize];
                        memcpy(this->ArrBody,ad.ArrBody,ad.ArrSize*sizeof(ushort));
                      }
                      return *this;
                    }
                    ushort operator [] (int I) const {
                      if (I < ArrSize){
                        return ArrBody[I];
                      }else{
                        return 0;
                      }
                    }
                    void Reset(){
                      if (ArrBody){
                        memset (ArrBody, 0, ArrSize
                                *sizeof(ushort));
                      }
                    }
    };

    class sDoubleArray
    {
      public:
        int         ArrSize;
        double *    ArrBody;
        virtual    ~sDoubleArray(){
                      if (ArrBody){
                        delete [] ArrBody;
                        ArrBody = NULL;
                        ArrSize = 0;
                      }
                    }
                    sDoubleArray():ArrSize(0),ArrBody(NULL){}
                    int Size(){
                      return ArrSize;
                    }
                    void SetSize(int size){
                      if (ArrSize > 0){
                        delete [] ArrBody;
                        ArrBody = NULL;
                        ArrSize = 0;
                      }
                      if (size > 0){
                        ArrSize = size;
                        ArrBody = new double [ArrSize];
                        memset (ArrBody, 0, ArrSize*sizeof(double));
                      }
                    }
                    sDoubleArray(const sDoubleArray & ad){
                      this->ArrSize = ad.ArrSize;
                      this->ArrBody = new double [ad.ArrSize];
                      memcpy(this->ArrBody,ad.ArrBody,ad.ArrSize*sizeof(double));
                    }
                    sDoubleArray & operator = (const sDoubleArray & ad){
                      if (this != & ad) {
                        delete [] ArrBody;
                        this->ArrSize = ad.ArrSize;
                        this->ArrBody = new double [ad.ArrSize];
                        memcpy(this->ArrBody,ad.ArrBody,ad.ArrSize*sizeof(double));
                      }
                      return *this;
                    }
                    double operator [] (int I) const {
                      if (I < ArrSize){
                        return ArrBody[I];
                      }else{
                        return 0;
                      }
                    }
                    void Reset(){
                      if (ArrBody){
                        memset (ArrBody, 0, ArrSize
                                *sizeof(double));
                      }
                    }
    };
    class sParameters
    {
      public:
        sPhysValue   IntegrationTime;
        uchar        Gain;
        uchar        TriggerEdge;
        uchar        TriggerMode;
        virtual      ~sParameters() {}
                     sParameters(): IntegrationTime(sPhysValue(0.1, "s"))
                                   ,Gain(0)
                                   ,TriggerEdge(0)
                                   ,TriggerMode(0){}

    };

  public: //commands & directives
    static csCommand          cStart;
    static csCommand          cStop;
    static const
      sDirective<sCCDDriver::sParameters>         cSetParameters;

  public:
    virtual                  ~sCCDDriver ();

  signals: //outgoing
    void                      NewData (sCCDDriver::sUShortArray);
    void                      NewWavelengths (sCCDDriver::sDoubleArray);
    void                      IsBusy();
    void                      ParamsSet();

  public: //evolution of the inherited state machine
    psState                   State_W_Connect;
    psState                   State_W_GetWavelength;
    psState                   State_O_Stopped;
    psState                   State_O_Measuring;
    psState                   State_O_GetMeasured;

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  protected slots: //reimplementing inherited

  protected slots:
    virtual void              ConnectUSBDevice () = 0;
    virtual void              GetCCD_Data () = 0;
    virtual void              CalculateWavelengths () = 0; //should emit NewWavelengths (sCCDDriver::sDoubleArray)
    virtual void              StartCCD () = 0;
    virtual void              StopCCD () = 0;
    virtual void              SetCCD_Parameters (sCCDDriver::sParameters Params) = 0; //should emit ParamsSet()
                                                                                      // or IsBusy()
  signals: //for internal needs
    void                      Connected ();

  protected: //fields
    sCCDDriver::sParameters   Parameters;
    sCCDDriver::sUShortArray  DataArray;
    sCCDDriver::sDoubleArray  Wavelengths;
    int                       NumOfPixels;

  private: //fields
};

//------------------------------------------------------------------------------
} //namespace RA

Q_DECLARE_METATYPE(RA::sCCDDriver::sUShortArray)
Q_DECLARE_METATYPE(RA::sCCDDriver::sDoubleArray)
Q_DECLARE_METATYPE(RA::sCCDDriver::sParameters)

#endif
