////////////////////////////////////////////////////////////////////////////////
/*! @file CCDScaner.h  Базовый класс спектрометра с ПЗС-линейкой.
- Part of BasicDevices - a Research Assistant Plugin package.
- Uses  RAGUI - the Research Assistant Graphical User Interface.
- Uses  QtGui v.5.0  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2017, Dmitry K. Nelson, St.Petersburg, Russia.
                          D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef CCDScaner_H
#define CCDScaner_H
#include "BasicDevices.h"
#include "Scanner.h"
#include "DataTableFolder.h"
#include "DataTableRecord.h"
#include <QComboBox>
#include <QCheckBox>
#include <QSpinBox>
#include "CCDDriver.h"

namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sCCDScaner)

//------------------------------------------------------------- sCCDScaner ---
/*! Базовый класс спектрометра с ПЗС-линейкой.
*/

class BasicDevices_EXPORT  sCCDScaner : public sScanner
{
  Q_OBJECT
  CONSOLE(sCCDScaner)
  public:
    virtual                   ~sCCDScaner ();
    virtual void              RestoreState ();
    virtual void              StoreState ();

  protected: //evolution of the inherited state machine
    psState                   State_W_CCD_Connecting;
    psState                   State_O_Sc_Prepare;

  signals: //for internal needs
    void                      CCDNoAnswer ();
    void                      CCDDeviceError(QString = QString());
    void                      CCDWavelengths(sCCDDriver::sDoubleArray);
    void                      DevicesBusy();
    void                      CCDParamsSet();

  protected: //reimplementing inherited
    virtual void              AppearanceMakeUp ();
    virtual void              Retranslate ()
                                {
                                  sScanner::Retranslate();
                                  Translate();
                                }

  protected slots: //reimplementing inherited
    virtual void              EarlySetAttributes ();
    virtual void              LateSetAttributes ();

  protected slots: //implementing pure virtual
    virtual void              CreateNewDependence ();
    virtual void              InitializeScanning ();
    virtual void              StartScanning ();
    virtual void              StopScanning ();

  protected: // types
    enum                      eScanMode
                                {cMonitoring  = 0
                                ,cAcquisition = 1
                                ,cEvolution   = 2
                                };
  protected slots:
    virtual void              ReportNoAnswerError ();
    virtual void              AcceptWavelengths(sCCDDriver::sDoubleArray _WLs);
    virtual void              ReportCCDDeviceError (QString);
    virtual void              LinkToCCDDevice ();
    virtual void              TakeControlAndSetParameters();
    virtual void              ReleaseControl();
    virtual void              AddNewData (sCCDDriver::sUShortArray _aSample);

  protected:
    sPhysRange                CurrentScanRange();
    void                      CalculateWavelengths();

  protected: //fields
    sCCDDriver::sLink         CCDDevice;

    QLabel *                  LblRangeTitle;
    QLabel *                  LblWLRange;
    QLabel *                  LblMode;
    QComboBox *               FldMode;
    QLabel *                  LblIntegrationTime;
    sPhysValueField *         FldIntegrationTime;
    QLabel *                  LblTriggerEdge;
    QComboBox *               FldTriggerEdge;
    QLabel *                  LblTriggerMode;
    QComboBox *               FldTriggerMode;
    QLabel *                  LblNumOfSamples;
    QSpinBox *                FldNumOfSamples;
    QLabel *                  LblNumOfAcquisitions;
    QSpinBox *                FldNumOfAcquisitions;
    QCheckBox *               ChkBackConstant;
    QSpinBox *                FldBackConstant;
    QCheckBox *               ChkBackground;
    psDropNodeWidget          FldBackground;
    QCheckBox *               ChkCorrection;
    psDropNodeWidget          FldCorrection;
    sPtr<sDataTableRecord>    SpectrumDependence;
    sPtr<sDataTableFolder>    SubSpectrum;
    sNodePtr                  PreviousSubSpectrum;
    eScanMode                 ScanMode;
    int                       NumOfSamples;
    int                       CurrentSamplingNo;
    int                       NumOfAcquis;
    int                       CurrentAcquisNo;

    int                       NumOfPixels;
    sCCDDriver::sDoubleArray  Wavelengths;
    bool                      SubtractConstant;
    int                       BackConstant; //!< @todo{Nelson} Why integer?
    ulong *                   Data;
    double *                  BackgroundData;
    double *                  CorrectionData;

  private:
    void                      Translate ();
    sTime                     StartTime;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif //CCDScaner_H
