////////////////////////////////////////////////////////////////////////////////
/*! @file ClassicMultiScanner.cpp  Классический сканер с накоплением.
- Part of BasicDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "ClassicMultiScanner.h"
#include "attr.h"
#include "StringsAndQStrings.h"
#include "Trace.h"
#include "Log.h"
#include <QMessageBox>
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sClassicMultiScanner)

//---------------------------------------------------- sPeriodicGageRecorder ---

void  sClassicMultiScanner::Preconfigure (sNodePtr  DeviceCfg)
{
  sPtr<sFolderNode>              DeviceSettings;
  sPtr<sFolderNode>              DeviceLinks;
  DeviceCfg |= "Links";
  DeviceLinks = DeviceCfg >> "Links";
  if( DeviceLinks.IsCorrect() ){
    DeviceLinks |= "Gauge";
    DeviceLinks >> "Gauge" |= attr("Class","sGauge");
  };
  DeviceCfg |= "Settings";
  DeviceSettings = DeviceCfg >> "Settings";
  if( DeviceSettings.IsCorrect() ){
    DeviceSettings |= attr("ValidScanRange",sPhysRange(0.0,real_inf,"s"));
    DeviceSettings |= attr("ValidScanSteps",sPhysRange(0.01,1000,"s"));
//    DeviceSettings |= attr("ValidSpeeds",sPhysRange(1,1,_Unitsless_));
    DeviceSettings |= attr("ScanRange",sPhysRange(0.0,real_inf,"s",0.005));
    DeviceSettings |= attr("ScanStep",sPhysValue(0.3,0.005,"s"));
//    DeviceSettings |= attr("Speed",sPhysValue(1,_Unitsless_));
  };
  sScanner::Preconfigure(DeviceCfg);
}


sClassicMultiScanner::~sClassicMultiScanner()
{
  Gauge.SetToNULL();
}


sClassicMultiScanner::sClassicMultiScanner (sString   DeviceId
                                           ,sNodePtr  DeviceCfg
                                           ,sNodePtr  WidgetCfg
                                           ,QWidget *  parent  )
    :sScanner(DeviceId,DeviceCfg,WidgetCfg,parent)
    ,Gauge(this)
{
  //evolution of the inherited state machine
  {
    //evolution of the State_Warming
    {
      State_W_Gauge = new sState(State_Warming());
      State_W_Gauge
          ->SetAsInitial()
          ->AddTransition(State_Warming()->Fin());
      //State_W_Gauge structure and behaviour
      {
        State_W_Gauge_Warming = new sState(State_W_Gauge);
        State_W_Gauge_Warming
            ->SetAsInitial()
            ->OnEnterCall( SLOT(LinkToGaugeAndMakeItWarming()) )
            ->AddTransition( SIGNAL(GaugeWarmed()) ,State_W_Gauge->Fin() )
            ->AddTransition( SIGNAL(GaugeError())  ,State_Cooling()      );
      };
    };
    connect(this ,SIGNAL(GaugeError(QString))
           ,this ,SLOT(ReportGaugeError(QString)) );
  };
}


void  sClassicMultiScanner::AppearanceMakeUp ()
{
  sScanner::AppearanceMakeUp();
}


void  sClassicMultiScanner::RestoreState ()
{
  sScanner::RestoreState();
  if( ! WidgetConfig.IsNULL() ){
    get_from<sPhysRangeNode>(WidgetConfig >> "ScanRange" ,ScanRange);
    get_from<sPhysValueNode>(WidgetConfig >> "ScanStep" ,ScanStep);
//    get_from<sPhysValueNode>(WidgetConfig >> "Speed" ,Speed);
  };
  ScanRangeField->SetValidRange(ValidScanRange);
  ScanRangeField->SetValue(ScanRange);
  ScanStepField->SetValidRange(ValidScanSteps);
  ScanStepField->SetValue(ScanStep);
//  SpeedField->SetValidRange(ValidSpeeds);
//  SpeedField->SetValue(Speed);
}


void  sClassicMultiScanner::StoreState ()
{
  sScanner::StoreState();
  if( ! WidgetConfig.IsNULL() ){
    WidgetConfig << attr("ScanRange",ScanRangeField->Value())
                 << attr("ScanStep",ScanStepField->Value());
//    WidgetConfig << attr("Speed",SpeedField->Value());
  };
  if( DeviceSettings.IsCorrect() ){
    //здесь это работает
    DeviceSettings << attr("ScanRange",ScanRangeField->Value())
                   << attr("ScanStep",ScanStepField->Value());
  };
}


void sClassicMultiScanner::ReportGaugeError (QString  error_text)
{
  QMessageBox::critical(this ,tr("Device error")
                       ,tr("Device '%1' says:\n\n%2"
                           "\n\nTry to turn on again.")
                       .arg(ToQString(Gauge->DeviceIdentifier()))
                       .arg(error_text)     );
}


void sClassicMultiScanner::LinkToGaugeAndMakeItWarming ()
{
  sPtr<sTextNode>             GaugeId(DeviceLinks >> "Gauge" >> "DeviceId");
  if( GaugeId.IsCorrect() ){
    Gauge.SetTo(sString(*GaugeId));
  };
  if( ! Gauge.IsCorrect() ){
    SetInvalid();
  }else{
    Connect(Gauge ,SIGNAL(Warmed())       ,SIGNAL(GaugeWarmed()) );
    Connect(Gauge ,SIGNAL(ErrorMessage(QString)) ,SIGNAL(GaugeError(QString)) );
  };
}


void sClassicMultiScanner::OnEnterToCoolingState ()
{
  Disconnect(Gauge ,0 ,0);
  Gauge.ReleaseExclusiveControl();
  Gauge.SetToNULL();
}


void  sClassicMultiScanner::CreateNewDependence ()
{


}



void  sClassicMultiScanner::StartScanning ()
{
  CurrentPointNo=0;

  //! @todo{Instruments}    Gauge->SetPeriod(ScanStep/Speed); once again???
//  if( Gauge.IsCorrect() ){
//    Gauge->SetPeriod(ScanStep/Speed);
//  };

//  Connect(Gauge ,SIGNAL(NewData(sPhysValue)) ,SLOT(AddNewData(sPhysValue)) );
//  if( SendCommand(Gauge,sGauge::cStart,cTakeControl) ){
//  }else{
//    QMessageBox::critical(this,tr("Denied")
//                         ,tr("Device '%1' is busy.\nTry again later.")
//                         .arg(ToQString(Gauge->DeviceIdentifier()))     );
//  };
}


void  sClassicMultiScanner::StopScanning ()
{
//  SendCommand(Gauge,sGauge::cStop,cReleaseControl);
//  Disconnect(Gauge,SIGNAL(NewData(sPhysValue)) ,SLOT(AddNewData(sPhysValue)));
}


//void  sClassicMultiScanner::Connect ()
//{
//  if( DriverState() == Offline ){
//    if( Gage.IsCorrect() ){
//      disconnect(Gage.operator ->(),SIGNAL(NowStandby()),0,0);
//      connect(Gage.operator ->(),SIGNAL(NowStandby())
//             ,this              ,SLOT(EnterStandby()));
//      disconnect(Gage.operator ->(),SIGNAL(NowOffline()),0,0);
//      connect(Gage.operator ->(),SIGNAL(NowOffline())
//             ,this              ,SLOT(ReportError()));
//      SetDriverState(Warming);
//      Gage->Connect();
//    }else{
//      QMessageBox::critical(this
//          ,tr("%1 failure.").arg(ToQString(DeviceIdentifier()))
//          ,tr("The gage '%1' has not been found. Check configuration settings.")
//              .arg(ToQString(Gage->DeviceIdentifier()))
//          );
//    };
//  };
//}


//void  sClassicMultiScanner::EnterStandby ()
//{
//  disconnect(Gage.operator ->(),SIGNAL(NowStandby())
//            ,this              ,SLOT(EnterStandby()));
//  SetDriverState(Standby);
//}


//void  sClassicMultiScanner::ReportError ()
//{
//  QMessageBox::critical(this
//      ,tr("%1 failure.").arg(ToQString(DeviceIdentifier()))
//      ,tr("The gage '%1' does not respond.  Check connections.")
//          .arg(ToQString(Gage->DeviceIdentifier()))
//      );
//  SetDriverState(Offline);
//}


//void  sClassicMultiScanner::StartScan ()
//{
//  if( DriverState() != Standby )
//    return;
//  ScanRange = ScanRangeField->Value();
//  ScanStep = ScanStepField->Value();

//  if( Multiplier.X.IsNaN() ){
//    Multiplier.X = sPhysValue(ScanStep.Units());
//    sPhysRange                  RX(0*Multiplier.X,100*Multiplier.X);
//    SetChartScales(sChartScales(sChartScale(ChartScales().X.Name,RX),ChartScales().Y));
//  };
//  sPhysValue                  NOP(_Unitsless_);
//  NOP = ScanRange.Size() / ScanStep;
//  NumOfPoints=Round(NOP.ArbValue())+1;
//  sPhysValue                  ArbFrom(_Unitsless_);
//  ArbFrom = ScanRange.From() / Multiplier.X;
//  sPhysValue                  ArbStep(_Unitsless_);
//  ArbStep = ScanStep / Multiplier.X;
//  TRACE(Multiplier.X.Text());
//  TRACE(NOP.Text());
//  Dependence = new sArrayTypeDependence(NumOfPoints,ArbFrom.ArbValue()
//                                        ,ArbStep.ArbValue() );
//  ArrayTypeDependence = Dependence;
//  Dependence->SetMultiplier(sPhysPair(Multiplier.X,Multiplier.Y));
//  CurrentPointNo=0;

////  InitiateScanning();
//  sAScanner::StartScan();

//  CurrentPassNo=1;
//  CurrentPointNo=0;
//  Gage->SetPeriod(ScanStep);
//  Dependence << attr("Step",ScanStep);
//  connect(Gage.operator ->(),SIGNAL(NewData(sPhysValue))
//         ,this              ,SLOT(AddNewData(sPhysValue)));
//  Gage->Start();
//}


//void  sClassicMultiScanner::FinishScan ()
//{
//  Gage->Stop();
//  disconnect(Gage.operator ->(),SIGNAL(NewData(sPhysValue))
//            ,this              ,SLOT(AddNewData(sPhysValue)));
////  CompleteScanning();
//  sAScanner::FinishScan();
//}


//void  sClassicMultiScanner::AddNewData (sPhysValue  Data)
//{
//  SetValue(CurrentPointNo,Data);
//  if( CurrentPointNo < NumOfPoints-1 ){
//    CurrentPointNo++;
//  }else{
//    FinishScan();
//  };
//}


//void  sClassicMultiScanner::SetValue (int  point_number  ,sPhysValue  value)
//{
//  if( Multiplier.Y.IsNaN() ){
//    Multiplier.Y = sPhysValue(value.Units());
//    sPhysRange                  RY(-value,value);
//    SetChartScales(sChartScales(ChartScales().X
//                               ,sChartScale(ChartScales().Y.Name,RY)
//                               ));
//  };
//  sPhysValue                  Value(_Unitsless_);
//  Value = value / Multiplier.Y;
//  SetValue(point_number,Value.ArbValue());
//}


//void  sClassicMultiScanner::SetValue (int  point_number  ,real  value)
//{
//  if( ! ArrayTypeDependence.IsCorrect() ){
//    return;
//  };
//  if( Multiplier.Y.IsNaN() ){
//    return;
//  };
//  if( ArrayTypeDependence->Multiplier().Y.IsNaN() ){
//    ArrayTypeDependence->SetMultiplier(sPhysPair(Multiplier.X,Multiplier.Y));
//  };
//  ArrayTypeDependence->SetValue(point_number,value);
//}

//------------------------------------------------------------------------------
} //namespace RA
