////////////////////////////////////////////////////////////////////////////////
/*! @file ClassicMultiScanner.h   Классический сканер с накоплением.
- Part of BasicDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef ClassicMultiScanner_H
#define ClassicMultiScanner_H
#include "BasicDevices.h"
#include "Scanner.h"
#include "ArrayTypeDependence.h"
#include "Gauge.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sClassicMultiScanner)

//----------------------------------------------------- sClassicMultiScanner ---
/*! Классический многопроходный сканер с накоплением.
*/
class BasicDevices_EXPORT  sClassicMultiScanner : public sScanner
{
  Q_OBJECT
  CONSOLE(sClassicMultiScanner)
  public:
    virtual                   ~sClassicMultiScanner ();
    virtual void              RestoreState ();
    virtual void              StoreState ();

  protected: //evolution of the inherited state machine
    psState                   State_W_Gauge;
    psState                   State_W_Gauge_Warming;

  signals: //for internal needs
    void                      GaugeWarmed ();
    void                      GaugeError(QString = QString());

  protected: //reimplementing inherited
    virtual void              AppearanceMakeUp ();

  protected slots: //reimplementing inherited
//    virtual void              OnEnterToOperatingState ();
//    virtual void              OnExitFromOperatingState ();
    virtual void              OnEnterToCoolingState ();

  protected slots:
    virtual void              CreateNewDependence ();
    virtual void              StartScanning ();
    virtual void              StopScanning ();
//    virtual void              AddNewData (sPhysValue  Data);
    virtual void              ReportGaugeError (QString);

  protected: //fields
    sGauge::sLink             Gauge;
    sPhysRange                ValidScanRange;
    sPhysRange                ValidScanSteps;
    sPhysRange                ScanRange;
    sPhysValue                ScanStep;
    sPhysValue                Speed;

//  public slots:
//    virtual void              Connect ();

//  protected slots:
//    virtual void              StartScan ();
//    virtual void              FinishScan ();
//    virtual void              EnterStandby ();
//    virtual void              ReportError ();
//    virtual void              AddNewData (sPhysValue  Data);

//    virtual void              SetValue (int  point_number  ,sPhysValue  value);
//    virtual void              SetValue (int  point_number  ,real  value);

  protected: //fields
//    int                       NumOfPoints;
//    sPtr<sArrayTypeDependence>
//                              ArrayTypeDependence;
    int                       CurrentPassNo;
    int                       CurrentPointNo;

  protected: //fields
    psPhysRangeField          ScanRangeField;
    psPhysValueField          ScanStepField;
    psPhysValueField          SpeedField;

  private slots:
    void                      LinkToGaugeAndMakeItWarming ();

  private: //fields
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
