////////////////////////////////////////////////////////////////////////////////
/*! @file EffectorConsole.h   Консоль управления эффектором.
- Part of BasicDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2013-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef EffectorConsole_H
#define EffectorConsole_H
#include "BasicDevices.h"
#include "Console.h"
#include "Effectors.h"
#include "PhysValueField.h"
#include <QLabel>
#include <QPushButton>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sEffectorConsole)

//--------------------------------------------------------- sEffectorConsole ---
/*! Консоль управления эффектором.

Сделана для тестирования, но может использоваться как полноценный прибор,
управляющий любым эффектором типа sEffector.

Можно использовать как пример, в частности, консоли с одним подчинённым
драйвером: подключение, проверка готовности, отключение, двусторонний обмен
данными сложного типа (sPhysValue).


*/
class  BasicDevices_EXPORT  sEffectorConsole : public sConsole
{
  Q_OBJECT
  CONSOLE(sEffectorConsole)
  public:
    virtual                   ~sEffectorConsole ();
  
  public: //reimplementing inherited
    virtual void              RestoreState ();
    virtual void              StoreState ();

  protected:
    virtual void              AppearanceMakeUp ();

  protected: //evolution of the inherited state machine
    psState                   State_W_Effector;
    psState                   State_W_Effector_Linking;
    psState                   State_O_Watching;
    psState                   State_O_Controling;
    psState                   State_O_C_Setting;
    psState                   State_O_C_Breaking;

  signals: //for internal needs
    void                      EffectorLinked ();
    void                      EffectorError (QString = QString());
    void                      SetValue ();
    void                      Break ();
    void                      EffectorControlTaken ();
    void                      EffectorSaysDone ();

  protected slots: //reimplementing inherited
    virtual void              OnEnterToOperatingState ();
    virtual void              OnExitFromOperatingState ();
    virtual void              OnEnterToCoolingState ();

  protected slots:
    virtual void              ShowData (sPhysValue);
    virtual void              ShowNaN ();
    virtual void              ReportEffectorError (QString);
    virtual void              ReleaseEffectorControl ();
    virtual void              ApplyRange (sPhysRange);

  protected: //fields
    bool                      DoSetWithButton;
    sEffector::sLink          Effector;
    QLabel *                  Screen;
    psPhysValueField          SetValueField;
    QPushButton *             BtnSet;
    QPushButton *             BtnBreak;

  private slots:
    void                      LinkToEffector ();
    void                      OnSetValue ();
    void                      OnBreak();
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
