////////////////////////////////////////////////////////////////////////////////
/*! @file Effectors.cpp     Few effector prototypes.
- Part of BasicDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2013-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "Effectors.h"
#include "attr.h"
#include <QDialogButtonBox>
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sEffector)
DEFINE_CLASS_TAG(sStepper)

/*! Директива установки физвеличины на выходе эффектора
    (attached data: значение физвеличины).
*/
const sDirective<sPhysValue>  sEffector::cSetTo("cSetTo");


/*! Command breaking any movement (either initiated by cSetTo or by cMoveBy).
*/
csCommand                     sEffector::cBreak("cBreak");

/*! Директива, устанавливающая время на шаг и инициирующая сканирование
    (attached data: время на один шаг).
*/
const sDirective<sPhysValue>  sStepper::cSetTimePerStep
                                             ("cSetTimePerStep");

/*! Директива, предписывающая сделать k шагов с установленным предыдущей директивой временем
    (attached data: значение k).
*/
const sDirective<int>         sStepper::cMoveBy("cMoveBy");

//------------------------------------------------------ sEffectorInitDialog ---

sEffectorInitDialog::sEffectorInitDialog (sPhysValue _value)
    :QDialog()
{
  setLayout(new QVBoxLayout);

  QLabel * lblWL = new QLabel(this);
  lblWL->setText(tr("Enter the initial position"));
  layout()->addWidget(lblWL);

  InitValueField = new sPhysValueField(this,tr("Position:"),_value);

  layout()->addWidget(InitValueField);

  QDialogButtonBox * buttonBox = new QDialogButtonBox(this);
  buttonBox->setOrientation(Qt::Horizontal);
  buttonBox->setStandardButtons(QDialogButtonBox::Ok);

  layout()->addWidget(buttonBox);
  connect(buttonBox,SIGNAL(accepted()),this,SLOT(accept()));
//  Retranslate();
}

//---------------------------------------------------------------- sEffector ---

void  sEffector::Preconfigure (sNodePtr  DeviceCfg)
{
  sPtr<sFolderNode>              DeviceSettings;
  DeviceCfg |= "Settings";
  DeviceSettings = DeviceCfg >> "Settings";
  if( DeviceSettings.IsCorrect() ){
    DeviceSettings |= attr("Range",sPhysRange());
  }
  sGauge::Preconfigure(DeviceCfg);
}


sEffector::sEffector (sString   DevicetId
                     ,sNodePtr  DeviceCfg)
    :sGauge(DevicetId,DeviceCfg)
{
}


void  sEffector::PostConstructor ()
{
  sGauge::PostConstructor();
  try{ //get config parameters
    get_from<sPhysRangeNode>(DeviceSettings >> "Range" ,TheRange);
  }catch(...){
    SetInvalid(tr("Something wrong in configuration."));
  }
}


//----------------------------------------------------------------- sStepper ---

void  sStepper::Preconfigure (sNodePtr  DeviceCfg)
{
  sEffector::Preconfigure(DeviceCfg);
}


sStepper::sStepper (sString   DevicetId
                   ,sNodePtr  DeviceCfg)
    :sEffector(DevicetId,DeviceCfg)
{
}

//------------------------------------------------------------------------------
} //namespace RA
