////////////////////////////////////////////////////////////////////////////////
/*! @file Effectors.h       Few effector prototypes.
- Part of BasicDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2013-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef Effectors_H
#define Effectors_H
#include "BasicDevices.h"
#include "Gauge.h"
#include "Ranges.h"
#include "PhysValueField.h"
#include <QDialog>
namespace RA {
//------------------------------------------------------- EffectorInitDialog ---
/*!

*/
class  BasicDevices_EXPORT  sEffectorInitDialog : public QDialog
{
  Q_OBJECT
  public:
                              sEffectorInitDialog (sPhysValue _value);
//  protected:
    psPhysValueField          InitValueField;
};

//------------------------------------------------------------------------------

ANNOUNCE_ABSTRACT_CLASS(sEffector)
ANNOUNCE_ABSTRACT_CLASS(sStepper)

//---------------------------------------------------------------- sEffector ---
/*! An effector prototype.


*/
class BasicDevices_EXPORT  sEffector : public sGauge
{
  Q_OBJECT
  ABSTRACT_DRIVER(sEffector)

  public: //commands & directives
    static const
      sDirective<sPhysValue>  cSetTo;
  static csCommand            cBreak;

  public:
    virtual sPhysRange        Range () const
                                { return TheRange; }

  signals: //outgoing
    void                      Done (sPhysValue = sPhysValue());
    void                      RangeChanged (sPhysRange);

  protected:
    virtual void              PostConstructor ();

  protected: //fields
    sPhysRange                TheRange;
};

//----------------------------------------------------------------- sStepper ---
/*! A stepper (stepping motor) prototype.


Особенности:
- медленный, но исполнительный (рано или поздно дойдёт точно туда,
  куда послали);
- дискретность (положение определяется целым K, меняющимся в целочисленном
  диапазоне от MinK до MaxK);
- необязательно линейная, но однозначная зависимость sPhysValue Position(int K);
- люфт
- разные скорости, разгонные и тормозные рампы (хотелось бы, чтобы это было
  внутренней кухней).

*/
class BasicDevices_EXPORT  sStepper : public sEffector
{
  Q_OBJECT
  ABSTRACT_DRIVER(sStepper)

  public: //commands & directives
    static const
      sDirective<sPhysValue>  cSetTimePerStep;
    static const
      sDirective<int>         cMoveBy;

  signals:
    void                      TimePerStepSet(sPhysValue);

  public:
    virtual sPhysValue        Position (int  steps) const =0;
    virtual int               Steps  (sPhysValue  position) const =0;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
