////////////////////////////////////////////////////////////////////////////////
/*! @file FixedUnitsLCD.h Жидко-кристаллический дисплей с фиксированными единицами.
- Part of BasicDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2014, Dmitry K. Nelson, St.Petersburg, Russia.
                          D.Nelson@mail.ioffe.ru
- Copyright(C) 2013,      Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef FixedUnitsLCD_H
#define FixedUnitsLCD_H
#include "BasicDevices.h"
#include "GaugeDisplay.h"
#include <QLCDNumber>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sFixedUnitsLCD)

//----------------------------------------------------------- sFixedUnitsLCD ---
/*! Жидко-кристаллический дисплей с фиксированными единицами.

Показывает отсчёты любого датчика типа sGauge по мере их поступления
в единицах измерения, указанных в конфигурации (UnitsToShow).
Положение десятичной точки фиксировано (DigitsAfterTheDot).
Для удобства чтения цифры числа разбиваются пробелами на тройки.

@par The state machine
Такая же, как у sGaugeDisplay.

*/
class  BasicDevices_EXPORT  sFixedUnitsLCD : public sGaugeDisplay
{
  Q_OBJECT
  CONSOLE(sFixedUnitsLCD)

  protected: //reimplementing inherited
    virtual void              AppearanceMakeUp ();

  protected slots: //reimplementing inherited
    virtual void              ShowData (sPhysValue);
    virtual void              ShowNaN ();

  protected: //fields
    sString                   UnitsToShow;
    int                       DigitsAfterTheDot;

  private: //fields
    QLCDNumber *              ValueLCD;
    QLabel *                  UnitsDisplay;
    real                      Multiplier; //!< power10(DigitsAfterTheDot)
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
