////////////////////////////////////////////////////////////////////////////////
/*! @file GaugeDisplay.cpp Показометр.
- Part of BasicDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2012-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "GaugeDisplay.h"
#include "attr.h"
#include "StringsAndQStrings.h"
#include <QVBoxLayout>
#include <QMessageBox>
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sGaugeDisplay)

//----------------------------------------------------------- sGaugeDisplay ---

void  sGaugeDisplay::Preconfigure (sNodePtr  DeviceCfg)
{
  sPtr<sFolderNode>              DeviceLinks;
  DeviceCfg |= "Links";
  DeviceLinks = DeviceCfg >> "Links";
  if( DeviceLinks.IsCorrect() ){
    DeviceLinks |= "Gauge";
    DeviceLinks >> "Gauge" |= attr("Class","sGauge");
  };
//  sPtr<sFolderNode>              DeviceSettings;
//  DeviceCfg |= "Settings";
//  DeviceSettings = DeviceCfg >> "Settings";
//  if( DeviceSettings.IsCorrect() ){
//    DeviceSettings |= attr("Label"," v a l u e :");
//  };
  sConsole::Preconfigure(DeviceCfg);
}


sGaugeDisplay::~sGaugeDisplay ()
{
  Gauge.SetToNULL();
}


sGaugeDisplay::sGaugeDisplay (sString   DeviceId
                             ,sNodePtr  DeviceCfg
                             ,sNodePtr  WidgetCfg
                             ,QWidget *  parent    )
    :sConsole(DeviceId,DeviceCfg,WidgetCfg,parent)
    ,Gauge(this)
{
  //evolution of the inherited state machine
  {
    //evolution of the State_Warming
    {
      State_W_Gauge 
          = (new sState(State_Warming(),"State_W_Gauge"))
          ->SetAsInitial()
          ->AddTransition(State_Warming()->Fin());

      //State_W_Gauge structure and behaviour
      {
        State_W_Gauge_Linking 
            = (new sState(State_W_Gauge,"State_W_Gauge_Linking"))
            ->SetAsInitial()
            ->OnEnterCall( SLOT(LinkToGauge()) )
            ->AddTransition( SIGNAL(GaugeLinked()) ,State_W_Gauge->Fin() )
            ->AddTransition( SIGNAL(ErrorMessage())  ,State_Cooling() );
      }
    }

    connect(this ,SIGNAL(GaugeError(QString))
           ,this ,SLOT(ReportGaugeError(QString)) );
  }
}


void  sGaugeDisplay::AppearanceMakeUp ()
{
  sConsole::AppearanceMakeUp();
//  try{ //get config parameters
//    get_from<sIntegerNode>(DeviceSettings>>"IntParameter",IntParameter);
//  }catch(...){
//    SetInvalid(tr("Something wrong in configuration."));
//  }
  setLayout(new QVBoxLayout);
  Screen = new QLabel(this);
  layout()->addWidget(Screen);
  ShowNaN();
}


void sGaugeDisplay::ShowData (sPhysValue  V)
{
  Screen->setText( QString::fromUtf8("<font size=20 color=blue>")
                 + ToQString(V.Text(HTML))
                 );
}


void sGaugeDisplay::ShowNaN ()
{
  Screen->setText( QString::fromUtf8("<font size=20 color=red>")
                   + "--------");
}


void sGaugeDisplay::ReportGaugeError (QString  error_text)
{
  emit ErrorMessage( tr("Got error from '%1':\n\n%2")
                       .arg(ToQString(Gauge->DeviceIdentifier()))
                       .arg(error_text)     );
}


void sGaugeDisplay::LinkToGauge ()
{
  try{
    Gauge.SetTo(DeviceLinks,"Gauge");
    Connect(Gauge ,SIGNAL(Warmed())       ,SIGNAL(GaugeLinked()) );
    Connect(Gauge ,SIGNAL(ErrorMessage(QString))
           ,SIGNAL(GaugeError(QString)) );
  }catch( rxDeviceLaunch E ){
    emit ErrorMessage( tr("Error while linking to a submissive device.\n"
                          "%1\nCheck configuration.")
                         .arg(ToQString(E.MsgText()))   );
  }
}


void sGaugeDisplay::OnEnterToCoolingState ()
{
  Disconnect(Gauge,0,0);//all signals from Gauge connected to any of my slots
  Gauge.ReleaseExclusiveControl();
  Gauge.SetToNULL();
  sConsole::OnEnterToCoolingState();
}


void sGaugeDisplay::OnEnterToOperatingState()
{
  sConsole::OnEnterToOperatingState();
  Connect(Gauge ,SIGNAL(NewData(sPhysValue)) ,SLOT(ShowData(sPhysValue)) );
}


void sGaugeDisplay::OnExitFromOperatingState()
{
  Disconnect(Gauge ,SIGNAL(NewData(sPhysValue)) ,SLOT(ShowData(sPhysValue)) );
  ShowNaN();
  sConsole::OnExitFromOperatingState();
}

//------------------------------------------------------------------------------
} //namespace RA
