////////////////////////////////////////////////////////////////////////////////
/*! @file GaugeDisplay.h   Показометр.
- Part of BasicDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2012-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef GaugeDisplay_H
#define GaugeDisplay_H
#include "BasicDevices.h"
#include "Console.h"
#include "Gauge.h"
#include <QLabel>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sGaugeDisplay)

//----------------------------------------------------------- sGaugeDisplay ---
/*! Показометр.

Сделан для тестирования, но может использоваться как полноценный прибор,
показывающий отсчёты любого датчика типа sGauge.

Можно использовать как пример, в частности, консоли с одним подчинённым
драйвером (подключение, проверка готовности, пассивное получение сигналов,
отключение).

Кроме того, классы sGaugeDisplay, sGaugeTestboard и sGaugeLCD дают
пример того, как грамотно разделять функциональность и внешний вид, а также,
как расширить или полностью подменить внешний вид потомка.

@see sGaugeLCD, sGaugeTestboard.

@par The state machine
@dotfile "GaugeDisplay.state_graph"

*/
class  BasicDevices_EXPORT  sGaugeDisplay : public sConsole
{
  Q_OBJECT
  CONSOLE(sGaugeDisplay)
  
  public:
    virtual                   ~sGaugeDisplay ();

  protected: //evolution of the inherited state machine
    psState                   State_W_Gauge;
    psState                   State_W_Gauge_Linking;

  signals: //for internal needs
    void                      GaugeLinked ();
    void                      GaugeError (QString = QString());

  protected: //reimplementing inherited
    virtual void              AppearanceMakeUp ();

  protected slots: //reimplementing inherited
    virtual void              OnEnterToOperatingState ();
    virtual void              OnExitFromOperatingState ();
    virtual void              OnEnterToCoolingState ();

  protected slots:
    virtual void              ShowData (sPhysValue);
    virtual void              ShowNaN ();
    virtual void              ReportGaugeError (QString);
    virtual void              LinkToGauge ();

  protected: //fields
    sGauge::sLink             Gauge;
    QLabel *                  Screen;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
