////////////////////////////////////////////////////////////////////////////////
/*! @file GaugeLCD.cpp Жидко-кристаллический показометр.
- Part of BasicDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2013, Dmitry K. Nelson, St.Petersburg, Russia.
                          D.Nelson@mail.ioffe.ru
- Copyright(C) 2013,      Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "GaugeLCD.h"
#include "attr.h"
#include "StringsAndQStrings.h"
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QSpacerItem>
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sGaugeLCD)

//--------------------------------------------------------------- sGaugeLCD ---

void  sGaugeLCD::Preconfigure (sNodePtr  DeviceCfg)
{
  sGaugeDisplay::Preconfigure(DeviceCfg);
}


sGaugeLCD::sGaugeLCD (sString   DeviceId
                     ,sNodePtr  DeviceCfg
                     ,sNodePtr  WidgetCfg
                     ,QWidget *  parent    )
    :sGaugeDisplay(DeviceId,DeviceCfg,WidgetCfg,parent)
{
}


void  sGaugeLCD::AppearanceMakeUp ()
{
  setLayout(new QVBoxLayout);

  ValueLCD = new QLCDNumber(this);
  ValueLCD->setSegmentStyle(QLCDNumber::Flat);
  layout()->addWidget(ValueLCD);

  QHBoxLayout *               L( new QHBoxLayout );
  layout()->addItem(L);
  UnitsDisplay = new QLabel(this);
  L->addStretch();
  L->addWidget(UnitsDisplay);
  UnitsDisplay->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);

  ShowNaN();
}


void sGaugeLCD::ShowData (sPhysValue  V)
{
  V.Normalize();
  QString            T( ToQString( (V/sPhysValue(1.0,V.Units())).Text() ) );
  ValueLCD->setDigitCount(T.length());
  ValueLCD->display(T);
  UnitsDisplay->setText(ToQString( V.Units().Text(HTML) ));
}


void sGaugeLCD::ShowNaN ()
{
  ValueLCD->display(QString("-----"));
  UnitsDisplay->setText("");
}

//------------------------------------------------------------------------------
} //namespace RA


