////////////////////////////////////////////////////////////////////////////////
/*! @file GaugeLCD.h   Жидко-кристаллический показометр.
- Part of BasicDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2013, Dmitry K. Nelson, St.Petersburg, Russia.
                          D.Nelson@mail.ioffe.ru
- Copyright(C) 2013,      Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef GaugeLCD_H
#define GaugeLCD_H
#include "BasicDevices.h"
#include "GaugeDisplay.h"
#include <QLCDNumber>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sGaugeLCD)

//--------------------------------------------------------------- sGaugeLCD ---
/*! Жидко-кристаллический показометр.

Показывает отсчёты любого датчика типа sGauge по мере их поступления.

Кроме того, классы sGaugeDisplay и sGaugeLCD дают пример того, как грамотно
разделять функциональность и внешний вид, и как полностью подменить внешний вид
у потомка, не меняя функциональности.

@see sGaugeTestboard, который только расширяет внешний вид
sGaugeDisplay, а заодно и функциональность.

@par The state machine
Такая же, как у sGaugeDisplay.

*/
class  BasicDevices_EXPORT  sGaugeLCD : public sGaugeDisplay
{
  Q_OBJECT
  CONSOLE(sGaugeLCD)

  protected: //reimplementing inherited
    virtual void              AppearanceMakeUp ();

  protected slots: //reimplementing inherited
    virtual void              ShowData (sPhysValue);
    virtual void              ShowNaN ();

  private: //fields
    QLCDNumber *              ValueLCD;
    QLabel *                  UnitsDisplay;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
