////////////////////////////////////////////////////////////////////////////////
/*! @file PeriodicGaugeRecorder.h   Самописец.
- Part of BasicDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2013, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef PeriodicGaugeRecorder_H
#define PeriodicGaugeRecorder_H
#include "BasicDevices.h"
#include "Scanner.h"
#include "DataListRecord.h"
#include "Gauge.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sPeriodicGaugeRecorder)

//--------------------------------------------------- sPeriodicGaugeRecorder ---
/*! Самописец.
*/
class BasicDevices_EXPORT  sPeriodicGaugeRecorder : public sScanner
{
  Q_OBJECT
  CONSOLE(sPeriodicGaugeRecorder)
  public:
    virtual                   ~sPeriodicGaugeRecorder ();
    virtual void              RestoreState ();
    virtual void              StoreState ();

  protected: //evolution of the inherited state machine
    psState                   State_W_Parallel;
    psState                   State_W_Gauge;
    psState                   State_W_Gauge_Linking;
    psState                   State_O_Sc_SettingGauge;
    psState                   State_O_Sc_SG_Waiting;
    psState                   State_O_Sc_ResettingGauge;
    psState                   State_O_Sc_RG_Waiting;

  signals: //for internal needs
    void                      GaugeLinked ();
    void                      GaugeError(QString = QString());
    void                      GaugeBusy ();
    void                      GaugePeriodChanged (sPhysValue);

  protected: //reimplementing inherited
    virtual void              AppearanceMakeUp ();
    virtual void              Retranslate ()
                                {
                                  sScanner::Retranslate();
                                  Translate();
                                }

  protected slots: //reimplementing inherited
    virtual void              OnEnterToCoolingState ();

  protected slots: //implementing pure virtual
    virtual void              CreateNewDependence ();
    virtual void              StartScanning ();
    virtual void              StopScanning ();

  protected slots:
    virtual void              SetGaugeRegime ();
    virtual void              CorrectStep (sPhysValue  /*gauge_period*/);
    virtual void              ResetGaugeRegime ();
    virtual void              RestartGauge ();
    virtual void              AddNewData (sPhysValue  Data);
    virtual void              ReportGaugeError (QString);
    void                      ReleaseGaugeControl ();
    void                      EraseTrueStepToolTip ();

  protected: //fields
    sGauge::sLink             Gauge;
    sPhysRange                ValidScanRange;
    sPhysRange                ValidScanSteps;
    sPhysRange                ScanRange;
    sPhysValue                ScanStep;
    sPhysValue                Speed;
    sPtr<sDataListRecord> DataListRecord;
    int                       CurrentPointNo;
    bool                      GaugeWasStarted;
    bool                      GaugeWasInAutoRepeatMode;
    sPhysValue                OldPeriod;

  protected: //fields
    psPhysRangeField          ScanRangeField;
    psPhysValueField          ScanStepField;

  private slots:
    void                      LinkToGauge ();

  private:
    void                      Translate ();

  private: //fields
    sPhysValue                ArbFrom;
    sPhysValue                ArbStep;
    sPhysValue                NumOfPointsEvaluation;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
