////////////////////////////////////////////////////////////////////////////////
/*! @file ProtoAccumulator.h   Прообраз накопителя данных,
                               поступающих в виде массива.
- Part of BasicDevices - a Research Assistant Plugin package.
- Uses  RAGUI - the Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef ProtoAccumulator_H
#define ProtoAccumulator_H
#include "ArrayTypeDependence.h"
#include "AScanner.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sProtoAccumulator)

//-------------------------------------------------------------- sProtoAccumulator ---
/*! Абстрактный базовый класс консоли (пользовательского интерфейса)
    накопителя параллельных данных (массива данных).
*/
class BasicDevices_EXPORT  sProtoAccumulator : public sAScanner
{
  Q_OBJECT
  ABSTRACT_CONSOLE(sProtoAccumulator)
  public:
    virtual                   ~sProtoAccumulator ();
    virtual void              RestoreState ();
    virtual void              StoreState ();

protected slots:
    virtual void              StartScan ();
    virtual void              FinishScan ();
    virtual void              AddNewData (real* Value);

protected: //fields
    sPhysRange                ValidScanRange;
    sPhysRange                ValidScanSteps;
    sPhysRange                ScanRange;
    sPhysValue                ScanStep;
    sArrayTypeDependence::sPtr      ArrayTypeDependence;
    int                       NumOfPoints;

protected: //fields
    psPhysRangeField          ScanRangeField;
    psPhysValueField          ScanStepField;

private: //fields
    sPhysValue                ArbFrom;
    sPhysValue                ArbStep;
    int                       CurrentSamplingNo;
};

//------------------------------------------------------------------------------
}; //namespace RA
#endif // ProtoAccumulator_H
