////////////////////////////////////////////////////////////////////////////////
/*! @file Scanner.h   Довольно-таки универсальный прототип
                           сканирующего прибора.
- Part of BasicDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2010-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef Scanner_H
#define Scanner_H
#include "BasicDevices.h"
#include "Console.h"
#include "Chart.h"
#include "NetWidgets.h"
#include "PhysRangeField.h"
#include "Record.h"
#include <QTimer>
#include <QSplitter>
#include <QPushButton>
//#include <QDoubleSpinBox>
#include <QTextEdit>
#include <QScrollArea>
//#include <QFormLayout>
#include <QGridLayout>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sScanner)

//---------------------------------------------------------------- sScanner ---
/*! Абстрактный базовый класс консоли (пользовательского интерфейса)
    более или менее универсального сканирующего прибора.
  
*/
class  BasicDevices_EXPORT  sScanner : public sConsole
{
  Q_OBJECT
  ABSTRACT_CONSOLE(sScanner)
  public:
    virtual                   ~sScanner ();
  
  public: //reimplementing inherited
    virtual void              RestoreState ();
    virtual void              StoreState ();
    virtual bool              CanClose ();

  public:
    sChartScales              ChartScales () const
                                { return Chart->ChartScales(); }
    void                      SetChartScales (sChartScales);

  public: //evolution of the inherited state machine
    psState                   State_O_Idle;
    psState                   State_O_Scanning;
    psState                   State_O_Sc_Acquisition;
    psState                   State_O_Sc_Deciding;
    psState                   State_O_Sc_D_Waiting;

  signals: //for internal needs
    void                      Scan ();
    void                      Sweep ();
    void                      Stop ();
    void                      Break ();
    void                      ScanningHasCompleted ();
    void                      Accept ();
    void                      Disregard ();

  protected: //reimplementing inherited
    virtual void              AppearanceMakeUp ();
    virtual void              Retranslate ()
                                {
                                  sConsole::Retranslate();
                                  Translate();
                                }

  protected:
    virtual void              ButtonsMakeUp (QWidget *  parent
                                            ,QLayout *  button_layout);
    virtual void              UpdateHistory ();

  protected slots: //reimplementing inherited
    virtual void              AfterStartInitiation ();

  protected slots:
    virtual void              OnCoolDownWhileScanning ();
    virtual void              InitializeScanning ();
    virtual void              FinalizeScanning ();
    virtual void              CreateNewDependence () =0;
    virtual void              EarlySetAttributes ();
    virtual void              StartScanning () =0;
    virtual void              StopScanning () =0;
    virtual void              LateSetAttributes ();
    virtual void              OnConsoleStateChanged (sDeviceStateMachine::eState
                                                                 new_state);

  protected: //fields
    sPhysPair                 Multiplier;
    sPtr<sRecord>             Dependence;
    sPtr<sFolderNode>         Sample;
    sPtr<sFolderNode>         Topic;
    sString                   Conditions;

  protected: //fields
//    QSplitter *               MainSplitter;
    int                       Row;

  private slots:
    virtual void              OnAccept ();
    virtual void              OnDisregard ();
    void                      ResetColorSequence ();
    virtual void              OnSampleChange ();
    virtual void              OnTopicChange ();

  private:
    void                      Translate ();
    void                      PutSpectrumIntoSecondaryDestinations();
    void                      RemoveSpectrumFromSecondaryDestinations();

  protected: //fields
//  private: //fields
    int                       HistorySize;
    int                       LineThickness;
    sPtr<sColorSetNode>       LineColorSet;
    sPtr<sFolderNode>         Content;
    sPtr<sFolderNode>         SecondaryDestinations;
    psNet2TreeEvolution       Evolution;
    psChart                   Chart;
    QTimer *                  RefreshTimer;
    QLayout *                 Layout;
    QSplitter *               Splitter;

    QWidget *                 SideFrame;
    QLayout *                 SideLayout;
    QWidget *                 ButtonArea;
    QGridLayout *             ButtonLayout;
    QString                   TextState;
    QLabel *                  LblState;
    QPushButton *             BtnTurnOn;
    QPushButton *             BtnScan;
    QPushButton *             BtnBreak;
    QPushButton *             BtnAccept;
    QPushButton *             BtnDisregard;
    QScrollArea *             ParamScroll;

    QAction *                 ActionLegentBottom;//SplitHorizontalAction
    QAction *                 ActionLegentRight;//SplitVerticalAction

  protected: //fields
    QWidget *                 ParamArea;
    QGridLayout *             ParamLayout;

//  private: //fields
    QLabel *                  LblSample;
    psDropNodeWidget          FldSample;
    QLabel *                  LblTopic;
    psDropNodeWidget          FldTopic;
    QLabel *                  LblConditions;
    QTextEdit *               FldConditions;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
