////////////////////////////////////////////////////////////////////////////////
/*! @file StepByStepScanner.h   Прибор пошагового сканирования.
- Part of BasicDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5 - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2014-2015, Dmitry K. Nelson, St.Petersburg, Russia.
                          D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef StepByStepScanner_H
#define StepByStepScanner_H
#include "BasicDevices.h"
#include "Scanner.h"
#include "DataArrayRecord.h"
#include "Gauge.h"
#include "Effectors.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sStepByStepScanner)

//--------------------------------------------------- sStepByStepScanner ---
/*! Самописец.
*/
class BasicDevices_EXPORT  sStepByStepScanner : public sScanner
{
  Q_OBJECT
  CONSOLE(sStepByStepScanner)
  public:
    virtual                   ~sStepByStepScanner ();
    virtual void              RestoreState ();
    virtual void              StoreState ();

  protected: //evolution of the inherited state machine
    psState                   State_W_Parallel;
    psState                   State_W_Gauge;
    psState                   State_W_Gauge_Linking;
    psState                   State_W_Effector;
    psState                   State_W_Effector_Linking;
    psState                   State_O_Sc_PrepareDevices;
    psState                   State_O_Sc_SettingGauge;
    psState                   State_O_Sc_SG_Waiting;
    psState                   State_O_Sc_ResettingGauge;
    psState                   State_O_Sc_RG_Waiting;
    psState                   State_O_Sc_Ac_SettingX;
    psState                   State_O_Sc_Ac_GettingY;
    psState                   State_O_Sc_Ac_CheckingFin;

  signals: //for internal needs
    void                      GaugeLinked ();
    void                      GaugeError(QString = QString());
    void                      GaugeBusy ();
    void                      GaugeMeasured (sPhysValue);
    void                      GaugePeriodChanged (sPhysValue);
    void                      EffectorLinked ();
    void                      EffectorError(QString = QString());
    void                      EffectorIsAtPos (sPhysValue);
    void                      DevicesBusy();
    void                      DevicesOK();
    void                      ScanContinue();

  protected: //reimplementing inherited
    virtual void              AppearanceMakeUp ();
    virtual void              Retranslate ()
                                {
                                  sScanner::Retranslate();
                                  Translate();
                                }

  protected slots: //reimplementing inherited
    virtual void              OnEnterToCoolingState ();

  protected slots: //implementing pure virtual
    virtual void              CreateNewDependence ();
    virtual void              StartScanning ();
    virtual void              StopScanning ();

  protected slots:
    virtual void              ReportGaugeError (QString);
    virtual void              ReportEffectorError (QString);
    virtual void              LinkToGauge ();
    virtual void              LinkToEffector ();
    virtual void              TakeDevicesControl ();
    virtual void              SetGaugeRegime ();
    virtual void              ResetGaugeRegime ();
    virtual void              RestartGauge ();
    virtual void              GaugeStartMeasure ();
    virtual void              BreakScanning ();
    virtual void              AddNewData (sPhysValue  Data);
    virtual void              CheckScanningFinished ();
    void                      ReleaseControl ();

  protected: //fields
    sGauge::sLink             Gauge;
    sEffector::sLink          Effector;
    sPhysValue                MeasureTime;
    sPhysRange                ValidScanRange;
    sPhysRange                ValidScanSteps;
    sPhysRange                ScanRange;
    sPhysValue                ScanStep;
    sPtr<sDataArrayRecord> DataArrayRecord;
    int                       NumOfPoints;
    int                       CurrentPointNo;
    bool                      GaugeWasStarted;
    bool                      GaugeWasInAutoRepeatMode;
    sPhysValue                OldPeriod;

    psPhysRangeField          ScanRangeField;
    psPhysValueField          ScanStepField;
    psPhysValueField          MeasureTimeField;

  private:
    void                      Translate ();
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
