////////////////////////////////////////////////////////////////////////////////
/*! @file TController.cpp Прототип контроллера термостата.
- Part of BasicDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.8  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2013, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "TController.h"
#include "attr.h"
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sTController)

//------------------------------------------------------------- sTController ---

void  sTController::Preconfigure (sNodePtr  DeviceCfg)
{
  sPtr<sFolderNode>              DeviceSettings;
  DeviceCfg |= "Settings";
  DeviceSettings = DeviceCfg >> "Settings";
  if( DeviceSettings.IsCorrect() ){
    DeviceSettings |= attr("HasSecondGage",false);
    DeviceSettings |= attr("HasSecondHeater",false);
    DeviceSettings |= attr("SecondHeaterIsAutocontrolled",false);
  };
  sEffector::Preconfigure(DeviceCfg);
}


sTController::~sTController ()
{
  if( DeviceSettings.IsCorrect() ){
//    DeviceSettings << attr("Period",ThePeriod);
  };
}


sTController::sTController (sString   DevicetId
                           ,sNodePtr  DeviceCfg)
    :sEffector(DevicetId,DeviceCfg)
    ,HasSecondGageFlag(false)
    ,HasSecondHeaterFlag(false)
    ,SecondHeaterAutocontrolledFlag(false)
{
}


void  sTController::PostConstructor ()
{
  sEffector::PostConstructor();
  try{ //get config parameters
    get_from<sBooleanNode>(DeviceSettings>>"HasSecondGage",HasSecondGageFlag);
    get_from<sBooleanNode>(DeviceSettings>>"HasSecondHeater"
                                           ,HasSecondHeaterFlag);
    get_from<sBooleanNode>(DeviceSettings>>"SecondHeaterIsAutocontrolled"
                                           ,SecondHeaterAutocontrolledFlag);
  }catch(...){
    SetInvalid(tr("Something wrong in configuration."));
  }
}


void  sTController::SetHasSecondGageFlag (bool has_second_gage)
{
  HasSecondGageFlag = has_second_gage;
}


void  sTController::SetHasSecondHeaterFlag (bool has_second_heater)
{
  HasSecondHeaterFlag = has_second_heater;
}


void  sTController::SetSecondHeaterAutocontrolledFlag (bool autocontrolled)
{
  SecondHeaterAutocontrolledFlag = autocontrolled;
}


void  sTController::SetSecondHeaterPower (real  power)
{
  ChangeSecondHeaterPower(power);
}


void  sTController::ChangeSecondHeaterPower (real)
{
}

//------------------------------------------------------------------------------
} //namespace RA




