////////////////////////////////////////////////////////////////////////////////
/*! @file TController.h   Прототип контроллера термостата.
- Part of BasicDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.8  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2013, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef TController_H
#define TController_H
#include "BasicDevices.h"
#include "Effectors.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_ABSTRACT_CLASS(sTController)

//------------------------------------------------------------- sTController ---
/*! Прототип контроллера термостата.
*/
class BasicDevices_EXPORT  sTController : public sEffector
{
  Q_OBJECT
  ABSTRACT_DRIVER(sTController)
  public: 
    virtual                   ~sTController();
    virtual bool              HasSecondGage ()
                                { return HasSecondGageFlag; }
    virtual void              SetHasSecondGageFlag (bool);
    virtual bool              HasSecondHeater ()
                                { return HasSecondHeaterFlag; }
    virtual void              SetHasSecondHeaterFlag (bool);
    virtual bool              SecondHeaterIsAutocontrolled ()
                                { return SecondHeaterAutocontrolledFlag; }
    virtual void              SetSecondHeaterAutocontrolledFlag (bool);

  public slots:
    virtual void              SetSecondHeaterPower (real  power);

  public:

  signals: //outgoing
    void                      NowInRange ();
    void                      NowOffRange ();
    void                      NewData (sPhysValue  T
                                      ,sPhysValue  DT
                                      ,sPhysValue  IT);
    void                      NewData (sPhysValue  TrueT
                                      ,sPhysValue  SecondT
                                      ,sPhysValue  T
                                      ,sPhysValue  DT
                                      ,sPhysValue  IT);

  signals: //for internal needs

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  protected:
    virtual void              ChangeHeaterPower (real  power) =0;
    virtual void              ChangeSecondHeaterPower (real  power);

  protected: //fields
  private: //fields
    bool                      HasSecondGageFlag;
    bool                      HasSecondHeaterFlag;
    bool                      SecondHeaterAutocontrolledFlag;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
