////////////////////////////////////////////////////////////////////////////////
/*! @file TeleCom.h   Short message communicator.
- Part of BasicDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2010-2013, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
- Copyright(C) 2010-2013, Dmitry K. Nelson, St.Petersburg, Russia.
                          D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef TeleCom_H
#define TeleCom_H
#include "BasicDevices.h"
#include "Driver.h"
#include <QByteArray>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sTelex)
ANNOUNCE_ABSTRACT_CLASS(sTeleCom)

//------------------------------------------------------------------- sTelex ---
/*! Telecommunication message - an elementary object,
    which is sent to or received from sTeleCom.
*/
struct BasicDevices_EXPORT  sTelex : public QByteArray
{
    virtual                   ~sTelex ();
    explicit                  sTelex ();
                              sTelex (rcsTelex);
                              sTelex (const QByteArray &  BA)
                                  :QByteArray(BA)
                                {}
    explicit                  sTelex (const char *);
//    bool                      operator == (rcsTelex  T) const;
    explicit                  sTelex (const QString &  S);
                              operator QString () const;
};

//------------------------------------------------------------------------------
} //namespace RA

Q_DECLARE_METATYPE(RA::sTelex)

namespace RA {
//------------------------------------------------------------------------------


//----------------------------------------------------------------- sTeleCom ---
/*! Прототип драйвера телекоммуникационного канала для обмена сообщениями.


@par The state machine
@dotfile "TeleCom.state_graph"

*/
class BasicDevices_EXPORT  sTeleCom : public sDriver
{
  Q_OBJECT
  ABSTRACT_DRIVER(sTeleCom)

  public: //commands & directives
    static const
      sDirective<sTelex>      cSendTelex;
//    static const
//      sDirective<sPhysValue>  cSetTimeout;

  public:
    virtual                   ~sTeleCom ();
//    virtual sPhysValue        Timeout ()
//                                { return TheTimeout; }

  signals: //outgoing
    void                      GotTelex (sTelex);

  public slots:

  protected:
    virtual void              PostConstructor ();

  protected slots:
    virtual void              PutTelex (sTelex) =0;

  private: //fields
//    sPhysValue                TheTimeout;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
