////////////////////////////////////////////////////////////////////////////////
/*! @file  	TunableCCDDriver.h   Прототип драйвера ПЗС-линейки с перестройкой длины волны.
- Part of BasicDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2017, Dmitrii Nelson, St.Petersburg, Russia.
                          d.nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "TunableCCDDriver.h"
#include "attr.h"
#include "StringsAndQStrings.h"
#include <QDebug>
#include "Log.h"

namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sTunableCCDDriver)

//-------------------------------------------------- sTunableCCDDriver ----------

void  sTunableCCDDriver::Preconfigure (sNodePtr  DeviceCfg)
{
  sPtr<sFolderNode>              DeviceSettings;
  sPtr<sFolderNode>              DeviceLinks;

  DeviceCfg |= "Links";
  DeviceLinks = DeviceCfg >> "Links";
  if( DeviceLinks.IsCorrect() ){
    DeviceLinks |= "WLDriver";
    DeviceLinks >> "WLDriver" |= attr("Class","sEffector");
  };

  DeviceCfg |= "Settings";
  DeviceSettings = DeviceCfg >> "Settings";
  if( DeviceSettings.IsCorrect() ){
    DeviceSettings |= attr("MiddlePixel",0);
  };
  sCCDDriver::Preconfigure(DeviceCfg);
}

sTunableCCDDriver::~sTunableCCDDriver ()
{
  WLDriver.SetToNULL();
}

sTunableCCDDriver::sTunableCCDDriver (sString   DevicetId
                       ,sNodePtr  DeviceCfg)
    :sCCDDriver(DevicetId,DeviceCfg)
    ,WLDriver(this)
    ,WLPosition(sPhysValue(500.0,0.01,"nm"))
    ,MiddlePixel(0)
{
}


void  sTunableCCDDriver::PostConstructor ()
{
  sCCDDriver::PostConstructor();
  try{ //get config parameters
    get_from<sIntegerNode>(DeviceSettings >> "MiddlePixel"   ,MiddlePixel);
  }catch(...){
    SetInvalid(tr("Something wrong in configuration."));
  }
  //evolution of the inherited state machine (behaviour)
  {
    { // evolution of the State_Warming
      State_W_LinkToWLDriver = new sState(State_Warming(),"State_W_LinkToWLDriver");
      State_W_LinkToWLDriver
          ->SetAsInitial()
          ->OnEnterCall( SLOT(ConnectWLDriver()) )
          ->AddTransition(SIGNAL(WLDriverWarmed()),State_W_Connect)
          ->AddTransition(SIGNAL(ErrorMessage()),State_Cooling());
    }
    { // evolution of the State_Operating
      State_O_Stopped
          ->AddReaction<sPhysValue>(SIGNAL(NewWL(sPhysValue))
                                   ,SLOT(AcceptWLPosition (sPhysValue)));
    }
    connect(this ,SIGNAL(WLDriverError(QString))
           ,this ,SLOT(ReportWLDriverError(QString)) );
  };
}

void sTunableCCDDriver::ReportWLDriverError (QString  error_text)
{
  emit ErrorMessage( tr("Got error from '%1':\n\n%2")
                       .arg(ToQString(WLDriver->DeviceIdentifier()))
                       .arg(error_text)     );
}

void sTunableCCDDriver::ConnectWLDriver ()
{
  try{
    WLDriver.SetTo(DeviceLinks,"WLDriver");
    Connect(WLDriver ,SIGNAL(Warmed()) ,SIGNAL(WLDriverWarmed()) );
    Connect(WLDriver ,SIGNAL(ErrorMessage(QString)) ,SIGNAL(WLDriverError(QString)) );
    Connect(WLDriver ,SIGNAL(NewData(sPhysValue)),SIGNAL(NewWL(sPhysValue)) );
  }catch( rxDeviceLaunch E ){
    emit ErrorMessage( tr("Error while linking to a submissive device.\n"
                          "%1\nCheck configuration.")
                         .arg(ToQString(E.MsgText()))   );
  }
}

void sTunableCCDDriver::AcceptWLPosition (sPhysValue _newWL)
{
  WLPosition = _newWL;
  CalculateWavelengths();
}


//------------------------------------------------------------------------------
} //namespace RA
