////////////////////////////////////////////////////////////////////////////////
/*! @file TunedEffectorConsole.cpp Консоль управления эффектором с инициализацией.
- Part of BasicDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.5.0  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2013, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
- Copyright(C) 2018, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "TunedEffectorConsole.h"
#include "attr.h"
#include "StringsAndQStrings.h"
#include <QVBoxLayout>
#include <QMessageBox>
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sTunedEffectorConsole)

//------------------------------------------------------- sTunedEffectorConsole ---

void  sTunedEffectorConsole::Preconfigure (sNodePtr  DeviceCfg)
{
//  sPtr<sFolderNode>              DeviceSettings;
//  DeviceCfg |= "Settings";
//  DeviceSettings = DeviceCfg >> "Settings";
//  if( DeviceSettings.IsCorrect() ){
//    DeviceSettings |= attr("Label"," v a l u e :");
//  };
  sEffectorConsole::Preconfigure(DeviceCfg);
}


//sTunedEffectorConsole::~sTunedEffectorConsole ()
//{
//}


sTunedEffectorConsole::sTunedEffectorConsole (sString   DeviceId
                                           ,sNodePtr  DeviceCfg
                                           ,sNodePtr  WidgetCfg
                                          ,QWidget *  parent    )
    :sEffectorConsole(DeviceId,DeviceCfg,WidgetCfg,parent)
{
  //evolution of the inherited state machine
  {
     //State_W_Effector internal structure and behaviour
     {
       State_W_Effector_Linking
           ->OnExitCall( SLOT(ConnectReadySignal()) );
     }
    //State_Operating internal structure and behaviour
    {
      State_O_Initiating = new sState(State_Operating(),"State_O_Initiating");
      State_O_Watching
          ->AddTransition(SIGNAL(Init()) ,State_O_Initiating);
      State_O_Initiating
          ->OnEnterCall(SLOT(OnInit()))
          ->AddTransition(SIGNAL(EffectorReady()) ,State_O_Watching)
          ;
    }
  }
}


void  sTunedEffectorConsole::AppearanceMakeUp ()
{
  sEffectorConsole::AppearanceMakeUp();
  BtnInit = new QPushButton(this);
  BtnInit->setText("Initiate");
  layout()->addWidget(BtnInit);

  connect(BtnInit ,SIGNAL(clicked()) ,this ,SIGNAL(Init()));
  State_O_Controling
      ->OnEnterCall(BtnInit ,SLOT(hide()) )
      ->OnExitCall(BtnInit ,SLOT(show()) )
      ;

}

void sTunedEffectorConsole::ConnectReadySignal ()
{
  Connect(Effector ,SIGNAL(TuneFinished()) ,SIGNAL(EffectorReady()) );
}

void sTunedEffectorConsole::OnInit()
{
  SendCommand(Effector ,sEffector::cTune );
}
/*
void sTunedEffectorConsole::ApplyRange (sPhysRange)
{

}
*/

//------------------------------------------------------------------------------
} //namespace RA
