////////////////////////////////////////////////////////////////////////////////
/*! @file LomoCCDDriver.h   Драйвер ПЗС-линейки ЛОМО.
- Part of Nelson - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2010-2017, Dmitrii Nelson, St.Petersburg, Russia.
                          d.nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef LomoCCDDriver_H
#define LomoCCDDriver_H
#include "CCDLomo.h"
#include "CCDDriver.h"
#include "TunableCCDDriver.h"
#include <windows.h>
#include "ftd2xx.h"
namespace RA {

#define FRAME_START     0x80
#define FRAME_MIDDLE    0x90
#define FRAME_END       0xB0

//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sLomoCCDDriver)

//----------------------------------------------------------- sLomoCCDDriver ---
/*! Драйвер ПЗС-линейки ЛОМО.

*/

class CCDLomo_EXPORT  sLomoCCDDriver : public sTunableCCDDriver
{
  Q_OBJECT
  DRIVER(sLomoCCDDriver)

  public: //commands & directives

  public:
    virtual                  ~sLomoCCDDriver ();

  signals: //outgoing

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  protected slots: //reimplementing inherited

  protected slots:
    virtual void              ConnectUSBDevice ();
    virtual void              GetCCD_Data ();
    virtual void              CalculateWavelengths ();
    virtual void              StartCCD ();
    virtual void              StopCCD ();
    virtual void              SetCCD_Parameters (sCCDDriver::sParameters Params);

  protected:
    QString                   FormCCDErrorMessage(FT_STATUS CCDStatus);
    void                      ResetTriadeBuffer();
    WORD                      ConvertTriadeBuffer();
    bool                      SendParam(BYTE Command,WORD Param);
    void                      AddByte(BYTE DataByte);

  protected: //fields
    WORD                      CCD_SH;
    WORD                      CCD_ICG;
    sPhysValue                WL_0;
    sPhysValue                WL_k;

  private: //fields
    sString                   CCDDeviceName;
    FT_HANDLE                 CCDDeviceHandle;
    BYTE                      TriadeBuffer[3];
    BYTE                      NumInTriade;
    int                       PointNo;
};

//------------------------------------------------------------------------------
} //namespace RA

#endif
