////////////////////////////////////////////////////////////////////////////////
/*! @file COMPortTeleCom.h   Short message service via serial port.
- Part of COMPort - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
- Copyright(C) 2010, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef COMPortTeleCom_H
#define COMPortTeleCom_H
#include "COMPort.h"
#include "TeleCom.h"
#include <QtAddOnSerialPort/serialport.h>
#include <QString>

QT_USE_NAMESPACE_SERIALPORT

namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sSerialPort)
ANNOUNCE_CLASS(sAskingWhoSerialPort)
ANNOUNCE_CLASS(sListeningForIntroducementSerialPort)
ANNOUNCE_CLASS(sCOMPortTeleCom)



//-------------------------------------------------------------- sSerialPort ---
/*!

*/
class COMPort_EXPORT  sSerialSequencer : public SerialPort
{
  Q_OBJECT
  public:
    virtual                   ~sSerialSequencer ();
                              sSerialSequencer (const QString &  port_name
                                          ,int              baud_rate
                                          ,QObject *        parent)
                                  :SerialPort(port_name,parent)
                                {

                                }

};

//---------------------------------------------------------- sCOMPortTeleCom ---
/*!

*/
class COMPort_EXPORT  sCOMPortTeleCom : public sTeleCom
{
  Q_OBJECT
  DRIVER(sCOMPortTeleCom)

  public:
    virtual                   ~sCOMPortTeleCom ();

  protected: //evolution of the inherited state machine
    QState *                  State_W_OpeningPort;

  signals: //for internal needs
    void                      IOError (QString);

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  protected slots: //reimplementing inherited
    virtual void              OnEnterToOperatingState ();
    virtual void              OnExitFromOperatingState ();
    virtual void              OnEnterToCoolingState ();

  protected slots:
    virtual void              PutTelex (sTelex);
    virtual void              ReadAvailableBytes ();

  protected: //fields
    QString                   Name;
    bool                      NameIsDeviceName;
    int                       BaudRate;
    QByteArray                TerminalSymbols;

  private slots:
    void                      OpenPort ();

  private: //fields
    sSerialSequencer *             ThePort;
    QByteArray                InputBuffer;
/*
    virtual void              Connect ();
    virtual void              Disconnect ();

  public slots:
    virtual bool              SendMessage (const QByteArray &);

  private slots:
    void                      on_ReadyToRead();

  private:
    bool                      FindDeviceAndSetPortName();

*/
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
