////////////////////////////////////////////////////////////////////////////////
/*! @file TeleCOMPort.h   Short message service via serial port.
- Part of COMPort - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
- Copyright(C) 2010, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef TeleCOMPort_H
#define TeleCOMPort_H
#include "COMPort.h"
#include "TeleCom.h"
#include <QtSerialPort/QSerialPort>
#include <QString>
#include <list>

//QT_USE_NAMESPACE_SERIALPORT

namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sSerialSequencer)
ANNOUNCE_CLASS(sPortOpeningState)
ANNOUNCE_CLASS(sTeleCOMPort)

//--------------------------------------------------------- sSerialSequencer ---
/*!

*/
class COMPort_EXPORT  sSerialSequencer : public QObject
{
  Q_OBJECT
  public:
    virtual                   ~sSerialSequencer ();
                              sSerialSequencer
                                      (QObject *          parent
                                      ,sPtr<sFolderNode>  settings
                                      ,const QString &    port_name);
                              sSerialSequencer
                                      (QObject *                parent
                                      ,sPtr<sFolderNode>        settings
                                      ,const QSerialPortInfo &  info);

  signals: //outgoing
    void                      Ready ();
    void                      GotTelex (sTelex);
    void                      OpeningError (QString);
    void                      IOError (QString);

  public slots:
    virtual void              Open ();
    virtual void              PutTelex (sTelex);

  protected slots:
    virtual void              ReadAvailableBytes ();

  protected: //fields
    sPtr<sFolderNode>         DeviceSettings;
    QSerialPort *             Port;
    integer                   BaudRate;
    QSerialPort::Parity       Parity;
    QSerialPort::DataBits     DataBits;
    QSerialPort::StopBits     StopBits;
    QByteArray                TerminalSymbols;
    QByteArray                InputBuffer;
  private:
    void                      GetConfigParameters ();
};

//------------------------------------------------------------------------------
} //namespace RA

Q_DECLARE_METATYPE(RA::psSerialSequencer)

namespace RA {
//------------------------------------------------------------------------------


//-------------------------------------------------------- sPortOpeningState ---
/*! A composite state that tries to establish connection to specified
    hardware device through a certain serial port.

@note Its final state (Fin()) means failure.

*/
class  sPortOpeningState : public sState
{
  Q_OBJECT
  public:
    virtual                   ~sPortOpeningState ();
                              sPortOpeningState (psState      parent_state
                                                ,sString      introducement
                                                ,psSerialSequencer);
  signals:
    void                      Success (psSerialSequencer);

  public: //fields
    psSerialSequencer         Sequencer;

  protected: //fields
    psState                   State_Opening;
    psState                   State_WaitingForIntroducement;
    psState                   State_Success;

  private slots:
//    void                      SendRequest ();
    void                      OnEnterTo_State_WaitingForDeviceName ()
                                {
                                  connect(Sequencer ,SIGNAL(GotTelex(sTelex))
                                         ,this ,SLOT(CheckDeviceName(sTelex)) );
                                }
    void                      OnExitFrom_State_WaitingForDeviceName ()
                                {
                                  disconnect(Sequencer ,0 ,this ,0 );
                                }
    void                      CheckDeviceName (sTelex);

  private: //fields
    sString                   Introducement;
};


//------------------------------------------------------------- sTeleCOMPort ---
/*!

*/
class COMPort_EXPORT  sTeleCOMPort : public sTeleCom
{
  Q_OBJECT
  DRIVER(sTeleCOMPort)

//  public: //types
//    enum                      eDeviceChecking  {cNoChecking
//                                               ,cListeningForIntroducement
//                                               ,cAskingWho
//                                               };

  public: //static const
    static csString           DefaultTerminalSymbols;

  public:
    virtual                   ~sTeleCOMPort ();

  protected: //evolution of the inherited state machine
    psParallelStateGroup      State_W_Parallel;
    psState                   State_W_Opening;

  signals: //for internal needs
    void                      PortOK (psSerialSequencer  SS =NULL);

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  protected slots: //reimplementing inherited
    virtual void              OnExitFromWarmingState ();
    virtual void              OnEnterToOperatingState ();
    virtual void              OnExitFromOperatingState ();
    virtual void              OnEnterToCoolingState ();
    virtual void              PutTelex (sTelex);

  protected: //fields

  private slots:
    void                      OnOpeningError (QString);
    void                      OnPortOK (psSerialSequencer);

  private: //fields
    bool                      SearchThroughAvailablePorts;
    sString                   PortName;
    sString                   Description;
    sString                   Manufacturer;
    int                       VendorId;
    int                       ProductId;
    sString                   IntroducementPrefix;
    sString                   DeviceName;
//    sString                   RequestToBeSent;

    std::list
        <psPortOpeningState>  OpeningStates;
    psSerialSequencer         TheSequencer;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
