////////////////////////////////////////////////////////////////////////////////
/*! @file Arithmetics.cpp Arithmetics on functions.
- Part of CommonMath - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2015, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "Arithmetics.h"
#include "TableDataFunction.h"
namespace RA {
//------------------------------------------------------------------------------


psMathValue  UnaryMinusDataFunction (pcsMathValue  arg1)
{
  pcsDataFunction             Arg1(dynamic_cast<pcsDataFunction>(arg1));
  ASSERT( Arg1 );
  psMathValue                 Result = NULL;
  psTableDataFunction         R = NULL;
  sCurve::psTracer            T = NULL;
  int                         N = Arg1->NumberOfPoints();
  R = new sTableDataFunction(N);
  R->SetMultiplier(Arg1->Multiplier());
  T = Arg1->GetCurveTracer(sScales(sScale(Arg1->Multiplier().X)
                                  ,sScale(Arg1->Multiplier().Y)));
  int                         PointNo = 0;
  while( T->NextPoint() ){
    R->SetValue(PointNo ,sPoint( T->Point.X , - T->Point.Y ) );
    PointNo++;
  }
  Result = R;
  if( T ){ delete T;  T=NULL; };
  return Result;
}



psMathValue  DataFunctionPlusFunction (pcsMathValue  arg1
                                      ,pcsMathValue  arg2)
{
  pcsDataFunction             Arg1(dynamic_cast<pcsDataFunction>(arg1));
  pcsFunction                 Arg2(dynamic_cast<pcsFunction>(arg2));
  ASSERT( Arg1 && Arg2 );
  psMathValue                 Result = NULL;
  psTableDataFunction         R = NULL;
  sCurve::psTracer            T = NULL;
  try{
    int                         N = 0;
    real                        From = ( Arg2->AbscissaRange().From()
                                       / Arg1->Multiplier().X
                                       ).ToReal();
    real                        To =   ( Arg2->AbscissaRange().To()
                                       / Arg1->Multiplier().X
                                       ).ToReal();
    T = Arg1->GetCurveTracer(sScales(sScale(Arg1->Multiplier().X)
                                    ,sScale(Arg1->Multiplier().Y)));
    while( T->NextPoint() && (T->Point.X <= To ) ){
      if( T->Point.X >= From )
        N++;
    }
    if( N < 2 ){
      Result = new sMathError("abscissa ranges aren't overlapping");
    }else{
      if( T ){ delete T;  T=NULL; };
      T = Arg1->GetCurveTracer(sScales(sScale(Arg1->Multiplier().X)
                                      ,sScale(Arg1->Multiplier().Y)));
      R = new sTableDataFunction(N);
      R->SetMultiplier(Arg1->Multiplier());
      int                         PointNo = 0;
      while( T->NextPoint() && (T->Point.X <= To ) ){
        if( T->Point.X >= From ){
          R->SetValue
              (PointNo
              ,sPoint( T->Point.X
                     , T->Point.Y
                      +( Arg2->Value(T->Point.X * Arg1->Multiplier().X)
                        /Arg1->Multiplier().Y
                       ).ToReal()
                     ) );
          PointNo++;
        }
      }
      Result = R;
    }
  }catch( rxRAlgebra ){
    if( R ){ delete R;  R=NULL; };
    Result = new sMathError("uncomparable units");
  }
  if( T ){ delete T;  T=NULL; };
  return Result;
}



psMathValue  DataFunctionMinusFunction (pcsMathValue  arg1
                                       ,pcsMathValue  arg2)
{
  pcsDataFunction             Arg1(dynamic_cast<pcsDataFunction>(arg1));
  pcsFunction                 Arg2(dynamic_cast<pcsFunction>(arg2));
  ASSERT( Arg1 && Arg2 );
  psMathValue                 Result = NULL;
  psTableDataFunction         R = NULL;
  sCurve::psTracer            T = NULL;
  try{
    int                         N = 0;
    real                        From = ( Arg2->AbscissaRange().From()
                                       / Arg1->Multiplier().X
                                       ).ToReal();
    real                        To =   ( Arg2->AbscissaRange().To()
                                       / Arg1->Multiplier().X
                                       ).ToReal();
    T = Arg1->GetCurveTracer(sScales(sScale(Arg1->Multiplier().X)
                                    ,sScale(Arg1->Multiplier().Y)));
    while( T->NextPoint() && (T->Point.X <= To ) ){
      if( T->Point.X >= From )
        N++;
    }
    if( N < 2 ){
      Result = new sMathError("abscissa ranges aren't overlapping");
    }else{
      if( T ){ delete T;  T=NULL; };
      T = Arg1->GetCurveTracer(sScales(sScale(Arg1->Multiplier().X)
                                      ,sScale(Arg1->Multiplier().Y)));
      R = new sTableDataFunction(N);
      R->SetMultiplier(Arg1->Multiplier());
      int                         PointNo = 0;
      while( T->NextPoint() && (T->Point.X <= To ) ){
        if( T->Point.X >= From ){
          R->SetValue
              (PointNo
              ,sPoint( T->Point.X
                     , T->Point.Y
                      -( Arg2->Value(T->Point.X * Arg1->Multiplier().X)
                        /Arg1->Multiplier().Y
                       ).ToReal()
                     ) );
          PointNo++;
        }
      }
      Result = R;
    }
  }catch( rxRAlgebra ){
    if( R ){ delete R;  R=NULL; };
    Result = new sMathError("uncomparable units");
  }
  if( T ){ delete T;  T=NULL; };
  return Result;
}



psMathValue  DataFunctionMultFunction (pcsMathValue  arg1
                                      ,pcsMathValue  arg2)
{
  pcsDataFunction             Arg1(dynamic_cast<pcsDataFunction>(arg1));
  pcsFunction                 Arg2(dynamic_cast<pcsFunction>(arg2));
  ASSERT( Arg1 && Arg2 );
  psMathValue                 Result = NULL;
  psTableDataFunction         R = NULL;
  sCurve::psTracer            T = NULL;
  try{
    int                         N = 0;
    real                        From = ( Arg2->AbscissaRange().From()
                                       / Arg1->Multiplier().X
                                       ).ToReal();
    real                        To =   ( Arg2->AbscissaRange().To()
                                       / Arg1->Multiplier().X
                                       ).ToReal();
    sPhysValue                  M2Y;
    sPhysValue                  MY;
    M2Y = sPhysValue(Arg2->OrdinateUnits());
    MY = Arg1->Multiplier().Y * M2Y;
    MY.Normalize();
    T = Arg1->GetCurveTracer(sScales(sScale(Arg1->Multiplier().X)
                                    ,sScale(Arg1->Multiplier().Y)));
    while( T->NextPoint() && (T->Point.X <= To ) ){
      if( T->Point.X >= From )
        N++;
    }
    if( N < 2 ){
      Result = new sMathError("abscissa ranges aren't overlapping");
    }else{
      if( T ){ delete T;  T=NULL; };
      T = Arg1->GetCurveTracer(sScales(sScale(Arg1->Multiplier().X)
                                      ,sScale(Arg1->Multiplier().Y)));
      R = new sTableDataFunction(N);
      R->SetMultiplier(sPhysPair(Arg1->Multiplier().X,MY));
      int                         PointNo = 0;
      while( T->NextPoint() && (T->Point.X <= To ) ){
        if( T->Point.X >= From ){
          R->SetValue
              (PointNo
              ,sPoint( T->Point.X
                     , T->Point.Y
                      *( Arg2->Value(T->Point.X * Arg1->Multiplier().X)
                        /M2Y
                       ).ToReal()
                     ) );
          PointNo++;
        }
      }
      Result = R;
    }
  }catch( rxRAlgebra ){
    if( R ){ delete R;  R=NULL; };
    Result = new sMathError("uncomparable units");
  }
  if( T ){ delete T;  T=NULL; };
  return Result;
}



psMathValue  DataFunctionDivFunction (pcsMathValue  arg1
                                     ,pcsMathValue  arg2)
{
  pcsDataFunction             Arg1(dynamic_cast<pcsDataFunction>(arg1));
  pcsFunction                 Arg2(dynamic_cast<pcsFunction>(arg2));
  ASSERT( Arg1 && Arg2 );
  psMathValue                 Result = NULL;
  psTableDataFunction         R = NULL;
  sCurve::psTracer            T = NULL;
  try{
    int                         N = 0;
    real                        From = ( Arg2->AbscissaRange().From()
                                       / Arg1->Multiplier().X
                                       ).ToReal();
    real                        To =   ( Arg2->AbscissaRange().To()
                                       / Arg1->Multiplier().X
                                       ).ToReal();
    sPhysValue                  M2Y;
    sPhysValue                  MY;
    M2Y = sPhysValue(Arg2->OrdinateUnits());
    MY = Arg1->Multiplier().Y / M2Y;
    MY.Normalize();
    T = Arg1->GetCurveTracer(sScales(sScale(Arg1->Multiplier().X)
                                    ,sScale(Arg1->Multiplier().Y)));
    while( T->NextPoint() && (T->Point.X <= To ) ){
      if( T->Point.X >= From )
        N++;
    }
    if( N < 2 ){
      Result = new sMathError("abscissa ranges aren't overlapping");
    }else{
      if( T ){ delete T;  T=NULL; };
      T = Arg1->GetCurveTracer(sScales(sScale(Arg1->Multiplier().X)
                                      ,sScale(Arg1->Multiplier().Y)));
      R = new sTableDataFunction(N);
      R->SetMultiplier(sPhysPair(Arg1->Multiplier().X,MY));
      int                         PointNo = 0;
      while( T->NextPoint() && (T->Point.X <= To ) ){
        if( T->Point.X >= From ){
          R->SetValue
              (PointNo
              ,sPoint( T->Point.X
                     , T->Point.Y
                      /( Arg2->Value(T->Point.X * Arg1->Multiplier().X)
                        /M2Y
                       ).ToReal()
                     ) );
          PointNo++;
        }
      }
      Result = R;
    }
  }catch( rxRAlgebra ){
    if( R ){ delete R;  R=NULL; };
    Result = new sMathError("uncomparable units");
  }
  if( T ){ delete T;  T=NULL; };
  return Result;
}



psMathValue  DataFunctionPlusPhysValue (pcsMathValue  arg1
                                       ,pcsMathValue  arg2)
{
  pcsDataFunction             Arg1(dynamic_cast<pcsDataFunction>(arg1));
  pcsPhysValue                Arg2(dynamic_cast<pcsPhysValue>(arg2));
  ASSERT( Arg1 && Arg2 );
  psMathValue                 Result = NULL;
  psTableDataFunction         R = NULL;
  sCurve::psTracer            T = NULL;
  try{
    real                        A2 = ((*Arg2)/Arg1->Multiplier().Y).ToReal();
    int                         N = Arg1->NumberOfPoints();
    R = new sTableDataFunction(N);
    R->SetMultiplier(Arg1->Multiplier());
    T = Arg1->GetCurveTracer(sScales(sScale(Arg1->Multiplier().X)
                                    ,sScale(Arg1->Multiplier().Y)));
    int                         PointNo = 0;
    while( T->NextPoint() ){
      R->SetValue(PointNo ,sPoint( T->Point.X , T->Point.Y + A2 ) );
      PointNo++;
    }
    Result = R;
  }catch( rxRAlgebra ){
    if( R ){ delete R;  R=NULL; };
    Result = new sMathError("uncomparable units");
  }
  if( T ){ delete T;  T=NULL; };
  return Result;
}



psMathValue  DataFunctionMinusPhysValue (pcsMathValue  arg1
                                        ,pcsMathValue  arg2)
{
  pcsPhysValue                Arg2(dynamic_cast<pcsPhysValue>(arg2));
  ASSERT( Arg2 );
  sPhysValue                  A2(-(*Arg2));
  return DataFunctionPlusPhysValue(arg1,&A2);
}



psMathValue  DataFunctionMultPhysValue (pcsMathValue  arg1
                                       ,pcsMathValue  arg2)
{
  pcsDataFunction             Arg1(dynamic_cast<pcsDataFunction>(arg1));
  pcsPhysValue                Arg2(dynamic_cast<pcsPhysValue>(arg2));
  ASSERT( Arg1 && Arg2 );
  psMathValue                 Result = NULL;
  psTableDataFunction         R = NULL;
  sCurve::psTracer            T = NULL;
  try{
    int                         N = Arg1->NumberOfPoints();
    sPhysValue                  M2(Arg2->Units());
    real                        ArbArg2 = ((*Arg2)/M2).ToReal();
    sPhysValue                  MY;
    MY = Arg1->Multiplier().Y * M2;
    MY.Normalize();
    R = new sTableDataFunction(N);
    R->SetMultiplier(sPhysPair(Arg1->Multiplier().X,MY));
    T = Arg1->GetCurveTracer(sScales(sScale(Arg1->Multiplier().X)
                                    ,sScale(Arg1->Multiplier().Y)));
    int                         PointNo = 0;
    while( T->NextPoint() ){
      R->SetValue(PointNo , sPoint( T->Point.X , T->Point.Y*ArbArg2 ) );
      PointNo++;
    }
    Result = R;
  }catch( rxRAlgebra ){
    if( R ){ delete R;  R=NULL; };
    Result = new sMathError("xRAlgebra");
  }
  if( T ){ delete T;  T=NULL; };
  return Result;
}



psMathValue  DataFunctionDivPhysValue (pcsMathValue  arg1
                                      ,pcsMathValue  arg2)
{
  pcsPhysValue                Arg2(dynamic_cast<pcsPhysValue>(arg2));
  ASSERT( Arg2 );
  sPhysValue                  A2(1.0/(*Arg2));
  return DataFunctionMultPhysValue(arg1,&A2);
}



psMathValue  PhysValuePlusDataFunction (pcsMathValue  arg1
                                       ,pcsMathValue  arg2)
{
  return DataFunctionPlusPhysValue(arg2,arg1);
}



psMathValue  PhysValueMinusDataFunction (pcsMathValue  arg1
                                        ,pcsMathValue  arg2)
{
  return DataFunctionPlusPhysValue(UnaryMinusDataFunction(arg2),arg1);
}



psMathValue  PhysValueMultDataFunction (pcsMathValue  arg1
                                       ,pcsMathValue  arg2)
{
  return DataFunctionMultPhysValue(arg2,arg1);
}



psMathValue  PhysValueDivDataFunction (pcsMathValue  arg1
                                      ,pcsMathValue  arg2)
{
  pcsPhysValue                Arg1(dynamic_cast<pcsPhysValue>(arg1));
  pcsDataFunction             Arg2(dynamic_cast<pcsDataFunction>(arg2));
  ASSERT( Arg1 && Arg2 );
  psMathValue                 Result = NULL;
  psTableDataFunction         R = NULL;
  sCurve::psTracer            T = NULL;
  try{
    int                         N = Arg2->NumberOfPoints();
    sPhysValue                  M1(Arg1->Units());
    real                        ArbArg1 = ((*Arg1)/M1).ToReal();
    sPhysValue                  MY;
    MY = M1 / Arg2->Multiplier().Y;
    MY.Normalize();
    R = new sTableDataFunction(N);
    R->SetMultiplier(sPhysPair(Arg2->Multiplier().X,MY));
    T = Arg2->GetCurveTracer(sScales(sScale(Arg2->Multiplier().X)
                                    ,sScale(Arg2->Multiplier().Y)));
    int                         PointNo = 0;
    while( T->NextPoint() ){
      R->SetValue(PointNo , sPoint( T->Point.X , ArbArg1 / T->Point.Y ) );
      PointNo++;
    }
    Result = R;
  }catch( rxRAlgebra ){
    if( R ){ delete R;  R=NULL; };
    Result = new sMathError("xRAlgebra");
  }
  if( T ){ delete T;  T=NULL; };
  return Result;
}


//------------------------------------------------------------------------------
} //namespace RA
