////////////////////////////////////////////////////////////////////////////////
/*! @file CommonMath.cpp Common mathematical things.
- Part of CommonMath - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2015, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "CommonMath.h"
#include "Arithmetics.h"
#include "FAlgebra.h"
#include "Redigitizing.h"
#include "MathAn.h"
#include "PhysValue.h"
#include "Functions.h"
//#include <QMetaType>
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sCommonMath)

//------------------------------------------------------------------------------

sVersion  sCommonMath::Version ()
{
  return sVersion(RAP_VERS_MAJOR
                 ,RAP_VERS_MINOR
                 ,ZAKAVYCH(RAP_VERS_PATCH)
                 ,__DATE__
                 );
}


sVersion  sCommonMath::CoreVersionUsed ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}


sVersion  sCommonMath::CoreVersionMin ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}


QString  sCommonMath::UserFriendlyName ()
{
  return trUtf8("Common mathematics");
}


QString  sCommonMath::About ()
{
  return trUtf8(
      "The package provides algorithms for common mathematic operations on "
      "functions.\n\n"
      "Copyright (C) 2015, Viktor E. Bursian, St.Petersburg, Russia.\n"
      "                    Viktor.Bursian@mail.ioffe.ru"
                );
}


sCommonMath::sCommonMath ()
{
  //........... from Arithmetics ................

  sMath::Definitions.Add(&UnaryMinusDataFunction,"-",sDataFunction::Type());
  sMath::Definitions.Add(&DataFunctionPlusFunction,"+"
                        ,sDataFunction::Type(),sFunction::Type());
  sMath::Definitions.Add(&DataFunctionMinusFunction,"-"
                        ,sDataFunction::Type(),sFunction::Type());
  sMath::Definitions.Add(&DataFunctionMultFunction,"*"
                        ,sDataFunction::Type(),sFunction::Type());
  sMath::Definitions.Add(&DataFunctionDivFunction,"/"
                        ,sDataFunction::Type(),sFunction::Type());

  sMath::Definitions.Add(&DataFunctionPlusPhysValue,"+"
                        ,sDataFunction::Type(),sPhysValue::Type());
  sMath::Definitions.Add(&DataFunctionMinusPhysValue,"-"
                        ,sDataFunction::Type(),sPhysValue::Type());
  sMath::Definitions.Add(&DataFunctionMultPhysValue,"*"
                        ,sDataFunction::Type(),sPhysValue::Type());
  sMath::Definitions.Add(&DataFunctionDivPhysValue,"/"
                        ,sDataFunction::Type(),sPhysValue::Type());

  sMath::Definitions.Add(&PhysValuePlusDataFunction,"+"
                        ,sPhysValue::Type(),sDataFunction::Type());
  sMath::Definitions.Add(&PhysValueMinusDataFunction,"-"
                        ,sPhysValue::Type(),sDataFunction::Type());
  sMath::Definitions.Add(&PhysValueMultDataFunction,"*"
                        ,sPhysValue::Type(),sDataFunction::Type());
  sMath::Definitions.Add(&PhysValueDivDataFunction,"/"
                        ,sPhysValue::Type(),sDataFunction::Type());

  //........... from FAlgebra ................

  sMath::Definitions.Add(&LgOfDataFunction,"lg"
                        ,sDataFunction::Type());
  sMath::Definitions.Add(&LnOfDataFunction,"ln"
                        ,sDataFunction::Type());
  sMath::Definitions.Add(&LogOfDataFunction,"log"
                        ,sRealValue::Type(),sDataFunction::Type());

  //........... from Redigitizing ................

  sMath::Definitions.Add(&digitize,"digitize"
                        ,sPhysValue::Type(),sPhysValue::Type()
                        ,sPhysValue::Type(),sFunction::Type());
  sMath::Definitions.Add(&CutDataFunction,"cut"
                        ,sPhysValue::Type(),sPhysValue::Type()
                        ,sFunction::Type());
  sMath::Definitions.Add(&ShiftDataFunction,"shift"
                        ,sPhysValue::Type()
                        ,sDataFunction::Type());
  sMath::Definitions.Add(&ReverseFunction,"reverse"
                        ,sDataFunction::Type());
  sMath::Definitions.Add(&Fold2v,"fold2v"
                        ,sDataFunction::Type());
  sMath::Definitions.Add(&Fold4v,"fold4v"
                        ,sDataFunction::Type());

  //........... from MathAn ................

  sMath::Definitions.Add(&minX,"minX",sFunction::Type());
  sMath::Definitions.Add(&maxX,"maxX",sFunction::Type());
  sMath::Definitions.Add(&ValueAt,"value_at"
                        ,sPhysValue::Type()
                        ,sFunction::Type());
  sMath::Definitions.Add(&Maximum,"maximum"
                        ,sDataFunction::Type());
  sMath::Definitions.Add(&MaximumPos,"maximum_pos"
                        ,sDataFunction::Type());
  sMath::Definitions.Add(&RunningIntegral,"integral"
                        ,sDataFunction::Type());
  sMath::Definitions.Add(&FullIntegral,"Integral"
                        ,sDataFunction::Type());
  sMath::Definitions.Add(&lorentz,"lorentz"
                        ,sPhysValue::Type(),sPhysValue::Type());
  sMath::Definitions.Add(&gauss,"gauss"
                        ,sPhysValue::Type(),sPhysValue::Type());
}

//------------------------------------------------------------------------------
} //namespace RA
