////////////////////////////////////////////////////////////////////////////////
/*! @file FAlgebra.cpp Algebra on functions.
- Part of CommonMath - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2015, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "FAlgebra.h"
#include "TableDataFunction.h"
namespace RA {
//------------------------------------------------------------------------------


psMathValue  LgOfDataFunction (pcsMathValue  arg)
{
  pcsDataFunction             Arg(dynamic_cast<pcsDataFunction>(arg));
  ASSERT( Arg );
  try{
    Arg->Multiplier().Y.ToReal();
  }catch( rxRAlgebra ){
    return new sMathError("argument of lg() must have unitsless ordinates");
  }
  psTableDataFunction         R = NULL;
  sCurve::psTracer            T = NULL;
  int                         N = Arg->NumberOfPoints();
  R = new sTableDataFunction(N);
  R->SetMultiplier(Arg->Multiplier());
  T = Arg->GetCurveTracer(sScales(sScale(Arg->Multiplier().X)
                                  ,sScale(Arg->Multiplier().Y)));
  int                         PointNo = 0;
  while( T->NextPoint() ){
    R->SetValue(PointNo ,sPoint( T->Point.X , log10( T->Point.Y ) ) );
    PointNo++;
  }
  if( T ){ delete T;  T=NULL; };
  return R;
}



psMathValue  LnOfDataFunction (pcsMathValue  arg)
{
  pcsDataFunction             Arg(dynamic_cast<pcsDataFunction>(arg));
  ASSERT( Arg );
  try{
    Arg->Multiplier().Y.ToReal();
  }catch( rxRAlgebra ){
    return new sMathError("argument of ln() must have unitsless ordinates");
  }
  psTableDataFunction         R = NULL;
  sCurve::psTracer            T = NULL;
  int                         N = Arg->NumberOfPoints();
  R = new sTableDataFunction(N);
  R->SetMultiplier(Arg->Multiplier());
  T = Arg->GetCurveTracer(sScales(sScale(Arg->Multiplier().X)
                                  ,sScale(Arg->Multiplier().Y)));
  int                         PointNo = 0;
  while( T->NextPoint() ){
    R->SetValue(PointNo ,sPoint( T->Point.X , log( T->Point.Y ) ) );
    PointNo++;
  }
  if( T ){ delete T;  T=NULL; };
  return R;
}



psMathValue  LogOfDataFunction (pcsMathValue  arg1
                               ,pcsMathValue  arg2)
{
  pcsRealValue                Arg1(dynamic_cast<pcsRealValue>(arg1));
  ASSERT( Arg1 );
  pcsDataFunction             Arg2(dynamic_cast<pcsDataFunction>(arg2));
  ASSERT( Arg2 );
  try{
    Arg2->Multiplier().Y.ToReal();
  }catch( rxRAlgebra ){
    return new sMathError("second argument of log(,)"
                          " must have unitsless ordinates");
  }
  psTableDataFunction         R = NULL;
  sCurve::psTracer            T = NULL;
  int                         N = Arg2->NumberOfPoints();
  R = new sTableDataFunction(N);
  R->SetMultiplier(Arg2->Multiplier());
  T = Arg2->GetCurveTracer(sScales(sScale(Arg2->Multiplier().X)
                                  ,sScale(Arg2->Multiplier().Y)));
  int                         PointNo = 0;
  while( T->NextPoint() ){
    R->SetValue(PointNo ,sPoint( T->Point.X ,  log( T->Point.Y )
                                             / log( real(*Arg1) ) ) );
    PointNo++;
  }
  if( T ){ delete T;  T=NULL; };
  return R;
}


//------------------------------------------------------------------------------
} //namespace RA
