////////////////////////////////////////////////////////////////////////////////
/*! @file MathAn.h   MathAn on functions.
- Part of CommonMath - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2015, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef MathAn_H
#define MathAn_H
#include "CommonMath.h"
#include "Functions.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sLorentzFunction)
ANNOUNCE_CLASS(sGaussFunction)

//------------------------------------------------------------------------------
// todo:
// smooth diff|derivative convolution
// delta theta
// linear poly
// baseline (1) (а) вычитание или (б) только вычисление подставки для ЭПР
//          (2) вычитание линейной подставки
// WL2WN Modulate
//------------------------------------------------------------------------------


CommonMath_EXPORT extern psMathValue  minX (pcsMathValue  function);

CommonMath_EXPORT extern psMathValue  maxX (pcsMathValue  function);

CommonMath_EXPORT extern psMathValue  ValueAt
                                          (pcsMathValue  position
                                          ,pcsMathValue  function);

CommonMath_EXPORT extern psMathValue  Maximum
                                          (pcsMathValue  data_function );

CommonMath_EXPORT extern psMathValue  MaximumPos
                                          (pcsMathValue  data_function );

CommonMath_EXPORT extern psMathValue  RunningIntegral
                                          (pcsMathValue  data_function );

CommonMath_EXPORT extern psMathValue  FullIntegral
                                          (pcsMathValue  data_function );

CommonMath_EXPORT extern psMathValue  lorentz
                                          (pcsMathValue  position
                                          ,pcsMathValue  width   );

CommonMath_EXPORT extern psMathValue  gauss
                                          (pcsMathValue  position
                                          ,pcsMathValue  width   );


//--------------------------------------------------------- sLorentzFunction ---

class CommonMath_EXPORT  sLorentzFunction : public sAnalyticFunction
{
  STORABLE(sLorentzFunction)
  public:
    virtual                   ~sLorentzFunction ()
                                {}
                              sLorentzFunction (rcsPhysValue  position
                                               ,rcsPhysValue  width   );

  public:
    virtual psMathValue       Replica () const;
    virtual sBoundaries       Boundaries () const;
    virtual sPhysRange        AbscissaRange () const;
    virtual sUnits            OrdinateUnits () const;
    virtual sPhysValue        Value (rcsPhysValue  x) const;
    sPhysValue                Position () const
                                { return ThePosition*TheMultiplier.X; }
    sPhysValue                Width () const  //!< full width at half maximum
                                { return 2*TheGamma*TheMultiplier.X; }
    sPhysValue                Gamma () const
                                { return TheGamma*TheMultiplier.X; }

  protected:

  protected://fields
    sPhysPair                 TheMultiplier;
    real                      ThePosition;
    real                      TheGamma;
};

//--------------------------------------------------------- sGaussFunction ---

class CommonMath_EXPORT  sGaussFunction : public sAnalyticFunction
{
  STORABLE(sGaussFunction)
  public:
    virtual                   ~sGaussFunction ()
                                {}
                              sGaussFunction (rcsPhysValue  position
                                               ,rcsPhysValue  width   );

  public:
    virtual psMathValue       Replica () const;
    virtual sBoundaries       Boundaries () const;
    virtual sPhysRange        AbscissaRange () const;
    virtual sUnits            OrdinateUnits () const;
    virtual sPhysValue        Value (rcsPhysValue  x) const;
    sPhysValue                Position () const
                                { return ThePosition*TheMultiplier.X; }
    sPhysValue                Width () const  //!< full width at half maximum
                                { return WidthVsSigma*TheSigma*TheMultiplier.X;}
    sPhysValue                Sigma () const
                                { return TheSigma*TheMultiplier.X; }

  protected:
    static const real         WidthVsSigma; //!< =2*sqrt(2*ln(2))=2.35482
    static const real         Sqrt2pi; //!< =sqrt(2*pi)

  protected://fields
    sPhysPair                 TheMultiplier;
    real                      ThePosition;
    real                      TheSigma;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
