////////////////////////////////////////////////////////////////////////////////
/*! @file DemoDevices.cpp Пакет фиктивных устройств для демонстрации.
- Part of DemoDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "DemoDevices.h"
#include "attr.h"
#include "GaugeDisplay.h"
#include "RadiofictivityGauge.h"
#include "PeriodicGaugeRecorder.h"
#include "StepperFantastic317.h"
#include "EffectorConsole.h"
#include "Symbols.h"
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sDemoDevices)

//------------------------------------------------------------------------------

sVersion  sDemoDevices::Version ()
{
  return sVersion(RAP_VERS_MAJOR
                 ,RAP_VERS_MINOR
                 ,ZAKAVYCH(RAP_VERS_PATCH)
                 ,__DATE__
                 );
}


sVersion  sDemoDevices::CoreVersionUsed ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}


sVersion  sDemoDevices::CoreVersionMin ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}


QString  sDemoDevices::UserFriendlyName ()
{
  return trUtf8("Demo devices");
}


QString  sDemoDevices::About ()
{
  return trUtf8(
      "The package includes emulators of fictive devices.  Their work does not "
      "require any real hardware -- the data are generated progammatically "
      "with reasonable formulae and random noise.\n\n"
      "Copyright (C) 2010, Viktor E. Bursian, St.Petersburg, Russia.\n"
      "                    Viktor.Bursian@mail.ioffe.ru"
                );
}


void  sDemoDevices::InstallDefaults (sPtr<sFolderNode>  DeviceList)
{
  try{
    sPtr<sFolderNode>              DeviceCfg;

    DeviceList |= "Radiofictivity Gauge (broken)";
    DeviceCfg = DeviceList >> "Radiofictivity Gauge (broken)";
    DeviceCfg << attr("Class","sRadiofictivityGauge");
    sRadiofictivityGauge::Preconfigure(DeviceCfg);

    DeviceList |= "Radiofictivity Gauge (good one)";
    DeviceCfg = DeviceList >> "Radiofictivity Gauge (good one)";
    DeviceCfg << attr("Class","sRadiofictivityGauge");
    sRadiofictivityGauge::Preconfigure(DeviceCfg);

    DeviceList |= "Radiofictivity Indicator";
    DeviceCfg = DeviceList >> "Radiofictivity Indicator";
    DeviceCfg << attr("Class","sGaugeDisplay");
    sGaugeDisplay::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Gauge"
        << attr("DeviceId","Radiofictivity Gauge (broken)");

    DeviceList |= "Radiofictivity Recorder";
    DeviceCfg = DeviceList >> "Radiofictivity Recorder";
    DeviceCfg << attr("Class","sPeriodicGaugeRecorder");
    sPeriodicGaugeRecorder::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Gauge"
          << attr("DeviceId","Radiofictivity Gauge (good one)");
    
    DeviceList |= "ScanMotorEmulator";
    DeviceCfg = DeviceList >> "ScanMotorEmulator";
    DeviceCfg << attr("Class","sStepperFantastic317");
    sStepperFantastic317::Preconfigure(DeviceCfg);
    
    DeviceList |= "RotatorEmulator";
    DeviceCfg = DeviceList >> "RotatorEmulator";
    DeviceCfg << attr("Class","sStepperFantastic317");
    sStepperFantastic317::Preconfigure(DeviceCfg);
    DeviceCfg >> "Settings" 
        << attr("Range",sPhysRange(-real_inf,real_inf,Symbol::Degree,0.1))
        << attr("Step",sPhysValue(0.1,0.05,Symbol::Degree));
    
    DeviceList |= "Scan Motor Controller";
    DeviceCfg = DeviceList >> "Scan Motor Controller";
    DeviceCfg << attr("Class","sEffectorConsole");
    sEffectorConsole::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Effector"
        << attr("DeviceId","ScanMotorEmulator");
    
    DeviceList |= "Rotator Controller";
    DeviceCfg = DeviceList >> "Rotator Controller";
    DeviceCfg << attr("Class","sEffectorConsole");
    sEffectorConsole::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Effector"
        << attr("DeviceId","RotatorEmulator");

    /*________________________________________template for device
    DeviceList |= "__DEVICE_ID__";
    DeviceCfg = DeviceList >> "__DEVICE_ID__";
    DeviceCfg << attr("Class","__DRIVER_CLASS_NAME__");
    __DRIVER_CLASS_NAME__::Preconfigure(DeviceCfg);
    //if there are links:
    DeviceCfg >> "Links" >> "__LINK_NAME__"
        << attr("DeviceId","__DEVICE_ID__FOR_THE_LINK__");
    //if you wanna change some of the default settings:
    DeviceCfg >> "Settings" << attr("",);
    DeviceCfg >> "Settings" << attr("",);
    */
  }catch(...){
    ;
  };
}


sDemoDevices::sDemoDevices ()
{
  sUnit::Register("photon");
}

//------------------------------------------------------------------------------
} //namespace RA


//Q_EXPORT_PLUGIN2( rapDemoDevices     //dll name (as set by TARGET in .pro file)
//                , RA::sDemoDevices   //name of the class, representing
//                                     //this whole package of drivers,
//                                     //consoles, etc., as well as
//                                     //supporting them classes, functions, ...,
//                                     //whatsoever
//                )

////------------------------------------------------------------------------------
