////////////////////////////////////////////////////////////////////////////////
/*! @file DummyCCDDriver.cpp Имитатор драйвера ПЗС-линейки.
- Part of DemoDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2017, Dmitrii Nelson, St.Petersburg, Russia.
                          d.nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "DummyCCDDriver.h"
#include "attr.h"
#include <QDebug>
#include "Log.h"
#include <cstdlib>

namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sDummyCCDDriver)

//---------------------------------------------------- sDummyCCDDriver ----------


void  sDummyCCDDriver::Preconfigure (sNodePtr  DeviceCfg)
{
  sPtr<sFolderNode>              DeviceSettings;
  DeviceCfg |= "Settings";
  DeviceSettings = DeviceCfg >> "Settings";
  if( DeviceSettings.IsCorrect() ){
    DeviceSettings |= attr("NumberOfPixels",3648);
  };
  sCCDDriver::Preconfigure(DeviceCfg);
}

sDummyCCDDriver::~sDummyCCDDriver ()
{
  DataArray.SetSize(0);
  Wavelengths.SetSize(0);
}

sDummyCCDDriver::sDummyCCDDriver (sString   DevicetId
                               ,sNodePtr  DeviceCfg)
  :sCCDDriver(DevicetId,DeviceCfg)
  ,Amplitude(50000)
{
}

void  sDummyCCDDriver::ConnectUSBDevice ()
{
  qDebug() << "sDummyCCDDriver::ConnectUSBDevice";
  emit Connected();
}

void  sDummyCCDDriver::GetCCD_Data ()
{
    ushort                    NoiseMagnitude = 320;
    Amplitude = Amplitude - 1000;
    if (Amplitude < 2000) Amplitude = 50000;
    ushort Width = 1000;
    for( int i = 0 ; i < NumOfPixels ; i ++ ){
        DataArray.ArrBody[i] =
            Amplitude/(1+(i-NumOfPixels/2)*(i-NumOfPixels/2)/Width)
            +Round(std::rand()*real(NoiseMagnitude)/RAND_MAX);
    }
    emit NewData(DataArray);
}

void  sDummyCCDDriver::CalculateWavelengths ()
{
  double dX = (1000.0 - 200.0)/NumOfPixels; // in "nm/pixel"
  for (int i = 0; i < NumOfPixels; i++) {
    Wavelengths.ArrBody[i] = 200.0 + dX * i ;
  }
  qDebug() << "sDummyCCDDriver::CalculateWavelengths";
  emit NewWavelengths (Wavelengths);
}

void  sDummyCCDDriver::StartCCD ()
{
  try{
    int _IT = Round((Parameters.IntegrationTime/(sPhysValue(1.0e-3,"s"))).ToReal());
    if (_IT < 1) _IT = 1;
    State_O_Measuring->SetTimeout(_IT);
  }catch(rxRAlgebra){
    SetInvalid(tr("Something wrong with units."));
  }
}

void  sDummyCCDDriver::StopCCD ()
{
}

void  sDummyCCDDriver::SetCCD_Parameters (sCCDDriver::sParameters Params)
{
  Parameters =  Params;
  emit ParamsSet();
}


//------------------------------------------------------------------------------
} //namespace RA
