////////////////////////////////////////////////////////////////////////////////
/*! @file RAlgebraFrame.cpp Widget, showing a simple math over physvalues.
- Part of DemoDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.8  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2014, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "RAlgebraFrame.h"
#include <QVBoxLayout>
#include <QHBoxLayout>
#include "flowlayout.h"
namespace RA {
//------------------------------------------------------------------------------

//DEFINE_CLASS_TAG(sRAlgebraFrame)

//----------------------------------------------------------- sRAlgebraFrame ---

sRAlgebraFrame::sRAlgebraFrame (QWidget *  parent)
    :QFrame(parent)
{
  R1 = new sPhysValueField(this,"R<sub>1</sub>"
                     ,sPhysValue(3.1415926,"m*s-2")
                     ,false,true,false);

  R2 = new sPhysValueField(this,"R<sub>2</sub>"
                     ,sPhysValue("m*s-2")
                     ,true,true);

  R3 = new sPhysValueField(this,"R<sub>3</sub>"
                    ,sPhysValue(log(-0.0))
                    ,true,true,true);

  R4 = new sPhysValueField(this,"R<sub>4</sub>"
//                     ,sPhysValue(-log(0.0)*0.0)
                     ,sPhysValue(1.0/R3->Value().ToReal())
                     ,true,false,true);

  Limits12 = new sPhysRangeField(this ,sPhysRange("m*s-2") ,true);
  Limits3 = new sPhysRangeField(this ,sPhysRange() ,true);
  Limits4 = new sPhysRangeField(this ,sPhysRange(0,1,"m") ,false);

  Limits12->setEnabled(false);
  Limits3->setEnabled(false);
  Limits4->setStyleSheet("QLineEdit {color: pink} QLabel {color: pink}");
  
  QBoxLayout *       Layout = new QVBoxLayout;
  setLayout(Layout);
  
  //
  {
    QLayout *       L = new FlowLayout(0,30,3);
    layout()->addItem(L);
    L->addWidget(R1);
    L->addWidget(R2);
    L->addWidget(R3);
    L->addWidget(R4);
  }
  //
  {
    QBoxLayout *       L = new QHBoxLayout();
    L->setSpacing(3);
    layout()->addItem(L);
    L->addWidget( chkbxLimits12 = new QCheckBox(this) );
    L->addWidget(Limits12);
    L->addStretch(1);
  }
  //
  {
    QBoxLayout *       L = new QHBoxLayout();
    layout()->addItem(L);
    chkbxLimits3 = new QCheckBox(this);
    L->addWidget(chkbxLimits3);
    L->addWidget(Limits3);
    L->addStretch(1);
  }
  //
  {
    QBoxLayout *       L = new QHBoxLayout();
    layout()->addItem(L);
    chkbxLimits4 = new QCheckBox(this);
    L->addWidget(chkbxLimits4);
    L->addWidget(Limits4);
  }

  Expr = new QTextEdit(this);
  Expr->setEnabled(false);
  Layout->addWidget(Expr);

  connect(R1,SIGNAL(Edited()),this,SLOT(UpdateExpr()));
  connect(R2,SIGNAL(Edited()),this,SLOT(UpdateExpr()));
  connect(R3,SIGNAL(Edited()),this,SLOT(UpdateExpr()));
  connect(R4,SIGNAL(Edited()),this,SLOT(UpdateExpr()));
  connect(Limits12,SIGNAL(Edited()),this,SLOT(ApplyLimits12()));
  connect(Limits3,SIGNAL(Edited()),this,SLOT(ApplyLimits3()));
  connect(chkbxLimits12,SIGNAL(toggled(bool)),Limits12,SLOT(setEnabled(bool)));
  connect(chkbxLimits3,SIGNAL(toggled(bool)),Limits3,SLOT(setEnabled(bool)));
  connect(chkbxLimits4,SIGNAL(toggled(bool)),Limits4,SLOT(setEnabled(bool)));
  connect(chkbxLimits12,SIGNAL(clicked()),this,SLOT(ApplyLimits12()));
  connect(chkbxLimits3,SIGNAL(clicked()),this,SLOT(ApplyLimits3()));
  connect(chkbxLimits4,SIGNAL(clicked()),this,SLOT(ApplyLimits4()));
  
  setStyleSheet("font-size: 14px; font-family: \"DejaVu Sans\"; "
                "QFrame {color: blue; background: lightyellow}"
                "QLineEdit {color: navy}"
               );
//  setStyleSheet("color: blue");//changes color of all sub-widgets, 
//                               //including menu
  

  UpdateExpr();
}


void  sRAlgebraFrame::Translate ()
{
  QString  LimitsLabel(tr("Limits for"));
  Limits12->SetLabel(LimitsLabel+" R<sub>1</sub> & R<sub>2</sub>");
  Limits3->SetLabel(LimitsLabel+" R<sub>3</sub>");
  Limits4->SetLabel(LimitsLabel+" R<sub>4</sub>");
}


void  sRAlgebraFrame::UpdateExpr ()
{
  sString                     T;
  eTextDetalization           D = Casual;
  T+="R<sub>1</sub>+R<sub>2</sub>=";
  try{
    T+=(R1->Value()+R2->Value()).Text(HTML,D);
  }catch(...){
    T+="error";
  }
  T+=";<p>";
  T+="R<sub>1</sub>-R<sub>2</sub>=";
  try{
    T+=(R1->Value()-R2->Value()).Text(HTML,D);
  }catch(...){
    T+="error";
  }
  T+=";<p>";
  T+="R<sub>3</sub>*R<sub>4</sub>=";
  try{
    T+=(R3->Value()*R4->Value()).Text(HTML,D);
  }catch(...){
    T+="error";
  }
  T+=";<p>";
  T+="R<sub>3</sub>/R<sub>4</sub>=";
  try{
    T+=(R3->Value()/R4->Value()).Text(HTML,D);
  }catch(...){
    T+="error";
  }
  Expr->setText( ToQString( T ) );
}


void  sRAlgebraFrame::ApplyLimits12 ()
{
  if( chkbxLimits12->isChecked() ){
    R1->SetValidRange(Limits12->Value());
    R2->SetValidRange(Limits12->Value());
  }
}


void  sRAlgebraFrame::ApplyLimits3 ()
{
  if( chkbxLimits3->isChecked() ){
    R3->SetValidRange(Limits3->Value());
  }
}


void  sRAlgebraFrame::ApplyLimits4 ()
{
  if( chkbxLimits4->isChecked() ){
    Limits4->setStyleSheet(QString());
    R4->SetValidRange(Limits4->Value());
  }else{
    Limits4->setStyleSheet("QLineEdit {color: pink} QLabel {color: pink}");
  }
}

//------------------------------------------------------------------------------
} //namespace RA
