////////////////////////////////////////////////////////////////////////////////
/*! @file RadiofictivityGauge.h   Эмулятор счётчика Гейгера.
- Part of DemoDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2010-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef RadiofictivityGauge_H
#define RadiofictivityGauge_H
#include "DemoDevices.h"
#include "Gauge.h"
#include <QTimer>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sRadiofictivityGauge)

//----------------------------------------------------- sRadiofictivityGauge ---
/*! Драйвер-эмулятор некоего вымышленного счётчика Гейгера,
    расположенного около быстро распадающегося радиоактивного источника
    (демонстрирует двустадийный "прогрев" с возможной неудачей,
    а также некую глючность в процессе измерений).

На первой стадия прогрева просто ждёт 5 секунд.
В начале второй стадии решает, есть ли у него настроение поработать.
Если в идентификаторе экземпляра устройства есть слово "broken", то "рабочее
настроение" возникает с вероятностью 1/2. Эта глючность введена намеренно.

Если настроение есть, то счётчик переходит в состояние Operating.
Если нет, то переходит в состояние Frozen.
Использующие его инструменты должны учитывать обе возможности.

По команде Start запускает непрерывный цикл измерений и на каждый отсчёт выдаёт
сигнал NewData с аргументом, равным измеренному значению.

@bug Имеет пикантную особенность зависать в процессе измерений: если в
идентификаторе экземпляра устройства есть слово "broken", то с вероятностью
1/1000 данные перестают поступать (вырубается Clock) - типа контакт
отвалился :-). В этой версии драйвер не обрабатывает такую абсдачу, хотя должен
бы. "Недоработка" оставлена умышленно для проверки того, как будут "ломаться"
использующие его инструменты (они и "ломаются", но не критично).

Создаваемая по умолчанию конфигурация устройств содержит два экземпляра
счётчиков типа sRadiofictivityGauge с идентификаторами
"Radiofictivity Gauge (broken)" и "Radiofictivity Gauge (good one)".
Они подключены к разным приборам, соответственно,
к "Radiofictivity Indicator" и к "Radiofictivity Recorder".
Можно переподключить их, или к обоим приборам подключить неглючный
"Radiofictivity Gauge (good one)".

@par The state machine
@dotfile "RadiofictivityGauge.state_graph"

*/
class  DemoDevices_EXPORT  sRadiofictivityGauge : public sGauge
{
  Q_OBJECT
  DRIVER(sRadiofictivityGauge)

  public: //commands & directives
    static const
      sDirective<sPhysValue>  cCheat;
    static const
      sDirective<QString>     cSay;

  public:
    virtual                   ~sRadiofictivityGauge ();

  protected: //evolution of the inherited state machine
    psState                   State_W_Pending;
    psState                   State_W_Check;
    psState                   State_C_Pausing;
    QTimer *                  Clock;

  signals: //outgoing
    void                      Say (QString);

  signals: //for internal needs
    void                      NotInTheMood ();
    void                      OK ();
    void                      ClockSignal ();
    void                      CoolingClockAlarm ();

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  protected slots: //reimplementing inherited
    virtual void              SetWantedPeriod ();

  protected slots:
    void                      RunCoolingClock ();

  private slots:
    void                      SetUpThePendingTimer ();
    void                      CheckMood ();
    void                      RunTheClock ();
    void                      ExtractData ();


  private: //fields
    int                       Interval;//ms

    //нижеследующее не будет иметь аналога в реальных устройствах
    //это добавлено для того, чтобы заставить счётчик показывать нечто,
    //приятное глазу (в реальном случае об этом позаботится Природа)
    sPhysValue                CurrentActivity;
    sPhysValue                DecayTau;
    real                      noise;
    bool                      i_am_lasy ();
    bool                      shit_happens ();
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
