////////////////////////////////////////////////////////////////////////////////
/*! @file SpectroFiction.cpp  Продвинутый эмулятор спектрометра.
- Part of FakeDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "SpectroFiction.h"
#include "attr.h"
#include "GeneralGUI.h"
#include <QMessageBox>
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sSpectroFiction)

//---------------------------------------------------------- sSpectroFiction ---

sSpectroFiction::~sSpectroFiction()
{
  if( PhotonCounter ){
    delete PhotonCounter;  PhotonCounter=NULL;
  };
  if( WavelengthController && WavelengthControllerIsMine ){
    delete WavelengthController;  WavelengthController=NULL;
  };
};


sSpectroFiction::sSpectroFiction (sString   DeviceId
                                 ,sNodePtr  DeviceCfg
                                 ,sNodePtr  WidgetCfg)
    :sUniScanner(DeviceId,DeviceCfg,WidgetCfg)
    ,PhotonCounter(NULL)
    ,WavelengthController(NULL)
    ,WavelengthControllerIsMine(false)
{
//  ColorSet.Clear();
//  ColorSet.Add(sColor::cRed).Add(sColor::cGreen).Add(sColor::cBlue);


  LblLine1 = new QLabel(tr("<u>First line:</u>"),ParamArea);
  LblLine1->setObjectName("LblLine1");
  ParamLayout->addWidget(LblLine1,Row++,0,1,2);

  LblPos1 = new QLabel(tr("position"),ParamArea);
  LblPos1->setObjectName("LblPos1");
  ParamLayout->addWidget(LblPos1,Row,0,1,1);
  FldPos1 = new QDoubleSpinBox(ParamArea);
  FldPos1->setObjectName("FldPos1");
  FldPos1->setRange(400.0,800.0);
  FldPos1->setValue(550.0);
  ParamLayout->addWidget(FldPos1,Row++,1,1,1);

  LblHalfWidth1 = new QLabel(tr("width"),ParamArea);
  LblHalfWidth1->setObjectName("LblHalfWidth1");
  ParamLayout->addWidget(LblHalfWidth1,Row,0,1,1);
  FldHalfWidth1 = new QDoubleSpinBox(ParamArea);
  FldHalfWidth1->setObjectName("FldHalfWidth1");
  FldHalfWidth1->setRange(0.1,400.0);
  FldHalfWidth1->setValue(2.0);
  ParamLayout->addWidget(FldHalfWidth1,Row++,1,1,1);

  LblAmplitude1 = new QLabel(tr("amplitude"),ParamArea);
  LblAmplitude1->setObjectName("LblAmplitude1");
  ParamLayout->addWidget(LblAmplitude1,Row,0,1,1);
  FldAmplitude1 = new QDoubleSpinBox(ParamArea);
  FldAmplitude1->setObjectName("FldAmplitude1");
  FldAmplitude1->setRange(0.0,1e9);
  FldAmplitude1->setValue(1e4);
  ParamLayout->addWidget(FldAmplitude1,Row++,1,1,1);

  LblLine2 = new QLabel(tr("<u>Second line:</u>"),ParamArea);
  LblLine2->setObjectName("LblLine2");
  ParamLayout->addWidget(LblLine2,Row++,0,1,2);

  LblPos2 = new QLabel(tr("position"),ParamArea);
  LblPos2->setObjectName("LblPos2");
  ParamLayout->addWidget(LblPos2,Row,0,1,1);
  FldPos2 = new QDoubleSpinBox(ParamArea);
  FldPos2->setObjectName("FldPos2");
  FldPos2->setRange(400.0,800.0);
  FldPos2->setValue(450.0);
  ParamLayout->addWidget(FldPos2,Row++,1,1,1);

  LblHalfWidth2 = new QLabel(tr("width"),ParamArea);
  LblHalfWidth2->setObjectName("LblHalfWidth2");
  ParamLayout->addWidget(LblHalfWidth2,Row,0,1,1);
  FldHalfWidth2 = new QDoubleSpinBox(ParamArea);
  FldHalfWidth2->setObjectName("FldHalfWidth2");
  FldHalfWidth2->setRange(0.1,400.0);
  FldHalfWidth2->setValue(50.0);
  ParamLayout->addWidget(FldHalfWidth2,Row++,1,1,1);

  LblAmplitude2 = new QLabel(tr("amplitude"),ParamArea);
  LblAmplitude2->setObjectName("LblAmplitude2");
  ParamLayout->addWidget(LblAmplitude2,Row,0,1,1);
  FldAmplitude2 = new QDoubleSpinBox(ParamArea);
  FldAmplitude2->setObjectName("FldAmplitude2");
  FldAmplitude2->setRange(0.0,1e9);
  FldAmplitude2->setValue(3e3);
  ParamLayout->addWidget(FldAmplitude2,Row++,1,1,1);

  DeviceConfig |= attr("HardwareDriver","SF300");
  sPtr<sTextNode>             HardwareDriverId(DeviceConfig >> "Counter");
  if( HardwareDriverId.IsCorrect() ){
    PhotonCounter = dynamic_cast<psCounterFantastic51>
                         (sDevice::CreateDevice(sString(*HardwareDriverId)));
    if( PhotonCounter ){
      connect(PhotonCounter,SIGNAL(NewData(int))
             ,this         ,SLOT(OnNewData(int)));
      connect(PhotonCounter,SIGNAL(NowOffline())
             ,this         ,SLOT(ReportCounterError()));
      connect(PhotonCounter,SIGNAL(NowStandby())
             ,this         ,SLOT(LinkToWLController()));
    };
  };
  connect(this,SIGNAL(ScanIsStarting()) ,this,SLOT(OnStartScan()));
  connect(this,SIGNAL(ScanIsFinished()) ,this,SLOT(OnFinishScan()));
  connect(this,SIGNAL(Accepted()) ,this,SLOT(OnAccept()));
};


void  sSpectroFiction::RestoreState ()
{
  sUniScanner::RestoreState();
  if( ! WidgetConfig.IsNULL() ){
  };
};


void  sSpectroFiction::StoreState ()
{
  sUniScanner::StoreState();
  if( ! WidgetConfig.IsNULL() ){
  };
};


void  sSpectroFiction::Connect ()
{
  if( DeviceState() == Offline ){
    if( PhotonCounter ){
      SetDeviceState(Warming);
      PhotonCounter->Connect();
    }else{
      QMessageBox::critical(this
                       ,sString2QString(DeviceIdentifier()+" failure.")
                       ,tr("The counter has not been found. Check config.") );
    };
  };
};


//void  sSpectroFiction::Disconnect ()
//{
//};


void  sSpectroFiction::ReportCounterError ()
{
  QMessageBox::critical(this
                       ,sString2QString(DeviceIdentifier()+" failure.")
                       ,tr("The counter '")
                       +sString2QString(PhotonCounter->DeviceIdentifier())
                       +tr("' is not in the mood.  Try later.")
                       );
  SetDeviceState(Offline);
};


void  sSpectroFiction::LinkToWLController ()
{
  sPtr<sTextNode>             HardwareDriverId;
  HardwareDriverId = DeviceConfig >> "WavelengthController";
  if( ! HardwareDriverId.IsCorrect() ){
    QMessageBox::critical(this
        ,sString2QString(DeviceIdentifier()+" failure.")
        ,tr("Wrong configuration.") );
    PhotonCounter->Disconnect();
    SetDeviceState(Offline);
    return;
  };
  WavelengthController = dynamic_cast<psStepperFantastic314>
                       (sDevice::FindCreatedDevice(sString(*HardwareDriverId)));
  if( ! WavelengthController ){
    QMessageBox::critical(this
        ,sString2QString(DeviceIdentifier()+" failure.")
        ,tr("The wavelength controller has not been found. "
            "Launch it manually, please, and re-connect ")
        +sString2QString(DeviceIdentifier()) );
    PhotonCounter->Disconnect();
    SetDeviceState(Offline);
    return;
  };
  connect(WavelengthController,SIGNAL(NowOffline())
         ,this                ,SLOT(ReportControllerError()));
  connect(WavelengthController,SIGNAL(NowStandby())
         ,this                ,SLOT(EnterStandby()));
  if( WavelengthController->DeviceState() == Standby ){
    EnterStandby();
  }else{
    WavelengthController->Connect();
  };
};


void  sSpectroFiction::ReportControllerError ()
{
  if( DeviceState() == Warming ){
    QMessageBox::critical(this
        ,sString2QString(DeviceIdentifier()+" failure.")
        ,tr("Cannot connect to the wavelength controller '")
        +sString2QString(PhotonCounter->DeviceIdentifier())
        +tr("'.")
        );
    disconnect(WavelengthController,SIGNAL(NowOffline())
              ,this                ,SLOT(ReportControllerError()));
    disconnect(WavelengthController,SIGNAL(NowStandby())
              ,this                ,SLOT(EnterStandby()));
  }else{
    QMessageBox::critical(this
        ,sString2QString(DeviceIdentifier()+" failure.")
        ,tr("The wavelength controller '")
        +sString2QString(WavelengthController->DeviceIdentifier())
        +tr("' suddenly goes off-line.")
        );
    disconnect(WavelengthController,SIGNAL(NowOffline())
              ,this                ,SLOT(ReportControllerError()));
  };
  PhotonCounter->Disconnect();
  SetDeviceState(Offline);
};


void  sSpectroFiction::EnterStandby ()
{
  disconnect(WavelengthController,SIGNAL(NowStandby())
            ,this                ,SLOT(EnterStandby()));
  SetDeviceState(Standby);
};


void  sSpectroFiction::OnStartScan ()
{
  Spectrum |= "Lines";
  Spectrum>>"Lines" |= 1;
  Spectrum>>"Lines">>1 << attr("position",(real)(FldPos1->value()));
  Spectrum>>"Lines">>1 << attr("halfwidth",(real)(FldHalfWidth1->value()));
  Spectrum>>"Lines">>1 << attr("amplitude",(real)(FldAmplitude1->value()));
  Spectrum>>"Lines" |= 2;
  Spectrum>>"Lines">>2 << attr("position",(real)(FldPos2->value()));
  Spectrum>>"Lines">>2 << attr("halfwidth",(real)(FldHalfWidth2->value()));
  Spectrum>>"Lines">>2 << attr("amplitude",(real)(FldAmplitude2->value()));

  WavelengthController->setEnabled(false);
  connect(WavelengthController ,SIGNAL(NowStandby())
         ,this                 ,SLOT(OnWavelengthReached()));
  NextWavelength=From;
  MoveToNextWavelength();
};


void  sSpectroFiction::MoveToNextWavelength ()
{
  WavelengthController->GoTo(NextWavelength);
  NextWavelength += Step;
};


void  sSpectroFiction::OnWavelengthReached ()
{
  if( Scanning ){
    SecretelyTellTheCounterWhatToMeasure();
    PhotonCounter->Start(100);
  };
};


void  sSpectroFiction::OnNewData (int Counts)
{
  PhotonCounter->Stop();
  AddNewPoint(Counts);
  if( Scanning )
    MoveToNextWavelength();
};


void  sSpectroFiction::OnFinishScan ()
{
  PhotonCounter->Stop();
  disconnect(WavelengthController ,SIGNAL(NowStandby())
            ,this                 ,SLOT(OnWavelengthReached()));
  WavelengthController->setEnabled(true);
};


void  sSpectroFiction::OnAccept ()
{
  FldPos1->setValue(FldPos1->value()*1.01);
  FldHalfWidth1->setValue(FldHalfWidth1->value()*1.05);
  FldAmplitude1->setValue(FldAmplitude1->value()/1.05);
  FldPos2->setValue(FldPos2->value()*1.03);
  FldHalfWidth2->setValue(FldHalfWidth2->value()/1.05);
  FldAmplitude2->setValue(FldAmplitude2->value()*1.05);
};


void  sSpectroFiction::SecretelyTellTheCounterWhatToMeasure ()
{
  real                        X = WavelengthController->CurrentWavelength();
  real                        Value;
  Value=  FldAmplitude1->value()
           / ( 1 + (X-FldPos1->value())*(X-FldPos1->value())
                  /FldHalfWidth1->value()/FldHalfWidth1->value())
        + FldAmplitude2->value()
           / ( 1 + (X-FldPos2->value())*(X-FldPos2->value())
                  /FldHalfWidth2->value()/FldHalfWidth2->value())  ;
  PhotonCounter->relaxor=Value;
};

//------------------------------------------------------------------------------
}; //namespace RA
