////////////////////////////////////////////////////////////////////////////////
/*! @file SpectroFiction.h   Продвинутый эмулятор спектрометра.
- Part of FakeDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef SpectroFiction_H
#define SpectroFiction_H
#include "ClassicMultiScanner.h"
#include "CounterFantastic51.h"
#include "StepperFantastic314.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sSpectroFiction)

//---------------------------------------------------------- sSpectroFiction ---
/*! Консоль (пользовательский интерфейс) продвинутого эмулятора спектрометра.


*/
class sSpectroFiction : public sClassicMultiScanner
{
  Q_OBJECT
  INSTRUMENT(sSpectroFiction)
  public:
    virtual                   ~sSpectroFiction ();
    virtual void              RestoreState ();
    virtual void              StoreState ();

  public slots:
    virtual void              Connect ();
//    virtual void              Disconnect ();

  private slots:
    void                      LinkToWLController ();
    void                      ReportCounterError ();
    void                      EnterStandby ();
    void                      ReportControllerError ();
    void                      OnStartScan ();
    void                      OnFinishScan ();
    void                      OnAccept ();
    void                      OnWavelengthReached ();
    void                      OnNewData (int PhotonCount);

  private:
    void                      MoveToNextWavelength ();
    void                      SecretelyTellTheCounterWhatToMeasure ();

  private: //fields
    psCounterFantastic51      PhotonCounter;
    psStepperFantastic314     WavelengthController;
    bool                      WavelengthControllerIsMine;
    real                      NextWavelength;
    QLabel *                  LblLine1;
    QLabel *                  LblPos1;
    QDoubleSpinBox *          FldPos1;
    QLabel *                  LblHalfWidth1;
    QDoubleSpinBox *          FldHalfWidth1;
    QLabel *                  LblAmplitude1;
    QDoubleSpinBox *          FldAmplitude1;
    QLabel *                  LblLine2;
    QLabel *                  LblPos2;
    QDoubleSpinBox *          FldPos2;
    QLabel *                  LblHalfWidth2;
    QDoubleSpinBox *          FldHalfWidth2;
    QLabel *                  LblAmplitude2;
    QDoubleSpinBox *          FldAmplitude2;
};

//------------------------------------------------------------------------------
}; //namespace RA
#endif // SpectroFiction_H
