////////////////////////////////////////////////////////////////////////////////
/*! @file StepperFantastic317.h   Эмулятор устойства управления развёрткой
                                  спектрометра с шаговым двигателем.
- Part of DemoDevices - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-14, Viktor E. Bursian, St.Petersburg, Russia.
                        Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef StepperFantastic317_H
#define StepperFantastic317_H
#include "DemoDevices.h"
#include "Effectors.h"
#include <QSoundEffect>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sStepperFantastic317)

//----------------------------------------------------- sStepperFantastic317 ---
/*! Драйвер-эмулятор устойства управления развёрткой
    спектрометра с шаговым двигателем.
*/
class  DemoDevices_EXPORT  sStepperFantastic317 : public sStepper
{
  Q_OBJECT
  DRIVER(sStepperFantastic317)
  
  public:
    virtual                   ~sStepperFantastic317 ();
  
  public: //implementing pure virtual
    virtual sPhysValue        Position (int steps) const;
    virtual int               Steps (sPhysValue  position) const;

  protected: //evolution of the inherited state machine
    psParallelStateGroup      State_O_Parallel;
    psState                   State_OP_Measuring;
    psState                   State_OP_Control;
    psState                   State_OPM_Hanging;
    psState                   State_OPC_Idle;
    psState                   State_OPC_SettingTo;
    psState                   State_OPC_SettingTo_Check;
    psState                   State_OPC_SettingTo_Zh;
    psState                   State_OPC_MovingBy;
    psState                   State_OPC_MovingBy_MakeStep;

  protected slots: //reimplementing inherited
    virtual void              SetWantedPeriod ();

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  signals: //for internal needs
    void                      SettingDone ();
    void                      MovingDone ();

  protected:
    sPhysValue                Step;

  private slots:
    void                      ReportPosition ();
    void                      SetDestination (sPhysValue);
    void                      SetScanTimePerStep (sPhysValue  time_per_step);
    void                      SayDone();
    void                      CalculateShift ();
    void                      SayZh ();
    void                      ShiftPosition ();
    void                      SetNumberOfStepsToMoveBy (int  num_of_steps);
    void                      DoStep ();

  private: //static constants
    static const int          MaxShift;

  private: //fields
    sPhysValue                StepTimeQuantum;
    int                       CurrentIndex;
    int                       Destination;
    int                       Shift;
    sPhysValue                StepTime;
    QSoundEffect              Zhzhzh;
    QSoundEffect              Schelk;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif 
