////////////////////////////////////////////////////////////////////////////////
/*! @file CardioFiction.cpp  Kinda cardograph.
- Part of DemoMicrosin - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2010-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "CardioFiction.h"
//#include "attr.h"
//#include "StringsAndQStrings.h"
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sCardioFiction)

/*! Директива, вынуждающая датчик однократно выдать ложный отсчёт
    (attached data: значение ложного отсчёта).
*/
const sDirective<sPhysValue>  sCardioFiction::cCheat("cCheat");

/*! Директива, вынуждающая датчик выдать сигнал Say(QString) и остановиться
    (attached data: what to say).
*/
const sDirective<QString>     sCardioFiction::cSay("cSay");

//----------------------------------------------------- sCardioFiction ---

void  sCardioFiction::Preconfigure (sNodePtr  DeviceCfg)
{
  sPtr<sFolderNode>              DeviceSettings;
  DeviceCfg |= "Settings";
  DeviceSettings = DeviceCfg >> "Settings";
  if( DeviceSettings.IsCorrect() ){
    DeviceSettings |= attr("CurrentActivity",sPhysValue()
                                            =sPhysValue(1e4,"photon/s"));
    DeviceSettings |= attr("DecayTau",sPhysValue(30.0,"s"));
    // изменение умолчаний для sGauge:
    DeviceSettings |= attr("Period",sPhysValue(0.1,"s"));
  }
  sGauge::Preconfigure(DeviceCfg);
}


sCardioFiction::~sCardioFiction ()
{
  if( DeviceSettings.IsCorrect() ){
    // Не все конфигурационные параметры следует сохранять после работы.
    // В данном случае, я хочу, чтобы "сохранялось" только это:
    DeviceSettings << attr("CurrentActivity",CurrentActivity*100);
  }
}


sCardioFiction::sCardioFiction (sString   DevicetId
                                           ,sNodePtr  DeviceCfg)
    :sGauge(DevicetId,DeviceCfg)
    ,Clock(new QTimer(this))
    ,Interval(100)
    ,CurrentActivity()
    ,DecayTau("s")
    ,noise(5e-2)
{
}


void  sCardioFiction::PostConstructor ()
{
  sGauge::PostConstructor();
  try{ //get config parameters
    get_from<sPhysValueNode>(DeviceSettings >> "CurrentActivity"
                                               ,CurrentActivity);
    get_from<sPhysValueNode>(DeviceSettings >> "DecayTau"
                                               ,DecayTau);
  }catch(...){
    SetInvalid(tr("Something wrong in configuration."));
  }
  // evolution of the inherited state machine
  {
    // evolution of the State_Warming
    {
      State_W_Pending = new sState(State_Warming());
      State_W_Check = new sState(State_Warming());
      State_W_Pending
          ->SetAsInitial()
          ->OnEnterCall( SLOT(SetUpThePendingTimer()) )
          ->AddTransition(Clock ,SIGNAL(timeout()) ,State_W_Check);
      State_W_Check
          ->OnEnterCall( SLOT(CheckMood()) )
          ->AddTransition( SIGNAL(NotInTheMood()) ,State_Frozen() )
          ->AddTransition( SIGNAL(OK()) ,State_Warming()->Fin() );
    }
    // evolution of the State_Operating
    {
      State_O_M_Acquiring
          ->OnEnterCall(SLOT(RunTheClock()))
          ->AddTransition(SIGNAL(ClockSignal()),SLOT(ExtractData())
                         ,State_O_M_Cycling)
          ->AddTransition(cCheat ,SIGNAL(NewData(sPhysValue))
                         ,State_O_M_Cycling)
          ->AddTransition(cSay ,SIGNAL(Say(QString))
                         ,State_O_M_Stopped)
          ->OnExitCall(Clock ,SLOT(stop()))
          ;
      connect(Clock,SIGNAL(timeout()),this,SIGNAL(ClockSignal()));
    }
    //evolution of the State_Cooling
    {
      State_C_Pausing = new sState(State_Cooling());
      State_C_Pausing
          ->SetAsInitial()
          ->OnEnterCall(SLOT(RunCoolingClock()))
          ->AddTransition( SIGNAL(CoolingClockAlarm()) ,State_Cooling()->Fin());
    }
  }
}

//------------------------------------------------------------------------------
} //namespace RA
