////////////////////////////////////////////////////////////////////////////////
/*! @file DevelopmentTools.cpp Пакет инструментов для разработчика.
- Part of DevelopmentTools - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2012-2013, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "DevelopmentTools.h"
#include "attr.h"
#include "RadiofictivityGauge.h"
#include "GaugeTestboard.h"
#include "TelePipe.h"
#include "TeleType.h"
#include "TeleSpy.h"
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sDevelopmentTools)

//------------------------------------------------------------------------------

sVersion  sDevelopmentTools::Version ()
{
  return sVersion(RAP_VERS_MAJOR
                 ,RAP_VERS_MINOR
                 ,ZAKAVYCH(RAP_VERS_PATCH)
                 ,__DATE__
                 );
}


sVersion  sDevelopmentTools::CoreVersionUsed ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}


sVersion  sDevelopmentTools::CoreVersionMin ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}


QString  sDevelopmentTools::UserFriendlyName ()
{
  return trUtf8("DevelopmentTools");
}


QString  sDevelopmentTools::About ()
{
  return trUtf8(
      "The package contains some instruments, which were designed "
      "with the aim of testing both: software and hardware.\n\n"
      "Copyright (C) 2012, Viktor E. Bursian, St.Petersburg, Russia.\n"
      "                    Viktor.Bursian@mail.ioffe.ru"
                );
}


void  sDevelopmentTools::InstallDefaults (sPtr<sFolderNode>  DeviceList)
{
  try{
    sPtr<sFolderNode>              DeviceCfg;

    DeviceList |= "Radiofictivity Gauge (slow and broken)";
    DeviceCfg = DeviceList >> "Radiofictivity Gauge (slow and broken)";
    DeviceCfg << attr("Class","sRadiofictivityGauge");
    sRadiofictivityGauge::Preconfigure(DeviceCfg);
    DeviceCfg >> "Settings" << attr("Period",sPhysValue(2.0,"s"));

    DeviceList |= "Gauge Testboard 1";
    DeviceCfg = DeviceList >> "Gauge Testboard 1";
    DeviceCfg << attr("Class","sGaugeTestboard");
    sGaugeTestboard::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Gauge"
        << attr("DeviceId","Radiofictivity Gauge (slow and broken)");

    DeviceList |= "Gauge Testboard 2";
    DeviceCfg = DeviceList >> "Gauge Testboard 2";
    DeviceCfg << attr("Class","sGaugeTestboard");
    sGaugeTestboard::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Gauge"
        << attr("DeviceId","Radiofictivity Gauge (slow and broken)");

    DeviceList |= "Pipe1_A";
    DeviceCfg = DeviceList >> "Pipe1_A";
    DeviceCfg << attr("Class","sTelePipe");
    sTelePipe::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Mate"
        << attr("DeviceId","Pipe1_B");

    DeviceList |= "Pipe1_B";
    DeviceCfg = DeviceList >> "Pipe1_B";
    DeviceCfg << attr("Class","sTelePipe");
    sTelePipe::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Mate"
        << attr("DeviceId","Pipe1_A");

    DeviceList |= "Pipe2_A";
    DeviceCfg = DeviceList >> "Pipe2_A";
    DeviceCfg << attr("Class","sTelePipe");
    sTelePipe::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Mate"
        << attr("DeviceId","Pipe2_B");

    DeviceList |= "Pipe2_B";
    DeviceCfg = DeviceList >> "Pipe2_B";
    DeviceCfg << attr("Class","sTelePipe");
    sTelePipe::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Mate"
        << attr("DeviceId","Pipe2_A");

    DeviceList |= "Teletype1";
    DeviceCfg = DeviceList >> "Teletype1";
    DeviceCfg << attr("Class","sTeleType");
    sTeleType::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "TeleCom"
        << attr("DeviceId","Pipe1_A");

    DeviceList |= "Teletype2";
    DeviceCfg = DeviceList >> "Teletype2";
    DeviceCfg << attr("Class","sTeleType");
    sTeleType::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "TeleCom"
        << attr("DeviceId","Pipe2_B");

    DeviceList |= "Spy";
    DeviceCfg = DeviceList >> "Spy";
    DeviceCfg << attr("Class","sTeleSpy");
    sTeleSpy::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "LeftTeleCom"
        << attr("DeviceId","Pipe1_B");
    DeviceCfg >> "Links" >> "RightTeleCom"
        << attr("DeviceId","Pipe2_A");

    /*________________________________________template for device
    DeviceList |= "__DEVICE_ID__";
    DeviceCfg = DeviceList >> "__DEVICE_ID__";
    DeviceCfg << attr("Class","__DRIVER_CLASS_NAME__");
    __DRIVER_CLASS_NAME__::Preconfigure(DeviceCfg);
    //if there are links:
    DeviceCfg >> "Links" >> "__LINK_NAME__"
        << attr("DeviceId","__DEVICE_ID__FOR_THE_LINK__");
    //if you wanna change some of the default settings:
    DeviceCfg >> "Settings" << attr("",);
    DeviceCfg >> "Settings" << attr("",);
    */
  }catch(...){
    ;
  };
}


sDevelopmentTools::sDevelopmentTools ()
{
}

//------------------------------------------------------------------------------
} //namespace RA
