////////////////////////////////////////////////////////////////////////////////
/*! @file GaugeTestboard.h   A testboard for any gauge (sGauge inheritor).
- Part of DevelopmentTools - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2012-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef GaugeTestboard_H
#define GaugeTestboard_H
#include "DevelopmentTools.h"
#include "GaugeDisplay.h"
#include "PhysValueField.h"
#include <QCheckBox>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sGaugeTestboard)

//---------------------------------------------------------- sGaugeTestboard ---
/*! A testboard for any gauge (sGauge inheritor).

Сделан для тестирования и демонстрации взаимодействия консоли с глючным
драйвером, а также нескольких одновременно запущенных консолей с одним общим
для них драйвером.

Можно использовать как пример, в частности, того как брать драйвер под
эксклюзивный контроль.

Кроме того, классы sGaugeDisplay и sGaugeTestboard дают пример того,
как грамотно разделять функциональность и внешний вид, и как расширить внешний
вид потомка.

@see sGaugeLCD, который полностью подменяет внешний вид sGaugeDisplay.

@par The state machine
@dotfile "GaugeTestboard.state_graph"

*/
class  DevelopmentTools_EXPORT  sGaugeTestboard : public sGaugeDisplay
{
  Q_OBJECT
  CONSOLE(sGaugeTestboard)

  protected:
    virtual void              AppearanceMakeUp ();

  protected: //evolution of the inherited state machine
    psState                   State_O_Watching;
    psState                   State_O_Controling;

  signals: //for internal needs
    void                      Start ();
    void                      Stop ();
    void                      StartExclusive ();
    void                      ControlTaken ();
    void                      ControlReleased ();

  protected slots: //reimplementing inherited
    virtual void              OnEnterToOperatingState ();
    virtual void              OnExitFromOperatingState ();

  protected slots:
    virtual void              OnStart ();
    virtual void              OnStop ();
    virtual void              OnStartExclusive ();
    virtual void              BerateTheUser ();
    virtual void              OnPeriodFieldEdited ();
    virtual void              OnPeriodChanged (sPhysValue);
    virtual void              OnAutoRepeatCheckBoxEdited (bool);
    virtual void              OnAutoRepeatModeChanged (bool);

  protected: //fields
    psPhysValueField          PeriodField;
    QCheckBox *               AutoRepeatCheckBox;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
