////////////////////////////////////////////////////////////////////////////////
/*! @file MSDNGauge.h   A gauge, which takes readings from periodically comming
                        telexes (for instance, generated by an MSDN device).
- Part of DevelopmentTools - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2016, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef MSDNGauge_H
#define MSDNGauge_H
#include "DevelopmentTools.h"
#include "Gauge.h"
#include "TeleCom.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sMSDNGauge)

//--------------------------------------------------------------- sMSDNGauge ---
/*! A gauge, which takes readings from periodically comming
    telexes (for instance, generated by an MSDN device).
*/
class  DevelopmentTools_EXPORT  sMSDNGauge : public sGauge
{
  Q_OBJECT
  DRIVER(sMSDNGauge)

  public:
    virtual                   ~sMSDNGauge ();

  protected: //evolution of the inherited state machine
    psState                   State_W_MSDNPort;
    psState                   State_W_MSDNPort_Linking;

  signals: //for internal needs
    void                      MSDNPortLinked ();
    void                      MSDNPortError (QString = QString());
    void                      GotIncommingMessage (sTelex);
//    void                      GotData ();

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  protected slots: //reimplementing inherited
    virtual void              OnEnterToOperatingState ();
    virtual void              OnExitFromOperatingState ();
    virtual void              OnEnterToCoolingState ();
    virtual void              SetWantedPeriod ();

  protected slots:
    virtual void              ReportMSDNPortError (QString);
    virtual void              ProcessIncommingMessage (sTelex);

  protected: //fields
    sTeleCom::sLink           MSDNPort;
    char                      ReadingsPrefix;
    char                      SetPeriodPrefix;
    char                      GetPeriodPrefix;
    char                      StartCommandChar;
    char                      StopCommandChar;
    sString                   ToSendOnConnect;
    sPhysValue                Calibration;
    sPhysValue                ZeroCorrection;
    sPhysValue                PeriodConversion;

  private slots:
    void                      LinkToMSDNPort ();

  private: //fields
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
