////////////////////////////////////////////////////////////////////////////////
/*! @file NewConsole.h   A new console pattern for the developers.
- Part of Examples - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  - http://qt-project.org/
- Copyright(C) 2015, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef NewConsole_H
#define NewConsole_H
#include "Examples.h"
#include "Console.h"
#include "NewDriver.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sNewConsole)

//-------------------------------------------------------------- sNewConsole ---
/*! A new console pattern for the developers.
*/
class  Examples_EXPORT  sNewConsole : public sConsole
{
  Q_OBJECT
  CONSOLE(sNewConsole)
  public:
    virtual                   ~sNewConsole ();
    virtual void              RestoreState ();
    virtual void              StoreState ();
    virtual bool              CanClose ();

  protected: //reimplementing inherited
    virtual void              AppearanceMakeUp ();
    virtual void              Retranslate ()
                                {
                                  sConsole::Retranslate();
                                  Translate();
                                }

  protected: //evolution of the inherited state machine
    psState                   State_W_Parallel;
    psState                   State_W_Device1;
    psState                   State_W_Device1_Linking;
    psState                   State_W_Device2;
    psState                   State_W_Device2_Linking;
    psState                   State_O_Watching;
    psState                   State_O_Controling;

  signals: //for internal needs
    void                      Device1Linked ();
    void                      Device1Error(QString = QString());
    void                      Device2Linked ();
    void                      Device2Error(QString = QString());

  protected:

  protected slots: //reimplementing inherited
    virtual void              OnEnterToOperatingState ();
    virtual void              OnExitFromOperatingState ();
    virtual void              OnEnterToCoolingState ();

  protected slots: //implementing pure virtual

  protected slots:
    virtual void              ReportDevice1Error (QString);
    virtual void              ReportDevice2Error (QString);

  protected: //fields
    sNewDriver::sLink         Device1;
    sNewDriver::sLink         Device2;

  private slots:
    void                      LinkToDevice1 ();
    void                      LinkToDevice2 ();

  private:
    void                      Translate ();

  private: //fields
    integer                   IntParameter;
    sPhysValue                PhysParameter;
};


//------------------------------------------------------------------------------
} //namespace RA
#endif
