////////////////////////////////////////////////////////////////////////////////
/*! @file NewDriver.h   A new driver pattern for the developers.
- Part of Examples - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  - http://qt-project.org/
- Copyright(C) 2015, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef NewDriver_H
#define NewDriver_H
#include "Examples.h"
#include "Driver.h"
#include "PhysValue.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sNewDriver)

//--------------------------------------------------------------- sNewDriver ---
/*! A new driver pattern for the developers.
*/
class Examples_EXPORT  sNewDriver : public sDriver
{
  Q_OBJECT
  DRIVER(sNewDriver)

  public: //commands & directives
    static csCommand          cStandUp; //!< Встать.
    static csCommand          cSitDown; //!< Сесть.
    static const
      sDirective<sPhysValue>  cRockNRoll; //!< Крутиться
                                          //!< (attached data: скорость).

  public:
    virtual                   ~sNewDriver ();

  protected: //evolution of the inherited state machine
    psState                   State_W_Stage1;
    psState                   State_W_Stage2;
    psState                   State_O_Sitting;
    psState                   State_O_Standing;
    psState                   State_O_Rolling;

  signals: //outgoing
    void                      NowSitting ();
    void                      NowStanding ();
    void                      NowRolling ();
    void                      Angle (sPhysValue);

  signals: //for internal needs

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  protected slots: //reimplementing inherited
    virtual void              OnEnterToOperatingState ();
    virtual void              OnExitFromOperatingState ();
    virtual void              OnEnterToCoolingState ();

  protected slots:

  protected:

  protected: //fields
    sPhysValue                Speed;
    sPhysValue                CurrentAngle;

  private slots:

  private:

  private: //fields
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
