////////////////////////////////////////////////////////////////////////////////
/*! @file RadiofictivityTestboard.cpp A testboard for sRadiofictivityGauge
                                      or its inheritor.
- Part of BursiPan - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2012-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "RadiofictivityTestboard.h"
#include "attr.h"
#include "StringsAndQStrings.h"
#include <QHBoxLayout>
#include <QPushButton>
#include <QMessageBox>
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sRadiofictivityTestboard)

//------------------------------------------------- sRadiofictivityTestboard ---

void  sRadiofictivityTestboard::Preconfigure (sNodePtr  DeviceCfg)
{
  sGaugeTestboard::Preconfigure(DeviceCfg);
}


sRadiofictivityTestboard::~sRadiofictivityTestboard ()
{
  RadiofictivityGauge.SetToNULL();
}


sRadiofictivityTestboard::sRadiofictivityTestboard (sString   DeviceId
                                 ,sNodePtr  DeviceCfg
                                 ,sNodePtr  WidgetCfg
                                 ,QWidget *  parent    )
    :sGaugeTestboard(DeviceId,DeviceCfg,WidgetCfg,parent)
    ,RadiofictivityGauge(this)
{
  //evolution of the inherited state machine
  {
    //evolution of the State_Cooling
    {
      State_C_Pausing 
          = (new sState(State_Cooling(),"State_C_Pausing"))
          ->SetAsInitial()
          ->OnEnterCall(SLOT(RunCoolingClock()))
          ->AddTransition( SIGNAL(CoolingClockAlarm()) ,State_Cooling()->Fin());
    }
  }
}


void  sRadiofictivityTestboard::AppearanceMakeUp ()
{
  sGaugeTestboard::AppearanceMakeUp();

  QHBoxLayout *             ButtonLayout;
  QPushButton *             Button;

  (ButtonLayout = new QHBoxLayout)->setObjectName("ButtonLayout4");
  ButtonLayout->setSpacing(0);
  ButtonLayout->setMargin(0);
  ((QBoxLayout*)(layout()))->insertLayout(0,ButtonLayout);

  Button = new QPushButton("Say Meow",this);
  ButtonLayout->addWidget(Button);
  Button->setVisible(false);
  connect(Button ,SIGNAL(clicked()) ,this ,SLOT(SayMeow()));
  State_Operating()->OnEnterCall(Button,SLOT(show()))
                   ->OnExitCall(Button,SLOT(hide()));

  Button = new QPushButton("Cheat",this);
  ButtonLayout->addWidget(Button);
  Button->setVisible(false);
  connect(Button ,SIGNAL(clicked()) ,this ,SLOT(Cheat()));
  State_Operating()->OnEnterCall(Button,SLOT(show()))
                   ->OnExitCall(Button,SLOT(hide()));
}


void  sRadiofictivityTestboard::LinkToGauge ()
{
  //Note: if you call the inherited sGaugeTestboard::LinkToGauge() at first
  //      then it will start the submissive driver even in the case when
  //      it is mistakenly set to, say, sRotatorEmulator.  
  try{
    RadiofictivityGauge.SetTo(DeviceLinks,"Gauge");
    Gauge.SetTo(RadiofictivityGauge->DeviceIdentifier());
    Connect(Gauge ,SIGNAL(Warmed()) ,SIGNAL(GaugeLinked()) );
    Connect(Gauge ,SIGNAL(ErrorMessage(QString)) 
           ,SIGNAL(GaugeError(QString)) );
  }catch( rxDeviceLaunch E ){
    emit ErrorMessage( tr("Error while linking to a submissive device.\n"
                          "%1\nCheck configuration.")
                         .arg(ToQString(E.MsgText()))   );
  }
}


void  sRadiofictivityTestboard::OnEnterToOperatingState ()
{
  sGaugeTestboard::OnEnterToOperatingState();
  Connect(Gauge,SIGNAL(Say(QString)),SLOT(ReportGaugeInfo(QString)));
}


void  sRadiofictivityTestboard::OnExitFromOperatingState ()
{
  Disconnect(Gauge,SIGNAL(Say(QString)),SLOT(ReportGaugeInfo(QString)));
  sGaugeTestboard::OnExitFromOperatingState();
}


void sRadiofictivityTestboard::OnEnterToCoolingState ()
{
  RadiofictivityGauge.SetToNULL();
  sGaugeTestboard::OnEnterToCoolingState();
}


void  sRadiofictivityTestboard::RunCoolingClock ()
{
  QTimer::singleShot(2000,this,SIGNAL(CoolingClockAlarm()));
}


void sRadiofictivityTestboard::ReportGaugeInfo (QString  text)
{
  QMessageBox::information(this ,tr("Device Info")
                       ,tr("from '%1':\n\n"
                           "My gauge '%2' says:\n\n"
                           "%3")
                       .arg(ToQString(DeviceIdentifier()))
                       .arg(ToQString(Gauge->DeviceIdentifier()))
                       .arg(text)     );
}


void  sRadiofictivityTestboard::SayMeow ()
{
  if( ! SendDirective<QString>(RadiofictivityGauge
                              ,sRadiofictivityGauge::cSay,"Meow!!!") ){
    QMessageBox::critical(this,tr("Denied")
          ,tr("Device '%1' won't say anything because it is busy.")
             .arg(ToQString(Gauge->DeviceIdentifier()))     );
  }
}


void  sRadiofictivityTestboard::Cheat ()
{
  SendDirective(RadiofictivityGauge
               ,sRadiofictivityGauge::cCheat,sPhysValue(666,0.5,"devils"));
}

//------------------------------------------------------------------------------
} //namespace RA


