////////////////////////////////////////////////////////////////////////////////
/*! @file TeleSpy.h   Шпион-диверсант для телекоммуникационного канала.
- Part of DevelopmentTools - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2013, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef TeleSpy_H
#define TeleSpy_H
#include "DevelopmentTools.h"
#include "Console.h"
#include "TeleCom.h"
#include <QTextEdit>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sTeleSpy)

//----------------------------------------------------------------- sTeleSpy ---
/*! Шпион-диверсант для телекоммуникационного канала: шпионит
    за траффиком в канале и позволяет вмешиваться в него,
    вставляя телексы, как бы исходящие от одного из участников.


*/
class  DevelopmentTools_EXPORT  sTeleSpy : public sConsole
{
  Q_OBJECT
  CONSOLE(sTeleSpy)
  public:
    virtual                   ~sTeleSpy ();

  protected: //evolution of the inherited state machine
    psState                   State_W_Parallel;
    psState                   State_W_LeftTeleCom;
    psState                   State_W_LeftTeleCom_Linking;
    psState                   State_W_RightTeleCom;
    psState                   State_W_RightTeleCom_Linking;

  signals: //for internal needs
    void                      LeftTeleComLinked ();
    void                      LeftTeleComError(QString = QString());
    void                      RightTeleComLinked ();
    void                      RightTeleComError (QString = QString());

  protected:
    virtual void              AppearanceMakeUp ();
    virtual void              Show (literal,sTelex);

  protected slots: //reimplementing inherited
    virtual void              OnEnterToOperatingState ();
    virtual void              OnExitFromOperatingState ();
    virtual void              OnEnterToCoolingState ();

  protected slots:
    virtual void              ReportLeftTeleComError (QString);
    virtual void              ReportRightTeleComError (QString);
    virtual void              OnGotFromLeft (sTelex);
    virtual void              OnGotFromRight (sTelex);
    virtual void              OnTypedInLeft (sTelex);
    virtual void              OnTypedInRight (sTelex);

  protected: //fields
    sTeleCom::sLink           LeftTeleCom;
    sTeleCom::sLink           RightTeleCom;
    QTextEdit *               Screen;

  private slots:
    void                      LinkToLeftTeleCom ();
    void                      LinkToRightTeleCom ();
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
