////////////////////////////////////////////////////////////////////////////////
/*! @file TeleType.h   Teletype (a terminal for a telecommunication channel).
- Part of DevelopmentTools - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2013, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef TeleType_H
#define TeleType_H
#include "DevelopmentTools.h"
#include "Console.h"
#include "TeleCom.h"
#include <QLineEdit>
#include <QTextEdit>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sTelexField)
ANNOUNCE_CLASS(sTeleType)

//-------------------------------------------------------------- sTelexField ---

class DevelopmentTools_EXPORT  sTelexField : public QLineEdit
{
  Q_OBJECT
  public:
    explicit                  sTelexField (QWidget *  parent = 0);
  signals:
    void                      TelexEntered (sTelex);
  private slots:
    void                      OnEnterKeyPressed ();
};


//---------------------------------------------------------------- sTeleType ---
/*! Teletype (a terminal for a telecommunication channel of sTeleCom class).
*/
class  DevelopmentTools_EXPORT  sTeleType : public sConsole
{
  Q_OBJECT
  CONSOLE(sTeleType)
  public:
    virtual                   ~sTeleType ();

  protected: //evolution of the inherited state machine
    psState                   State_W_TeleCom;
    psState                   State_W_TeleCom_Linking;

  signals: //for internal needs
    void                      TeleComLinked ();
    void                      TeleComError (QString = QString());
    void                      Send (sTelex);
    void                      Got (sTelex);

  protected:
    virtual void              AppearanceMakeUp ();

  protected slots: //reimplementing inherited
    virtual void              OnEnterToOperatingState ();
    virtual void              OnExitFromOperatingState ();
    virtual void              OnEnterToCoolingState ();

  protected slots:
    virtual void              ReportTeleComError (QString);
    virtual void              ShowNavy (sTelex);
    virtual void              ShowMaroon (sTelex);

  protected: //fields
    sTeleCom::sLink           TeleCom;
    bool                      ControlExclusively;
    bool                      SendEchoOnPrefix;
    sString                   EchoPrefix;

  private slots:
    void                      LinkToTeleCom ();
    void                      OnSend (sTelex);
    void                      SendEchoIfAsked (sTelex);

  signals: //for funny needs
    void                      OffenceInTurn ();
  private slots:
    void                      SaySamDurak ();
                                //!< Undocumented feature :).

  private: //fields
    QTextEdit *               Screen;
    bool                      Irritable;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
