////////////////////////////////////////////////////////////////////////////////
/*! @file FSDminispectroDriver.h   Драйвер спектрометра FSD-10.
- Part of FSDminispectro - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2018, Dmitrii Nelson, St.Petersburg, Russia.
                          d.nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef FSDminispectroDriver_H
#define FSDminispectroDriver_H
#include "FSDminispectro.h"
#include "CCDDriver.h"
#include "windows.h"
#include "FSD10LIB.h"
#include <QWindow>
namespace RA {

//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sFSDminispectroDriver)

//------------------------------------------------------ sFSDminispectroDriver ---
/*! Драйвер спектрометра FSD-10.

*/

#define PIXELSNUM 3694 //число пикселей CCD TDC1304

class FSDminispectro_EXPORT  sFSDminispectroDriver : public sCCDDriver
{
  Q_OBJECT
  DRIVER(sFSDminispectroDriver)

  protected: // types
    class sWinAPI : public QWindow
    {
      public:
        sWinAPI(psFSDminispectroDriver _host):QWindow()
        {
          this->Host = _host;
        }
      protected:
        virtual bool nativeEvent(const QByteArray& baType,
                                 void* pmessage,
                                 long* result
                                 )
                     {
                       MSG* pmsg = reinterpret_cast<MSG*>(pmessage);
                       switch(pmsg->message) {
                         case WM_DATARECIVED:
                           Host->GetFSD_Data(pmsg->wParam,pmsg->lParam);
                           return true;
                           break;
                         case WM_NAMERECIVED:
                           Host->GetCCDDevice(pmsg->wParam);
                           return true;
                           break;
                         default:
                           return QWindow::nativeEvent(baType, pmessage, result);
                       }

                     }
      private:
        psFSDminispectroDriver   Host;
    };

  public:
    virtual                  ~sFSDminispectroDriver ();

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  protected slots: //reimplementing inherited
    virtual void              ConnectUSBDevice ();
    virtual void              GetCCD_Data ();
    virtual void              CalculateWavelengths ();
    virtual void              StartCCD ();
    virtual void              StopCCD ();
    virtual void              SetCCD_Parameters (sCCDDriver::sParameters Params);

  protected slots:
    virtual void              GetCCDDevice (WPARAM wParam);
    virtual void              GetFSD_Data (WPARAM wParam, LPARAM lParam);

  protected: //fields
    double                    Calibration[3];
    sWinAPI*                  ControlWindow;
    uchar                     Shift;
    int                       Pause;
};

//------------------------------------------------------------------------------
} //namespace RA

#endif
