////////////////////////////////////////////////////////////////////////////////
/*! @file Feofilov.cpp Feofilov's experimental setup.
- Part of Feofilov - a Research Assistant Plugin package.
- Uses  RAGUI - the Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
- Copyright(C) 2010-2013, Dmitry K. Nelson, St.Petersburg, Russia.
                          D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "Feofilov.h"
#include "attr.h"
#include "Petrovich.h"
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sFeofilov)

//------------------------------------------------------------------------------

sVersion  sFeofilov::Version ()
{
  return sVersion(RAP_VERS_MAJOR
                 ,RAP_VERS_MINOR
                 ,ZAKAVYCH(RAP_VERS_PATCH)
                 ,__DATE__
                 );
}

sVersion  sFeofilov::CoreVersionUsed ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}

sVersion  sFeofilov::CoreVersionMin ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}

QString  sFeofilov::UserFriendlyName ()
{
  return trUtf8("Feofilov's experimental setup");
}

QString  sFeofilov::About ()
{
  return trUtf8(
      "Provides 1 console:\n"
      "sPetrovich -- user interface (console) for DFS-24 scanning.\n"
      "Copyright (C) 2011, Dmitry K. Nelson, St.Petersburg, Russia.\n"
      "                    D.Nelson@mail.ioffe.ru\n"
      "Copyright (C) 2011, Viktor E. Bursian, St.Petersburg, Russia.\n"
      "                    Viktor.Bursian@mail.ioffe.ru"
                );
}

void  sFeofilov::InstallDefaults (sPtr<sFolderNode>  DeviceList)
{
  try{
    sPtr<sFolderNode>            DeviceCfg;

    DeviceList |= "ДФС-24 Петрович";
    DeviceCfg = DeviceList >> "ДФС-24 Петрович";
    DeviceCfg << attr("Class","sPetrovich");
    sPetrovich::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Gauge"
        << attr("DeviceId","NIDAQ Counter");

  }catch(...){
    ;
  };

    /*________________________________________template for device
    DeviceList |= "__DEVICE_ID__";
    DeviceCfg = DeviceList >> "__DEVICE_ID__";
    DeviceCfg << attr("Class","__DRIVER_CLASS_NAME__");
    __DRIVER_CLASS_NAME__::Preconfigure(DeviceCfg);
    //if there are links:
    DeviceCfg >> "Links" >> "__LINK_NAME__"
        << attr("DeviceId","__DEVICE_ID__FOR_THE_LINK__");
    //if you wanna change some of the default settings:
    DeviceCfg >> "Settings" << attr("",);
    DeviceCfg >> "Settings" << attr("",);
    */

  sUnit::Register("m"
                 ,"Y..Z..E..P..T..G..M..k--0-cm..-..n..-..f..a..z..y"
                 //If you set here the prefix allowance, it will act only once,
                 //when the user chooses "Install defaults" for the new plugin.
                 //It can be changed later by editing the unit settings in
                 //the RANet Knowledgebase.
                 );

//                 //for a nerd (with hecto and deca):
//                 ,"Y..Z..E..P..T..G..M..khd0dcm..u..n..p..f..a..z..y"
//                 ,sMetricPrefix::StandardAllowance
//                 ,"Y..Z..E..P..T..G..M..k..0..m..u..n..p..f..a..z..y"
//                 ,sMetricPrefix::NoPrefixAllowed
//                 ,"-..-..-..-..-..-..-..-..0..-..-..-..-..-..-..-..-"
}

sFeofilov::sFeofilov ()
{
  sUnit::Register("m"
//                 ,"Y..Z..E..P..T..G..M..k--0-cm..-..n..-..f..a..z..y"
                 //Attention! When you change here the prefix allowance, the
                 //program ignores (and resets!) the settings made by user in
                 //the RANet Knowledgebase.
                 );
}

//------------------------------------------------------------------------------
} //namespace RA
