////////////////////////////////////////////////////////////////////////////////
/*! @file Petrovich.cpp   Петрович - ДФС-24 Феофилова.
- Part of Feofilov - a Research Assistant Plugin package.
- Uses  RAGUI - the Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "Petrovich.h"
#include "attr.h"
#include <QMessageBox>
#include <QSound>
#include <QApplication>
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sPetrovich)

//-------------------------------------------------------- sStateMakingBeeps ---

void sStateMakingBeeps::Construct (int  number_of_beeps
                                  ,int  interval_in_ms)
{
  NumberOfBeeps = number_of_beeps;
  Interval = interval_in_ms;
          
  OnEnterCall(this ,SLOT(Init()));
  
  Substate_Beeping = new sState(this ,Interval 
                               ,objectName()+"_Beeping");
  Substate_Beeping
      ->SetAsInitial()
      ->OnEnterCall(this ,SLOT(Beep()))
      ->AddTransition(Substate_Beeping,SIGNAL(Timeout()),Substate_Beeping)
      ->AddTransition(this ,SIGNAL(Done()) ,Fin());
}

void  sStateMakingBeeps::Beep ()
{
  BeepingStep++;
  if( BeepingStep < NumberOfBeeps ){
    QSound::play(qApp->applicationDirPath()
                +"/../extrafiles/Feofilov/start_countdwn.WAV");
  } else {
    QSound::play(qApp->applicationDirPath()
                +"/../extrafiles/Feofilov/start_countgo.WAV");
    emit Done();
  }
}

//--------------------------------------------------------------- sPetrovich ---

void  sPetrovich::Preconfigure (sNodePtr  DeviceCfg)
{
  sPtr<sFolderNode>              DeviceSettings;

  DeviceCfg |= "Settings";
  DeviceSettings = DeviceCfg >> "Settings";
  if( DeviceSettings.IsCorrect() ){
    DeviceSettings |= attr("StartQuantum",sPhysValue(0.1,"nm"));
    DeviceSettings |= attr("SpeedIndex",6);
    DeviceSettings |= "Speeds";
    DeviceSettings >> "Speeds" |= attr(1 ,sPhysValue(  0.012   ,"nm/min"));
    DeviceSettings >> "Speeds" |= attr(2 ,sPhysValue(  0.04    ,"nm/min"));
    DeviceSettings >> "Speeds" |= attr(3 ,sPhysValue(  0.119216,"nm/min"));
    DeviceSettings >> "Speeds" |= attr(4 ,sPhysValue(  0.357872,"nm/min"));
    DeviceSettings >> "Speeds" |= attr(5 ,sPhysValue(  1.073504,"nm/min"));
    DeviceSettings >> "Speeds" |= attr(6 ,sPhysValue(  6.657277,"nm/min"));
    DeviceSettings >> "Speeds" |= attr(7 ,sPhysValue( 19.970433,"nm/min"));

    //изменение умолчательных шкал для sScanner
    sChartScales      ChartScales(sChartScale("Wavelength"
                                              ,sPhysRange(400,850,"nm"))
                                 ,sChartScale(""
                                             ,sPhysRange(0,200,"kphoton/s"))
                                 );
    ChartScales.StoreSettings(DeviceSettings);

    //изменение умолчательных значений атрибутов для sPeriodicGaugeRecorder
    DeviceSettings |= attr("ValidScanRange",sPhysRange(400,850,"nm"));
    DeviceSettings |= attr("ScanRange",sPhysRange(500,700,"nm",0.01));
    DeviceSettings |= attr("ValidScanSteps",sPhysRange(0.01,10.0,"nm"));
    DeviceSettings |= attr("ScanStep",sPhysValue(0.2,0.005,"nm"));

  };
  sPeriodicGaugeRecorder::Preconfigure(DeviceCfg);
}


sPetrovich::~sPetrovich()
{
/*
  SpeedIndex = FldSpeedIndex->value();
  Speed = Speeds[SpeedIndex-1];
  if( DeviceSettings.IsCorrect() ){
    DeviceConfig << attr("SpeedIndex",SpeedIndex);
    DeviceSettings << attr("ScanRange",ScanRangeField->Value())
                   << attr("ScanStep",ScanStepField->Value());
  }
*/
}


sPetrovich::sPetrovich (sString    DevicetId
                       ,sNodePtr   DeviceCfg
                       ,sNodePtr   WidgetCfg
                       ,QWidget *  parent   )
    :sPeriodicGaugeRecorder(DevicetId,DeviceCfg,WidgetCfg,parent)
    ,StartQuantum(0.1,"nm")
    ,SpeedIndex(6)
{
  Speeds[0] = sPhysValue("nm/s"); Speeds[0].FixTheUnits();
  Speeds[1] = sPhysValue("nm/s"); Speeds[1].FixTheUnits();
  Speeds[2] = sPhysValue("nm/s"); Speeds[2].FixTheUnits();
  Speeds[3] = sPhysValue("nm/s"); Speeds[3].FixTheUnits();
  Speeds[4] = sPhysValue("nm/s"); Speeds[4].FixTheUnits();
  Speeds[5] = sPhysValue("nm/s"); Speeds[5].FixTheUnits();
  Speeds[6] = sPhysValue("nm/s"); Speeds[6].FixTheUnits();
  Speeds[0] = sPhysValue(  0.012   ,"nm/min");
  Speeds[1] = sPhysValue(  0.04    ,"nm/min");
  Speeds[2] = sPhysValue(  0.119216,"nm/min");
  Speeds[3] = sPhysValue(  0.357872,"nm/min");
  Speeds[4] = sPhysValue(  1.073504,"nm/min");
  Speeds[5] = sPhysValue(  6.657277,"nm/min");
  Speeds[6] = sPhysValue( 19.970433,"nm/min");
  Speed = Speeds[SpeedIndex-1];
  Multiplier.X = sPhysValue(1,"nm");
  Multiplier.Y = sPhysValue(1,"photon/s");
  //evolution of the inherited state machine
  {
    //evolution of the State_O_Scanning
    {
      State_O_Sc_Prompting = new sState(State_O_Scanning
                                       ,"State_O_Sc_Prompting");
      State_O_Sc_MakingBeeps = new sStateMakingBeeps(4 ,1000 
                                                    ,State_O_Scanning
                                                    ,"State_O_Sc_MakingBeeps");
      
      State_O_Sc_SettingGauge
          ->RemoveOnFinishTransition()
          ->AddTransition(State_O_Sc_Prompting);
          
      State_O_Sc_Prompting
          ->OnEnterCall(SLOT(ShowPrompt()))
          ->AddTransition(SIGNAL(PromptAcknowledged()) ,State_O_Sc_MakingBeeps);
      
      State_O_Sc_MakingBeeps
          ->AddTransition( SIGNAL(Break()) ,State_O_Sc_ResettingGauge )
          ->AddTransition(State_O_Sc_Acquisition);
    }
  }
}


void  sPetrovich::AppearanceMakeUp ()
{
  sPeriodicGaugeRecorder::AppearanceMakeUp();
  try{ //get config parameters
    get_from<sPhysValueNode>(DeviceSettings >> "StartQuantum" ,StartQuantum);
    get_from<sIntegerNode>(DeviceSettings >> "SpeedIndex" ,SpeedIndex);
    for( int i = 0 ; i < 7 ; ++i ){
      get_from<sPhysValueNode>(DeviceSettings >> "Speeds" >> (i+1) ,Speeds[i]);
    }
    Speed = Speeds[SpeedIndex-1];
  }catch(...){
    SetInvalid(tr("Something wrong in configuration."));
  }
  //add new widgets
  {
    LblSpeedIndex = new QLabel(ParamArea);
    LblSpeedIndex->setObjectName("LblSpeedIndex");
    ParamLayout->addWidget(LblSpeedIndex,Row,0,1,1);
    FldSpeedIndex = new QSpinBox(ParamArea);
    FldSpeedIndex->setObjectName("FldSpeedIndex");
    FldSpeedIndex->setRange(1,7);
    FldSpeedIndex->setValue(SpeedIndex);
    ParamLayout->addWidget(FldSpeedIndex,Row,1,1,1);
    Translate();
  };
}

void  sPetrovich::Translate ()
{
  LblSpeedIndex->setText(tr("Speed"));
}

void  sPetrovich::RestoreState ()
{
  sPeriodicGaugeRecorder::RestoreState();
  if( WidgetConfig.IsCorrect() ){
    if( get_from<sIntegerNode>(WidgetConfig >> "SpeedIndex" ,SpeedIndex) ){
      FldSpeedIndex->setValue(SpeedIndex);
      Speed = Speeds[SpeedIndex-1];
    }
  }
}


void  sPetrovich::StoreState ()
{
  SpeedIndex = FldSpeedIndex->value();
  Speed = Speeds[SpeedIndex-1];
  sPeriodicGaugeRecorder::StoreState();
  if( WidgetConfig.IsCorrect() ){
    WidgetConfig << attr("SpeedIndex" ,SpeedIndex);
  }
}


void  sPetrovich::EarlySetAttributes ()
{
  sPeriodicGaugeRecorder::EarlySetAttributes();
  Dependence << attr("Scan range",ScanRange)
             << attr("Scan step",ScanStep)
             << attr("Speed No",SpeedIndex);
}


void  sPetrovich::StopScanning ()
{
  sPeriodicGaugeRecorder::StopScanning ();
  QMessageBox::information(this, tr("Scanning is finished")
                                ,tr("Switch off the motor.")
                                ,QMessageBox::Ok);
}

void  sPetrovich::SetGaugeRegime ()
{
  SpeedIndex = FldSpeedIndex->value();
  Speed = Speeds[SpeedIndex-1];
  sPeriodicGaugeRecorder::SetGaugeRegime ();
}


void  sPetrovich::ShowPrompt ()
{
  QMessageBox::information(this, tr("Start scanning")
                          ,tr("Set the starting wavelength ")
                          +ScanRangeField->Value().From().Text()
                          +tr(" and switch on the motor at the 4th beep.")
                          ,QMessageBox::Ok);
  emit PromptAcknowledged();
}

//------------------------------------------------------------------------------
} //namespace RA
