////////////////////////////////////////////////////////////////////////////////
/*! @file Petrovich.h   Петрович - ДФС-24 Феофилова.
- Part of Feofilov - a Research Assistant Plugin package.
- Uses  RAGUI - the Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef Petrovich_H
#define Petrovich_H
#include "Feofilov.h"
#include "PeriodicGaugeRecorder.h"
#include <QSpinBox>
#include <QTimer>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sStateMakingBeeps)
ANNOUNCE_CLASS(sPetrovich)

//-------------------------------------------------------- sStateMakingBeeps ---
/*! An element of a state machine which makes n short beeps.
*/
class Feofilov_EXPORT sStateMakingBeeps : public sState
{
  Q_OBJECT

  public:
                              sStateMakingBeeps (int         number_of_beeps
                                                ,int         interval_in_ms
                                                ,QState *    parent_state
                                                ,int         timeout_in_ms = -1
                                                ,QString     object_name
                                                                    = QString())
                                  :sState(parent_state
                                         ,timeout_in_ms,object_name)
                                { Construct(number_of_beeps,interval_in_ms); }
                              
                              sStateMakingBeeps (int         number_of_beeps
                                                ,int         interval_in_ms
                                                ,QState *    parent_state
                                                ,QString     object_name
                                                                    = QString())
                                  :sState(parent_state,object_name)
                                { Construct(number_of_beeps,interval_in_ms); }
                                
  protected: //internal state machine
    psState                   Substate_Beeping;

  signals: //for internal needs
    void                      Done ();
  
  protected slots:
    virtual void              Init ()
                                { BeepingStep=0; }
    virtual void              Beep ();

  protected: //fields
    int                       NumberOfBeeps;
    int                       Interval;
    
  private:
    void                      Construct (int  number_of_beeps
                                        ,int  interval_in_ms);

  private: //fields
    int                       BeepingStep;
};

//--------------------------------------------------------------- sPetrovich ---
/*! Консоль Петрович.
*/
class Feofilov_EXPORT sPetrovich : public sPeriodicGaugeRecorder
{
  Q_OBJECT
  CONSOLE(sPetrovich)

  public:
    virtual                   ~sPetrovich ();
    virtual void              RestoreState ();
    virtual void              StoreState ();

  protected: //evolution of the inherited state machine
    psState                   State_O_Sc_Prompting;
    psStateMakingBeeps        State_O_Sc_MakingBeeps;

  signals: //for internal needs
    void                      PromptAcknowledged ();

  protected: //reimplementing inherited
    virtual void              AppearanceMakeUp ();
    virtual void              Retranslate ()
                                {
                                  sPeriodicGaugeRecorder::Retranslate();
                                  Translate();
                                }
    virtual void              EarlySetAttributes ();
    virtual void              StopScanning ();
    virtual void              SetGaugeRegime ();

  protected slots:
    virtual void              ShowPrompt ();

  protected: //fields
    sPhysValue                StartQuantum;
    integer                   SpeedIndex;
    sPhysValue                Speeds[7];

  protected: //fields
    QLabel *                  LblSpeedIndex;
    QSpinBox *                FldSpeedIndex;

  private:
    void                      Translate ();
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
