////////////////////////////////////////////////////////////////////////////////
/*! @file HamamatsuCCDDriver.h   Драйвер спектрометра Hamamatsu.
- Part of Hamamatsu - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2017, Dmitrii Nelson, St.Petersburg, Russia.
                          d.nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef HamamatsuCCDDriver_H
#define HamamatsuCCDDriver_H
#include "Hamamatsu.h"
#include "CCDDriver.h"
#include "specu1a.h"
namespace RA {

//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sHamamatsuCCDDriver)

//------------------------------------------------------ sHamamatsuCCDDriver ---
/*! Драйвер спектрометра Hamamatsu.

*/

#define PRODUCT_ID 0x2909 // for C10082CA, C10083CA, C10082CAH, C10083CAH
#define PIXELSNUM 2048 // for C10083CA, C10083CAH

class Hamamatsu_EXPORT  sHamamatsuCCDDriver : public sCCDDriver
{
  Q_OBJECT
  DRIVER(sHamamatsuCCDDriver)

  public:
    virtual                  ~sHamamatsuCCDDriver ();

  protected slots: //reimplementing inherited
    virtual void              ConnectUSBDevice ();
    virtual void              GetCCD_Data ();
    virtual void              CalculateWavelengths ();
    virtual void              StartCCD ();
    virtual void              StopCCD ();
    virtual void              SetCCD_Parameters (sCCDDriver::sParameters Params);

  protected: //fields
    HANDLE                    DeviceHandle;	/* device handle */
    HANDLE                    PipeHandle;	/* pipe handle */
    double                    Calibration[6];
    ushort                    Buffer[PIXELSNUM];
};

//------------------------------------------------------------------------------
} //namespace RA

#endif
