////////////////////////////////////////////////////////////////////////////////
/*! @file Cubic001Film.h   Formulae for magnetization of a cubic (001) film.
- Part of MagnetizationCalc - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  - http://qt-project.org/
- Copyright(C) 2017, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef Cubic001Film_H
#define Cubic001Film_H
#include "Magneto.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sCubic001FilmParams)

//------------------------------------------------------------------------------

double  Cubic001Film_U_polar (const gsl_vector *  theta_phi
                             ,void *              params
                             );

void  Cubic001Film_dU_polar (const gsl_vector *  theta_phi
                            ,void *              params
                            ,gsl_vector *        gradient
                            );

void  Cubic001Film_UdU_polar (const gsl_vector *  theta_phi
                             ,void *              params
                             ,double *            energy
                             ,gsl_vector *        gradient
                             );

int  Cubic001Film_ddU_polar (const gsl_vector *  theta_phi
                            ,void *              params
                            ,gsl_matrix *        hessian
                            );

//------------------------------------------------------------------------------

double  Cubic001Film_U_xy (const gsl_vector *  mxmy
                          ,void *              params
                          );

void  Cubic001Film_dU_xy (const gsl_vector *  mxmy
                         ,void *              params
                         ,gsl_vector *        gradient
                         );

void  Cubic001Film_UdU_xy (const gsl_vector *  mxmy
                          ,void *              params
                          ,double *            energy
                          ,gsl_vector *        gradient
                          );

//------------------------------------------------------ sCubic001FilmParams ---
/*!
*/
struct  sCubic001FilmParams : sMagnetizationParams
{
                              sCubic001FilmParams ();

    psCubic001FilmParams      Set_kc1 (rcsPhysValue);
    psCubic001FilmParams      Set_ku1 (rcsPhysValue);

    virtual void              ProvideAttributesTo (sPtr<sFolderNode>);

    double                    kc1;
    double                    ku1;

    sPhysValue                _kc1;
    sPhysValue                _ku1;
};


//------------------------------------------------------------------------------
} //namespace RA
#endif
