////////////////////////////////////////////////////////////////////////////////
/*! @file MagnetizationCalc.cpp Calculator for magnetization and its effects.
- Part of MagnetizationCalc - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2017, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "MagnetizationCalc.h"
#include "Magneto.h"
#include "MagnetoCalculator.h"
#include "Symbols.h"
#include "attr.h"
//#include <QMetaType>
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sMagnetizationCalc)

//------------------------------------------------------------------------------

sVersion  sMagnetizationCalc::Version ()
{
  return sVersion(RAP_VERS_MAJOR
                 ,RAP_VERS_MINOR
                 ,ZAKAVYCH(RAP_VERS_PATCH)
                 ,__DATE__
                 );
}


sVersion  sMagnetizationCalc::CoreVersionUsed ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}


sVersion  sMagnetizationCalc::CoreVersionMin ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}


QString  sMagnetizationCalc::UserFriendlyName ()
{
  return trUtf8("Calculator for MagnetizationCalc and its effects");
}


QString  sMagnetizationCalc::About ()
{
  return trUtf8(
      "The package provides calculator for magnetization and its effects.\n\n"
      "Copyright (C) 2017, Viktor E. Bursian, St.Petersburg, Russia.\n"
      "                    Viktor.Bursian@mail.ioffe.ru"
                );
}


void  sMagnetizationCalc::InstallDefaults (sPtr<sFolderNode>  DeviceList)
{
  try{
    sPtr<sFolderNode>              DeviceCfg;

    DeviceList |= "Magnetization Calculator";
    DeviceCfg = DeviceList >> "Magnetization Calculator";
    DeviceCfg << attr("Class","sMagnetoCalculator");
    sMagnetoCalculator::Preconfigure(DeviceCfg);
  }catch(...){
  }
}


sMagnetizationCalc::sMagnetizationCalc ()
{
  sUnit::Register("G");
  sUnit::Register("Hz");
  sMagnetoTask::Tasks.insert(std::pair<sMagnetoTask::eTaskType,sMagnetoTask>
      (sMagnetoTask::cM_AllSolutions
      ,sMagnetoTask("M (all solutions)",&M_AllSolutions)));
  sMagnetoTask::Tasks.insert(std::pair<sMagnetoTask::eTaskType,sMagnetoTask>
      (sMagnetoTask::cM_vs_H
      ,sMagnetoTask("M(H)",&M_vs_H)));
  sMagnetoTask::Tasks.insert(std::pair<sMagnetoTask::eTaskType,sMagnetoTask>
      (sMagnetoTask::cFMR_Hres_vs_xi
      ,sMagnetoTask("FMR in-plane",&FMR_Hres_vs_xi)));
  sMagnetoTask::Tasks.insert(std::pair<sMagnetoTask::eTaskType,sMagnetoTask>
      (sMagnetoTask::cFMR_Hres_vs_eta
      ,sMagnetoTask("FMR out-of-plane",&FMR_Hres_vs_eta)));
  sMagnetoTask::Tasks.insert(std::pair<sMagnetoTask::eTaskType,sMagnetoTask>
      (sMagnetoTask::cLMOKE_shoulders
      ,sMagnetoTask("LMOKE_shoulders"
                   ,&LMOKE_shoulders)));
  sMagnetoTask::Tasks.insert(std::pair<sMagnetoTask::eTaskType,sMagnetoTask>
      (sMagnetoTask::cVSM
      ,sMagnetoTask("VSM"
                   ,&VSM)));
}

//------------------------------------------------------------------------------
} //namespace RA
