////////////////////////////////////////////////////////////////////////////////
/*! @file MagnetoCalculator.h   Calculator for magnetization and its effects.
- Part of MagnetizationCalc - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  - http://qt-project.org/
- Copyright(C) 2017, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef MagnetoCalculator_H
#define MagnetoCalculator_H
#include "MagnetizationCalc.h"
#include "Console.h"
#include "NetWidgets.h"
#include "Navigator.h"
#include "Chart.h"
#include "PhysValueField.h"
#include <QTimer>
#include <QCheckBox>
#include <QComboBox>
#include <QSplitter>
#include <QLineEdit>
#include <QPushButton>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sMagnetoCalculator)

//------------------------------------------------------- sMagnetoCalculator ---
/*! Calculator for magnetization and its effects.
*/
class  MagnetizationCalc_EXPORT  sMagnetoCalculator : public sConsole
{
  Q_OBJECT
  CONSOLE(sMagnetoCalculator)
  public:
    virtual                   ~sMagnetoCalculator ();
    virtual void              RestoreState ();
    virtual void              StoreState ();

  public slots:
    void                      PutLegendAtTheBottom (bool);

  signals:
    void                      CursorPositionChanged (sPhysPair  position
                                                    ,sPhysPair  reference
                                                    ,sString    x_title
                                                    ,sString    y_title);

  protected: //reimplementing inherited
    virtual void              AppearanceMakeUp ();
    virtual void              contextMenuEvent (QContextMenuEvent *);

  protected slots:
    void                      OnTheLinkFieldEdit ();
    void                      Calculate ();

  protected: //fields

  private:
    void                      Clear ();
    void                      RemoveChartAndLegend ();
    bool                      CreateChartAndLegend (sNodePtr  folder);

  private: //fields
    QLabel *                  TheLinkLabel;
    psDropNodeWidget          TheLinkField;
    QLabel *                  SymmetryLabel;
    QComboBox *               SymmetryComboBox;
    QLabel *                  TaskLabel;
    QComboBox *               TaskComboBox;
    QPushButton *             CalcButton;

//    QCheckBox *               SampleBindingCheckBox;
//    QLabel *                  SampleLabel;
//    QLineEdit *               SampleTextField;
    psPhysValueField          g_Field;
    psPhysValueField          _4piMs_Field;
    psPhysValueField          kc1_Field;
    psPhysValueField          ku1_Field;

    psPhysValueField          f0_Field;
    psPhysValueField          H0_Field;
    psPhysValueField          eta_Field;
    psPhysValueField          xi_Field;

    psNet2TreeEvolution       Evolution;
    psChart                   Chart;
    psNavigator               Legend;
    QTimer *                  UpdatingTimer;
    QSplitter *               Splitter;
    QAction *                 ActnLegendBottom;
    QAction *                 ActnLegendRight;
};


//------------------------------------------------------------------------------
} //namespace RA
#endif
