////////////////////////////////////////////////////////////////////////////////
/*! @file DoubleCounterDisplay.h Дисплей для отображения счетчиков NIDAQDoubleCounter.
- Part of NIDAQmxDrivers - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2016, Dmitry K. Nelson, St.Petersburg, Russia.
                          D.Nelson@mail.ioffe.ru
- Copyright(C) 2013,      Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "DoubleCounterDisplay.h"
#include "attr.h"
#include "StringsAndQStrings.h"
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QFormLayout>
#include <QSpacerItem>
#include <QMessageBox>
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sDoubleCounterDisplay)

//---------------------------------------------------- sDoubleCounterDisplay ---

void  sDoubleCounterDisplay::Preconfigure (sNodePtr  DeviceCfg)
{
  sPtr<sFolderNode>              DeviceSettings;
  DeviceCfg |= "Settings";
  DeviceSettings = DeviceCfg >> "Settings";
  if( DeviceSettings.IsCorrect() ){
    DeviceSettings |= attr("MeasuringMode",0);
    DeviceSettings |= attr("DisplayUnits_Intens","photon/s");
    DeviceSettings |= attr("DigitsAfterTheDot_Intens",0);
    DeviceSettings |= attr("DisplayUnits_Polariz","%");
    DeviceSettings |= attr("DigitsAfterTheDot_Polariz",2);
  };
  sGaugeDisplay::Preconfigure(DeviceCfg);
}


sDoubleCounterDisplay::sDoubleCounterDisplay (sString   DeviceId
                                             ,sNodePtr  DeviceCfg
                                             ,sNodePtr  WidgetCfg
                                             ,QWidget *  parent    )
    :sGaugeDisplay(DeviceId,DeviceCfg,WidgetCfg,parent)
    ,DoubleCounter(this)
    ,UnitsToShow("")
    ,DigitsAfterTheDot(0)
    ,UnitsToShow_2("")
    ,DigitsAfterTheDot_2(0)
    ,UnitsToShow_I("")
    ,DigitsAfterTheDot_I(0)
    ,UnitsToShow_P("")
    ,DigitsAfterTheDot_P(0)
    ,Multiplier(1.0)
    ,Multiplier_2(1.0)
    ,MeasuringMode(0)
{
}

sDoubleCounterDisplay::~sDoubleCounterDisplay()
{
  if( DeviceSettings.IsCorrect() ){
    DeviceSettings
        << attr("MeasuringMode",MeasuringMode);
  }
  DoubleCounter.SetToNULL();
}

void  sDoubleCounterDisplay::AppearanceMakeUp ()
{
  try{ //get config parameters
    get_from<sIntegerNode>(DeviceSettings >> "MeasuringMode"
                                             ,MeasuringMode);
    get_from<sTextNode>(DeviceSettings >> "DisplayUnits_Intens" ,UnitsToShow_I);
    get_from<sIntegerNode>(DeviceSettings >> "DigitsAfterTheDot_Intens"
                                             ,DigitsAfterTheDot_I);
    get_from<sTextNode>(DeviceSettings >> "DisplayUnits_Polariz" ,UnitsToShow_P);
    get_from<sIntegerNode>(DeviceSettings >> "DigitsAfterTheDot_Polariz"
                                             ,DigitsAfterTheDot_P);
    SetUnitsAndDigits();
    if( DigitsAfterTheDot < 0 ){
      DigitsAfterTheDot = 0;
    }else if( DigitsAfterTheDot > 9 ){
      DigitsAfterTheDot = 9;
    }
    if( DigitsAfterTheDot_2 < 0 ){
      DigitsAfterTheDot_2 = 0;
    }else if( DigitsAfterTheDot_2 > 9 ){
      DigitsAfterTheDot_2 = 9;
    }
  }catch(...){
    SetInvalid(tr("Something wrong in configuration."));
  }

  Multiplier = power10(DigitsAfterTheDot);
  Multiplier_2 = power10(DigitsAfterTheDot_2);

  setLayout(new QVBoxLayout);

  ValueLCD = new QLCDNumber(this);
  ValueLCD->setSegmentStyle(QLCDNumber::Flat);
  layout()->addWidget(ValueLCD);

  QHBoxLayout *               L( new QHBoxLayout );
  layout()->addItem(L);
  UnitsDisplay = new QLabel(this);
  L->addStretch();
  L->addWidget(UnitsDisplay);
  UnitsDisplay->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);

  ShowNaN();

  ValueLCD_2 = new QLCDNumber(this);
  ValueLCD_2->setSegmentStyle(QLCDNumber::Flat);
  layout()->addWidget(ValueLCD_2);

  QHBoxLayout *               L_2( new QHBoxLayout );
  layout()->addItem(L_2);
  UnitsDisplay_2 = new QLabel(this);
  L_2->addStretch();
  L_2->addWidget(UnitsDisplay_2);
  UnitsDisplay_2->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);

  ShowNaN_2();

  QFormLayout *               M( new QFormLayout );
  layout()->addItem(M);
  LblMode = new QLabel(this);
  LblMode->setText(tr("Measuring Mode"));
  M->addWidget(LblMode);
  FldMode = new QComboBox(this);
  FldMode->insertItem(0,"Counter A [Counter B]");
  FldMode->insertItem(1,"Counter B [Counter A]");
  FldMode->insertItem(2,"Polariz. degree [Counter A+Counter B]");
  FldMode->setCurrentIndex(MeasuringMode);
  M->addWidget(FldMode);
//  Translate();

  connect(FldMode,SIGNAL(currentIndexChanged(int)) ,this,SLOT(OnModeChanged(int)));
}

void sDoubleCounterDisplay::LinkToGauge ()
{
  sGaugeDisplay::LinkToGauge ();
  DoubleCounter.SetTo(Gauge->DeviceIdentifier());
  if( ! DoubleCounter.IsCorrect() ){
    emit ErrorMessage( tr("The 'DoubleCounter' link is set to '%1', which "
                          "does not look correct. Check configuration.")
                         .arg(ToQString(Gauge->DeviceIdentifier())) );
  } else {
    DoubleCounter->SetInitialMode(MeasuringMode);
  }
}

void sDoubleCounterDisplay::ShowData (sPhysValue  V)
{
  integer                     VI;
  QString                     T;
  try{
    VI = Round( (V/sPhysValue(1.0,UnitsToShow)).ToReal() * Multiplier );
  }catch(rxRAlgebra){
    emit Failure(tr("Wrong units in the configuration settings.\n"
                    "Cannot convert '%1' into '%2'.")
                .arg(ToQString(V.Text()))
                .arg(ToQString(UnitsToShow))    );
    return;
  };
  if(   ( VI == std::numeric_limits<integer>::min() )
      ||( VI == std::numeric_limits<integer>::max() ) ){
    T = "Error";
  }else{
    T.setNum(VI);
    for( int i = 3+DigitsAfterTheDot ; i < T.length()-( VI<0 ? 1 : 0 ) ; i+=4 )
      T = T.left(T.length()-i) + ' ' + T.right(i);
    if( DigitsAfterTheDot > 0 ){
      T = T.left(T.length()-DigitsAfterTheDot)
          + '.' + T.right(DigitsAfterTheDot);
      for( int i = 3 ; i < DigitsAfterTheDot ; i+=3 )
        T = T.left(T.length()-DigitsAfterTheDot+i)
            + ' ' + T.right(DigitsAfterTheDot-i);
    }
  }
  ValueLCD->setDigitCount(T.length());
  ValueLCD->display(T);
  UnitsDisplay->setText(ToQString( sUnits(UnitsToShow).Text(HTML) ));
}

void sDoubleCounterDisplay::ShowData_2 (sPhysValue  V)
{
  integer                     VI;
  QString                     T;
  try{
    VI = Round( (V/sPhysValue(1.0,UnitsToShow_2)).ToReal() * Multiplier_2 );
  }catch(rxRAlgebra){
    emit Failure(tr("Wrong units in the configuration settings.\n"
                    "Cannot convert '%1' into '%2'.")
                .arg(ToQString(V.Text()))
                .arg(ToQString(UnitsToShow_2))    );
    return;
  };
  if(   ( VI == std::numeric_limits<integer>::min() )
      ||( VI == std::numeric_limits<integer>::max() ) ){
    T = "Error";
  }else{
    T.setNum(VI);
    for( int i = 3+DigitsAfterTheDot_2 ; i < T.length()-( VI<0 ? 1 : 0 ) ; i+=4 )
      T = T.left(T.length()-i) + ' ' + T.right(i);
    if( DigitsAfterTheDot_2 > 0 ){
      T = T.left(T.length()-DigitsAfterTheDot_2)
          + '.' + T.right(DigitsAfterTheDot_2);
      for( int i = 3 ; i < DigitsAfterTheDot_2 ; i+=3 )
        T = T.left(T.length()-DigitsAfterTheDot_2+i)
            + ' ' + T.right(DigitsAfterTheDot_2-i);
    }
  }
  ValueLCD_2->setDigitCount(T.length());
  ValueLCD_2->display(T);
  UnitsDisplay_2->setText(ToQString( sUnits(UnitsToShow_2).Text(HTML) ));
}

void sDoubleCounterDisplay::ShowNaN ()
{
  ValueLCD->display(QString("-----"));
  UnitsDisplay->setText(ToQString( UnitsToShow ));
}

void sDoubleCounterDisplay::ShowNaN_2 ()
{
  ValueLCD_2->display(QString("-----"));
  UnitsDisplay_2->setText(ToQString( UnitsToShow_2 ));
}

void sDoubleCounterDisplay::OnEnterToCoolingState ()
{
  DoubleCounter.SetToNULL();
  sGaugeDisplay::OnEnterToCoolingState();
}

void sDoubleCounterDisplay::OnEnterToOperatingState()
{
  sGaugeDisplay::OnEnterToOperatingState();
  Connect(Gauge ,SIGNAL(NewData_2(sPhysValue)) ,SLOT(ShowData_2(sPhysValue)) );
}


void sDoubleCounterDisplay::OnExitFromOperatingState()
{
  Disconnect(Gauge ,SIGNAL(NewData_2(sPhysValue)) ,SLOT(ShowData_2(sPhysValue)) );
  ShowNaN_2();
  sGaugeDisplay::OnExitFromOperatingState();
}

void sDoubleCounterDisplay::OnModeChanged(int _newMode)
{
  SendCommand(Gauge,sGauge::cStop);
  SendDirective<int>(DoubleCounter,sNIDAQDoubleCounter::cChangeCountingMode,_newMode);
  MeasuringMode = _newMode;
  SetUnitsAndDigits();
  Multiplier = power10(DigitsAfterTheDot);
  Multiplier_2 = power10(DigitsAfterTheDot_2);
  SendCommand(Gauge,sGauge::cStart);
}

void sDoubleCounterDisplay::SetUnitsAndDigits()
{
  switch (MeasuringMode){
    case 0:
      UnitsToShow = UnitsToShow_I;
      DigitsAfterTheDot = DigitsAfterTheDot_I;
      UnitsToShow_2 = UnitsToShow_I;
      DigitsAfterTheDot_2 = DigitsAfterTheDot_I;
      break;
    case 1:
      UnitsToShow = UnitsToShow_I;
      DigitsAfterTheDot = DigitsAfterTheDot_I;
      UnitsToShow_2 = UnitsToShow_I;
      DigitsAfterTheDot_2 = DigitsAfterTheDot_I;
      break;
    case 2:
      UnitsToShow = UnitsToShow_P;
      DigitsAfterTheDot = DigitsAfterTheDot_P;
      UnitsToShow_2 = UnitsToShow_I;
      DigitsAfterTheDot_2 = DigitsAfterTheDot_I;
      break;
  };
}

//------------------------------------------------------------------------------
} //namespace RA


