////////////////////////////////////////////////////////////////////////////////
/*! @file DoubleCounterDisplay.h Дисплей для отображения счетчиков NIDAQDoubleCounter.
- Part of NIDAQmxDrivers - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2016, Dmitry K. Nelson, St.Petersburg, Russia.
                          D.Nelson@mail.ioffe.ru
- Copyright(C) 2013,      Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef DoubleCounterDisplay_H
#define DoubleCounterDisplay_H
#include "NIDAQmxDrivers.h"
#include "GaugeDisplay.h"
#include "NIDAQDoubleCounter.h"
#include <QLCDNumber>
#include <QComboBox>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sDoubleCounterDisplay)

//---------------------------------------------------- sDoubleCounterDisplay ---
/*! Дисплей для отображения счетчиков NIDAQDoubleCounter.

Показывает отсчёты счетчиков по мере их поступления
в единицах измерения, указанных в конфигурации (UnitsToShow).
Положение десятичной точки фиксировано (DigitsAfterTheDot).
Для удобства чтения цифры числа разбиваются пробелами на тройки.
Позволяет переключать режимы работы NIDAQDoubleCounter.

*/
class  NIDAQmxDrivers_EXPORT  sDoubleCounterDisplay : public sGaugeDisplay
{
  Q_OBJECT
  CONSOLE(sDoubleCounterDisplay)

  public:
    virtual                   ~sDoubleCounterDisplay ();

  protected: //reimplementing inherited
    virtual void              AppearanceMakeUp ();

  protected slots: //reimplementing inherited
    virtual void              ShowData (sPhysValue);
    virtual void              ShowNaN ();
    virtual void              OnEnterToOperatingState();
    virtual void              OnExitFromOperatingState();
    virtual void              OnEnterToCoolingState ();
    virtual void              OnModeChanged(int);
    virtual void              LinkToGauge ();

  protected slots:
    virtual void              ShowData_2 (sPhysValue);
    virtual void              ShowNaN_2 ();
    virtual void              SetUnitsAndDigits();


  protected: //fields
    sNIDAQDoubleCounter::sLink      DoubleCounter;
    sString                   UnitsToShow;
    int                       DigitsAfterTheDot;
    sString                   UnitsToShow_2;
    int                       DigitsAfterTheDot_2;

  private: //fields
    sString                   UnitsToShow_I;
    int                       DigitsAfterTheDot_I;
    sString                   UnitsToShow_P;
    int                       DigitsAfterTheDot_P;
    QLCDNumber *              ValueLCD;
    QLabel *                  UnitsDisplay;
    real                      Multiplier; //!< power10(DigitsAfterTheDot)
    QLCDNumber *              ValueLCD_2;
    QLabel *                  UnitsDisplay_2;
    real                      Multiplier_2; //!< power10(DigitsAfterTheDot)
    QLabel *                  LblMode;
    QComboBox *               FldMode;
    int                       MeasuringMode;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
