////////////////////////////////////////////////////////////////////////////////
/*! @file KineticsNIDAQ.h   Усреднитель данных кинетики NIDAQ.
- Part of NIDAQmxDrivers - a Research Assistant Plugin package.
- Uses  RAGUI - the Research Assistant Graphical User Interface.
- Uses  QtGui v.5.0  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2013, Dmitry K. Nelson, St.Petersburg, Russia.
                          D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef KineticsNIDAQ_H
#define KineticsNIDAQ_H
#include "NIDAQmxDrivers.h"
#include "Scanner.h"
#include "DataArrayRecord.h"
#include "NIDAQCounter.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sKineticsNIDAQ)

//--------------------------------------------------- sKineticsNIDAQ ---
/*! Усреднитель данных кинетики NIDAQ.
*/
class NIDAQmxDrivers_EXPORT  sKineticsNIDAQ : public sScanner
{
  Q_OBJECT
  CONSOLE(sKineticsNIDAQ)
  public:
    virtual                   ~sKineticsNIDAQ ();
    virtual void              RestoreState ();
    virtual void              StoreState ();

  protected: //evolution of the inherited state machine
    psState                   State_W_Gauge;
    psState                   State_O_Sc_SettingGauge;
    psState                   State_O_Sc_SG_Waiting;
    psState                   State_O_Sc_ResettingGauge;
    psState                   State_O_Sc_RG_Waiting;

  signals: //for internal needs
    void                      GaugeWarmed ();
    void                      GaugeError(QString = QString());
    void                      GaugeBusy ();
    void                      GaugePeriodChanged (sPhysValue);

  protected: //reimplementing inherited
    virtual void              AppearanceMakeUp ();
    virtual void              Retranslate ()
                                {
                                  sScanner::Retranslate();
                                  Translate();
                                }

  protected slots: //reimplementing inherited
    virtual void              OnEnterToCoolingState ();
    virtual void              EarlySetAttributes ();
    virtual void              LateSetAttributes ();

  protected slots: //implementing pure virtual
    virtual void              CreateNewDependence ();
    virtual void              StartScanning ();
    virtual void              StopScanning ();

  protected slots:
    virtual void              SetGaugeRegime ();
    virtual void              CorrectStep (sPhysValue  /*gauge_period*/);
    virtual void              ResetGaugeRegime ();
    virtual void              RestartGauge ();
    virtual void              AddNewDataArray (sNIDAQCounter::sArrayData  Data);
    virtual void              ReportGaugeError (QString);
    void                      ReleaseGaugeControl ();

  protected: //fields
    sNIDAQCounter::sLink      Gauge;
    sPhysRange                ValidScanRange;
    sPhysRange                ValidScanSteps;
    sPhysRange                ScanRange;
    sPhysValue                ScanStep;
    sPtr<sDataArrayRecord> DataArrayRecord;
    int                       NumOfPoints;
    int                       CurrentSamplingNo;

  protected: //fields
    psPhysRangeField          ScanRangeField;
    psPhysValueField          ScanStepField;

  private slots:
    void                      LinkToGaugeAndMakeItWarming ();

  private:
    void                      Translate ();

  private: //fields
    bool                      GaugeWasStarted;
    bool                      GaugeWasInAutoRepeatMode;
    sPhysValue                OldPeriod;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
