////////////////////////////////////////////////////////////////////////////////
/*! @file NIDAQCounter.h   Работа со счетчиками платы NIDAQ.
- Part of NIDAQmxDrivers - a Research Assistant Plugin package.
- Uses  RAGUI - the Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2013, Dmitry K. Nelson, St.Petersburg, Russia.
                          D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef NIDAQCounter_H
#define NIDAQCounter_H
#include "NIDAQmxDrivers.h"
#include "Gauge.h"
#include "NIDAQmx.h"
#include "StringsAndQStrings.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sNIDAQCounter)

//------------------------------------------------------------ sNIDAQCounter ---
/*! Счетчик платы NIDAQ.



*/
class NIDAQmxDrivers_EXPORT  sNIDAQCounter : public sGauge
{
  Q_OBJECT
  DRIVER(sNIDAQCounter)

  public: // types
    class sArrayData
    {
       public:
         virtual    ~sArrayData(){
                       if (ArrBody){
                         delete [] ArrBody;
                         ArrBody = NULL;
                         ArrSize = 0;
                       }
                     }
         int         ArrSize;
         double*     ArrBody;
                     sArrayData():ArrSize(0),ArrBody(NULL){}
                     sArrayData(const sArrayData & ad){
                       this->ArrSize = ad.ArrSize;
                       this->ArrBody = new double [ad.ArrSize];
                       memcpy(this->ArrBody,ad.ArrBody,ad.ArrSize*sizeof(double));
                     }
                     int Size(){
                       return ArrSize;
                     }
                     void SetSize(int size){
                       if (ArrSize > 0){
                         delete [] ArrBody;
                         ArrBody = NULL;
                         ArrSize = 0;
                       }
                       if (size > 0){
                         ArrSize = size;
                         ArrBody = new double [ArrSize];
                         memset (ArrBody, 0, ArrSize*sizeof(double));
                       }
                     }
                     double operator [] (int I) const {
                       if (I < ArrSize){
                         return ArrBody[I];
                       }else{
                         return 0;
                       }
                     }
    };
    struct sSSParams
    {
      char Mode;
      int  Size;
      int  NumCounts;
    };

  public: //commands & directives
    static const
      sDirective<int>         cStartKinetics;//int Size
    static const
      sDirective<sSSParams>   cStart_StartStop;

  public:
    virtual                   ~sNIDAQCounter ();

  protected: //evolution of the inherited state machine
    psState                   State_O_Kinetics;
    psState                   State_O_StartStop;

  public slots:

  signals: //outgoing
      void                      NewDataArray (sNIDAQCounter::sArrayData);
      void                      GotSingleCount (sPhysValue);

  signals: //for internal needs

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  private slots:
    void                      ReadCounterSampling ();
    void                      ReadSingleCount ();
    void                      ReadKineticsSampling ();
    void                      ReadStartStopSampling ();
    void                      StartCounting ();
    void                      StartContCounting ();
    void                      StartSingleCount ();
    void                      StartKineticsCounting (int size);
    void                      Start_StartStopCounting (sSSParams params);
    void                      StopCounting ();

  private:

    static  int32     CVICALLBACK OnReadyToReadC (TaskHandle theHandle
                             ,int32 eventType
                             ,uInt32 nSamples
                             ,void *DataPtr);
    static  int32     CVICALLBACK OnReadyToReadSC (TaskHandle theHandle
                             ,int32 OK
                             ,void *DataPtr);
    static  int32     CVICALLBACK OnReadyToReadK (TaskHandle theHandle
                             ,int32 OK
                             ,void *DataPtr);
    static  int32     CVICALLBACK OnReadyToReadSS (TaskHandle theHandle
                             ,int32 OK
                             ,void *DataPtr);

  private: //fields
    bool                      Counting;
    bool                      KinMeasuring;
    int                       ArraySize;
    unsigned long             PrevCount;
    TaskHandle                CounterHandle;
    TaskHandle                TimerHandle;
    unsigned long*            readArray;
    sArrayData                DataArray;
    int                       StartStopCounts;
    char                      SSMode;
    sString                   StrobeTermName;
    sString                   SignalTermName;
};

//------------------------------------------------------------------------------
} //namespace RA

Q_DECLARE_METATYPE(RA::sNIDAQCounter::sSSParams)
Q_DECLARE_METATYPE(RA::sNIDAQCounter::sArrayData)

#endif
