////////////////////////////////////////////////////////////////////////////////
/*! @file NIDAQDoubleCounter.h  Использование двух счетчиков платы NIDAQ
          для счета сигналов. Время счета задается микроконтроллером.
- Part of NIDAQmxDrivers - a Research Assistant Plugin package.
- Uses  RAGUI - the Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2016, Dmitry K. Nelson, St.Petersburg, Russia.
                          D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef NIDAQDoubleCounter_H
#define NIDAQDoubleCounter_H
#include "NIDAQmxDrivers.h"
#include "Gauge.h"
#include "NIDAQmx.h"
#include "TeleCom.h"
#include "StringsAndQStrings.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sNIDAQDoubleCounter)

//------------------------------------------------------------ sNIDAQDoubleCounter ---
/*! Двойной счетчик платы NIDAQ.



*/
class NIDAQmxDrivers_EXPORT  sNIDAQDoubleCounter : public sGauge
{
  Q_OBJECT
  DRIVER(sNIDAQDoubleCounter)

  public: //commands & directives
    static const
      sDirective<int> cChangeCountingMode;//int Mode of counting;
                                          // 1 - NewData is from Counter A (Counter1)
                                          // 2 - NewData is from Counter B (Counter0)
                                          // 3 - NewData is (A-B)/(A+B)
  public:
    virtual                   ~sNIDAQDoubleCounter ();

  public slots:
    void                      SetInitialMode (int _mode);

  protected: //evolution of the inherited state machine
    psState                   State_W_TimerPortLinking;
    psState                   State_W_TimerPortChecking;
    psState                   State_W_SettingPeriod;
    psState                   State_O_M_SetP_Checking;


  signals: //outgoing
    void                      NewData_2 (sPhysValue);

  signals: //for internal needs
    void                      TimerPortLinked ();
    void                      TimerPortError (QString = QString());
    void                      GotTimerMessage (sTelex);
    void                      NoAnswerFromTimer ();
    void                      GotSingleCount(sPhysValue);

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  protected slots: //reimplementing inherited
    virtual void              OnEnterToCoolingState ();
    virtual void              SetWantedPeriod ();

  protected slots:
    virtual void              ReportTimerPortError (QString);
    virtual void              OnNoAnswerFromTimer ();
    virtual void              AskTimerName ();
    virtual void              CorrectPeriod (sTelex);
    virtual void              SetCountingMode (int);

  protected: //fields
    sTeleCom::sLink           TimerPort;

  private slots:
    void                      LinkToTimerPort ();
    void                      ReadCounter1Sampling ();
    void                      ReadCounter0Sampling ();
    void                      CalcOutput();
    void                      StartCounting ();
    void                      StartContCounting ();
    void                      StartSingleCount ();
    void                      StopCounting ();

  private:

    static  int32     CVICALLBACK OnReadyToReadC1 (TaskHandle theHandle
                             ,int32 eventType
                             ,uInt32 nSamples
                             ,void *DataPtr);
    static  int32     CVICALLBACK OnReadyToReadC0 (TaskHandle theHandle
                             ,int32 eventType
                             ,uInt32 nSamples
                             ,void *DataPtr);

  private: //fields
    sString                   TimerName;
    bool                      Counting;
    unsigned long             PrevCount1;
    unsigned long             PrevCount0;
    long                      Count1;
    long                      Count0;
    bool                      C1Ready;
    bool                      C0Ready;
    TaskHandle                Counter1Handle;
    TaskHandle                Counter0Handle;
    int                       MeasuringMode;
    sString                   Timer1TermName;
    sString                   Timer0TermName;
};

//------------------------------------------------------------------------------
} //namespace RA

#endif
