////////////////////////////////////////////////////////////////////////////////
/*! @file NIDAQVoltmeter.h   Вольтметр на базе платы NIDAQ.
- Part of NIDAQmxDrivers - a Research Assistant Plugin package.
- Uses  RAGUI - the Research Assistant Graphical User Interface.
- Uses  QtGui v.5.0  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2014, Dmitry K. Nelson, St.Petersburg, Russia.
                          D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef NIDAQVoltmeter_H
#define NIDAQVoltmeter_H
#include "NIDAQmxDrivers.h"
#include "Gauge.h"
#include "NIDAQmx.h"
#include <Ranges.h>

namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sNIDAQVoltmeter)

//---------------------------------------------------------- sNIDAQVoltmeter ---
/*! Вольтметр на базе платы NIDAQ.



*/
class NIDAQmxDrivers_EXPORT  sNIDAQVoltmeter : public sGauge
{
  Q_OBJECT
  DRIVER(sNIDAQVoltmeter)

  public:
    virtual                   ~sNIDAQVoltmeter ();

  public slots:

  signals: //outgoing
    void                      GotSingleShot (sPhysValue);

  signals: //for internal needs

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  protected slots: //reimplementing inherited

  protected: //fields
    sPhysRange                MeasuringRange;
    sPhysValue                SamplingPeriod;

  private slots:
    void                      ReadSampling ();
    void                      ReadSingleShotSampling ();
    void                      Start ();
    void                      Stop ();

  private:

    static  int32     CVICALLBACK OnReadyToRead (TaskHandle theHandle
                             ,int32 eventType
                             ,uInt32 nSamples
                             ,void *DataPtr);
    static  int32     CVICALLBACK OnReadyToReadSS (TaskHandle theHandle
                             ,int32 OK
                             ,void *DataPtr);

  private: //fields
    TaskHandle                VoltmeterHandle;
    int                       NumOfSamples;
    float64*                  readArray;
};

//------------------------------------------------------------------------------
} //namespace RA

#endif
