////////////////////////////////////////////////////////////////////////////////
/*! @file P138Magnet.h   Driver for Superconducting magnet.
- Part of NIDAQmxDrivers - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.5.0  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2013, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
- Copyright(C) 2014, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef P138Magnet_H
#define P138Magnet_H
#include "NIDAQmxDrivers.h"
#include "Effectors.h"
#include "NIDAQmx.h"
namespace RA {
//-----------------------------------------------------------------------------

ANNOUNCE_CLASS(sP138Magnet)

//----------------------------------------------------------- sP138Magnet ---
/*!

*/
class  NIDAQmxDrivers_EXPORT  sP138Magnet : public sEffector
{
  Q_OBJECT
  DRIVER(sP138Magnet)
  
  public:
    virtual                   ~sP138Magnet ();


  protected: //evolution of the inherited state machine
    psState                   State_W_CreateTask;
    psState                   State_O_M_Pausing;
    psState                   State_O_SettingTo;
    psState                   State_O_SettingTo_CheckValue;
    psState                   State_O_SettingTo_ChangeValue;

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  signals: //for internal needs
    void                      TaskOK();
    void                      FieldOK();
    void                      FieldNotYet();

  protected slots: //reimplementing inherited
    virtual void              SetWantedPeriod ();

  protected slots:
    void                      CreateNIDAQTask ();
    void                      MeasureField ();
    void                      SetDestination (sPhysValue _newField);
    void                      SayDone ();
    void                      ChangeOutput ();
    void                      CheckField (sPhysValue _field);

  protected:

  protected: //fields
    sPhysValue                MagnetCoeff;
    sPhysValue                MaxInput;
    sPhysValue                MaxOutput;
    sPhysValue                MeasuringTime;
    sPhysValue                Tolerance;

  private slots:
    void                      ReadSampling ();

  private:

    static  int32     CVICALLBACK OnReadyToRead (TaskHandle theHandle
                             ,int32 eventType
                             ,void *DataPtr);

  private: //fields
    sPhysValue                Destination;
    float64                   OutValue;
    float64                   MaxOut;
    int                       OutputSign;
    int                       TimeDelay;
    TaskHandle                MeasureHandle;
    TaskHandle                SettingHandle;
    int                       NumOfSamples;
    float64*                  readArray;
    bool                      Up;
    bool                      Down;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
