////////////////////////////////////////////////////////////////////////////////
/*! @file StartStopNIDAQ.cpp  Измеритель кинетики NIDAQ методом Старт-Стоп.
- Part of NIDAQmxDrivers - a Research Assistant Plugin package.
- Uses  RAGUI - the Research Assistant Graphical User Interface.
- Uses  QtGui v.5.0  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2014, Dmitry K. Nelson, St.Petersburg, Russia.
                          D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "StartStopNIDAQ.h"
#include "attr.h"
#include "StringsAndQStrings.h"
#include <QMessageBox>
#include <QDebug>


namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sStartStopNIDAQ)

//------------------------------------------------------------------ sStartStopNIDAQ ---

void  sStartStopNIDAQ::Preconfigure (sNodePtr  DeviceCfg)
{
  sPtr<sFolderNode>              DeviceSettings;
  sPtr<sFolderNode>              DeviceLinks;
  DeviceCfg |= "Links";
  DeviceLinks = DeviceCfg >> "Links";
  if( DeviceLinks.IsCorrect() ){
      DeviceLinks |= "Gauge";
      DeviceLinks >> "Gauge" |= attr("Class","sNIDAQCounter");
  };
  DeviceCfg |= "Settings";
  DeviceSettings = DeviceCfg >> "Settings";
  if( DeviceSettings.IsCorrect() ){
    //изменение умолчательных шкал для sScanner
    sChartScales      ChartScales(sChartScale("Time"
                                              ,sPhysRange(0,1.0e-6,"s"))
                                 ,sChartScale(""
                                             ,sPhysRange(0,200,"kphoton/s"))
                                 );
    ChartScales.StoreSettings(DeviceSettings);

    DeviceSettings |= attr("ValidScanRange",sPhysRange(0.0,real_inf,"s"));
    DeviceSettings |= attr("ValidScanSteps",sPhysRange(1.25e-8,5.00e-8,"s"));
    DeviceSettings |= attr("ScanRange",sPhysRange(0.0,1.0e-5,"s",1.0e-9));
    DeviceSettings |= attr("Mode",0);
    DeviceSettings |= attr("NumOfCountsNo",2);
    DeviceSettings |= attr("TimeBaseNo1",80);
    DeviceSettings |= attr("TimeBaseNo2",20);
  };
  sKineticsNIDAQ::Preconfigure(DeviceCfg);
}

sStartStopNIDAQ::~sStartStopNIDAQ()
{
/* Здесь это не нужно
  if( DeviceSettings.IsCorrect() ){
    DeviceSettings << attr("Mode",FldMode->currentIndex());
    DeviceSettings << attr("NumOfCountsNo",FldNumOfCounts->currentIndex());
  };
*/
  Gauge.SetToNULL();
}

sStartStopNIDAQ::sStartStopNIDAQ (sString    DevicetId
                                 ,sNodePtr   DeviceCfg
                                 ,sNodePtr   WidgetCfg
                                 ,QWidget *  parent   )
    :sKineticsNIDAQ(DevicetId,DeviceCfg,WidgetCfg,parent)
    ,NumOfCNo(2)
    ,Mode(0)
{
    TimeBases[0] = 80;
    TimeBases[1] = 20;
    NumOfCounts[0] = 10;
    NumOfCounts[1] = 100;
    NumOfCounts[2] = 1000;
}

void  sStartStopNIDAQ::AppearanceMakeUp ()
{
  sKineticsNIDAQ::AppearanceMakeUp();
  try{ //get config parameters
    get_from<sIntegerNode>(DeviceSettings >> "Mode" ,Mode);
    get_from<sIntegerNode>(DeviceSettings >> "NumOfCountsNo" ,NumOfCNo);
    get_from<sIntegerNode>(DeviceSettings >> "TimeBaseNo1" ,TimeBases[0]);
    get_from<sIntegerNode>(DeviceSettings >> "TimeBaseNo2" ,TimeBases[1]);
  }catch(...){
    SetInvalid(tr("Something wrong in configuration."));
  }
  //add new widgets
  {
    LblMode = new QLabel(ParamArea);
    LblMode->setObjectName("LblMode");
    ParamLayout->addWidget(LblMode,Row,0,1,1);
    FldMode = new QComboBox(ParamArea);
    FldMode->setObjectName("FldMode");
    FldMode->insertItem(0,"Mode1");
    FldMode->insertItem(1,"Mode2");
    FldMode->setCurrentIndex(Mode);
    ParamLayout->addWidget(FldMode,Row,1,1,1);

    LblNumOfCounts = new QLabel(ParamArea);
    LblNumOfCounts->setObjectName("LblNumOfCounts");
    ParamLayout->addWidget(LblNumOfCounts,Row,0,1,1);
    FldNumOfCounts = new QComboBox(ParamArea);
    FldNumOfCounts->setObjectName("FldNumOfCounts");
    FldNumOfCounts->insertItem(0,"10");
    FldNumOfCounts->insertItem(1,"100");
    FldNumOfCounts->insertItem(2,"1000");
    FldNumOfCounts->setCurrentIndex(NumOfCNo);
    ParamLayout->addWidget(FldNumOfCounts,Row,1,1,1);

    QString SS;
    for (int Index = 0; Index <=1; Index++ ) {
        SS.setNum(TimeBases[Index]);
        FldMode->setItemText(Index,SS + " MHz");
    }
  };
  //modify widgets
  {
  ScanStep = sPhysValue(1.0e-6/TimeBases[Mode], "s");
  ScanStepField->SetValue(ScanStep);
  ScanStepField->setEnabled(false);  //This field should be made not editable !!!
  Translate();
  };
  //connect new widgets
  {
  connect(FldMode,SIGNAL(currentIndexChanged(int)) ,this,SLOT(OnModeChanged(int)));
  connect(FldNumOfCounts,SIGNAL(currentIndexChanged(int)) ,this,SLOT(OnNumOfCountsChanged(int)));
  }
}

void  sStartStopNIDAQ::Translate ()
{
  LblMode->setText(tr("Mode"));
  LblNumOfCounts->setText(tr("Number of counts"));
}

void  sStartStopNIDAQ::RestoreState ()
{
  sKineticsNIDAQ::RestoreState();
  if( ! WidgetConfig.IsNULL() ){
    try{ //get stored parameters
      get_from<sIntegerNode>(DeviceSettings >> "Mode" ,Mode);
      get_from<sIntegerNode>(DeviceSettings >> "NumOfCountsNo" ,NumOfCNo);
      get_from<sIntegerNode>(DeviceSettings >> "TimeBaseNo1" ,TimeBases[0]);
      get_from<sIntegerNode>(DeviceSettings >> "TimeBaseNo2" ,TimeBases[1]);

      QString SS;
      for (int Index = 0; Index <=1; Index++ ) {
        SS.setNum(TimeBases[Index]);
        FldMode->setItemText(Index,SS + " MHz");
      }
      FldMode->setCurrentIndex(Mode);
      FldNumOfCounts->setCurrentIndex(NumOfCNo);
      ScanStep = sPhysValue(1.0e-6/TimeBases[Mode], "s");
      ScanStepField->SetValue(ScanStep);
      ScanStepField->setEnabled(false);  //This field should be made not editable !!!
    }catch(...){
      SetInvalid(tr("Something wrong in desktop stored."));
    }
  }
}

void  sStartStopNIDAQ::StoreState ()
{
  sKineticsNIDAQ::StoreState();
  if( ! WidgetConfig.IsNULL() ){
    WidgetConfig << attr("Mode",FldMode->currentIndex ());
    WidgetConfig << attr("NumOfCountsNo",FldNumOfCounts->currentIndex ());
  };
  if( DeviceSettings.IsCorrect() ){
    //здесь это работает
    DeviceSettings << attr("Mode",FldMode->currentIndex ());
    DeviceSettings << attr("NumOfCountsNo",FldNumOfCounts->currentIndex ());
  };
}

void  sStartStopNIDAQ::StartScanning ()
{
  CurrentSamplingNo=0;
  sNIDAQCounter::sSSParams _Params;
  _Params.Mode = Mode;
  _Params.NumCounts = NumOfCounts[NumOfCNo];
  _Params.Size = NumOfPoints;
  Connect(Gauge ,SIGNAL(NewDataArray (sNIDAQCounter::sArrayData))
                ,SLOT(AddNewDataArray(sNIDAQCounter::sArrayData)) );
  Gauge->SetAutoRepeat(true);
  SendDirective(Gauge,sNIDAQCounter::cStart_StartStop,_Params);
}

void  sStartStopNIDAQ::OnModeChanged(int NewIndex)
{
    Mode = NewIndex;
    ScanStep = sPhysValue(1.0e-6/TimeBases[Mode], "s");
    ScanStepField->SetValue(ScanStep);
    ScanStepField->update();
}

void  sStartStopNIDAQ::OnNumOfCountsChanged(int NewIndex)
{
    NumOfCNo = NewIndex;
}
//------------------------------------------------------------------------------
} //namespace RA
