////////////////////////////////////////////////////////////////////////////////
/*! @file DNSpectroScanner.h   Спектрометр непрерывного сканирования.
- Снабжен драйвером шагового двигателя, сообщающим свое положение.
- Part of Nelson - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.5.0  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2013, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
- Copyright(C) 2014, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef DNSpectroScanner_H
#define DNSpectroScanner_H
#include "Nelson.h"
#include "PeriodicGaugeRecorder.h"
//#include "DataListRecord.h"
//#include "Gauge.h"
#include "Effectors.h"

//#include <QTime>

namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sDNSpectroScanner)

//-------------------------------------------------------- sDNSpectroScanner ---
/*! Сканер с шаговым двигателем.
*/
class Nelson_EXPORT  sDNSpectroScanner : public sPeriodicGaugeRecorder
{
  Q_OBJECT
  CONSOLE(sDNSpectroScanner)
  public:
    virtual                   ~sDNSpectroScanner ();
    virtual void              RestoreState ();
    virtual void              StoreState ();

  protected: //evolution of the inherited state machine
    psState                   State_W_Stepper;
    psState                   State_W_Stepper_Warming;
    psState                   State_O_Sc_GoToTheStart;
    psState                   State_O_Sc_SettingTimePerStep;
    psState                   State_O_Sc_A_Parallel;
    psState                   State_O_Sc_A_P_Measuring;
    psState                   State_O_Sc_A_P_Stepping;
    psState                   State_O_Sc_A_P_M_WaitingFin;
    psState                   State_O_Sc_A_P_S_WaitingFin;

  signals: //for internal needs
    void                      StepperWarmed ();
    void                      StepperError(QString = QString());
    void                      DevicesBusy();
    void                      StepperIsAtPos(sPhysValue);
    void                      StepperTimePerStepSet(sPhysValue);
    void                      MeasuringCompleted();
    void                      SteppingCompleted();

  protected: //reimplementing inherited
    virtual void              AppearanceMakeUp ();
    virtual void              Retranslate ()
                                {
                                  sPeriodicGaugeRecorder::Retranslate();
                                  Translate();
                                }

  protected slots: //reimplementing inherited
    virtual void              OnEnterToCoolingState ();
    virtual void              EarlySetAttributes ();
    virtual void              StartScanning ();
    virtual void              StopScanning ();
    virtual void              SetGaugeRegime ();
    virtual void              CorrectStep (sPhysValue  /*gauge_period*/);
    virtual void              AddNewData (sPhysValue  Data);

  protected slots:
    virtual void              SetStepperToStartPos();
    virtual void              SetStepperTimePerStep();
    virtual void              AcceptStepperTimePerStep(sPhysValue TPSmod);
    virtual void              ReportStepperError (QString);
    virtual void              LinkToStepper ();
    virtual void              SayScanIsCompleted();
    virtual void              BreakScanning ();
    void                      StopStepper ();
    void                      ReleaseStepperControl ();

  protected: //fields
    sStepper::sLink           Stepper;
    sPhysValue                MeasureTime;

  protected: //fields
    psPhysValueField          MeasureTimeField;

  private:
    void                      Translate ();

  private: //fields
    sPhysValue                TimePerMotorStep;
    sPhysValue                TimeCorrCoef;
    int                       NumOfPoints;
    int                       NumOfStepsFrom;
    int                       NumOfStepsToScan;
// For Scan time measuring:
//    QTime                     DataCollectTime;
//    QTime                     SteppingTime;
//  private slots:
//    void                      ScanningTimeShow ();

};

//------------------------------------------------------------------------------
} //namespace RA
#endif
