////////////////////////////////////////////////////////////////////////////////
/*! @file DigitalLIA.h   Driver for the Digital Lock-In-Amplifier.
- Part of Nelson - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.5.0  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2013, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
- Copyright(C) 2017, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef DigitalLIA_H
#define DigitalLIA_H
#include "Nelson.h"
#include "Gauge.h"
#include "TeleCom.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sDigitalLIA)

//--------------------------------------------------------------- DigitalLIA ---
/*!

*/
class  Nelson_EXPORT  sDigitalLIA : public sGauge
{
Q_OBJECT
  DRIVER(sDigitalLIA)

  public: //commands & directives
    static const
      sDirective<int> cSetGain;//int Gain index;
                               // 0 - Gain = 10^0*10=10
                               // 1 - Gain = 10^1*10=100
                               // 2 - Gain = 10^2*10=1000
                               // 3 - Gain = 10^3*10=10000
  public:
    virtual                   ~sDigitalLIA ();

  protected: //evolution of the inherited state machine
    psState                   State_W_Linking;
    psState                   State_W_Checking;
    psState                   State_W_SettingPeriod;
    psState                   State_O_M_SetP_Checking;

  signals: //outgoing
    void                      NewAmplitude (sPhysValue);



  signals: //for internal needs
    void                      LIADeviceLinked ();
    void                      LIADeviceError(QString = QString());
    void                      GotLIADeviceMessage(sTelex);

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  protected slots: //reimplementing inherited
    virtual void              SetWantedPeriod ();

  protected slots:
    virtual void              ReportLIADeviceError (QString);
    virtual void              LinkToLIADevice ();
    virtual void              AskLIADeviceName ();
    virtual void              CorrectPeriod (sTelex);
    virtual void              SetGain (int);
    virtual void              CalcOutput(sTelex);
    virtual void              StartMeasuring ();
    virtual void              StopMeasuring ();

  protected:

  protected: //fields
    sTeleCom::sLink           LIADeviceCom;
    sString                   LIADeviceName;
    int                       GainIndex;
    sPhysValue                Multiplier[4];

  private:
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
