////////////////////////////////////////////////////////////////////////////////
/*! @file DigitalLIADisplay.cpp Дисплей для отображения измерителя DigitalLIA.
- Part of Nelson - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2017,      Dmitry K. Nelson, St.Petersburg, Russia.
                          D.Nelson@mail.ioffe.ru
- Copyright(C) 2013,      Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "DigitalLIADisplay.h"
#include "attr.h"
#include "StringsAndQStrings.h"
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QFormLayout>
#include <QSpacerItem>
#include <QMessageBox>
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sDigitalLIADisplay)

//---------------------------------------------------- sDigitalLIADisplay -----

void  sDigitalLIADisplay::Preconfigure (sNodePtr  DeviceCfg)
{
/*
  sPtr<sFolderNode>              DeviceSettings;
  DeviceCfg |= "Settings";
  DeviceSettings = DeviceCfg >> "Settings";
  if( DeviceSettings.IsCorrect() ){
    DeviceSettings |= attr("GainIndex",0);
  };
*/
  sGaugeDisplay::Preconfigure(DeviceCfg);
}


sDigitalLIADisplay::sDigitalLIADisplay (sString   DeviceId
                                       ,sNodePtr  DeviceCfg
                                       ,sNodePtr  WidgetCfg
                                       ,QWidget *  parent    )
    :sGaugeDisplay(DeviceId,DeviceCfg,WidgetCfg,parent)
    ,LIAGauge(this)
{
}

sDigitalLIADisplay::~sDigitalLIADisplay()
{
/*
  if( DeviceSettings.IsCorrect() ){
    DeviceSettings
        << attr("GainIndex",GainIndex);
  }
*/
  LIAGauge.SetToNULL();
}

void  sDigitalLIADisplay::AppearanceMakeUp ()
{
/*
  try{ //get config parameters
    get_from<sIntegerNode>(DeviceSettings >> "GainIndex"
                                             ,GainIndex);
  }catch(...){
    SetInvalid(tr("Something wrong in configuration."));
  }
*/
  setLayout(new QVBoxLayout);

  ValueLCD = new QLCDNumber(this);
  ValueLCD->setSegmentStyle(QLCDNumber::Flat);
  layout()->addWidget(ValueLCD);

  QHBoxLayout *               L( new QHBoxLayout );
  layout()->addItem(L);
  UnitsDisplay = new QLabel(this);
  L->addStretch();
  L->addWidget(UnitsDisplay);
  UnitsDisplay->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);

  ShowNaN();

  AmplitudeLCD = new QLCDNumber(this);
  AmplitudeLCD->setSegmentStyle(QLCDNumber::Flat);
  layout()->addWidget(AmplitudeLCD);
  NormalStyle = AmplitudeLCD->styleSheet();

  QHBoxLayout *               L_2( new QHBoxLayout );
  layout()->addItem(L_2);
  UnitsDisplay_A = new QLabel(this);
  L_2->addStretch();
  L_2->addWidget(UnitsDisplay_A);
  UnitsDisplay_A->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);

  ShowNaN_A();

  BtnReset = new QPushButton(this);
  BtnReset->setText("Reset Amplitude Indication");
  layout()->addWidget(BtnReset);
  connect(BtnReset ,SIGNAL(clicked()) ,this ,SLOT(OnReset()));

  QFormLayout *               M( new QFormLayout );
  layout()->addItem(M);
  LblGain = new QLabel(this);
  LblGain->setText(tr("Gain"));
  M->addWidget(LblGain);
  FldGain = new QComboBox(this);
  FldGain->insertItem(0,"1");
  FldGain->insertItem(1,"10");
  FldGain->insertItem(2,"100");
  FldGain->insertItem(3,"1000");
  FldGain->setCurrentIndex(0);
  M->addWidget(FldGain);
//  Translate();

  connect(FldGain,SIGNAL(currentIndexChanged(int)) ,this,SLOT(OnGainChanged(int)));
}

void sDigitalLIADisplay::LinkToGauge ()
{
  sGaugeDisplay::LinkToGauge ();
  LIAGauge.SetTo(Gauge->DeviceIdentifier());
  if( ! LIAGauge.IsCorrect() ){
    emit ErrorMessage( tr("The 'DigitalLIA' link is set to '%1', which "
                          "does not look correct. Check configuration.")
                         .arg(ToQString(Gauge->DeviceIdentifier())) );
  }
}

void sDigitalLIADisplay::ShowData (sPhysValue  V)
{
  V.Normalize();
  QString            T( ToQString( (V/sPhysValue(1.0,V.Units())).Text() ) );
  ValueLCD->setDigitCount(T.length());
  ValueLCD->display(T);
  UnitsDisplay->setText("Signal, " + ToQString( V.Units().Text(HTML) ));
}

void sDigitalLIADisplay::ShowAmplitude (sPhysValue V)
{
  if ((V /sPhysValue(1.0,"V")).ToReal() > 5.0)
    AmplitudeLCD->setStyleSheet("background: red");
  V.Normalize();
  QString            T( ToQString( (V/sPhysValue(1.0,V.Units())).Text() ) );
  AmplitudeLCD->setDigitCount(T.length());
  AmplitudeLCD->display(T);
  UnitsDisplay_A->setText("Amplitude, "+ ToQString( V.Units().Text(HTML) ));
}


void sDigitalLIADisplay::ShowNaN ()
{
  ValueLCD->display(QString("-----"));
  UnitsDisplay->setText("");
}

void sDigitalLIADisplay::ShowNaN_A ()
{
  AmplitudeLCD->display(QString("-----"));
  UnitsDisplay_A->setText("");
}

void sDigitalLIADisplay::OnEnterToCoolingState ()
{
  LIAGauge.SetToNULL();
  sGaugeDisplay::OnEnterToCoolingState();
}

void sDigitalLIADisplay::OnEnterToOperatingState()
{
  sGaugeDisplay::OnEnterToOperatingState();
  Connect(Gauge ,SIGNAL(NewAmplitude(sPhysValue)) ,SLOT(ShowAmplitude(sPhysValue)) );
  SendDirective<int>(LIAGauge,sDigitalLIA::cSetGain,0);
}


void sDigitalLIADisplay::OnExitFromOperatingState()
{
  Disconnect(Gauge ,SIGNAL(NewAmplitude(sPhysValue)) ,SLOT(ShowAmplitude(sPhysValue)) );
  ShowNaN_A();
  sGaugeDisplay::OnExitFromOperatingState();
}

void sDigitalLIADisplay::OnGainChanged(int _newGain)
{
//  SendCommand(Gauge,sGauge::cStop);
  SendDirective<int>(LIAGauge,sDigitalLIA::cSetGain,_newGain);
//  SendCommand(Gauge,sGauge::cStart);
}

void sDigitalLIADisplay::OnReset()
{
  AmplitudeLCD->setStyleSheet(NormalStyle);
}

//------------------------------------------------------------------------------
} //namespace RA


