////////////////////////////////////////////////////////////////////////////////
/*! @file DigitalLIADisplay.h Дисплей для отображения измерителя DigitalLIA.
- Part of Nelson - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2017,      Dmitry K. Nelson, St.Petersburg, Russia.
                          D.Nelson@mail.ioffe.ru
- Copyright(C) 2013,      Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef DigitalLIADisplay_H
#define DigitalLIADisplay_H
#include "Nelson.h"
#include "GaugeDisplay.h"
#include "DigitalLIA.h"
#include <QLCDNumber>
#include <QComboBox>
#include <QPushButton>

namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sDigitalLIADisplay)

//---------------------------------------------------- sDigitalLIADisplay ---
/*! Дисплей для отображения измерителя DigitalLIA.

Показывает отсчёты измерителя по мере их поступления
Позволяет переключать усиление DigitalLIA.

*/
class  Nelson_EXPORT  sDigitalLIADisplay : public sGaugeDisplay
{
  Q_OBJECT
  CONSOLE(sDigitalLIADisplay)

  public:
    virtual                   ~sDigitalLIADisplay ();

  protected: //reimplementing inherited
    virtual void              AppearanceMakeUp ();

  protected slots: //reimplementing inherited
    virtual void              ShowData (sPhysValue);
    virtual void              ShowNaN ();
    virtual void              OnEnterToOperatingState();
    virtual void              OnExitFromOperatingState();
    virtual void              OnEnterToCoolingState ();
    virtual void              OnGainChanged(int _newGain);
    virtual void              LinkToGauge ();

  protected slots:
    virtual void              ShowAmplitude (sPhysValue);
    virtual void              ShowNaN_A ();
    virtual void              OnReset();


  protected: //fields
    sDigitalLIA::sLink        LIAGauge;

  private: //fields
    QLCDNumber *              ValueLCD;
    QLabel *                  UnitsDisplay;
    QLCDNumber *              AmplitudeLCD;
    QLabel *                  UnitsDisplay_A;
    QLabel *                  LblGain;
    QComboBox *               FldGain;
    QPushButton *             BtnReset;
    QString                   NormalStyle;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
