////////////////////////////////////////////////////////////////////////////////
/*! @file FixedWLDriver.cpp Driver for the spectrometer with manual wavelength setting.
- Part of Nelson - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.5.0  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2013, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
- Copyright(C) 2020, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "FixedWLDriver.h"
#include "attr.h"
#include "StringsAndQStrings.h"
#include <QDebug>
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sFixedWLDriver)

//--------------------------------------------------------------- FixedWLDriver ---

void  sFixedWLDriver::Preconfigure (sNodePtr  DeviceCfg)
{
  sPtr<sFolderNode>              DeviceSettings;
  DeviceCfg |= "Settings";
  DeviceSettings = DeviceCfg >> "Settings";
  if( DeviceSettings.IsCorrect() ){
    DeviceSettings |= attr("Range",sPhysRange(10,1000,"nm"));
    DeviceSettings |= attr("WavelengthSetting",sPhysValue(500.0,0.01,"nm"));
  };
  sEffector::Preconfigure(DeviceCfg);
}

sFixedWLDriver::sFixedWLDriver(sString   DevicetId
                        ,sNodePtr  DeviceCfg)
    :sEffector(DevicetId,DeviceCfg)
    ,WavelengthPosition(sPhysValue(500.0,0.01,"nm"))
{
}

sFixedWLDriver::~sFixedWLDriver()
{
}

void  sFixedWLDriver::PostConstructor ()
{
  sEffector::PostConstructor();

  try{ //get config parameters
    get_from<sPhysValueNode>(DeviceSettings >> "WavelengthSetting" ,WavelengthPosition);
  }catch(...){
    SetInvalid(tr("Something wrong in configuration."));
  }

  //evolution of the inherited state machine (behaviour)
  {
    //evolution of the State_Warming
    {
      State_Warming()
          ->OnExitCall(SLOT(SayPosition()));
    }
    //evolution of the State_Operating
    {
      State_O_M_Pausing
          = new sState(State_O_Measuring,ThePeriod,"State_O_M_Pausing");
      State_O_M_Acquiring
          ->OnEnterCall( SLOT(SayPosition()) )
          ->AddTransition(State_O_M_Pausing)
          ->AddTransition(SIGNAL(ErrorMessage(QString))
                         ,State_Cooling())
          ;
      State_O_M_Pausing
          ->AddTransition(State_O_M_Pausing,SIGNAL(Timeout())
                         ,State_O_M_Cycling)
          ->AddTransition(SIGNAL(ErrorMessage(QString))
                         ,State_Cooling())
          ;
    }
  }
}

void sFixedWLDriver::SayPosition()
{
  emit NewData(WavelengthPosition);
}


//------------------------------------------------------------------------------
} //namespace RA




