////////////////////////////////////////////////////////////////////////////////
/*! @file MDRSpectroScanner.cpp   Спектрометр МДР непрерывного сканирования.
- Снабжен драйвером шагового двигателя, сообщающим свое положение.
- Имеет индикацию установленной решетки
- Part of Nelson - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.5.0  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2013, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
- Copyright(C) 2014, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "MDRSpectroScanner.h"
#include "attr.h"
#include "StringsAndQStrings.h"
#include "Trace.h"
#include "Log.h"
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sMDRSpectroScanner)

//--------------------------------------------------- sMDRSpectroScanner ---

void  sMDRSpectroScanner::Preconfigure (sNodePtr  DeviceCfg)
{
  sDNSpectroScanner::Preconfigure(DeviceCfg);
}


sMDRSpectroScanner::sMDRSpectroScanner (sString    DeviceId
                                       ,sNodePtr   DeviceCfg
                                       ,sNodePtr   WidgetCfg
                                       ,QWidget *  parent   )
    :sDNSpectroScanner(DeviceId,DeviceCfg,WidgetCfg,parent)
    ,WLCoefStore(1)
{
  //evolution of the inherited state machine
  {
    // evolution of State_O_Idle
    {
       State_O_Idle
           ->AddReaction<char>(SIGNAL(GratingNoValue (char))
                                 ,SLOT(ShowGratingNo(char)));
    };
  };
}

void  sMDRSpectroScanner::AppearanceMakeUp ()
{
  sDNSpectroScanner::AppearanceMakeUp();

  //add new widgets
  {
    LblGrTitle = new QLabel(ParamArea);
    LblGrTitle->setObjectName("LblGrTitle");
    ParamLayout->addWidget(LblGrTitle,Row,0,1,1);
    LblGrIndicator = new QLabel("1",ParamArea);
    LblGrIndicator->setObjectName("LblGrIndicator");
    ParamLayout->addWidget(LblGrIndicator,Row++,1,1,1);
  };
  Translate();
  //connect new widgets
  {
  };
}

void  sMDRSpectroScanner::Translate ()
{
  LblGrTitle->setText(tr("GratingNo"));
}

void  sMDRSpectroScanner::EarlySetAttributes ()
{
  sDNSpectroScanner::EarlySetAttributes ();
  Dependence << attr("Grating No", FromQString(LblGrIndicator->text()));
}

void sMDRSpectroScanner::LinkToStepper ()
{
  sDNSpectroScanner::LinkToStepper ();
  Connect(Stepper ,SIGNAL(TheGratingNo(char))  ,SIGNAL(GratingNoValue(char)) );
}

void sMDRSpectroScanner::ShowGratingNo(char GrNo)
{
    int WLCoef = 1;
    switch (GrNo) {
        case '1' : {
            LblGrIndicator->setText(QString(GrNo));
            WLCoef = 1;
            break;
        }
        case '2' : {
            LblGrIndicator->setText(QString(GrNo));
            WLCoef = 1;
            break;
        }
        case '3' : {
            LblGrIndicator->setText(QString(GrNo));
            WLCoef = 2;
            break;
        }
        case '4' : {
            LblGrIndicator->setText(QString(GrNo));
            WLCoef = 4;
            break;
        }
        case 'E' : {
            LblGrIndicator->setText("Error");
            WLCoef = 1;
            break;
        }
    }
    if (WLCoef != WLCoefStore) {
        ScanRangeField->SetValidRange(sPhysRange(WLCoef*ValidScanRange.From()
                                                ,WLCoef*ValidScanRange.To()));
        WLCoefStore = WLCoef;
    }
}

//------------------------------------------------------------------------------
} //namespace RA
