################################################################################
#//! @file MSDN2044.mk
#//! @brief
#//!                      MSDN firmware makefile.
#//!
#//! - A complimentary part of the Research Assistant project.
#//! - MSDN stands for Microsin & D.Nelson.  It is a series of periferial
#//!   devices, which are
#//!   - based on Microsin prototyping boards (http://microsin.net)
#//!   - and extended by Dmitry K. Nelson, St.Petersburg, Russia,
#//!                          D.Nelson@mail.ioffe.ru.
#//! - This firmware uses LUFA Library written by
#//!   - Copyright(C) 2012, Dean Camera, dean [at] fourwalledcubicle [dot] com,
#//!                                      www.lufa-lib.org.
#//! - This makefile is based on the examples provided with LUFA Library.
#//! - It has been modified by
#//!   - Copyright(C) 2014, D.K. Nelson, St.Petersburg, Russia.
#//!                        D.Nelson@mail.ioffe.ru.
#//!   - Copyright(C) 2016, Viktor E. Bursian, St.Petersburg, Russia,
#//!                        VBursian@gmail.com.
################################################################################

MSDN_PID = 2044
         # ^^^^ - 4 hexadecimal digits
MSDN_DESCRIPTION = DN StepMotor Driver

CC_FLAGS    += -DCR_LF_ENDINGS  #comment it out to use LF message endings

LUFA_PATH      = ../../../../../Libraries/LUFA120730/LUFA
MSDN_LUFA_PATH = ../../../MSDN_LUFA

MCU          = at90usb162
ARCH         = AVR8
BOARD        = MICROSIN162
F_CPU        = 16000000
F_USB        = $(F_CPU)
OPTIMIZATION = s

MSDN_SHORTNAME = MSDN$(MSDN_PID)
MSDN_LONGNAME  = $(MSDN_SHORTNAME) $(MSDN_DESCRIPTION)

TARGET       = $(MSDN_SHORTNAME)
SRC         += $(MSDN_SHORTNAME).c
SRC         += $(MSDN_LUFA_PATH)/CommandParser.c
SRC         += $(MSDN_LUFA_PATH)/Microsin_USB_IO.c
SRC         += $(MSDN_LUFA_PATH)/DescriptorsMSDN.c
SRC         += $(LUFA_SRC_USB) $(LUFA_SRC_USBCLASS)
CC_FLAGS    += -DMSDN_PID=$(MSDN_PID)
CC_FLAGS    += -DMSDN_SHORTNAME="$(MSDN_SHORTNAME)"
CC_FLAGS    += -DMSDN_LONGNAME="$(MSDN_LONGNAME)"
CC_FLAGS    += -I$(MSDN_LUFA_PATH)/
CC_FLAGS    += -DUSE_LUFA_CONFIG_HEADER
LD_FLAGS     =

# Default target
all:

# Include LUFA build script makefiles
include $(LUFA_PATH)/Build/lufa_core.mk
include $(LUFA_PATH)/Build/lufa_sources.mk
include $(LUFA_PATH)/Build/lufa_build.mk
include $(LUFA_PATH)/Build/lufa_cppcheck.mk
include $(LUFA_PATH)/Build/lufa_doxygen.mk
include $(LUFA_PATH)/Build/lufa_dfu.mk
include $(LUFA_PATH)/Build/lufa_hid.mk
include $(LUFA_PATH)/Build/lufa_avrdude.mk
include $(LUFA_PATH)/Build/lufa_atprogram.mk
