////////////////////////////////////////////////////////////////////////////////
/*! @file MSDN2045.c  Firmware for Wavelength Counter.
- Part of the MSDN firmware.
- Part of the Research Assistant project.
- Uses  LUFA library written by
   - Copyright(C) 2012, Dean Camera, dean [at] fourwalledcubicle [dot] com,
                                     www.lufa-lib.org.

- Copyright(C) 2017, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
- Copyright(C) 2016, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "Microsin_USB_IO.h"
#include "CommandParser.h"
#include <avr/wdt.h>
#include <avr/power.h>
#include <util/delay.h>
#include <avr/iousb162.h>
#include <avr/interrupt.h>
//! @todo{MSDN} Неясная роль inttypes здесь
#include <inttypes.h>
//! @todo{MSDN} Неясная роль math
#include <math.h>

//=============================================================================
// Вывод данных на индикатор. Биты данных подаются на пин 0 порта D
//                            Тактовый сигнал подается на пин 1 порта D
// Ввод данных энкодера       Бит 5 порта D - данные с регистра сдвига
//                            Бит 2 порта D - сигнал записи в регистр сдвига низким уровнем.
//                            Бит 3 порта D - тактовый сигнал сдвига.
// Ввод данных энкодера       Бит 5 порта B - сигнал записи в регистр сдвига низким уровнем.
// счета оборотов             Бит 6 порта B - тактовый сигнал сдвига
//                            Бит 7 порта C - данные с регистра сдвига
//=============================================================================
/*
const uint8_t Position[] = {255, 56, 40, 55, 24, 255, 39, 52, 8, 57, 255, 255, 23, 255, 36,
                            13, 120, 255, 41, 54, 255, 255, 255, 53, 7, 255, 255, 255, 20,
                            19, 125, 18, 104, 105, 255, 255, 25, 106, 38, 255, 255, 58, 255,
                            255, 255, 255, 37, 14, 119, 118, 255, 255, 255, 107, 255, 255,
                            4, 255, 3, 255, 109, 108, 2, 1, 88, 255, 89, 255, 255, 255, 255,
                            51, 9, 10, 90, 255, 22, 11, 255, 12, 255, 255, 42, 43, 255, 255,
                            255, 255, 255, 255, 255, 255, 21, 255, 126, 127, 103, 255, 102,
                            255, 255, 255, 255, 255, 255, 255, 91, 255, 255, 255, 255, 255,
                            116, 117, 255, 255, 115, 255, 255, 255, 93, 94, 92, 255, 114, 95,
                            113, 0, 72, 71, 255, 68, 73, 255, 255, 29, 255, 70, 255, 69, 255,
                            255, 35, 34, 121, 255, 122, 255, 74, 255, 255, 30, 6, 255, 123,
                            255, 255, 255, 124, 17, 255, 255, 255, 67, 26, 255, 27, 28, 255,
                            59, 255, 255, 255, 255, 255, 15, 255, 255, 255, 255, 255, 255, 255,
                            255, 5, 255, 255, 255, 110, 255, 111, 16, 87, 84, 255, 45, 86, 85,
                            255, 50, 255, 255, 255, 46, 255, 255, 255, 33, 255, 83, 255, 44, 75,
                            255, 255, 31, 255, 255, 255, 255, 255, 255, 255, 32, 100, 61, 101,
                            66, 255, 62, 255, 49, 99, 60, 255, 47, 255, 255, 255, 48, 77, 82,
                            78, 65, 76, 63, 255, 64, 98, 81, 79, 80, 97, 96, 112};
*/
static uint32_t CurrentWLIndex = 0;		// Текущее значение кода длины волны
static uint16_t InputData1 = 0;      	// Переменная считывания кода внутри оборота
static uint8_t InputData2 = 0;       	// Переменная считывания кода числа оборотов

static uint8_t RevNoCode;
static uint8_t RevolutionNo;

static uint32_t IndicationData = 0;		// Число для вывода на индикатор

static uint8_t DelayVar = 0;       		// Переменная для задержки
static uint8_t OutCicleVar = 0;       	// Переменная цикла вывода индикации

char To7Segments (char Digit)
{
    char S = 0;
    switch (Digit){
    case 0:{
        S = 0b11111100;
        break;
    }
    case 1:{
        S = 0b01100000;
        break;
    }
    case 2:{
        S = 0b11011010;
        break;
    }
    case 3:{
        S = 0b11110010;
        break;
    }
    case 4:{
        S = 0b01100110;
        break;
    }
    case 5:{
        S = 0b10110110;
        break;
    }
    case 6:{
        S = 0b10111110;
        break;
    }
    case 7:{
        S = 0b11100000;
        break;
    }
    case 8:{
        S = 0b11111110;
        break;
    }
    case 9:{
        S = 0b11110110;
        break;
    }
    }
    return S;
}

uint8_t ACEPosition(uint8_t Code)
{
  uint8_t Pos = 255;
  switch (Code){
    case 127 : Pos = 0 ; break;
    case 63 : Pos = 1 ; break;
    case 62 : Pos = 2 ; break;
    case 58 : Pos = 3 ; break;
    case 56 : Pos = 4 ; break;
    case 184 : Pos = 5 ; break;
    case 152 : Pos = 6 ; break;
    case 24 : Pos = 7 ; break;
    case 8 : Pos = 8 ; break;
    case 72 : Pos = 9 ; break;
    case 73 : Pos = 10 ; break;
    case 77 : Pos = 11 ; break;
    case 79 : Pos = 12 ; break;
    case 15 : Pos = 13 ; break;
    case 47 : Pos = 14 ; break;
    case 175 : Pos = 15 ; break;
    case 191 : Pos = 16 ; break;
    case 159 : Pos = 17 ; break;
    case 31 : Pos = 18 ; break;
    case 29 : Pos = 19 ; break;
    case 28 : Pos = 20 ; break;
    case 92 : Pos = 21 ; break;
    case 76 : Pos = 22 ; break;
    case 12 : Pos = 23 ; break;
    case 4 : Pos = 24 ; break;
    case 36 : Pos = 25 ; break;
    case 164 : Pos = 26 ; break;
    case 166 : Pos = 27 ; break;
    case 167 : Pos = 28 ; break;
    case 135 : Pos = 29 ; break;
    case 151 : Pos = 30 ; break;
    case 215 : Pos = 31 ; break;
    case 223 : Pos = 32 ; break;
    case 207 : Pos = 33 ; break;
    case 143 : Pos = 34 ; break;
    case 142 : Pos = 35 ; break;
    case 14 : Pos = 36 ; break;
    case 46 : Pos = 37 ; break;
    case 38 : Pos = 38 ; break;
    case 6 : Pos = 39 ; break;
    case 2 : Pos = 40 ; break;
    case 18 : Pos = 41 ; break;
    case 82 : Pos = 42 ; break;
    case 83 : Pos = 43 ; break;
    case 211 : Pos = 44 ; break;
    case 195 : Pos = 45 ; break;
    case 203 : Pos = 46 ; break;
    case 235 : Pos = 47 ; break;
    case 239 : Pos = 48 ; break;
    case 231 : Pos = 49 ; break;
    case 199 : Pos = 50 ; break;
    case 71 : Pos = 51 ; break;
    case 7 : Pos = 52 ; break;
    case 23 : Pos = 53 ; break;
    case 19 : Pos = 54 ; break;
    case 3 : Pos = 55 ; break;
    case 1 : Pos = 56 ; break;
    case 9 : Pos = 57 ; break;
    case 41 : Pos = 58 ; break;
    case 169 : Pos = 59 ; break;
    case 233 : Pos = 60 ; break;
    case 225 : Pos = 61 ; break;
    case 229 : Pos = 62 ; break;
    case 245 : Pos = 63 ; break;
    case 247 : Pos = 64 ; break;
    case 243 : Pos = 65 ; break;
    case 227 : Pos = 66 ; break;
    case 163 : Pos = 67 ; break;
    case 131 : Pos = 68 ; break;
    case 139 : Pos = 69 ; break;
    case 137 : Pos = 70 ; break;
    case 129 : Pos = 71 ; break;
    case 128 : Pos = 72 ; break;
    case 132 : Pos = 73 ; break;
    case 148 : Pos = 74 ; break;
    case 212 : Pos = 75 ; break;
    case 244 : Pos = 76 ; break;
    case 240 : Pos = 77 ; break;
    case 242 : Pos = 78 ; break;
    case 250 : Pos = 79 ; break;
    case 251 : Pos = 80 ; break;
    case 249 : Pos = 81 ; break;
    case 241 : Pos = 82 ; break;
    case 209 : Pos = 83 ; break;
    case 193 : Pos = 84 ; break;
    case 197 : Pos = 85 ; break;
    case 196 : Pos = 86 ; break;
    case 192 : Pos = 87 ; break;
    case 64 : Pos = 88 ; break;
    case 66 : Pos = 89 ; break;
    case 74 : Pos = 90 ; break;
    case 106 : Pos = 91 ; break;
    case 122 : Pos = 92 ; break;
    case 120 : Pos = 93 ; break;
    case 121 : Pos = 94 ; break;
    case 125 : Pos = 95 ; break;
    case 253 : Pos = 96 ; break;
    case 252 : Pos = 97 ; break;
    case 248 : Pos = 98 ; break;
    case 232 : Pos = 99 ; break;
    case 224 : Pos = 100 ; break;
    case 226 : Pos = 101 ; break;
    case 98 : Pos = 102 ; break;
    case 96 : Pos = 103 ; break;
    case 32 : Pos = 104 ; break;
    case 33 : Pos = 105 ; break;
    case 37 : Pos = 106 ; break;
    case 53 : Pos = 107 ; break;
    case 61 : Pos = 108 ; break;
    case 60 : Pos = 109 ; break;
    case 188 : Pos = 110 ; break;
    case 190 : Pos = 111 ; break;
    case 254 : Pos = 112 ; break;
    case 126 : Pos = 113 ; break;
    case 124 : Pos = 114 ; break;
    case 116 : Pos = 115 ; break;
    case 112 : Pos = 116 ; break;
    case 113 : Pos = 117 ; break;
    case 49 : Pos = 118 ; break;
    case 48 : Pos = 119 ; break;
    case 16 : Pos = 120 ; break;
    case 144 : Pos = 121 ; break;
    case 146 : Pos = 122 ; break;
    case 154 : Pos = 123 ; break;
    case 158 : Pos = 124 ; break;
    case 30 : Pos = 125 ; break;
    case 94 : Pos = 126 ; break;
    case 95 : Pos = 127 ; break;
  }
  return Pos;
}

// Коэффициенты для расчета длины волны
 static double  A = 1.26359E-5;
 static double  B = -0.02761;
/*
static double  A = -95.833333;
static double  B = 0.025903;
static double  C = - 1.5394E-7;
static double  D = 1.1502E-12;
*/
static double  _X = 20000.0;

void Index2WL (void)
{
    _X = CurrentWLIndex;
//    IndicationData = lround((A + B*_X + C*_X*_X + D*_X*_X*_X)*100);
    IndicationData = lround((1.0/6.0)*1.0E4*sin(A*_X + B)*100);
//    IndicationData = _X;  // для настройки !!!!
}

void IndicationOut (uint32_t OutData)
{
    DelayVar = 0;
    for (char i = 0; i < 5; i++)
    {
        char dig = OutData % 10;
        dig = To7Segments (dig);
        for (char k = 0; k < 8; k++)
        {
            PORTD &= ~(1);				// Обнуляем бит для вывода
            if (dig & 1) PORTD |= 1;	// Если надо, устанавливаем его в 1
            PORTD |= (1 << 1);  		// Выводим 1 на пин PORTD1 - сдвиг регистра
            DelayVar++;	DelayVar++;		// Для задержки
            PORTD &= ~(1 << 1);  		// Выводим 0 на пин PORTD1 - возврат в исх. сост.
            DelayVar++;	DelayVar++;		// Для задержки
            dig = dig >> 1;
        };
        OutData = OutData/10;
    };
}

void GetIndex(void)
{
    unsigned char i;
    InputData1 = 0;
// ---------------Ввод данных из энкодера в переменную InputData1 ---------------------
    PORTD &= ~(1 << 2);  			// Выводим 0 на пин PORTD2 - загрузка регистра
    DelayVar = 0;DelayVar++;DelayVar++;	// Для задержки
    PORTD |= (1 << 2);  			// Выводим 1 на пин PORTD2 - возврат в исх. сост.
    for (i = 0; i < 16; i++) {
        InputData1 <<= 1;
        if (PIND & (1<<PIND5))
            InputData1 |= 1;		// Считываем бит 5 и добавляем его в накопитель
        PORTD |= (1 << 3);  		// Выводим 1 на пин PORTD3 - сдвиг регистра
        DelayVar++;	DelayVar++;		// Для задержки
        PORTD &= ~(1 << 3);  		// Выводим 0 на пин PORTD3 - возврат в исх. сост.
    }
    InputData1 = 1023 - InputData1; // Энкодер крутится не в ту сторону
// --------------------------------------------------------------------------------------
// ---------------Ввод данных из энкодера числа оборотов в переменную InputData2 ----------------------------
    PORTB &= ~(1 << 5);  			// Выводим 0 на пин PORTB5 - загрузка регистра
    DelayVar = 0;DelayVar++;DelayVar++;	// Для задержки
    PORTB |= (1 << 5);  			// Выводим 1 на пин PORTB5 - возврат в исх. сост.
    for (i = 0; i < 8; i++) {
        InputData2 <<= 1;
        if (PINC & (1<<PINC7))
            InputData2 |= 1;		// Считываем бит 5 и добавляем его в накопитель
        PORTB |= (1 << 6);  		// Выводим 1 на пин PORTB6 - сдвиг регистра
        DelayVar++;	DelayVar++;		// Для задержки
        PORTB &= ~(1 << 6);  		// Выводим 0 на пин PORTB6 - возврат в исх. сост.
    }
// ---------------Расчет Индекса длины волны ---------------------------------------------
//    InputData2 = Position[InputData2];
    RevNoCode = ACEPosition(InputData2);
    if (InputData1 < 160)  RevolutionNo = (RevNoCode+1)/2;
    else if (InputData1 > 650)  RevolutionNo = (RevNoCode-1)/2;
    else RevolutionNo = RevNoCode/2;
    CurrentWLIndex = RevolutionNo;
    CurrentWLIndex = (CurrentWLIndex << 10) + InputData1;
}

int8_t  ExecuteCommand0 (const char  command)
{
  switch( command ){
    case 'W':
      SendNumberToUSB(CurrentWLIndex);
      Send_EOL_ToUSB ();
      break;
    case 'R':
      SendNumberToUSB(RevolutionNo);
      Send_EOL_ToUSB ();
      break;
    case 'C':
      SendNumberToUSB(RevNoCode);
      Send_EOL_ToUSB ();
      break;
    default:
        return 1;
  }
  return 0;
}

int8_t  ExecuteCommand2 (const char     command
                        ,const uint8_t  argument)
{
  return 1;
}

int8_t  ExecuteCommand4 (const char      command
                        ,const uint16_t  argument)
{
  return 1;
}

int8_t  ExecuteCommand8 (const char      command
                        ,const uint32_t  argument)
{
  return 1;
}

void  SetupHardware (void)
{
  /* Disable watchdog if enabled by bootloader/fuses */
  MCUSR &= ~(1 << WDRF);
  wdt_disable();

  /* Disable clock division */
  clock_prescale_set(clock_div_1);

  // Установки для управления  --------------------------------
  // Чтение энкодера и вывод индикации
     DDRD |= 0b00001111; 	// Биты 0-3,6 на вывод (0 - данные на индикацию, 1 - такт сдвига вывода
                            // 2- загрузка входных данных, 3 - такты сдвига ввода)
     DDRD &= 0b11011111;	// Бит 5 на ввод (ввод данных из энкодера)
     PORTD &= 0b11110100;	// Устанавливаем в ноль биты 0,1,3
     PORTD |= 0b00100100;	// Устанавливаем в единицу бит 2, подтягиваем в 1 бит 5
  // Ввод данных из энкодера числа оборотов
  // настройка PC7 как входа с pullup - ввод данных из энкодера
     DDRC &= ~(1<<PC7);
     PORTC |= (1<<PC7);
  // настройка управляющих выходов (5 - загрузка входных данных, 6 - такты сдвига ввода)
  // Устанавливаем в единицу бит 5, в ноль бит 6
     DDRB |= ((1<<PB5)|(1<<PB6));
     PORTB |= (1<<PB5);
     PORTB &= ~(1<<PB6);
  //------------------------------------------------------------------------------
}


void  BeforeLoop (void)
{
  OutCicleVar = 0;
}


void  InLoop (void)
{
  GetIndex();
  OutCicleVar++;
  if (OutCicleVar > 250) {
    OutCicleVar = 0;
    Index2WL();
    IndicationOut(IndicationData);			// Вывод на индикатор !!!!!!!!!!
  }
}

