////////////////////////////////////////////////////////////////////////////////
/*! @file pid.h  Firmware for PID regulator.
* \brief Header file for pid.c.
- Part of the MSDN firmware.
- Part of the Research Assistant project.

- Copyright(C) 2017, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////

#ifndef PID_H
#define PID_H
#include <math.h>
#include "stdint.h"

// Setpoints and data used by the PID control algorithm
// Output is P = K * ( Err + (1/Ti)*Int + Td*dErr )
// Output power P is in percent of maximum power:
// Pmax = 100%
#define MAX_OUTPUT 100

typedef struct PID_DATA{
  double lastError; // Last process value
  double sumError;  // Summation of errors

  double K_Factor;   // The Proportional tuning constant

  double I_Factor;   // The Integral tuning constant,
                     // 1/Ti, where Ti is in regulating periods
  uint16_t D_Factor; // The Derivative tuning constant,
                     // time constant in regulating periods

} pidData_t;

void pid_Init(double k_factor, int16_t i_time, int16_t d_time, struct PID_DATA *pid);
double pid_Controller(double setPoint, double processValue, struct PID_DATA *pid_st);
void pid_Reset_Integrator(pidData_t *pid_st);

#endif
