////////////////////////////////////////////////////////////////////////////////
/*! @file Nelson.cpp Drivers for installations in 429 room.
- Part of Nelson - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2012-2013, Dmitry K. Nelson, St.Petersburg, Russia.
                          D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "Nelson.h"
#include "attr.h"
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sNelson)

//------------------------------------------------------------------------------

sVersion  sNelson::Version ()
{
  return sVersion(RAP_VERS_MAJOR
                 ,RAP_VERS_MINOR
                 ,ZAKAVYCH(RAP_VERS_PATCH)
                 ,__DATE__
                 );
}


sVersion  sNelson::CoreVersionUsed ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}


sVersion  sNelson::CoreVersionMin ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}


QString  sNelson::UserFriendlyName ()
{
  return trUtf8("Drivers for installations in 429 room");
}


QString  sNelson::About ()
{
  return trUtf8(
      "Provides 2 console:\n"
      "sDNSpectroScanner -- user interface (console) for the scanning technique.\n\n"
      "sMDRSpectroScanner -- user interface (console) for scanning MDR-23.\n\n"
      "Copyright (C) 2012, Dmitry K. Nelson, St.Petersburg, Russia.\n"
      "                    D.Nelson@mail.ioffe.ru\n"
      "Copyright (C) 2011, Viktor E. Bursian, St.Petersburg, Russia.\n"
      "                    Viktor.Bursian@mail.ioffe.ru"
                );
}


void  sNelson::InstallDefaults (sPtr<sFolderNode>  /*DeviceList*/)
{
  try{
/*
    sPtr<sFolderNode>              DeviceCfg;

    DeviceList |= "StepMotorCOM";
    DeviceCfg = DeviceList >> "StepMotorCOM";
    DeviceCfg << attr("Class","sTeleCOMPort");
    sTeleCOMPort::Preconfigure(DeviceCfg);
    DeviceCfg >> "Settings" << attr("PortName","COM4");
    DeviceCfg >> "Settings" << attr("BaudRate",115200);
    DeviceCfg >> "Settings" << attr("Timeout",sPhysValue(10.0,"s"));

    DeviceList |= "WaveLengthCOM";
    DeviceCfg = DeviceList >> "WaveLengthCOM";
    DeviceCfg << attr("Class","sTeleCOMPort");
    sTeleCOMPort::Preconfigure(DeviceCfg);
    DeviceCfg >> "Settings" << attr("PortName","COM5");
    DeviceCfg >> "Settings" << attr("BaudRate",115200);
    DeviceCfg >> "Settings" << attr("Timeout",sPhysValue(10.0,"s"));

    DeviceList |= "WLDriver_DFS-12";
    DeviceCfg = DeviceList >> "WLDriver_DFS-12";
    DeviceCfg << attr("Class","sStepper428");
    sStepper428::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "StepperCom"
        << attr("DeviceId","StepMotorControl1");
    DeviceCfg >> "Links" >> "WLIndexCom"
        << attr("DeviceId","WaveLengthCounter");

      DeviceList |= "DFS-12 Spectrometer";
      DeviceCfg = DeviceList >> "DFS-12 Spectrometer";
      DeviceCfg << attr("Class","sDNSpectroScanner");
      sNelson_DFSScan::Preconfigure(DeviceCfg);
      DeviceCfg >> "Links" >> "Gage" << attr("DeviceId","Counters NIDAQ");
*/
  }catch(...){
    ;
  };

    /*________________________________________template for device
    DeviceList |= "__DEVICE_ID__";
    DeviceCfg = DeviceList >> "__DEVICE_ID__";
    DeviceCfg << attr("Class","__DRIVER_CLASS_NAME__");
    __DRIVER_CLASS_NAME__::Preconfigure(DeviceCfg);
    //if there are links:
    DeviceCfg >> "Links" >> "__LINK_NAME__"
        << attr("DeviceId","__DEVICE_ID__FOR_THE_LINK__");
    //if you wanna change some of the default settings:
    DeviceCfg >> "Settings" << attr("",);
    DeviceCfg >> "Settings" << attr("",);
    */

  sUnit::Register("m"
                 ,"Y..Z..E..P..T..G..M..k--0-cm..-..n..-..f..a..z..y"
                 //If you set here the prefix allowance, it will act only once,
                 //when the user chooses "Install defaults" for the new plugin.
                 //It can be changed later by editing the unit settings in
                 //the RANet Knowledgebase.
                 );

//                 //for a nerd (with hecto and deca):
//                 ,"Y..Z..E..P..T..G..M..khd0dcm..u..n..p..f..a..z..y"
//                 ,sMetricPrefix::StandardAllowance
//                 ,"Y..Z..E..P..T..G..M..k..0..m..u..n..p..f..a..z..y"
//                 ,sMetricPrefix::NoPrefixAllowed
//                 ,"-..-..-..-..-..-..-..-..0..-..-..-..-..-..-..-..-"
}


sNelson::sNelson ()
{
  sUnit::Register("m"
//                 ,"Y..Z..E..P..T..G..M..k--0-cm..-..n..-..f..a..z..y"
                 //Attention! When you change here the prefix allowance, the
                 //program ignores (and resets!) the settings made by user in
                 //the RANet Knowledgebase.
                 );
  sUnit::Register("V");
}

//------------------------------------------------------------------------------
} //namespace RA
