////////////////////////////////////////////////////////////////////////////////
/*! @file PeriodicXYRecorder.h   Самописец.
- Регистрирует периодический сигнал Y в зависимости от поступающего значения X.
- Part of Nelson - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.5.0  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2013, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
- Copyright(C) 2016, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef PeriodicXYRecorder_H
#define PeriodicXYRecorder_H
#include "Nelson.h"
#include "PeriodicGaugeRecorder.h"
//#include "DataListRecord.h"
//#include "Gauge.h"

namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sPeriodicXYRecorder)

//------------------------------------------------------ sPeriodicXYRecorder ---
/*! Самописец, регистрирующий Y в зависимости от X.
*/
class Nelson_EXPORT  sPeriodicXYRecorder : public sPeriodicGaugeRecorder
{
  Q_OBJECT
  CONSOLE(sPeriodicXYRecorder)
  public:
    virtual                   ~sPeriodicXYRecorder ();

  protected: //evolution of the inherited state machine
    psState                   State_W_XGauge;
    psState                   State_W_XGauge_Warming;

  signals: //for internal needs
    void                      XGaugeWarmed ();
    void                      XGaugeError(QString = QString());

  protected: //reimplementing inherited

  protected slots: //reimplementing inherited
    virtual void              OnEnterToCoolingState ();
    virtual void              CreateNewDependence ();
    virtual void              AddNewData (sPhysValue  Data);

  protected slots:
    virtual void              ReportXGaugeError (QString);
    virtual void              LinkToXGauge ();
    virtual void              StoreXValue (sPhysValue _Xdata);

  protected: //fields
    sGauge::sLink             XGauge;

  protected: //fields
    sPhysValue                XValue;
    int                       NumOfPoints;
    bool                      FirstPoint;
  private:

  private: //fields

};

//------------------------------------------------------------------------------
} //namespace RA
#endif
