////////////////////////////////////////////////////////////////////////////////
/*! @file Stepper428.h   Driver for Stepping motor microcontroller board.
- Part of Nelson - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.5.0  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2013, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
- Copyright(C) 2014, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef Stepper428_H
#define Stepper428_H
#include "Nelson.h"
#include "Effectors.h"
#include "TeleCom.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sStepper428)

//--------------------------------------------------------------- Stepper428 ---
/*!

*/
class  Nelson_EXPORT  sStepper428 : public sStepper
{
  Q_OBJECT
  DRIVER(sStepper428)
  
  public:
    virtual                   ~sStepper428 ();
    virtual sPhysValue        Position (int steps) const;
    virtual int               Steps  (sPhysValue  position) const;

  protected: //evolution of the inherited state machine
    psParallelStateGroup      State_W_Parallel;
    psState                   State_W_StepperCom;
    psState                   State_W_StepperCom_Linking;
    psState                   State_W_StepperCom_Checking;
    psState                   State_W_WLIndexCom;
    psState                   State_W_WLIndexCom_Linking;
    psState                   State_W_WLIndexCom_Checking;
    psParallelStateGroup      State_O_Parallel;
    psState                   State_O_M_Pausing;
    psState                   State_O_Control;
    psState                   State_OC_Idle;
    psState                   State_OC_SettingTo;
    psState                   State_OC_SettingTo_CheckPos;
    psState                   State_OC_SettingTo_WaitingRoger;
    psState                   State_OC_SettingTo_WaitingDone;
    psState                   State_OC_SettingTo_MovingBackwardCheckPos;
    psState                   State_OC_SettingTo_MovingBackwardWaitingRoger;
    psState                   State_OC_SettingTo_MovingBackwardWaitingDone;
    psState                   State_OC_ScanningTo;
    psState                   State_OC_ScanningTo_WaitingRoger;
    psState                   State_OC_ScanningTo_WaitingDone;
    psState                   State_OC_Breaking;
    psState                   State_OC_Breaking_WaitingConfirm;

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  signals: //for internal needs
    void                      StepperComLinked ();
    void                      StepperComError(QString = QString());
    void                      GotStepperMessage(sTelex);
    void                      WLIndexComLinked ();
    void                      WLIndexComError (QString = QString());
    void                      GotWLIndexMessage(sTelex);
    void                      TheIndex(int);
    void                      PushWLMeter ();
    void                      DoneSetTo ();
    void                      MoveForward ();
    void                      MoveBackward ();
    void                      BreakConfirmed();

  protected slots: //reimplementing inherited
    virtual void              SetWantedPeriod ();

  protected slots:
    virtual void              ReportStepperComError (QString);
    virtual void              ReportWLIndexComError (QString);

  protected:
    virtual sPhysValue        Index2WL (double index)  const;
    virtual double            WL2Index  (sPhysValue  wavelength)  const;
    virtual double            Steps2Index (int steps)  const;
    virtual int               Index2Steps (int index)  const;
    virtual int               Index2Steps (double index)  const;
    virtual sPhysValue        Wavelength (int index)  const;
    virtual int               Index  (sPhysValue  wavelength)  const;

    protected: //fields
    sTeleCom::sLink           StepperCom;
    sTeleCom::sLink           WLIndexCom;

  private slots:
    void                      LinkToStepperComAndMakeItWarming ();
    void                      LinkToWLIndexComAndMakeItWarming ();
    void                      StepperControllerAskName ();
    void                      WavelengthCounterAskName ();
    void                      StepperControllerInitiation ();
    void                      AskWL ();
    void                      RecalculateWL (sTelex theIndex);
    void                      SetDestination (sPhysValue);
    void                      MakeDecision (int  current_index);
    void                      SayDone();
    void                      StartMoving ();
    void                      MakeDecisionBackward (int  current_index);
    void                      StartMovingBackward ();
    void                      SetScanTimePerStep(sPhysValue time_per_step);
    void                      StartScanning(int numSteps);
    void                      SendBreakCommand();
    void                      CheckIfBreaked(sTelex Msg);

  private:
    sString                   StepperName;
    sString                   WLIndexName;
    int                       MinimalTicksPerStep;
    int                       AccelAddition;
    int                       RampCoef;
    int                       StoppingZone;
    int                       Overstep;
    sPhysValue                CoefA;			// Коэффициенты для расчета
    sPhysValue                CoefB;			// длины волны
    sPhysValue                TimePerStepQuantum;

    int                       Destination;
    int                       CurrentIndex;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
