////////////////////////////////////////////////////////////////////////////////
/*! @file StepperMDR.h   Driver for Stepping motor microcontroller board.
- Part of Nelson - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.5.0  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2013, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
- Copyright(C) 2014, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef StepperMDR_H
#define StepperMDR_H
#include "Nelson.h"
#include "Effectors.h"
#include "TeleCom.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sStepperMDR)

//--------------------------------------------------------------- StepperMDR ---
/*!

*/
class  Nelson_EXPORT  sStepperMDR : public sStepper
{
  Q_OBJECT
  DRIVER(sStepperMDR)
  
  public:
    virtual                   ~sStepperMDR ();
    virtual sPhysValue        Position (int steps) const;
    virtual int               Steps  (sPhysValue  position) const;

  signals: //outgoing
    void                      TheGratingNo(char);

  protected: //evolution of the inherited state machine
    psState                   State_W_Linking;
    psState                   State_W_Checking;
    psState                   State_W_Ch_Waiting;
    psState                   State_O_M_Pausing;
    psState                   State_O_SettingTo;
    psState                   State_O_SettingTo_WaitingWLData;
    psState                   State_O_SettingTo_WaitingRoger;
    psState                   State_O_SettingTo_WaitingDone;
    psState                   State_O_ScanningTo;
    psState                   State_O_ScanningTo_WaitingWLData;
    psState                   State_O_ScanningTo_WaitingRoger;
    psState                   State_O_ScanningTo_WaitingDone;
    psState                   State_O_Breaking;
    psState                   State_O_Breaking_WaitingConfirm;

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  signals: //for internal needs
    void                      StepperComLinked ();
    void                      StepperComError(QString = QString());
    void                      GotStepperMessage(sTelex);

  protected slots: //reimplementing inherited
    virtual void              SetWantedPeriod ();

  protected slots:
    virtual void              ReportStepperComError (QString);
    virtual void              LinkToStepperComAndMakeItWarming ();
    virtual void              StepperControllerAskName ();
    virtual void              StepperControllerInitiation ();
    virtual void              AskWL ();
    virtual void              RecalculateWL (sTelex theIndex);
    virtual void              SetDestination (sPhysValue);
    virtual void              SayDone();
    virtual void              SetScanTimePerStep(sPhysValue time_per_step);
    virtual void              StartScanning(int numSteps);
    virtual void              SendBreakCommand();

  protected:
    virtual sPhysValue        Wavelength (int index)  const;
    virtual int               Index  (sPhysValue  wavelength)  const;

  protected: //fields
    sTeleCom::sLink           StepperCom;
    sString                   StepperName;
    int                       MinimalTicksPerStep; // минимально допустимый параметр для сканирования
    int                       MaxSpeedTicksPerStep; // параметр, передаваемый контроллеру для перемещения
    int                       AccelAddition;
    int                       RampCoef;
    int                       StoppingZone;
    sPhysValue                TimePerStepQuantum;
    sPhysValue                WLCoef[4];
    sPhysValue                WLShift[4];
    int                       GratingNo;
    sPhysRange                IndexRange;
    int                       Destination;

  private slots:

  private:
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
