////////////////////////////////////////////////////////////////////////////////
/*! @file StepperMDRNew.h   Driver for Stepping motor microcontroller board.
                            Version without DUK, but with steps counting.
- Part of Nelson - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.5.0  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2013, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
- Copyright(C) 2016, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef StepperMDRNew_H
#define StepperMDRNew_H
#include "Nelson.h"
#include "StepperMDR.h"
#include "TeleCom.h"
#include "PhysValueField.h"
#include <QDialog>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sStepperMDRNew)


//--------------------------------------------------------------- StepperMDRNew ---
/*!

*/
class  Nelson_EXPORT  sStepperMDRNew : public sStepperMDR
{
Q_OBJECT
  DRIVER(sStepperMDRNew)

  public:
    virtual                   ~sStepperMDRNew ();
    virtual sPhysValue        Position (int steps) const;
    virtual int               Steps  (sPhysValue  position) const;

  protected: //evolution of the inherited state machine
    psState                   State_W_Initiating;
    psState                   State_W_Initiating_WaitingRoger1;
    psState                   State_W_Initiating_WaitingInput;
    psState                   State_W_Initiating_WaitingRoger2;
    psState                   State_O_Initiating;
    psState                   State_O_Initiating_WaitingRoger1;
    psState                   State_O_Initiating_WaitingInput;
    psState                   State_O_Initiating_WaitingRoger2;

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  signals: //for internal needs
    void                      InitialWLSent ();
  protected slots: //reimplementing inherited

  protected slots:
    virtual void              StepperControllerInitiation ();
    virtual void              AskInitialWavelength ();
    virtual void              RecalculateWL (sTelex theIndex);

  protected:

  protected: //fields
    sPhysValue                InitWavelengthPar;
  private slots:
    void                      SendingReady();
  private:
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
