////////////////////////////////////////////////////////////////////////////////
/*! @file ZScanDriver.h   Driver for Stepping motor microcontroller board.
- Part of Nelson - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.5.0  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2013, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
- Copyright(C) 2018, Dmitry K. Nelson, St.Petersburg, Russia.
                     D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef ZScanDriver_H
#define ZScanDriver_H
#include "Nelson.h"
#include "Effectors.h"
#include "TeleCom.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sZScanDriver)

//------------------------------------------------------------- ZScanDriver ---
/*!

*/
class  Nelson_EXPORT  sZScanDriver : public sEffector
{
  Q_OBJECT
  DRIVER(sZScanDriver)
  
  public:
    virtual                   ~sZScanDriver ();

  protected: //evolution of the inherited state machine
    psState                   State_W_Linking;
    psState                   State_W_Checking;
    psState                   State_W_Ch_Waiting;
    psState                   State_W_Initiating;
    psState                   State_O_Initiating;
    psState                   State_O_M_Pausing;
    psState                   State_O_SettingTo;
    psState                   State_O_SettingTo_WaitingRoger;
    psState                   State_O_SettingTo_WaitingDone;

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  signals: //for internal needs
    void                      StepperComLinked ();
    void                      StepperComError(QString = QString());
    void                      GotStepperMessage(sTelex);

  protected slots: //reimplementing inherited
    virtual void              SetWantedPeriod ();

  protected slots:
    virtual void              ReportStepperComError (QString);
    void                      LinkToStepperComAndMakeItWarming ();
    void                      StepperControllerAskName ();
    void                      StepperControllerInitiation ();
    void                      AskInitialPosition();
    void                      SetDestination (sPhysValue);
    void                      SayPosition();
    void                      SayDone();

  protected:

  protected: //fields
    sTeleCom::sLink           StepperCom;
    sPhysValue                InitValuePar;

  private slots:
    void                      SendingReady();

  private:
    sString                   StepperName;
    int                       TicksPerStep;
    sPhysValue                LengthPerStep;			// мм на шаг
    sPhysValue                CurrentPosition;
    sPhysValue                NewPosition;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
