////////////////////////////////////////////////////////////////////////////////
/*! @file APRI_TController.h   Контроллер термостата с приборами APRI.
- Part of BursiPan - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.8  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2013, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef APRI_TController_H
#define APRI_TController_H
#include "PanBursian.h"
#include "TController.h"
#include "TeleCom.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sAPRI_TController)

//-------------------------------------------------------- sAPRI_TController ---
/*! Контроллер термостата с приборами APRI.

@par Описание прибора MI-6 от производителя (орфография сохранена :)))

общение происходить строками формат строки "xxx...xxx\n"
в дальнейшем все \n опускаються

после включения питания нужно подать "G" для запуска устройства

после этого можно управлять устройством посылаю команды формата "1xxx"
если команда опознана то устройство ответит "^1" иначе "#E" то есть формат
команды не правильный

формат команды может выглядить "0xxx" в этом случае кроме выполнения команды
и ответа "^1" вся стока передасться обратно

если команда начинается не с '1'или '0' то строка передасться обратно не
выполнившись

команды ( каждый символ x соответсвует одной шестнадцатиричной цифре в
верхнем регистре)

	включить передачу "T +"

	выключить передачу "T -"

	настроить соотношение каналов в передаче "R xx:xx"


	загрузить коэфициент "D xxx"

	настроить частоту "F xx"

	настроить полярность биполярная "P B"

	настроить полярность униполярная"P U"

	настоить усиление первого канала "G1 x"

	настоить усиление второго канала "G1 x"

	включить генератор тока "I +"

	вылючить генератор тока "I -"

	включить генератор тока Burn-Out "B +"

	вылючить генератор тока Burn-Out "B -"

	режимы

	"Mn x" где n номер канала '1' или '2' соответственно
	x в соответсвии со следующей таблицей
	'0' Normal Mode.
	'1' Activate Self-Calibration.
	'2' Activate System Calibration.
	'3' Activate System Calibration.
	'4' Activate System Offset Calibration.
	'5' Activate Background Calibration.
not support	'6' Read/Write Zero-Scale Calibration Coefficients.
not support	'7'Read/Write Full-Scale Calibration Coefficients.


ответ
rem нету в смысле	"!ADC"	сменилось контрольное слово


передача данных происходит в соответствии с заданой частотой при смене настроек
канала частота имеет право сбиться то есть на каждый отчет выдается
соответсвующее слово формата "Nxxxxxx" где N '1'или '2' в зависимости от
выбраного канала

При смене канала (без изменения настроек) пропускается два
(следует разобраться) отчета

ну и еще есть команда сменить адрес устройства "#x" в случае смены адреса в
команде посылаемой к устройству нужно вместо '1' устройства

*/
class  PanBursian_EXPORT  sAPRI_TController : public sTController
{
  Q_OBJECT
  DRIVER(sAPRI_TController)
  public:
    virtual                   ~sAPRI_TController();
    virtual sPhysValue        Period ();

  public slots:
    virtual void              SetPeriod (sPhysValue  period);

  public:

  protected:
    virtual void              ChangeHeaterPower (real  power);
    virtual void              ChangeSecondHeaterPower (real  power);

  protected: //fields

  private slots:
//    void on_MessageReceived(QByteArray Msg);
//    void SendErrorMessage();

  private: //fields
    sTeleCom::sLink           Communicator;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
