////////////////////////////////////////////////////////////////////////////////
/*! @file BursiPan.cpp BursiPan EPR spectrometer abstract interface.
- Part of PanBursian - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2014, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "BursiPan.h"
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sBursiPan)

const sDirective<sPhysValue>  sBursiPan::cSetEPRMeasurePeriod
                                           ("sBursiPan::cSetEPRMeasurePeriod");
const sDirective<sPhysValue>  sBursiPan::cSetMagneticField
                                           ("sBursiPan::cSetMagneticField");
const sDirective<sPhysValue>  sBursiPan::cSetScanStep
                                           ("sBursiPan::cSetScanStep");
const sDirective<sPhysValue>  sBursiPan::cSetTimePerStep
                                           ("sBursiPan::cSetTimePerStep");
const sDirective<sPhysValue>  sBursiPan::cSetHo
                                           ("sBursiPan::cSetHo");
const sDirective<sPhysValue>  sBursiPan::cSetDeltaH
                                           ("sBursiPan::cSetDeltaH");
const sDirective<sPhysValue>  sBursiPan::cSetFrequency
                                           ("sBursiPan::cSetFrequency");
const sDirective<sPhysValue>  sBursiPan::cSetAttenuation
                                           ("sBursiPan::cSetAttenuation");
const sDirective<sPhysValue>  sBursiPan::cSetGain
                                           ("sBursiPan::cSetGain");
const sDirective<sPhysValue>  sBursiPan::cSetModulation
                                           ("sBursiPan::cSetModulation");
const sDirective<sBursiPan::eModulationMode>
                              sBursiPan::cSetModulationMode
                                           ("sBursiPan::cSetModulationMode");

//const sDirective<sBursiPan::sScanJobData>
//                              sBursiPan::cScan
//                                           ("sBursiPan::cScan");
csCommand                     sBursiPan::cScanStart
                                           ("sBursiPan::cScanStart");
csCommand                     sBursiPan::cScanStop
                                           ("sBursiPan::cScanStop");
csCommand                     sBursiPan::cScanBreak
                                           ("sBursiPan::cScanBreak");

csString                      sBursiPan::cModeText [14] =
                                           {"12kHz"
                                           ,"25kHz"
                                           ,"50kHz"
                                           ,"100kHz"
                                           ,"80Hz"
                                           ,"80Hz+12kHz"
                                           ,"80Hz+25kHz"
                                           ,"80Hz+50kHz"
                                           ,"80Hz+100kHz"
                                           ,"35Hz+12kHz"
                                           ,"35Hz+25kHz"
                                           ,"35Hz+50kHz"
                                           ,"35Hz+100kHz"
                                           ,"mode?"
                                           };

//---------------------------------------------------------------- sBursiPan ---

void  sBursiPan::Preconfigure (sNodePtr  DeviceCfg)
{
  sDriver::Preconfigure(DeviceCfg);
}


sBursiPan::sBursiPan (sString   DevicetId
                     ,sNodePtr  DeviceCfg)
    :sDriver(DevicetId,DeviceCfg)
{
}


sPhysRange sBursiPan::FieldRange()
{
  return sPhysRange(Ho()-DeltaH()/2 ,Ho()+DeltaH()/2);
}

//------------------------------------------------------------------------------
} //namespace RA
