////////////////////////////////////////////////////////////////////////////////
/*! @file BursiPan.h   BursiPan EPR spectrometer abstract interface.
- Part of PanBursian - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2014, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef BursiPan_H
#define BursiPan_H
#include "PanBursian.h"
#include "Driver.h"
#include "PhysValue.h"
#include "Ranges.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sBursiPan)

//---------------------------------------------------------------- sBursiPan ---
/*! BursiPan EPR spectrometer abstract interface.
*/
class PanBursian_EXPORT  sBursiPan : public sDriver
{
  Q_OBJECT
  ABSTRACT_DRIVER(sBursiPan)

  public: //types

    enum                         eModulationMode
                                     {cMod12kHz
                                     ,cMod25kHz
                                     ,cMod50kHz
                                     ,cMod100kHz
                                     ,cMod80Hz
                                     ,cMod80Hz12kHz
                                     ,cMod80Hz25kHz
                                     ,cMod80Hz50kHz
                                     ,cMod80Hz100kHz
                                     ,cMod35Hz12kHz
                                     ,cMod35Hz25kHz
                                     ,cMod35Hz50kHz
                                     ,cMod35Hz100kHz
                                     ,cModUnknown
                                     };

//    struct  sScanJobData
//    {
//      int                         NumOfPasses;
//      sPhysRange                  Range;
//      sPhysValue                  Step;
//      sPhysValue                  TimePerStep;
//    };

//    struct  sScanStartData
//    {
//      sPhysValue                  From;
//      sPhysValue                  Step;
//      int                         NumOfPoints;
//      int                         NumOfCountsPerPoint;
//    };

    struct  sScanPointData
    {
      int                         PassNo;
      int                         PointNo;
      sPhysValue                  Signal;
    };

  public: //commands & directives
    static const
      sDirective<sPhysValue>  cSetEPRMeasurePeriod;
    static const
      sDirective<sPhysValue>  cSetMagneticField;
    static const
      sDirective<sPhysValue>  cSetScanStep;
    static const
      sDirective<sPhysValue>  cSetTimePerStep;
    static const
      sDirective<sPhysValue>  cSetHo;
    static const
      sDirective<sPhysValue>  cSetDeltaH;
    static const
      sDirective<sPhysValue>  cSetFrequency;
    static const
      sDirective<sPhysValue>  cSetAttenuation;
    static const
      sDirective<sPhysValue>  cSetGain;
    static const
      sDirective<sPhysValue>  cSetModulation;
    static const
      sDirective
          <eModulationMode>   cSetModulationMode;
//    static const
//        sDirective
//            <sScanJobData>    cScan;
    static csCommand          cScanStart;
    static csCommand          cScanStop;
    static csCommand          cScanBreak;

  public:
    static csString           cModeText [];

  public:
    virtual sPhysValue        Ho () =0;
    virtual sPhysValue        DeltaH () =0;
    virtual sPhysValue        Frequency () =0;
    virtual sPhysValue        Attenuation () =0;
    virtual sPhysValue        Gain () =0;
    virtual sPhysValue        Modulation () =0;
    virtual eModulationMode   ModulationMode () =0;

    virtual sPhysRange        FieldRange ();

  public:
    virtual void              SetScanRange (sPhysRange) =0;
    virtual bool              ScanSettingsOK () =0;
    virtual sPhysRange        ScanRange () =0;
    virtual sPhysValue        ScanStep () =0;
    virtual int               NumOfPoints () =0;
    virtual sPhysValue        TimePerStep () =0;

  signals: //outgoing
    void                      EPR_Signal (sPhysValue);
    void                      EPRMeasurePeriodChanged (sPhysValue);
    void                      MagneticField (sPhysValue);
    void                      StepChanged (sPhysValue);
    void                      TimePerStepChanged (sPhysValue);
    void                      NewHo (sPhysValue);
    void                      NewDeltaH (sPhysValue);
    void                      NewFrequency (sPhysValue);
    void                      NewAttenuation (sPhysValue);
    void                      NewGain (sPhysValue);
    void                      NewModulation (sPhysValue);
    void                      NewModulationMode (sBursiPan::eModulationMode);
//    void                      ScanStarts (sBursiPan::sScanStartData);
    void                      ScanPoint (sBursiPan::sScanPointData);
    void                      ScanDone ();
    void                      ScanBroken ();
//    void                      Done ();
    void                      OutOfRange ();
    void                      RangeChanged (sPhysRange);
};

//------------------------------------------------------------------------------
} //namespace RA

#include <QMetaType>
Q_DECLARE_METATYPE(RA::sBursiPan::eModulationMode)
//Q_DECLARE_METATYPE(RA::sBursiPan::sScanJobData)
//Q_DECLARE_METATYPE(RA::sBursiPan::sScanStartData)
Q_DECLARE_METATYPE(RA::sBursiPan::sScanPointData)

#endif
