////////////////////////////////////////////////////////////////////////////////
/*! @file BursiPanHandles.h   Joint console for RadioPAN handles.
- Part of PanBursian - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2013-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef BursiPanHandles_H
#define BursiPanHandles_H
#include "PanBursian.h"
#include "Console.h"
#include "BursiPan.h"
#include "PhysValueField.h"
#include "NetWidgets.h"
#include <QLabel>
#include <QSplashScreen>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sBursiPanHandleSplash)
ANNOUNCE_CLASS(sBursiPanHandles)

//---------------------------------------------------- sBursiPanHandleSplash ---
/*! A splash screen showing the latest change of RadioPAN handles.
*/
class  PanBursian_EXPORT  sBursiPanHandleSplash : public QSplashScreen
{
  Q_OBJECT
  public:
                              sBursiPanHandleSplash ();
  public slots:
    void                      Display (QString);

  public: //field
    int                       Exposition;

  private slots:
    void                      OnClock ();

  private: //fields
    int                       ExpositionCounter;
    QTimer *                  Clock;
    QLabel *                  Text;
};

//--------------------------------------------------------- sBursiPanHandles ---
/*! Joint console for RadioPAN handles.
*/
class  PanBursian_EXPORT  sBursiPanHandles : public sConsole
{
  Q_OBJECT
  CONSOLE(sBursiPanHandles)
  public:
    virtual                   ~sBursiPanHandles ();

  protected:
    virtual void              AppearanceMakeUp ();
    virtual void              Retranslate ()
                                {
                                  sConsole::Retranslate();
                                  Translate();
                                }

  protected: //evolution of the inherited state machine
    psState                   State_W_BursiPan;
    psState                   State_W_BursiPan_Linking;
    psState                   State_O_Watching;
    psState                   State_O_Controling;

  signals: //for internal needs
    void                      BursiPanLinked ();
    void                      BursiPanError(QString = QString());
    void                      BursiPanControlTaken ();
    void                      RodgerFromBursiPan ();
    void                      NewHo (sPhysValue);
    void                      NewDeltaH (sPhysValue);
    void                      NewFrequency (sPhysValue);
    void                      NewAttenuation (sPhysValue);
//    void                      NewGain (sPhysValue);
//    void                      NewModulation (sPhysValue);
//    void                      NewModulationMode (sBursiPan::eModulationMode);
    void                      SetHo ();
    void                      SetDeltaH ();
    void                      SetFrequency ();
    void                      SetAttenuation ();
//    void                      SetGain ();
//    void                      SetModulation ();
//    void                      SetModulationMode ();

  protected slots: //reimplementing inherited
    virtual void              OnEnterToOperatingState ();
    virtual void              OnEnterToCoolingState ();

  protected slots:
    virtual void              ReportBursiPanError (QString);
    virtual void              ReleaseBursiPanControl ();

  protected: //fields
    sBursiPan::sLink          BursiPan;
    psBursiPanHandleSplash    SplashScreen;
    psPhysValueField          HoField;
    psPhysValueField          DeltaHField;
    psPhysValueField          FrequencyField;
    psPhysValueField          AttenuationField;
    psPhysValueField          GainField;
    psPhysValueField          ModulationField;
    psFlexibleField           ModulationModeField;
//    sSelectNodeWidget *       CavityField;

  private slots:
    void                      LinkToBursiPan ();
    void                      DisconnectRodger ();
    void                      OnSetHo ();
    void                      OnSetDeltaH ();
    void                      OnSetFrequency ();
    void                      OnSetAttenuation ();

//    void                      OnSetGain ();
//    void                      OnSetModulation ();
//    void                      OnSetModulationMode ();

    void                      OnNewGain (sPhysValue);
    void                      OnNewModulation (sPhysValue);
    void                      OnNewModulationMode
                                        (sBursiPan::eModulationMode);

  private:
    void                      Translate ();
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
