////////////////////////////////////////////////////////////////////////////////
/*! @file BursiPanMagnet.h   BursiPan magnetic field effector.
- Part of PanBursian - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2014, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef BursiPanMagnet_H
#define BursiPanMagnet_H
#include "PanBursian.h"
#include "Effectors.h"
#include "BursiPan.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sBursiPanMagnet)

//---------------------------------------------------------- sBursiPanMagnet ---
/*! BursiPan magnetic field effector.
*/
class PanBursian_EXPORT  sBursiPanMagnet : public sEffector
{
  Q_OBJECT
  DRIVER(sBursiPanMagnet)

  public:
    virtual                   ~sBursiPanMagnet ();

  protected: //evolution of the inherited state machine
    psState                   State_W_BursiPan;
    psState                   State_W_BursiPan_Linking;
    psParallelStateGroup      State_O_ParallelMeasuringAndControlling;
    psState                   State_O_Controlling;
    psState                   State_O_C_Idle;
    psState                   State_O_C_SettingTo;
    psState                   State_O_C_SettingTo_Check;

  signals: //outgoing
    void                      BursiPanLinked ();
    void                      BursiPanError (QString = QString());

  signals: //for internal needs
    void                      GotMagneticField (sPhysValue);

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  protected slots: //reimplementing inherited
    virtual void              OnEnterToCoolingState ();

  protected slots:
    virtual void              SetH (sPhysValue);
    virtual void              ReportBursiPanError (QString);
    void                      ReleaseBursiPanControl ();

  protected:

  protected: //fields
    sBursiPan::sLink          BursiPan;

  private slots:
    void                      LinkToBursiPan ();

  private:

  private: //fields
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
