////////////////////////////////////////////////////////////////////////////////
/*! @file BursiPanScanner.h   EPR spectrometer BursiPan Scanner.
- Part of PanBursian - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2014, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef BursiPanScanner_H
#define BursiPanScanner_H
#include "PanBursian.h"
#include "Scanner.h"
#include "DataArrayRecord.h"
#include "DataTableRecord.h"
#include "BursiPan.h"
#include <QSpinBox>
#include <QComboBox>
#include <QSoundEffect>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sBursiPanScanner)

//--------------------------------------------------------- sBursiPanScanner ---
/*! EPR spectrometer BursiPan (modified RadioPAN) Scanner.
*/
class  PanBursian_EXPORT  sBursiPanScanner : public sScanner
{
  Q_OBJECT
  CONSOLE(sBursiPanScanner)
  public:
    enum                      eScanDirection {cForwards
                                             ,cBackwards
                                             ,cForthAndBack
                                             ,cBackAndForth
                                             };

  public:
    virtual                   ~sBursiPanScanner();
    virtual void              RestoreState ();
    virtual void              StoreState ();

  protected: //evolution of the inherited state machine
    psState                   State_W_Parallel;
    psState                   State_W_BursiPan;
    psState                   State_W_BursiPan_Linking;
    psState                   State_O_StepSetting;
    psState                   State_O_TimePerStepSetting;
    psState                   State_O_AllScanParamResetting;
    psState                   State_O_ASPR_Step;
    psState                   State_O_ASPR_TimePerStep;
    psState                   State_O_Sc_Starting;
    psState                   State_O_Sc_S_Waiting;
    psState                   State_O_Sc_A_TakingPoints;
    psState                   State_O_Sc_Breaking;
    psState                   State_O_Sc_B_Waiting;

  signals: //for internal needs
    void                      BursiPanLinked ();
    void                      BursiPanError(QString = QString());
    void                      BursiPanBusy ();
    void                      BursiPanRangeChanged (sPhysRange);
    void                      StepEdited ();
    void                      StepSetting ();
    void                      StepChanged (sPhysValue);
    void                      TimePerStepEdited ();
    void                      TimePerStepSetting ();
    void                      TimePerStepChanged (sPhysValue);
//    void                      ScanStarted (sBursiPan::sScanStartData);
//    void                      GotScanPoint (sBursiPan::sScanPointData);
    void                      EPR_Signal (sPhysValue);
    void                      ScanningInterrupted ();

  protected: //reimplementing inherited
    virtual void              AppearanceMakeUp ();
    virtual void              Retranslate ()
                                {
                                  sConsole::Retranslate();
                                  Translate();
                                }

  protected slots: //reimplementing inherited
    virtual void              OnEnterToOperatingState ();
    virtual void              OnExitFromOperatingState ();
    virtual void              OnEnterToCoolingState ();

  protected slots: //implementing pure virtual
    virtual void              CreateNewDependence ();
    virtual void              EarlySetAttributes ();
    virtual void              LateSetAttributes ();
    virtual void              StartScanning ();
    virtual void              StopScanning ();

  protected slots:
    virtual void              ReportBursiPanError (QString);
    virtual void              ReleaseBursiPanControl ();
//    virtual void              DisconnectParamSettingFinished ();
//    virtual void              OnFullRangeChange (sPhysRange  full_range);
    virtual bool              ValidateRangeAndStep ();
    virtual void              ShowParamComments ();
    virtual void              EnableParamSetting ();
    virtual void              DisableParamSetting ();
    virtual void              OnNumOfPassesEdited ();
    virtual void              OnScanRangeEdited ();
    virtual void              OnStepEdited ();
    virtual void              OnTimePerStepEdited ();
//    virtual void              StartScanJob ();
    virtual void              AddNewData (sPhysValue);
    virtual void              OnBreak ();

  protected:

  protected: //fields
    sBursiPan::sLink          BursiPan;
    integer                   NumOfPasses;
//    eScanDirection            ScanDirection;
    sPtr
      <sDataArrayRecord>  DataArrayRecord;
    int                       PassNo;
    int                       PointNo;
    bool                      StopRequested;

  protected: //fields
    QSpinBox *                FldNumOfPasses;
    QComboBox *               DirectionField;
    psPhysRangeField          ScanRangeField;
    psPhysValueField          ScanStepField;
    psPhysValueField          TimePerStepField;
    QLabel *                  NumOfPassesSuffix;
    QLabel *                  ScanStepSuffix;
    QLabel *                  TimePerStepSuffix;
    psPhysValueField          FldTheta;
    psPhysValueField          FldPhi;
    QLabel *                  LblCavity;
    psDropNodeWidget          FldCavity;
    QSoundEffect *            ScanningFinished;

  private slots:
    void                      LinkToBursiPan ();

  private:
    void                      Translate ();

  private: //fields
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
