////////////////////////////////////////////////////////////////////////////////
/*! @file BursiPanSignal.cpp BursiPan signal gauge.
- Part of PanBursian - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2014, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "BursiPanSignal.h"
#include "attr.h"
#include "StringsAndQStrings.h"
#include "Log.h"
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sBursiPanSignal)

//---------------------------------------------------------- sBursiPanSignal ---

void  sBursiPanSignal::Preconfigure (sNodePtr  DeviceCfg)
{
  sPtr<sFolderNode>              DeviceLinks;
  DeviceCfg |= "Links";
  DeviceLinks = DeviceCfg >> "Links";
  if( DeviceLinks.IsCorrect() ){
    DeviceLinks |= "BursiPan";
    DeviceLinks >> "BursiPan" |= attr("Class","sBursiPan");
  }
  sGauge::Preconfigure(DeviceCfg);
}


sBursiPanSignal::~sBursiPanSignal ()
{
  BursiPan.SetToNULL();
}


sBursiPanSignal::sBursiPanSignal (sString   DevicetId
                                 ,sNodePtr  DeviceCfg)
    :sGauge(DevicetId,DeviceCfg)
    ,BursiPan(this)
{
}


void  sBursiPanSignal::PostConstructor ()
{
  sGauge::PostConstructor();
  //evolution of the inherited state machine (behaviour)
  {
    //evolution of the State_Warming
    {
      State_W_BursiPan = new sState(State_Warming());
      State_W_BursiPan
          ->SetAsInitial()
          ->AddTransition(State_Warming()->Fin())
          ;
      //State_W_BursiPan structure and behaviour
      {
        State_W_BursiPan_Linking = new sState(State_W_BursiPan);
        State_W_BursiPan_Linking
            ->SetAsInitial()
            ->OnEnterCall( SLOT(LinkToBursiPan()) )
            ->AddTransition( SIGNAL(BursiPanLinked()) ,State_W_BursiPan->Fin() )
            ->AddTransition( SIGNAL(ErrorMessage()) ,State_Cooling() )
            ;
      }
    }
    //evolution of the State_Operating
    {
      //evolution of the State_O_Measuring
      {
        State_O_M_Acquiring
            ->AddTransition(SIGNAL(NewData(sPhysValue)) ,State_O_M_Cycling);
      }
    }
    connect(this ,SIGNAL(BursiPanError(QString))
           ,this ,SLOT(ReportBursiPanError(QString)) );
  }
}


void  sBursiPanSignal::OnEnterToCoolingState ()
{
  Disconnect(BursiPan ,0 ,0);
  BursiPan.ReleaseExclusiveControl();
  BursiPan.SetToNULL();
  sGauge::OnEnterToCoolingState();
}


void sBursiPanSignal::SetWantedPeriod()
{
  SendDirective<sPhysValue>(BursiPan,sBursiPan::cSetEPRMeasurePeriod
                           ,WantedPeriod ,cTakeControl);
}


void  sBursiPanSignal::ReportBursiPanError (QString  error_text)
{
  emit ErrorMessage( tr("Got error from '%1':\n\n%2")
                       .arg(ToQString(BursiPan->DeviceIdentifier()))
                       .arg(error_text)     );
}


void  sBursiPanSignal::LinkToBursiPan ()
{
  try{
    BursiPan.SetTo(DeviceLinks,"BursiPan");
    Connect(BursiPan ,SIGNAL(Warmed()) ,SIGNAL(BursiPanLinked()) );
    Connect(BursiPan ,SIGNAL(ErrorMessage(QString))
                     ,SIGNAL(BursiPanError(QString)) );
    Connect(BursiPan ,SIGNAL(EPR_Signal(sPhysValue))
                     ,SIGNAL(NewData(sPhysValue)));
    Connect(BursiPan ,SIGNAL(EPRMeasurePeriodChanged(sPhysValue))
                     ,SIGNAL(PeriodChanged(sPhysValue)));
  }catch( rxDeviceLaunch E ){
    emit ErrorMessage( tr("Error while linking to a submissive device.\n"
                          "%1\nCheck configuration.")
                         .arg(ToQString(E.MsgText()))   );
  }
}

//------------------------------------------------------------------------------
} //namespace RA
