////////////////////////////////////////////////////////////////////////////////
/*! @file BursiPan_RASOS.h   BursiPan implementation with RASOS on Genie.
- Part of PanBursian - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2014, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef BursiPan_RASOS_H
#define BursiPan_RASOS_H
#include "PanBursian.h"
#include "BursiPan.h"
#include "TeleCom.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sBursiPan_RASOS)

//---------------------------------------------------------- sBursiPan_RASOS ---
/*! BursiPan implementation with RASOS on Genie.
*/
class PanBursian_EXPORT  sBursiPan_RASOS : public sBursiPan
{
  Q_OBJECT
  DRIVER(sBursiPan_RASOS)

  public:
    virtual                   ~sBursiPan_RASOS ();
    virtual sPhysValue        Ho ()
                                { return LatestHo; }
    virtual sPhysValue        DeltaH ()
                                { return LatestDeltaH; }
    virtual sPhysValue        Frequency ()
                                { return LatestFrequency; }
    virtual sPhysValue        Attenuation ()
                                { return LatestAttenuation; }
    virtual sPhysValue        Gain ()
                                { return LatestGain; }
    virtual sPhysValue        Modulation ()
                                { return LatestModulation; }
    virtual eModulationMode   ModulationMode ()
                                { return LatestModulationMode; }

  protected: //evolution of the inherited state machine
    psParallelStateGroup      State_W_Parallel;
    psState                   State_W_Genie;
    psState                   State_W_Genie_Linking;
    psState                   State_W_Genie_Checking;
    psState                   State_O_Idle;
    psState                   State_O_WaitingRodger;
    psState                   State_O_Scanning;
//    psState                   State_O_Sc_Starting;
//    psState                   State_O_Sc_Rest;

  signals: //for internal needs
    void                      GenieLinked ();
    void                      GenieError (QString = QString());
    void                      GotGenieMessage (sTelex);
    void                      NoAnswerFromSOS ();
    void                      GotSOSVersion ();
    void                      WaitRodgerFromSOS ();
    void                      RodgerFromSOS ();
//    void                      GenieScanDone ();

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  protected slots: //reimplementing inherited
//    virtual void              AfterStartInitiation ();
    virtual void              OnEnterToOperatingState ();
    virtual void              OnExitFromOperatingState ();
    virtual void              OnEnterToCoolingState ();

  protected slots:
    virtual void              ReportGenieError (QString);
    virtual void              SendMagneticField (sPhysValue);
    virtual void              StartScanJob (sScanJobData);
//    virtual void              JustifyScanParams (sScanStartData);

  protected: //fields
    sPhysValue                Calibration;
    sTeleCom::sLink           Genie;
    QByteArray                RASOS_Version;
    sPhysValue                LatestHo;
    sPhysValue                LatestDeltaH;
    sPhysValue                LatestFrequency;
    sPhysValue                LatestAttenuation;
    sPhysValue                LatestGain;
    sPhysValue                LatestModulation;
    eModulationMode           LatestModulationMode;
    int                       NumOfCountsPerPoint;

  private slots:
    void                      LinkToGenie ();
    void                      GetSOSVersion ();
    void                      OnNoAnswerFromSOS ();
    void                      DisconnectRodger ();

    void                      SendHo (sPhysValue);
    void                      SendDeltaH (sPhysValue);
    void                      SendFrequency (sPhysValue);
    void                      SendAttenuation (sPhysValue);
    void                      SendGain (sPhysValue);
    void                      SendModulation (sPhysValue);
    void                      SendModulationMode (eModulationMode);

    void                      ProcessGenieMessage (sTelex);
    void                      SignalRangeChanging (sPhysValue);

  private:
    void                      Put (sPhysValue  value
                                  ,sPhysValue  unit
                                  ,short       digits_after_dot
                                  ,literal     SOS_field);
    void                      Get (literal  prefix
                                  ,literal  SOS_field);

  private: //fields
    bool                      Emulate;
    sString                   Magic;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
