////////////////////////////////////////////////////////////////////////////////
/*! @file BursiPan_RASOS_MSDNx47.h   BursiPan implementation
                                     with RASOS on Genie and
                                     with handles from MSDN2047.
- Part of PanBursian - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2014, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef BursiPan_RASOS_MSDNx47_H
#define BursiPan_RASOS_MSDNx47_H
#include "PanBursian.h"
#include "BursiPan.h"
#include "TeleCom.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sBursiPan_RASOS_MSDNx47)

//-------------------------------------------------- sBursiPan_RASOS_MSDNx47 ---
/*! BursiPan implementation with RASOS on Genie
    and with handles from MSDN2047.
*/
class PanBursian_EXPORT  sBursiPan_RASOS_MSDNx47 : public sBursiPan
{
  Q_OBJECT
  DRIVER(sBursiPan_RASOS_MSDNx47)

  public: //commands & directives

  public:
    virtual                   ~sBursiPan_RASOS_MSDNx47 ();
    virtual sPhysValue        Ho ()
                                { return LatestHo; }
    virtual sPhysValue        DeltaH ()
                                { return LatestDeltaH; }
    virtual sPhysValue        Frequency ()
                                { return LatestFrequency; }
    virtual sPhysValue        Attenuation ()
                                { return LatestAttenuation; }
    virtual sPhysValue        Gain ();
    virtual sPhysValue        Modulation ();
    virtual eModulationMode   ModulationMode ()
                                { return TheModulationMode; }

  protected: //evolution of the inherited state machine
    psParallelStateGroup      State_W_Parallel;
    psState                   State_W_Genie;
    psState                   State_W_Genie_Linking;
    psState                   State_W_Genie_Checking;
    psState                   State_W_MSDNx47;
    psState                   State_W_MSDNx47_Linking;
    psState                   State_W_MSDNx47_Checking;
    psState                   State_O_Idle;
    psState                   State_O_WaitingRodger;
    psState                   State_O_Scanning;
//    psState                   State_O_Sc_Starting;
//    psState                   State_O_Sc_Rest;

  signals: //for internal needs
    void                      GenieLinked ();
    void                      GenieError (QString = QString());
    void                      GotGenieMessage (sTelex);
    void                      NoAnswerFromSOS ();
    void                      GotSOSVersion ();
    void                      WaitRodgerFromSOS ();
    void                      RodgerFromSOS ();
//    void                      GenieScanDone ();
    void                      MSDNx47Linked ();
    void                      MSDNx47Error (QString = QString());
    void                      GotMSDNx47Message (sTelex);
    void                      NoAnswerFromMSDNx47 ();
    void                      GotMSDNx47Version ();

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  protected slots: //reimplementing inherited
//    virtual void              AfterStartInitiation ();
    virtual void              OnEnterToOperatingState ();
    virtual void              OnExitFromOperatingState ();
    virtual void              OnEnterToCoolingState ();

  protected slots:
    virtual void              ReportGenieError (QString);
    virtual void              ReportMSDNx47Error (QString);
    virtual void              SendMagneticField (sPhysValue);
    virtual void              StartScanJob (sScanJobData);
//    virtual void              JustifyScanParams (sScanStartData);

  protected: //fields
    sPhysValue                Calibration;
    sTeleCom::sLink           Genie;
    sTeleCom::sLink           MSDNx47;
    QByteArray                RASOS_Version;
    QByteArray                MSDNx47_Version;
    sPhysValue                LatestHo;
    sPhysValue                LatestDeltaH;
    sPhysValue                LatestFrequency;
    sPhysValue                LatestAttenuation;
    int                       GainMantissaIndex;
                                //!< 0: 0.50; ...; 9: 4.0
    int                       GainOrder;
                                //!< 0: *100; ...; 4: *1000000
    int                       ModulationMantissaIndex;
                                //!< 0: 1.00; ...; 9: 0.125
    int                       ModulationOrder;
                                //!< 0: *10; ...; 3: *0.01
    eModulationMode           TheModulationMode;
    int                       NumOfCountsPerPoint;

  private slots:
    void                      LinkToGenie ();
    void                      LinkToMSDNx47 ();
    void                      GetSOSVersion ();
    void                      GetMSDNx47Version ();
    void                      OnNoAnswerFromSOS ();
    void                      OnNoAnswerFromMSDNx47 ();
    void                      DisconnectRodger ();

    void                      SendHo (sPhysValue);
    void                      SendDeltaH (sPhysValue);
    void                      SendFrequency (sPhysValue);
    void                      SendAttenuation (sPhysValue);
    void                      SendGain (sPhysValue);
    void                      SendModulation (sPhysValue);
    void                      SendModulationMode (eModulationMode);

    void                      ProcessGenieMessage (sTelex);
    void                      SignalRangeChanging (sPhysValue);
    void                      ProcessMSDNx47Message (sTelex);
    void                      ParseHandles (int  handles);

  private:
    void                      Put (sPhysValue  value
                                  ,sPhysValue  unit
                                  ,short       digits_after_dot
                                  ,literal     SOS_field);
    void                      Get (literal  prefix
                                  ,literal  SOS_field);

  private: //fields
    bool                      Emulate;
    sString                   Magic;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
