////////////////////////////////////////////////////////////////////////////////
/*! @file EPRMath.cpp EPR specific math.
- Part of PanBursian - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2014, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "EPRMath.h"
#include "TableDataFunction.h"
namespace RA {
//------------------------------------------------------------------------------

psMathValue  SubtractBaseLine (pcsMathValue  arg1)
{
  pcsDataFunction             Arg1(dynamic_cast<pcsDataFunction>(arg1));
  ASSERT( Arg1 );
  psMathValue                 Result = NULL;
  psTableDataFunction         R = NULL;
  sCurve::psTracer            T = NULL;
  int                         N = Arg1->NumberOfPoints();
  real                        Integral = 0.0;
  real                        SumDX = 0.0;
  T = Arg1->GetCurveTracer(sScales(sScale(Arg1->Multiplier().X)
                                  ,sScale(Arg1->Multiplier().Y)));
  R = new sTableDataFunction(N);
  R->SetMultiplier(Arg1->Multiplier());
  int                         PointNo = 0;
  sPoint                      Prev;
  while( T->NextPoint() ){
    R->SetValue(PointNo ,sPoint( T->Point.X , T->Point.Y ) );
    if( PointNo == 0 ){
      Prev = T->Point;
    }else{
      real                        DX = std::abs( T->Point.X - Prev.X );
      SumDX += DX;
      Integral += ( T->Point.Y + Prev.Y ) / 2 *  DX;
    }
    PointNo++;
  }
  Result = R;
  if( T ){ delete T;  T=NULL; };
  if( N > 1 ){
    T = R->GetCurveTracer(sScales(sScale(R->Multiplier().X)
                                 ,sScale(R->Multiplier().Y)));
    PointNo = 0;
    while( T->NextPoint() ){
      R->SetYValue(PointNo , T->Point.Y - Integral / SumDX );
      PointNo++;
    }
  }
  if( T ){ delete T;  T=NULL; };
  return Result;
}

//------------------------------------------------------------------------------
} //namespace RA
