////////////////////////////////////////////////////////////////////////////////
/*! @file LakeShore475Gaussmeter.h   LakeShore Model 475 DSP Gaussmeter.
- Part of PanBursian - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2015, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef LakeShore475Gaussmeter_H
#define LakeShore475Gaussmeter_H
#include "PanBursian.h"
#include "Gauge.h"
#include "TeleCom.h"
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sLakeShore475Gaussmeter)

//-------------------------------------------------- sLakeShore475Gaussmeter ---
/*! LakeShore Model 475 DSP Gaussmeter.
*/
class PanBursian_EXPORT  sLakeShore475Gaussmeter : public sGauge
{
  Q_OBJECT
  DRIVER(sLakeShore475Gaussmeter)

  public:
    virtual                   ~sLakeShore475Gaussmeter ();

  protected: //evolution of the inherited state machine
    psState                   State_W_GaussmeterPort;
    psState                   State_W_GaussmeterPort_Linking;
    psState                   State_W_GaussmeterPort_Checking;
    psState                   State_O_M_A_Pausing;
    psState                   State_O_M_A_Reading;

  signals: //for internal needs
    void                      GaussmeterPortLinked ();
    void                      GaussmeterPortError (QString = QString());
    void                      GotGaussmeterMessage (sTelex);
    void                      NoAnswerFromGaussmeter ();
    void                      GotGaussmeterName ();

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  protected slots: //reimplementing inherited
    virtual void              OnEnterToCoolingState ();
    virtual void              SetWantedPeriod ();

  protected slots:
    virtual void              ReportGaussmeterPortError (QString);
    virtual void              OnNoAnswerFromGaussmeter ();
    virtual void              RequestGaussmeterName ();
    virtual void              ParseGaussmeterName (sTelex);
    virtual void              RunTheClockIfNotYet ();
    virtual void              RequestField ();
    virtual void              ParseField (sTelex);

  protected: //fields
    sTeleCom::sLink           GaussmeterPort;
    QTimer *                  Clock;
    int                       Interval;//ms

  private slots:
    void                      LinkToGaussmeterPort ();
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
