/*! @file ADC_AD7321_VB.h  ADC interface.
- Part of the MSDN2047 firmware.
- Part of the Research Assistant project.
- Copyright(C) 2016, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef ADC_AD7321_VB_H
#define ADC_AD7321_VB_H
#include <stdbool.h>
#include <inttypes.h>
//------------------------------------------------------------------------------

#define INT6ENABLE 0b01000000


enum eADCReportingMode {cNever,cPeriodically,cSynchronized,cWaiting};


struct sADC
{
    bool                      EvaluateNoise;
    uint32_t                  MinIntervalBetweenReadings;
    enum eADCReportingMode    ReportingMode;
    bool                      GotSynchroimpulse;
    uint16_t                  NumberOfPointsToCollect;
    int32_t                   MaxNumOfSamplesToAverage; //if <= 2^19 then
                                                        //no overflow can happen

    int16_t                   InputValue; //!< Holds the result of Read().

    // intermediate averaging data
    bool                      AveragingJustCleared;
    int16_t                   FirstValue; //!< First in an averaging cycle.
    int32_t                   NumOfAveragedSamples;
    int32_t                   Sum;
    uint8_t                   NumOfNoiseSamples; // <= 2^6
    int32_t                   NoiseSum;
    uint32_t                  ShiftedDispersion;

    // last completely averaged result
    bool                      ToBeSent;
    int32_t                   Value;
    int32_t                   ValueDivider;
    uint32_t                  Dispersion;
    uint8_t                   DispersionDivider;

    // debug
    bool                      ReadADC;
    bool                      EmulateADC;
    int16_t                   NoiseLevel;
    uint8_t                   SkippedBits;
    int32_t                   OutputCounter;
    int32_t                   UnsentCounter;
    int32_t                   ClockSignalCounter;
    int32_t                   ClockProcessCounter;
};


extern struct sADC  ADC;


//public:
void      ADC_Init (void);
void      ADC_BreakAndReinitialize (void);
void      ADC_ReportParams (void);
void      ADC_SetRegime (void);
void      ADC_SetRange (int8_t  range);
int8_t    ADC_SetPeriod (uint32_t  period /*ns*/);
uint32_t  ADC_Period (void); /*ns*/
void      ADC_Stop (void);
int8_t    ADC_MeasurePeriodically (void);
int8_t    ADC_MeasureSynchronized (uint16_t  num_of_points);

//private:
void  ADC_Read (void);
void  ADC_AveragingClear (void);
void  ADC_AveragingFinalize (void);
void  ADC_GetNextReadingAndAverage (void);
void  ADC_Report (void);


//public:
void      ADC_OnClock (void);
void      ADC_OnSynchroimpulse (void);


//------------------------------------------------------------------------------
#endif
