////////////////////////////////////////////////////////////////////////////////
/*! @file SEXCalibration.cpp   Calibration of spectra from SEX-2544.
- Part of SOS2RA - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2015, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "SEXCalibration.h"
#include "ArrayDataFunction.h"
#include "TableDataFunction.h"
#include "DataArrayRecord.h"
namespace RA {
//------------------------------------------------------------------------------

const real DeltaHRatio = 1.034;

//------------------------------------------------------------------------------

psMathValue  calibrate1 (pcsMathValue  spectrum
                       ,pcsMathValue  h0)
{
//!@todo{math} sDataFunction::Replica() преобразует всё в sTableDataFunction
  pcsArrayDataFunction        Source(dynamic_cast<pcsArrayDataFunction>
                                                   (spectrum));
  pcsPhysValue                H0(dynamic_cast<pcsPhysValue>(h0));
  ASSERT( Source && H0 );
  psArrayDataFunction         Result;
  sPhysValue                  MX( Source->Multiplier().X );
  sPhysValue                  MY( Source->Multiplier().Y );
  real                        ArbH0;
  try{
    ArbH0 = ( *H0 / sPhysValue("Э") ).ToReal();
  }catch( rxRAlgebra ){
    return new sMathError("2nd argument (H0) must be in 'Э' units");
  }
  Result = new sArrayDataFunction(sPhysPair(MX,MY)
                                 ,Source->NumberOfPoints()
                                 ,ArbH0+DeltaHRatio*(Source->X0()-ArbH0)
                                 ,Source->Step()*DeltaHRatio
                                 );
  for( int PointNo=0 ; PointNo < Result->NumberOfPoints() ; PointNo++ ){
    Result->SetValue(PointNo,Source->PointY(PointNo));
  }
  return Result;
}

//------------------------------------------------------------------------------

psMathValue  calibrate2 (pcsMathValue  spectrum
                       ,pcsMathValue  h0)
{
  pcsTableDataFunction        Source(dynamic_cast<pcsTableDataFunction>
                                                   (spectrum));
  pcsPhysValue                H0(dynamic_cast<pcsPhysValue>(h0));
  ASSERT( Source && H0 );
  psTableDataFunction         Result;
  sPhysValue                  MX,MY;
  MX = Source->Multiplier().X * sPhysValue("Oe/Э");
  MY = Source->Multiplier().Y;
  real                        ArbH0;
  try{
    ArbH0 = ( *H0 / sPhysValue("Э") ).ToReal();
  }catch( rxRAlgebra ){
    return new sMathError("2nd argument (H0) must be in 'Э' units");
  }
  Result = new sTableDataFunction(sPhysPair(MX,MY)
                                 ,Source->NumberOfPoints()
                                 );
  for( int PointNo=0 ; PointNo < Result->NumberOfPoints() ; PointNo++ ){
    Result->SetValue(PointNo
                    ,sPoint(ArbH0 + DeltaHRatio*(Source->PointX(PointNo)-ArbH0)
                           ,Source->PointY(PointNo))
                    );
  }
  return Result;
}

//------------------------------------------------------------------------------

psMathValue  calibrate3 (pcsMathValue  spectrum)
{
  pcsDataArrayRecord      Source(dynamic_cast<pcsDataArrayRecord>
                                                   (spectrum));
  ASSERT( Source );
  psArrayDataFunction         Result;
  sPhysValue                  MX( Source->Multiplier().X );
  sPhysValue                  MY( Source->Multiplier().Y );
  real                        ArbH0;
  try{
    ArbH0 = ( 9.9999999 / sPhysValue("Э") ).ToReal();
  }catch( rxRAlgebra ){
    return new sMathError("2nd argument (H0) must be in 'Э' units");
  }
  Result = new sArrayDataFunction(sPhysPair(MX,MY)
                                 ,Source->NumberOfPoints()
                                 ,ArbH0+DeltaHRatio*(Source->X0()-ArbH0)
                                 ,Source->Step()*DeltaHRatio
                                 );
  for( int PointNo=0 ; PointNo < Result->NumberOfPoints() ; PointNo++ ){
    Result->SetValue(PointNo,Source->PointY(PointNo));
  }
  return Result;
}

//------------------------------------------------------------------------------
} //namespace RA
