////////////////////////////////////////////////////////////////////////////////
/*! @file SOS2RA.cpp SOS to RA transitional package.
- Part of SOS2RA - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  - http://qt-project.org/
- Copyright(C) 2015, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#include "SOS2RA.h"
#include "SOSDropPanel.h"
//#include "SOSImporter.h"
#include "attr.h"
#include "Symbols.h"
#include "SEXCalibration.h"
#include "ArrayDataFunction.h"
#include "TableDataFunction.h"
#include "DataArrayRecord.h"
//#include <QMetaType>
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sSOS2RA)

//------------------------------------------------------------------------------

sVersion  sSOS2RA::Version ()
{
  return sVersion(RAP_VERS_MAJOR
                 ,RAP_VERS_MINOR
                 ,ZAKAVYCH(RAP_VERS_PATCH)
                 ,__DATE__
                 );
}


sVersion  sSOS2RA::CoreVersionUsed ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}


sVersion  sSOS2RA::CoreVersionMin ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}


QString  sSOS2RA::UserFriendlyName ()
{
  return trUtf8("Import from SOS");
}


QString  sSOS2RA::About ()
{
  return trUtf8(
      "The package provides transitional tools from SOS to RA.\n\n"
      "Copyright (C) 2015, Viktor E. Bursian, St.Petersburg, Russia.\n"
      "                    Viktor.Bursian@mail.ioffe.ru"
                );
}


void  sSOS2RA::InstallDefaults (sPtr<sFolderNode>  DeviceList)
{
  try{
    sPtr<sFolderNode>              DeviceCfg;

    DeviceList |= "SOS Drop Panel";
    DeviceCfg = DeviceList >> "SOS Drop Panel";
    DeviceCfg << attr("Class","sSOSDropPanel");
    sSOSDropPanel::Preconfigure(DeviceCfg);

//    DeviceList |= "Batch Import from SOS";
//    DeviceCfg = DeviceList >> "Batch Import from SOS";
//    DeviceCfg << attr("Class","sSOSImporter");
//    sSOSImporter::Preconfigure(DeviceCfg);

    /*________________________________________template for device
    DeviceList |= "__DEVICE_ID__";
    DeviceCfg = DeviceList >> "__DEVICE_ID__";
    DeviceCfg << attr("Class","__DRIVER_CLASS_NAME__");
    __DRIVER_CLASS_NAME__::Preconfigure(DeviceCfg);
    //if there are links:
    DeviceCfg >> "Links" >> "__LINK_NAME__"
        << attr("DeviceId","__DEVICE_ID__FOR_THE_LINK__");
    //if you wanna change some of the default settings:
    DeviceCfg >> "Settings" << attr("",);
    DeviceCfg >> "Settings" << attr("",);
    */
  }catch(...){
    ;
  };
}


sSOS2RA::sSOS2RA ()
{
  sUnit::Register("B"
                 ,"-..-..-..-..-..-..-..----d--..-..-..-..-..-..-..-"
                 //Attention! When you change here the prefix allowance, the
                 //program ignores (and resets!) the settings made by user in
                 //the RANet Knowledgebase.
                 );
  sUnit::Register("Э");
  sUnit::Register("Oe");
  sUnit::Register("Hz");
  sUnit::Register(Symbol::SpinUnit);
  sUnit::Register("Spin" ,sPhysValue(Symbol::SpinUnit));
  sUnit::Register(Symbol::PhotonUnit);

  sMath::Definitions.Add(&calibrate1,"calibrate",sArrayDataFunction::Type()
                                               ,sPhysValue::Type());
  sMath::Definitions.Add(&calibrate2,"calibrate",sTableDataFunction::Type()
                                               ,sPhysValue::Type());
//  sMath::Definitions.Add(&calibrate3,"calibrate",sDataArrayRecord::Type());
}

//------------------------------------------------------------------------------
} //namespace RA
