////////////////////////////////////////////////////////////////////////////////
/*! @file SOSDropPanel.cpp Import and show SOS files one by one.
- Part of SOS2RA - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  - http://qt-project.org/
- Copyright(C) 2015, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#include "SOSDropPanel.h"
#include "attr.h"
#include "StringsAndQStrings.h"
//#include "tp2c.h"
#include "Log.h"
#include "Trace.h"
#include <QMimeData>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QMessageBox>
#include <iostream>
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sSOSDropPanel)

//------------------------------------------------------------ sSOSDropPanel ---
/*! @class sSOSDropPanel

В зависимости от значения Linked и Topic имеет 3 состояния. Это не
состояния statemachine, а то, что показывается на графике, и, соответственно,
куда идёт импорт.
- (A) -- Linked && Topic.IsNULL() -- ничего не показывается (окна отсутствуют)
- (B) -- ! Linked && Topic.IsNULL() -- в точности как sPlot создаёт собственный
  sFolderNode, размещённый в WidgetConfig, как значение WidgetConfig>>"Content",
  и его показывает
- (C) -- Linked && Topic.IsCorrect()

*/


void  sSOSDropPanel::Preconfigure (sNodePtr  DeviceCfg)
{
  sPtr<sFolderNode>              DeviceSettings;
  DeviceCfg |= "Settings";
  DeviceSettings = DeviceCfg >> "Settings";
//  if( DeviceSettings.IsCorrect() ){
//    DeviceSettings |= attr("SOSRelease","Bursian");
//    DeviceSettings |= attr("PreserveColors",true);
//  }
  sConsole::Preconfigure(DeviceCfg);
}


sSOSDropPanel::~sSOSDropPanel()
{
//  if( DeviceSettings.IsCorrect() ){
//    DeviceSettings << attr("PhysParameter",PhysParameter);
//  }
  Clear();
}


sSOSDropPanel::sSOSDropPanel (sString    DeviceId
                           ,sNodePtr   DeviceCfg
                           ,sNodePtr   WidgetCfg
                           ,QWidget *  parent   )
    :sConsole(DeviceId,DeviceCfg,WidgetCfg,parent)
    ,ShouldBeLinked(false)
    ,SOSRelease(sSOSFileImport::SOSRelease_Bursian)
    ,PreserveColors(true)
    ,Evolution(NULL)
    ,Chart(NULL)
    ,Legend(NULL)
{
  try{ //get config parameters
//    sString                     R;
//    if( get_from<sTextNode>(DeviceSettings>>"SOSRelease",R) ){
//      if( R == "Bursian" )  SOSRelease=sSOSFileImport::SOSRelease_Bursian;
//      if( R == "Nelson" )  SOSRelease=sSOSFileImport::SOSRelease_Nelson;
//    };
//    get_from<sBooleanNode>(DeviceSettings>>"PreserveColors",PreserveColors);
  }catch(...){
    SetInvalid(tr("Something wrong in configuration."));
  }
  //evolution of the inherited state machine
  {
    //evolution of the State_Operating
    {
    }
  }
}


void  sSOSDropPanel::AppearanceMakeUp ()
{
  sConsole::AppearanceMakeUp();
  try{ //get config parameters (which are necessary for appearance)
//    if( ! get_from<sIntegerNode>(DeviceSettings>>"IntParameter",IntParameter) )
//      throw xDeviceLaunch;
  }catch(...){
    SetInvalid(tr("Something wrong in configuration."));
  }
  //add new widgets
  {
    SOSReleaseLabel = new QLabel("Type:",this);
    SOSReleaseComboBox = new QComboBox(this);
    SOSReleaseComboBox->addItem("Bursian",sSOSFileImport::SOSRelease_Bursian);
    SOSReleaseComboBox->addItem("Feofilov",sSOSFileImport::SOSRelease_Feofilov);
    SOSReleaseComboBox->addItem("Nelson",sSOSFileImport::SOSRelease_Nelson);
    HistoricalAllocationCheckBox = new QCheckBox("Recognize record date-number and "
                                            "allocate in Records."
                                           ,this);
    HistoricalAllocationCheckBox->setChecked(true);
    DecadeLabel = new QLabel("Decade:",this);
    DecadeComboBox = new QComboBox(this);
    DecadeComboBox->addItem("1990-1999",1990);
    DecadeComboBox->addItem("2000-2009",2000);
    DecadeComboBox->addItem("2010-2019",2010);
    DecadeComboBox->setCurrentText("2010-2019");

    LinkedCheckBox = new QCheckBox("Link to",this);
    TheLinkField = new sDropNodeWidget(this);

    SampleBindingCheckBox = new QCheckBox("Replace text",this);
    SampleTextField = new QLineEdit(this);
    SampleLabel = new QLabel("=> sample:",this);
    SampleField = new sDropNodeWidget(this);

    Splitter = new QSplitter(this);
    Splitter->setOrientation(Qt::Vertical);

    QActionGroup *              AG = new QActionGroup(this);
    ActnLegendBottom = new QAction(AG);
    ActnLegendBottom->setIcon(QIcon(":/RAGUI/icons/LegendBottom.png"));
    ActnLegendBottom->setCheckable(true);
    ActnLegendBottom->setChecked(true);
    addAction(ActnLegendBottom);
    ActnLegendRight = new QAction(AG);
    ActnLegendRight->setIcon(QIcon(":/RAGUI/icons/LegendRight.png"));
    ActnLegendRight->setCheckable(true);
    addAction(ActnLegendRight);
  }
  //layout widgets
  {
    QVBoxLayout *               Layout( new QVBoxLayout );
    QHBoxLayout *               Layout0( new QHBoxLayout );
    QHBoxLayout *               Layout1( new QHBoxLayout );
    QHBoxLayout *               Layout2( new QHBoxLayout );
    setLayout(Layout);

    Layout0->addWidget(SOSReleaseLabel);
    Layout0->addWidget(SOSReleaseComboBox);
    Layout0->addWidget(HistoricalAllocationCheckBox);
    Layout0->addWidget(DecadeLabel);
    Layout0->addWidget(DecadeComboBox);
    Layout0->setStretchFactor(SOSReleaseLabel,0);
    Layout0->setStretchFactor(SOSReleaseComboBox,2);
    Layout0->setStretchFactor(HistoricalAllocationCheckBox,0);
    Layout0->setStretchFactor(DecadeLabel,0);
    Layout0->setStretchFactor(DecadeComboBox,1);
    Layout->addItem(Layout0);

    Layout1->addWidget(LinkedCheckBox);
    Layout1->addWidget(TheLinkField);
    Layout1->setStretchFactor(LinkedCheckBox,0);
    Layout1->setStretchFactor(TheLinkField,1);
    Layout->addItem(Layout1);

    Layout2->addWidget(SampleBindingCheckBox);
    Layout2->addWidget(SampleTextField);
    Layout2->addWidget(SampleLabel);
    Layout2->addWidget(SampleField);
    Layout2->setStretchFactor(SampleBindingCheckBox,0);
    Layout2->setStretchFactor(SampleTextField,2);
    Layout2->setStretchFactor(SampleLabel,0);
    Layout2->setStretchFactor(SampleField,1);
    Layout->addItem(Layout2);

    Layout->addWidget(Splitter);
    Layout->setStretchFactor(Layout0,0);
    Layout->setStretchFactor(Layout1,0);
    Layout->setStretchFactor(Layout2,0);
    Layout->setStretchFactor(Splitter,1);
//    Layout->addStretch(100);
  }
  connect(LinkedCheckBox ,SIGNAL(clicked(bool))
         ,this      ,SLOT(OnLinkedCheckBoxClick(bool)));
  connect(TheLinkField ,SIGNAL(Edited())
         ,this     ,SLOT(OnTheLinkFieldEdit()));
  connect(this  ,SIGNAL(CursorPositionChanged(sPhysPair,sPhysPair
                                             ,sString,sString))
         ,MainWindow() ,SLOT(ShowCursorInfo(sPhysPair,sPhysPair
                                           ,sString,sString))
         ,Qt::QueuedConnection);
  UpdatingTimer = new QTimer(this);
  UpdatingTimer->start(sNavigator::UpdatingPeriod);
  connect(ActnLegendBottom,SIGNAL(toggled(bool))
         ,this,SLOT(PutLegendAtTheBottom(bool)));
}


void  sSOSDropPanel::RestoreState ()
{
  sConsole::RestoreState();
  try{
    bool                      B;
    int                       I,I1,I2;
    if( get_from<sIntegerNode>(WidgetConfig>>"SOSRelease",I) ){
      SOSReleaseComboBox->setCurrentIndex(I);
    }
    if( get_from<sBooleanNode>(WidgetConfig>>"HistoricalAllocation",B) ){
      HistoricalAllocationCheckBox->setChecked(B);
    }
    if( get_from<sIntegerNode>(WidgetConfig>>"Decade",I) ){
      DecadeComboBox->setCurrentIndex(I);
    }
    if( get_from<sBooleanNode>(WidgetConfig>>"SampleBinding",B) ){
      SampleBindingCheckBox->setChecked(B);
    }
    SampleField->SetValue(WidgetConfig>>"Sample");
    get_from<sBooleanNode>(WidgetConfig>>"Linked" ,ShouldBeLinked);
    LinkedCheckBox->setChecked(ShouldBeLinked);
    TheLinkField->setVisible(ShouldBeLinked);
    if( ! Evolution ){ /*! @patch This is a workaround to prevent second execution
                                  on second call of ResoreState()
                       */
      if( ShouldBeLinked ){
        TheLinkField->SetValue(WidgetConfig >> "Content");
        OnTheLinkFieldEdit();
      }else{
        CreateChartAndLegend( WidgetConfig >= "Content" );
      }
    }
    if( get_from<sIntegerNode>(WidgetConfig>>"Splitter.Orientation" ,I) ){
      Splitter->setOrientation(Qt::Orientation(I));
      ActnLegendBottom->setChecked(Splitter->orientation() == Qt::Vertical);
      ActnLegendRight->setChecked(Splitter->orientation() == Qt::Horizontal);
    }
    if(    get_from<sIntegerNode>(WidgetConfig>>"Splitter.Part1" ,I1)
        && get_from<sIntegerNode>(WidgetConfig>>"Splitter.Part2" ,I2) ){
      QList<int>    List;
      List.append(I1);
      List.append(I2);
      Splitter->setSizes(List);
    }
    if( Chart )
      Chart->RestoreScales(WidgetConfig);
    sString                   S;
    if( get_from<sTextNode>(WidgetConfig>>"SampleText" ,S) ){
      SampleTextField->setText(ToQString(S));
    }
  }catch(...){
  }
}


void  sSOSDropPanel::StoreState ()
{
  disconnect(TheLinkField,0,0,0);
  sConsole::StoreState();
  try{
    WidgetConfig << attr("SOSRelease",integer(SOSReleaseComboBox->currentIndex()));
    WidgetConfig << attr("HistoricalAllocation"
                        ,HistoricalAllocationCheckBox->isChecked());
    WidgetConfig << attr("Decade",integer(DecadeComboBox->currentIndex()));
    WidgetConfig << attr("SampleBinding"
                        ,SampleBindingCheckBox->isChecked());
    WidgetConfig << attr("SampleText" ,FromQString(SampleTextField->text()));
    if( SampleField->Value.IsNULL() ){
      WidgetConfig.DelAttr("Sample");
    }else{
      WidgetConfig << attr("Sample" ,SampleField->Value);
    }
    WidgetConfig << attr("Linked" ,ShouldBeLinked);
    WidgetConfig << attr("Splitter.Orientation" ,(int)Splitter->orientation());
    if( Chart ){
      WidgetConfig << attr("Splitter.Part1" ,(int)Splitter->sizes().at(0));
      WidgetConfig << attr("Splitter.Part2" ,(int)Splitter->sizes().at(1));
      Chart->StoreScales(WidgetConfig);
    }
  }catch(...){
  }
}


void  sSOSDropPanel::contextMenuEvent (QContextMenuEvent *  event)
{
  QMenu                       PopupMenu/*(this)*/;
  PopupMenu.addActions(actions());
  PopupMenu.exec(event->globalPos());
}


void  sSOSDropPanel::PutLegendAtTheBottom (bool  AtTheBottom)
{
  if( AtTheBottom ){
    Splitter->setOrientation(Qt::Vertical);
  }else{
    Splitter->setOrientation(Qt::Horizontal);
  }
}


void  sSOSDropPanel::OnTheLinkFieldEdit ()
{
  ASSERT(ShouldBeLinked);
  if( Evolution ){
    RemoveChartAndLegend();
  }
  if( ! TheLinkField->Value.IsNULL() ){
    if( ! CreateChartAndLegend(TheLinkField->Value) ){ //case A to case C
      TheLinkField->Clear();
    }
  }
}


void  sSOSDropPanel::OnLinkedCheckBoxClick (bool  link_to)
{
  ASSERT( link_to != ShouldBeLinked );
  if( ! ShouldBeLinked ){ //case B to case A
    RemoveChartAndLegend();
    ShouldBeLinked = true;
  }else if( TheLinkField->Value.IsNULL() ){ //case A to case B
    CreateChartAndLegend( sNodePtr( new sFolderNode ) );
    ShouldBeLinked = false;
  }else{ //case C to case B
    RemoveChartAndLegend();
    TheLinkField->Clear();
    CreateChartAndLegend( sNodePtr( new sFolderNode ) );
    ShouldBeLinked = false;
  }
  TheLinkField->setVisible(ShouldBeLinked);
}


void  sSOSDropPanel::Clear ()
{
  if( Legend ){ delete Legend;  Legend=NULL; };
  if( Chart ){ delete Chart;  Chart=NULL; };
  if( Evolution ){ delete Evolution;  Evolution=NULL; };
  TheLinkField->Clear();
}


void  sSOSDropPanel::RemoveChartAndLegend ()
{
  setAcceptDrops(false);
  try{
    if( Chart )
      Chart->StoreScales(WidgetConfig);
    Clear();
    WidgetConfig.DelAttr("Content");
  }catch(...){
  }
}


bool  sSOSDropPanel::CreateChartAndLegend (sNodePtr  folder)
{
  sPtr<sFolderNode>           Content(folder);
  if( Content.IsCorrect() ){
    WidgetConfig << attr("Content",Content);
    Evolution = new sNet2TreeEvolution(sAttribute(sAttrName(Title()),Content));
    Chart = new sChart(Evolution ,this);
    Legend = new sNavigator(Evolution ,this);

    Splitter->addWidget(Chart);
    Splitter->addWidget(Legend);

    connect(Chart ,SIGNAL(CursorPositionChanged(sPhysPair,sPhysPair
                                               ,sString,sString))
           ,this  ,SIGNAL(CursorPositionChanged(sPhysPair,sPhysPair
                                               ,sString,sString)));
    connect(Evolution,SIGNAL(Changed()),Legend,SLOT(OnNetChange()));
    connect(Evolution,SIGNAL(Changed()),Chart,SLOT(OnNetChange()));
    Evolution->Initiate();
    connect(UpdatingTimer, SIGNAL(timeout()), Evolution, SLOT(Update()));
    Chart->RestoreScales(WidgetConfig);
    setAcceptDrops(true);
    return true;
  }
  return false;
}


void  sSOSDropPanel::dragEnterEvent (QDragEnterEvent *  event)
{
  if( event->mimeData()->hasUrls() ){
    if( event->mimeData()->urls().size() == 1 ){
      QString                     Path( event->mimeData()->urls().at(0)
                                                          .toLocalFile() );
      if( ! Path.isEmpty() && Path.endsWith(".sop",Qt::CaseInsensitive) ){
        event->acceptProposedAction();
      }
    }
  }
}


void  sSOSDropPanel::dropEvent (QDropEvent *  event)
{
  if( event->mimeData()->hasUrls() ){
    if( event->mimeData()->urls().size() == 1 ){
      QString                     Path( event->mimeData()->urls().at(0)
                                                          .toLocalFile() );
      if( ! Path.isEmpty() && Path.endsWith(".sop",Qt::CaseInsensitive) ){
        RANet::Log.Put(sLog::Info,"Import",FromQString(Path)+" drop");
        try{
          sPtr<sFolderNode>           Content(WidgetConfig>>"Content");
          if( Content.IsCorrect() ){
            sSOSFileImport              I(SOSRelease,PreserveColors);
            I.SetSample(FromQString(SampleTextField->text().simplified())
                       ,SampleField->Value);
            sPtr<sDataArrayRecord>  R;
            R = I.Import_SOP(FromQString(Path));
            if( R.IsCorrect() ){
              sPtr<sFolderNode>         Sample(R>>"sample");
              if( Sample.IsCorrect() ){
                R->AllocateHistorically(Sample >= "records");
              }
              if( R->RecordNo() > 0 ){
                R->AllocateRecord();
                Content << attr(R);
              }else{
                Content << attr(sPath(FromQString(Path)).Name(),R);
              }
              event->acceptProposedAction();
            }
          }
        }catch(...){
          RANet::Log.Put(sLog::Info,"Import",FromQString(Path)+" FAILURE!");
        }
      }
    }
  }
}

//------------------------------------------------------------------------------
} //namespace RA


