////////////////////////////////////////////////////////////////////////////////
/*! @file SOSDropPanel.h   Import and show SOS files one by one.
- Part of SOS2RA - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  - http://qt-project.org/
- Copyright(C) 2015, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef SOSDropPanel_H
#define SOSDropPanel_H
#include "SOS2RA.h"
#include "Console.h"
#include "NetWidgets.h"
#include "SOSFileImport.h"
#include "Navigator.h"
#include "Chart.h"
#include <QTimer>
#include <QCheckBox>
#include <QComboBox>
#include <QSplitter>
#include <QLineEdit>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sSOSDropPanel)

//------------------------------------------------------------ sSOSDropPanel ---
/*! Import and show SOS files one by one.
*/
class  SOS2RA_EXPORT  sSOSDropPanel : public sConsole
{
  Q_OBJECT
  CONSOLE(sSOSDropPanel)
  public:
    virtual                   ~sSOSDropPanel ();
    virtual void              RestoreState ();
    virtual void              StoreState ();

  public slots:
    void                      PutLegendAtTheBottom (bool);

  signals:
    void                      CursorPositionChanged (sPhysPair  position
                                                    ,sPhysPair  reference
                                                    ,sString    x_title
                                                    ,sString    y_title);

  protected: //reimplementing inherited
    virtual void              AppearanceMakeUp ();
    virtual void              contextMenuEvent (QContextMenuEvent *);
    virtual void              dragEnterEvent (QDragEnterEvent * event);
    virtual void              dropEvent (QDropEvent *  event);

  protected slots:
    void                      OnTheLinkFieldEdit ();
    void                      OnLinkedCheckBoxClick (bool);

  protected: //fields
    bool                      ShouldBeLinked;

  private:
    void                      Clear ();
    void                      RemoveChartAndLegend ();
    bool                      CreateChartAndLegend (sNodePtr  folder);

  private: //fields
    sSOSFileImport
               ::eSOSRelease  SOSRelease;
    bool                      PreserveColors;
    QLabel *                  SOSReleaseLabel;
    QComboBox *               SOSReleaseComboBox;
    QCheckBox *               HistoricalAllocationCheckBox;
    QLabel *                  DecadeLabel;
    QComboBox *               DecadeComboBox;
    QCheckBox *               LinkedCheckBox;
    psDropNodeWidget          TheLinkField;
    QCheckBox *               SampleBindingCheckBox;
    QLineEdit *               SampleTextField;
    QLabel *                  SampleLabel;
    psDropNodeWidget          SampleField;
    psNet2TreeEvolution       Evolution;
    psChart                   Chart;
    psNavigator               Legend;
    QTimer *                  UpdatingTimer;
    QSplitter *               Splitter;
    QAction *                 ActnLegendBottom;
    QAction *                 ActnLegendRight;
};


//------------------------------------------------------------------------------
} //namespace RA
#endif
