////////////////////////////////////////////////////////////////////////////////
/*! @file SOSFileImport.h   Class sSOSFileImport (imports SOP and SOS files).
- Part of SOS2RA - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  - http://qt-project.org/
- Copyright(C) 2000-2015, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef SOSFileImport_H
#define SOSFileImport_H
#include "SOS2RA.h"
#include "Files.h"
#include "NodePtr.h"
#include "DataArrayRecord.h"
namespace RA {
//------------------------------------------------------------------------------
//----------------------------------------------------------- sSOSFileImport ---


class SOS2RA_EXPORT  sSOSFileImport : private sFile
{

  public: //types
    enum                      eSOSRelease {SOSRelease_Bursian
                                          ,SOSRelease_Feofilov
                                          ,SOSRelease_Nelson
                                          };
    enum                      eFileType {FileTypeSOP
                                        ,FileTypeSOS
                                        };

  public:
                              sSOSFileImport (eSOSRelease  release
                                             ,bool         preserve_colors);
    void                      SetSample (rcsString          sample_text
                                        ,sPtr<sFolderNode>  sample);
    sPtr<sDataArrayRecord>
                              Import_SOP (rcsPath  file_path);
  private:
    void                      Modify_SOP_Bursian (sPtr<sDataArrayRecord> &
                                                                  result
                                                 ,sPtr<sDataArrayRecord> &
                                                                  imported
                                                 ,rcsString       picture_name
                                                 ,rcsPath         file_path
                                                 ,rcsTime         file_time);
    bool                      DetermineDateAndTime (sPtr<sDataArrayRecord>
                                                                  record
                                                 ,rcsString       picture_name
                                                 ,rcsPath         file_path
                                                 ,rcsTime         file_time
                                                 ,int             decade_start);
    bool                      SEX_Pars_Transform (sPtr<sDataArrayRecord>
                                                                  result
                                                 ,sPtr<sDataArrayRecord>
                                                                  imported);
    bool                      ExtractSample (rsString  text);
    void                      CleanComment (rsString  text);
    void                      Modify_SOP_Nelson (sPtr<sDataArrayRecord> &
                                                                 result
                                                ,sPtr<sDataArrayRecord> &
                                                                 imported
                                                ,rcsString       picture_name
                                                ,rcsPath         file_path
                                                ,rcsTime         file_time);
    sAttribute                Get_SOS_Object ();
    sString                   Load_SOS_Item ();
    sAttribute                Load_SOS_Remark ();
    sAttribute                Load_SOS_Field ();
    sAttribute                Load_SOS_IntegerField ();
    sAttribute                Load_SOS_LongintField ();
    sAttribute                Load_SOS_StringField ();
    sAttribute                Load_SOS_FixedField ();
    sAttribute                Load_SOS_FloatField ();
    sAttribute                Load_SOS_Figure ();
    sAttribute                Load_SOS_Window ();
    psCurveAppearance         Load_SOS_Line(rsString  name);
    sAttribute                Load_SOS_Curve ();
    sAttribute                Load_SOS_Picture ();
    sAttribute                Load_SOS_ColorIndicator ();
    sAttribute                Load_SOS_Spectrum ();
    void                      Skip (int);
    sString                   DOStoUTF8 (literal) const;

  private: //fields
    eSOSRelease               SOSRelease;
    bool                      PreserveColors;
    eFileType                 FileType;
    sString                   SampleText;
    sPtr<sFolderNode>         Sample;
//    rcsFileLocation FileLocation;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
