////////////////////////////////////////////////////////////////////////////////
/*! @file SOSImporter.cpp A new console pattern for the developers.
- Part of Examples - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  - http://qt-project.org/
- Copyright(C) 2015, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#include "SOSImporter.h"
#include "attr.h"
#include "StringsAndQStrings.h"
//#include "tp2c.h"
#include "Log.h"
#include "Trace.h"
#include <QMimeData>
#include <QGridLayout>
#include <QMessageBox>
//#include <iostream>
//#include <iostream>
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sSOSImporter)

//------------------------------------------------------------- sSOSImporter ---
/*! @class sSOSImporter

@par The state machine

The upper-level state machine is inherited from sConsole.  
Below is the evolution of some states.

*//*dotfile "SOSImporter.state_graph"
*/


void  sSOSImporter::Preconfigure (sNodePtr  DeviceCfg)
{
  sPtr<sFolderNode>              DeviceSettings;
  DeviceCfg |= "Settings";
  DeviceSettings = DeviceCfg >> "Settings";
  if( DeviceSettings.IsCorrect() ){
    DeviceSettings |= attr("SOSRelease","Bursian");
    DeviceSettings |= attr("PreserveColors",true);
  }
  sConsole::Preconfigure(DeviceCfg);
}


sSOSImporter::~sSOSImporter()
{
  if( DeviceSettings.IsCorrect() ){
//    DeviceSettings << attr("PhysParameter",PhysParameter);
  }
}


sSOSImporter::sSOSImporter (sString    DeviceId
                           ,sNodePtr   DeviceCfg
                           ,sNodePtr   WidgetCfg
                           ,QWidget *  parent   )
    :sConsole(DeviceId,DeviceCfg,WidgetCfg,parent)
    ,AlsoPutIntoChronologicalStorage(false)
    ,SOSRelease(sSOSFileImport::SOSRelease_Bursian)
    ,PreserveColors(true)
{
  try{ //get config parameters
    sString                     R;
    if( get_from<sTextNode>(DeviceSettings>>"SOSRelease",R) ){
      if( R == "Bursian" )  SOSRelease=sSOSFileImport::SOSRelease_Bursian;
      if( R == "Nelson" )  SOSRelease=sSOSFileImport::SOSRelease_Nelson;
    };
    get_from<sBooleanNode>(DeviceSettings>>"PreserveColors",PreserveColors);
  }catch(...){
    SetInvalid(tr("Something wrong in configuration."));
  }
  //evolution of the inherited state machine
  {
    //evolution of the State_Operating
    {
    }
  }
  setAcceptDrops(true);
}


void  sSOSImporter::AppearanceMakeUp ()
{
  sConsole::AppearanceMakeUp();
  try{ //get config parameters (which are necessary for appearance)
//    if( ! get_from<sIntegerNode>(DeviceSettings>>"IntParameter",IntParameter) )
//      throw xDeviceLaunch;
  }catch(...){
    SetInvalid(tr("Something wrong in configuration."));
  }
  //add new widgets
  {
    SourceTopPathLabel = new QLabel("SOS data root directory:",this);
    SourceTopPathField = new QLineEdit(this);
    PathToProcessLabel = new QLabel("Subdirectory or file to process:",this);
    PathToProcessField = new QLineEdit(this);
    CatalogizedStorageLabel = new QLabel("Import into:",this);
    CatalogizedStorageField = new sDropNodeWidget(this);
    ChronologicalStorageCheck = new QCheckBox("also put chronologically into:"
                                             ,this);
    ChronologicalStorageField = new sDropNodeWidget(this);
    ImportBtn = new QPushButton("Import",this);
    StopBtn = new QPushButton("Stop",this);
  }
  //layout widgets
  {
    QGridLayout *               Layout( new QGridLayout );
    setLayout(Layout);
    Layout->setMargin(3);
    Layout->setHorizontalSpacing(6);
    Layout->setVerticalSpacing(2);
    Layout->addWidget(SourceTopPathLabel        ,0,0);
    Layout->addWidget(CatalogizedStorageLabel   ,0,1,1,2);
    Layout->addWidget(SourceTopPathField        ,1,0);
    Layout->addWidget(CatalogizedStorageField   ,1,1,1,2);
    Layout->addWidget(ChronologicalStorageCheck ,2,1,1,2);
    Layout->addWidget(ChronologicalStorageField ,3,1,1,2);
    Layout->addWidget(PathToProcessLabel        ,4,0);
    Layout->addWidget(PathToProcessField        ,5,0,1,3);
    Layout->addWidget(ImportBtn       ,8,1);
    Layout->addWidget(StopBtn         ,8,2);
//    Layout->addStretch(100);
  }
//  connect(FldTopic ,SIGNAL(Changed()) ,this ,SLOT(OnTopicChange()));
  connect(ImportBtn ,SIGNAL(clicked()) ,this ,SIGNAL(Import()));
  connect(StopBtn ,SIGNAL(clicked()) ,this ,SIGNAL(Stop()));
}


void  sSOSImporter::RestoreState ()
{
  sConsole::RestoreState();
  try{ //get stored settings
    get_from<sBooleanNode>(WidgetConfig>>"AlsoPutIntoChronologicalStorage"
                          ,AlsoPutIntoChronologicalStorage);
  }catch(...){
    SetInvalid(tr("Something wrong in the desktop."));
  }
  ChronologicalStorageCheck->setChecked(AlsoPutIntoChronologicalStorage);
}


void  sSOSImporter::StoreState ()
{
  sConsole::StoreState();
  if( WidgetConfig.IsCorrect() ){
    WidgetConfig << attr("AlsoPutIntoChronologicalStorage"
                        ,AlsoPutIntoChronologicalStorage);
  }
}


bool  sSOSImporter::CanClose ()
{
//  if(  )
//    return false;
  return true;
}


void  sSOSImporter::OnEnterToOperatingState ()
{
  sConsole::OnEnterToOperatingState();
}


void  sSOSImporter::ImportStart ()
{
//QString QDir::relativeFilePath(const QString & fileName) const
}


void  sSOSImporter::ImportFile ()
{

}

//------------------------------------------------------------------------------
} //namespace RA


