////////////////////////////////////////////////////////////////////////////////
/*! @file SOSImporter.h   A new console pattern for the developers.
- Part of SOS2RA - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  - http://qt-project.org/
- Copyright(C) 2015, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor_dot_Bursian_at_mail_dot_ioffe_dot_ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef SOSImporter_H
#define SOSImporter_H
#include "SOS2RA.h"
#include "Console.h"
#include "NetWidgets.h"
#include "SOSFileImport.h"
#include <QDir>
#include <QFileInfo>
#include <QFileInfoList>
#include <QLabel>
#include <QLineEdit>
#include <QCheckBox>
#include <QPushButton>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sSOSImporter)

//-------------------------------------------------------------- sSOSImporter ---
/*! A new console pattern for the developers.
*/
class  SOS2RA_EXPORT  sSOSImporter : public sConsole
{
  Q_OBJECT
  CONSOLE(sSOSImporter)
  public:
    virtual                   ~sSOSImporter ();
    virtual void              RestoreState ();
    virtual void              StoreState ();
    virtual bool              CanClose ();

  protected: //reimplementing inherited
    virtual void              AppearanceMakeUp ();

  protected: //evolution of the inherited state machine
    psState                   State_O_Standby;
    psState                   State_O_Importing;

  signals: //for internal needs
    void                      Import ();
    void                      Stop ();

  protected:

  protected slots: //reimplementing inherited
    virtual void              OnEnterToOperatingState ();

  protected slots:
    virtual void              ImportStart ();
    virtual void              ImportFile ();

  protected: //fields
    QString                   SourceRootDirPath;
    sPtr<sFolderNode>         CatalogizedStorage; //!< target root
    bool                      AlsoPutIntoChronologicalStorage;
    sPtr<sFolderNode>         ChronologicalStorage;

    QDir                      SourceTopDir;
    sPtr<sFolderNode>         TargetTop;
    QDir                      SourceDir;
    sPtr<sFolderNode>         Target;
    QString                   SourceName;

  private slots:

  private:

  private: //fields
    sSOSFileImport
               ::eSOSRelease  SOSRelease;
    bool                      PreserveColors;

    QLabel *                  SourceTopPathLabel;
    QLineEdit *               SourceTopPathField;
    QLabel *                  PathToProcessLabel;
    QLineEdit *               PathToProcessField;
    QLabel *                  CatalogizedStorageLabel;
    psDropNodeWidget          CatalogizedStorageField;
    QCheckBox *               ChronologicalStorageCheck;
    psDropNodeWidget          ChronologicalStorageField;
    QPushButton *             ImportBtn;
    QPushButton *             StopBtn;
};


//------------------------------------------------------------------------------
} //namespace RA
#endif
