////////////////////////////////////////////////////////////////////////////////
/*! @file SR400Counter.h   Управление счетчиком SR400.
- Part of SR400Drivers - a Research Assistant Plugin package.
- Uses  RAGUI - the Research Assistant Graphical User Interface.
- Uses  QtGui v.5.0  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010-2016, Dmitry K. Nelson, St.Petersburg, Russia.
                          D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef SR400Counter_H
#define SR400Counter_H
#include "SR400Drivers.h"
#include "Gauge.h"
#include "TeleCom.h"
#include <QTime>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sSR400Counter)

//---------------------------------------------------------- sSR400Counter ---
/*! Управление счетчиком SR400.
*/
class SR400Drivers_EXPORT  sSR400Counter : public sGauge
{
  Q_OBJECT
  DRIVER(sSR400Counter)

  public:
    virtual                   ~sSR400Counter ();
    virtual bool              AutoRepeat () const;
    virtual void              SetAutoRepeat (bool auto_repeat = true);

  protected: //evolution of the inherited state machine
    psState                   State_W_SR400Port;
    psState                   State_W_SettingPeriod;
    psState                   State_W_SR400Port_Linking;
    psState                   State_W_SR400Port_Flashing;
    psState                   State_W_SR400Port_Checking;
    psState                   State_O_M_SetP_Checking;
    psState                   State_O_M_A_Waiting;
    psState                   State_O_M_A_Reading;

  signals: //for internal needs
    void                      SR400PortLinked ();
    void                      SR400PortError (QString = QString());
    void                      GotSR400Message (sTelex);
    void                      NoAnswerFromSR400 ();
    void                      GotSR400ID ();

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  protected slots: //reimplementing inherited
    virtual void              SetWantedAutoRepeatMode ();
    virtual void              OnEnterToCoolingState ();
    virtual void              SetWantedPeriod ();

  protected slots:
    virtual void              ReportSR400PortError (QString);
    virtual void              OnNoAnswerFromSR400 ();
    virtual void              SendDummyCommand ();
    virtual void              RequestSR400ID ();
    virtual void              ParseSR400ID (sTelex);
    virtual void              CorrectPeriod (sTelex);
    virtual void              RequestCount ();
    virtual void              ParseCount (sTelex);

  protected: //fields
    sTeleCom::sLink           SR400Port;
    QTimer *                  Clock;
    int                       PollingInterval;//ms
    sPhysValue                CountingTime;
    sPhysValue                MinPeriod;

  private slots:
    void                      Start ();
    void                      Stop ();
    void                      LinkToSR400Port ();

  private:


  private: //fields
    int                       ChNum;
// For Debug ------------------------
    QTime                     _time;
    int                       _C;
// ----------------------------------
};

//------------------------------------------------------------------------------
} //namespace RA

#endif
