////////////////////////////////////////////////////////////////////////////////
/*! @file UT60EMeter.h   Получение данных от мультиметра UT60E.
- Part of UTMeters - a Research Assistant Plugin package.
- Uses  RAGUI - the Research Assistant Graphical User Interface.
- Uses  QtGui v.5.0  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2016-2017, Dmitry K. Nelson, St.Petersburg, Russia.
                          D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef UT60EMeter_H
#define UT60EMeter_H
#include "UTMeters.h"
#include "Gauge.h"
#include <QtSerialPort/QSerialPort>

namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sUT60EMeter)

//---------------------------------------------------------- sUT60EMeter ---
/*! Получение данных от мультиметра UT60E.
*/
class UTMeters_EXPORT  sUT60EMeter : public sGauge
{
  Q_OBJECT
  DRIVER(sUT60EMeter)

  public:
    virtual                   ~sUT60EMeter ();

  protected: //evolution of the inherited state machine
    psState                   State_W_PortOpening;

  signals: //for internal needs
    void                      PortOK ();

  protected: //reimplementing inherited
    virtual void              PostConstructor ();

  protected slots: //reimplementing inherited
    virtual void              OnEnterToCoolingState ();

  protected slots:
    virtual void              PortOpen ();
    virtual void              ReadAvailableBytes ();
    virtual void              Start ();
    virtual void              Stop ();
    virtual void              ParseSample ();

  protected: //fields
    QSerialPort *             Port;
    integer                   BaudRate;
    QSerialPort::Parity       Parity;
    QSerialPort::DataBits     DataBits;
    QSerialPort::StopBits     StopBits;
    QByteArray                InputBuffer;

  private slots:

  private:


  private: //fields
    sString                   PortName;
    int                       Sample[14];
    bool                      Has_first;
};

//------------------------------------------------------------------------------
} //namespace RA

#endif
