////////////////////////////////////////////////////////////////////////////////
/*! @file UTMeters.cpp Drivers for Multimeters of UT60 series.
- Part of UTMeters - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2016-2017, Dmitry K. Nelson, St.Petersburg, Russia.
                          D.Nelson@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "UTMeters.h"
#include "Symbols.h"
#include "attr.h"
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sUTMeters)

//------------------------------------------------------------------------------

sVersion  sUTMeters::Version ()
{
  return sVersion(RAP_VERS_MAJOR
                 ,RAP_VERS_MINOR
                 ,ZAKAVYCH(RAP_VERS_PATCH)
                 ,__DATE__
                 );
}


sVersion  sUTMeters::CoreVersionUsed ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}


sVersion  sUTMeters::CoreVersionMin ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}


QString  sUTMeters::UserFriendlyName ()
{
  return trUtf8("Drivers for Multimeters of UT60 series");
}


QString  sUTMeters::About ()
{
  return trUtf8(
      "Copyright (C) 2016, Dmitry K. Nelson, St.Petersburg, Russia.\n"
      "                    D.Nelson@mail.ioffe.ru\n"
                );
}


//void  sUTMeters::InstallDefaults (sPtr<sFolderNode>  DeviceList)
//{
//  try{
//    sPtr<sFolderNode>              DeviceCfg;

//    /*________________________________________template for device
//    DeviceList |= "__DEVICE_ID__";
//    DeviceCfg = DeviceList >> "__DEVICE_ID__";
//    DeviceCfg << attr("Class","__DRIVER_CLASS_NAME__");
//    __DRIVER_CLASS_NAME__::Preconfigure(DeviceCfg);
//    //if there are links:
//    DeviceCfg >> "Links" >> "__LINK_NAME__"
//        << attr("DeviceId","__DEVICE_ID__FOR_THE_LINK__");
//    //if you wanna change some of the default settings:
//    DeviceCfg >> "Settings" << attr("",);
//    DeviceCfg >> "Settings" << attr("",);
//    */

//  }catch(...){
//    ;
//  };

//}


sUTMeters::sUTMeters ()
{
  sUnit::Register(Symbol::CelsiusDegree
                 ,sMetricPrefix::NoPrefixAllowed
                 //Attention! When you change here the prefix allowance, the
                 //program ignores (and resets!) the settings made by user in
                 //the RANet Knowledgebase.
                 );
  sUnit::Register(sString(Symbol::Degree)+"C"
                 ,sMetricPrefix::NoPrefixAllowed
                 ,sPhysValue(Symbol::CelsiusDegree)
                 //Same about the definition of a derivative unit
                 );
  sUnit::Register("V");
  sUnit::Register("A");
  sUnit::Register(Symbol::Ohm);
  sUnit::Register("Ohm",sPhysValue(Symbol::Ohm));
  sUnit::Register("F");
}

//------------------------------------------------------------------------------
} //namespace RA
