// Copyright(C) 1994-2010, Viktor E. Bursian, St.Petersburg, Russia
//                         Viktor.Bursian@mail.ioffe.ru
////////////////////////////////////////////////////////////////////////////////
#include <string>
//#include <iostream>
#include <fstream>
#include <sstream>
#include <exception>
#include <stdexcept>
#include <limits>
//#include <boost/lexical_cast.hpp>
#include <typeinfo>
#include "../RANet/General.h"
#include "../RANet/Strings.h"
#include "../RANet/Trace.h"
#include "../RANet/Excepts.h"
#include "../RANet/Adam.h"
#include "../RANet/Storable.h"
#include "StorableExample.h"

using namespace RA;


void test_string_exception_etc ()
{
  std::string     s1 ="abcdefgh";
  std::string     s2 ="12345";

  std::set_terminate (__gnu_cxx::__verbose_terminate_handler);

  try{
    TRACE( s1.at(10) );
  }catch(std::out_of_range& e){
    std::cerr << "ПРОМАЗАЛ " << e.what() << std::endl;
  }catch(std::exception& e){
    std::cerr << e.what() << std::endl;
  }catch(...){
    std::cerr << "Ooops" << std::endl;
  };

  TRACE( std::numeric_limits<double>::round_style );
  std::cout << __FILE__ << __LINE__ << std::endl;
  if( s1 == s2 ){
    std::cerr << "Wow!" << s1 << s2 << std::endl;
  };

}

void test_types_etc ()
{
  RA::sSwitchable             Switchable(15);
  RA::sReciever               Reciever(17);
  RA::sTransmitter            Transmitter(19);
  RA::sRadio                  Radio("S o n y");
  RA::sVokiToki               VokiToki(31,31);
  RA::psStorable              pSt = &Radio;
  RA::psVokiToki              pVK;

  TRACE( typeid(RA::sStorable).name() );
  TRACE( typeid(&Radio).name() );
  TRACE( typeid(*pSt).name() );
  TRACE( typeid(RA::xException).name() );

  TRACE( RA::sStorable::Type()->Name() );
  TRACE( RA::sRadio::Type()->Name() );
  TRACE( Radio.Type()->Name() );
  TRACE( pSt->TrueType()->Name() );

  TRACE( RA::sSwitchable::Type()->CanCast(pSt) );
  TRACE( RA::sSwitchable::Type()->CanCast(&Radio) );
  TRACE( RA::sSelective::Type()->CanCast(&Radio) );
  TRACE( RA::sSwitchable::Type()->CanCast(&Reciever) );
  TRACE( RA::sSwitchable::Type()->CanCast(&Transmitter) );
  TRACE( RA::sSwitchable::Type()->CanCast(&VokiToki) );
  TRACE( RA::sStorable::Type()->CanCast(&VokiToki) );
  TRACE( RA::sVokiToki::Type()->CanCast(pSt=&VokiToki) );
  if( (pVK = dynamic_cast<RA::psVokiToki>(pSt)) ){
    TRACE( pVK->FullDuplex() );
  };

}

void test_io ()
{
  std::fstream                F;
  sRadio                      R("Panasonic");
  sVokiToki                   VokiToki(32,33);

  TRACE( R.Model );

  F.open("tempfile" ,std::ios::out | std::ios::binary | std::ios::trunc);
  R.Store(F);
  R.Store(F);
  VokiToki.Store(F);
  F.close();

  F.open("tempfile" ,std::ios::in | std::ios::binary);
  sRadio                      Rcopy(F);
  TRACE( Rcopy.Model );
  TRACE( Rcopy.Frequency * 1e3 );
  sRadio                      Rcopy1(F);
  TRACE( Rcopy1.Model );
  psStorable                  pSt = new sVokiToki(F);
  TRACE( dynamic_cast<psVokiToki>(pSt)->sReciever::ChanelNumber );
  F.close();

  F.open("tempfile" ,std::ios::out | std::ios::binary | std::ios::trunc);
  sStorable::StorePolymorphic(F,&R);
  sStorable::StorePolymorphic(F,&VokiToki);
  sStorable::StorePolymorphic(F,&VokiToki);
  sStorable::StorePolymorphic(F,&Rcopy);
  F.close();

  F.open("tempfile" ,std::ios::in | std::ios::binary);
  pSt=sStorable::PolymorphicLoad(F);  TRACE( pSt->TrueType()->Name() );
  pSt=sStorable::PolymorphicLoad(F);  TRACE( pSt->TrueType()->Name() );
  pSt=sStorable::PolymorphicLoad(F);  TRACE( pSt->TrueType()->Name() );
  pSt=sStorable::PolymorphicLoad(F);  TRACE( pSt->TrueType()->Name() );
  F.close();

  std::stringstream            S;

  sStorable::StorePolymorphic(S,&R);
  sStorable::StorePolymorphic(S,&VokiToki);
  sStorable::StorePolymorphic(S,&Rcopy);
  sStorable::StorePolymorphic(S,&VokiToki);

  std::cout << S.str() << std::endl;
  std::cout << S.str().length() << std::endl;
  std::cout << S.str().size() << std::endl;
  std::cout << S.str().capacity() << std::endl;
  S.seekp(0, std::ios::end);
  std::stringstream::pos_type offset = S.tellp();
  std::cout << offset << std::endl;

  pSt=sStorable::PolymorphicLoad(S);  TRACE( pSt->TrueType()->Name() );
  pSt=sStorable::PolymorphicLoad(S);  TRACE( pSt->TrueType()->Name() );
  pSt=sStorable::PolymorphicLoad(S);  TRACE( pSt->TrueType()->Name() );
  pSt=sStorable::PolymorphicLoad(S);  TRACE( pSt->TrueType()->Name() );
}

int main()
{
  test_string_exception_etc();
  test_types_etc();
  test_io();

  return 0;
}
