////////////////////////////////////////////////////////////////////////////////
/*! @file RAlgebra.cpp   Тест.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include <QApplication>
#include "RAlgebraWnd.h"
#include "PhysValue.h"
#include "Units.h"
#include "Trace.h"
#include "Log.h"
#include <QDebug>
#include <QMessageBox>
using namespace RA;


void  AboutExtendedNumbers ()
{
  TRACE("##################### AboutExtendedNumbers");
  TRACE( (long) std::numeric_limits<int>::has_quiet_NaN );
  TRACE( (long) std::numeric_limits<int>::has_signaling_NaN );
  TRACE( (long) std::numeric_limits<int>::has_infinity );
  TRACE( (long) std::numeric_limits<long>::has_quiet_NaN );
  TRACE( (long) std::numeric_limits<long>::has_signaling_NaN );
  TRACE( (long) std::numeric_limits<long>::has_infinity );
  TRACE( (long) std::numeric_limits<real>::has_quiet_NaN );
  TRACE( (long) std::numeric_limits<real>::has_signaling_NaN );
  TRACE( (long) std::numeric_limits<real>::has_infinity );
  TRACE( (long) std::numeric_limits<real>::is_iec559 );
  TRACE(long( floor(-3.1 / 3) ));
}


void  PhysValueNormalization ()
{
  RANet::Log.OpenBlock(sLog::Info,"PhysValueNormalization");
  sPhysValue    L0(9000     ,30. ,"m");  TRACE( L0.Text() );
  sPhysValue    L1( 900     ,3. ,"m");  TRACE( L1.Text() );
  sPhysValue    L2(  50     ,0.3 ,"m");  TRACE( L2.Text() );
  sPhysValue    L3(   3     ,0.03 ,"m");  TRACE( L3.Text() );
  sPhysValue    L4(   0.2   ,0.003 ,"m");  TRACE( L4.Text() );
  sPhysValue    L5(   0.02  ,0.00003 ,"m");  TRACE( L5.Text() );
  sPhysValue    L6(   0.002 ,0.000003 ,"m");  TRACE( L6.Text() );
  sPhysValue    L7(   0.0002,0.0000003 ,"m");  TRACE( L7.Text() );

  sPhysValue    T0(9000 ,0 ,"s");  TRACE( T0.Text() );
  sPhysValue    T1(900 ,0 ,"s");  TRACE( T1.Text() );
  sPhysValue    T2(50 ,0 ,"s");  TRACE( T2.Text() );
  sPhysValue    T3(3 ,0 ,"s");  TRACE( T3.Text() );
  sPhysValue    T4(0.2 ,0 ,"s");  TRACE( T4.Text() );

  sPhysValue    W0(0 ,0 ,"kYg");  TRACE( W0.Text() );     //кило-йотта-грамм
  sPhysValue    W1(0 ,0.005 ,"kYg");  TRACE( W1.Text() ); //не каноническая, но
  sPhysValue    W2(4 ,0.005 ,"kYg");  TRACE( W2.Text() ); //разрешённая форма,
  sPhysValue    W3(4 ,0.005 ,"nyg");  TRACE( W3.Text() ); //особенно, когда
                                                          //не хватает стандарта

  sPhysValue    W4(0 ,0.05 ,"kg");  TRACE( W4.Text() );
  sPhysValue    W5(0 ,0.5 ,"g");  TRACE( W5.Text() );
  sPhysValue    W6(0 ,5 ,"g");  TRACE( W6.Text() );
  sPhysValue    W7(0 ,50000 ,"g");  TRACE( W7.Text() );

  try{
    TRACE( (L0+L1).Text(Plain,Verbose) );
    TRACE( (L1-L2).Text(Plain,Verbose) );
    TRACE( (L1+L4).Text(Plain,Verbose) );
    TRACE( (L0/T4).Text(Plain,Verbose) );
    TRACE( (L1/W0).Text(Plain,Verbose) );
    TRACE( ((L1/W0)*L5).Text(Plain,Verbose) );
    TRACE( (W0/L5).Text(Plain,Verbose) );
    TRACE( (L1+W0).Text(Plain,Verbose) );
  }catch(rxRAlgebra){
    TRACE("Uups!");
  };
  RANet::Log.CloseBlock(sLog::Info,"PhysValueNormalization");
}


bool  RegisterRequest (rsString  lexem)
{
  if( QMessageBox::question(NULL,"New unit registration"
                           ,QString("It looks like you are going to "
                                    "register a new physical unit '")
                           +QString::fromUtf8(lexem)
                           +QString("'.  Correct?")
                           ,QMessageBox::Yes | QMessageBox::No)
                           == QMessageBox::Yes ){
    sUnit::Register(lexem);
    return true;
  };
  return false;
}


void  RegistrationOnTheFly ()
{
  RANet::Log.OpenBlock(sLog::Info,"RegistrationOnTheFly");

  sUnit::RegisterDialog = & RegisterRequest;

  try{

    sPhysValue   L1(27,"попугаев");
    sPhysValue   L2(5,"удавов");

    sUnit::Register("удавов",sPhysValue(38,"попугаев"));

    sPhysValue   L;
    L=L1+L2;
    TRACE(L.Text());

    L=L2+L1;
    TRACE(L.Text());

    sUnit::Register("попугаев",sPhysValue(15,"cm"));

    sPhysValue   M("dm");
    M=L;
    TRACE(M.Text());

  }catch(rxRAlgebra){
    TRACE("Uups!");
  };
  sUnit::RegisterDialog = NULL;
  RANet::Log.CloseBlock(sLog::Info,"RegistrationOnTheFly");
}


void  PhysicalExample ()
{
  //Вычислить среднее давление, оказываемое пулями массой m = 940 mg,
  //выстреливаемыми со скоростью v=1.5km/s в прямоугольную мишень.
  //Ширина мишени a = 20 дюймов, высота - b = 5 футов 11 дюймов.
  // Частота выстрелов - f=11Hz.  Процент попадания - k=30%.
  //Пули отскакивают упруго.
  //Ответ дать как для нормальных людей в системе СИ,
  //так и для извращенцев по ту сторону Атлантики.

  RANet::Log.OpenBlock(sLog::Info,"PhysicalExample");
  try{
    //регистрируем единицы (метр, грамм и секунда зарегистрированы в main)
    sUnit::Register("N",sPhysValue("kg*m/s2"));      //Ньютон
    sUnit::Register("Pa",sPhysValue("N/m2"));        //Паскаль
    sUnit::Register("Hz",sPhysValue("1/s"));         //Герц
    //и кривые единицы
    sUnit::Register("%",sPhysValue(0.01));          //процент
    sUnit::Register("\"",sPhysValue(25.4,"mm"));    //дюйм
    sUnit::Register("ft",sPhysValue(12,"\""));      //фут
    //чтобы определить фунт веса (а не фунт массы!)
    //нам потребуется ускорение свободного падения
    sPhysValue   g(9.82,"m/s2");
    sUnit::Register("pound"                          //фунт (веса)
                   ,sPhysValue( sPhysValue(451.0,"g") * g ));

    //данные из условия (погрешность - половина от единицы последнего знака)
    sPhysValue   m( 940 ,0.5 ,"mg");
    sPhysValue   v( 1.5 ,0.05 ,"km/s");
    sPhysValue   a( 20 ,0.5 ,"\"");
    sPhysValue   b( sPhysValue(5,"ft") + sPhysValue(11,0.5,"\"") );
    sPhysValue   f( 11 ,0.5 ,"Hz");
    sPhysValue   k( 30 ,0.5 ,"%");

    sPhysValue   p;             //единицы любые - как получится
    sPhysValue   p_SI("Pa");
    sPhysValue   pervert_p("pound/\"2");

    p=(2*m*v)*k*f/(a*b);

    TRACE(p.Text());
    TRACE(p.Text(Plain,Verbose));

    p_SI = p;
    TRACE(p_SI.Text());               //правильный ответ  10 Pa
    TRACE(p_SI.Text(Plain,Verbose));  //правильный ответ  (10.2 ± 1.3) Pa

    pervert_p = p;
    TRACE(pervert_p.Text());                // 1.5 mpound/"2
    TRACE(pervert_p.Text(Plain,Verbose));   // (1.48 ± 0.19) mpound/"2

//    csPhysValue   pp( (2*m*v)*k*f/(a*b) );
//    TRACE(pp.Text());

  }catch(rxRAlgebra){
    TRACE("Uups!");
  };
  RANet::Log.CloseBlock(sLog::Info,"PhysicalExample");
}


void  PhysicalExample2 ()
{
  //

  RANet::Log.OpenBlock(sLog::Info,"PhysicalExample2");
  try{

    TRACE(sUnits("degree/min").Text());
    TRACE(sUnits("min/degree").Text());
    TRACE(sUnits("°/min").Text());
    TRACE(sPhysValue(1,"⌘/min/⌘*ホ").Text());

  }catch(rxRAlgebra){
    TRACE("Uups!");
  };
  RANet::Log.CloseBlock(sLog::Info,"PhysicalExample2");
}


int main(int argc, char *argv[])
{
  RANet::Log.OpenBlock(sLog::Conditionless,"RAlgebra"
                      ,"#####################################################");
  QApplication a(argc, argv);
  RANet::Log.SetShowSeverity(sLog::Info);
//  RANet::Log.SetShowSeverity(sLog::Debug);
//  RANet::Log.AlwaysShow("sPhysValue::Normalize");
  RANet::Log.AlwaysShow("sUnits::Parse");

  sUnit::Register("m");
  sUnit::Register("g");
  sUnit::Register("s");

  AboutExtendedNumbers();
  PhysValueNormalization();
  RegistrationOnTheFly();
  PhysicalExample();
  PhysicalExample2();

  sRAlgebraWnd w;
  w.show();
  sUnit::RegisterDialog = & RegisterRequest;
  return a.exec();

  RANet::Log.CloseBlock(sLog::Conditionless,"RAlgebra");
  return 0;
}
