////////////////////////////////////////////////////////////////////////////////
/*! @file AdminWindow.h   RANet administrator window.
- Part of RAdmin project.
- Uses  RAGUI - Research Assistant Graphical User Interface.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  - http://qt.io/
- Copyright(C) 2013-2020, Viktor E. Bursian, St.Petersburg, Russia.
                          VBursian AT gmail DOT com
*///////////////////////////////////////////////////////////////////////////////
#ifndef AdminWindow_H
#define AdminWindow_H
#include "RAGUI_global.h"
#include "ExecutionConsole.h"
#include "Navigator.h"
#include "Globals.h"
#include "DB_SQL_Net.h"
#include "EditScriptDialog.h"
#include <QMainWindow>
#include <QString>
#include <QLabel>
#include <QAction>
#include <QMenuBar>
#include <QTimer>
#include <QFrame>
#include <QTextEdit>
#include <QProgressBar>
namespace RA {
//------------------------------------------------------------------------------
//------------------------------------------------------------- sAdminWindow ---

class  sAdminWindow : public QMainWindow
{
  Q_OBJECT
  public:
    enum                      eOpenMode {cNothingIsOpen
                                        ,cJobScript
                                        ,cPartition
                                        ,cPartitionAndRelated
                                        };
  public://static const
    static const sString      Title;
    static const sString      WorkstationAttrName;

  public:
    virtual                   ~sAdminWindow ();
    explicit                  sAdminWindow (psNet  net);
    sPtr<sFolderNode>         Data ()
                                {return Net->Root()>>GlobalAttr::Data; }
    sPtr<sFolderNode>         Workstation ()
                                {return Net->Root()>>WorkstationAttrName; }

  signals:
    void                      GotJob (QString);
    void                      GotJobFile (QString);

  protected slots:
    void                      ShowHelpContents ();
  private slots:
    void                      OpenRANet ();
    void                      OpenRANet (sString  textual_URL);
    void                      OpenPartition ();
    void                      OpenPartition (sString  textual_URL);
    void                      Close ();
    void                      Reopen (bool   on_success);
    void                      UpdateMenu ();
    void                      UpdateMenuFile ();
    void                      UpdateMenuInstall ();
    void                      FormMenuDatabases ();
    void                      ShowPartitionProperties ();
    void                      ShowPartitionProperties (QString  part_id);
    void                      DeleteDatabase (QString  part_id);
    void                      EditDatabase (QString  part_id);
    void                      AddDatabase ();

    void                      DoDummyJobs ();
    void                      ShowSQLDrivers ();
    void                      OpenNew ();
    void                      RunJobScript ();
    void                      RunJobScript (QString  script);
    void                      RunJobScriptFromFile (QString  file_path);
    void                      CureRelatedPartitionsTable ();
    void                      ReplicatePartition ();
    void                      AbsorbAllRelatedPartitions ();
    void                      InstallSimple ();
    void                      Autocomplete ();
    void                      AutocompleteData ();
    void                      AutocompleteWorkstation ();
    void                      CreateStyles ();
//    void                      Upgrade_1_1_to_1_2 ();

  private:
    void                      ShowRA ();
    void                      CreateNavigator ();
    void                      DeleteNavigator ();
    void                      ExecuteInConsole (const QString &  script
                                  ,bool   autoclose_on_success = false
                                  ,bool   autoclose_on_abend = false
                                  ,int    autoclose_delay_on_success = 5
                                                             //!< in seconds
                                  ,int    autoclose_delay_on_abend = 20
                                                             //!< in seconds
                                  );
    void                      DeleteConsole ();

  private: //fields
    eOpenMode                 OpenMode;
    sString                   RootURL;
    sString                   PartURL;
    eOpenMode                 ReopenMode;
    sString                   ReopenURL;
    eOpenMode                 ReopenModeOnSuccess;
    sString                   ReopenURLOnSuccess;
    psNet                     Net;
    psNet2TreeEvolution       Evolution;
    psNavigator               Navigator;
    QLabel *                  NameInfo;
    QLabel *                  ErrorStatus;
    QTimer *                  UpdatingTimer;
    sExecutionConsole *       Console;

    QAction *                 ActnNew;
    QAction *                 ActnOpen;
    QAction *                 ActnOpenPartition;
    QAction *                 ActnProperties;
    QAction *                 ActnClose;
    QAction *                 ActnExit;
    QAction *                 ActnShowHelpContents;
    QAction *                 ActnAddDatabaseLink;
    QAction *                 ActnDoDummyJobs;
    QAction *                 ActnShowSQLDrivers;
    QAction *                 ActnRunJobScript;
    QAction *                 ActnCureRelatedPartitionsTable;
    QAction *                 ActnReplicatePartition;
    QAction *                 ActnAbsorbAllRelatedPartitions;
    QAction *                 ActnInstallSimple;
    QAction *                 ActnAutocomplete;
    QAction *                 ActnCreateStyles;
//    QAction *                 ActnUpgrade_1_1_to_1_2;
    QMenu *                   MenuFile;
    QMenu *                   MenuDatabases;
    sEditScriptDialog *       JobScriptEditor;
};

//------------------------------------------------------------------------------
} //namespace RA
#endif
