////////////////////////////////////////////////////////////////////////////////
/*! @file EditScriptDialog.cpp   Редактирование текстового атрибута.
- Part of RAGUI - Research Assistant Graphical User Interface.
- Uses  QtGui v.4.6  -  http://qt.nokia.com/
- Uses  RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "EditScriptDialog.h"
#include "Helper.h"
#include <QVBoxLayout>
#include <QDialogButtonBox>
#include <QMessageBox>
namespace RA {
//------------------------------------------------------------------------------

sEditScriptDialog::~sEditScriptDialog ()
{
}


sEditScriptDialog::sEditScriptDialog (QWidget *  parent)
  :QDialog(parent)
{
  setWindowTitle(tr("Edit and run an arbitrary job script"));
  resize(600,300);

  Editor = new QTextEdit(this);

  QDialogButtonBox *          ButtonBox = new QDialogButtonBox(this);
  ButtonBox->setOrientation(Qt::Horizontal);
  ButtonBox->setStandardButtons(QDialogButtonBox::Cancel
                               |QDialogButtonBox::Ok
                               |QDialogButtonBox::Help);

  setLayout(new QVBoxLayout(this));
  layout()->addWidget(Editor);
  layout()->addWidget(ButtonBox);

  connect(ButtonBox,SIGNAL(accepted()),this,SLOT(accept()));
  connect(ButtonBox,SIGNAL(rejected()),this,SLOT(reject()));
  connect(ButtonBox,SIGNAL(helpRequested()),this,SLOT(ShowHelp()));
};


void  sEditScriptDialog::SetText (const QString &  value)
{
  Editor->setPlainText(value);
  Editor->setFocus();
};


QString  sEditScriptDialog::Text ()
{
  return Editor->toPlainText();
}


void  sEditScriptDialog::ShowHelp ()
{
  sHelper::ShowKeyId("sEditScriptDialog"); // it works

//  sHelper::ShowKeyword("job script"); // it works
//  sHelper::ShowQtHelpSchemeURL("RAdmin.html#script"); // it works
//  sHelper::ShowRelativeURL("core/RAdmin.html#script"); // it doesn't
//  sHelper::ShowRelativeURL("../core/RAdmin.html#script"); //it works
//  sHelper::ShowRelativeURL("RAdmin.html#script"); //it doesn't
//  sHelper::ShowQtHelpSchemeURL("Rigaku.html","Rigaku"); // it works
//  sHelper::ShowQtHelpSchemeURL("Rigaku.html","core"); // it doesn't
//  sHelper::ShowRelativeURL("../Rigaku/Rigaku.html"); // it works


//  Editor->clear();
//  Editor->setPlainText(
//      "DUMMY | 200 | 400 | 300 || DUMMY | 4500 | 2000\n"
//      "DUMMY || DUMMY || DUMMY || DUMMY || DUMMY\n"
//      "DUMMY | 1000 || DUMMY | 400 | 300 |200|100|1000\n"
//      "SQLDL"
//      );
//  QMessageBox::information(this,"Edit and run an arbitrary job script"
//      ,"Ещё чего! Хелп - в следующей версии, а пока только наощупь."
//       "\n\n. Можно давать любые команды, кроме"
//       "\n   CN | URL                 - create new partition"
//       "\n   RP | URL | URL   - replicate partition"
//       "\n   AARP | URL         - absorb all related partitions"
//       "\n   CRPT | URL          - cure related partitions table"
//       "\n- они для тех, кому хелп не нужен."
//       "\n\nА специально для тех, кто залез сюда из праздного любопытства,"
//       " предлагается безопасный пример. Жмите ОК и наслаждайтесь!"
//      );
};

//------------------------------------------------------------------------------
}; //namespace RA


