////////////////////////////////////////////////////////////////////////////////
/*! @file RAdmin.cpp  Research Assistant: RAdmin utility.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  - http://qt.io/
- Copyright(C) 2010-2020, Viktor E. Bursian, St.Petersburg, Russia.
                          VBursian AT gmail DOT com
*///////////////////////////////////////////////////////////////////////////////
#include "Log.h"
#include "RAGUI_global.h"
#include "Environment.h"
#include "DB_SQL_Net.h"
#include "AdminWindow.h"
#include "Helper.h"
#include "TestMemUsage.h"
#include <QApplication>
#include <QDir>
#include <QLocale>
#include <QMessageBox>

using namespace RA;


namespace RA {
extern void  SetTracing ();
}//namespace RA


int main(int argc, char *argv[])
{
  RANet::Log.OpenBlock(sLog::Conditionless,"RAdmin"
                      ,"#####################################################");
  SetTracing();

  int                         ReturnCode = 255;
  std::set_terminate (__gnu_cxx::__verbose_terminate_handler);
  TestMemoryUsage("initially");

  InitiateRAGUI(); // инициализирует ресурсы, проверяет соответствие версий

  try{

    //initiates sEnvironment's static members
    { sEnvironment(argc,argv); };
    sEnvironment::User.MaxAccessLevel = sUser::cAdmin;
    sEnvironment::User.AccessLevel = sUser::cAdmin;

    {
      QLocale::setDefault(QLocale::c());

      QApplication                Application(argc,argv);

      //Setting libraryPaths
      {
        //At this point, it is too late for the dlls which are linked
        //directly to RA.  However, it is necessary for the plugins.
        QCoreApplication::addLibraryPath(QCoreApplication::applicationDirPath()
                                        +QDir::separator()
                                        +"plugins");
        QCoreApplication::addLibraryPath(QCoreApplication
                                                       ::applicationDirPath());
        RANet::Log.Put(sLog::Conditionless,"RAdmin"
                      ,sString("libraryPaths=")
                      +FromQString(QCoreApplication::libraryPaths().join(':')));
      }

      try{
        sDB_SQL_Net                 Net;

        sHelper::Initiate(sEnvironment::HelpFilePath.PathStr());

        sAdminWindow                MainWindow (&Net);
        MainWindow.show();
        ReturnCode = Application.exec();
        sHelper::Terminate();
        RANet::Log.Put(sLog::Debug,"Quitting","correct quitting from GUI");
      }catch( rxException E ){
        QMessageBox * M(new QMessageBox(QMessageBox::Icon::Critical
            ,"Research Assistant Necrology"
            ,QString("Oops!!!")
            +"\n\n"+ToQString(E.SorryText())
            ));
        M->show();
        Application.exec();
        throw;
      }
    }

//    TestMemoryUsage("1");

//    RANet::Log.Put(sLog::Debug,"RAdmin"
//                  ,sString("Undo.Size() at the end ")
//                  +sString::FromInteger(sNet::Manager->Undo.Size()));
//    sNet::Manager->Undo.Clear();
//    TestMemoryUsage("2");
//    sNet::Manager->Root=NULL;
//    TestMemoryUsage("3");

  }catch( rxException E ){
    RANet::Log.Put(sLog::ProgramError,"RAdmin",E.SorryText());
    E.Report2cerr();
    //!@todo{admin} ReturnCode=RA::ReturnCodeBase(=10)+something
    ReturnCode = 230;
  }catch( const std::exception & E ){
    RANet::Log.Put(sLog::ProgramError,"RAdmin",E.what());
    xException::Report2cerr(E);
    ReturnCode = 253;
  }catch(...){
    RANet::Log.Put(sLog::ProgramError,"RAdmin","unknown exception");
    ReturnCode = 254;
  };
  TestMemoryUsage("finally");
  RANet::Log.Put(sLog::Debug,"Quitting"
                ,"######################## Bye-bye ##########################");
  RANet::Log.CloseBlock(sLog::Conditionless,"RAdmin");
  RANet::Log.Close(ReturnCode);
  return ReturnCode;
}
