////////////////////////////////////////////////////////////////////////////////
/*! @file BasicNet.h   Реализация сети RANet, опирающаяся на файловую систему
                       размещающая сеть в системе многих катологов и файлов
                       и подгружающая части по мере надобности.
- Part of RANet - Research Assistant Net Library (based on ANSI C++).
- Copyright(C) 2006-2010, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#ifndef BasicNet_H
#define BasicNet_H
#include "Net.h"
#include "Environment.h"
#include "Strings.h"
//#include <map>
namespace RA {
//------------------------------------------------------------------------------

ANNOUNCE_CLASS(sBasicNet)
typedef std::map<tNodeUID,psNode>  tNodeIndex;
ANNOUNCE_CLASS(xStripFailed)
ANNOUNCE_CLASS(xNetInitiation)

//------------------------------------------------------------------------------

class RANet_EXPORT  sBasicNet : public sNet
{
  public:// static const
    static const sString      cLastUIDFileExt;
    static const sString      cStreamFileExt;
    static const sString      cTextFileExt;

  public:
    virtual                   ~sBasicNet ();
                              sBasicNet (sNodeLocation  root_location);
                              sBasicNet (sString        owner
                                        ,int            copy
                                        ,sNodeLocation  root_location
                                        ,sPath          lastUID_path );
  public:
    virtual tNodeUID          GetNewNodeUID ();
    virtual psNode            GetNode (psNodeLocation);
    virtual void              ReleaseNode (tNodeUID);
    virtual void              Allocate (rcsAttribute A ,psNode Host);
    virtual void              AllocateMandatory (psNode                Node
                                                ,sNodeLocation::eType  LType
                                                ,rcsString             LPath);
    virtual void              Allocate (psNode Node);
    virtual void              PreallocateNode (psNode          Node
                                              ,sNodePtr        Host
                                              ,sAttributeName  AN   );
    virtual bool              SaveNode (psNode);
    virtual bool              GetReadingAccess (sNodePtr);
    virtual void              ReleaseReadingAccess (sNodePtr);
    virtual bool              GetFullAccess (sNodePtr);
    virtual void              ReleaseFullAccess (sNodePtr);
    virtual bool              TooLongWaiting ();

  protected: //fields
    sNodeLocation             RootLocation;
    sObjectFile               LastUIDFile;
    tNodeIndex                LoadedNodes;
    //open files
};

//------------------------------------------------------------------------------

class RANet_EXPORT  xStripFailed : public xNet
{
  public:
};

//------------------------------------------------------------------------------

class RANet_EXPORT  xNetInitiation : public xNet
{
  public:
};

//------------------------------------------------------------------------------
}; //namespace RA
#endif // BasicNet_H
