unit Calcutor;

interface

  type
    TValueOfName     = procedure (Name   : string
                                 ;var OK : boolean
                                 ;var Value : single);

  function Calculate (var Source, ErrorMes : string; var C : byte
                     ;var Complete, NoCalc : boolean
                     ;ValueOfName  : TValueOfName
                     ) : single;

{----------------------------------------------------------------------}
{                                                                      }
{ Expression ::= [ UnaryOperator ] Term [ AddingOperator Term ]...     }
{                                                                      }
{ UnaryOperator ::= + │ -                                              }
{                                                                      }
{ AddingOperator ::= + │ -                                             }
{                                                                      }
{ Term ::= Factor [ MultiplyingOperator Factor ]...                    }
{                                                                      }
{ MultiplyingOperator ::= * │ / │ :                                    }
{                                                                      }
{ Factor ::= FunctionExpr │ FunctionExpr ExponentSign Factor           }
{                                (calculate from right to left)        }
{                                                                      }
{ ExponentSign ::= ** │ ^                                              }
{                                                                      }
{ FunctionExpr ::= [ FunctionName ] FunctionExpr │ SimpleExpr          }
{                                                                      }
{ FunctionName ::= ln │ lg │ exp │ sin │ cos │ tg │ ctg                }
{                                                                      }
{ SimpleExpr ::=  Number │ LeftBracket Expression RightBracket         }
{               │ Constant │ Reference                                 }
{                                                                      }
{ Reference ::=  LeftSquareBracket Name RightSquareBracket             }
{                                                                      }
{ LeftBracket ::= ( │ { │                                              }
{                                                                      }
{ RightBracket must be corresponding to LeftBracket                    }
{                                                                      }
{ Constant ::= pi                                                      }
{                                                                      }
{ Number ::= Figures [ . [ Figures ] ] [ e [ UnaryOperator ] Figures ] }
{          │  . Figures [ e [ UnaryOperator ] Figures ]                }
{                                                                      }
{ Figures ::= Figure [ Figure ]...                                     }
{                                                                      }
{ Figure ::= 0 │ 1 │ 2 │ 3 │ 4 │ 5 │ 6 │ 7 │ 8 │ 9                     }
{                                                                      }
{----------------------------------------------------------------------}

implementation  uses Trans,Limits;

  type
    UnaryOperator   = procedure (X : extended; var R : extended);
    BinaryOperator  = procedure (X,Y : extended; var R : extended
                                ;var OK : boolean);
    FuncNameType = string[8];

  function Expression (var Source, ErrorMes : string; var C: byte
                      ;var Complete, NoCalc : boolean
                      ;ValueOfName  : TValueOfName
                      ) : extended;                           forward;

  procedure PassSpaces (var Source : string; var C : byte);
  begin
    while  (C <= length(Source)) and (Source[C] = ' ')  do
      Inc(C);
  end;

  function Number (var Source, ErrorMes : string; var C : byte
                  ;var Complete : boolean
                  ) : extended;
    var
      CC     : byte;
      N      : extended;
      RC     : integer;
  begin
    PassSpaces(Source,C);
    CC:=C;
    while  (CC <= length(Source)) and (Source[CC] in ['0'..'9'])  do
      Inc(CC);
    if  (CC <= length(Source)) and (Source[CC] = '.')  then
      Inc(CC);
    while  (CC <= length(Source)) and (Source[CC] in ['0'..'9'])  do
      Inc(CC);
    if  (CC <= length(Source)) and (Source[CC] in ['e','E'])  then begin
      Inc(CC);
      if  (CC <= length(Source)) and (Source[CC] in ['-','+'])  then
        Inc(CC);
      while  (CC <= length(Source)) and (Source[CC] in ['0'..'9'])  do
        Inc(CC);
    end;
    Val( copy(Source,C,CC-C) ,N,RC);
    if  RC = 0  then begin
      Number:=N;
      C:=CC;
      Complete:=true;
    end else begin
      Complete:=false;
      ErrorMes:=' - Неверно набрано число';
    end;
  end;

  function RefrExpr (var Source, ErrorMes : string; var C: byte
                    ;var Complete : boolean
                    ;ValueOfName  : TValueOfName
                    ) : extended;
    var
      CC    : byte;
      Name  : string;
      Value : single;
  begin
    PassSpaces(Source,C);
    CC:=C;
    while  (C <= length(Source)) and (Source[C] <> ']') do
      Inc(C);
    if (Source[C] = ']') then begin
      Name:=copy(Source,CC,C-CC);
      ValueOfName(Name, Complete, Value);
      if abs(Value) > 1E10 then Complete:=false;
      RefrExpr:=Value;
      Inc(C);
    end else begin
      Complete:=false;
      ErrorMes:=' - Отсутствует скобка ]';
      RefrExpr:=1;
    end;
  end;

  function SimpleExpr (var Source, ErrorMes : string; var C: byte
                      ;var Complete, NoCalc : boolean
                      ;ValueOfName  : TValueOfName
                      ) : extended;
    var
      SE    : extended;
  begin
    PassSpaces(Source,C);
    if  C > length(Source)  then begin
      Complete:=false;
    end else if Source[C] in ['.','0'..'9']  then begin
      SE:=Number(Source, ErrorMes,C,Complete);
    end else if  Source[C] = '['  then begin
      Inc(C);
      SE:=RefrExpr(Source, ErrorMes,C,Complete,ValueOfName);
      if  not Complete  then begin
        ErrorMes:=' - Нет такой методики';
        EXIT;
      end;
    end else if  Source[C] = '{'  then begin
      Inc(C);
      SE:=Expression(Source, ErrorMes,C,Complete, NoCalc,ValueOfName);
      if  not Complete  then  exit;
      if  (C <= length(Source)) and (Source[C] = '}')  then begin
        Inc(C);
        Complete:=true;
      end else begin
        Complete:=false;
        ErrorMes:=' - Отсутствует скобка }';
      end;
    end else if  Source[C] = '('  then begin
      Inc(C);
      SE:=Expression(Source, ErrorMes,C,Complete, NoCalc,ValueOfName);
      if  not Complete  then  exit;
      if  (C <= length(Source)) and (Source[C] = ')')  then begin
        Inc(C);
        Complete:=true;
      end else begin
        Complete:=false;
        ErrorMes:=' - Отсутствует скобка )';
      end;
    end else
      Complete:=false;
    if NoCalc then
      SimpleExpr:=1
    else
      SimpleExpr:=SE;
  end;

  function FunctionExpr (var Source, ErrorMes : string; var C: byte
                        ;var Complete, NoCalc : boolean
                        ;ValueOfName  : TValueOfName
                        ) : extended;
    var
      CC       : byte;
      FuncName : FuncNameType;
      FE, Y    : extended;
  begin
    PassSpaces(Source,C);
    CC:=C;
    while  (C <= length(Source)) and
           ( (Source[C] in ['A'..'Z']) or (Source[C] in ['a'..'z']) ) do
      Inc(C);
    FuncName:=UpperCase(copy(Source,CC,C-CC));
    if FuncName = 'EXP' then begin
      FE:=SimpleExpr(Source, ErrorMes,C,Complete, NoCalc,ValueOfName);
      if  not Complete  then  exit;
      if not NoCalc then
        if abs(FE) > 50 then begin
          Complete:=false;
          FunctionExpr:=1;
        end else
          FunctionExpr:=exp(FE)
      else FunctionExpr:=1;
    end else if FuncName = 'SIN' then begin
      FE:=SimpleExpr(Source, ErrorMes,C,Complete, NoCalc,ValueOfName);
      if  not Complete  then  exit;
      if not NoCalc then FunctionExpr:=sin(FE)
      else FunctionExpr:=1;
    end else if FuncName = 'COS' then begin
      FE:=SimpleExpr(Source, ErrorMes,C,Complete, NoCalc,ValueOfName);
      if  not Complete  then  exit;
      if not NoCalc then FunctionExpr:=cos(FE)
      else FunctionExpr:=1;
    end else if FuncName = 'TG' then begin
      FE:=SimpleExpr(Source, ErrorMes,C,Complete, NoCalc,ValueOfName);
      if  not Complete  then  exit;
      if not NoCalc then begin
        Y:=cos(FE);
        if  abs(Y) < 1e-18 then begin
          Complete:=false;
          FunctionExpr:=1;
        end else
          FunctionExpr:=sin(FE)/Y;
      end else FunctionExpr:=1;
    end else if FuncName = 'LN' then begin
      FE:=SimpleExpr(Source, ErrorMes,C,Complete, NoCalc,ValueOfName);
      if  not Complete  then  exit;
      if not NoCalc then
        if FE <= 0 then begin
          Complete:=false;
          FunctionExpr:=1;
        end else
          FunctionExpr:=ln(FE)
      else FunctionExpr:=1;
    end else if FuncName = 'LG' then begin
      FE:=SimpleExpr(Source, ErrorMes,C,Complete, NoCalc,ValueOfName);
      if  not Complete  then  exit;
      if not NoCalc then
        if FE <= 0 then begin
          Complete:=false;
          FunctionExpr:=1;
        end else
          FunctionExpr:=LG(FE)
      else FunctionExpr:=1;
    end else begin
      C:=CC;
      FunctionExpr:=SimpleExpr(Source, ErrorMes,C
                              ,Complete, NoCalc,ValueOfName);
    end;
  end;

  procedure ExponentSignInput (var Source, ErrorMes : string; var C: byte
                              ;var Complete : boolean);
  begin
    Complete:=true;
    PassSpaces(Source,C);
    if  C > length(Source)  then
      Complete:=false
    else if  Source[C] = '^'  then
      Inc(C)
    else if  (Source[C] = '*') and (C < length(Source))
         and (Source[C+1] = '*')  then
      Inc(C,2)
    else
      Complete:=false;
  end;

  function Factor (var Source, ErrorMes : string; var C: byte
                  ;var Complete, NoCalc : boolean
                  ;ValueOfName  : TValueOfName
                  ) : extended;
    var
      F,P   : extended;
      Found : boolean;
  begin
    F:=FunctionExpr(Source, ErrorMes,C,Complete, NoCalc,ValueOfName);
    if  not Complete  then  exit;
    ExponentSignInput(Source, ErrorMes,C,Found);
    if  Found  then begin
      P:=Factor(Source, ErrorMes,C,Complete, NoCalc,ValueOfName);
      if  not Complete  then  exit;
      if NoCalc then
        F:=1
      else begin
        if  F <= 0  then
          Complete:=false
        else begin
          F:=exp( ln(F) * P );
          if (abs(F) > 1E20) or (abs(F) < 1E-20) then
            Complete:=false;
        end;
      end;
    end;
    if Complete then
      Factor:=F
    else
      Factor:=1;
  end;

  procedure Mult (X,Y : extended; var R : extended;var OK : boolean);
  begin
    R:=X*Y;
    OK:=true;
  end;

  procedure Divide (X,Y : extended; var R : extended;var OK : boolean);
  begin
    if  abs(Y) < 1e-18 then begin
      R:=1;
      OK:=false;
    end else begin
      R:=X/Y;
      OK:=true;
    end;
  end;

  procedure MultiplyingOperatorInput (var Source, ErrorMes : string
                                    ; var C: byte
                                     ;var Complete : boolean
                                     ;var Operator : BinaryOperator);
  begin
    Complete:=true;
    PassSpaces(Source,C);
    if  C > length(Source)  then
      Complete:=false
    else if  Source[C] = '*'  then begin
      Inc(C);
      Operator:=Mult;
    end else if  (Source[C] = '/') or (Source[C] = ':')  then begin
      Inc(C);
      Operator:=Divide;
    end else
      Complete:=false;
  end;

  function Term (var Source, ErrorMes : string; var C: byte
                ;var Complete, NoCalc : boolean
                ;ValueOfName  : TValueOfName
                ) : extended;
    var
      O     : BinaryOperator;
      T,F   : extended;
      Found : boolean;
  begin
    T:=Factor(Source, ErrorMes,C,Complete, NoCalc,ValueOfName);
    if  not Complete  then  exit;
    repeat
      MultiplyingOperatorInput(Source, ErrorMes,C,Found,O);
      if  Found  then begin
        F:=Factor(Source, ErrorMes,C,Complete, NoCalc,ValueOfName);
        if  not Complete  then  exit;
        if not NoCalc then O(T,F,T,Complete);
        if  not Complete  then  exit;
      end;
    until  not Found;
    Term:=T;
  end;

  procedure UnaryPlus (X : extended; var R : extended);
  begin  R:=X;  end;

  procedure UnaryMinus (X : extended; var R : extended);
  begin  R:=-X;  end;

  procedure UnaryOperatorInput (var Source, ErrorMes : string; var C: byte
                               ;var Complete : boolean
                               ;var Operator : UnaryOperator);
  begin
    Complete:=true;
    PassSpaces(Source,C);
    if  C > length(Source)  then
      Complete:=false
    else if  Source[C] = '+'  then begin
      Inc(C);
      Operator:=UnaryPlus;
    end else if  Source[C] = '-'  then begin
      Inc(C);
      Operator:=UnaryMinus;
    end else
      Complete:=false;
  end;

  procedure BinaryPlus (X,Y : extended; var R : extended;var OK : boolean);
  begin
    R:=X+Y;
    OK:=true;
  end;

  procedure BinaryMinus (X,Y : extended; var R : extended;var OK : boolean);
  begin
    R:=X-Y;
    OK:=true;
  end;

  procedure AddingOperatorInput (var Source, ErrorMes : string; var C: byte
                                ;var Complete : boolean
                                ;var Operator : BinaryOperator);
  begin
    Complete:=true;
    PassSpaces(Source,C);
    if  C > length(Source)  then
      Complete:=false
    else if  Source[C] = '+'  then begin
      Inc(C);
      Operator:=BinaryPlus;
    end else if  Source[C] = '-'  then begin
      Inc(C);
      Operator:=BinaryMinus;
    end else
      Complete:=false;
  end;

  function Expression (var Source, ErrorMes : string; var C: byte
                      ;var Complete, NoCalc : boolean
                      ;ValueOfName  : TValueOfName
                      ) : extended;
    var
      UO    : UnaryOperator;
      O     : BinaryOperator;
      E,T   : extended;
      Found : boolean;
  begin
    UnaryOperatorInput(Source, ErrorMes,C,Found,UO);
    E:=Term(Source, ErrorMes,C,Complete, NoCalc,ValueOfName);
    if  not Complete  then  exit;
    if  Found  then
      UO(E,E);
    repeat
      AddingOperatorInput(Source, ErrorMes,C,Found,O);
      if  Found  then begin
        T:=Term(Source, ErrorMes,C,Complete, NoCalc,ValueOfName);
        if  not Complete  then  exit;
        if not NoCalc then O(E,T,E,Complete);
      end;
    until  not Found;
    Expression:=E;
  end;

  function Calculate (var Source, ErrorMes : string; var C : byte
                     ;var Complete, NoCalc : boolean
                     ;ValueOfName  : TValueOfName
                     ) : single;
    var
      E        : extended;
  begin
    C:=1;
    PassSpaces(Source, C);
    if C > length(Source) then begin
      Complete:=true;
      Calculate:=Unmeasured;
    end else begin
      ErrorMes:='';
      E:=Expression(Source,ErrorMes,C,Complete,NoCalc,ValueOfName);
      Complete:=Complete and (C > length(Source))
                         and (abs(E) < 1e8 );
      if  Complete then
        Calculate:=E
      else
        Calculate:=Unmeasured;
      ErrorMes:='Ошибка в формуле'+ErrorMes;
    end;
  end;

end.
