////////////////////////////////////////////////////////////////////////////////
/*! @file MSDNSpy.cpp Шпион-диверсант для отладки устройств MSDN.
- Part of DevelopmentTools - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  Qt v.5.0.2  - http://qt-project.org/
- Uses  RANet - Research Assistant Net Library.
- Copyright(C) 2013, Viktor E. Bursian, St.Petersburg, Russia.
                     Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "MSDNSpy.h"
//#include "attr.h"
//#include "StringsAndQStrings.h"
//#include "TeleType.h"
//#include <QHBoxLayout>
//#include <QVBoxLayout>
//#include <QScrollArea>
//#include <QLabel>
//#include <QMessageBox>
#include <limits>
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sMSDNSpy)

//----------------------------------------------------------------- sMSDNSpy ---
/*! @class sMSDNSpy
@details

When typing anything, if the second character in the typed line is
the quotation mark (or the apostrophe, or the grave accent)
and the rest of line can be interpreted as a decimal integer possibly
having a sign and liyng within limits of int32 (or int16, or int8,
correspondently), then this number is substituted with its hexadecimal
representation (and the preceding special character is, of course, omitted).

@code
P"-1        will be send as     PFFFFFFFF
P'-1        will be send as     PFFFF
P`-1        will be send as     PFF
x"128       will be send as     x00080    (leading zeroes to indicate int32)
x'128       will be send as     x080      (leading zeroes to indicate int16)
x`128       will be send as     x`128     (out of limits, not substituted)
x`-128      will be send as     x80
/"11        will be send as     /0000B
2'11        will be send as     200B
@endcode

*/


void  sMSDNSpy::Preconfigure (sNodePtr  DeviceCfg)
{
  sTeleSpy::Preconfigure(DeviceCfg);
}


sMSDNSpy::sMSDNSpy (sString   DeviceId
                   ,sNodePtr  DeviceCfg
                   ,sNodePtr  WidgetCfg
                   ,QWidget *  parent    )
    :sTeleSpy(DeviceId,DeviceCfg,WidgetCfg,parent)
{
}


void sMSDNSpy::Show (literal color ,sTelex  T)
{
  if( T.isEmpty() ){
    Screen->append(QString::fromUtf8("<font color=")
                  +QString::fromUtf8(color)
                  +QString::fromUtf8(">∅</font>"));
  }else{
    QString                     Comment;
    if( T.length() > 1 ){
      bool                        OK = true;
      bool                        HasDivider = false;
      qint32                      Number;
      qint32                      Divider = 1;
      real                        V = 1.0;
      int                         P;
      if( (P = T.indexOf('/',2)) > 0 ){
        Divider = T.mid(P+1).toLongLong(&OK,16);
        if( OK ){
          V /= Divider;
          HasDivider = true;
        }
      }else{
        P = -1;
      }
      if( OK ){
        Number = T.mid(1,P-1).toLongLong(&OK,16);
        if( OK ){
          V *= Number;
          Comment = QString::fromUtf8("   <font color=grey>[")
                   +QString::number(Number)
                   +( HasDivider
                    ? QString("/") + QString::number(Divider)
                      + "=" + QString::number((double)V)
                    : ""
                    );
          if( ! HasDivider && (T[1] > '7') ){
            if( T.length() == 3 ){
              Comment += QString::fromUtf8("  or  ")
                        +QString::number(Number-0x100);
            }else if( T.length() == 5 ){
              Comment += QString::fromUtf8("  or  ")
                        +QString::number(Number-0x10000);
            }else if( T.length() == 9 ){
              Comment += QString::fromUtf8("  or  ")
                        +QString::number((ulong)Number);
            }
          }
          Comment += QString::fromUtf8("]</font>");
        }
      }
    }
    Screen->append(QString::fromUtf8("<font color=")
                  +QString::fromUtf8(color)
                  +QString::fromUtf8(">")
                  +(QString)T
                  +QString::fromUtf8("</font>")
                  +Comment);
  }
}


void sMSDNSpy::Translate2Hex (rsTelex  T)
{
  bool                        OK = true;
  qint32                      Param;
  if( T.length() > 3 ){
    Param = T.mid(2,T.length()-2).toLongLong(&OK,10);
    if( OK ){
      QByteArray                  ParamText;
      ParamText = QByteArray::number(Param, 16).toUpper();
      if( T.indexOf('"',1) == 1 ){
        if( Param >= 0 )
          ParamText.prepend(QByteArray(5-ParamText.length(),'0'));
        T = ParamText.prepend(T[0]);
      }else if( T.indexOf('\'',1) == 1 ){
        if( (Param >= 0) && (Param <  std::numeric_limits<qint16>::max()) ){
          T = ParamText.prepend(QByteArray(3-ParamText.length(),'0'))
                       .prepend(T[0]);
        }else if( (Param < 0) && (Param > std::numeric_limits<qint16>::min()) ){
          T = ParamText.mid(4).prepend(T[0]);
        }
      }else if( T.indexOf('`',1) == 1 ){
        if( (Param >= 0) && (Param <  std::numeric_limits<qint8>::max()) ){
          T = ParamText.prepend(T[0]);
        }else if( (Param < 0) && (Param > std::numeric_limits<qint8>::min()) ){
          T = ParamText.mid(6).prepend(T[0]);
        }
      }
    }
  }
}


void sMSDNSpy::OnTypedInLeft (sTelex  T)
{
  Translate2Hex(T);
  sTeleSpy::OnTypedInLeft(T);
}


void sMSDNSpy::OnTypedInRight (sTelex  T)
{
  Translate2Hex(T);
  sTeleSpy::OnTypedInRight(T);
}

//------------------------------------------------------------------------------
} //namespace RA


