////////////////////////////////////////////////////////////////////////////////
/*! @file PanBursian.cpp Instrumental package (plugin) for the Bursian's
                         experimental setup.
- Part of PanBursian - Research Assistant Plugin package.
- Uses  RAGUI - Research Assistant Graphical User Interface Library.
- Uses  RANet - Research Assistant Net Library.
- Uses  Qt v.5  -  http://qt-project.org/
- Copyright(C) 2012-2014, Viktor E. Bursian, St.Petersburg, Russia.
                          Viktor.Bursian@mail.ioffe.ru
*///////////////////////////////////////////////////////////////////////////////
#include "PanBursian.h"
#include "attr.h"
#include "PhysValue.h"
#include "Symbols.h"
#include "EPRMath.h"
#include "TeleCOMPort.h"
#include "TelePipe.h"
#include "TeleSpy.h"
#include "MSDNSpy.h"
//#include "BursiPan_RASOS.h"
//#include "BursiPan_RASOS_MSDNx47.h"
#include "BursiPan_MSDN.h"
#include "BursiPanSignal.h"
#include "GaugeDisplay.h"
#include "BursiPanMagnet.h"
#include "EffectorConsole.h"
#include "BursiPanHandles.h"
#include "BursiPanScanner.h"
#include "LakeShore475Gaussmeter.h"
//#include "APRI_TController.h"
namespace RA {
//------------------------------------------------------------------------------

DEFINE_CLASS_TAG(sPanBursian)

//------------------------------------------------------------------------------

sVersion  sPanBursian::Version ()
{
  return sVersion(RAP_VERS_MAJOR
                 ,RAP_VERS_MINOR
                 ,ZAKAVYCH(RAP_VERS_PATCH)
                 ,__DATE__
                 );
}


sVersion  sPanBursian::CoreVersionUsed ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}


sVersion  sPanBursian::CoreVersionMin ()
{
  return sVersion( RA_VERS_MAJOR , RA_VERS_MINOR );
}


QString  sPanBursian::UserFriendlyName ()
{
  return trUtf8("PanBursian devices");
}


QString  sPanBursian::About ()
{
  return trUtf8(
      "The package includes drivers and consoles for Bursian's' experimental "
      "setup, based on a modified EPR spectrometer RadioPAN SE/X-2544.\n\n"
      "Copyright (C) 2012-2014, Viktor E. Bursian, St.Petersburg, Russia.\n"
      "                         Viktor.Bursian@mail.ioffe.ru"
        );
}


void  sPanBursian::InstallDefaults (sPtr<sFolderNode>  DeviceList)
{
  try{
    sPtr<sFolderNode>              DeviceCfg;

//    DeviceList |= "Genie_Port";
//    DeviceCfg = DeviceList >> "Genie_Port";
//    DeviceCfg << attr("Class","sTeleCOMPort");
//    sTeleCOMPort::Preconfigure(DeviceCfg);
//    DeviceCfg >> "Settings" << attr("PortName","COM1");
//    DeviceCfg >> "Settings" << attr("BaudRate",9600);
//    DeviceCfg >> "Settings" << attr("TerminalSymbols","0a");
//    DeviceCfg >> "Settings" << attr("Timeout",sPhysValue(5.0,"s"));

//    DeviceList |= "Genie_Pipe_Front_End";
//    DeviceCfg = DeviceList >> "Genie_Pipe_Front_End";
//    DeviceCfg << attr("Class","sTelePipe");
//    sTelePipe::Preconfigure(DeviceCfg);
//    DeviceCfg >> "Links" >> "Mate"
//        << attr("DeviceId","Genie_Pipe_Spy_End");

//    DeviceList |= "Genie_Pipe_Spy_End";
//    DeviceCfg = DeviceList >> "Genie_Pipe_Spy_End";
//    DeviceCfg << attr("Class","sTelePipe");
//    sTelePipe::Preconfigure(DeviceCfg);
//    DeviceCfg >> "Links" >> "Mate"
//        << attr("DeviceId","Genie_Pipe_Front_End");

//    DeviceList |= "Genie_Spy";
//    DeviceCfg = DeviceList >> "Genie_Spy";
//    DeviceCfg << attr("Class","sTeleSpy");
//    sTeleSpy::Preconfigure(DeviceCfg);
//    DeviceCfg >> "Links" >> "LeftTeleCom"
//        << attr("DeviceId","Genie_Pipe_Spy_End");
//    DeviceCfg >> "Links" >> "RightTeleCom"
//        << attr("DeviceId","Genie_Port");

    DeviceList |= "MSDN2047_Port";
    DeviceCfg = DeviceList >> "MSDN2047_Port";
    DeviceCfg << attr("Class","sTeleCOMPort");
    sTeleCOMPort::Preconfigure(DeviceCfg);
    DeviceCfg >> "Settings" << attr("PortName","COM4");
//    DeviceCfg >> "Settings" << attr("SearchThroughAvailablePorts",true);
    DeviceCfg >> "Settings" << attr("VendorId",0x03EB);
    DeviceCfg >> "Settings" << attr("ProductId",0x2047);
    DeviceCfg >> "Settings" << attr("BaudRate",115200);
    DeviceCfg >> "Settings" << attr("Timeout",sPhysValue(5.0,"s"));

    DeviceList |= "MSDN2047_Pipe_Front_End";
    DeviceCfg = DeviceList >> "MSDN2047_Pipe_Front_End";
    DeviceCfg << attr("Class","sTelePipe");
    sTelePipe::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Mate"
        << attr("DeviceId","MSDN2047_Pipe_Spy_End");

    DeviceList |= "MSDN2047_Pipe_Spy_End";
    DeviceCfg = DeviceList >> "MSDN2047_Pipe_Spy_End";
    DeviceCfg << attr("Class","sTelePipe");
    sTelePipe::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Mate"
        << attr("DeviceId","MSDN2047_Pipe_Front_End");

    DeviceList |= "MSDN2047_Spy";
    DeviceCfg = DeviceList >> "MSDN2047_Spy";
    DeviceCfg << attr("Class","sMSDNSpy");
    sMSDNSpy::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "LeftTeleCom"
        << attr("DeviceId","MSDN2047_Pipe_Spy_End");
    DeviceCfg >> "Links" >> "RightTeleCom"
        << attr("DeviceId","MSDN2047_Port");

    DeviceList |= "MSDN2048_Port";
    DeviceCfg = DeviceList >> "MSDN2048_Port";
    DeviceCfg << attr("Class","sTeleCOMPort");
    sTeleCOMPort::Preconfigure(DeviceCfg);
    DeviceCfg >> "Settings" << attr("PortName","COM5");
    DeviceCfg >> "Settings" << attr("BaudRate",115200);
    DeviceCfg >> "Settings" << attr("Timeout",sPhysValue(5.0,"s"));

    DeviceList |= "MSDN2048_Pipe_Front_End";
    DeviceCfg = DeviceList >> "MSDN2048_Pipe_Front_End";
    DeviceCfg << attr("Class","sTelePipe");
    sTelePipe::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Mate"
        << attr("DeviceId","MSDN2048_Pipe_Spy_End");

    DeviceList |= "MSDN2048_Pipe_Spy_End";
    DeviceCfg = DeviceList >> "MSDN2048_Pipe_Spy_End";
    DeviceCfg << attr("Class","sTelePipe");
    sTelePipe::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Mate"
        << attr("DeviceId","MSDN2048_Pipe_Front_End");

    DeviceList |= "MSDN2048_Spy";
    DeviceCfg = DeviceList >> "MSDN2048_Spy";
    DeviceCfg << attr("Class","sMSDNSpy");
    sMSDNSpy::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "LeftTeleCom"
        << attr("DeviceId","MSDN2048_Pipe_Spy_End");
    DeviceCfg >> "Links" >> "RightTeleCom"
        << attr("DeviceId","MSDN2048_Port");

//    DeviceList |= "BursiPan_RASOS";
//    DeviceCfg = DeviceList >> "BursiPan_RASOS";
//    DeviceCfg << attr("Class","sBursiPan_RASOS");
//    sBursiPan_RASOS::Preconfigure(DeviceCfg);
//    DeviceCfg >> "Links" >> "Genie"
//        << attr("DeviceId","Genie_Port");

//    DeviceList |= "BursiPan_RASOS_MSDN";
//    DeviceCfg = DeviceList >> "BursiPan_RASOS_MSDN";
//    DeviceCfg << attr("Class","sBursiPan_RASOS_MSDNx47");
//    sBursiPan_RASOS_MSDNx47::Preconfigure(DeviceCfg);
//    DeviceCfg >> "Links" >> "Genie"
//        << attr("DeviceId","Genie_Port");
//    DeviceCfg >> "Links" >> "MSDNx47"
//        << attr("DeviceId","MSDN2047_Port");

    DeviceList |= "BursiPan_MSDN";
    DeviceCfg = DeviceList >> "BursiPan_MSDN";
    DeviceCfg << attr("Class","sBursiPan_MSDN");
    sBursiPan_MSDN::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "MSDNx47"
//        << attr("DeviceId","MSDN2047_Port");
        << attr("DeviceId","MSDN2047_Pipe_Front_End");
    DeviceCfg >> "Links" >> "MSDNx48"
//        << attr("DeviceId","MSDN2048_Port");
        << attr("DeviceId","MSDN2048_Pipe_Front_End");

    DeviceList |= "BursiPanSignal";
    DeviceCfg = DeviceList >> "BursiPanSignal";
    DeviceCfg << attr("Class","sBursiPanSignal");
    sBursiPanSignal::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "BursiPan"
//        << attr("DeviceId","BursiPan_RASOS_MSDN");
        << attr("DeviceId","BursiPan_MSDN");

    DeviceList |= "EPR Signal";
    DeviceCfg = DeviceList >> "EPR Signal";
    DeviceCfg << attr("Class","sGaugeDisplay");
    sGaugeDisplay::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Gauge"
        << attr("DeviceId","BursiPanSignal");

    DeviceList |= "BursiPanMagnet";
    DeviceCfg = DeviceList >> "BursiPanMagnet";
    DeviceCfg << attr("Class","sBursiPanMagnet");
    sBursiPanMagnet::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "BursiPan"
//        << attr("DeviceId","BursiPan_RASOS_MSDN");
        << attr("DeviceId","BursiPan_MSDN");

    DeviceList |= "H Field";
    DeviceCfg = DeviceList >> "H Field";
    DeviceCfg << attr("Class","sEffectorConsole");
    sEffectorConsole::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Effector"
        << attr("DeviceId","BursiPanMagnet");

    DeviceList |= "BursiPan Handles";
    DeviceCfg = DeviceList >> "BursiPan Handles";
    DeviceCfg << attr("Class","sBursiPanHandles");
    sBursiPanHandles::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "BursiPan"
//        << attr("DeviceId","BursiPan_RASOS_MSDN");
        << attr("DeviceId","BursiPan_MSDN");

    DeviceList |= "BursiPan Scanner";
    DeviceCfg = DeviceList >> "BursiPan Scanner";
    DeviceCfg << attr("Class","sBursiPanScanner");
    sBursiPanScanner::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "BursiPan"
//        << attr("DeviceId","BursiPan_RASOS_MSDN");
        << attr("DeviceId","BursiPan_MSDN");

    DeviceList |= "Gaussmeter_Port";
    DeviceCfg = DeviceList >> "Gaussmeter_Port";
    DeviceCfg << attr("Class","sTeleCOMPort");
    sTeleCOMPort::Preconfigure(DeviceCfg);
    DeviceCfg >> "Settings" << attr("PortName","COM6");
    DeviceCfg >> "Settings" << attr("BaudRate",9600);
    DeviceCfg >> "Settings" << attr("Parity","o");
    DeviceCfg >> "Settings" << attr("DataBits",7);
    DeviceCfg >> "Settings" << attr("Timeout",sPhysValue(5.0,"s"));

    DeviceList |= "Gaussmeter_Pipe_Front_End";
    DeviceCfg = DeviceList >> "Gaussmeter_Pipe_Front_End";
    DeviceCfg << attr("Class","sTelePipe");
    sTelePipe::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Mate"
        << attr("DeviceId","Gaussmeter_Pipe_Spy_End");

    DeviceList |= "Gaussmeter_Pipe_Spy_End";
    DeviceCfg = DeviceList >> "Gaussmeter_Pipe_Spy_End";
    DeviceCfg << attr("Class","sTelePipe");
    sTelePipe::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Mate"
        << attr("DeviceId","Gaussmeter_Pipe_Front_End");

    DeviceList |= "Gaussmeter_Spy";
    DeviceCfg = DeviceList >> "Gaussmeter_Spy";
    DeviceCfg << attr("Class","sTeleSpy");
    sTeleSpy::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "LeftTeleCom"
        << attr("DeviceId","Gaussmeter_Pipe_Spy_End");
    DeviceCfg >> "Links" >> "RightTeleCom"
        << attr("DeviceId","Gaussmeter_Port");

    DeviceList |= "LakeShore Gaussmeter";
    DeviceCfg = DeviceList >> "LakeShore Gaussmeter";
    DeviceCfg << attr("Class","sLakeShore475Gaussmeter");
    sLakeShore475Gaussmeter::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "GaussmeterPort"
        << attr("DeviceId","Gaussmeter_Port");

    DeviceList |= "Gaussmeter";
    DeviceCfg = DeviceList >> "Gaussmeter";
    DeviceCfg << attr("Class","sGaugeDisplay");
    sGaugeDisplay::Preconfigure(DeviceCfg);
    DeviceCfg >> "Links" >> "Gauge"
        << attr("DeviceId","LakeShore Gaussmeter");

//    DeviceList |= "Telecom for APRI_TController";
//    DeviceCfg = DeviceList >> "Telecom for APRI_TController";
//    DeviceCfg << attr("Class","sTeleCOMPort");
//    sTeleCOMPort::Preconfigure(DeviceCfg);
//    DeviceCfg >> "Settings" << attr("Name","COM1");
//    DeviceCfg >> "Settings" << attr("BaudRate",115200);
//    DeviceCfg >> "Settings" << attr("Timeout",sPhysValue(5.0,"s"));

//    DeviceList |= "Gas-flow cryostat (APRI)";
//    DeviceCfg = DeviceList >> "Gas-flow cryostat (APRI)";
//    DeviceCfg << attr("Class","sAPRI_TController");
//    sAPRI_TController::Preconfigure(DeviceCfg);

  }catch(...){
    ;
  };

    /*________________________________________template for device
    DeviceList |= "__DEVICE_ID__";
    DeviceCfg = DeviceList >> "__DEVICE_ID__";
    DeviceCfg << attr("Class","__DRIVER_CLASS_NAME__");
    __DRIVER_CLASS_NAME__::Preconfigure(DeviceCfg);
    //if there are links:
    DeviceCfg >> "Links" >> "__LINK_NAME__"
        << attr("DeviceId","__DEVICE_ID__FOR_THE_LINK__");
    //if you wanna change some of the default settings:
    DeviceCfg >> "Settings" << attr("",);
    DeviceCfg >> "Settings" << attr("",);
    */
}


sPanBursian::sPanBursian ()
{
  qRegisterMetaType<RA::sBursiPan::eModulationMode>
                  ("sBursiPan::eModulationMode");
//  qRegisterMetaType<RA::sBursiPan::sScanJobData>
//                  ("sBursiPan::sScanJobData");
//  qRegisterMetaType<RA::sBursiPan::sScanStartData>
//                  ("sBursiPan::sScanStartData");
  qRegisterMetaType<RA::sBursiPan::sScanPointData>
                  ("sBursiPan::sScanPointData");

  sUnit::Register("m");
  sUnit::Register("V");
  sUnit::Register("B"
                 ,"-..-..-..-..-..-..-..----d--..-..-..-..-..-..-..-"
                 //Attention! When you change here the prefix allowance, the
                 //program ignores (and resets!) the settings made by user in
                 //the RANet Knowledgebase.
                 );
  sUnit::Register("Э");
  sUnit::Register("Oe");
  sUnit::Register("Hz");
  sUnit::Register(Symbol::SpinUnit);
  sUnit::Register("Spin" ,sPhysValue(Symbol::SpinUnit));
  sUnit::Register("step");
  sUnit::Register("pass");

//  sUnit::Register("m"
//                 ,"Y..Z..E..P..T..G..M..k--0-cm..-..n..-..f..a..z..y"

//                 //for a nerd (with hecto and deca):
//                 ,"Y..Z..E..P..T..G..M..khd0dcm..u..n..p..f..a..z..y"
//                 ,sMetricPrefix::StandardAllowance
//                 ,"Y..Z..E..P..T..G..M..k..0..m..u..n..p..f..a..z..y"
//                 ,sMetricPrefix::NoPrefixAllowed
//                 ,"-..-..-..-..-..-..-..-..0..-..-..-..-..-..-..-..-"
//                 );

  sMath::Definitions.Add(&SubtractBaseLine,"sbl",sDataFunction::Type());
}

//------------------------------------------------------------------------------
} //namespace RA
